
      subroutine WWWPentcheck(psi, p, z1, qz1, z2, qz2, z3, qz3, 
     1                                      j5z1z2)

c****************************************************************
c
c this subroutine computes the born and virtual matrix elements with 
c Z3, A3 replaced by q_3 and compares it with the analytic result
c which consists of a sum of diagrams with 2 vector bosons
c in the final state.
c
c***************************************************************

      implicit none

c  input / output
      double complex psi(2,-1:1,2)
      double precision p(0:3,8)
      double complex j5z1z2(0:3,3)
      double precision qz1(0:4), qz2(0:4), qz3(0:4)
      double complex z1(6), z2(6), z3(6)

c  local variables
      integer i, mu, k
      double complex m1(3), m2(3), m3(3)
      double complex mv1(3), mv2(3), mv3(3)
      double complex mvirt(3)
      double complex mvcheck(3)
      
      double complex psiz1(2,-1:1,2), psiz2(2,-1:1,2)
      double precision fqz1(0:4,2), fqz2(0:4,2)
      double complex jz1(0:5,-1:1,2), jz2(0:5,-1:1,2)
      double complex jvz1(0:5,-1:1,2), jvz2(0:5,-1:1,2)

      integer icount, bcount
      data bcount /0/
      data icount /0/
      save icount, bcount

      double complex zero
      parameter (zero=(0d0,0d0))

      logical ldebug
      parameter (ldebug = .false.)

c  external functions
      double complex s1c, dotcc, dotrc
      external s1c, dotcc, dotrc

c--------------------------- Begin Code ----------------------------

c calculate bras and kets
      call ket2c(psi(1,-1,1),.true.,p(0,1),-1,qz1(0),z1(1),
     &           psiz1(1,-1,1),fqz1(0,1)) 
      call bra2c(psi(1,-1,2),.true.,p(0,2),-1,qz2(0),z2(1),
     &           psiz2(1,-1,2),fqz2(0,2))

      call bra2c(psi(1,-1,2),.true.,p(0,2),-1,qz1(0),z1(1),
     &           psiz1(1,-1,2),fqz1(0,2))        !<q~Z1|

c different currents
      call curr6(-1,psiz2(1,-1,2),fqz2(0,2),psi(1,-1,1),p(0,1),jz2(0,-1,2))
      call curr6(-1,psi(1,-1,2),p(0,2),psiz1(1,-1,1),fqz1(0,1),jz1(0,-1,1))

      call curr6(-1,psiz1(1,-1,2),fqz1(0,2),psi(1,-1,1),p(0,1),jz1(0,-1,2))  ! <q~Z1|gamma_mu|q>

      call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1     .true., z1(1), 2, jz1(0,-1,1), jvz1(0,-1,1) )
      call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1     .true., z2(1), 1, jz2(0,-1,2), jvz2(0,-1,2) )

      call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1     .true., z1(1), 1, jz1(0,-1,2), jvz1(0,-1,2) )          ! <q~Z1|gamma_mu|q>_box

c q qbar -> VVV, z3 -> q(Z_3)
      m1(3) = -dotcc(z2(1),jz1(0,-1,1))    !Z1 Z2 Z3
      m2(3) = -dotcc(z1(1),jz2(0,-1,2))    !Z1 Z3 Z2
     1                +dotcc(z2(1),jz1(0,-1,1))
c      m3(3) = +dotcc(z1(1),jz2(0,-1,2))    !Z3 Z1 Z2

      m3(3) = +dotcc(z2(1),jz1(0,-1,2))    !Z3 Z2 Z1

c Virtual diagrams only consist of boxes * cvirt Mborn
      mv1(3) = -dotcc(z2(1),jvz1(0,-1,1))
      mv2(3) = +dotcc(z2(1),jvz1(0,-1,1))
     1                 -dotcc(z1(1),jvz2(0,-1,2)) 
c      mv3(3) = +dotcc(z1(1),jvz2(0,-1,2)) 

      mv3(3) = +dotcc(z2(1),jvz1(0,-1,2)) 

      mvcheck(1) = + mv3(3)
      mvcheck(2) = + mv2(3)
      mvcheck(3) = + mv1(3)

      do i = 1,3
         mvirt(i) = -dotrc(qz3(0),j5z1z2(0,i))
      enddo

      if( abs(mvirt(1)/mvcheck(1)-1d0).gt.1d-1 .or.
     1    abs(mvcheck(1)/mvirt(1)-1d0).gt.1d-1 ) then
         icount = icount+1
c         print*,'gauge check1 not passed'
         do mu = 0,3
            j5z1z2(mu,1) = (0.0d0,0.0d0)
            j5z1z2(mu,2) = (0.0d0,0.0d0)
            j5z1z2(mu,3) = (0.0d0,0.0d0)
         enddo
      else if( abs(mvirt(3)/mvcheck(3)-1d0).gt.1d-1 .or.
     1         abs(mvcheck(3)/mvirt(3)-1d0).gt.1d-1 ) then
         icount = icount+1
c         print*,'gauge check3 not passed'
         do mu = 0,3
            j5z1z2(mu,1) = (0.0d0,0.0d0)
            j5z1z2(mu,2) = (0.0d0,0.0d0)
            j5z1z2(mu,3) = (0.0d0,0.0d0)
         enddo
      else if (mvcheck(1).eq.0) then
         icount = icount+1
         print*,'gauge check4 not passed'
         do mu = 0,3
            j5z1z2(mu,1) = (0.0d0,0.0d0)
            j5z1z2(mu,2) = (0.0d0,0.0d0)
            j5z1z2(mu,3) = (0.0d0,0.0d0)
         enddo
      else if (mvcheck(3).eq.0) then
         icount = icount+1
         print*,'gauge check6 not passed'
         do mu = 0,3
            j5z1z2(mu,1) = (0.0d0,0.0d0)
            j5z1z2(mu,2) = (0.0d0,0.0d0)
            j5z1z2(mu,3) = (0.0d0,0.0d0)
         enddo
      else
         bcount = bcount+1
c         print*,'gauge check passed'
      endif      

      do i = 10,10000000,1000
      if (bcount.eq.1d2*i) then 
c        print*,'ratio= ',icount*1.0d0/(bcount+icount)
      endif
      enddo

      return
      end






      subroutine WWWPacoPentcheck(psi, p, z1, qz1, z2, qz2, z3, qz3, 
     1                                      pentgauge,j5z1z2)

c****************************************************************
c
c this subroutine computes the born and virtual matrix elements with 
c Z3, A3 replaced by q_3 and compares it with the analytic result
c which consists of a sum of diagrams with 2 vector bosons
c in the final state.
c
c***************************************************************

      implicit none

c  input / output
      double complex psi(2,-1:1,2)
      double precision p(0:3,8)
      double complex pentgauge(3,3)
      double complex j5z1z2(3)
      double precision qz1(0:4), qz2(0:4), qz3(0:4)
      double complex z1(6), z2(6), z3(6)

c  local variables
      integer i, j, mu, k
      double complex m1(3), m2(3), m3(3)
      double complex mv1(3), mv2(3), mv3(3)
      double complex mvirt(3,3)
      double complex mvcheck(3,3)
      double complex output(3)
      
      double complex psiz1(2,-1:1,2), psiz2(2,-1:1,2), psiz3(2,-1:1,2)
      double precision fqz1(0:4,2), fqz2(0:4,2), fqz3(0:4,2)
      double complex jz1(0:5,-1:1,2), jz2(0:5,-1:1,2), jz3(0:5,-1:1,2)
      double complex jvz1(0:5,-1:1,2), jvz2(0:5,-1:1,2), jvz3(0:5,-1:1,2)

      integer icount, bcount,oldicount,oldbcount
      data bcount /0/
      data icount /0/
      data oldicount /0/
      data oldbcount /0/
      save icount, bcount,oldicount,oldbcount

      double complex zero
      parameter (zero=(0d0,0d0))

      logical ldebug
      parameter (ldebug = .false.)

c  external functions
      double complex s1c, dotcc, dotrc
      external s1c, dotcc, dotrc

c--------------------------- Begin Code ----------------------------

      do i=1,3
         output(i) = j5z1z2(i)
      enddo

c calculate bras and kets
      call ket2c(psi(1,-1,1),.true.,p(0,1),-1,qz1(0),z1(1),
     &           psiz1(1,-1,1),fqz1(0,1))                                      !|Z1 q>
      call ket2c(psi(1,-1,1),.true.,p(0,1),-1,qz2(0),z2(1),
     &           psiz2(1,-1,1),fqz2(0,1))                                      !|Z2 q>
      call ket2c(psi(1,-1,1),.true.,p(0,1),-1,qz3(0),z3(1),
     &           psiz3(1,-1,1),fqz3(0,1))                                      !|Z3 q>

      call bra2c(psi(1,-1,2),.true.,p(0,2),-1,qz1(0),z1(1),
     &           psiz1(1,-1,2),fqz1(0,2))                                      !<q~Z1|
      call bra2c(psi(1,-1,2),.true.,p(0,2),-1,qz2(0),z2(1),
     &           psiz2(1,-1,2),fqz2(0,2))                                      !<q~Z2|
      call bra2c(psi(1,-1,2),.true.,p(0,2),-1,qz3(0),z3(1),
     &           psiz3(1,-1,2),fqz3(0,2))                                      !<q~Z3|

c different currents
      call curr6(-1,psiz1(1,-1,2),fqz1(0,2),psi(1,-1,1),p(0,1),jz1(0,-1,2))    ! <q~Z1|gamma_mu|q>
      call curr6(-1,psiz2(1,-1,2),fqz2(0,2),psi(1,-1,1),p(0,1),jz2(0,-1,2))    ! <q~Z2|gamma_mu|q>
      call curr6(-1,psiz3(1,-1,2),fqz3(0,2),psi(1,-1,1),p(0,1),jz3(0,-1,2))    ! <q~Z3|gamma_mu|q>

      call curr6(-1,psi(1,-1,2),p(0,2),psiz1(1,-1,1),fqz1(0,1),jz1(0,-1,1))    ! <q~|gamma_mu|Z1 q>
      call curr6(-1,psi(1,-1,2),p(0,2),psiz2(1,-1,1),fqz2(0,1),jz2(0,-1,1))    ! <q~|gamma_mu|Z2 q>
      call curr6(-1,psi(1,-1,2),p(0,2),psiz3(1,-1,1),fqz3(0,1),jz3(0,-1,1))    ! <q~|gamma_mu|Z3 q>

c different box currents
      call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1     .true., z1(1), 2, jz1(0,-1,1), jvz1(0,-1,1) )                       ! <q~|gamma_mu|Z1q>_box
      call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1     .true., z2(1), 2, jz2(0,-1,1), jvz2(0,-1,1) )                       ! <q~|gamma_mu|Z2q>_box
      call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1     .true., z3(1), 2, jz3(0,-1,1), jvz3(0,-1,1) )                       ! <q~|gamma_mu|Z3q>_box

      call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1     .true., z1(1), 1, jz1(0,-1,2), jvz1(0,-1,2) )                       ! <q~Z1|gamma_mu|q>_box
      call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1     .true., z2(1), 1, jz2(0,-1,2), jvz2(0,-1,2) )                       ! <q~Z2|gamma_mu|q>_box
      call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1     .true., z3(1), 1, jz3(0,-1,2), jvz3(0,-1,2) )                       ! <q~Z3|gamma_mu|q>_box

c qqbar -> VVV, z3 -> q(Z3)
      m1(3) = -dotcc(z2(1),jz1(0,-1,1))               !Z1 Z2 p_Z3
      m2(3) = -dotcc(z1(1),jz2(0,-1,2))               !Z1 p_Z3 Z2 (not possible for VVV = WWW)
     1        +dotcc(z2(1),jz1(0,-1,1))
c      m3(3) = +dotcc(z1(1),jz2(0,-1,2))    !p_Z3 Z1 Z2 (not possible for VVV = WWW)
      m3(3) = +dotcc(z2(1),jz1(0,-1,2))               !p_Z3 Z2 Z1

c qqbar -> VVV z1 -> q(Z1)
      m1(1) = -dotcc(z2(1),jz3(0,-1,1))               !Z3 Z2 p_Z1
      m2(1) = -dotcc(z3(1),jz2(0,-1,2))               !Z3 p_Z1 Z2 (not possible for VVV = WWW)
     1        +dotcc(z2(1),jz3(0,-1,1))
      m3(1) = -dotcc(z2(1),jz3(0,-1,2))               !p_Z1 Z2 Z3

c Virtual diagrams only consist of boxes * cvirt Mborn
      mv1(3) = -dotcc(z2(1),jvz1(0,-1,1))             !Z1 Z2 p_Z3 (BOX)
      mv2(3) = +dotcc(z2(1),jvz1(0,-1,1))             !Z1 p_Z3 Z2 (BOX) (not possible for VVV = WWW)
     1         -dotcc(z1(1),jvz2(0,-1,2)) 
      mv3(3) = +dotcc(z2(1),jvz1(0,-1,2))             !p_Z3 Z2 Z1 (BOX)

      mv1(1) = -dotcc(z2(1),jvz3(0,-1,1))             !Z3 Z2 p_Z1 (BOX)
      mv2(1) = +dotcc(z2(1),jvz3(0,-1,1))             !Z3 p_Z1 Z2 (BOX) (not possible for VVV = WWW)
     1         -dotcc(z3(1),jvz2(0,-1,2)) 
      mv3(1) = +dotcc(z2(1),jvz3(0,-1,2))             !p_Z1 Z2 Z3 (BOX)


      mvcheck(1,1) = + mv3(3)
      mvcheck(1,2) = 0.0d0
      mvcheck(1,3) = + mv1(1)

      mvcheck(2,1) = 0.0d0
      mvcheck(2,2) = 0.0d0
      mvcheck(2,3) = 0.0d0

      mvcheck(3,1) = + mv3(1)
      mvcheck(3,2) = 0.0d0
      mvcheck(3,3) = + mv1(3)

      do i = 1,3
         do j=1,3
         mvirt(i,j) = -pentgauge(i,j)
         enddo
      enddo

      if(    (abs(mvirt(1,1)/mvcheck(1,1)-1d0).gt.1d-1 .or.
     1        abs(mvcheck(1,1)/mvirt(1,1)-1d0).gt.1d-1)
     2 .and.(abs(mvirt(1,3)/mvcheck(1,3)-1d0).gt.1d-1 .or.
     3        abs(mvcheck(1,3)/mvirt(1,3)-1d0).gt.1d-1)) then
         icount = icount+1
c         print*,'gauge check1 not passed'
         output(1) = (0.0d0,0.0d0)
         output(2) = (0.0d0,0.0d0)
         output(3) = (0.0d0,0.0d0)
      else if( (abs(mvirt(3,1)/mvcheck(3,1)-1d0).gt.1d-1 .or.
     1           abs(mvcheck(3,1)/mvirt(3,1)-1d0).gt.1d-1)
     2 .and.   (abs(mvirt(3,3)/mvcheck(3,3)-1d0).gt.1d-1 .or.
     3           abs(mvcheck(3,3)/mvirt(3,3)-1d0).gt.1d-1)) then
         icount = icount+1
c         print*,'gauge check3 not passed'
         output(1) = (0.0d0,0.0d0)
         output(2) = (0.0d0,0.0d0)
         output(3) = (0.0d0,0.0d0)
      else if (mvcheck(1,1).eq.0.and.mvcheck(1,3).eq.0) then
         icount = icount+1
         print*,'gauge check4.1 not passed'
         output(1) = (0.0d0,0.0d0)
         output(2) = (0.0d0,0.0d0)
         output(3) = (0.0d0,0.0d0)
      else if (mvcheck(3,1).eq.0.and.mvcheck(3,3).eq.0) then
         icount = icount+1
         print*,'gauge check6.1 not passed'
         output(1) = (0.0d0,0.0d0)
         output(2) = (0.0d0,0.0d0)
         output(3) = (0.0d0,0.0d0)
      else
         bcount = bcount+1
c         print*,'gauge check passed'
      endif

      do i=1,3
         j5z1z2(i) = output(i)
      enddo

      do i = 10,10000000,1000
      if (bcount.eq.1d2*i) then
c         print*,'ratio= ',icount*1.0d0/(bcount+icount)
      endif
      enddo

      return
      end












