c************************************************************************


      subroutine Wardtest(psi, p, z1, qz1, z2, qz2, z3, qz3, 
     1                       j5contracted, mborn, gaugetest, alfas)

c****************************************************************
c
c this subroutine computes the born and virtual matrix elements with 
c Z3, A3 replaced by q_3 and compares it with the analytic result
c which consists of a sum of diagrams with 2 vector bosons
c in the final state.
c
c***************************************************************

      implicit none

c  input / output
      double complex psi(2,-1:1,2)
      double precision p(0:3,8)
      double precision qz1(0:4), qz2(0:4), qz3(0:4)
      double complex z1(6), z2(6), z3(6)
      integer number !contraction with which external leg?
      double complex j5contracted(3), mborn
      integer gaugetest
      double precision alfas

c  local variables
      integer i, mu, k
      double complex m(1:3), mv(1:3)
      double complex mvcheck(1:3)
      double precision qz1temp(0:4), qz2temp(0:4), qz3temp(0:4)
      double complex z1temp(6), z2temp(6), z3temp(6)

      double complex psiz1(2,-1:1,2), psiz2(2,-1:1,2), psiz3(2,-1:1,2)
      double precision fqz1(0:4,2), fqz2(0:4,2), fqz3(0:4,2)
      double complex jz1(0:5,-1:1,2), jz2(0:5,-1:1,2), jz3(0:5,-1:1,2)
      double complex jvz1(0:5,-1:1,2), jvz2(0:5,-1:1,2), jvz3(0:5,-1:1,2)
      double precision pi, colorfactor, kinfactor, factor, temp
      double precision couplfactor
      double precision fac(3), deltaP(3)
      parameter (colorfactor = 4.0d0/3.0d0)
      parameter (couplfactor = 0.01d0)
      parameter (pi = 3.141592653589793d0)

      integer icount, bcount, lbox
      data lbox /0/
      data bcount /0/
      data icount /0/
      save icount, bcount, lbox

      double complex zero
      parameter (zero=(0d0,0d0))

      logical ldebug
      parameter (ldebug = .false.)

c  external functions
      double complex s1c, dotcc, dotrc
      external s1c, dotcc, dotrc

c--------------------------- Begin Code ----------------------------
c eps_mu ~ q_mu * fac, fac = |eps(0)|/q(0)
      fac(1) = abs(z1(1))/qz1(0)
      fac(2) = abs(z2(1))/qz2(0)
      fac(3) = abs(z3(1))/qz3(0)

c calculate bras and kets
      call bra2c(psi(1,-1,2),.true.,p(0,2),-1,qz3(0),z3(1),
     &           psiz3(1,-1,2),fqz3(0,2))
      call ket2c(psi(1,-1,1),.true.,p(0,1),-1,qz1(0),z1(1),
     &           psiz1(1,-1,1),fqz1(0,1)) 

c different currents
      call curr6(-1,psiz3(1,-1,2),fqz3(0,2),psi(1,-1,1),p(0,1),jz3(0,-1,2))
      call curr6(-1,psi(1,-1,2),p(0,2),psiz1(1,-1,1),fqz1(0,1),jz1(0,-1,1))
      call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1     .true., z3(1), 1, jz3(0,-1,2), jvz3(0,-1,2) )
      call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1     .true., z1(1), 2, jz1(0,-1,1), jvz1(0,-1,1) )

c q qbar -> V1 V2 V3, eps_1 -> q_1
      m(1) = +dotcc(z2(1),jz3(0,-1,2))
      m(2) = -dotcc(z1(1),jz3(0,-1,2))
     1                +dotcc(z3(1),jz1(0,-1,1))
      m(3) = -dotcc(z2(1),jz1(0,-1,1))

c Virtual diagrams only consist of boxes * cvirt Mborn
      mv(1) = +dotcc(z2,jvz3(0,-1,2))
      mvcheck(1) = -mv(1)+(3*pi*(0d0,1d0)-4.0d0*pi**2/3.0d0+7.0d0)*m(1)
      mv(2) = -dotcc(z1(1),jvz3(0,-1,2))
     1                +dotcc(z3(1),jvz1(0,-1,1)) 
      mvcheck(2) = -mv(2)+(3*pi*(0d0,1d0)-4.0d0*pi**2/3.0d0+7.0d0)*m(2)
      mv(3) = -dotcc(z2(1),jvz1(0,-1,1))
      mvcheck(3) = -mv(3)+(3*pi*(0d0,1d0)-4.0d0*pi**2/3.0d0+7.0d0)*m(3)

      deltaP(1) = abs(fac(1)*(j5contracted(1)-mvcheck(1)))
      deltaP(2) = abs(fac(2)*(j5contracted(2)-mvcheck(2)))
      deltaP(3) = abs(fac(3)*(j5contracted(3)-mvcheck(3)))

c select the largest pentagon contribution
            temp = max(deltaP(1), deltaP(2), deltaP(3))
            if (deltaP(1).eq.temp) number = 1
            if (deltaP(2).eq.temp) number = 2
            if (deltaP(3).eq.temp) number = 3

            factor = colorfactor*couplfactor*fac(number)*alfas/2.0d0/pi

         if( abs((j5contracted(number)-mvcheck(number))/mborn*factor)
     1                          .gt.1.0d-3)then
           if (mv(number).eq.0) lbox = lbox + 1
c            icount = icount+1
            gaugetest = 1
         else
c            bcount = bcount+1
            gaugetest = 0
         endif      

c      do i = 10,10000000,1000
c      if (bcount.eq.1d2*i) print*,"ratio= ",icount*1.0d0/(bcount+icount),
c     1                              lbox*1.0d0/(bcount+icount)
c      enddo

      return
      end

