*****************************************************************************
c
c    begin function m2s_qqWWW
c
c*****************************************************************************
c
c       Adapted from m2s_wbfzh by D. Zeppenfeld
c
c this function computes |ME|^2 for qq-> ve e+ mu-vm~ vt/ta- ta+/vt~ ("WWW")
c at LO and virtual corrections via call of 'qqWWWm' or 'qqWWWp' depending on
c overall charge
c
c
c ------------------------------------------------------------------------

      double precision FUNCTION m2s_qqWWW(
     &      bos,  !in:  XXX Boson identifier,X=1 photon, X=2 Z, X=3,4 W+-
     &      nlo,  !in:  Born only for nlo=0, +interference for 1
     &      xi,   !in:  Feynman x parameters of incoming fermions
     &      p,    !in:  momenta associated with external fermions
     &      v,    !in:  momenta associated with gauge boson dec. fermions
     &      rn,   !in:  random number array
     &      NW    !in:  NW=1: Narrow width approximation, NW=0: full calculation
     &                        )
      IMPLICIT NONE

c declare global variables

#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/scales.inc"
#include "VBFNLO/utilities/lha.inc"
#include "tensor.inc"

c declare input/output variables

      INTEGER bos, nlo
      double precision xi(nx), p(0:3,max_p), v(0:3,n_v,3), rn(1), m2s
      INTEGER NW

c declare external functions
      double precision dotrr
      double complex dotrc
      INTEGER FL_WWW
      EXTERNAL dotrr, dotrc
      EXTERNAL FL_WWW

c declare local variables
      double complex zero
      PARAMETER (zero=(0d0,0d0))

      double precision q_sf, qq
      INTEGER init/0/, I, J, mu, nu, id
      SAVE init

c declare variables for summation process
      INTEGER ires

      INTEGER FSIGN(2+n_v), physToDiag(2), nmin, nmax
      SAVE nmax

c variables for leshouches interface
      double precision weight,rnumb,RandomNumber

c store contributions from subprocess ip in res(ip) where
      double precision res(maxnumsubproc)
      double precision rmax

c declare parton distribution variables
      double precision pdf(-6:6,2)

c define program switches
      LOGICAL ldebug
      PARAMETER (ldebug=.false.)
      double precision swmomenta(0:3,6)

c --------------------------- BEGIN CODE --------------------------------------

      if ( init .eq. 0 ) then

         write(6,*) ' '
         if( bos.eq.343 ) then
            write(6,*) 'W+W-W+ -Amplitude square information:'
         else if(bos.eq.434) then
            write(6,*) 'W-W+W- -Amplitude square information:'
         else 
            write(6,*) 'Process not implemented, check boson identifier'
         endif
         write(6,*) '-----------------------------------------------'
         write(6,*) ' '
         call vtowww_reset
         if (nfl.eq.4) then
            print*,' External b-quark contributions are excluded'
         elseif (nfl.eq.5) then
            print*,' External b-quark contributions are included'
         else
            print*,' Check setting of nfl in global.inc '
         endif
         if (NW.eq.0) then
            print*,'Calculation of amplitude with all off-shell contributions'
         else
            print*,'Calcutation of amplitude in narrow width approximation'
         endif
         if (procID.eq.7321) then
             print*,'Process with two identical pairs of leptons in final state'
         endif
         init = 1
         do i = 1,maxnumsubproc
            res(i) = 0
         enddo
         print*
      else
         do i = 1,nmax
            res(i) = 0
         enddo
      endif

c Les Houches interface
      if(lha.and.(.not.doNLO)) then
         helicity(1) = -1
         helicity(2) =  1
         helicity(3) = -1
         helicity(4) =  1
         helicity(5) = -1
         helicity(6) =  1
      endif

c switch momenta so that electron spinor is always created out of v(0,1,j)
      do i=1,6
         do mu=0,3
            swmomenta(mu,i) = 0.0d0
         enddo
      enddo

      if(bos.eq.343) then
         do mu=0,3
         swmomenta(mu,1) = v(mu,1,1) ! ve
         swmomenta(mu,2) = v(mu,2,1) ! e+
         swmomenta(mu,3) = v(mu,3,1) ! mu-
         swmomenta(mu,4) = v(mu,4,1) ! vm~
         swmomenta(mu,5) = v(mu,5,1) ! vt
         swmomenta(mu,6) = v(mu,6,1) ! ta+
         enddo
         do i=1,6
            do mu=0,3
                v(mu,i,1) = swmomenta(mu,i)
            enddo
         enddo
      else if(bos.eq.434) then
         do mu=0,3
         swmomenta(mu,1) = v(mu,3,1) ! ve
         swmomenta(mu,2) = v(mu,4,1) ! e+
         swmomenta(mu,3) = v(mu,1,1) ! mu-
         swmomenta(mu,4) = v(mu,2,1) ! vm~
         swmomenta(mu,5) = v(mu,5,1) ! ta-
         swmomenta(mu,6) = v(mu,6,1) ! vt~
         enddo
         do i=1,6
            do mu=0,3
                v(mu,i,1) = swmomenta(mu,i)
            enddo
         enddo
      endif

      do i=1,6
         do mu=0,3
            swmomenta(mu,i) = 0.0d0
         enddo
      enddo


c lepton spinors and W+, W-, W+/- polarisation vectors

      if(bos.eq.343) then                        ! lepton-wavefunctions for process u d~ --> w+ w- w+
         CALL OXXXXX(v(0,1,1),ZERO ,-1,1,wve1(1,1)) !W(1,3)         !ve
         CALL IXXXXX(v(0,2,1),ZERO ,1,-1,wep1(1,1)) !W(1,4)         !e+
         CALL OXXXXX(v(0,3,1),ZERO ,-1,1,wmu1(1,1)) !W(1,5)         !mu-
         CALL IXXXXX(v(0,4,1),ZERO ,1,-1,wvm1(1,1)) !W(1,6)         !vm~
         CALL OXXXXX(v(0,5,1),ZERO ,-1,1,wvt(1,1) ) !W(1,7)         !vtau
         CALL IXXXXX(v(0,6,1),ZERO ,1,-1,wtp(1,1)) !W(1,8)          !ta+
         CALL JIOXXX(wep1(1,1),wve1(1,1),GWF,WMASS,WWIDTH, wp1(1,1))    !W(1,9)  W+ #1
         CALL JIOXXX(wvm1(1,1),wmu1(1,1),GWF,WMASS,WWIDTH, w1m(1,1))    !W(1,10) W- #1
         CALL JIOXXX(wtp(1,1) ,wvt(1,1) ,GWF,WMASS,WWIDTH, wp2(1,1))    !        W+ #2
         do mu = 0,3
            qp1(mu,1)     = v(mu,1,1)+v(mu,2,1)       !W+ #1
            q1m(mu,1)     = v(mu,3,1)+v(mu,4,1)       !W-
            qp2(mu,1)     = v(mu,5,1)+v(mu,6,1)       !W+ #2
            qwp1wm(mu,1)  = qp1(mu,1)+ q1m(mu,1)      !momentum of W+#1 and W-
            qwp2wm(mu,1)  = qp2(mu,1)+ q1m(mu,1)      !momentum of W+#2 and W-
            qwpwmwp(mu,1) = qwp1wm(mu,1) + qp2(mu,1)  !momentum of W+W-W+ bosons
         enddo
         qp1(4,1)     = dotrr(qp1(0,1),qp1(0,1))
         q1m(4,1)     = dotrr(q1m(0,1),q1m(0,1))
         qp2(4,1)     = dotrr(qp2(0,1),qp2(0,1))
         qwp1wm(4,1)  = dotrr(qwp1wm(0,1),qwp1wm(0,1))
         qwp2wm(4,1)  = dotrr(qwp2wm(0,1),qwp2wm(0,1))
         qwpwmwp(4,1) = dotrr(qwpwmwp(0,1),qwpwmwp(0,1))

      else if (bos.eq.434) then                  ! lepton-wavefunctions for process u~d --> w+ w- w-
         CALL OXXXXX(v(0,1,1),ZERO ,-1,1,wve2(1,1)) !W(1,3)         !ve
         CALL IXXXXX(v(0,2,1),ZERO ,1,-1,wep2(1,1)) !W(1,4)         !e+
         CALL OXXXXX(v(0,3,1),ZERO ,-1,1,wmu2(1,1)) !W(1,5)         !mu-
         CALL IXXXXX(v(0,4,1),ZERO ,1,-1,wvm2(1,1)) !W(1,6)         !vm~
         CALL OXXXXX(v(0,5,1),ZERO,-1,1,wtm(1,1) ) !W(1,7)          ! tau-
         CALL IXXXXX(v(0,6,1),ZERO,1,-1,wvtbar(1,1)) !w(1,8)        ! vtau~
         CALL JIOXXX(wep2(1,1),wve2(1,1),GWF,WMASS,WWIDTH, w1p(1,1))    !W(1,9)  W+ #1
         CALL JIOXXX(wvm2(1,1),wmu2(1,1),GWF,WMASS,WWIDTH, wm1(1,1))    !W(1,10) W- #1
         CALL JIOXXX(wvtbar(1,1),wtm(1,1),GWF,WMASS,WWIDTH,wm2(1,1))    !        W- #2
         do mu = 0,3
            q1p(mu,1)     = v(mu,1,1)+v(mu,2,1)       !W+ #2
            qm1(mu,1)     = v(mu,3,1)+v(mu,4,1)       !W- #1
            qm2(mu,1)     = v(mu,5,1)+v(mu,6,1)       !W- #2
            qwpwm1(mu,1)  = q1p(mu,1)+ qm1(mu,1)      !momentum of W+ and W-#1
            qwpwm2(mu,1)  = q1p(mu,1)+ qm2(mu,1)      !momentum of W+ and W-#2
            qwpwmwm(mu,1) = qwpwm1(mu,1) + qm2(mu,1)  !momentum of W+W-W- bosons
         enddo
         q1p(4,1)     = dotrr(q1p(0,1),q1p(0,1))
         qm1(4,1)     = dotrr(qm1(0,1),qm1(0,1))
         qm2(4,1)     = dotrr(qm2(0,1),qm2(0,1))
         qwpwm1(4,1)  = dotrr(qwpwm1(0,1),qwpwm1(0,1))
         qwpwm2(4,1)  = dotrr(qwpwm2(0,1),qwpwm2(0,1))
         qwpwmwm(4,1) = dotrr(qwpwmwm(0,1),qwpwmwm(0,1))

      endif

      if (abs(nlo).eq.5 .or. nlo.eq.-4) then
         if(bos.eq.343) then
            qq  = dotrr(qp1(0,1),qwpwmwp(0,1))
            xp1 = dotrc(qwpwmwp(0,1),wp1(1,1))/qq
            qq  = dotrr(qp2(0,1),qwpwmwp(0,1))
            xp2 = dotrc(qwpwmwp(0,1),wp2(1,1))/qq
            qq  = dotrr(q1m(0,1),qwpwmwp(0,1))
            x1m = dotrc(qwpwmwp(0,1),w1m(1,1))/qq
            do mu = 0,3
               wpp1(mu+1,1) = wp1(mu+1,1) - xp1*qp1(mu,1)
               wpp2(mu+1,1) = wp2(mu+1,1) - xp2*qp2(mu,1)
               w1mp(mu+1,1) = w1m(mu+1,1) - x1m*q1m(mu,1)
            enddo
            do mu = 5,6
               wpp1(mu,1) = wp1(mu,1)
               wpp2(mu,1) = wp2(mu,1)
               w1mp(mu,1) = w1m(mu,1)
            enddo
         else
            qq  = dotrr(qm1(0,1),qwpwmwm(0,1))
            xm1 = dotrc(qwpwmwm(0,1),wm1(1,1))/qq
            qq  = dotrr(qm2(0,1),qwpwmwm(0,1))
            xmm2= dotrc(qwpwmwm(0,1),wm2(1,1))/qq   !named xmm2 since xm2(i) are m**2 of bosons i!
            qq  = dotrr(q1p(0,1),qwpwmwm(0,1))
            x1p = dotrc(qwpwmwm(0,1),w1p(1,1))/qq
            do mu = 0,3
               wmm1(mu+1,1) = wm1(mu+1,1) - xm1*qm1(mu,1)
               wmm2(mu+1,1) = wm2(mu+1,1) - xmm2*qm2(mu,1)
               w1pm(mu+1,1) = w1p(mu+1,1) - x1p*q1p(mu,1)
            enddo
            do mu = 5,6
               wmm1(mu,1) = wm1(mu,1)
               wmm2(mu,1) = wm2(mu,1)
               w1pm(mu,1) = w1p(mu,1)
            enddo
         endif
      endif   ! end nlo = +5,-5,-4 block

c calculate leptonic tensors

      if(bos.eq.343) then
         call WWW_atoww1(v(0,1,1),1,343,WWW_aww(0,1,1),NW)           !A -> W+ W- -> ve  e+  mu-  vm~
         call WWW_atoww2(v(0,1,1),1,343,WWW_aww(0,1,2),NW)           !A -> W+ W- -> mu- vm~ vt   tau+
         call WWW_ztoww1(v(0,1,1),1,343,WWW_zww(0,1,1),NW)           !Z -> W+ W- -> ve  e+  mu-  vm~
         call WWW_ztoww2(v(0,1,1),1,343,WWW_zww(0,1,2),NW)           !Z -> W+ W- -> mu- vm~ vt   tau+
         call WWW_wptowww(v(0,1,1),1,1,wptowww(0,1),NW)              !W+ -> WWW -> ve e+ mu- vm~ vt ta+
      else
         call WWW_atoww1(v(0,1,1),1,434,WWW_aww(0,1,1),NW)           !A -> W+ W- -> ve  e+  mu-  vm~
         call WWW_atoww3(v(0,1,1),1,434,WWW_aww(0,1,3),NW)           !A -> W+ W- -> ve  e+  tau- vt~
         call WWW_ztoww1(v(0,1,1),1,434,WWW_zww(0,1,1),NW)           !Z -> W+ W- -> ve  e+  mu-  vm~
         call WWW_ztoww3(v(0,1,1),1,434,WWW_zww(0,1,3),NW)           !Z -> W+ W- -> ve  e+  tau- vt~
         call WWW_wmtowww(v(0,1,1),1,1,wmtowww(0,1),NW)              !W- -> WWW -> ve e+ mu- vm~ ta- vt~
      endif

c
c call PDF subroutine in order to determine parton
c distributions in the incoming (anti)protons.
c
      q_sf = sqrt(mufsq(1,1))

      call pdfproton( xi(1), q_sf, pdf(-6,1) )        ! f_a(x1)
      call pdfproton( xi(2), q_sf, pdf(-6,2) )        ! f_a(x2)

C******************************************************************************

      nmax = FL_WWW(fsign,-1)            !reset counter for subprocesses to 0

c******************************************************************************

      if(bos.eq.343) then
C*******************  q1 q2bar ---> W+ W- W+   ********************************
c   physToDiag(ext.momentum label) = Feynman diagram label

      physToDiag(1)=1     !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=2

      fsign(1) = 1
      fsign(2) = -1
      fsign(3) = 1
      fsign(4) = -1
      fsign(5) = 1
      fsign(6) = -1
      fsign(7) = 1
      fsign(8) = -1

      call qq_WWW(xi,p,v,physToDiag,fsign,bos,nlo,
     1            pdf,res,nmin,nmax,NW)

C*******************  q2bar q1 ---> W+ W- W+   ********************************

      physToDiag(1)=2    !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=1

      fsign(1) = 1
      fsign(2) =-1

      call qq_WWW(xi,p,v,physToDiag,fsign,bos, nlo,
     1            pdf,res,nmin,nmax,NW)

c******************************************************************************

      else if(bos.eq.434) then
C*******************  q1 qbar2 ---> W- W+ W-   ********************************

c   physToDiag(ext.momentum label) = Feynman diagram label

      physToDiag(1)=1     !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=2

      fsign(1) = 1
      fsign(2) =-1
      fsign(3) = 1
      fsign(4) = -1
      fsign(5) = 1
      fsign(6) = -1
      fsign(7) = 1
      fsign(8) = -1

      call qq_WWW(xi,p,v,physToDiag,fsign,bos,nlo,
     1            pdf,res,nmin,nmax,NW)

C*******************  q2bar q1 ---> W- W+ W-   ********************************

      physToDiag(1)=2    !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=1

      fsign(1) = 1
      fsign(2) =-1

      call qq_WWW(xi,p,v,physToDiag,fsign,bos, nlo,
     1            pdf,res,nmin,nmax,NW)

c******************************************************************************

      else
         write(6,*) 'Process not implemented, check boson identifier'
      endif   !end of ud~/du~ process calls

c*****************  end of process evaluation  ********************************

      if (init.eq.1) then
         init = init+1
 199     format(a,4i5)
      endif

      m2s = 0
      DO IRES = 1,NMAX
         m2s = m2s + RES(IRES)
      ENDDO

c...Les Houches interface - the most propable subprocess
      if (lha.and..not.doNLO) then
         i=0
         weight=0.d0
         rnumb=RandomNumber()
         do while((i.le.nmax).and.(weight.le.rnumb*m2s))
            i=i+1
            weight=weight+res(i)
            iprocess=i
         enddo
      endif

      m2s_qqWWW = m2s

      RETURN
      END

c******************************************************************************
c
c    end function m2s_qqWWW
c
c******************************************************************************


c******************************************************************************
c
c   begin subroutine qq_WWW
c
c******************************************************************************

      subroutine qq_WWW(xi,p,v,physToDiag,fsign,bos, nlo,
     1                  pdf,res,nmin,nmax,NW)
c
c qq_WWW calls the amplitude square routines 
c             qqVVV    for qq --> VVV --> 6 leptons
c
c  INPUT:  xi(1:2)         momentum fraction xi of external partons
c          p(0:3,4)        external physical parton momenta
c          v(0:3,nv)       Z decay momenta, for Higgs production only
c                          the sum q(mu) = v(mu,1)+...+v(mu,nv) is needed
c          physToDiag(4)   physToDiag(ext.mom. label) = Feynman diagram label
c          fsign           sign factors for amplitude calls; see qqZqq
c          bos             XXX Boson identifier,X=1 photon, X=2 Z, X=3,4 W+-
c          nlo             nlo = 0: calculate LO only
c                          nlo = 1: calculate full NLO (Born + interference)
c          pdf             complete pdf's as computed in pdfproton
c          NW              NW=1: Narrow width approximation, NW=0: full calculation
c  OUTPUT:
c          uuvv(ID)   output in format as in qqZqq(j), but corresponding
c          etc.            to res(*) in m2s_qqZqq
c     or   res(k)   
c
c  In and Output
c    nmin, nmax            range of process numbers for this call

      implicit none

#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"

      double precision p(0:3,n_p), v(0:3,n_v), xi(nx)
      double precision pdf(-6:6,2), res(maxnumsubproc)
      integer physToDiag(2), fsign(2+n_v), bos, nlo, nmin, nmax,NW

      double precision N                              ! color factors
      parameter(N=3d0)

c alfas, scales etc
#include "VBFNLO/utilities/scales.inc"
#include "tensor.inc"
#include "VBFNLO/utilities/coupl.inc"

c local variables
      double precision udbarWWW, udbarWWW2                 !for process pp --> W+W-W+ --> Leptons
      double precision dubarWWW, dubarWWW2                 !for process pp --> W+W-W- --> Leptons

      double precision pbar(0:3,2+n_v), dotrr, polcol
      double precision Matrixelt(0:1,0:1)
      double complex zero
      integer iflav(2), diagToPhys(2), FL_WWW
      external dotrr, FL_WWW

      logical ChargedCurrent, sametype, oneAntiparticle
      logical ldebug
      parameter (ldebug = .false.)
      parameter (zero = (0d0,0d0) )
      integer i,if1,if2,icc1,j,k,mu

      integer stopp
      data stopp /1/
      save stopp
      integer counter                   ! to count number of M^2 differing to madgraph by epsilon
      data counter/0/
      save counter

      integer debugpspoints              ! number of phasespace points tested
      parameter (debugpspoints = 2)
      double precision epsilon         ! parameter in which madgraph and handwritten M^2 agree
      parameter ( epsilon = 1.0d-8)

c initialize output

      udbarWWW = 0.0d0
      dubarWWW = 0.0d0

      udbarWWW2 = 0.0d0
      dubarWWW2 = 0.0d0

      nmin = nmax+1

      polcol = 1d0/(4d0*N**2*xi(1)*xi(2))

      do mu = 0,3
         do i = 1,2
            pbar(mu,physToDiag(i))=p(mu,i)
         enddo
      enddo

      do mu = 0,3             ! kinematics for w+w-w+ | w-w+w- --> 6 lepton decay
         pbar(mu,3) = v(mu,1) ! ve    | ve
         pbar(mu,4) = v(mu,2) ! e+    | e+
         pbar(mu,5) = v(mu,3) ! mu-   | mu-
         pbar(mu,6) = v(mu,4) ! vm~   | vm~
         pbar(mu,7) = v(mu,5) ! vt    | ta-
         pbar(mu,8) = v(mu,6) ! tau+  | vt~
      enddo
c choose process via boson identifier bos

      if(bos.eq.343) then
#ifdef WITH_MADGRAPH
         if (ldebug) then
             call sudbar_WWW(pbar,fsign,udbarWWW2,NW)     ! Madgraph code for u d~ -> ve e+ mu- vm~ vt ta+
         endif
#endif

         call QQWWWp(pbar,fsign,nlo,1,udbarWWW,NW)

         Matrixelt(0,1)=udbarWWW

         do if1=1,nfl,2
            iflav(1)=( if1 - PhysToDiag(1) + 2 )*fsign(PhysToDiag(1))
            iflav(2)=( if1 - PhysToDiag(2) + 2 )*fsign(PhysToDiag(2))
            k=FL_WWW(iflav,1)

c sign1, sign2 read from global.inc: identifier of hadrons
c        in colliding beams 1 and 2 (proton or antiproton)

            res(k)=pdf(sign1*iflav(1),1)
     &            *pdf(sign2*iflav(2),2)
     &            *Matrixelt(mod(if1+1,2),mod(if1,2))*polcol

            if(ldebug) then
               print*,'sign1 :',sign1
               print*,'sign2 :',sign2
               print*,'iflav(1) :',iflav(1)
               print*,'iflav(2) :',iflav(2)
            endif

            res(k) = res(k)
         enddo

#ifdef WITH_MADGRAPH
         if (ldebug) then
            if (abs(udbarWWW/udbarWWW2 - 1) .gt. epsilon) then
c               print*,' '
c               print*,'udbarWWW =  ',udbarWWW
c               print*,'udbarWWW2 = ',udbarWWW2
c               print*,'udbarWWW/udbarWWW2 - 1 = ',udbarWWW/udbarWWW2 - 1
c               print*,' '
               counter = counter +1
c               print*,'------------------------------------------------------------'
            endif
            if (stopp .eq. debugpspoints) then
               print*,counter, ' points in ',debugpspoints,' differ by number greater than ',epsilon
               print*,'Leaving VBFNLO'
               stop
            endif
            stopp = stopp +1
         endif
#endif

      else if(bos.eq.434) then         !end of udbar-computation block, start of ubard-comp.

#ifdef WITH_MADGRAPH
         if(ldebug) then
            call sdubar_WWW(pbar,fsign,dubarWWW2,NW)     ! Madgraph code for d u~ -> ve e+ mu- vm~ ta- vt~
         endif
#endif

         call QQWWWm(pbar,fsign,nlo,1,dubarWWW,NW)

         Matrixelt(1,0)=dubarWWW

         do if1=1,nfl,2
            iflav(1)=( if1 + PhysToDiag(1) - 1 )*fsign(PhysToDiag(1))
            iflav(2)=( if1 + PhysToDiag(2) - 1 )*fsign(PhysToDiag(2))
            k=FL_WWW(iflav,1)

            res(k)=pdf(sign1*iflav(1),1)
     &            *pdf(sign2*iflav(2),2)
     &            *Matrixelt(mod(if1,2),mod(if1+1,2))*polcol

            if(ldebug) then
               print*,'sign1 :',sign1
               print*,'sign2 :',sign2
               print*,'iflav(1) :',iflav(1)
               print*,'iflav(2) :',iflav(2)
            endif

            res(k) = res(k)
         enddo

#ifdef WITH_MADGRAPH
         if (ldebug) then
            if (abs(dubarWWW/dubarWWW2 - 1) .gt. epsilon) then
c               print*,' '
c               print*,'dubarWWW =  ',dubarWWW
c               print*,'dubarWWW2 = ',dubarWWW2
c               print*,'dubarWWW/dubarWWW2 - 1 = ',dubarWWW/dubarWWW2 - 1
c               print*,' '
               counter = counter +1
c               print*,'------------------------------------------------------------'
            endif
            if (stopp .eq. debugpspoints) then
               print*,counter, ' points in ',debugpspoints,' differ by number greater than ',epsilon
               print*,'Leaving VBFNLO'
               stop
            endif
            stopp = stopp +1
         endif
#endif

      else
         write(6,*) 'Process not implemented, check boson identifier'
      endif                                                !end of dubar-computation block

      nmax = FL_WWW(iflav,-2)

      end
c******************************************************************
c
c   end subroutine qq_WWW
c
c*****************************************************************






c******************************************************************
c
c   begin function TRIBOSFL_ZH
c
c*****************************************************************
      integer function FL_WWW(iflav,colstruc)
      implicit none
      integer iflav(4),colstruc    ! input for color and flavor assignment
c  fill hepup color and flavor assignments for WBF processes with
c  two quark lines only

c  id1,2 are flavor identifiers for incoming quarks
c  id3,4 are flavor identifiers for outgoing quarks
c  colstruc = is irrelevant here
c
c  color structure of these interactions is trivial
c  and colstruc was included only to make this program 
c  more consistent with the others.

c  Note that colstruc is also a flag:
c  if colstruc=-1, then we are resetting.

c  The color structure is always the same:
c  color always flows from id1->id3 and id2->id4
c  no gluons are involved.


#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"

      integer lkup
      common /localkup/ lkup(numParticles,7,maxNumSubProc)

      integer listposition
      integer numresets
      save listposition
      save numresets

      data numresets /-1/
      data listposition /0/

      if (colstruc.eq.-1) then  ! we are resetting
c       write(*,*) 'we are resetting'
        listposition=0
        numresets=numresets+1
      else if (colstruc.eq.-2) then
c       don't do anything, just return the number of subprocesses.
      else
c increment the counter regardless of whether or not it's the 1st time through.
        listposition=listposition+1

c       we fill info for this subprocess,if required
        if(numresets.eq.0) then
          call fillColoredPartons_WWW(iflav,
     1                               listposition+numdecay)
        endif       ! numresets.eq.0
      endif         
      FL_WWW=listposition
      end
c******************************************************************************
c
c   end function TRIBOSFL_ZH
c
c******************************************************************************


c*****************************************************************************
c
c    begin  subroutine fillColoredPartons_WWW
c
c*****************************************************************************


      subroutine fillColoredPartons_WWW(iflav,listposition)

c  assigns values to the variables in the common block localHEPUP
c  in particular, this subroutine assigns values to those variables that 
c  will be stored in the lookup tables generated by writeHEPUPtable.  
c  As the name suggests, this routine only stores the information for the 
c  colored partons.  Particles without color will be dealt with in the 
c  subroutine fillColorless.

      IMPLICIT NONE

c  include files
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
#include "VBFNLO/utilities/process.inc"

c  input parameters
      INTEGER iflav(2),id1,id2,id3,id4,listposition !2 quark flavors and subprocess number

c  local variables
      INTEGER i
      
      id1=iflav(1)
      id2=iflav(2)
   
      select case(process)
      case(WWZ,ZZWP,ZZWM,WWWM,WWWP) 
         lnup(listposition)=numParticles+3
      end select

      listup(1,listposition)=-1   !incoming quarks
      listup(2,listposition)=-1
 
      lidup(1,listposition)=id1     ! the 2 quarks
      lidup(2,listposition)=id2

      if (id1.gt.0) then
        licolup(1,1,listposition)=501
        licolup(2,1,listposition)=0
        licolup(1,2,listposition)=0
        licolup(2,2,listposition)=501
      else
        licolup(1,1,listposition)=0
        licolup(2,1,listposition)=501
        licolup(1,2,listposition)=501
        licolup(2,2,listposition)=0
      endif

      end  ! fillColoredPartons_WWW
c*****************************************************************************
c
c    end subroutine fillColoredPartons_WWW
c
c*****************************************************************************


      subroutine vtowww_reset
      implicit none
#include "tensor.inc"
      double complex zero
      parameter (zero=(0d0,0d0))
      integer j,jj,mu,nu

      do mu = 0,5
         do j = 1,3
            do jj =1,3
              WWW_aww(mu,j,jj)  = zero
              WWW_zww(mu,j,jj)  = zero
            enddo
            wptowww(mu,j) = zero
            wmtowww(mu,j) = zero
         enddo
      enddo
      print*,' precalculated tensors initialized to ',zero 
      return
      end
