*****************************************************************************
c
c    begin function m2s_qqWWZ
c
c*****************************************************************************
c
c       Adapted from m2s_wbfzh by D. Zeppenfeld
c
c       this function computes |ME|^2 for qq-> ve e+ mu-vm~ ta- ta+ ("WWZ")
c	at LO and virtual corrections via call of "qqWWZ.f"
c
c	For the decay Z -> ta- ta+ random helicity summation is employed
c
c ------------------------------------------------------------------------

      DOUBLE PRECISION FUNCTION m2s_qqWWZ(
     &      bos,  !in:  XXX Boson identifier,X=1 photon, X=2 Z, X=3,4 W+-
     &      nlo,  !in:  Born only for nlo=0, +interference for 1
     &      xi,   !in:  Feynman x parameters of incoming fermions
     &      p,    !in:  momenta associated with external fermions
     &      v,    !in:  momenta associated with gauge boson dec. fermions
     &      rn,   !in:  random number array
     &      NW    !in:  = 1:  NW Approximation, = 0: Full calculation
     &                        )
      IMPLICIT NONE

c declare global variables
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/scales.inc"
#include "VBFNLO/utilities/lha.inc"
#include "tensor.inc"

c declare input/output variables
      INTEGER bos, nlo, NW
      DOUBLE PRECISION xi(nx), p(0:3,max_p), v(0:3,n_v,3), rn(1), m2s

c declare external functions
      DOUBLE PRECISION dotrr
      DOUBLE COMPLEX dotrc
      INTEGER FL_VVV
      EXTERNAL dotrr, dotrc
      EXTERNAL FL_VVV

c  helicity selection
      INTEGER h
      COMMON /hcount / h

      COMPLEX*16 zero
      PARAMETER (zero=(0d0,0d0))

      DOUBLE PRECISION q_sf, qq
      INTEGER init/0/, I, J, mu, nu, id
      SAVE init

c declare variables for summation process
      INTEGER ires

      INTEGER FSIGN(2+n_v), physToDiag(2), nmin, nmax
      SAVE nmax

c store contributions from subprocess ip in res(ip) where
      DOUBLE PRECISION res(maxnumsubproc)

c declare parton distribution variables
      DOUBLE PRECISION pdf(-6:6,2)

c define program switches
      LOGICAL ldebug
      PARAMETER (ldebug=.false.)

c variables for leshouches interface
      DOUBLE PRECISION weight,rnumb,RandomNumber

c ------------- BEGIN CODE -------------------

      if ( init .eq. 0 ) then

         write(6,*) " "
         write(6,*) "W+W-Z amplitude square information:"
         write(6,*) "-----------------------------------------------"
         write(6,*) " "
         call vtowwz_reset

         if (nfl.eq.4) then
            print*," External b-quark contributions are excluded"
         elseif (nfl.eq.5) then
            print*," External b-quark contributions are included"
         else
            print*," Check setting of nfl in global.inc "
         endif
         h = 1
         init = 1
         do i = 1,maxnumsubproc
            res(i) = 0
         enddo
      else
         do i = 1,nmax
            res(i) = 0
         enddo
      endif

      do mu = 0,3
         qp(mu,1) = v(mu,1,1)+v(mu,2,1)   !W+
         qm(mu,1) = v(mu,3,1)+v(mu,4,1)   !W-
         qz(mu,1) = v(mu,5,1)+v(mu,6,1)   !Z
         qww(mu,1) = qp(mu,1) + qm(mu,1)
         qwwz(mu,1) = qww(mu,1) + qz(mu,1)
      enddo

      qp(4,1) = dotrr(qp(0,1),qp(0,1))
      qm(4,1) = dotrr(qm(0,1),qm(0,1))
      qz(4,1) = dotrr(qz(0,1),qz(0,1))
      qww(4,1) = dotrr(qww(0,1),qww(0,1))
      qwwz(4,1) = dotrr(qwwz(0,1),qwwz(0,1))

c for WWZ->6l precalculate A->WWZ,Z->WWZ etc..

c lepton spinors and W+, W-, Z polarisation vectors
c lepton helicities for Z -> ta- ta+ not fixed
c -> sum over all possible helicities in |M|**2
 
c select helicity: h ... random number for lepton helicity combination (h=1:2)
        h = mod(h,2) + 1
	it = (-1)**(h+1)

c...Les Houches interface
        if (lha.and..not.doNLO) then
           helicity(1)=-1
           helicity(2)= 1
           helicity(3)=-1
           helicity(4)= 1
           helicity(5)=-it
           helicity(6)= it
        endif

      CALL OXXXXX(v(0,1,1),ZERO ,-1,1,wve(1,1)) !W(1,3))          !ve 
      CALL IXXXXX(v(0,2,1),ZERO ,1,-1,wep(1,1)) !W(1,4))          !e+        
      CALL OXXXXX(v(0,3,1),ZERO ,-1,1,wmu(1,1)) !W(1,5))          !mu-      
      CALL IXXXXX(v(0,4,1),ZERO ,1,-1,wvm(1,1)) !W(1,6))          !vm~
      CALL OXXXXX(v(0,5,1),LMASS ,-it,1,ztm(1,1)) !W(1,7))        !ta-      
      CALL IXXXXX(v(0,6,1),LMASS ,it,-1,ztp(1,1)) !W(1,8))        !ta+

      CALL JIOXXX(wep(1,1),wve(1,1),GWF,WMASS,WWIDTH, wp(1,1))  !W(1,9)  W+
      CALL JIOXXX(wvm(1,1),wmu(1,1),GWF,WMASS,WWIDTH, wm(1,1))  !W(1,10) W-
      CALL JIOXXX(ztp(1,1),ztm(1,1),GAL,ZERO ,ZERO  ,atau(1,1)) !W(1,11) A
      CALL JIOXXX(ztp(1,1),ztm(1,1),GZL,ZMASS,ZWIDTH,ztau(1,1)) !W(1,14) Z

         if (abs(nlo).eq.5 .or. nlo.eq.-4) then
c wpp=wp-xp*qp, wmp=wm-xm*qm, ztaup=ztau-xz*qz, ataup=atau-xa*qz
c in eps~ * q_WW = 0
            qq = qp(0,1)*qww(0,1)-qp(1,1)*qww(1,1)-qp(2,1)*qww(2,1)
     &          -qp(3,1)*qww(3,1)
            xp = dotrc(qww(0,1),wp(1,1))/qq
            qq = qm(0,1)*qww(0,1)-qm(1,1)*qww(1,1)-qm(2,1)*qww(2,1)
     &          -qm(3,1)*qww(3,1)
            xm = dotrc(qww(0,1),wm(1,1))/qq
            qq = qz(0,1)*qww(0,1)-qz(1,1)*qww(1,1)-qz(2,1)*qww(2,1)
     &          -qz(3,1)*qww(3,1)
            xz = dotrc(qww(0,1),ztau(1,1))/qq
            xa = dotrc(qww(0,1),atau(1,1))/qq
            do mu = 0,3
               wpp(mu+1,1) = wp(mu+1,1) - xp*qp(mu,1)
               wmp(mu+1,1) = wm(mu+1,1) - xm*qm(mu,1)
               ztaup(mu+1,1) = ztau(mu+1,1) - xz*qz(mu,1)
               ataup(mu+1,1) = atau(mu+1,1) - xa*qz(mu,1)
            enddo
            do mu = 5,6
               wpp(mu,1) = wp(mu,1)
               wmp(mu,1) = wm(mu,1)
               ztaup(mu,1) = ztau(mu,1)
               ataup(mu,1) = atau(mu,1)
            enddo
         endif

c leptonic tensors, last entry defines whether the non resonant diagrams
c contribute ( = 0: all diagrams, = 1: no non resonant diagrams)
      call TRIBOSatoww(v(0,1,1),1,aww(0,1),NW)   !A -> W+ W- -> ve e+ mu- vm~
      call TRIBOSztoww(v(0,1,1),1,zww(0,1),NW)   !Z -> W+ W- -> ve e+ mu- vm~

      call wptowz(v(0,1,1),1,h,wpwz(0,1),NW) !W+ -> W+ Z -> ve e+ ta- ta+
      call wmtowz(v(0,1,1),1,h,wmwz(0,1),NW) !W- -> W- Z -> mu- vm~ ta- ta+
      call atowwz(v(0,1,1),1,h,awwz(0,1),NW) !A -> WWZ -> ve e+ mu- vm~ ta- ta+
      call ztowwz(v(0,1,1),1,h,zwwz(0,1),NW) !Z -> WWZ -> ve e+ mu- vm~ ta- ta+

c call PDF subroutine in order to determine parton
c distributions in the incoming (anti)protons.
      q_sf = sqrt(mufsq(1,1))

      call pdfproton( xi(1), q_sf, pdf(-6,1) )        ! f_a(x1)
      call pdfproton( xi(2), q_sf, pdf(-6,2) )        ! f_a(x2)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     Now sum over the subprocesses contributing to Z production

      nmax = FL_VVV(fsign,-1)    !reset counter for subprocesses to 0

C*******************  q1 qbar2 ---> W W Z   **************************

c   physToDiag(ext.momentum label) = Feynman diagram label

      physToDiag(1)=1     !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=2

      fsign(1) = 1
      fsign(2) = -1
      fsign(3) = 1
      fsign(4) = -1
      fsign(5) = 1
      fsign(6) = -1
      fsign(7) = 1
      fsign(8) = -1

      call qq_WWZ(xi,p,v,physToDiag,fsign,bos,nlo,
     1            pdf,res,nmin,nmax, NW)

C*******************  q1bar q2 ---> W W Z   **********************

      physToDiag(1)=2    !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=1

      fsign(1) = 1
      fsign(2) = -1

      call qq_WWZ(xi,p,v,physToDiag,fsign,bos, nlo,
     1            pdf,res,nmin,nmax, NW)

c*****************  end of process evaluation  *********************

      if (init.eq.1) then
         init = init+1
      endif

      m2s = 0
      DO IRES = 1,NMAX
         m2s = m2s + RES(IRES)
      ENDDO

c...Les Houches interface - the most propable subprocess   
      if (lha.and..not.doNLO) then
         i=0
         weight=0.d0
         rnumb=RandomNumber()
         do while((i.le.nmax).and.(weight.le.rnumb*m2s))
            i=i+1
            weight=weight+res(i)
            iprocess=i
         enddo
      endif

      m2s = m2s * 2d0     ! factor 2 for random helicity summation

      m2s_qqWWZ = m2s
      
      RETURN
      END

c*****************************************************************************
c
c    end function m2s_qqWWZ
c
c*****************************************************************************

c******************************************************************
c
c   begin subroutine qq_WWZ
c
c*****************************************************************

      subroutine qq_WWZ(xi,p,v,physToDiag,fsign,bos, nlo,
     1                  pdf,res,nmin,nmax, NW)
      IMPLICIT NONE

#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"

      DOUBLE PRECISION p(0:3,n_p), v(0:3,n_v), xi(nx)
      DOUBLE PRECISION pdf(-6:6,2), res(maxnumsubproc)
      INTEGER physToDiag(2), fsign(2+n_v), bos, nlo, nmin, nmax
      INTEGER NW
c
c wbf_zh calls the amplitude square routines 
c             qqVV    for qq --> VV --> 6 leptons
c
c  INPUT:  p(0:3,4)        external physical parton momenta
c          v(0:3,nv)       Z decay momenta, for Higgs production only
c                          the sum q(mu) = v(mu,1)+...+v(mu,nv) is needed
c          physToDiag(4)   physToDiag(ext.mom. label) = Feynman diagram label
c          fsign           sign factors for amplitude calls; see qqZqq
c          nlo             nlo = 0: calculate LO only
c                          nlo = 1: calculate full NLO (Born + interference)
c          NW              NW = 1: NW approximation, NW = 0: full calc.
c  OUTPUT:
c          uuvv(ID)   output in format as in qqZqq(j), but corresponding
c          etc.            to res(*) in m2s_qqZqq
c     or   res(k)   
c
c  In and Output
c    nmin, nmax            range of process numbers for this call
c
      DOUBLE PRECISION N                              ! color factors
      PARAMETER(N=3d0)
c
c alfas, scales etc
#include "VBFNLO/utilities/scales.inc"
#include "tensor.inc"
#include "VBFNLO/utilities/coupl.inc"

c local variables
      DOUBLE PRECISION uuwwz, ddwwz, uuwwz2, ddwwz2
      DOUBLE PRECISION pbar(0:3,2+n_v), dotrr, polcol
      DOUBLE PRECISION NCmatrixelt(0:1,0:1)
      DOUBLE COMPLEX zero
      INTEGER iflav(2), diagToPhys(2), FL_VVV
      EXTERNAL dotrr, FL_VVV

      LOGICAL ChargedCurrent, sametype, oneAntiparticle
      LOGICAL ldebug
      PARAMETER (ldebug=.false.)
      PARAMETER (zero = (0d0,0d0) )
      INTEGER i,if1,if2,icc1,j,k,mu

      uuwwz = 0d0
      uuwwz2 = 0d0
      ddwwz = 0d0
      ddwwz2 = 0d0

      nmin = nmax+1
      do i = 1,2
         diagToPhys(physToDiag(i)) = i
      enddo

      polcol = 1d0/(4d0*N**2*xi(1)*xi(2))

      do mu = 0,3
         do i = 1,2
            pbar(mu,physToDiag(i))=p(mu,i)
         enddo
      enddo

      do mu = 0,3             ! kinematics for WWZ --> 6 lepton decay
         pbar(mu,3) = v(mu,1) ! ve
         pbar(mu,4) = v(mu,2) ! e+
         pbar(mu,5) = v(mu,3) ! mu-
         pbar(mu,6) = v(mu,4) ! vm~
         pbar(mu,7) = v(mu,5) ! tau-
         pbar(mu,8) = v(mu,6) ! tau+
      enddo

#ifdef WITH_MADGRAPH
      if (ldebug) then
         call suu_wwz(pbar,fsign,uuwwz2,nw)
         call sdd_wwz(pbar,fsign,ddwwz2,nw)
      endif
#endif

      call QQWWZ(pbar,fsign,nlo,1,uuwwz,ddwwz,NW)

      if (ldebug) then
         print*,"uuwwz =   ",uuwwz
         print*,"uuwwz2 =  ",uuwwz2
         print*,"ddwwz =   ",ddwwz
         print*,"ddwwz2 =  ",ddwwz2
      endif

      NCmatrixelt(0,0)=uuwwz
      NCmatrixelt(1,1)=ddwwz

      do if1=1,nfl
            iflav(1)=if1*fsign(physToDiag(1))
            iflav(2)=if1*fsign(physToDiag(2))
            k=FL_VVV(iflav,1)

            res(k)=pdf(sign1*iflav(1),1)
     &            *pdf(sign2*iflav(2),2)
     &            *NCmatrixelt(mod(if1,2),mod(if1,2))*polcol

      enddo 

      nmax = FL_VVV(iflav,-2)

      end
c******************************************************************
c
c   end subroutine qq_WWZ
c
c*****************************************************************

c******************************************************************
c
c   begin function FL_VVV
c
c*****************************************************************
      INTEGER function FL_VVV(iflav,colstruc)

      IMPLICIT NONE

      INTEGER iflav(2),colstruc    ! input for color and flavor assignment
c  fill hepup color and flavor assignments for VVV processes with
c  two incoming quarks only

c  id1,2 are flavor identifiers for incoming quarks
c  colstruc = is irrelevant here
c
c  color structure of these interactions is trivial
c  and colstruc was included only to make this program 
c  more consistent with the others.

c  Note that colstruc is also a flag:
c  if colstruc=-1, then we are resetting.

c  The color structure is always the same:
c  color always flows from id1->id2
c  no gluons are involved.

#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
      
      INTEGER lkup
      COMMON /localkup/ lkup(numParticles,7,maxNumSubProc)
 
      INTEGER listposition
      INTEGER numresets
      SAVE listposition
      SAVE numresets

      DATA numresets /-1/
      DATA listposition /0/

      if (colstruc.eq.-1) then  ! we are resetting
c       write(*,*) "we are resetting"
        listposition=0
        numresets=numresets+1
      else if (colstruc.eq.-2) then
c       don"t do anything, just return the number of subprocesses.
      else
c increment the counter regardless of whether or not it"s the 1st time through.
        listposition=listposition+1

c       we fill info for this subprocess,if required
        if(numresets.eq.0 .and. lha) then
          call fillColoredPartons_VVV(iflav,
     1                               listposition+numdecay)
        endif       ! numresets.eq.0
      endif         
      FL_VVV=listposition
      end
c******************************************************************************
c
c   end function FL_VVV
c
c******************************************************************************

c*****************************************************************************
c
c    begin  subroutine fillColoredPartons_VVV
c
c*****************************************************************************

      subroutine fillColoredPartons_VVV(iflav,listposition)

c  assigns values to the variables in the common block localHEPUP
c  in particular, this subroutine assigns values to those variables that 
c  will be stored in the lookup tables generated by writeHEPUPtable.  
c  As the name suggests, this routine only stores the information for the 
c  colored partons.  Particles without color will be dealt with in the 
c  subroutine fillColorless.

      IMPLICIT NONE

c  include files
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
#include "VBFNLO/utilities/process.inc"

c  input parameters
      INTEGER iflav(2),id1,id2,id3,id4,listposition !2 quark flavors and subprocess number

c  local variables
      INTEGER i
      
      id1=iflav(1)
      id2=iflav(2)
   
      select case(process)
      case(WWZ,ZZWP,ZZWM) 
         lnup(listposition)=numParticles+3
      end select

      listup(1,listposition)=-1   !incoming quarks
      listup(2,listposition)=-1
 
      lidup(1,listposition)=id1     ! the 2 quarks
      lidup(2,listposition)=id2

      if (id1.gt.0) then
        licolup(1,1,listposition)=501
        licolup(2,1,listposition)=0
        licolup(1,2,listposition)=0
        licolup(2,2,listposition)=501
      else
        licolup(1,1,listposition)=0
        licolup(2,1,listposition)=501
        licolup(1,2,listposition)=501
        licolup(2,2,listposition)=0
      endif

      end  ! fillColoredPartons_VVV
c*****************************************************************************
c
c    end subroutine fillColoredPartons_VVV
c
c*****************************************************************************

      subroutine vtowwz_reset

      IMPLICIT NONE

#include "tensor.inc"

      COMPLEX*16 zero
      PARAMETER (zero=(0d0,0d0))
      INTEGER j,jj,mu,nu

      do mu = 0,5
         do j = 1,3
            aww(mu,j)  = zero
            zww(mu,j)  = zero
            awwz(mu,j) = zero
            zwwz(mu,j) = zero
            wpwz(mu,j) = zero
            wmwz(mu,j) = zero
         enddo
      enddo
      print*," precalculated tensors initialized to ",zero 
      return
      end
