c****************subroutine QQWWZ *************************************
c
c	Vera Hankele, <vera@particle.physik.uni-karlsruhe.de>
c	Initial version:  2007, January
c	Last modified: 2007, August
C
C  QQVVV calculates the matrix elements**2 for 
C        q1 q2    ---->   W+ W- Z,   
C         W+ ---> f1-bar f2, W- ---> f3-bar f4, Z ---> f5-bar f6
C
C  and crossing related processes. Pauli interference terms for
c  identical fermions are neglected.
c
C  This code is modified to allow for virtual corrections, more precisely
C  the interference of Born with the finite part of virtual diagrams
C  for 
c
c  INPUT:  NLO = 1       return uuww = |M_born|^2 + 2Re(M_Born^* M_virt)
c          NLO = 0       return uuww = |M_born|^2   etc.
c  for steering the calculation of finite box and pentagon contributions
c  the following additional options are implemented
c          NLO = +4      set all finite box and pentagon contributions to 0
c                        but include cvirt*M_Born contribution
c          NLO = +5      true finite pentagons = 0, all other virtual
c                        contributions included, including leading q^mu/m_V
c                        terms of pentagons which can be related to boxes
c       remaining cases:   return uuww = 2Re(M_born^* M_virt). more precisely
c       the piece which is missing for the full NLO=1 when calling +|NLO|     
c          NLO = -4      finite Box + q^mu/m_V terms of Pentagon contributions
c          NLO = -5      true finite Pentagon contributions only
c
c          N = 1         Momentum configuration 1 (LO)
c          N = 2         Momentum configuration 2: emission of parton 1
c          N = 3         Momentum configuration 3: emission of parton 2
c
c*********************************************************************

      subroutine qqWWZ(pbar,sign, nlo, N, uuwwz, ddwwz, NW)

      implicit none

#include "VBFNLO/utilities/global.inc"
#include "tensor.inc"
#include "VBFNLO/utilities/scales.inc"

c     electroweak couplings are taken from KOPPLN
      double precision  clr, xm2, xmg, b, v, a
      COMMON /BKOPOU/   CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),
     1                  V(4,5),A(4,5)
      double precision  pbar(0:3,2+n_v)
      double precision  uuwwz, ddwwz, res(2),resv(2)
      double precision  p(0:3,2+n_v), pwpz(0:4), pwmz(0:4),
     1                  pwm(0:4), pww(0:4), fqp(0:4,2), fqm(0:4,2),
     2                  fqz(0:4,2), fq(0:4,2)
      double precision  fqmz(0:4,2), fqmp(0:4,2), fqzm(0:4,2)
      integer  sign(2+n_v), nlo, mu, i, j, k, isig, N
      integer  ifl(2)
      double complex psiwp(2,-1:1,2), psiwm(2,-1:1,2),
     1               psia(2,-1:1,2), psiz(2,-1:1,2),
     2               jwp(0:5,-1:1,2), jwm(0:5,-1:1,2),
     3               ja(0:5,-1:1,2), jz(0:5,-1:1,2)
      double complex psiwmz(2,-1:1,2), psiwmwp(2,-1:1,2),
     1               psizwm(2,-1:1,2), 
     2               jwmz(0:5,-1:1,2), jwmwp(0:5,-1:1,2),
     3               jzwm(0:5,-1:1,2)
      double complex jva(0:5,-1:1,2), jvz(0:5,-1:1,2)
      double complex jvwp(0:5,-1:1,2), jvwm(0:5,-1:1,2)
      double complex j5pm(0:3,3,-1:1),j5mp(0:3,3,-1:1)
      double complex j5za1(3), j5za2(3)
      double complex temp, temp1
      double complex mat(2,-1:1,10), matv(2,-1:1,10), mborn1, mborn2
      double complex madgr(10,-1:1,2)
      double complex mm(2,-1:1), mv(2,-1:1), zero
      double complex ma, mz, m1, m2, m3, m4
      double complex mva, mvz, mv1, mv2, mv3, mv4
      double complex psi(2,-1:1,2), jqq(0:5,-1:1)
      double complex dotcc, s1c, dotrc
      double precision dotrr, psumsq, temp2
      external dotcc, s1c, dotrr, dotrc, psumsq
      save ifl
      parameter (zero = (0d0,0d0) )

c variable for q_mu q_nu terms in the propagators of the massive vector bosons
      double complex zm2i(2:3)
      save zm2i

c  helicity selection
      INTEGER h
      COMMON /hcount / h

c variables for virtual corrections
      double precision scale
      double precision c2,c2o4pi
      parameter (c2=4d0/3d0, c2o4pi=c2/4d0/pi)
      logical lnlo, lbox, lpq, lpt, lpent, ldebug, ldebugm, linit
      integer wardidtest1, wardidtest2, wardidtest3
      data linit /.true./
      save linit
      parameter(ldebugm = .false., ldebug=.false.)
      integer pentcount, pentcount2
      data pentcount /0/
      data pentcount2 /0/
      save pentcount, pentcount2

c variables for pacos implementation of the pentagon routine
      double complex j5wpwm(1:6), j5wmwp(1:6)
      double complex jbwpwm(1:6), jbwmwp(1:6)
      double complex resultgauge, resultgaugeb, borntest(3), penttest(3)
      double precision minus_qp(0:3),minus_qm(0:3),minus_qz(0:3)
      double precision p1(0:3), p2(0:3) 
      double complex pentgauge1(1:3), pentgauge2(1:3)
      double complex pentcontr1(3), pentcontr2(3), pentcontr3(3)

c variables for test purposes
      double precision qmu(0:3), sc, q1sqc, q2sqc, tc, p1p2
      double precision psq, qsq, lsq, tsq, pq, pl, pt, ql, qt, lt
      logical veratest
      parameter (veratest=.false.)
      integer NW !NW = 1 means Narrow width approximaion
                 !NW = 0 means full calculation

c---------------------------- BEGIN CODE -----------------------

      lnlo =  NLO.ne.0       ! include some virtual stuff if T
      lbox = (NLO.eq.-4) .or. (NLO.eq.1) .or. (NLO.eq.5) ! call boxline if T
      lpt =  (NLO.eq.1) .or. (NLO.eq.-5)   ! true pentagon contribution
      lpq =  (NLO.eq.5) .or. (NLO.eq.-4)   ! q^mu/m_V terms of Pentagon
      lpent = lpt .or. lpq

c define flavors of external quarks for the 2 subprocesses
      if (linit) then
         linit = .false.
         k = 1                  ! uuwwz: u = 3
         ifl(k) = 3
         k = 2                  ! ddwwz: d = 4
         ifl(k) = 4
         zm2i(2) = 1d0/dcmplx(xm2(2),-xmg(2))
         zm2i(3) = 1d0/dcmplx(xm2(3),-xmg(3))
      endif

      do k = 1,2
         do isig = -1,1,2
            do i = 1,10
                  mat(k,isig,i) = (0.0d0,0.0d0)
                  matv(k,isig,i) = (0.0d0,0.0d0)
            enddo
         enddo
      enddo

c define the internal momenta
      do mu = 0,3
         do i = 1,2+n_v
            p(mu,i) = pbar(mu,i)*sign(i)
         enddo
         pwpz(mu) = qp(mu,N) + qz(mu,N)
         pwmz(mu) = qm(mu,N) + qz(mu,N)
      enddo
      pwpz(4) = pwpz(0)**2-pwpz(1)**2-pwpz(2)**2-pwpz(3)**2
      pwmz(4) = pwmz(0)**2-pwmz(1)**2-pwmz(2)**2-pwmz(3)**2

c------------------------------------------------

c get the external quark spinors (including factor sqrt(2E) )
      call psi0m(2,pbar(0,1),sign(1),psi)

c get other bra and ket vectors
      call bra2c(psi(1,-1,2),.true.,p(0,2),-1,qp(0,N),wp(1,N),
     &           psiwp(1,-1,2),fqp(0,2))
      call bra2c(psi(1,-1,2),.true.,p(0,2),-1,qm(0,N),wm(1,N),
     &           psiwm(1,-1,2),fqm(0,2))
      call ket2c(psi(1,-1,1),.true.,p(0,1),-1,qp(0,N),wp(1,N),
     &           psiwp(1,-1,1),fqp(0,1))
      call ket2c(psi(1,-1,1),.true.,p(0,1),-1,qm(0,N),wm(1,N),
     &           psiwm(1,-1,1),fqm(0,1))

      do isig =-1,1,2

         call bra2c(psi(1,isig,2),.true.,p(0,2),isig,qz(0,N),
     &                 atau(1,N), psia(1,isig,2),fqz(0,2))
         call bra2c(psi(1,isig,2),.true.,p(0,2),isig,qz(0,N),
     &                 ztau(1,N), psiz(1,isig,2),fqz(0,2))
         call ket2c(psi(1,isig,1),.true.,p(0,1),isig,qz(0,N),
     &                 atau(1,N), psia(1,isig,1),fqz(0,1))
         call ket2c(psi(1,isig,1),.true.,p(0,1),isig,qz(0,N),
     &                 ztau(1,N), psiz(1,isig,1),fqz(0,1))
      enddo

**************************************************************
*
* Calculation of the born matrix element
*
**************************************************************

c Calculation of processes qq -> W W V ---------------------

c   u u -> W W Z
      m1= -s1c(psiwm(1,-1,2),ztau(1,N),.true.,-1,psiwp(1,-1,1))  !WZW
     1         *clr(4,2,-1)
     2    -s1c(psiwm(1,-1,2),wp(1,N),.true.,-1,psiz(1,-1,1))*clr(3,2,-1) !ZWW
     3    -s1c(psiz(1,-1,2),wm(1,N),.true.,-1,psiwp(1,-1,1))*clr(3,2,-1) !WWZ

      mat(1,-1,1) = m1 * clr(3,3,-1)**2 ! uu -> WWZ

c   d d -> W W Z
      m2= -s1c(psiwp(1,-1,2),ztau(1,N),.true.,-1,psiwm(1,-1,1))
     1         *clr(3,2,-1)
     2    -s1c(psiwp(1,-1,2),wm(1,N),.true.,-1,psiz(1,-1,1))*clr(4,2,-1)
     3    -s1c(psiz(1,-1,2),wp(1,N),.true.,-1,psiwm(1,-1,1))*clr(4,2,-1)

      mat(2,-1,1) = m2 * clr(3,3,-1)**2 ! uu -> WWZ

c qq -> WWA and qq -> A A/Z(->WW) contributions are not calculated in 
c narrow width approximation
      if (NW.eq.0) then

c   u u -> W W A
      m3= -s1c(psiwm(1,-1,2),atau(1,N),.true.,-1,psiwp(1,-1,1))
     1          *clr(4,1,-1)
     2    -s1c(psiwm(1,-1,2),wp(1,N),.true.,-1,psia(1,-1,1))*clr(3,1,-1)
     3    -s1c(psia(1,-1,2),wm(1,N),.true.,-1,psiwp(1,-1,1))*clr(3,1,-1)

      mat(1,-1,2) = m3 * clr(3,3,-1)**2 ! uu -> WWA

c   d d -> W W A
      m4= -s1c(psiwp(1,-1,2),atau(1,N),.true.,-1,psiwm(1,-1,1))
     1         *clr(3,1,-1)
     2    -s1c(psiwp(1,-1,2),wm(1,N),.true.,-1,psia(1,-1,1))*clr(4,1,-1)
     3    -s1c(psia(1,-1,2),wp(1,N),.true.,-1,psiwm(1,-1,1))*clr(4,1,-1)

      mat(2,-1,2) = m4 * clr(3,3,-1)**2 ! dd -> WWA

      endif !NW.eq.0

c Calculation of processes qq -> V V -> W W Z -------------
      if (NW.eq.0) then
c qq -> A A/Z(-> W W)
      call curr6(1,psi(1,-1,2),p(0,2),
     &           psia(1,-1,1),fqz(0,1), ja(0,-1,1) )

c contract with atoww/ztoww tensor
      do isig = -1,1,2
         m1 = -dotcc(aww(0,N),ja(0,isig,1))
         m2 = -dotcc(zww(0,N),ja(0,isig,1))

         do k = 1,2
            mat(k,isig,3) = (m1*clr(ifl(k),1,isig)
     1                     +  m2*clr(ifl(k),2,isig))
     2                       * clr(ifl(k),1,isig)  ! qq -> AWW
         enddo
      enddo

c qq -> A/Z(-> W W) A
      call curr6(1,psia(1,-1,2),fqz(0,2),
     &           psi(1,-1,1),p(0,1), ja(0,-1,2) )

c contract with atoww/ztoww tensor
      do isig = -1,1,2
         m1 = -dotcc(aww(0,N),ja(0,isig,2))
         m2 = -dotcc(zww(0,N),ja(0,isig,2))
         do k = 1,2
            mat(k,isig,4) = (m1*clr(ifl(k),1,isig)
     1                     +  m2*clr(ifl(k),2,isig))
     2                       * clr(ifl(k),1,isig)  ! qq -> WWA
         enddo
      enddo

      endif !NW.eq.0

c qq -> Z A/Z(-> W W)
      call curr6(1,psi(1,-1,2),p(0,2),
     &           psiz(1,-1,1),fqz(0,1), jz(0,-1,1) )

c contract with atoww/ztoww tensor
      do isig = -1,1,2
         m1 = -dotcc(aww(0,N),jz(0,isig,1))
         m2 = -dotcc(zww(0,N),jz(0,isig,1))
         do k = 1,2
            mat(k,isig,5) = (m1*clr(ifl(k),1,isig)
     1                     +  m2*clr(ifl(k),2,isig))
     2                       * clr(ifl(k),2,isig)  ! qq -> ZWW
         enddo
      enddo

c qq -> A/Z(-> W W) Z
      call curr6(1,psiz(1,-1,2),fqz(0,2),
     &              psi(1,-1,1),p(0,1), jz(0,-1,2) )

c contract with atoww/ztoww tensor
      do isig = -1,1,2
         m1 = -dotcc(aww(0,N),jz(0,isig,2))
         m2 = -dotcc(zww(0,N),jz(0,isig,2))
         do k = 1,2
            mat(k,isig,6) = (m1*clr(ifl(k),1,isig)
     1                     +  m2*clr(ifl(k),2,isig))
     2                       * clr(ifl(k),2,isig)  ! qq -> WWZ
         enddo
      enddo

c qq -> W+ W-(-> W Z)
      call curr6(-1, psiwp(1,-1,2), fqp(0,2),
     &              psi(1,-1,1), p(0,1), jwp(0,-1,2) )

      call curr6(-1, psi(1,-1,2), p(0,2),
     &              psiwp(1,-1,1), fqp(0,1), jwp(0,-1,1) )

c contract with wmwz tensor
      m1 = -dotcc(wmwz(0,N),jwp(0,-1,1)) ! uu -> W+ W-(-> W Z)
      m2 = -dotcc(wmwz(0,N),jwp(0,-1,2)) ! dd -> W-(-> W Z) W+

c additional contributions due to q_mu q_nu/ m^2 terms in the propagator
      m3 = zm2i(3)*dotrc(pwmz(0),wmwz(0,N))*dotrc(pwmz(0),jwp(0,-1,1))
      m4 = zm2i(3)*dotrc(pwmz(0),wmwz(0,N))*dotrc(pwmz(0),jwp(0,-1,2))

      mat(1,-1,7) = m1 * clr(3,3,-1)**2 + m3 * clr(3,3,-1)**2
      mat(2,-1,7) = m2 * clr(3,3,-1)**2 + m4 * clr(3,3,-1)**2

c qq -> W- W+(-> W Z)
      call curr6(-1, psiwm(1,-1,2), fqm(0,2),
     &              psi(1,-1,1), p(0,1), jwm(0,-1,2) )

      call curr6(-1, psi(1,-1,2), p(0,2),
     &              psiwm(1,-1,1), fqm(0,1), jwm(0,-1,1) )

c contract with wpwz tensor
      m1 = -dotcc(wpwz(0,N),jwm(0,-1,2)) ! uu -> W+(-> W Z) W-
      m2 = -dotcc(wpwz(0,N),jwm(0,-1,1)) ! dd -> W- W+(-> W Z)

c additional contributions due to q_mu q_nu/ m^2 terms in the propagator
      m3 = zm2i(3)*dotrc(pwpz(0),wpwz(0,N))*dotrc(pwpz(0),jwm(0,-1,2))
      m4 = zm2i(3)*dotrc(pwpz(0),wpwz(0,N))*dotrc(pwpz(0),jwm(0,-1,1))

      mat(1,-1,8) = m1 * clr(3,3,-1)**2 + m3 * clr(3,3,-1)**2
      mat(2,-1,8) = m2 * clr(3,3,-1)**2 + m4 * clr(3,3,-1)**2

c Calculation of processes qq -> A/Z -> W W Z ---------------------
      call curr6(1,psi(1,-1,2),p(0,2),psi(1,-1,1),p(0,1),jqq(0,-1))

c contract with atowwz/ztowwz tensor
      do isig = -1,1,2
         ma = dotcc(awwz(0,N),jqq(0,isig))
         mz = dotcc(zwwz(0,N),jqq(0,isig))
         do k = 1,2
            mat(k,isig,9)   = ma*clr(ifl(k),1,isig)
            mat(k,isig,10)  = mz*clr(ifl(k),2,isig)
         enddo
      enddo

*************************************************************
*
* Calculation of the virtual contributions
*
*************************************************************

c Calculation of processes qq -> W W V ---------------------
#ifdef WITH_NLO
      if (lpent) then
c calculate pentagon contributions
         scale = -2d0*dotrr(p(0,1),p(0,2))

         if (NLO.eq.1) then ! complete NLO

c calculate born matrix element and store it in mborn for gauge check:
      mborn1=mat(1,-1,1)+mat(1,-1,2)+mat(1,-1,3)+mat(1,-1,4)+mat(1,-1,5)
     1     +mat(1,-1,6)+mat(1,-1,7)+mat(1,-1,8)+mat(1,-1,9)+mat(1,-1,10)
      mborn2=mat(2,-1,1)+mat(2,-1,2)+mat(2,-1,3)+mat(2,-1,4)+mat(2,-1,5)
     1     +mat(2,-1,6)+mat(2,-1,7)+mat(2,-1,8)+mat(2,-1,9)+mat(2,-1,10)

            do mu = 0,3
               p1(mu) = pbar(mu,1)
               p2(mu) = pbar(mu,2)
               minus_qp(mu) = -qp(mu,1)
               minus_qm(mu) = -qm(mu,1)
               minus_qz(mu) = -p1(mu)-p2(mu)+qp(mu,1)+qm(mu,1)
            enddo

            call penlinemmm(p1(0),minus_qp(0),minus_qm(0),minus_qz(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wp(1,1),wm(1,1),ztau(1,1),
     2         -1,scale,4,1,pentcontr1,j5wpwm(3),borntest,jbwpwm(3))
            call penlinemmm(p1(0),minus_qp(0),minus_qm(0),minus_qz(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wp(1,1),wm(1,1),atau(1,1),
     2         -1,scale,9,-1,penttest,j5wpwm(6),borntest,jbwpwm(6))

            call penlinemmm(p1(0),minus_qp(0),minus_qz(0),minus_qm(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wp(1,1),ztau(1,1),wm(1,1),
     2         -1,scale,4,1,pentcontr2,j5wpwm(2),borntest,jbwpwm(2))
            call penlinemmm(p1(0),minus_qp(0),minus_qz(0),minus_qm(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wp(1,1),atau(1,1),wm(1,1),
     2         -1,scale,9,-1,penttest,j5wpwm(5),borntest,jbwpwm(5))

            call penlinemmm(p1(0),minus_qz(0),minus_qp(0),minus_qm(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),ztau(1,1),wp(1,1),wm(1,1),
     2         -1,scale,4,1,pentcontr3,j5wpwm(1),borntest,jbwpwm(1))
            call penlinemmm(p1(0),minus_qz(0),minus_qp(0),minus_qm(0),p2(0),
     1          psi(1,-1,2),psi(1,-1,1),atau(1,1),wp(1,1),wm(1,1),
     2         -1,scale,9,-1,penttest,j5wpwm(4),borntest,jbwpwm(4))

            call Wardtest(psi, p, wp(1,1),qp(0,1),wm(1,1),qm(0,1), 
     1         ztau(1,1), qz(0,1), pentcontr1, mborn1, wardidtest1, als(1,1))
            call Wardtest(psi, p, wp(1,1),qp(0,1),ztau(1,1), qz(0,1),
     1         wm(1,1),qm(0,1), pentcontr2, mborn1, wardidtest2, als(1,1))
            call Wardtest(psi, p, ztau(1,1), qz(0,1), wp(1,1),qp(0,1),
     1         wm(1,1),qm(0,1), pentcontr3, mborn1, wardidtest3, als(1,1))

            if ( (wardidtest1.eq.1) .or. (wardidtest2.eq.1) 
     1                     .or. (wardidtest3.eq.1) ) then
               do i = 1,6
                  j5wpwm(i)= (0.0d0, 0.0d0)
               enddo
c               pentcount = pentcount+1
            else
               do i = 1,6
                  j5wpwm(i)=j5wpwm(i)+(3*pi*(0d0,1d0)-4.0d0*pi**2/3.0d0
     1                                                 +7.0d0)*jbwpwm(i)
               enddo
c               pentcount2 = pentcount2+1
            endif

            call penlinemmm(p1(0),minus_qm(0),minus_qp(0),minus_qz(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wm(1,1),wp(1,1),ztau(1,1),
     2         -1,scale,4,1,pentcontr1,j5wmwp(3),borntest,jbwmwp(3))
            call penlinemmm(p1(0),minus_qm(0),minus_qp(0),minus_qz(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wm(1,1),wp(1,1),atau(1,1),
     2         -1,scale,9,-1,penttest,j5wmwp(6),borntest,jbwmwp(6))

            call penlinemmm(p1(0),minus_qm(0),minus_qz(0),minus_qp(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wm(1,1),ztau(1,1),wp(1,1),
     2         -1,scale,4,1,pentcontr2,j5wmwp(2),borntest,jbwmwp(2))
            call penlinemmm(p1(0),minus_qm(0),minus_qz(0),minus_qp(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wm(1,1),atau(1,1),wp(1,1),
     2         -1,scale,9,-1,penttest,j5wmwp(5),borntest,jbwmwp(5))

            call penlinemmm(p1(0),minus_qz(0),minus_qm(0),minus_qp(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),ztau(1,1),wm(1,1),wp(1,1),
     2         -1,scale,4,1,pentcontr3,j5wmwp(1),borntest,jbwmwp(1))
            call penlinemmm(p1(0),minus_qz(0),minus_qm(0),minus_qp(0),p2(0),
     1          psi(1,-1,2),psi(1,-1,1),atau(1,1),wm(1,1),wp(1,1),
     2         -1,scale,9,-1,penttest,j5wmwp(4),borntest,jbwmwp(4))

            call Wardtest(psi, p, wm(1,1),qm(0,1),wp(1,1),qp(0,1), 
     1         ztau(1,1), qz(0,1), pentcontr1, mborn2, wardidtest1, als(1,1))
            call Wardtest(psi, p, wm(1,1),qm(0,1),ztau(1,1), qz(0,1),
     1         wp(1,1),qp(0,1), pentcontr2, mborn2, wardidtest2, als(1,1))
            call Wardtest(psi, p, ztau(1,1), qz(0,1), wm(1,1),qm(0,1),
     1         wp(1,1),qp(0,1), pentcontr3, mborn2, wardidtest3, als(1,1))

            if ( (wardidtest1.eq.1) .or. (wardidtest2.eq.1) 
     1                     .or. (wardidtest3.eq.1) ) then
               do i = 1,6
                  j5wmwp(i)= (0.0d0, 0.0d0)
               enddo
c               pentcount = pentcount+1
            else
                do i = 1,6
                   j5wmwp(i)=j5wmwp(i)+(3*pi*(0d0,1d0)
     1                               -4.0d0*pi**2/3.0d0+7.0d0)*jbwmwp(i)
                enddo
c               pentcount2 = pentcount2+1
            endif

c            do i = 10,10000000,1000
c               if (pentcount2.eq.1d2*i) print*,"ratio, pent= ",
c     1              pentcount*1.0d0/(pentcount+pentcount2)
c            enddo

         elseif (NLO.eq.-5) then ! only true pentagon

c calculate born matrix element and store it in mborn for gauge check:
      mborn1=mat(1,-1,1)+mat(1,-1,2)+mat(1,-1,3)+mat(1,-1,4)+mat(1,-1,5)
     1     +mat(1,-1,6)+mat(1,-1,7)+mat(1,-1,8)+mat(1,-1,9)+mat(1,-1,10)
      mborn2=mat(2,-1,1)+mat(2,-1,2)+mat(2,-1,3)+mat(2,-1,4)+mat(2,-1,5)
     1     +mat(2,-1,6)+mat(2,-1,7)+mat(2,-1,8)+mat(2,-1,9)+mat(2,-1,10)

            do mu = 0,3
               p1(mu) = pbar(mu,1)
               p2(mu) = pbar(mu,2)
               minus_qp(mu) = -qp(mu,1)
               minus_qm(mu) = -qm(mu,1)
               minus_qz(mu) = -p1(mu)-p2(mu)+qp(mu,1)+qm(mu,1)
            enddo

            call penlinemmm(p1(0),minus_qp(0),minus_qm(0),minus_qz(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wpp(1,1),wmp(1,1),ztaup(1,1),
     2         -1,scale,4,1,pentcontr1,j5wpwm(3),borntest,jbwpwm(3))
            call penlinemmm(p1(0),minus_qp(0),minus_qm(0),minus_qz(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wpp(1,1),wmp(1,1),ataup(1,1),
     2         -1,scale,9,-1,penttest,j5wpwm(6),borntest,jbwpwm(6))

            call penlinemmm(p1(0),minus_qp(0),minus_qz(0),minus_qm(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wpp(1,1),ztaup(1,1),wmp(1,1),
     2         -1,scale,4,1,pentcontr2,j5wpwm(2),borntest,jbwpwm(2))
            call penlinemmm(p1(0),minus_qp(0),minus_qz(0),minus_qm(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wpp(1,1),ataup(1,1),wmp(1,1),
     2         -1,scale,9,-1,penttest,j5wpwm(5),borntest,jbwpwm(5))

            call penlinemmm(p1(0),minus_qz(0),minus_qp(0),minus_qm(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),ztaup(1,1),wpp(1,1),wmp(1,1),
     2         -1,scale,4,1,pentcontr3,j5wpwm(1),borntest,jbwpwm(1))
            call penlinemmm(p1(0),minus_qz(0),minus_qp(0),minus_qm(0),p2(0),
     1          psi(1,-1,2),psi(1,-1,1),ataup(1,1),wpp(1,1),wmp(1,1),
     2         -1,scale,9,-1,penttest,j5wpwm(4),borntest,jbwpwm(4))

            call Wardtest(psi, p, wpp(1,1),qp(0,1),wmp(1,1),qm(0,1), 
     1         ztaup(1,1), qz(0,1), pentcontr1, mborn1, wardidtest1, als(1,1))
            call Wardtest(psi, p, wpp(1,1),qp(0,1),ztaup(1,1), qz(0,1),
     1         wmp(1,1),qm(0,1), pentcontr2, mborn1, wardidtest2, als(1,1))
            call Wardtest(psi, p, ztaup(1,1), qz(0,1), wpp(1,1),qp(0,1),
     1         wmp(1,1),qm(0,1), pentcontr3, mborn1, wardidtest3, als(1,1))

            if ( (wardidtest1.eq.1) .or. (wardidtest2.eq.1) 
     1                     .or. (wardidtest3.eq.1) ) then
               do i = 1,6
                  j5wpwm(i)= (0.0d0, 0.0d0)
               enddo
c               pentcount = pentcount+1
            else
               do i = 1,6
                  j5wpwm(i)=j5wpwm(i)+(3*pi*(0d0,1d0)-4.0d0*pi**2/3.0d0
     1                                                 +7.0d0)*jbwpwm(i)
               enddo
c               pentcount2 = pentcount2+1
            endif

            call penlinemmm(p1(0),minus_qm(0),minus_qp(0),minus_qz(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wmp(1,1),wpp(1,1),ztaup(1,1),
     2         -1,scale,4,1,pentcontr1,j5wmwp(3),borntest,jbwmwp(3))
            call penlinemmm(p1(0),minus_qm(0),minus_qp(0),minus_qz(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wmp(1,1),wpp(1,1),ataup(1,1),
     2         -1,scale,9,-1,penttest,j5wmwp(6),borntest,jbwmwp(6))

            call penlinemmm(p1(0),minus_qm(0),minus_qz(0),minus_qp(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wmp(1,1),ztaup(1,1),wpp(1,1),
     2         -1,scale,4,1,pentcontr2,j5wmwp(2),borntest,jbwmwp(2))
            call penlinemmm(p1(0),minus_qm(0),minus_qz(0),minus_qp(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),wmp(1,1),ataup(1,1),wpp(1,1),
     2         -1,scale,9,-1,penttest,j5wmwp(5),borntest,jbwmwp(5))

            call penlinemmm(p1(0),minus_qz(0),minus_qm(0),minus_qp(0),p2(0),
     1         psi(1,-1,2),psi(1,-1,1),ztaup(1,1),wmp(1,1),wpp(1,1),
     2         -1,scale,4,1,pentcontr3,j5wmwp(1),borntest,jbwmwp(1))
            call penlinemmm(p1(0),minus_qz(0),minus_qm(0),minus_qp(0),p2(0),
     1          psi(1,-1,2),psi(1,-1,1),ataup(1,1),wmp(1,1),wpp(1,1),
     2         -1,scale,9,-1,penttest,j5wmwp(4),borntest,jbwmwp(4))

            call Wardtest(psi, p, wmp(1,1),qm(0,1),wpp(1,1),qp(0,1), 
     1         ztaup(1,1), qz(0,1), pentcontr1, mborn2, wardidtest1, als(1,1))
            call Wardtest(psi, p, wmp(1,1),qm(0,1),ztaup(1,1), qz(0,1),
     1         wpp(1,1),qp(0,1), pentcontr2, mborn2, wardidtest2, als(1,1))
            call Wardtest(psi, p, ztaup(1,1), qz(0,1), wmp(1,1),qm(0,1),
     1         wpp(1,1),qp(0,1), pentcontr3, mborn2, wardidtest3, als(1,1))

            if ( (wardidtest1.eq.1) .or. (wardidtest2.eq.1) 
     1                     .or. (wardidtest3.eq.1) ) then
               do i = 1,6
                  j5wmwp(i)= (0.0d0, 0.0d0)
               enddo
c               pentcount = pentcount+1
            else
                do i = 1,6
                   j5wmwp(i)=j5wmwp(i)+(3*pi*(0d0,1d0)
     1                               -4.0d0*pi**2/3.0d0+7.0d0)*jbwmwp(i)
                enddo
c               pentcount2 = pentcount2+1
            endif

c            do i = 10,10000000,1000
c               if (pentcount2.eq.1d2*i) print*,"ratio, pent= ",
c     1                 pentcount*1.0d0/(pentcount+pentcount2)
c            enddo

         else ! pentagon parts ~ box
            do i = 1,3
               call VVV_J_virtual_tri_box_pent45(psi,p,1,2,sign,scale,wpp(1,1),
     1                  wmp(1,1),xp,xm,i,5,j5pm(0,i,-1),j5za1(i))
               call VVV_J_virtual_tri_box_pent45(psi,p,1,2,sign,scale,wmp(1,1),
     1                  wpp(1,1),xm,xp,i,5,j5mp(0,i,-1),j5za2(i))
            enddo
         endif !NLO

c   u u -> W W Z
         if (nlo.eq.-5 .or. nlo.eq.1) then 
            mv1 = j5wpwm(1)*clr(3,2,-1) !ZWW
     1           +j5wpwm(2)*clr(4,2,-1) !WZW
     2           +j5wpwm(3)*clr(3,2,-1) !WWZ
         else
            mv1 = -dotcc(j5pm(0,1,-1),ztau(1,N))*clr(3,2,-1) !ZWW
     1            + j5za1(1)*xz*clr(3,2,-1)                  !ZWW
     2            -dotcc(j5pm(0,2,-1),ztau(1,N))*clr(4,2,-1) !WZW
     3            + j5za1(2)*xz*clr(4,2,-1)                  !WZW
     4            -dotcc(j5pm(0,3,-1),ztau(1,N))*clr(3,2,-1) !WWZ
     5            + j5za1(3)*xz*clr(3,2,-1)                  !WWZ
         endif
         matv(1,-1,1) = mv1 * clr(3,3,-1)**2

c   d d -> W W Z
         if (nlo.eq.-5 .or. nlo.eq.1) then
            mv2 = j5wmwp(1)*clr(4,2,-1) !ZWW
     1           +j5wmwp(2)*clr(3,2,-1) !WZW
     2           +j5wmwp(3)*clr(4,2,-1) !WWZ
         else
            mv2 = -dotcc(j5mp(0,1,-1),ztau(1,N))*clr(4,2,-1)
     1            + j5za2(1)*xz*clr(4,2,-1)
     2            -dotcc(j5mp(0,2,-1),ztau(1,N))*clr(3,2,-1)
     3            + j5za2(2)*xz*clr(3,2,-1)
     4            -dotcc(j5mp(0,3,-1),ztau(1,N))*clr(4,2,-1)
     5            + j5za2(3)*xz*clr(4,2,-1)
         endif
         matv(2,-1,1) = mv2 * clr(3,3,-1)**2

c qq -> WWA and qq -> A A/Z(->WW) contributions are not calculated in 
c narrow width approximation
      if (NW.eq.0) then

c   u u -> W W A
         if (nlo.eq.-5 .or. nlo.eq.1) then
            mv3 = j5wpwm(4)*clr(3,1,-1) !AWW
     1           +j5wpwm(5)*clr(4,1,-1) !WAW
     2           +j5wpwm(6)*clr(3,1,-1) !WWA
         else
            mv3 = -dotcc(j5pm(0,1,-1),atau(1,N))*clr(3,1,-1) !AWW
     1            + j5za1(1)*xa*clr(3,1,-1)                  !AWW
     2            -dotcc(j5pm(0,2,-1),atau(1,N))*clr(4,1,-1) !WAW
     3            + j5za1(2)*xa*clr(4,1,-1)                  !WAW
     4            -dotcc(j5pm(0,3,-1),atau(1,N))*clr(3,1,-1) !WWA
     5            + j5za1(3)*xa*clr(3,1,-1)                  !WWA
         endif
         matv(1,-1,2) = mv3 * clr(3,3,-1)**2

c   d d -> W W A
         if (nlo.eq.-5 .or. nlo.eq.1) then
            mv4 = j5wmwp(4)*clr(4,1,-1) !AWW
     1           +j5wmwp(5)*clr(3,1,-1) !WAW
     2           +j5wmwp(6)*clr(4,1,-1) !WWA
         else
            mv4 = -dotcc(j5mp(0,1,-1),atau(1,N))*clr(4,1,-1)
     1            + j5za2(1)*xa*clr(4,1,-1)
     2            -dotcc(j5mp(0,2,-1),atau(1,N))*clr(3,1,-1)
     3            + j5za2(2)*xa*clr(3,1,-1)
     4            -dotcc(j5mp(0,3,-1),atau(1,N))*clr(4,1,-1)
     5            + j5za2(3)*xa*clr(4,1,-1)
         endif
         matv(2,-1,2) = mv4 * clr(3,3,-1)**2

      endif !NW.eq.0

      endif !lpent
#endif

c Calculation of processes qq -> V V -> W W Z -------------
#ifdef WITH_NLO
      if (lbox) then
      if (NW.eq.0) then
c qq -> A A/Z(-> W W)
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., atau(1,N), 2, ja(0,-1,1),
     2                 jva(0,-1,1) )

c contract with atoww/ztoww tensor
         do isig = -1,1,2
            mv1 = -dotcc(jva(0,isig,1),aww(0,N))
            mv2 = -dotcc(jva(0,isig,1),zww(0,N))
            do k = 1,2
               matv(k,isig,3) = (mv1*clr(ifl(k),1,isig)
     1                     +  mv2*clr(ifl(k),2,isig))
     2                       * clr(ifl(k),1,isig)
            enddo
         enddo

c qq -> A/Z(-> W W) A
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., atau(1,N), 1, ja(0,-1,2),
     2                 jva(0,-1,2) )

c contract with atoww/ztoww tensor
         do isig = -1,1,2
            mv1 = -dotcc(jva(0,isig,2),aww(0,N))
            mv2 = -dotcc(jva(0,isig,2),zww(0,N))
            do k = 1,2
               matv(k,isig,4) = (mv1*clr(ifl(k),1,isig)
     1                     +  mv2*clr(ifl(k),2,isig))
     2                       * clr(ifl(k),1,isig)
            enddo
         enddo

      endif !NW.eq.0

c qq -> Z A/Z(-> W W)
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., ztau(1,N), 2, jz(0,-1,1),
     2                 jvz(0,-1,1) )

c contract with atoww/ztoww tensor
         do isig = -1,1,2
            mv1 = -dotcc(jvz(0,isig,1),aww(0,N))
            mv2 = -dotcc(jvz(0,isig,1),zww(0,N))
            do k = 1,2
               matv(k,isig,5) = (mv1*clr(ifl(k),1,isig)
     1                     +  mv2*clr(ifl(k),2,isig))
     2                       * clr(ifl(k),2,isig)
            enddo
         enddo

c qq -> A/Z(-> W W) Z
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., ztau(1,N), 1, jz(0,-1,2),
     2                 jvz(0,-1,2) )

c contract with atoww/ztoww tensor
         do isig = -1,1,2
            mv1 = -dotcc(jvz(0,isig,2),aww(0,N))
            mv2 = -dotcc(jvz(0,isig,2),zww(0,N))
            do k = 1,2
               matv(k,isig,6) = (mv1*clr(ifl(k),1,isig)
     1                     +  mv2*clr(ifl(k),2,isig))
     2                       * clr(ifl(k),2,isig)
            enddo
         enddo

c qq -> W+ W-(-> W Z)
         call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., wp(1,N), 2, jwp(0,-1,1),
     2                 jvwp(0,-1,1) )
         call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., wp(1,N), 1, jwp(0,-1,2),
     2                 jvwp(0,-1,2) )

c contract with wmwz tensor
         mv1 = -dotcc(jvwp(0,-1,1),wmwz(0,N))
         mv2 = -dotcc(jvwp(0,-1,2),wmwz(0,N))

c additional contributions due to q_mu q_nu/ m^2 terms in the propagator
         mv3 = zm2i(3)*dotrc(pwmz(0),wmwz(0,N))*dotrc(pwmz(0),jvwp(0,-1,1))
         mv4 = zm2i(3)*dotrc(pwmz(0),wmwz(0,N))*dotrc(pwmz(0),jvwp(0,-1,2))

         matv(1,-1,7) = mv1 * clr(3,3,-1)**2 + mv3 * clr(3,3,-1)**2
         matv(2,-1,7) = mv2 * clr(3,3,-1)**2 + mv4 * clr(3,3,-1)**2

c qq -> W- W+(-> W Z)
         call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., wm(1,N), 2, jwm(0,-1,1),
     2                 jvwm(0,-1,1) )
         call TRIboxlinec(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., wm(1,N), 1, jwm(0,-1,2),
     2                 jvwm(0,-1,2) )

c contract with wpwz tensor
         mv1 = -dotcc(jvwm(0,-1,1),wpwz(0,N))
         mv2 = -dotcc(jvwm(0,-1,2),wpwz(0,N))

c additional contributions due to q_mu q_nu/ m^2 terms in the propagator
         mv3 = zm2i(3)*dotrc(pwpz(0),wpwz(0,N))*dotrc(pwpz(0),jvwm(0,-1,1))
         mv4 = zm2i(3)*dotrc(pwpz(0),wpwz(0,N))*dotrc(pwpz(0),jvwm(0,-1,2))

         matv(1,-1,8) = mv1 * clr(3,3,-1)**2 + mv3 * clr(3,3,-1)**2
         matv(2,-1,8) = mv2 * clr(3,3,-1)**2 + mv4 * clr(3,3,-1)**2
      endif !lbox
#endif

c-----------------------------------------------------
c------------------------------------------------------
c comparison with madgraph
#ifdef WITH_MADGRAPH
      if (ldebugm) then

         do isig = -1,1,2
            do i = 1,10
               do k = 1,2

                  madgr(i,isig,k) = 0d0

               enddo
            enddo
         enddo

         do isig = -1,1,2
            do k = 1,2
               do i =1,10

                  madgr(i,isig,k) = tamp(i,isig,k)

               enddo
            enddo
         enddo

c print results from checks:

         do i=1,10
            do k = 1,2
               do isig = -1,1,2

    	    print*,"ratio for k = ",k," isig = ",isig," i = ",i,":",
     &	    	    mat(k,isig,i)/madgr(i,isig,k)
            print*,"mat(",k,",",isig,",",i,") = ",mat(k,isig,i)
            temp = dreal(mat(k,isig,i))**2+dimag(mat(k,isig,i))**2
            print*,temp
            print*,"madgr(",i,",",isig,",",k,") = ",madgr(i,isig,k)
            temp1 = dreal(madgr(i,isig,k))**2+dimag(madgr(i,isig,k))**2
            print*,temp1
            print*,"temp/temp1 = ",temp/temp1
            print*," "

               enddo
            enddo
         enddo

      endif ! end of debugging block
#endif

c------------------------------------------------------


c sum the graphs, square them and map them onto uuww and ddww

      do k = 1,2
         res(k) = 0
         resv(k) = 0
         do isig = -1,1,2
            mm(k,isig) = 0
            do i = 1,10
               mm(k,isig) = mm(k,isig) +  mat(k,isig,i)
            enddo
            res(k)= res(k) + dreal(mm(k,isig))**2
     &                     + dimag(mm(k,isig))**2

c  add Born type term and multiply by F_q = alphas*C_2/4pi
c  the additional factor for the born term is after adding the 
c  subtraction term and the counter term for the renormalization
c  of the pdfs (see also D(x) in m2s_qqWWZj.f)

            if (lnlo) then
               mv(k,isig)=matv(k,isig,1)+matv(k,isig,2)+matv(k,isig,3)
     1                   +matv(k,isig,4)+matv(k,isig,5)+matv(k,isig,6)
     2                   +matv(k,isig,7)+matv(k,isig,8)+matv(k,isig,9)
     3                   +matv(k,isig,10)       !virtual not prop. to born
               if (veratest) then
                  mv(k,isig) = 0d0
               endif
            endif
            if (nlo.gt.0) then
               mv(k,isig) = als(1,1)*c2o4pi*
     1                  (mv(k,isig) + mm(k,isig)* cvirtVV) 
            else if(nlo.lt.0) then
               mv(k,isig) = als(1,1)*c2o4pi*mv(k,isig)
            else
                mv(k,isig) = (0d0, 0d0)
            endif

           resv(k) = resv(k) + 2d0*dreal( mm(k,isig)   *
     1              conjg( mv(k,isig) ) )

         enddo

         if (veratest) then
            if (nlo.eq.0) then
               res(k) = (res(k)+resv(k))*3d0      ! 3 is the color sum factor
            else
               res(k) = resv(k)*3d0               ! 3 is the color sum factor
            endif
         else
            if (nlo.ge.0) then
               res(k) = (res(k)+resv(k))*3d0      ! 3 is the color sum factor
            else
               res(k) = resv(k)*3d0               ! 3 is the color sum factor
            endif  
         endif
      enddo

      uuwwz = res(1)
      ddwwz = res(2)

      return
      end


c----------  TRIboxlinec(isigmax, ... ) -------------------------------------

      subroutine TRIboxlinec(isigmax, psi1,psi2,k1,k2,
     1                    ld0calc, jext, ivmu, 
     2                    jborn,    jvirt)
      implicit none
      integer isigmax, ivmu
      logical ld0calc
      double precision k1(0:3),k2(0:3)
      double complex psi1(2,-1:1),psi2(2,-1:1), jext(0:5), 
     1               jborn(0:5,-1:1),jvirt(0:5,-1:1)

c  variant of the boxline routines which returns the virtual corrections 
c  without contraction of one of the external gauge bosons, i.e. in form of a
c  current. Also, results for both fermion chiralities are returned for isgmax = +1
c 
c  determine the finite virtual corrections to the fundamental box-type
c  graph, i.e. the sum of all virtual corrections to the Born diagram
c
c    psi1(k1)    ---->---------->-------------->--  psi2(k2)
c                         S              S
c                         S              S
c                         S              S
c                     q1  V           q2 V
c                         S              S
c                         S              S
c
c                        mu           jext(q2)      for ivmu = 1 
c                     jext(q1)          mu          for ivmu = 2
c
c          ====       eps1(q1)        eps2(q2)
c
c   note: k1 = k2+q1+q2, k1.k1 = 0 = k2.k2, q1.q1,q2.q2 arbitrary
c
c  INPUT:      isigmax         -1: only Lefthanded fermions, +1: return L and R
c              psi1(2,-1:1)    the incoming Weyl spinor for both chiralities
c              psi2(2,-1:1)    the outgoing Weyl spinor, psi^+(k2)
c       k1(0:3),k2(0:3)        the corresponding momenta in direction of
c                              the fermion line
c       ld0calc                call BCD_fill_c if true
c       jext(0:3)              specified polarization vector/current" 
c                              corresponding to 1 external EW boson
c       jext(4:5)              outflowing momenta of this EW bosons
c                              q1(0:3) or q2(0:3) in DHELAS notation
c       jborn(0:5,-1:1)        the current of free index at ivmu for the 
c                              corresponding Born diagram and 2 quark chiralities
c
c  OUTPUT:   jvirt(0:5,-1:1)   the amplitude for the finite part of all 
c                              virtual 1-loop QCD corrections for free index at ivmu
c
      logical ldebug
      parameter (ldebug=.false.)

      double precision pi
      parameter (pi = 3.141592653589793d0)

common block for D_ij etc. information via bcd_fill_c
      double complex D0c, Dijc(3,13), Teps1c, Teps2c, Tbornc,Tg1c,Tg2c
      double precision sc, tc, q1sqc, q2sqc
      common /bcd_qqv_c/ sc,tc,q1sqc,q2sqc, D0c,Dijc, 
     1                   Teps1c,Teps2c,Tbornc,Tg1c,Tg2c
C   local variables
      integer mu, nu, isig
      double precision s,t,mom(0:3),dotrr
      external dotrr
      double precision q1(0:4),q2(0:4), temp
      double complex eps1(0:3),eps2(0:3),mborn,mvirt
      double complex me1, me2, mq, d0t, s1c, s1r, sc3, z
      external d0t, s1c, s1r, sc3
      double complex ce1,ce2,cq,cborn
      double complex d0, d11,d12,d13, d21,d22,d23,d24,d25,d26,d27
      double complex d31,d32,d33,d34,d35,d36,d37,d38,d39,d310,
     1               d311,d312,d313
      double complex eps1k1, eps1k2, eps1q2, eps1eps2, 
     1               eps2k1, eps2q1, eps2k2, eps1q1, eps2q2,
     2               gce1, gce2, gcq


c transfer momentum information of EW bosons to q1,q2
      if (ivmu.eq.1) then   ! eps2 and q2 are specified, eps1 = mu = free index
         q2(0) = dreal(jext(4))
         q2(1) = dreal(jext(5))
         q2(2) = dimag(jext(5))
         q2(3) = dimag(jext(4))
         do mu = 0,3
            eps2(mu) = jext(mu)
            mom(mu) = k2(mu)+q2(mu)
            q1(mu) = k1(mu)-mom(mu)
         enddo
   
      elseif (ivmu.eq.2) then     ! eps1 and q1 are specified, eps2 = mu = free index
         q1(0) = dreal(jext(4))
         q1(1) = dreal(jext(5))
         q1(2) = dimag(jext(5))
         q1(3) = dimag(jext(4))
         do mu = 0,3
            eps1(mu) = jext(mu)
            mom(mu) = k1(mu)-q1(mu)
            q2(mu) = mom(mu)-k2(mu)
         enddo
      endif
      q1(4) = q1(0)**2-q1(1)**2-q1(2)**2-q1(3)**2
      q2(4) = q2(0)**2-q2(1)**2-q2(2)**2-q2(3)**2

      if (ld0calc) call BCD_fill_triboson(k1,k2,q1,q2)
      d0 = d0c

      d11 = dijc(1,1)
      d12 = dijc(1,2)
      d13 = dijc(1,3)

      d21 = dijc(2,1)
      d22 = dijc(2,2)
      d23 = dijc(2,3)
      d24 = dijc(2,4)
      d25 = dijc(2,5)
      d26 = dijc(2,6)
      d27 = dijc(2,7)

      d31 = dijc(3,1)
      d32 = dijc(3,2)
      d33 = dijc(3,3)
      d34 = dijc(3,4)
      d35 = dijc(3,5)
      d36 = dijc(3,6)
      d37 = dijc(3,7)
      d38 = dijc(3,8)
      d39 = dijc(3,9)
      d310 = dijc(3,10)
      d311 = dijc(3,11)
      d312 = dijc(3,12)
      d313 = dijc(3,13)

Check consistency of kinematics
      if (ldebug) then
         print*," "
         print*," comparison of input to BOXLINE for ivmu = ",ivmu
         print("(a,3f16.4)")," q1sq = ",q1sqc,q1(4),q1sqc/q1(4)
         print("(a,3f16.4)")," q2sq = ",q2sqc,q2(4),q2sqc/q2(4)
         s = -2*dotrr(k1,k2)
         t =  2*dotrr(k2,q2)+q2(4)
         print("(a,3f16.4)")," s = ",sc,s,sc/s
         print("(a,3f16.4)")," t = ",tc,t,tc/t
c         do mu = 0,3
c            if (abs(k1(mu)-ak1(mu,id)).gt.(1d-10*k1(0)) .or.
c     1          abs(k2(mu)-ak2(mu,id)).gt.(1d-10*k2(0)) .or.
c     1          abs(q1(mu)-aq1(mu,id)).gt.(1d-10*k2(0)) .or.
c     1          abs(q2(mu)-aq2(mu,id)).gt.(1d-10*k2(0)) ) then
c               print*," WARNING: momentum mismatch in boxline, mu =",mu
c               print*," k1: ",k1(mu)/(ak1(mu,id)+1d-80)
c               print*," k2: ",k2(mu)/(ak2(mu,id)+1d-80)
c               print*," q1: ",q1(mu)/(aq1(mu,id)+1d-80)
c               print*," q2: ",q2(mu)/(aq2(mu,id)+1d-80)
c            endif
c         enddo
         print*," input psi1 ",psi1
         print*," input psi2 ",psi2
         print*," input jext ",jext
      endif

c  get matrix elements 
c
c  me1 = psibar(2) eps1slash psi(1)
c  me2 = psibar(2) eps2slash psi(1)
c  mq  = psibar(2) (q1-q2)slash psi(1)
      do mu = 0,3
         if (ivmu.eq.1) then
            do nu = 0,3
               eps1(nu) = 0d0
            enddo
            eps1(mu) = -1d0
            if (mu.eq.0) eps1(mu) = 1d0
         elseif (ivmu.eq.2) then
            do nu = 0,3
               eps2(nu) = 0d0
            enddo
            eps2(mu) = -1d0
            if (mu.eq.0) eps2(mu) = 1d0
         endif
         do isig = -1,isigmax,2
            me1 = s1c(psi2(1,isig),eps1,.true.,isig,psi1(1,isig))
            me2 = s1c(psi2(1,isig),eps2,.true.,isig,psi1(1,isig))
            mq  = 2*s1r(psi2(1,isig),q1,.true.,isig,psi1(1,isig))
            mborn = jborn(mu,isig)
            if (ldebug) then
               print*," check amplitudes for mu =",mu," isig =",isig
               print*," check M_q: ",
     1         -mq/s1r(psi2(1,isig),q2,.true.,isig,psi1(1,isig)),
     1         " = 2?"
               t = mom(0)**2-mom(1)**2-mom(2)**2-mom(3)**2
               z = sc3(psi2(1,isig),eps2,mom,eps1,psi1(1,isig),isig)/t
               print*," M_born:    ",mborn/z," = 1? Abs(ratio) = ",
     &              dreal(mborn/z),abs(mborn/z)
            endif
c
c prepare the coefficients
            s = sc
            t = tc

      eps1k1 = eps1(0)*k1(0)-eps1(1)*k1(1)-eps1(2)*k1(2)-eps1(3)*k1(3)
      eps1k2 = eps1(0)*k2(0)-eps1(1)*k2(1)-eps1(2)*k2(2)-eps1(3)*k2(3)
      eps1q2 = eps1(0)*q2(0)-eps1(1)*q2(1)-eps1(2)*q2(2)-eps1(3)*q2(3)
      eps1eps2 = eps1(0)*eps2(0)-eps1(1)*eps2(1)-
     1           eps1(2)*eps2(2)-eps1(3)*eps2(3)
      eps2k1 = eps2(0)*k1(0)-eps2(1)*k1(1)-eps2(2)*k1(2)-eps2(3)*k1(3)
      eps2k2 = eps2(0)*k2(0)-eps2(1)*k2(1)-eps2(2)*k2(2)-eps2(3)*k2(3)
      eps2q1 = eps2(0)*q1(0)-eps2(1)*q1(1)-eps2(2)*q1(2)-eps2(3)*q1(3)

      eps1q1 = eps1(0)*q1(0)-eps1(1)*q1(1)-eps1(2)*q1(2)-eps1(3)*q1(3)
      eps2q2 = eps2(0)*q2(0)-eps2(1)*q2(1)-eps2(2)*q2(2)-eps2(3)*q2(3)

c
      ce1 = -2*(d12+d24)*eps2k2*(q1sqc+q2sqc-3*s-4*t) - 
     1      2*(d12-d24)*eps2q1*(q2sqc-t) 
     2  +4*(- d11*eps2k2*s - d12*eps2k1*t + d13*eps2k2*
     3  (q2sqc-s-t) + d13*eps2q1*q2sqc - d21*eps2k2*s 
     4 - d22*eps2k2*t - d22*eps2q1*q2sqc + d23*eps2k2*q1sqc
     5 + d25*eps2k2*(q2sqc-s-2*t) 
     6 - d26*eps2k2*(q2sqc-s-t)+d26*eps2q1*t+2*d27*eps2q1
     7 - d32*eps2k2*q2sqc - d34*eps2k2*(q2sqc-t) 
     8 + d35*eps2k2*(q2sqc-s-t)+d36*eps2k2*(2*q2sqc-t) 
     9 + d37*eps2k2*q1sqc + d38*eps2k2*(q1sqc+q2sqc-s)
     & - d39*eps2k2*q1sqc 
     1 - d310*eps2k2*(q1sqc+2*q2sqc-2*s-t)
     2 - 4*d311*eps2k2 + 6*d312*eps2k2 + 2*d313*eps2q1)

      ce2= -2*(d12+d24)*(eps1k2*(q1sqc+q2sqc-s-2*t)
     1  +eps1q2*(q2sqc-s-3*t)) + 
     2  4*(d13*eps1k2*(q1sqc-2*s-t) - 
     3  d13*eps1q2*(2*s+t) + d22*eps1k2*q2sqc
     4  + d22*eps1q2*q2sqc - d23*eps1k2*t + 
     5    d23*eps1q2*(q1sqc-t) - d24*eps1k1*q2sqc 
     6  + d25*eps1k2*q1sqc + d25*eps1k1*(q2sqc-2*s-t) 
     7  + d26*eps1k2*t - d26*eps1k1*(q1sqc-s) - 
     8  2*d27*eps1q2 + d33*eps1k2*q1sqc + d33*eps1q2*q1sqc
     9  + d37*eps1k1*(q2sqc-s-t) + d38*eps1k1*q2sqc
     &  - d39*eps1k1*(q1sqc+q2sqc-s) - d310*eps1k1*(q2sqc-t)
     1  +2*d311*eps1k2+2*d312*eps1q2-6*d313*eps1k1)

C NOTE: cq = 1/2 * cq1 in my reduce code
      cq = (d12+d24)*eps1eps2*s + 2*(4*d12*eps2k2*eps1k2 + 
     1 3*d12*eps2k2*eps1q2+d12*eps2q1*eps1k2-4*d13*eps2k2*eps1k2-
     2 2*d13*eps2k2*eps1q2-2*d13*eps2q1*eps1k2-d13*eps1eps2*s+
     3 2*d22*eps2k2*eps1q2-d22*eps1eps2*t-2*d23*eps2q1*eps1k2-
     4 2*d23*eps2q1*eps1q2-d23*eps1eps2*t+6*d24*eps2k2*eps1k2+
     5 3*d24*eps2k2*eps1q2+d24*eps2q1*eps1k2-6*d25*eps2k2*eps1k2-
     6 2*d25*eps2k2*eps1q2-2*d25*eps2q1*eps1k2-d25*eps1eps2*s-
     7 4*d26*eps2k2*eps1q2+4*d26*eps2q1*eps1k2+2*d26*eps2q1*eps1q2+
     8 d26*eps1eps2*(s+2*t)-d32*eps1eps2*q2sqc+
     9 d33*eps1eps2*q1sqc+2*d34*eps2k2*eps1k2-2*d35*eps2k2*eps1k2+
     & 2*d36*eps2k2*eps1q2+d36*eps1eps2*(q2sqc-t)-
     1 2*d37*eps2q1*eps1k2+d37*eps1eps2*(q2sqc-s-t)+
     2 2*d38*eps2q1*eps1q2+d38*eps1eps2*(q1sqc+2*q2sqc-s) -
     3 2*d39*eps2q1*eps1q2-d39*eps1eps2*(2*q1sqc+q2sqc-s) -
     4 2*d310*eps2k2*eps1q2+2*d310*eps2q1*eps1k2-d310*eps1eps2*
     5 (2*q2sqc-s-2*t)+4*d312*eps1eps2-4*d313*eps1eps2)

      cborn = -2*((d36+d37-2*d310)*(q2sqc-t) + 
     1    d38*(q1sqc+2*q2sqc) - d39*(2*q1sqc+q2sqc) )
     2    -2*(d0+d11+d12-2*d13+d24-d25+d26-d37-d38+d39+d310)*s
     3    -2*(-(d22+d23-2*d26)*t + 2*d27 - d32*q2sqc
     4    +d33*q1sqc+ 6*(d312-d313))

      gce1 = -2*eps2q2*( d12*(2*q1sqc+q2sqc-s-3*t) - 
     1       2*(d13-d22)*(q1sqc+q2sqc-s-t)
     2     - d24*(q2sqc-s-t)-2*d26*q1sqc-4*d27-4*d312)

      gce2 = 2*eps1q1*(d12*(q2sqc+t)-2*d13*(q2sqc+s) + 
     1      2*d22*q2sqc + 2*d23*(q1sqc+q2sqc-s-2*t)
     2     +d24*(q2sqc+t)+2*d25*(-q2sqc+s+t)-2*d26*(q2sqc-t)
     3     -4*d27+2*d33*q1sqc + 4*d313)

      gcq = 4*eps1q1*eps2q2*(d12-d13+d22-d23+d38-d39) +
     1        2*eps1q1*eps2k2*(d12-4*d23+d24+2*d26-2*d37+2*d310) -
     2        4*eps1q1*eps2q1*(d23-d26+d33-d39) +
     3        2*eps2q2*eps1k2*(3*d12-4*d13+4*d22+d24-2*d25-
     4                                     2*d26+2*d36-2*d310) +
     5        4*eps2q2*eps1q2*(d12-d13+2*d22-2*d26+d32-d38)
            ce1 = ce1 + gce1
            ce2 = ce2 + gce2
            cq = cq + gcq

c  now add the terms from the vertex and propagator corrections
            ce1 = ce1 + 2*eps2k2*Teps2c + 2*eps2q2*Tg2c
            ce2 = ce2 + 2*eps1k1*Teps1c - 2*eps1q1*Tg1c
            cborn = cborn*t - Tbornc

c  and the final result
            mvirt = me1*ce1 + me2*ce2 + mq*cq + mborn*cborn

            if (ldebug) then
               print*," mvirt/mborn ",mvirt/mborn
               if (abs(eps1(0)/q1(0)-1).lt.1d-10) then
                  print*," mborn type ",(mborn*cborn+ me2*ce2)/mborn
                  print*," mq type    ",(me1*ce1+ mq*cq)/mborn
                  print*," me1     ",me1*ce1/mborn
                  print*," me2     ",me2*ce2/mborn
               else
                  print*," mborn type ",(mborn*cborn+ me1*ce1)/mborn
                  print*," mq type    ",(me2*ce2+ mq*cq)/mborn
                  print*," me1     ",me1*ce1/mborn
                  print*," me2     ",me2*ce2/mborn
               endif
            endif

            jvirt(mu,isig) = mvirt
     &          + (3*pi*(0d0,1d0)-pi**2)*jborn(mu,isig)

         enddo
      enddo

      if (ivmu.eq.1) then
         cq = jvirt(0,-1)*q1(0)-jvirt(1,-1)*q1(1)-
     1        jvirt(2,-1)*q1(2)-jvirt(3,-1)*q1(3)
         t = abs(cq)/max(abs(jborn(0,-1)*q1(0)),
     1              abs(jborn(1,-1)*q1(1)),abs(jborn(2,-1)*q1(2)),
     2              abs(jborn(3,-1)*q1(3)))
         if (t.gt.1d-3) then
               do mu = 0,3
                  do isig = -1,1,2
                     jvirt(mu,isig) = (0d0,0d0)
                  enddo
               enddo
         endif
      else if (ivmu.eq.2) then
         cq = jvirt(0,-1)*q2(0)-jvirt(1,-1)*q2(1)-
     1        jvirt(2,-1)*q2(2)-jvirt(3,-1)*q2(3)
         t = abs(cq)/max(abs(jborn(0,-1)*q2(0)),
     1              abs(jborn(1,-1)*q2(1)),abs(jborn(2,-1)*q2(2)),
     2              abs(jborn(3,-1)*q2(3)))
         if (t.gt.1d-3) then
               do mu = 0,3
                  do isig = -1,1,2
                     jvirt(mu,isig) = (0d0,0d0)
                  enddo
               enddo
         endif
      endif
      return
      end

C

c---------------------------------------------------------------------

#ifdef WITH_NLO

      subroutine  J_virtual_tri_box_pent_VVV(psi,p,nq1,nq2,sign, musq,
c     input currents (any ORDERED couple among [J1,J2,J3]) in DHELAS format
c     They can be  jmu1,jmu2  or  jmu1,jmu3   or   jmu2,jmu3
     #     jal,jbe,
c     uncontracted external current. It can be 1, 2 or 3
     #     num_ret_current,
c     the corresponding Born current
c           Jborn,
c     returned current
     #     Jret)

      implicit none

#include "VBFNLO/utilities/global.inc"
      
      complex * 16 psi(2,-1:1,4)
      double precision p(0:3,8),q(0:3),l(0:3),t(0:3)
      double precision musq
      complex * 16 jal(0:5),jbe(0:5)
      integer num_ret_current, nq1,nq2,nq3,nq4, sign(4)
      complex * 16 jborn(0:3), jret(0:3)
      logical lpr
      common /e0print/lpr

      integer mu
c map input momenta onto incoming standard and make use of 
c DHELAS information in currents
      if (nq1.eq.1 .and. nq2.eq.2) then
            nq3 = 8
            nq4 = 7
      else
         print*," check nq1 etc. in J_virtual_tri_box_pent_VVV call "
         print*,nq1,nq2,nq3,nq4
         stop
      endif
         
      if (num_ret_current.eq.1) then
         do mu = 0,3
            q(mu) = p(mu,nq3) - p(mu,nq4)     != -qz(mu)
         enddo
         l(0) = -dreal(jal(4))
         l(1) = -dreal(jal(5))
         l(2) = -dimag(jal(5))
         l(3) = -dimag(jal(4))
         t(0) = -dreal(jbe(4))
         t(1) = -dreal(jbe(5))
         t(2) = -dimag(jbe(5))
         t(3) = -dimag(jbe(4))
      elseif (num_ret_current.eq.2) then
         do mu = 0,3
            l(mu) = p(mu,nq3)-p(mu,nq4)
         enddo
         q(0) = -dreal(jal(4))
         q(1) = -dreal(jal(5))
         q(2) = -dimag(jal(5))
         q(3) = -dimag(jal(4))
         t(0) = -dreal(jbe(4))
         t(1) = -dreal(jbe(5))
         t(2) = -dimag(jbe(5))
         t(3) = -dimag(jbe(4))
      elseif (num_ret_current.eq.3) then
         do mu = 0,3
            t(mu) = p(mu,nq3)-p(mu,nq4)
         enddo
         q(0) = -dreal(jal(4))
         q(1) = -dreal(jal(5))
         q(2) = -dimag(jal(5))
         q(3) = -dimag(jal(4))
         l(0) = -dreal(jbe(4))
         l(1) = -dreal(jbe(5))
         l(2) = -dimag(jbe(5))
         l(3) = -dimag(jbe(4))
      endif

      do mu = 0,3
         jret(mu) = 0
      enddo

         call  J_virtual_tri_box_pent (
c     bra(p"), ket(p) and helicity 
     #     psi(1,-1,nq2),psi(1,-1,nq1),-1,
c     momenta and squared renormalization scale
     #     p(0,nq1),q,l,t,p(0,nq2),musq,
c     input currents (any ORDERED couple among [J1,J2,J3])
c     They can be  jmu1,jmu2  or  jmu1,jmu3   or   jmu2,jmu3
     #     jal,jbe,
c     uncontracted external current. It can be 1, 2 or 3
     #     num_ret_current,
c     returned current
     #     Jret)

      lpr = .false.
      if (lpr) then
         print*," input momenta for above: p,q,l,t,pp "
         call write_mom(p(0,nq1))
         call write_mom(q)
         call write_mom(l)
         call write_mom(t)
         call write_mom(p(0,nq2))
         print*," "
      endif

c check momentum conservation and subtract Born contribution
      do mu = 0,3
         t(mu) = q(mu)+l(mu)+t(mu)+p(mu,nq1)-p(mu,nq2) 
         if (abs(t(mu)).gt.1d-8) then
            print*," no momentum conservation in j_virtual_tri_box_pent"
            print*," mom diff(",mu,") = ",t(mu)
         endif
      enddo
      return
      end

#endif
