c==================  subroutine read_anomHVV ================================
c
c       this subroutine reads in anomalous couplings for the higgs 
c       boson in different parametrisation and transformes them 
c       into the Parametrisation of Phys. Lett. B591, 297
c
c========================================================================
c
      subroutine read_anomHVV
      implicit none

      real*8 g5hvv(2,1:4,1:4), lambda5, treefac, loopfac, d, dodd, dB,
     &  dBo, dg1Z, dkgam, kgamo,
     & fWW, fWWo, fBB, fBBo, fW, fB, fBo
      common /hcoupl/ g5hvv, lambda5
      common /lhcoup/ treefac, loopfac
      integer ffac
      real*8 m2ff,mff
      logical lff, switch1, switch2, switch3
      common/ formfacmass/ m2ff,mff,ffac,lff
      integer i,j, hvv1, hvv2
      character*50 gtext

c     elektroweak parameters used here:
      double precision s2, mW, mZ, al, g
      real*8 pi
      parameter(pi =3.141592653589793d0) 
      s2 = 0.2312d0
      mW = 80.425d0
      mZ = 91.189d0
      al = 0.00729927d0
      g = Sqrt(4*pi*al)/sqrt(s2)

c     initialize couplings to zero
      do i=1,4
         do j =1,4
            g5hvv(1,i,j) = 0.0d0
            g5hvv(2,i,j) = 0.0d0
         enddo
      enddo
      d = 0.0d0
      dB = 0.0d0
      dg1Z = 0.0d0
      dkgam = 0.0d0
      dodd = 0.0d0
      dBo = 0.0d0
      kgamo = 0.0d0
      fWW = 0.0d0
      fBB = 0.0d0
      fW = 0.0d0
      fB = 0.0d0
      fWWo = 0.0d0
      fBBo = 0.0d0
      fBo = 0.0d0
      hvv1 = 0
      hvv2 = 0
      switch1 = .false.
      switch2 = .false.
      switch3 = .false.

      print *," "
      print *,"  Information on anomalous coupling parameters  "
      print *,"------------------------------------------------"
      call loadfile("anom_HVV.dat",.true.)

c     Form Factor
       call read_logical("FORMFACTOR",lff,.false.)
       if(lff) then
          call read_real("MASS_SCALE",mff,200.0d0)
          call read_int("FFAC",ffac,2)
          m2ff = mff**2
       endif

c     Parametrisation: Phys. Lett. B591, 297
       call read_logical("PARAMETR1",switch1,.false.)
       if(switch1) then
          call read_real("LAMBDA5",lambda5,480.0d0)
c     read cp even couplings
          call read_real("G5E_HWW", g5hvv(1,3,4),0.d0)
          g5hvv(1,4,3) = g5hvv(1,3,4)
          call read_real("G5E_HZZ", g5hvv(1,2,2),0.d0)
          call read_real("G5E_HGG", g5hvv(1,1,1),0.d0)
          call read_real("G5E_HGZ", g5hvv(1,1,2),0.d0)
          g5hvv(1,2,1) = g5hvv(1,1,2)
c     read cp odd couplings
          call read_real("G5O_HWW", g5hvv(2,3,4),0.d0)
          g5hvv(2,4,3) = g5hvv(2,3,4)
          call read_real("G5O_HZZ", g5hvv(2,2,2),0.d0)
          call read_real("G5O_HGG", g5hvv(2,1,1),0.d0)
          call read_real("G5O_HGZ", g5hvv(2,1,2),0.d0)
          g5hvv(2,2,1) = g5hvv(2,1,2)
       endif

c     Parametrisation of the L3-Collaboration
       call read_logical("PARAMETR2",switch2,.false.)
       if(switch2) then
c     read cp even couplings
          call read_real("D_EVEN",d,0.d0)
          call read_real("DB_EVEN",dB,0.d0)
          call read_real("DG1Z_EVEN",dg1Z,0.d0)
          call read_real("DKGAM_EVEN",dkgam,0.d0)
c     read cp odd couplings
          call read_real("D_ODD",dodd,0.d0)
          call read_real("DB_ODD",dBo,0.d0)
          call read_real("KGAM_ODD",kgamo,0.d0)
          call read_int("HVV1",hvv1,4)
       endif

c     Parametrisation: Phys. Lett. B318, 155
       call read_logical("PARAMETR3",switch3,.false.)
       if(switch3) then
c     read cp even couplings
          call read_real("FWW_EVEN",fWW,0.d0)
          call read_real("FBB_EVEN",fBB,0.d0)
          call read_real("FW_EVEN",fW,0.d0)
          call read_real("FB_EVEN",fB,0.d0)
c     read cp odd couplings
          call read_real("FWW_ODD",fWWo,0.d0)
          call read_real("FBB_ODD",fBBo,0.d0)
          call read_real("FB_ODD",fBo,0.d0)
          call read_int("HVV2",hvv2,4)
       endif

c     Loopfactor and Treefactor
      call read_real("TREEFAC",treefac,1.d0)
      if(treefac.ne.1d0) then
         write(6,*) " SM type Higgs coupling multiplied by ",treefac 
      endif
      call read_real("LOOPFAC",loopfac,1.d0)
      if(loopfac.ne.1d0) then
         write(6,*) " SM contributions to hZZ and hZgamma loop induced "
         write(6,*) " Higgs couplings multiplied by ",loopfac 
      endif
      call closefile
      print *," "

      if((.not.switch1).and.(.not.switch2).and.(.not.switch3)) then
         lambda5 = 480.0d0
      endif

 800  format(a)

      if(switch1.eqv..false.) then
         do i=1,4
            do j=1,4
               g5hvv(2,i,j) = 0.0d0
               g5hvv(1,i,j) = 0.0d0
            enddo
         enddo
      endif

c     fill g5hvv with d, dB, dg1Z, dkgam  
      if((switch1.eqv..false.).and.(switch2.eqv..true.).and.
     &   (switch3.eqv..false.)) then
c      print*,"CP-even couplings:"
c      print*,"d =",d,"  dB =",dB,"  dg1Z =",dg1Z,"  dkgam =",dkgam
c      print*,"CP-odd couplings:"
c      print*,"dodd =",dodd,"  dBo =",dBo,"  kgamo =",kgamo
c     
c        h-w-w anomalous couplings
         if((hvv1==3).or.(hvv1==4).or.(hvv1==6)) then
            lambda5 = 480.0d0
            g5hvv(1,3,4) = lambda5*g/mW*(d+(mW**2)/(mZ**2)*dg1Z)
            g5hvv(1,4,3) = g5hvv(1,3,4)
            g5hvv(2,3,4) = lambda5*g/mW*dodd
            g5hvv(2,4,3) = g5hvv(2,3,4)
c        contribution to a1
            treefac = treefac+2*(1-s2)*dg1Z
         endif
c
c        h-z-z anomalous couplings
         if((hvv1==2).or.(hvv1==4).or.(hvv1==6).or.(hvv1==7)) then
            lambda5 = 480.0d0
            g5hvv(1,2,2) = lambda5*g/mW*((1-s2)*d+s2*dB+
     &      (1-2*s2)*dg1Z+s2/(1-s2)*dkgam)
            g5hvv(2,2,2) = lambda5*g/mW*((1-s2)*
     &      dodd+s2*dBo+s2/(1-s2)*kgamo)
c        contribution to a1
            treefac = treefac+2*((1-2*s2)*dg1Z+s2/(1-s2)*dkgam)      
         endif
c
c        h-gamma-gamma
         if(hvv1==1.or.hvv1==4.or.hvv1==5) then
c           SM + CPeven
            lambda5 = 480.0d0
            g5hvv(1,1,1) = lambda5*g/mW*(s2*d+(1-s2)*dB)
c           CP odd
            g5hvv(2,1,1) = lambda5*g/mW*(s2*dodd+(1-s2)*dBo)
         endif
c
c        h-z-gamma
         if(hvv1==0.or.hvv1==4.or.hvv1==5.or.(hvv1==7)) then
c           SM + CP-even
            lambda5 = 480.0d0
            g5hvv(1,1,2) = lambda5*g/2/mW*(2*sqrt(1-s2)*sqrt(s2)*
     &      (d-dB)+2*sqrt(1-s2)*sqrt(s2)*dg1Z-sqrt(s2)/sqrt(1-s2)*dkgam)
            g5hvv(1,2,1) = g5hvv(1,1,2)
c           CP-odd
            g5hvv(2,1,2) = lambda5*g/(2*mW)*(2*sqrt(1-s2)*sqrt(s2)*
     &      (dodd-dBo)-sqrt(s2)/sqrt(1-s2)*kgamo)
            g5hvv(2,2,1) = g5hvv(2,1,2)
         endif

c     fill ahvv with fWW, fBB, fW, fB
      elseif((switch1.eqv..false.).and.(switch2.eqv..false.).and.
     &       (switch3.eqv..true.)) then
c      print*,"CP-even couplings:"
c      print*,"fWW =",fWW,"  fBB =",fBB,"  fW =",fW,"  fB =",fB
c      print*,"CP-odd couplings:"
c      print*,"fWWo =",fWWo,"  fBBo =",fBBo,"  fBo =",fBo
c
c        h-w-w anomalous couplings
         if(hvv2==3.or.hvv2==4) then
            lambda5 = 480.0d0
            g5hvv(1,3,4) = lambda5*g*mW*(-fWW+1/2*fW)
            g5hvv(1,4,3) = g5hvv(1,3,4)
            g5hvv(2,3,4) = -lambda5*g*mW*fWWo
            g5hvv(2,4,3) = g5hvv(2,3,4)
c        contribution to a1
            treefac = treefac+mW**2*fW
         endif
c
c        h-z-z anomalous couplings
         if(hvv2==2.or.hvv2==4) then
            lambda5 = 480.0d0
            g5hvv(1,2,2) = lambda5*g*mW/(1-s2)*(-s2**2*fBB-
     &      (1-s2)**2*fWW+1/2*((1-s2)*fW+s2*fB))
            g5hvv(2,2,2) = lambda5*g*mW/(1-s2)*(-s2**2*
     &      fBBo-(1-s2)**2*fWWo+s2*fBo)
c        contribution to a1
            treefac = treefac+mZ**2*((1-s2)*fW+s2*fB)
         endif
c
c        h-gamma-gamma
         if(hvv2==1.or.hvv2==4) then
            lambda5 = 480.0d0
c           SM + CPeven
            g5hvv(1,1,1) = -lambda5*g*mW*s2*(fBB+fWW)
c           CP odd
            g5hvv(2,1,1) = -lambda5*g*mW*s2*(fBBo+fWWo)
         endif
c
c        h-z-gamma
         if(hvv2==0.or.hvv2==4) then
            lambda5 = 480.0d0
c           SM + CP-even
            g5hvv(1,1,2) = lambda5*g*mW*sqrt(s2)/sqrt(1-s2)*(s2*fBB-
     &      (1-s2)*fWW+1/4*(fW-fB))
            g5hvv(1,2,1) = g5hvv(1,1,2)
c           CP-odd
            g5hvv(2,1,2) = lambda5*g*mW*sqrt(s2)/sqrt(1-s2)*(s2*fBBo-
     &      (1-s2)*fWWo-1/4*fBo)
            g5hvv(2,2,1) = g5hvv(2,1,2)
         endif
      endif

      end


c===================subroutine fillanomhcoupl============================
c
c     this subroutine fills the array ahvv with g5hvv, lambda5
c     ahvv(i,v1,v2) i = 1,2,3 are coefficients of eq 1 of 
c     Physics Letters B 591,297
c     
c     v1, v2 = 1: photon
c     v1, v2 = 2: Z0 boson
c     v1, v2 = 3,4: W boson
c
c     init = 1: only initialization to zero
c     else:     fill the array ahvv with g5hvv, lambda
c
c========================================================================
c
      subroutine fillanomhcoupl(init,dum_hzg,dum_hgg,loopfac,treefac)
      
      implicit none

#include "VBFNLO/utilities/global.inc"

      real*8  g5hvv(2,1:4,1:4), lambda5, loopfac, treefac
      double complex ahvv(3,4,4)
      integer ffac
      real*8 m2ff,mff
      logical lff
      common/ formfacmass/ m2ff,mff,ffac,lff
      integer i,j,k,init
      complex*16 dum_hzg,dum_hgg !sm 1-loop contributions 
      common /hcoupl/ g5hvv, lambda5
      common/tensorhvv/ ahvv


      if(init.eq.1) then   ! initialze ahvv only
         do i = 1,4
            do j = 1,4
               ahvv(1,i,j) = (0.0d0,0.0d0)
               ahvv(2,i,j) = (0.0d0,0.0d0)
               ahvv(3,i,j) = (0.0d0,0.0d0)
            enddo
         enddo
c         if (with_anomHiggs) then
c            write(*,*)
c            write(6,*) " Anomalous HVV couplings are being used."
c            write(*,*)
c            write(*,"(a)") " G5HVV: "
c            write(*,"(a)") " CP-EVEN COUPLINGS: "
c            write(*,"(a6,g12.4)") " HGAMGAM: ",g5hvv(1,1,1)
c            write(*,"(a6,g12.4,a2,g12.4)") " HZGAM: ",g5hvv(1,1,2),", ",
c     &                                   g5hvv(1,2,1)
c            write(*,"(a6,g12.4)") " HZZ: ",g5hvv(1,2,2)
c            write(*,"(a6,g12.4,a2,g12.4)") " HWW: ",g5hvv(1,3,4),
c     &              ", ",g5hvv(1,4,3)
c            write(*,"(a)") " CP-ODD COUPLINGS: "
c            write(*,"(a6,g12.4)") " HGAMMA: ",g5hvv(2,1,1)
c            write(*,"(a6,g12.4,a2,g12.4)") " HZGAM: ",g5hvv(2,1,2),", ",
c     &                                   g5hvv(2,2,1)
c            write(*,"(a6,g12.4)") " HZZ: ",g5hvv(2,2,2)
c            write(*,"(a6,g12.4,a2,g12.4)") " HWW: ",g5hvv(2,3,4),
c     &              ", ",g5hvv(2,4,3)
c            write(*,*)
c            write(*,"(a11,g12.4,a3)") " LAMBDA5 = ",lambda5,"GeV"
c            write(*,*)

c            write(*,"(a11,g12.4)") " TREEFAC = ",treefac
c            write(*,"(a11,g12.4)") " LOOPFAC = ",loopfac
c            write(*,*)
c            write(*,"(a7,l4)") " LFF = ",lff  
c            write(*,"(a8,g12.4)") " M2FF = ",m2ff
c            write(*,"(a8,i4)") " FFAC = ",ffac
c            write(*,*)
c         else
c            write(*,*)
c            lff = .false. 
c            mff = 1.0d0
c            lambda5 = 1.0d0
c            do i =1,4
c               do j = 1,4
c                  g5hvv(2,i,j) =0.0d0        
c                  g5hvv(1,i,j) =0.0d0
c               enddo 
c            enddo
c            treefac = 1.0d0
c            loopfac = 1.0d0
c         endif

         return
      endif
        
c     fill ahvv coefficients with lambda5, g5hvv
c        h-w-w anomalous couplings
         do i = 3,4
            do j = 3,4
               ahvv(2,i,j) = -g5hvv(1,i,j)/lambda5*(2.0d0,0.0d0)
               ahvv(3,i,j) = g5hvv(2,i,j)/lambda5*(2.0d0,0.0d0)
            enddo
         enddo
c
c        h-z-z anomalous couplings
         ahvv(2,2,2) = -2.0d0*g5hvv(1,2,2)/lambda5*(1.0d0,0.0d0)
         ahvv(3,2,2) = 2.0d0*g5hvv(2,2,2)/lambda5*(1.0d0,0.0d0)
c
c        h-gamma-gamma
c        SM + CPeven
         ahvv(2,1,1) = loopfac*dum_hgg - 2.0d0*g5hvv(1,1,1)/lambda5
     $   *(1.0d0,0.0d0) 
c        CP odd
         ahvv(3,1,1) = 2.0d0*g5hvv(2,1,1)/lambda5 *(1.0d0,0.0d0)
c
c        h-z-gamma
c        SM + CP-even 
         ahvv(2,1,2) = -(loopfac*dum_hzg + 2.0d0*g5hvv(1,1,2)/lambda5
     $   *(1.0d0,0.0d0))
         ahvv(2,2,1) = -(loopfac*dum_hzg + 2.0d0*g5hvv(1,2,1)/lambda5
     $   *(1.0d0,0.0d0))
c        CP-odd
         ahvv(3,1,2) = 2.0d0*g5hvv(2,1,2)/lambda5*(1.0d0,0.0d0)
         ahvv(3,2,1) = 2.0d0*g5hvv(2,2,1)/lambda5*(1.0d0,0.0d0)

      return
      end


c     This subroutine computes the decay width of 
c     h -> Z/gamma Z/gamma -> f1 f1bar f3 f3bar for 
c     flav1,flav3
c     Final state fermions are treated as non-identical particles.
c
      subroutine flavor_sum(width)
c     only works for h->Z*Z* decay
      implicit none
#include "VBFNLO/utilities/koppln.inc"
      real*8 width,dum
      real*8 matrix(11,11),Nc(4),mass(11),gm(4),matrix1(4,4)
      real*8 Nf(4),factor
c     nu_e,e,nu_mu,mu,nu_tau,tau,u,d,c, s, b
c     1    2   3   4    5    6   7 8 9 10 11
      real*8 flavor(11)
      integer i,j,k,l
      real*8 pi
      parameter(pi=3.141592653589793d0)
      logical lcomputeALL
      data flavor/1,2,1,2,1,2,3,4,3,4,4/
      data mass/0.0d0,5.1d-3,0.0d0,0.106d0,0.0d0,1.78d0,
     $     4.0d-4,7.0d-3,1.5d0,0.2d0,4.7d0/
      parameter(lcomputeALL=.false.)
c     color factors 
      Nc(1) = 1.d0
      Nc(2) = 1.d0
      Nc(3) = 3.d0*(1.d0 + ALFAS/PI)   
      Nc(4) = 3.d0*(1.d0 + ALFAS/PI)
c     Number of fermion in generation
      Nf(1) = 3.0d0
      Nf(2) = 3.0d0 
      Nf(3) = 2.0d0             !no top
      Nf(4) = 3.0d0
c     compute geometric means of masses 
      gm(1) = 0.0d0
      gm(2) = (mass(2)*mass(4)*mass(6))**(1.0d0/3.d0)
      gm(3) = (mass(7)*mass(9))**(1.0d0/2.d0)
      gm(4) = (mass(8)*mass(10)*mass(11))**(1.0d0/3.d0)
c      print*,"masses=",gm
c      stop
      if(lcomputeALL) then
         dum = 0.0d0
         do k=1,11
            do l = 1,11
               if(l.ne.k) factor = 2.0d0
               i = flavor(k)
               j = flavor(l)
               call hzgammawidth(mass(k),mass(l),i,j,matrix(k,l))
               dum = matrix(k,l)*Nc(i)*Nc(j)/factor + dum
            enddo
         enddo
      else
c         print*,"This the long way:",dum
         dum =0.0d0
         factor = 2.0d0
         do i=1,4
            do j = 1,4         
               call hzgammawidth(gm(i),gm(j),i,j,matrix1(i,j))
c               call hzgammawidth(0.0d0,0.0d0,i,j,matrix1(i,j))
               dum = matrix1(i,j)*Nc(i)*Nc(j)*Nf(i)*Nf(j)/factor + dum
            enddo
         enddo
      endif
      width = dum
      return
      end
c
      subroutine hzgammawidth(mf1,mf3,iflav1,iflav3,width)
      implicit none
      real*8 width,mf1,mf3,mh,qsq1,qsq2,ss1,ss2
      integer flav1,flav3
      integer iflav1,iflav3      
      common/flavors/flav1,flav3
      real*8 mu1,mu3,x1,x2
      common/masses/mu1,mu3
      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)
      integer id                !id = 3,4 W"s 2 is Z
      common /partid/ id 
         
      id = 2                    !always
      flav1 = iflav1
      flav3 = iflav3
      
      mu1 = mf1
      mu3 = mf3
      

      x1 = 0.0d0
      x2 = 1.0d0

      call quad2dim(x1,x2,ss2)
      width = ss2 
      return
      end
c    
c
      
      real*8 function dgam(xx,yy)
      implicit none
      integer flav1,flav3,i,j
      common/flavors/flav1,flav3
      real*8 qsq1,qsq2,xx,yy
      double complex F1,F2,F3,F1c,F2c,F3c
      real*8 mu1,mu3
      common/masses/mu1,mu3
      real*8 dum1
      real*8 ints(3,3),lamb1,lamb2,factor,pi
      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)
      parameter(pi=3.141592653589793d0)
      real*8 Jac2,Jac1
      real*8 lowq2(2),hiq2(2)
 

      
c     convention here is 2 is yy
c     1 is xx 
c      qsq1 = yy
c      qsq2 = xx
      lowq2(1) = 4.0d0*mu1**2
      hiq2(1) = xm2(6)
c      print*,"xm2(6)=",xm2(6)


         
      call com_Jq2(1,lowq2,hiq2,xx,qsq1,Jac1) !outer integral
      
c     compute bound on inner integration dy or 2
      lowq2(2) = 4.0d0*mu3**2
      hiq2(2) = (sqrt(xm2(6))-sqrt(qsq1))**2

      call com_Jq2(2,lowq2,hiq2,yy,qsq2,Jac2) !inner integral

c      print*,"hiq2 in unit g(x,y)=",hiq2
c      print*,"lowq2 in unit g(x,y)=",lowq2
      dum1 = 0.0d0
      call computeIs(qsq1,qsq2,ints)
      do i=-1,1,2
         do j = -1,1,2
            call computeFs(qsq1,qsq2,i,j,flav1,flav3,2,2,
     $           F1,F2,F3)  
c            print*,"flav11,flav3=",flav1,flav3

            F1c = conjg(F1)
            F2c = conjg(F2)
            F3c = conjg(F3)

            dum1 = (dble(f1)**2+dimag(f1)**2)*ints(1,1)+
     $           (dble(f2)**2+dimag(f2)**2)*ints(2,2)+
     $           (dble(f3)**2+dimag(f3)**2)*ints(3,3)+
     $           2.0d0*dble(f1*f2c)*ints(2,1) + dum1
         enddo
      enddo

      lamb1 = qsq1/xm2(6)
      lamb2 = qsq2/xm2(6)
      call lambda1(1.0d0,lamb1,lamb2,factor)
c      print*,"dum=",dum
      dgam = dum1*factor*(1.0d0/(8.0d0*pi))*
     $     (1.0d0/(32.d0*pi**2))**2 
     $     *(1.d0/(2.0d0*pi))**2 * 1.d0/(2.0d0*sqrt(xm2(6)))

      dgam = dgam*Jac1*Jac2     !mult by jacobians

      return
      end
c      
c
      real*8 function ddwidth(yy)
      implicit none
      real*8 yy,x,y,dgam
      common/xy/ x,y
      external dgam
      y = yy
      ddwidth = dgam(x,y)
      return
      end

      real*8 function dwidth(xx)
      implicit none
      real*8 eps
      parameter(eps = 1.0d-5)
      real*8 xx,y1,y2,x,y
      common /xy/ x,y
      real*8 ss,gaus,ddwidth      
      
      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)
     
      integer flav1,flav3      
      common/flavors/flav1,flav3
      
     
      external ddwidth
      external gaus     
c
      x = xx
      
      y1 = 0.0d0
      y2 = 1.0d0
      ss = gaus(ddwidth,y1,y2,eps)
        
      dwidth = ss
      return
      end

      subroutine quad2dim(x1,x2,ss)
      implicit none
      integer i
      real*8 eps
      parameter(eps = -1.0d-2)
      real*8 ss,x1,x2,dwidth,gaus2
      external dwidth
      external gaus2
     
c
      ss = gaus2(dwidth,x1,x2,eps)  

      return
      end

c     F1,F2,F3
      subroutine computeFs(qsq1,qsq2,tau1,tau3,flav1,flav3,
     $     bos1,bos2,F1,F2,F3)
      implicit none
      real*8 qsq1,qsq2          !q1^2 and q2^2
      integer tau1,tau3         !helicities of f1 and f3
      integer flav1,flav3       !flavor of fermions
      integer bos1,bos2         !boson type : W+ 3
c                                             W- 4
c                                             Z  2 
c                                         photon 1
c
      double complex prop1(4),prop2(4) !1 = photon propagator
c     2 = Z propagator
      double complex F1,F2,F3
c     koppln commom blocks
      double complex ahvv(3,4,4)
      common/tensorhvv/ ahvv
      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)


      prop1(1) = 1.d0/dcmplx(qsq1,0.0d0)
      prop2(1) = 1.d0/dcmplx(qsq2,0.0d0)
      prop1(2) = 1.d0/dcmplx(qsq1-xm2(2),xmg(2))
      prop2(2) = 1.d0/dcmplx(qsq2-xm2(2),xmg(2))
      prop1(3) = 1.d0/dcmplx(qsq1-xm2(3),xmg(3))
      prop2(3) = 1.d0/dcmplx(qsq2-xm2(3),xmg(3))
      prop1(4) = prop1(3)
      prop2(4) = prop2(3)
      if(bos1.eq.2) then
         F1 = CLR(flav1,2,tau1)*CLR(flav3,2,tau3)*ahvv(1,2,2)*
     1                              prop1(2)*prop2(2)
         
         F2 = CLR(flav1,2,tau1)*CLR(flav3,2,tau3)*ahvv(2,2,2)*
     1                              prop1(2)*prop2(2)+
     $       V(flav1,1)*CLR(flav3,2,tau3)*ahvv(2,2,1)*prop1(1)*prop2(2)+
     $       V(flav3,1)*CLR(flav1,2,tau1)*ahvv(2,2,1)*prop1(2)*prop2(1)+
     $       V(flav1,1)*V(flav3,1)*ahvv(2,1,1)*prop1(1)*prop2(1)
c
         F3 = CLR(flav1,2,tau1)*CLR(flav3,2,tau3)*ahvv(3,2,2)*
     1                              prop1(2)*prop2(2)+
     $       V(flav1,1)*CLR(flav3,2,tau3)*ahvv(3,2,1)*prop1(1)*prop2(2)+
     $       V(flav3,1)*CLR(flav1,2,tau1)*ahvv(3,2,1)*prop1(2)*prop2(1)+
     $       V(flav1,1)*V(flav3,1)*ahvv(3,1,1)*prop1(1)*prop2(1)
      else
         F1 = CLR(flav1,3,tau1)*CLR(flav3,4,tau3)*ahvv(1,3,4)*
     1                              prop1(3)*prop2(4)
c         
         F2 = CLR(flav1,3,tau1)*CLR(flav3,4,tau3)*ahvv(2,3,4)*
     1                              prop1(3)*prop2(4)
c     
         F3 = CLR(flav1,3,tau1)*CLR(flav3,4,tau3)*ahvv(3,3,4)*
     1                              prop1(3)*prop2(4)
 
      endif
      return
      end
c
      subroutine computeIs(qsq1,qsq2,arrayOFints)
      implicit none
      integer i,j
      real*8 qsq1,qsq2
      real*8 arrayOFints(3,3)
      real*8 q1q2,pi               !q1.q2
      parameter(pi =3.141592653589793d0)
      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)
c     initialize
      do i=1,3
         do j=1,3
            arrayOFints(i,j) = 0.0d0
         enddo
      enddo
c     compute q1.q2
      q1q2 = (xm2(6) - qsq1 - qsq2)/2.d0
c
c      Fill arrayOFints
      arrayOFints(1,1) = (64.d0/9.d0)*pi**2 *
     1     (q1q2**2 + 2.0d0*qsq1*qsq2)
      arrayOFints(1,2) = (64.d0/3.d0)*pi**2* qsq1*qsq2* q1q2
      arrayOFints(3,3) = (128.d0/9.d0)*pi**2* 
     $     qsq1*qsq2*(q1q2**2 - qsq1*qsq2)
      arrayOFints(2,1) = arrayOFints(1,2)
      arrayOFints(2,2) = (64.d0/9.d0)*pi**2 *qsq1*qsq2 *
     1     (2.0d0*q1q2**2 + qsq1*qsq2)
      return
      end
c
c     compute jacobian and q2
      subroutine com_Jq2(ich,lowq2,hiq2,r,qsq,J)
      implicit none
      integer id                !id = 3,4 W"s 2 is Z
      common /partid/ id
      real*8 XM2s(6),XMGs(6)
      COMMON /BKOPOUshort/ XM2s,XMGs
      real*8 r,qsq,J             ! J is the jacobian and r is the variable being integrated over
      real*8 rmid,rmidl,a,c,b         !c is the midpoint on [a,b]
      real*8 x,xmin,q2,q2cut,xmax
      integer i,ich
 

      real*8 lowq2(2),hiq2(2)
     
      logical lmap,logmapon
      parameter(logmapon=.true.)
c
      q2cut = 60.d0**2
      rmid = 0.2d0
      
      i = ich                   !1 or 2 = inner and outer bounds
c     if hiq2(i) is less than q2cut then let q2cut = hiq2(i)
      if(hiq2(i).lt.q2cut) then ! whole is integrated using ln map
         a = lowq2(i)
         b = hiq2(i)
         q2cut = hiq2(i)
         c = q2cut
         rmidl = 1.0d0
      else 
         a = lowq2(i)
         b = hiq2(i)
         c = q2cut
         rmidl = rmid
      endif
         
c      print*,"rmid=",rmid
c      print*,"imap=",imap
      lmap = (r.lt.rmidl).and.(a.gt.0.0d0).and.logmapon
      if(lmap) then        !ln map

         qsq = a * exp(R*(dlog(c/a))/rmidl)
         J = q2 *(dlog(c/a))/rmidl
c         print*,J,q2,a
c         print*,"imap=",imap
c         print*,"masses=",mu1,mu3
      else
         if(a.eq.0.0d0) then ! for a =0 always use bw map
            c = a
            rmidl =0.0d0
         else 
            c= q2cut
            rmidl = rmid
         endif
c         print*,"rmid=",rmid
         call calZ(c,xmin)
         call calZ(b,xmax)
         x = xmin +(r - rmidl)/(1.0d0 - rmidl)*(xmax - xmin)
         call calQ2(x,qsq)
         
         J = (xmax - xmin)/(1.0d0 - rmidl) 
     $        * xmgs(id)*(dtan(x)**2 + 1.0d0)
      endif
c      print*,J,q2
      return
      end
c


      subroutine widthhgz(idf,mf,q2,width)
      implicit none
      integer idf,i,j               !1 for neutrino,2 for electron,
      real*8 cc(3),width(4),ratio(3) !1 is gg* 2 is gz* and 3 is the interfence term
      common/constants/cc
      real*8 q2,FF2(3),psfactor ! q squared of decaying boson
      real*8 delta,mg,mz,mh,mf,Pi,Lamb1,c1,c2,c3
      parameter(Pi= 3.141592653589793d0)
     
      double complex ahvv(3,4,4)
      common/tensorhvv/ ahvv

      DOUBLE PRECISION CLR,XM2,XMG,B,V,A
      COMMON /BKOPOU/ CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),V(4,5),A(4,5)
      real*8 mf1
      parameter(mf1=0.0d0)

      mh = Sqrt(xm2(6))
c      write(6,*) mh
      mz = Sqrt(xm2(2))
      mg = xmg(2)
c      mg =0.001d0
c     Put coupling factors here:
      do i =1,3
         FF2(i) = 0.0d0
         cc(i) = 0.0d0
      enddo

c      print*,"lambda =" ,lambda5e,lambda5o
      cc(1) = 2.0d0*v(idf,2)**2 * 
     1     ( dble(ahvv(2,1,1))**2+dimag(ahvv(2,1,1))**2 +
     2       dble(ahvv(3,1,1))**2+dimag(ahvv(3,1,1))**2 ) 
      cc(2) = (CLR(idf,2,-1)**2 + CLR(idf,2,+1)**2) * 
     1     ( dble(ahvv(2,2,1))**2+dimag(ahvv(2,2,1))**2 +
     2       dble(ahvv(3,2,1))**2 + dimag(ahvv(3,2,1))**2 ) 
      cc(3) = 2.d0*(CLR(idf,2,-1) + CLR(idf,2,+1))*v(idf,1) * 
     1     ( dble(ahvv(2,1,1)*conjg(ahvv(2,1,2)))+
     2       dble(ahvv(3,1,1)*conjg(ahvv(3,1,2))) )
c
c      write(6,*) cc
c      print*,"mz",mz
c      print*,"mg",mg
c
      FF2(1) = cc(1)/q2**2 
      FF2(2) = cc(2)/((q2 - mz**2)**2 + mg**2)
      FF2(3) = cc(3)*(q2-mz**2)/((q2 - mz**2)**2 + mg**2)/q2
c
    
c      write(6,*) "ff2=",ff2
       psfactor =(1.0d0/(mh**3))*(1.0d0/(32.0d0*Pi**2))**2 * 
     1           (mh**2 - q2)*sqrt((q2 - 4.0d0*mf1**2)/q2) 
c     We use a fermion mass to regulate photon pole.
      do i = 1,3

         width(i) = FF2(i) * psfactor * (4.0d0/3.0d0)* Pi *
     1             (q2-mf1**2)*(mh**2 - q2)**2 ! m2s is computed with massive fermions

c         write(6,*)"q2 =",q2
c         write(6,*)"dw/dq2 =",width(i),"for",i
      enddo
      
      width(4) = width(1)+width(2)+width(3)

      end

      function dw(xx)  ! interface with subroutine dwidth
      implicit none
      integer idf,iflav
      common/flav/ idf
      integer id                !id = 3,4 W"s 2 is Z
      common /partid/ id
      real*8 dw,xx,q2
      integer jj
      common/choice/jj
      real*8 mass(2),mf
      common/mass1/mass
      real*8 ss,gaus,dpw(4)
      real*8 jac,lowq2(2),hiq2(2)
      external gaus
      
      id = 2
c     
      lowq2(1) = 4.0d0*mass(1)**2
      hiq2(1) = mass(2)**2
      call com_Jq2(1,lowq2,hiq2,xx,q2,Jac)
      iflav = idf
      mf = mass(1)
      call widthhgz(iflav,mf,q2,dpw)
     
c      dw  = dpw(2)+dpw(3)+dpw(1)
      dw = dpw(jj)*jac          !mult by jacobian
      return
      end
c
      subroutine quad1d(ii,iflav,mf,hmass,ss)
      implicit none
      integer idf,iflav
      common/flav/ idf
      integer ii,jj
      common/choice/jj
      real*8 mass(2),mf,hmass
      common/mass1/mass
      real*8 eps
      parameter(eps = 1.0d-7)
      real*8 ss,x1,x2,dw,gaus
      external dw
      external gaus
      jj = ii
      idf = iflav
      mass(1) = mf
      mass(2) = hmass
      x1 = 0.0d0
      x2 = 1.0d0
      ss = gaus(dw,x1,x2,eps)  
c      write(6,*) ss
      return
      end
ccccccccccccccc h-> g/Z* g -> f fbar g
      subroutine width_hgamff(width)
      implicit none
#include "VBFNLO/utilities/koppln.inc"
      real*8 width,dum
      real*8 Nc(4),mass(11),gm(4),matrix1(4),matrix(11)
      real*8 Nf(4)
c     nu_e,e,nu_mu,mu,nu_tau,tau,u,d,c, s, b
c     1    2   3   4    5    6   7 8 9 10 11
      integer i,j,k,l,flavor(11)
       real*8 pi,q2min,q2max
      parameter(pi=3.141592653589793d0)
      logical ldoALL
      data mass/0.0d0,0.511d-3,0.0d0,0.106d0,0.0d0,1.78d0,
     $     0.14d0,0.14d0,1.5d0,0.5d0,4.7d0/
      data flavor/1,2,1,2,1,2,3,4,3,4,4/
      data ldoALL/.false./
c
c     color factors
      Nc(1) = 1.d0
      Nc(2) = 1.d0
      Nc(3) = 3.d0*(1.d0 + ALFAS/PI)
      Nc(4) = 3.d0*(1.d0 + ALFAS/PI)
c     Number of fermion in generation
      Nf(1) = 3.0d0
      Nf(2) = 3.0d0
      Nf(3) = 2.0d0             !no top
      Nf(4) = 3.0d0
c     compute geometric means of masses
      gm(1) = 0.0d0
      gm(2) = (mass(2)*mass(4)*mass(6))**(1.0d0/3.d0)
      gm(3) = (mass(7)*mass(9))**(1.0d0/2.d0)
      gm(4) = (mass(8)*mass(10)*mass(11))**(1.0d0/3.d0)
c      do i = 2,4
c         gm(i) = 10d0
c      enddo
      dum =0.0d0
      if(ldoALL) then
          do k=1,11
             j = flavor(k)
             
             call quad1d(4,j,gm(j),xmh,matrix(k))
             dum = matrix(k)*Nc(j) + dum
          enddo
       else
          do j = 1,4
             call quad1d(4,j,gm(j),xmh,matrix1(j))
c             call quad1d(4,j,0.0d0,xmh,matrix1(j))
             dum = matrix1(j)*Nc(j)*Nf(j) + dum
          enddo
       endif
       
      width = dum
      return
      end
