c
c	Dieter Zeppenfeld, <dieter@pheno.physics.wisc.edu>
c	Initial version:  2003 February 12
c	Last modified by Barbara Jager:   2006 July 11
c
c  this routine computes |ME|^2 for qq->qq e+ve mu-vm~g ("WW")
c	or qq->qq e+e-mu+mu-g or qq->qq e+e- vm vm~ g ("ZZ")
c	(real emission corr. to qq->qqVV, VV->leptons) 
c	 via call of "qqwwqqj.f" or "qqzzqqj.f"
c
c	flags can be set such that collin./soft subtraction is tested
c
c	for qq->qqZZg random helicity summation is employed for decay leptons
c
c ------------------------------------------------------------------------
c
      subroutine m2s_wbfzhg(
     &                   bos,   !in:  Boson identifier,
c     				!     2=Z,6=H,34/43=WW,22=ZZ->4l,21=ZZ->2l 2v 
     &                   bosdec,!in:  identifier for Boson decay
     &                   nlo,   !in:  NLO=1: create subtraction term; LO = 0
     &                   lok,   !in:  need to calculate m2s(1:3)? T or F 
     &                   xi,	!in:  Feynman x parameters of incoming fermions
     &                   p,	!in:  momenta associated with external fermions
     &                   v,	!in:  momenta associated with WW decay fermions
     &                   rn,    !in:  random number array
     &                   xuz,   !in:  x, u, z for subtraction terms
     &                   m2s    !out: |M|^2*pdf1*pdf2 
     &                        )
      implicit none
c
c declare input/output variables
c
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/coupl.inc"
#include "tensor.inc"
#include "tensorz.inc"
#include "VBFNLO/utilities/lha.inc"
      integer bos, bosdec
      real*8 xi(nx), p(0:3,max_p,max_kin), v(0:3,max_v), 
     1       rn(1), xuz(2,2:3), m2s(0:3)
      logical lok(3)

c  helicity selection
      integer h,hh
      integer jsig, jsig1, jsig3, jsig5
      common /chelsum/ jsig,jsig1,jsig3,jsig5
c
      complex*16 zero
      parameter (zero=(0d0,0d0))
c
c declare external functions
c
      integer FL_ZHg
      external FL_ZHg
c
c alfas, scales etc
#include "VBFNLO/utilities/scales.inc"
      real*8 x1,x2
      real*8 q12(0:4,3), q34(0:4,3), qww(0:3), qvv(0:3), 
     1 	     lnQomu(2:3), omxi(2:3), 
     1       Ax(2:3), Bx(2:3), Cx(2:3), Dxi(2:3), tgs2oqsq(2:3), 
     2       ln1mxi, lnrat, z, lnz
c
c declare local variables
c
c
      real*8 q_sf, wtot(0:3)
      real*8 qa3,qa5,q35, dotrr
      external dotrr

      integer init/0/, I, J, mu
      save init
c
c declare variables for summation process
c
      INTEGER LFLAVR(5:6), ires, nmaxold, FSIGN(4+max_v), gsign
      save LFLAVR, nmaxold

      integer physToDiag(5), nlo, nmin, nmax, nproc(8)
      save nmin, nmax, nproc
c
c store contributions from subprocess ip in res(ip,ID) where
c  ID = 1  : the real emission |M|^2 * pdf
c     = 2,3: sutraction terms for emision off 12 or 34 line
c     = 0  : subtracted result which drives integration, i.e
c res(*,0) = res(*,1)-res(*,2)-res(*,3)
      real*8 res(maxnumsubproc,0:3)
c
c declare parton distribution variables
c
      real*8 pdf(-6:6,2,3)
c
c define program switches
c
      logical ldebug
c      common/ dbug / ldebug	!store "debugging value"
      parameter (ldebug=.false.)
c      data ldebug /.true./		!output debug information
      logical lcoll,lsoft	! parameters for subtraction tests
      parameter (lcoll=.false.,lsoft=.false.)
c      
      common /hcount / h
      real*8 weight,rnumb,RandomNumber
c
c --------------------------------------------------------------
c
c if first iteration, output acceptance cuts and fix constant input 
c parameters for call of qqZWqqji.f
c
      if ( init .ne. 0) then
         do i = 1,nmax
            do j=0,3
               res(i,j) = 0
            enddo
         enddo
      else
         if (bos.eq.2) then
            write(6,*) " "
            write(6,*) "Zjjj amplitude square information:"
            write(6,*) "-----------------------------------------------"
            write(6,*) " "
         elseif (bos.eq.6) then
            write(6,*) " "
            write(6,*) "Hjjj amplitude square information:"
            write(6,*) "-----------------------------------------------"
            write(6,*) " "
         elseif (bos.eq.34 .or. bos.eq.43) then
            write(6,*) " "
            write(6,*) "W+W- jjj amplitude square information:"
            write(6,*) "-----------------------------------------------"
            write(6,*) " "
            call vtoww_reset
         elseif (bos.eq.22) then
            write(6,*) " "
            write(6,*) "ZZ jjj -> 4l amplitude square information:"
            write(6,*) "-----------------------------------------------"
            write(6,*) " "
            call vtozz_reset
         elseif (bos.eq.21) then
            write(6,*) " "
            write(6,*) "ZZ jjj -> 2l 2v amplitude square information:"
            write(6,*) "-----------------------------------------------"
            write(6,*) " "
            call vtozz_reset
         endif
         if (nfl.eq.4) then
            print*," For NC processes external b-quark contributions",
     1             " are excluded"
         elseif (nfl.eq.5) then
            print*," For NC processes external b-quark contributions",
     1             " are included"
         else
            print*," Check setting of nfl in global.inc "
         endif
         print*," "
         print*," creal = ",creal," cvirtual = ",cvirt
         print*," virt factor for alphas = 0.12 is ",
     1            1+0.12/pi*4./3.*cvirt
         h = 1
         init = 1
         do i = 1,maxnumsubproc
            do j = 0,3
               res(i,j) = 0
            enddo
         enddo
      endif
      
c
c -------------------------------------------------------------------
      call Calc_Momentum_Transfer(p, v, q12,q34,3)

c
c for WW->4l precalculate A->WW,Z->WW,AZ->WW etc leptonic tensors
c
      if (bos.eq.34 .or. bos.eq.43) then
c reset lfs to .true. to force recalculation of virtual contributions in qqwwqq
         do i = 1,4
            lfs(i) = .true.
         enddo
	 

c...Les Houches interface         
       if (lha.and..not.doNLO) then
          helicity(1)=-1
          helicity(2)= 1
          helicity(3)= 1
          helicity(4)=-1
       endif


c lepton spinors and W+- polarization vectors
         CALL IXXXXX(v(0,2),ZERO ,1,-1,wep) !W(1,5))          !e+       
         CALL OXXXXX(v(0,1),ZERO ,-1,1,wve) !W(1,6))          !ve 
         CALL OXXXXX(v(0,4),ZERO ,-1,1,wmu) !W(1,7))          !mu-      
         CALL IXXXXX(v(0,3),ZERO ,1,-1,wvm) !W(1,8))          !vm~
         CALL JIOXXX(wep,wve,GWF ,WMASS,WWIDTH,wp) !W(1,10))    !W+
         CALL JIOXXX(wvm,wmu,GWF ,WMASS,WWIDTH,wm) !W(1,11))    !W-


         do mu = 0,3
            qp(mu) = v(mu,1)+v(mu,2)
            qm(mu) = v(mu,3)+v(mu,4)
            qww(mu) = qp(mu) + qm(mu)
         enddo
         qp(4) = qp(0)**2-qp(1)**2-qp(2)**2-qp(3)**2
         qm(4) = qm(0)**2-qm(1)**2-qm(2)**2-qm(3)**2
c leptonic tensors
C for W+W-
         
         if (with_anom) then ! anomalous gauge boson couplings
            call atoww_anomal(v,aww)
            call ztoww_anomal(v,zww)
            do j = 2,3
               call aatoww_anomal(q12(0,j),q34(0,j),v,aaww(0,0,j))
               call aztoww_anomal(q12(0,j),q34(0,j),v,azww(0,0,j))
               call aztoww_anomal(q34(0,j),q12(0,j),v,zaww(0,0,j))
               call zztoww_anomal(q12(0,j),q34(0,j),v,zzww(0,0,j))
               call wwtoww_anomal(q12(0,j),q34(0,j),v,wwww6(0,0,j)) ! q12 = W-
               call wwtoww_anomal(q34(0,j),q12(0,j),v,wwww5(0,0,j)) ! q12 = W+
	    	    
C for WV --> e+ nu_e
c NCw tensors for NC process (k=1...4), CCw for CC (k=5,6)
               call WVtoWP_anomal(2,q34(0,j),v,NCwpa(0,0,1,j),NCwpz(0,0,1,j)) !emit W- on upper
               call WVtoWP_anomal(2,q12(0,j),v,NCwpa(0,0,2,j),NCwpz(0,0,2,j)) !emit W- on lower
               call WVtoWP_anomal(1,q34(0,j),v,CCwpa(0,0,1,j),CCwpz(0,0,1,j)) !emit W- on upper
               call WVtoWP_anomal(1,q12(0,j),v,CCwpa(0,0,2,j),CCwpz(0,0,2,j)) !emit W- on lower
C for WV --> mu- nu_mu
               call WVtoWM_anomal(2,q34(0,j),v,NCwma(0,0,1,j),NCwmz(0,0,1,j)) !emit W+ on upper
               call WVtoWM_anomal(2,q12(0,j),v,NCwma(0,0,2,j),NCwmz(0,0,2,j)) !emit W+ on lower
               call WVtoWM_anomal(1,q34(0,j),v,CCwma(0,0,1,j),CCwmz(0,0,1,j)) !emit W+ on upper
               call WVtoWM_anomal(1,q12(0,j),v,CCwma(0,0,2,j),CCwmz(0,0,2,j)) !emit W+ on lower
            enddo

         elseif (with_kk) then ! kaluza-klein scattering
#ifdef WITH_KK
            call atoww_kk(v,aww)
            call ztoww_kk(v,zww)
            do j = 2,3
               call aatoww_kk(q12(0,j),q34(0,j),v,aaww(0,0,j))
               call aztoww_kk(q12(0,j),q34(0,j),v,azww(0,0,j))
               call aztoww_kk(q34(0,j),q12(0,j),v,zaww(0,0,j))
               call zztoww_kk(q12(0,j),q34(0,j),v,zzww(0,0,j))
               call wwtoww_kk(q12(0,j),q34(0,j),v,wwww6(0,0,j)) ! q12 = W-
               call wwtoww_kk(q34(0,j),q12(0,j),v,wwww5(0,0,j)) ! q12 = W+
	    	    
C for WV --> e+ nu_e
c NCw tensors for NC process (k=1...4), CCw for CC (k=5,6)
               call WVtoWP_kk(2,q34(0,j),v,NCwpa(0,0,1,j),NCwpz(0,0,1,j)) !emit W- on upper
               call WVtoWP_kk(2,q12(0,j),v,NCwpa(0,0,2,j),NCwpz(0,0,2,j)) !emit W- on lower
               call WVtoWP_kk(1,q34(0,j),v,CCwpa(0,0,1,j),CCwpz(0,0,1,j)) !emit W- on upper
               call WVtoWP_kk(1,q12(0,j),v,CCwpa(0,0,2,j),CCwpz(0,0,2,j)) !emit W- on lower
C for WV --> mu- nu_mu
               call WVtoWM_kk(2,q34(0,j),v,NCwma(0,0,1,j),NCwmz(0,0,1,j)) !emit W+ on upper
               call WVtoWM_kk(2,q12(0,j),v,NCwma(0,0,2,j),NCwmz(0,0,2,j)) !emit W+ on lower
               call WVtoWM_kk(1,q34(0,j),v,CCwma(0,0,1,j),CCwmz(0,0,1,j)) !emit W+ on upper
               call WVtoWM_kk(1,q12(0,j),v,CCwma(0,0,2,j),CCwmz(0,0,2,j)) !emit W+ on lower
            enddo
#endif
         else ! SM
            call atoww(v,aww)
            call ztoww(v,zww)
            do j = 2,3
               call aatoww(q12(0,j),q34(0,j),v,aaww(0,0,j))
               call aztoww(q12(0,j),q34(0,j),v,azww(0,0,j))
               call aztoww(q34(0,j),q12(0,j),v,zaww(0,0,j))
               call zztoww(q12(0,j),q34(0,j),v,zzww(0,0,j))
               call wwtoww(q12(0,j),q34(0,j),v,wwww6(0,0,j)) ! q12 = W-
               call wwtoww(q34(0,j),q12(0,j),v,wwww5(0,0,j)) ! q12 = W+
	    	    
C for WV --> e+ nu_e
c NCw tensors for NC process (k=1...4), CCw for CC (k=5,6)
               call WVtoWP(2,q34(0,j),v,NCwpa(0,0,1,j),NCwpz(0,0,1,j)) !emit W- on upper
               call WVtoWP(2,q12(0,j),v,NCwpa(0,0,2,j),NCwpz(0,0,2,j)) !emit W- on lower
               call WVtoWP(1,q34(0,j),v,CCwpa(0,0,1,j),CCwpz(0,0,1,j)) !emit W- on upper
               call WVtoWP(1,q12(0,j),v,CCwpa(0,0,2,j),CCwpz(0,0,2,j)) !emit W- on lower
C for WV --> mu- nu_mu
               call WVtoWM(2,q34(0,j),v,NCwma(0,0,1,j),NCwmz(0,0,1,j)) !emit W+ on upper
               call WVtoWM(2,q12(0,j),v,NCwma(0,0,2,j),NCwmz(0,0,2,j)) !emit W+ on lower
               call WVtoWM(1,q34(0,j),v,CCwma(0,0,1,j),CCwmz(0,0,1,j)) !emit W+ on upper
               call WVtoWM(1,q12(0,j),v,CCwma(0,0,2,j),CCwmz(0,0,2,j)) !emit W+ on lower
            enddo
         endif
	 
c
c-----------------------------
c
c
c for ZZ->4l precalculate  leptonic tensors:

      elseif (bos.eq.22) then

c reset lzs to .true. to force recalculation of virtual contributions in qqzzqq
         do i = 1,4
            lzs(i) = .true.
         enddo
c lepton spinors and Z polarization vectors ! 
c	lepton helicities not fixed 
c 		-> sum over all possible helicites in |M|**2
c
c select helicity: h ... random number for lepton helicity
c			 combination (h=1:4) .. needed only for ZZ

         h = mod(h,4) + 1
	
	ie = sign(1,2-h)
	iu = (-1)**(h+1)


c...Les Houches interface
        if (lha.and..not.doNLO) then
           helicity(1)= ie
           helicity(2)=-ie
           helicity(3)= iu
           helicity(4)=-iu
        endif
		
		
         CALL IXXXXX(v(0,1),ZERO ,+ie,-1,lep)	      !e+	
         CALL OXXXXX(v(0,2),ZERO ,-ie, 1,lem)	      !e- 
         CALL IXXXXX(v(0,3),ZERO ,+iu,-1,lup)	      !mu+	
         CALL OXXXXX(v(0,4),ZERO ,-iu, 1,lum)	      !mu- 
	
	 CALL JIOXXX(lep,lem,GZL ,ZMASS,ZWIDTH,ze)     !Zl
	 CALL JIOXXX(lep,lem,GAL ,ZERO ,ZERO  ,ae)      !Al
         CALL JIOXXX(lup,lum,GZL ,ZMASS,ZWIDTH,zu)     !Zl
	 CALL JIOXXX(lup,lum,GAL ,ZERO ,ZERO  ,au)      !Al

      if (with_kk) then  ! Kaluza-Klein scenario
#ifdef WITH_KK
c for V-> llll:	
         call vto4l_KK(v,h,azz,zzz)	
         

         do mu = 0,3
            qe(mu) = v(mu,1)+v(mu,2)
            qu(mu) = v(mu,3)+v(mu,4)
            qvv(mu) = qe(mu) + qu(mu)
         enddo
	 	 
         qe(4) = qe(0)**2-qe(1)**2-qe(2)**2-qe(3)**2
         qu(4) = qu(0)**2-qu(1)**2-qu(2)**2-qu(3)**2

c leptonic tensors:

         do j = 2,3
C for AA,AZ,ZA,ZZ -> e+ e- mu+ mu-:
         call vvtozz_KK(q12(0,j),q34(0,j),v,h,aazz(0,0,j),azzz(0,0,j),
     # 					   zazz(0,0,j),zzzz(0,0,j))
c
C for W+W-
         call wwtozz_KK(q12(0,j),q34(0,j),v,h,wwzz5(0,0,j))  ! q12 = W+
         call wwtozz_KK(q34(0,j),q12(0,j),v,h,wwzz6(0,0,j))  ! q12 = W-

C for V1V2 --> e+ e- or  V1V2 -->  mu+ mu- (Vi=A,Z)
c NCl tensors for NC process (k=1...4)

         call VVtoll_KK(2,1,h,q34(0,j),v,aaee(0,0,1,j),azee(0,0,1,j),
     #	 	 zaee(0,0,1,j),zzee(0,0,1,j))    	 !emit V on upper        
	 call VVtoll_KK(1,1,h,q12(0,j),v,aaee(0,0,2,j),azee(0,0,2,j),
     #		 zaee(0,0,2,j),zzee(0,0,2,j))    	 !emit V on lower
         call VVtoll_KK(2,2,h,q34(0,j),v,aauu(0,0,1,j),azuu(0,0,1,j),
     #		 zauu(0,0,1,j),zzuu(0,0,1,j))    	 !emit V on upper        
	 call VVtoll_KK(1,2,h,q12(0,j),v,aauu(0,0,2,j),azuu(0,0,2,j),
     #		zauu(0,0,2,j),zzuu(0,0,2,j))     	!emit V on lower

C for W+W- --> e+ e- or  W+W- -->  mu+ mu-
c  CCl tensors for CC (k=5)

         call WWtoll_KK(2,1,h,q34(0,j),v,CCee(0,0,1,j))     !emit V on upper
         call WWtoll_KK(1,1,h,q12(0,j),v,CCee(0,0,2,j))     !emit V on lower
         call WWtoll_KK(2,2,h,q34(0,j),v,CCuu(0,0,1,j))     !emit V on upper
         call WWtoll_KK(1,2,h,q12(0,j),v,CCuu(0,0,2,j))     !emit V on lower
	 
C for W-W+ --> e+ e- or mu+ mu-
c CCl for CC (k=6)
	
         call WWtoll_KK(1,1,h,q34(0,j),v,CCee6(0,0,1,j))     !emit V on upper
         call WWtoll_KK(2,1,h,q12(0,j),v,CCee6(0,0,2,j))     !emit V on lower
         call WWtoll_KK(1,2,h,q34(0,j),v,CCuu6(0,0,1,j))     !emit V on upper
         call WWtoll_KK(2,2,h,q12(0,j),v,CCuu6(0,0,2,j))     !emit V on lower
	 
	 
	 enddo !j=2,3
#endif
      else !SM
c for V-> llll:	
         call vto4l(v,h,azz,zzz)	
         

         do mu = 0,3
            qe(mu) = v(mu,1)+v(mu,2)
            qu(mu) = v(mu,3)+v(mu,4)
            qvv(mu) = qe(mu) + qu(mu)
         enddo
	 	 
         qe(4) = qe(0)**2-qe(1)**2-qe(2)**2-qe(3)**2
         qu(4) = qu(0)**2-qu(1)**2-qu(2)**2-qu(3)**2

c leptonic tensors:

         do j = 2,3
C for AA,AZ,ZA,ZZ -> e+ e- mu+ mu-:
         call vvtozz(q12(0,j),q34(0,j),v,h,aazz(0,0,j),azzz(0,0,j),
     # 					   zazz(0,0,j),zzzz(0,0,j))
c
C for W+W-
         call wwtozz(q12(0,j),q34(0,j),v,h,wwzz5(0,0,j))  ! q12 = W+
         call wwtozz(q34(0,j),q12(0,j),v,h,wwzz6(0,0,j))  ! q12 = W-

C for V1V2 --> e+ e- or  V1V2 -->  mu+ mu- (Vi=A,Z)
c NCl tensors for NC process (k=1...4)

         call VVtoll(2,1,h,q34(0,j),v,aaee(0,0,1,j),azee(0,0,1,j),
     #	 	 zaee(0,0,1,j),zzee(0,0,1,j))    	 !emit V on upper        
	 call VVtoll(1,1,h,q12(0,j),v,aaee(0,0,2,j),azee(0,0,2,j),
     #		 zaee(0,0,2,j),zzee(0,0,2,j))    	 !emit V on lower
         call VVtoll(2,2,h,q34(0,j),v,aauu(0,0,1,j),azuu(0,0,1,j),
     #		 zauu(0,0,1,j),zzuu(0,0,1,j))    	 !emit V on upper        
	 call VVtoll(1,2,h,q12(0,j),v,aauu(0,0,2,j),azuu(0,0,2,j),
     #		zauu(0,0,2,j),zzuu(0,0,2,j))     	!emit V on lower

C for W+W- --> e+ e- or  W+W- -->  mu+ mu-
c  CCl tensors for CC (k=5)

         call WWtoll(2,1,h,q34(0,j),v,CCee(0,0,1,j))     !emit V on upper
         call WWtoll(1,1,h,q12(0,j),v,CCee(0,0,2,j))     !emit V on lower
         call WWtoll(2,2,h,q34(0,j),v,CCuu(0,0,1,j))     !emit V on upper
         call WWtoll(1,2,h,q12(0,j),v,CCuu(0,0,2,j))     !emit V on lower
	 
C for W-W+ --> e+ e- or mu+ mu-
c CCl for CC (k=6)
	
         call WWtoll(1,1,h,q34(0,j),v,CCee6(0,0,1,j))     !emit V on upper
         call WWtoll(2,1,h,q12(0,j),v,CCee6(0,0,2,j))     !emit V on lower
         call WWtoll(1,2,h,q34(0,j),v,CCuu6(0,0,1,j))     !emit V on upper
         call WWtoll(2,2,h,q12(0,j),v,CCuu6(0,0,2,j))     !emit V on lower
	 
	 
	 enddo !j=2,3
      endif
c-----------------------------
c

c for ZZ->2l 2v precalculate  leptonic tensors
      elseif (bos.eq.21) then
c reset lzs to .true. to force recalculation of virtual contributions in qqzzqq
         do i = 1,4
            lzs(i) = .true.
         enddo
c lepton spinors and Z polarization vectors ! 
c	lepton helicities not fixed 
c 		-> sum over all possible helicites in |M|**2
c
c select helicity: h ... random number for lepton helicity
c			 combination (h=1:4), but only h=1 or 3 
c	   	         contribute for ZZ -> 2l 2v (no righthanded neutrinos)
c
        h = mod(h+2,4)
	
	ie = sign(1,2-h)


c...Les Houches interface
        if (lha.and..not.doNLO) then
           helicity(1)= ie
           helicity(2)=-ie
           helicity(3)=-1
           helicity(4)= 1
        endif

		
        CALL IXXXXX(v(0,1),ZERO ,+ie,-1,lep)	 !e+   
        CALL OXXXXX(v(0,2),ZERO ,-ie, 1,lem)	 !e- 
        CALL OXXXXX(v(0,3),ZERO ,-1,+1,lum)	!vm	  
        CALL IXXXXX(v(0,4),ZERO ,+1,-1,lup)	!vm~	
	
	CALL JIOXXX(lep,lem,GZL ,ZMASS,ZWIDTH,ze)      !Ze
	CALL JIOXXX(lep,lem,GAL ,ZERO ,ZERO  ,ae)      !Ae
        CALL JIOXXX(lup,lum,GZN ,ZMASS,ZWIDTH,zu)      !Zv
	
	do i = 1,4
	   au(i) = 0d0
	   zu(i) = -zu(i)
	enddo   			
	do i = 5,6 !add momentum info to A->vv current (au vanishes anyway) 
	   au(i) = zu(i)
	enddo    	
	
c for V-> llvv
	if (with_kk) then ! Kaluza-Klein Scenario 
#ifdef WITH_KK
         call vto4ln_KK(v,h,azz,zzz)	
        
         do mu = 0,3
            qe(mu) = v(mu,1)+v(mu,2)
            qu(mu) = v(mu,3)+v(mu,4)
            qvv(mu) = qe(mu) + qu(mu)
         enddo
	 	 
         qe(4) = qe(0)**2-qe(1)**2-qe(2)**2-qe(3)**2
         qu(4) = qu(0)**2-qu(1)**2-qu(2)**2-qu(3)**2

c leptonic tensors
         do j = 2,3
C for AA,AZ,ZA,ZZ -> e+ e- vm vm~:
         call vvtozzn_KK(q12(0,j),q34(0,j),v,h,aazz(0,0,j),azzz(0,0,j),
     # 					    zazz(0,0,j),zzzz(0,0,j))
c
C for W+W-
         call wwtozzn_KK(q12(0,j),q34(0,j),v,h,wwzz5(0,0,j))  ! q12 = W+
         call wwtozzn_KK(q34(0,j),q12(0,j),v,h,wwzz6(0,0,j))  ! q12 = W-
	 
C for V1V2 --> e+ e- or  V1V2 -->  vm vm~ (Vi=A,Z)
c NCl tensors for NC process (k=1...4)

         call VVtoll_KK(2,1,h,q34(0,j),v,aaee(0,0,1,j),azee(0,0,1,j),
     #	 	 zaee(0,0,1,j),zzee(0,0,1,j))    	 !emit V on upper        
	 call VVtoll_KK(1,1,h,q12(0,j),v,aaee(0,0,2,j),azee(0,0,2,j),
     #		 zaee(0,0,2,j),zzee(0,0,2,j))    	 !emit V on lower
         call VVtolln_KK(2,q34(0,j),v,aauu(0,0,1,j),azuu(0,0,1,j),
     #		 zauu(0,0,1,j),zzuu(0,0,1,j))    	 !emit V on upper        
	 call VVtolln_KK(1,q12(0,j),v,aauu(0,0,2,j),azuu(0,0,2,j),
     #		zauu(0,0,2,j),zzuu(0,0,2,j))     	!emit V on lower


C for W+W- --> e+ e- or  W+W- -->  vm vm~
c  CCl tensors for CC (k=5)

         call WWtoll_KK(2,1,h,q34(0,j),v,CCee(0,0,1,j))     !emit V on upper
         call WWtoll_KK(1,1,h,q12(0,j),v,CCee(0,0,2,j))     !emit V on lower
         call WWtolln_KK(2,q34(0,j),v,CCuu(0,0,1,j))     !emit V on upper
         call WWtolln_KK(1,q12(0,j),v,CCuu(0,0,2,j))     !emit V on lower
	 
C for W-W+ --> e+ e- or vm vm~
c CCl for CC (k=6)
	
         call WWtoll_KK(1,1,h,q34(0,j),v,CCee6(0,0,1,j))     !emit V on upper
         call WWtoll_KK(2,1,h,q12(0,j),v,CCee6(0,0,2,j))     !emit V on lower
         call WWtolln_KK(1,q34(0,j),v,CCuu6(0,0,1,j))     !emit V on upper
         call WWtolln_KK(2,q12(0,j),v,CCuu6(0,0,2,j))     !emit V on lower
	 
	enddo !j	
#endif
      else !SM
         call vto4ln(v,h,azz,zzz)	
        
         do mu = 0,3
            qe(mu) = v(mu,1)+v(mu,2)
            qu(mu) = v(mu,3)+v(mu,4)
            qvv(mu) = qe(mu) + qu(mu)
         enddo
	 	 
         qe(4) = qe(0)**2-qe(1)**2-qe(2)**2-qe(3)**2
         qu(4) = qu(0)**2-qu(1)**2-qu(2)**2-qu(3)**2

c leptonic tensors
         do j = 2,3
C for AA,AZ,ZA,ZZ -> e+ e- vm vm~:
         call vvtozzn(q12(0,j),q34(0,j),v,h,aazz(0,0,j),azzz(0,0,j),
     # 					    zazz(0,0,j),zzzz(0,0,j))
c
C for W+W-
         call wwtozzn(q12(0,j),q34(0,j),v,h,wwzz5(0,0,j))  ! q12 = W+
         call wwtozzn(q34(0,j),q12(0,j),v,h,wwzz6(0,0,j))  ! q12 = W-
	 
C for V1V2 --> e+ e- or  V1V2 -->  vm vm~ (Vi=A,Z)
c NCl tensors for NC process (k=1...4)

         call VVtoll(2,1,h,q34(0,j),v,aaee(0,0,1,j),azee(0,0,1,j),
     #	 	 zaee(0,0,1,j),zzee(0,0,1,j))    	 !emit V on upper        
	 call VVtoll(1,1,h,q12(0,j),v,aaee(0,0,2,j),azee(0,0,2,j),
     #		 zaee(0,0,2,j),zzee(0,0,2,j))    	 !emit V on lower
         call VVtolln(2,q34(0,j),v,aauu(0,0,1,j),azuu(0,0,1,j),
     #		 zauu(0,0,1,j),zzuu(0,0,1,j))    	 !emit V on upper        
	 call VVtolln(1,q12(0,j),v,aauu(0,0,2,j),azuu(0,0,2,j),
     #		zauu(0,0,2,j),zzuu(0,0,2,j))     	!emit V on lower


C for W+W- --> e+ e- or  W+W- -->  vm vm~
c  CCl tensors for CC (k=5)

         call WWtoll(2,1,h,q34(0,j),v,CCee(0,0,1,j))     !emit V on upper
         call WWtoll(1,1,h,q12(0,j),v,CCee(0,0,2,j))     !emit V on lower
         call WWtolln(2,q34(0,j),v,CCuu(0,0,1,j))     !emit V on upper
         call WWtolln(1,q12(0,j),v,CCuu(0,0,2,j))     !emit V on lower
	 
C for W-W+ --> e+ e- or vm vm~
c CCl for CC (k=6)
	
         call WWtoll(1,1,h,q34(0,j),v,CCee6(0,0,1,j))     !emit V on upper
         call WWtoll(2,1,h,q12(0,j),v,CCee6(0,0,2,j))     !emit V on lower
         call WWtolln(1,q34(0,j),v,CCuu6(0,0,1,j))     !emit V on upper
         call WWtolln(2,q12(0,j),v,CCuu6(0,0,2,j))     !emit V on lower
	 
	enddo !j	
      endif

c --------------------------------------------
	 
      endif 

c
c  ---------------------------------------------------------------------
c
cc  debugging:
cc
cc collinear subtraction:
c	if (lcoll) then 
c      	    ldebug = lok(1).and.lok(2).and.lok(3).and.
c     #      	    dotrr(p(0,5,1),p(0,1,1)).lt.1
c	    if (ldebug) print*,"coll.dot.:",lok(1),lok(2),lok(3),
c     #      		       dotrr(p(0,5,1),p(0,1,1))
cc      
cc soft subtraction:	
c       elseif (lsoft) then		       
c	   ldebug = lok(1).and.lok(2).and.lok(3).and.p(0,5,1).lt.1	
c	   if (ldebug) print*,"soft.mom.:",lok(1),lok(2),lok(3),p(0,5,1)   
c       endif
c
c  ---------------------------------------------------------------------
c
c scales and als:

	if (.false.) then
         write(6,*) " "
         write(6,*) "xi(1), xi(2) =", xi
         write(6,*) "mu_f L=1:",sqrt(mufsq(1,1)),sqrt(mufsq(2,1))
         write(6,*) "mu_f L=2:",sqrt(mufsq(1,2)),sqrt(mufsq(2,2))
         write(6,*) "mu_f L=3:",sqrt(mufsq(1,3)),sqrt(mufsq(2,3))
         write(6,*) " "
         write(6,*) "xi(1), xi(2) =", xi
         write(6,*) "alphas L=1:",als(1,1), als(2,1)
         write(6,*) "alphas L=2:",als(1,2), als(2,2)
         write(6,*) "alphas L=3:",als(1,3), als(2,3)
        end if
      if (lwarn) then    ! check connection between scales
         if(abs(mufsq(1,2)/mufsq(1,1)-1).gt.1d-8) then
            print*," check muf 1,2 vs 1,1 ",mufsq(1,2),mufsq(1,1)
         endif
         if(abs(mufsq(2,3)/mufsq(2,1)-1).gt.1d-8) then
            print*," check muf 2,3 vs 2,1 ",mufsq(2,3),mufsq(2,1)
         endif
      endif

       if ( ldebug ) then

 10      format( " p(", i1, ") = ", 4(f10.3, 2x) )
 20      format( " v(", i1, ") = ", 4(f10.3, 2x) )
c
         write(6,*) " "
         print*," vector boson decay products momenta "
         do i = 1, n_v
            write(6,20) i, v(0,i), v(1,i), v(2,i), v(3,i)
         end do
         print*," parton momenta "
         do i = 1, n_p
            write(6,10) i, p(0,i,1), p(1,i,1), p(2,i,1), p(3,i,1)
         end do
c
         write(6,*) " "
         write(6,*) "xi(1), xi(2) =", xi
         write(6,*) "mu_f =",sqrt(mufsq(1,1)),sqrt(mufsq(2,1))
         read(*,*)

       end if


c
c  end debugging
c
c  ---------------------------------------------------------------------
c
c call PDF subroutine in order to determine parton
c distributions in the incoming (anti)protons.
c for the NLO contributions x1 = x*y in my notes with x=z=xuz(1,j), y=xi(j-1)
c
      x1 = xi(1)*xuz(1,2)
      q_sf = sqrt(mufsq(1,1))
      call pdfproton( xi(1), q_sf, pdf(-6,1,1) )      ! f_a(y)=f_a(x1/z)

      q_sf = sqrt(mufsq(1,2))                         ! f_a(x1) for upper line 
      call pdfproton( x1, q_sf, pdf(-6,1,2) )         !   NLO correction

      if (mufsq(1,3).ne.mufsq(1,1)) then              ! f_a(x1) for lower line
         q_sf = sqrt(mufsq(1,3))                      !   NLO correction
         call pdfproton( xi(1), q_sf, pdf(-6,1,3) )
      else
         do i = -6,6
            pdf(i,1,3) = pdf(i,1,1)
         enddo
      endif

      x2 = xi(2)*xuz(1,3)
      q_sf = sqrt(mufsq(2,1))
      call pdfproton( xi(2), q_sf, pdf(-6,2,1) )      ! f_b(y)=f_a(x2/z)

      q_sf = sqrt(mufsq(2,3))                         ! f_b(x2) for lower line 
      call pdfproton( x2, q_sf, pdf(-6,2,3) )         !   NLO correction

      if (mufsq(2,2).ne.mufsq(2,1)) then              ! f_b(x2) for upper line
         q_sf = sqrt(mufsq(2,2))                      !   NLO correction
         call pdfproton( xi(2), q_sf, pdf(-6,2,2) )
      else
         do i = -6,6
            pdf(i,2,2) = pdf(i,2,1)
         enddo
      endif

c
c and fill the coefficient functions for the finite subtraction terms
C Note that the color factors TR and C2 are NOT!!! included here
      lnQomu(2) = log(q12(4,2)/mufsq(1,2))
      lnQomu(3) = log(q34(4,3)/mufsq(2,3))
      omxi(2) = 1d0-x1
      omxi(3) = 1d0-x2
      tgs2oqsq(2) = 8d0*pi*als(1,2)/q12(4,2)         !2g_s^2/Q^2
      tgs2oqsq(3) = 8d0*pi*als(2,3)/q34(4,3)         !2g_s^2/Q^2

      do j = 2,3
         z = xuz(1,j)
         ln1mxi = log(omxi(j))
c         tgs2oqsq(j) = tgs2oqsq(j) * 6*xuz(2,j)*(1d0-xuz(2,j))
C cut off Int_x1^1 dz log(1-z) at z<1-5E-6. This assures that the relative
C error of Int_x1^1 dz log(1-z) is less than 1E-4/(1-x1). see notes p 28.4
         if (z.lt.0.999995) then
            lnrat = lnQomu(j) + log((1d0-z)/z)
            lnz = log(z)
            Ax(j) = (z**2+(1-z)**2)*lnrat + 2*z*(1-z)
            Bx(j) = ( 2d0*(lnrat+lnz)-1.5d0 )/(1d0-z)
            Cx(j) = 1-z - 2d0*lnz/(1d0-z) - (1+z)*lnrat
         else
            Ax(j) = 0d0
            Bx(j) = 0d0
            Cx(j) = 0d0
         endif
         Dxi(j) = 1.5d0*lnQomu(j) + 2d0*ln1mxi*lnQomu(j)
     1            + ln1mxi**2 - 1.5d0*ln1mxi 
     2            + creal
         do i = 1,5
            pdf(-i,j-1,j)=(pdf(-i,j-1,j)*(Dxi(j)/omxi(j)-Bx(j)) +
     1                     pdf(-i,j-1,1)*(Bx(j)+Cx(j)))*tgs2oqsq(j)
            pdf(i,j-1,j)= (pdf(i,j-1,j)*(Dxi(j)/omxi(j)-Bx(j)) +
     1                     pdf(i,j-1,1)*(Bx(j)+Cx(j)))*tgs2oqsq(j)
         enddo
         pdf(0,j-1,j)=pdf(0,j-1,1)*Ax(j)*tgs2oqsq(j)
      enddo
c select helicity
      if (bos.eq.2) then

         jsig = min(8*rn(1)+1d0,8.01d0)
c         jsig = 0
      else
         jsig = 0
      endif

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     Now sum over the subprocesses contributing to H production

      nmax = FL_ZHg(fsign,-1)            !reset counter for subprocesses to 0

C*******************  q1 q3 ---> q2 q4 g V V   **********************
         
c   physToDiag(ext.momentum label) = Feynman diagram label

      physToDiag(1)=1    !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=3
      physToDiag(3)=2    !physToDiag(3/4) are labels of outgoing quarks.
      physToDiag(4)=4
      physToDiag(5)=5   ! gluon
C NOTE: for call of wbf_zh3j it is important that p(*,1,*) and p(*,3,*)
c correspond to 1-2 fermion line ALWAYS, i.e physToDiag(1/2)={1,3} and 
c similarly physToDiag(3/4)={2,4} for the 3-4 fermion line
      fsign(1) = 1
      fsign(2) = 1
      fsign(3) = 1
      fsign(4) = 1
      gsign    = 1

      call wbf_zh3j(xi,p,v,physToDiag,fsign,gsign,bos, bosdec,  nlo,lok,xuz,
     1  	    pdf,res,nmin,nmax)

c      if (ldebug) call resprint(nmin,nmax,res)
      if (init.eq.1) nproc(1) = nmax

C*******************  q1 qb4 ---> q2 qb3 g V V   **********************

      physToDiag(1)=1    !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=4
      physToDiag(3)=2    !physToDiag(3/4) are labels of outgoing quarks.
      physToDiag(4)=3
      physToDiag(5)=5   ! gluon

      fsign(3) = -1
      fsign(4) = -1

      call wbf_zh3j(xi,p,v,physToDiag,fsign,gsign,bos, bosdec, nlo,lok,xuz,
     1  	   pdf,res,nmin,nmax)

c      if (ldebug) call resprint(nmin,nmax,res)
      if (init.eq.1) nproc(2) = nmax

C*******************  qbar2 q3 ---> qbar1 q4 g V V   **********************

      physToDiag(1)=2    !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=3
      physToDiag(3)=1    !physToDiag(3/4) are labels of outgoing quarks.
      physToDiag(4)=4
      physToDiag(5)=5   ! gluon

      fsign(1) = -1
      fsign(2) = -1
      fsign(3) = 1
      fsign(4) = 1
      
      call wbf_zh3j(xi,p,v,physToDiag,fsign,gsign,bos, bosdec, nlo,lok,xuz,
     1  	   pdf,res,nmin,nmax)

c      if (ldebug) call resprint(nmin,nmax,res)
      if (init.eq.1) nproc(3) = nmax

C*******************  qbar2 qb4 ---> qbar1 qb3 g V V  ******************

      physToDiag(1)=2    !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=4
      physToDiag(3)=1    !physToDiag(3/4) are labels of outgoing quarks.
      physToDiag(4)=3
      physToDiag(5)=5   ! gluon

      fsign(1) = -1
      fsign(2) = -1
      fsign(3) = -1 
      fsign(4) = -1
         
      call wbf_zh3j(xi,p,v,physToDiag,fsign,gsign,bos, bosdec, nlo,lok,xuz,
     1  	   pdf,res,nmin,nmax)

c      if (ldebug) call resprint(nmin,nmax,res)
      if (init.eq.1) nproc(4) = nmax

C*******************  g q3 ---> qb1 q2 q4 V V  **********************

      physToDiag(1)=5
      physToDiag(2)=3
      physToDiag(3)=1
      physToDiag(4)=4
      physToDiag(5)=2

      fsign(1) = -1
      fsign(2) =  1
      fsign(3) =  1
      fsign(4) =  1
      gsign    = -1

      call wbf_zh3j(xi,p,v,physToDiag,fsign,gsign,bos, bosdec, nlo,lok,xuz,
     1  	   pdf,res,nmin,nmax)

c      if (ldebug) call resprint(nmin,nmax,res)
      if (init.eq.1) nproc(5) = nmax

C*******************  q1 g ---> q2 qb3 q4 V V   **********************

      physToDiag(1)=1
      physToDiag(2)=5
      physToDiag(3)=2
      physToDiag(4)=3
      physToDiag(5)=4

      fsign(1) =  1
      fsign(2) =  1
      fsign(3) = -1
      fsign(4) =  1
      gsign    = -1

      call wbf_zh3j(xi,p,v,physToDiag,fsign,gsign,bos, bosdec, nlo,lok,xuz,
     1  	   pdf,res,nmin,nmax)

c      if (ldebug) call resprint(nmin,nmax,res)
      if (init.eq.1) nproc(6) = nmax

C*******************  g qbar4 ---> qbar3 qb1 q2 V V   **********************

      physToDiag(1)=5
      physToDiag(2)=4
      physToDiag(3)=1
      physToDiag(4)=3
      physToDiag(5)=2

      fsign(1) = -1
      fsign(2) =  1
      fsign(3) = -1
      fsign(4) = -1

      call wbf_zh3j(xi,p,v,physToDiag,fsign,gsign,bos, bosdec, nlo,lok,xuz,
     1  	   pdf,res,nmin,nmax)

c      if (ldebug) call resprint(nmin,nmax,res)
      if (init.eq.1) nproc(7) = nmax

C*******************  qbar2 g ---> qbar1 qb3 q4 V V  **********************

      physToDiag(1)=2
      physToDiag(2)=5
      physToDiag(3)=1
      physToDiag(4)=3
      physToDiag(5)=4
      
      fsign(1) = -1
      fsign(2) = -1
      fsign(3) = -1
      fsign(4) =  1

      call wbf_zh3j(xi,p,v,physToDiag,fsign,gsign,bos, bosdec,nlo,lok,xuz,
     1  	   pdf,res,nmin,nmax)

c      if (ldebug) call resprint(nmin,nmax,res)
      if (init.eq.1) nproc(8) = nmax

c*****************  end of process evaluation  *********************

      if (init.eq.1) then
         init = init+1
         if (lwarn) print 199," proc #s for Z/Hjjj are ",nproc
 199     format(a,8i5)
      endif
      do j=0,3
         m2s(j) = 0      
         DO IRES = 1,NMAX
            m2s(j) = m2s(j) + RES(IRES,j)
         ENDDO

         if(j.eq.0)then
c...Les Houches interface - the most propable subprocess 3jets at LO  
            if (lha.and..not.doNLO) then
               i=0
               weight=0.d0
               rnumb=RandomNumber()
               do while((i.le.nmax).and.(weight.le.rnumb*m2s(0)))
                  i=i+1
                  weight=weight+res(i,0)
                  iprocess=i
               enddo
            endif
         endif

c         DO IRES = nproc(4)+1,nproc(8)
c            m2s(j) = m2s(j) + RES(IRES,j)
c         ENDDO


         if (bos.eq.2) then
            if (jsig.eq.0) then
               m2s(j) = m2s(j)  !*2     ! factor 2 for electrons and muons
            else
               m2s(j) = m2s(j)*8d0   ! factor 8 for random helicity summation
            endif
         elseif (bos.eq.22) then ! qq->ZZqq (both Z->ll~)
               m2s(j) = m2s(j)*4d0   ! factor 4 for random helicity summation
         elseif (bos.eq.21) then ! qq->ZZqq (one Z->ll~)
               m2s(j) = m2s(j)*2d0   ! factor 4 for random helicity summation
         endif
      enddo

      if (lha.and..not.doNLO) then
         if (jsig.ne.0) then
            if (bos.eq.2) then
               helicity(2)= jsig5 !particle is at the 2nd place
               helicity(1)=-jsig5               
            endif
         endif
      endif

c
c ---------------------------------------------------------------
c
c debugging:
c
      if (ldebug) then
         do j=1,3
            if (lok(j) .and. m2s(j).eq.0 ) then
               print*,j," lok(j) = ",lok(j)," m2s = ",m2s(j)
            endif
         enddo
         if (m2s(0).ne.0) then
c            if (abs((m2s(1)+m2s(2)+m2s(3))/m2s(0)-1).gt.1d-5) then
               print*," m2s(0) = ",m2s(0),(m2s(1)+m2s(2)+m2s(3))/m2s(0)
               print*," ratio subtr/real = ",(m2s(2)+m2s(3))/m2s(1)
               print*," m2s(0,1,2,3) = ",m2s
	       print*
c            endif
         else
            print*," m2s = ",m2s
         endif
         if (nmax.ne.nmaxold) print*," nmax: ",nmaxold,nmax
         nmaxold = nmax
      endif
c      if (ldebug) print*," lokt = ",lok
c      if (ldebug) read(*,113) text
c 113  format(a)

      RETURN
      END


c******************************************************************
c
c   begin subroutine wbf_zh3j
c
c*****************************************************************
      subroutine wbf_zh3j(xi,p,v,physToDiag,fsign,gsign,bos, bosdec,
     1                    nlo,lok,xuz,
     1                    pdf,res,nmin,nmax)
      implicit none
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"

      real*8 p(0:3,max_p,max_kin), v(0:3,max_v), xi(nx), xuz(2,2:3)
      real*8 pdf(-6:6,2,3), res(maxnumsubproc,0:3)
      integer physToDiag(5), fsign(4+max_v), 
     1        gsign, h, bos, nlo, nmin, nmax
      logical lok(3)
c      
      common /hcount / h
c
c wbf_zh3j calls the amplitude square routines 
c             qqzqqj    for qq-->qqZ g   for bos=2      and
c             qqhqq     for qq-->qqH     for bos=6
c             qqwwqq     for qq-->qqWW     for bos=34.or.43
c             qqzzqq     for qq-->qqZZ     for bos=22.or.21
c for the subtraction terms for the NLO cross section calculation
c
c  INPUT:  p(0:3,5,3)      external physical parton momenta
c          v(0:3,nv)       Z decay momenta, for Higgs production only
c                          the sum q(mu) = v(mu,1)+...+v(mu,nv) is needed
c          physToDiag(5)   physToDiag(ext.mom. label) = Feynman diagram label
c          fsign,gsign     sign factors for amplitude calls; see qqZqq(j)
c	   h		   specify lepton helicity combination
c          nlo             nlo = 0: calculate LO only (i.e. no subtraction)
c                          nlo = 1: calculate full NLO subtraction
c          lok(3)          lok(ID)=T means momenta set ID passes acceptance
c                          cuts, i.e res(k,ID) needs to be calculated
c  OUTPUT:
c          uucc(ID)   output in format as in qqZqq(j), but corresponding
c          etc.            to res(*,ID) in m2s_qqZqq
c                          ID = 1  : the real emission |M|^2 * pdf
c                             = 2,3: sutraction terms for emision off 12 or 
c     or   res(k,ID)                 34 line
c                             = 0  : subtracted result which drives 
c                                    integration, i.e 
c                                    res(*,0) = res(*,1)+res(*,2)+res(*,3)
c
c  In and Output
c    nmin, nmax            range of process numbers for this call
c
      real*8 C2, TR, N                              ! color factors
      parameter(N=3d0,TR=0.5d0,C2=TR*(N**2-1d0)/N)

c Note: Factor 9 from color sum included in qqbqq. I am using color summed
c amplitudes here, while CS use color averages. Thus I later divide by
c 8*3 for an initial qg state and by 3*3 for an initial qq state. So
c
c  2*gs2*C2*9/(8*3) = 2 gs^2 * TR *8/3*9/(8*3) = 2 gs^2 * TR
c
c consistent with Catani 
c
c alfas, scales etc
#include "VBFNLO/utilities/scales.inc"
c
c  helicity selection
      integer jsig, jsig1, jsig3, jsig5
      common /chelsum/ jsig,jsig1,jsig3,jsig5
c
      real*8 uuccb(-1:1,-1:1,2:3), ddccb(-1:1,-1:1,2:3),
     1       uussb(-1:1,-1:1,2:3), ddssb(-1:1,-1:1,2:3),
     2       udscb(-1:1,-1:1,2:3), ducsb(-1:1,-1:1,2:3),
     3       uucc(0:3), uuss(0:3), ddcc(0:3),
     4       ddss(0:3), udsc(0:3), ducs(0:3),
     5       uuccr(0:3), uussr(0:3), ddccr(0:3),
     6       ddssr(0:3), udscr(0:3), ducsr(0:3)
      real*8 pbar(0:3,4+max_v), qbar(0:4), y,q2,v2,q3,v3,sub(2:3), 
     1       e_in(2), dotrr, gs2(2:3), polcolq, polcolg
      real*8 xa
      double precision NCmatrixelt(0:1,0:1,3,2)
      double precision CCmatrixelt(0:1,3,2)
      integer iflav(5), diagToPhys(5), FL_ZHg
      external dotrr, FL_ZHg

      logical ChargedCurrent, sametype, oneAntiparticle
      logical loldp, ldebug, lres
      integer bosdec
c      parameter (bosdec=3) !for H -> WW 
c      parameter (bosdec=2) ! for H -> ZZ
      data e_in/0d0,0d0/
      save e_in,loldp, uuccb,uussb,ddccb,ddssb,udscb,ducsb,
     1     gs2,polcolq,polcolg
      integer i,if1,if2,icc1,j,k,mu, fc(6), fcb(6)
      
      parameter (ldebug=.false.)
c      common/ dbug / ldebug  ! "debugging value" from calling subroutine m2s_wbfzhg

      do j = 0,3
         uucc(j) = 0d0
         uuss(j) = 0d0
         ddcc(j) = 0d0
         ddss(j) = 0d0
         udsc(j) = 0d0
         ducs(j) = 0d0
      enddo
      nmin = nmax+1
      do i = 1,5
         diagToPhys(physToDiag(i)) = i
      enddo

      loldp = e_in(1).eq.p(0,3,1) .and. e_in(2).eq.p(0,4,1)
c  reset the LO amplitude to 0 to avoid wrong subtraction for initial gluon
      if (.not.loldp) then            ! this is a new phase space point
         e_in(1) = p(0,3,1)
         e_in(2) = p(0,4,1)
         do i = -1,1
            do j = -1,1
               do k = 2,3
                  uuccb(i,j,k) = 0d0
                  uussb(i,j,k) = 0d0
                  ddccb(i,j,k) = 0d0
                  ddssb(i,j,k) = 0d0
                  udscb(i,j,k) = 0d0
                  ducsb(i,j,k) = 0d0
               enddo
            enddo
         enddo
c  determine strong coupling gs for the two quark lines and factor for 
c  polarization and spin average
         gs2(2) = 4d0*pi*als(1,1)
         gs2(3) = 4d0*pi*als(2,1)
         polcolq = 1d0/(4d0*N**2*xi(1)*xi(2))
         polcolg = 1d0/(4d0*N*(N**2-1)*xi(1)*xi(2))
      endif
         
c get the real emission amplitude squared, store it in uucc(-1,1) etc.
      do mu = 0,3
         do i = 1,5
            pbar(mu,physToDiag(i))=p(mu,i,1)
         enddo
         qbar(mu) = pbar(mu,5)
      enddo
      qbar(4)=0

      if (bos.eq.2) then
         do mu = 0,3             ! kinematics for Z-->l+l- decay
            pbar(mu,5) = v(mu,2) ! ebar
            pbar(mu,6) = v(mu,1) ! e
         enddo
         fsign(5) = -1
         fsign(6) = 1
         if (lok(1) .or. nlo.eq.0) 
     1      call qqzqqj_c(pbar,fsign,qbar,gsign, 
     1            uucc(1),uuss(1),ddcc(1),ddss(1),udsc(1),ducs(1))
      elseif (bos.eq.6) then
         if (bosdec.eq.3) then
            do mu = 0,3         ! kinematics for H-->WW -->4 lepton decay
               pbar(mu,5) = v(mu,2) ! l+
               pbar(mu,6) = v(mu,1) ! nu
               pbar(mu,7) = v(mu,3) ! nubar
               pbar(mu,8) = v(mu,4) ! l-
            enddo
            fsign(5) = -1
            fsign(6) = 1
            fsign(7) = -1
            fsign(8) = 1
         else
            do mu = 0,3
               pbar(mu,5) = 0       ! dummy momentum
               pbar(mu,6) = 0
               do i = 1,n_v
                  pbar(mu,6) = pbar(mu,6) + v(mu,i)   ! Higgs momentum
               enddo
            enddo
            fsign(5) = 0
            fsign(6) = 1
         endif                  
         if (lok(1) .or. nlo.eq.0) 
     1      call qqhqqj_c(pbar,fsign,qbar,gsign, bosdec,
     1            uucc(1),uuss(1),ddcc(1),ddss(1),udsc(1),ducs(1))
      elseif (bos.eq.34 .or. bos.eq.43) then         ! W+W- to 4 leptons
         do mu = 0,3            ! kinematics for H-->WW -->4 lepton decay
            pbar(mu,5) = v(mu,2) ! l+
            pbar(mu,6) = v(mu,1) ! nu
            pbar(mu,7) = v(mu,3) ! nubar
            pbar(mu,8) = v(mu,4) ! l-
         enddo
         fsign(5) = -1
         fsign(6) = 1
         fsign(7) = -1
         fsign(8) = 1
c
         if (lok(1) .or. nlo.eq.0) then
c original version for H->WW only:		 
c           call qqhqqj_c(pbar,fsign,qbar,gsign, bosdec,
c     1            uucc(1),uuss(1),ddcc(1),ddss(1),udsc(1),ducs(1))
c	
c new routine "bqqwwqqj" with madgraph-call:
c            call bqqwwqqj_comp(pbar,fsign,qbar,gsign,bosdec,
c     &           uucc(1),uuss(1),ddcc(1),ddss(1),udsc(1),ducs(1))
     
c new routine (with speed-up): uucc(1) stands for uucc(1:3) here!    
            call qqwwqqj(pbar,fsign,qbar,gsign,!bosdec,
     &            uucc(1),uuss(1),ddcc(1),ddss(1),udsc(1),ducs(1))  
         endif	

      elseif (bos.eq.22) then         ! ZZ to 4 leptons
         do mu = 0,3            ! kinematics for H-->ZZ -->4 lepton decay
            pbar(mu,5) = v(mu,1) ! l+
            pbar(mu,6) = v(mu,2) ! l-
            pbar(mu,7) = v(mu,3) ! l"+
            pbar(mu,8) = v(mu,4) ! l"-
         enddo
         fsign(5) = -1
         fsign(6) = 1
         fsign(7) = -1
         fsign(8) = 1
c
         if (lok(1) .or. nlo.eq.0) then
c routine "bqqzzqqj" with madgraph-call:
c            call bqqzzqqj_comp(pbar,fsign,qbar,gsign,bos,
c     &           uucc(1),uuss(1),ddcc(1),ddss(1),udsc(1),ducs(1))
c         
c new routine (with speed-up): uucc(1) stands for uucc(1:3) here!      
            call qqzzqqj(pbar,fsign,qbar,gsign,bos,
     &            uucc(1),uuss(1),ddcc(1),ddss(1),udsc(1),ducs(1)) 
	
c	compare to madgraph:
c            call bqqzzqqj_comp(pbar,fsign,qbar,gsign,bos,
c     &           uucc(1),uuss(1),ddcc(1),ddss(1),udsc(1),ducs(1))
     
        endif	!lok
        
      elseif (bos.eq.21) then         ! ZZ to 4 leptons
         do mu = 0,3            ! kinematics for H-->ZZ -->4 lepton decay
            pbar(mu,5) = v(mu,1) ! l+
            pbar(mu,6) = v(mu,2) ! l-
            pbar(mu,7) = v(mu,3) ! v
            pbar(mu,8) = v(mu,4) ! v~
         enddo
         fsign(5) = -1
         fsign(6) =  1
         fsign(7) =  1
         fsign(8) = -1
c
         if (lok(1) .or. nlo.eq.0) then
c routine "bqqzzqqj" with madgraph-call:
c            call bqqzzqqj_comp(pbar,fsign,qbar,gsign,bos,
c     &           uucc(1),uuss(1),ddcc(1),ddss(1),udsc(1),ducs(1))
c         
c new routine (with speed-up): uucc(1) stands for uucc(1:3) here!      
            call qqzzqqj(pbar,fsign,qbar,gsign,bos,
     &            uucc(1),uuss(1),ddcc(1),ddss(1),udsc(1),ducs(1)) 
	
c	compare to madgraph:
c            call bqqzzqqj_comp(pbar,fsign,qbar,gsign,bos,
c     &           uucc(1),uuss(1),ddcc(1),ddss(1),udsc(1),ducs(1))
     
        
         endif  !lok	

      else
         print*," Invalid entry BOS = ",bos
         print*," in call of wbf_zh3j. Must be 2, 6, 34 / 43, 22 or 21 "
         stop
      endif
      do j=2,3
         NCmatrixelt(0,0,j,1)=uucc(j)
         NCmatrixelt(0,1,j,1)=uuss(j)
         NCmatrixelt(1,0,j,1)=ddcc(j)
         NCmatrixelt(1,1,j,1)=ddss(j)
         CCmatrixelt(0,j,1)=udsc(j)
         CCmatrixelt(1,j,1)=ducs(j)
      enddo


c for the NLO case get the subtraction terms; 
c first the case with a final state gluon
      if (gsign.eq.1) then
         if (ldebug) then
            print*," final state gluon section in wbf_ZH3j "
            print*," jsig = ",jsig," jsig1,3 = ",jsig1,jsig3
            print 101," fsign = ",fsign
 101        format(a,6i5,a,i5)
         endif
         if (nlo.eq.1) then
            do j = 2,3               ! j=2: emission off 1,2 line
               do mu = 0,3           ! j=3: emission off 3,4 line
                  do i = 1,4
                     pbar(mu,physToDiag(i))=p(mu,i,j)
                  enddo
               enddo
               if (bos.eq.2 .and. lok(j)) then
                  call qqZqq(pbar,fsign,0,
     1                       uucc(j),uuss(j),ddcc(j),
     2                       ddss(j),udsc(j),ducs(j))
               elseif (bos.eq.6 .and. lok(j)) then
                  call qqHqq(pbar,fsign,0,bosdec,
     1                       uucc(j),uuss(j),ddcc(j),
     2                       ddss(j),udsc(j),ducs(j))
               elseif ((bos.eq.34 .or. bos.eq.43) .and. lok(j)) then
c H->WW only:                
c		 call qqHqq(pbar,fsign,0,bosdec,
c     1                       uucc(j),uuss(j),ddcc(j),
c     2                       ddss(j),udsc(j),ducs(j))
c
c general qq->qqWW (madgraph):
c        	  call qqwwqq_comp(pbar,fsign, 0, bosdec,
c     1                    uucc(j),uuss(j),ddcc(j),ddss(j),
c     2			  udsc(j),ducs(j))
c
c  new routine for speed-up:
         	  call qqwwqq(pbar,fsign, 0,j,
     1                    uucc(j),uuss(j),ddcc(j),ddss(j),
     2			  udsc(j),ducs(j))

               elseif ((bos.eq.22) .and. lok(j)) then
c general qq->qqZZ (madgraph):
c        	  call qqzzqq_comp(pbar,fsign, 0, bos,
c     1                    uucc(j),uuss(j),ddcc(j),ddss(j),
c     2			  udsc(j),ducs(j))

c     new routine for speed-up:
         	  call qqzzqq(pbar,fsign, 0,j,bos,
     1                    uucc(j),uuss(j),ddcc(j),ddss(j),
     2			  udsc(j),ducs(j))

               elseif ((bos.eq.21) .and. lok(j)) then
c general qq->qqZZ (madgraph):
c        	  call qqzzqq_comp(pbar,fsign, 0, bos,
c     1                    uucc(j),uuss(j),ddcc(j),ddss(j),
c     2			  udsc(j),ducs(j))

c     new routine for speed-up:
         	  call qqzzqq(pbar,fsign, 0,j,bos,
     1                    uucc(j),uuss(j),ddcc(j),ddss(j),
     2			  udsc(j),ducs(j))

               else
                  uucc(j) = 0d0
                  uuss(j) = 0d0
                  ddcc(j) = 0d0
                  ddss(j) = 0d0
                  udsc(j) = 0d0
                  ducs(j) = 0d0
               endif
               NCmatrixelt(0,0,j,2)=uucc(j)
               NCmatrixelt(0,1,j,2)=uuss(j)
               NCmatrixelt(1,0,j,2)=ddcc(j)
               NCmatrixelt(1,1,j,2)=ddss(j)
               CCmatrixelt(0,j,2)=udsc(j)
               CCmatrixelt(1,j,2)=ducs(j)

c save matrix elements for later use with initial gluons
               uuccb(fsign(1),fsign(3),j) = uucc(j)
               uussb(fsign(1),fsign(3),j) = uuss(j)
               ddccb(fsign(1),fsign(3),j) = ddcc(j)
               ddssb(fsign(1),fsign(3),j) = ddss(j)
               udscb(fsign(1),fsign(3),j) = udsc(j)
               ducsb(fsign(1),fsign(3),j) = ducs(j)
c
               q2 = 2*dotrr(qbar,p(0,j-1,1))*xuz(1,j) !p(mu,j-1,1) is inc.quark
               v2 = 2d0*gs2(j)*                      !i.e  j-1 = 1,2
     $            ( 2d0/(1-xuz(1,j)+xuz(2,j))-(1+xuz(1,j)))
               q3 = 2*dotrr(qbar,p(0,j+1,1))*xuz(1,j) !p(mu,j+1,1) is out.quark
               v3 = 2d0*gs2(j)*                      !i.e. j+1 = 3,4
     $            ( 2d0/(1-xuz(1,j)+xuz(2,j))-(2-xuz(2,j)))
               sub(j) = v2/q2+v3/q3
            enddo
         endif
         iflav(5) = 0           ! final state gluon id
         do if1=1,nfl
            do if2=1,nfl
               iflav(1)=if1*fsign(physToDiag(1))
               iflav(3)=if1*fsign(physToDiag(3))
               iflav(2)=if2*fsign(physToDiag(2))
               iflav(4)=if2*fsign(physToDiag(4))
               do j = 2,3
                  k=FL_ZHg(iflav,j)
                  if (lok(1)) then
                     res(k,1)=pdf(sign1*iflav(1),1,2*j-3)  !1 for j=2;3 for j=3
     &                       *pdf(sign2*iflav(2),2,4-j  )  !2 for j=2;1 for j=3
     &                *NCmatrixelt(mod(if1,2),mod(if2,2),j,1)*polcolq
                  else
                     res(k,1) = 0
                  endif
                  if (nlo.eq.1 .and. lok(j)) then
                     res(k,j)=(pdf(sign1*iflav(1),1,j)
     &                        *pdf(sign2*iflav(2),2,j)
     &                        -pdf(sign1*iflav(1),1,2*j-3)
     &                        *pdf(sign2*iflav(2),2,4-j  )*sub(j))*C2
     &                *NCmatrixelt(mod(if1,2),mod(if2,2),j,2)*polcolq
                     res(k,5-j)=0
                     res(k,0) = res(k,1)+res(k,j)
                  else
                     res(k,0) = res(k,1)
                     res(k,2) = 0
                     res(k,3) = 0
                  endif
               enddo
               if (ldebug .and. lok(1) .and. nlo.eq.1) then              !debug
                  xa = res(k-1,1)+res(k,1)                               !debug
c           print*," same for res(k,*), k=",k," if1,2= ",if1,if2         !debug
                  print*,k," NC:up,low,tot",res(k-1,2)/xa,res(k,3)/xa,   !debug
     &                                     (res(k-1,2)+res(k,3))/xa      !debug
               endif                                                     !debug
C Now check if there is a CC contribution for this choice of initial state
C flavors (i,j). First: check whether initial uu or dd, not ud
               sametype=(mod(if1,2)).eq.(mod(if2,2))
               oneAntiparticle=
     &            ((fsign(physToDiag(1))*fsign(physToDiag(2))).eq.-1) 
c true if only one particle is an antiparticle
               ChargedCurrent=(oneAntiparticle.and.sametype) .or.
     &                 ( (.not.oneAntiparticle).and.(.not.sametype) )
               if ( ChargedCurrent .and. if1.le.4 .and. if2.le.4 ) then   
c change 1<-->2 and 3<-->4 in outgoing quark flavors
                  iflav(3)=(if1+2*mod(if1,2)-1)*fsign(physToDiag(3))
                  iflav(4)=(if2+2*mod(if2,2)-1)*fsign(physToDiag(4))
                  icc1 = abs(iflav(diagtophys(1)))
                  do j = 2,3
                     k=FL_ZHg(iflav,j)
                     if (lok(1)) then
                        res(k,1)=pdf(sign1*iflav(1),1,2*j-3)
     &                          *pdf(sign2*iflav(2),2,4-j  )
     &                          *CCmatrixelt(mod(icc1,2),j,1)*polcolq
                     else
                        res(k,1) = 0
                     endif
                     if (nlo.eq.1 .and. lok(j)) then
                        res(k,j)=(pdf(sign1*iflav(1),1,j)
     &                           *pdf(sign2*iflav(2),2,j)
     &                           -pdf(sign1*iflav(1),1,2*j-3)
     &                           *pdf(sign2*iflav(2),2,4-j)*sub(j))*C2
     &                           *CCmatrixelt(mod(icc1,2),j,2)*polcolq
                        res(k,5-j) = 0
                        res(k,0) = res(k,1)+res(k,j)
                     else
                        res(k,0) = res(k,1)
                        res(k,2) = 0
                        res(k,3) = 0
                     endif
                  enddo               ! loop over upper vs. lower line
                  if (ldebug .and. (jsig.le.2.or.jsig.gt.8)           !debug
     &                       .and. lok(1) .and. nlo.eq.1    ) then    !debug
                     xa = res(k-1,1)+res(k,1)                         !debug
                     print*,k," CC:up,low,tot",res(k-1,2)/xa,         !debug
     &                      res(k,3)/xa,(res(k-1,2)+res(k,3))/xa      !debug
                  endif                                               !debug
               endif               ! endif(CC)
            enddo
         enddo 

      elseif (gsign.eq.-1) then           !initial gluon section
         j = 0
         if (nlo.eq.0) then
            if (fsign(1).eq.-fsign(2)) then 
               j=2              ! j=2: emission off 1,2 line
            elseif(fsign(3).eq.-fsign(4)) then 
               j=3              ! j=3: emission off 3,4 line
            endif
         elseif (nlo.eq.1) then
            do i=1,6
               fc(i)=fsign(i)
               fcb(i)=fsign(i)
            enddo
            if (fsign(1).eq.-fsign(2)) then 
               j=2              ! j=2: emission off 1,2 line
               fc(2)=-fsign(2)
               q2 = 2*dotrr(qbar,pbar(0,2))*xuz(1,j)
               fcb(1)=-fsign(1)
               q3 = 2*dotrr(qbar,pbar(0,1))*xuz(1,j)
            elseif(fsign(3).eq.-fsign(4)) then 
               j=3              ! j=3: emission off 3,4 line
               fc(4)=-fsign(4)
               q2 = 2*dotrr(qbar,pbar(0,4))*xuz(1,j)
               fcb(3)=-fsign(3)
               q3 = 2*dotrr(qbar,pbar(0,3))*xuz(1,j)
            endif
            v2 = 2d0*gs2(j) * ( (1-xuz(1,j))**2 + xuz(1,j)**2 )
            v3 = v2

            NCmatrixelt(0,0,2,2)=uuccb(fc(1),fc(3),j)
            NCmatrixelt(0,1,2,2)=uussb(fc(1),fc(3),j)
            NCmatrixelt(1,0,2,2)=ddccb(fc(1),fc(3),j)
            NCmatrixelt(1,1,2,2)=ddssb(fc(1),fc(3),j)
            CCmatrixelt(0,2,2)=udscb(fc(1),fc(3),j)
            CCmatrixelt(1,2,2)=ducsb(fc(1),fc(3),j)

            NCmatrixelt(0,0,3,2)=uuccb(fcb(1),fcb(3),j)
            NCmatrixelt(0,1,3,2)=uussb(fcb(1),fcb(3),j)
            NCmatrixelt(1,0,3,2)=ddccb(fcb(1),fcb(3),j)
            NCmatrixelt(1,1,3,2)=ddssb(fcb(1),fcb(3),j)
            CCmatrixelt(0,3,2)=udscb(fcb(1),fcb(3),j)
            CCmatrixelt(1,3,2)=ducsb(fcb(1),fcb(3),j)
         endif
         do if1=1,nfl
            do if2=1,nfl
               iflav(j-1) = 0
               iflav(4-j)=if1*fsign(physToDiag(4-j))
               iflav(6-j)=if1*fsign(physToDiag(6-j))
               iflav(j+1)=if2*fsign(physToDiag(j+1))
               iflav(5)  =if2*fsign(physToDiag(5))
               k=FL_ZHg(iflav,j)
               if (lok(1)) then
                  res(k,1)=pdf(sign1*iflav(1),1,2*j-3)    !1 for j=2;3 for j=3
     &                    *pdf(sign2*iflav(2),2,4-j)      !2 for j=2;1 for j=3
     &                 *NCmatrixelt(mod(if1,2),mod(if2,2),j,1)*polcolg
               else
                  res(k,1) = 0
               endif
               if ( nlo.eq.1 .and. lok(j) ) then
                  res(k,j) = ( pdf(sign1*iflav(1),1,j)
     &                        *pdf(sign2*iflav(2),2,j)
     &               *(NCmatrixelt(mod(if1,2),mod(if2,2),2,2) +
     &                 NCmatrixelt(mod(if1,2),mod(if2,2),3,2))
     &               - pdf(sign1*iflav(1),1,2*j-3)
     &               * pdf(sign2*iflav(2),2,4-j) *
     &                (NCmatrixelt(mod(if1,2),mod(if2,2),2,2)*v2/q2 +
     &                 NCmatrixelt(mod(if1,2),mod(if2,2),3,2)*v3/q3) )
     &                     * C2*polcolg
                  res(k,0) = res(k,1)+res(k,j)
                  res(k,5-j)=0
               else
                  res(k,0) = res(k,1)
                  res(k,2) = 0
                  res(k,3) = 0
               endif
               if (ldebug .and. lok(1) .and. nlo.eq.1) then             !debug
                  print*," k=",k," if1,2=",if1,if2                      !debug
                  if (j.eq.2) then                                      !debug
                    print*," upper line NC ",res(k,2)/res(k,1)          !debug
                  elseif (j.eq.3) then                                  !debug
                    print*," lower line NC ",res(k,3)/res(k,1)          !debug
                  endif                                                 !debug
               endif                                                    !debug
            enddo
         enddo
C Next the CC contributions; no b quarks allowed here
         iflav(diagToPhys(5)) = 0
         do if1 = 1,4           ! sum over all flavors for quark 1 and 2
            iflav(diagToPhys(1)) = fsign(1)*if1
            iflav(diagToPhys(2)) = fsign(2)*(if1-1+2*mod(if1,2))
            do if2 = mod(if1,2)+1,4,2 ! flavor of q3 set by q2 (mod 2)
               iflav(diagToPhys(3)) = fsign(3)*if2
               iflav(diagToPhys(4)) = fsign(4)*(if2-1+2*mod(if2,2))
               k=FL_ZHg(iflav,j)
               if (lok(1)) then
                  res(k,1)=pdf(sign1*iflav(1),1,2*j-3)
     &                    *pdf(sign2*iflav(2),2,4-j)
     &                    *CCmatrixelt(mod(if1,2),j,1)*polcolg
               else
                  res(k,1) = 0
               endif
               if ( nlo.eq.1 .and. lok(j) ) then
                  res(k,j)=(pdf(sign1*iflav(1),1,j)*
     &                      pdf(sign2*iflav(2),2,j)*
     &     (CCmatrixelt(mod(if1,2),2,2) + CCmatrixelt(mod(if1,2),3,2)) 
     &                    - pdf(sign1*iflav(1),1,2*j-3)*
     &                      pdf(sign2*iflav(2),2,4-j)*
     &                  (CCmatrixelt(mod(if1,2),2,2)*v2/q2 +
     &                   CCmatrixelt(mod(if1,2),3,2)*v3/q3) )
     &                    * C2*polcolg
                  res(k,0) = res(k,1)+res(k,j)
                  res(k,5-j)=0
               else
                  res(k,0) = res(k,1)
                  res(k,2) = 0
                  res(k,3) = 0
               endif
               if (ldebug .and. (jsig.le.2.or.jsig.gt.8)                !debug
     &                    .and. lok(1) .and. nlo.eq.1    ) then         !debug
                  print*," CC case for k = ",k," if1,2=",if1,if2        !debug
                  print*," iflav = ",iflav                              !debug
                  if (j.eq.2)                                           !debug
     &                  print*," upper line CC ",res(k,2)/res(k,1)      !debug
                  if (j.eq.3)                                           !debug
     &                  print*," lower line CC ",res(k,3)/res(k,1)      !debug
               endif                                                    !debug
            enddo
         enddo 
      endif

      nmax = FL_ZHg(iflav,-2)
      end
c******************************************************************
c
c   end subroutine wbf_zh3j
c
c*****************************************************************



c******************************************************************
c
c   begin function FL_ZHg
c
c*****************************************************************
      integer function FL_ZHg(iflav,colstruc)
      implicit none
      integer iflav(5),colstruc    ! input for color and flavor assignment
c  fill hepup color and flavor assignments for WBF processes with
c  one attached external gluon and count subprocesses. 
c  There are 2 color structures depending into which fermion line the 
c  gluon is inserted
c
c  colstruc = 2:upper
c             3:lower fermion line
c
c  cases can be distinguished according to which iflav(i)=0, i.e. corresponds
c  to the gluon

c  id1,2 are flavor identifiers for incoming quarks
c  id3,4 are flavor identifiers for outgoing quarks
c  gsign=+1: outgoing gluon, gsign=-1 for incoming gluon
c  colstruc =2,3 determines whether the gluon is coupled to the 
c  uppper or lower fermion line:

c  Note that colstruc is also a flag:
c  if colstruc=-1, then we are resetting.

#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
      
      integer lkup
      common /localkup/ lkup(numParticles,7,maxNumSubProc)
 
      integer listposition
      integer numresets
      save listposition
      save numresets

      data numresets /-1/
      data listposition /0/

      if (colstruc.eq.-1) then  ! we are resetting
c       write(*,*) "we are resetting"
        listposition=0
        numresets=numresets+1
      else if (colstruc.eq.-2) then
c       don"t do anything, just return the number of subprocesses.
      else
c increment the counter regardless of whether or not it"s the 1st time through.
        listposition=listposition+1

c       we fill info for this subprocess,if required
        if(numresets.eq.0) then
          call fillColoredPartons_ZHg(iflav,
     1                                listposition+numdecay,colstruc)
        endif       ! numresets.eq.0
      endif         
      FL_ZHg=listposition
      end
c******************************************************************************
c
c   end function FL_ZHg
c
c******************************************************************************


c*****************************************************************************
c
c    begin  subroutine fillColoredPartons_ZHg
c
c*****************************************************************************
      subroutine fillColoredPartons_ZHg(iflav,listposition,colstruc)
c  assigns values to the variables in the common block localHEPUP
c  in particular, this subroutine assigns values to those variables that 
c  will be stored in the lookup tables generated by writeHEPUPtable.  
c  As the name suggests, this routine only stores the information for the 
c  colored partons.  Particles without color will be dealt with in the 
c  subroutine fillColorless.

      implicit none
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
#include "VBFNLO/utilities/process.inc"

      integer iflav(5),id1,id2,id3,id4,id5,listposition,i,i1
      integer iflavour(5),colstruc

      logical ldebug
      parameter (ldebug=.false.)


      do i1=1,5
         iflavour(i1)=iflav(i1) 
         if(iflavour(i1).eq.0) iflavour(i1)=21   
      enddo

c...flavours of 4 quarks and gluon

      id1=iflavour(1)
      id2=iflavour(2)
      id3=iflavour(3)
      id4=iflavour(4)
      id5=iflavour(5)

      select case (process)
      case(HjjjLO_WW, HjjjLO_ZZ_ll, HjjjLO_ZZ_lnu, WPWMjjjLO,
     1     ZZjjjLO_ll, ZZjjjLO_lnu)
         lnup(listposition)=numParticles+3 
      case(HjjjLO)
         lnup(listposition)=numParticles-2
      case(HjjjLO_AA, HjjjLO_mu, HjjjLO_tau, HjjjLO_bbar, ZjjjLO_l,
     1     ZjjjLO_nu, WPjjjLO, WMjjjLO)
         lnup(listposition)=numParticles            
      end select
      
      listup(1,listposition)=-1 !incoming quarks
      listup(2,listposition)=-1
      listup(3,listposition)=1  !outgoing quarks
      listup(4,listposition)=1
      listup(5,listposition)=1
 
c...four quarks and gluon
      lidup(1,listposition)=id1    
      lidup(2,listposition)=id2
      lidup(3,listposition)=id3
      lidup(4,listposition)=id4
      lidup(5,listposition)=id5  
      
      do i=3,n_p
         lmothup(1,i,listposition)=1
         lmothup(2,i,listposition)=2
      enddo

c...final state gluon
      if(id5.eq.21)then
         if(colstruc.eq.2)then
            if(id1.gt.0) then
               licolup(1,1,listposition)=501
               licolup(1,3,listposition)=502
               licolup(1,5,listposition)=501
               licolup(2,1,listposition)=0
               licolup(2,3,listposition)=0
               licolup(2,5,listposition)=502
            else
               licolup(1,1,listposition)=0
               licolup(1,3,listposition)=0
               licolup(1,5,listposition)=502
               licolup(2,1,listposition)=501
               licolup(2,3,listposition)=502
               licolup(2,5,listposition)=501               
            endif            
            if(id2.gt.0) then
               licolup(1,2,listposition)=503
               licolup(1,4,listposition)=503
               licolup(2,2,listposition)=0
               licolup(2,4,listposition)=0
            else
               licolup(1,2,listposition)=0
               licolup(1,4,listposition)=0
               licolup(2,2,listposition)=503
               licolup(2,4,listposition)=503
            endif
         elseif(colstruc.eq.3)then
            if(id1.gt.0) then
               licolup(1,1,listposition)=501
               licolup(1,3,listposition)=501
               licolup(2,1,listposition)=0
               licolup(2,3,listposition)=0
            else
               licolup(1,1,listposition)=0
               licolup(1,3,listposition)=0
               licolup(2,1,listposition)=501
               licolup(2,3,listposition)=501
            endif         
            if(id2.gt.0) then
               licolup(1,2,listposition)=502
               licolup(1,4,listposition)=503
               licolup(1,5,listposition)=502
               licolup(2,2,listposition)=0
               licolup(2,4,listposition)=0
               licolup(2,5,listposition)=503
            else
               licolup(1,2,listposition)=0
               licolup(1,4,listposition)=0
               licolup(1,5,listposition)=503
               licolup(2,2,listposition)=502
               licolup(2,4,listposition)=503
               licolup(2,5,listposition)=502
            endif
         endif
      endif

c...initial particles gluon and quark/antiquark 
      if(id1.eq.21)then 
         if(id3.gt.0)then
            licolup(1,1,listposition)=502
            licolup(1,3,listposition)=502
            licolup(2,1,listposition)=501
            licolup(2,3,listposition)=0
         else
            licolup(1,1,listposition)=501
            licolup(1,3,listposition)=0
            licolup(2,1,listposition)=502
            licolup(2,3,listposition)=502
         endif 
         if(id5.gt.0)then
            licolup(1,5,listposition)=501
            licolup(2,5,listposition)=0
         else
            licolup(1,5,listposition)=0
            licolup(2,5,listposition)=501            
         endif
         if(id2.gt.0) then
            licolup(1,2,listposition)=503
            licolup(1,4,listposition)=503
            licolup(2,2,listposition)=0
            licolup(2,4,listposition)=0
         else
            licolup(1,2,listposition)=0
            licolup(1,4,listposition)=0
            licolup(2,2,listposition)=503
            licolup(2,4,listposition)=503
         endif         
      endif
      
c...initial particles quark/antiquark and gluon 
      if(id2.eq.21)then 
         if(id1.gt.0)then
            licolup(1,1,listposition)=501
            licolup(1,3,listposition)=501
            licolup(2,1,listposition)=0
            licolup(2,3,listposition)=0
         else
            licolup(1,1,listposition)=0
            licolup(1,3,listposition)=0
            licolup(2,1,listposition)=501
            licolup(2,3,listposition)=501
         endif 
         if(id5.gt.0)then
            licolup(1,5,listposition)=502
            licolup(2,5,listposition)=0
         else
            licolup(1,5,listposition)=0
            licolup(2,5,listposition)=502            
         endif
         if(id4.gt.0) then
            licolup(1,2,listposition)=503
            licolup(1,4,listposition)=503
            licolup(2,2,listposition)=502
            licolup(2,4,listposition)=0
         else
            licolup(1,2,listposition)=502
            licolup(1,4,listposition)=0
            licolup(2,2,listposition)=503
            licolup(2,4,listposition)=503
         endif
      endif

      if(ldebug)then
         print*,"====================================="
         print("(1P,5I6)"),id1,id2,id3,id4,id5
         print*,"-------------------------------------"
         print("(1P,5I6)"),(licolup(1,i1,listposition),i1=1,5)
         print("(1P,5I6)"),(licolup(2,i1,listposition),i1=1,5)
         print*,"====================================="
         print*,""
      endif
    
      end  ! fillColoredPartons_ZHg
c*****************************************************************************
c
c    end subroutine fillColoredPartons_ZHg
c
c*****************************************************************************

      subroutine resprint(nmin,nmax,res)
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
      integer nmin,nmax,k,i,ic
      save ic
      real*8 resold
      common /bresprint/ resold(16)
      real*8 xa,xb,xc,xd,res(maxnumsubproc,0:3),fac
      if (nmin.eq.1) ic=1
      xa = 0
      xb = 0
      xc = 0
      xd = 0
      if (nmin.le.190) then 
         fac = 4*9
      else
         fac = 4d0*24d0
      endif
      do k = nmin,nmax
         xa = xa + res(k,1)
         xb = xb + res(k,2)
         xc = xc + res(k,3)
         xd = xd + res(k,0)
      enddo
      if (nmax.lt.-50) then
         do k = nmin,nmax
            print*," res(",k,") ",(res(k,i),i=1,3)
         enddo
      endif
      write(6,10)" res([",nmin,",",nmax,"])",xa,xb/xa,xc/xa,xd/xa
c     1         ,xa/(resold(ic)+resold(ic+1))*fac
 10   format(a,i3,a,i3,a,g11.3,3f18.10)
      ic = ic+2
      end
