c
c  Dieter Zeppenfeld, <dieter@pheno.physics.wisc.edu>
c  Initial version:  2003 February 12
c  Last modified by Barbara Jager:  2006 July 11
c
c  this function computes |ME|^2 for qq->qq e+ve mu-vm~ ("WW")
c	or qq->qq e+e-mu+mu- or qq->qq e+e- vm vm~ ("ZZ")
c	at LO and virtual corrections via call of "qqwwqq.f" or "qqzzqq.f"
c
c	for qq->qqZZ random helicity summation is employed for decay leptons
c
c ------------------------------------------------------------------------
c
      real*8 function m2s_wbfvv(
     &                   bos,   !in:  Boson identifier, 2 = Z, 6=H, 34=W+W-
     &                   bosdec,!in:  identifier for Boson decay
     &                   nlo,   !in:  Born only for nlo=0, +interference for 1
     &                   xi,	!in:  Feynman x parameters of incoming fermions
     &                   p,	!in:  momenta associated with external fermions
     &                   v,	!in:  momenta associated with Z/H dec. fermions
     &                   rn     !in:  random number array
     &                        )
      implicit none
c
c declare input/output variables
c
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/coupl.inc"
#include "tensor.inc"
#include "tensorz.inc"
#include "VBFNLO/utilities/lha.inc"
#include "VBFNLO/utilities/tau_hel.inc"
      integer bos, nlo, bosdec
      real*8 xi(nx), p(0:3,max_p), v(0:3,max_v), 
     1       rn(1), m2s

c  helicity selection
      integer h, hh
      integer jsig, jsig1, jsig3, jsig5
      common /chelsum/ jsig,jsig1,jsig3,jsig5

      complex*16 zero, z1
      parameter (zero=(0d0,0d0))
c
c declare external functions
c
      integer FL_ZH
      double complex dotrc
      external FL_ZH, dotrc
c
c alfas, scales etc
#include "VBFNLO/utilities/scales.inc"

      real*8 q12(0:4,3), q34(0:4,3), qww(0:3), qvv(0:3)

c
c declare local variables
c
      real*8 q_sf, qq

      integer init/0/, I, J, mu, nu, id
      save init
c
c declare variables for summation process
c
      INTEGER ires

      integer FSIGN(4+max_v), physToDiag(4), nmin, nmax, nproc(4)
      save nmax, nproc
c
c store contributions from subprocess ip in res(ip) where
      real*8 res(maxnumsubproc)
c
c declare parton distribution variables
c
      real*8 pdf(-6:6,2)
c
c define program switches
c
      logical ldebug
      parameter (ldebug=.false.)		!output debug information

      common /hcount / h
      real*8 weight,rnumb,RandomNumber

c -----------------------------------------------------------------------

      if ( init .eq. 0 ) then

         if (bos.eq.2) then
            write(6,*) " "
            write(6,*) "Zjj amplitude square information:"
            write(6,*) "-----------------------------------------------"
            write(6,*) " "
         elseif (bos.eq.6) then
            write(6,*) " "
            write(6,*) "Hjj amplitude square information:"
            write(6,*) "-----------------------------------------------"
            write(6,*) " "
         elseif (bos.eq.34 .or. bos.eq.43) then
            write(6,*) " "
            write(6,*) "W+W- jj amplitude square information:"
            write(6,*) "-----------------------------------------------"
            write(6,*) " "
            call vtoww_reset
         elseif (bos.eq.22) then
            write(6,*) " "
            write(6,*) "ZZ jj -> 4l amplitude square information:"
            write(6,*) "-----------------------------------------------"
            write(6,*) " "
            call vtozz_reset
         elseif (bos.eq.21) then
            write(6,*) " "
            write(6,*) "ZZ jj -> 2l 2v amplitude square information:"
            write(6,*) "-----------------------------------------------"
            write(6,*) " "
            call vtozz_reset
         endif
         if (nfl.eq.4) then
            print*," For NC processes external b-quark contributions",
     1             " are excluded"
         elseif (nfl.eq.5) then
            print*," For NC processes external b-quark contributions",
     1             " are included"
         else
            print*," Check setting of nfl in global.inc "
         endif
         h = 1
         init = 1
         do i = 1,maxnumsubproc
            res(i) = 0
         enddo
      else
         do i = 1,nmax
            res(i) = 0
         enddo
      endif


c -----------------------------------------------------------------------
      call Calc_Momentum_Transfer(p, v, q12,q34,1)


c
c for WW->4l precalculate A->WW,Z->WW,AZ->WW etc leptonic tensors
c
c      if (.false.) then
      if (bos.eq.34 .or. bos.eq.43) then
c reset lfs to .true. to force recalculation of virtual contributions in qqwwqq
         do i = 1,4
            lfs(i) = .true.
         enddo

c...Les Houches interface         
       if (lha.and..not.doNLO) then
          helicity(1)=-1
          helicity(2)= 1
          helicity(3)= 1
          helicity(4)=-1
       endif


c lepton spinors and W+- polarization vectors
         CALL IXXXXX(v(0,2),ZERO ,1,-1,wep) !W(1,5))          !e+       
         CALL OXXXXX(v(0,1),ZERO ,-1,1,wve) !W(1,6))          !ve 
         CALL OXXXXX(v(0,4),ZERO ,-1,1,wmu) !W(1,7))          !mu-      
         CALL IXXXXX(v(0,3),ZERO ,1,-1,wvm) !W(1,8))          !vm~
         CALL JIOXXX(wep,wve,GWF ,WMASS,WWIDTH,wp) !W(1,10))    !W+
         CALL JIOXXX(wvm,wmu,GWF ,WMASS,WWIDTH,wm) !W(1,11))    !W-

         do mu = 0,3
            qp(mu) = v(mu,1)+v(mu,2)
            qm(mu) = v(mu,3)+v(mu,4)
            qww(mu) = qp(mu) + qm(mu)
         enddo
         qp(4) = qp(0)**2-qp(1)**2-qp(2)**2-qp(3)**2
         qm(4) = qm(0)**2-qm(1)**2-qm(2)**2-qm(3)**2
         if (abs(nlo).eq.5 .or. nlo.eq.-4) then     ! determine W+- polarization vectors 
c wpp(mu) = wp(mu) - xp*qp(mu) and wmp(mu) = wm(mu) - xm*qm(mu)
            qq = qp(0)*qww(0)-qp(1)*qww(1)-qp(2)*qww(2)-qp(3)*qww(3)
            xp = dotrc(qww,wp)/qq
            qq = qm(0)*qww(0)-qm(1)*qww(1)-qm(2)*qww(2)-qm(3)*qww(3)
            xm = dotrc(qww,wm)/qq
            do mu = 0,3
               wpp(mu+1) = wp(mu+1) - xp*qp(mu)
               wmp(mu+1) = wm(mu+1) - xm*qm(mu)
            enddo
            do mu = 5,6
               wpp(mu) = wp(mu)
               wmp(mu) = wm(mu)
            enddo
         endif
c leptonic tensors
C for W+W-
         j = 1
         if (with_anom) then ! anomalous gauge boson couplings
            call atoww_anomal(v,aww)
            call ztoww_anomal(v,zww)
            call aatoww_anomal(q12(0,j),q34(0,j),v,aaww(0,0,j))
            call aztoww_anomal(q12(0,j),q34(0,j),v,azww(0,0,j))
            call aztoww_anomal(q34(0,j),q12(0,j),v,zaww(0,0,j))
            call zztoww_anomal(q12(0,j),q34(0,j),v,zzww(0,0,j))
            call wwtoww_anomal(q12(0,j),q34(0,j),v,wwww6(0,0,j)) ! q12 = W-
            call wwtoww_anomal(q34(0,j),q12(0,j),v,wwww5(0,0,j)) ! q12 = W+

            call WVtoWP_anomal(2,q34(0,j),v,NCwpa(0,0,1,j),NCwpz(0,0,1,j)) !emit W- on upper
            call WVtoWP_anomal(2,q12(0,j),v,NCwpa(0,0,2,j),NCwpz(0,0,2,j)) !emit W- on lower
            call WVtoWP_anomal(1,q34(0,j),v,CCwpa(0,0,1,j),CCwpz(0,0,1,j)) !emit W- on upper
            call WVtoWP_anomal(1,q12(0,j),v,CCwpa(0,0,2,j),CCwpz(0,0,2,j)) !emit W- on lower
C     for WV --> mu- nu_mu
            call WVtoWM_anomal(2,q34(0,j),v,NCwma(0,0,1,j),NCwmz(0,0,1,j)) !emit W+ on upper
            call WVtoWM_anomal(2,q12(0,j),v,NCwma(0,0,2,j),NCwmz(0,0,2,j)) !emit W+ on lower
            call WVtoWM_anomal(1,q34(0,j),v,CCwma(0,0,1,j),CCwmz(0,0,1,j)) !emit W+ on upper
            call WVtoWM_anomal(1,q12(0,j),v,CCwma(0,0,2,j),CCwmz(0,0,2,j)) !emit W+ on lower

         elseif (with_kk) then ! kaluza-klein scenario
#ifdef WITH_KK
            call atoww_kk(v,aww)
            call ztoww_kk(v,zww)
            call aatoww_kk(q12(0,j),q34(0,j),v,aaww(0,0,j))
            call aztoww_kk(q12(0,j),q34(0,j),v,azww(0,0,j))
            call aztoww_kk(q34(0,j),q12(0,j),v,zaww(0,0,j))
            call zztoww_kk(q12(0,j),q34(0,j),v,zzww(0,0,j))
            call wwtoww_kk(q12(0,j),q34(0,j),v,wwww6(0,0,j)) ! q12 = W-
            call wwtoww_kk(q34(0,j),q12(0,j),v,wwww5(0,0,j)) ! q12 = W+

            call WVtoWP_kk(2,q34(0,j),v,NCwpa(0,0,1,j),NCwpz(0,0,1,j)) !emit W- on upper
            call WVtoWP_kk(2,q12(0,j),v,NCwpa(0,0,2,j),NCwpz(0,0,2,j)) !emit W- on lower
            call WVtoWP_kk(1,q34(0,j),v,CCwpa(0,0,1,j),CCwpz(0,0,1,j)) !emit W- on upper
            call WVtoWP_kk(1,q12(0,j),v,CCwpa(0,0,2,j),CCwpz(0,0,2,j)) !emit W- on lower
C     for WV --> mu- nu_mu
            call WVtoWM_kk(2,q34(0,j),v,NCwma(0,0,1,j),NCwmz(0,0,1,j)) !emit W+ on upper
            call WVtoWM_kk(2,q12(0,j),v,NCwma(0,0,2,j),NCwmz(0,0,2,j)) !emit W+ on lower
            call WVtoWM_kk(1,q34(0,j),v,CCwma(0,0,1,j),CCwmz(0,0,1,j)) !emit W+ on upper
            call WVtoWM_kk(1,q12(0,j),v,CCwma(0,0,2,j),CCwmz(0,0,2,j)) !emit W+ on lower
#endif
         else !(SM)
            call atoww(v,aww)
c         z1 = qww(0)*aww(0) - qww(1)*aww(1) - qww(2)*aww(2) - qww(3)*aww(3)
c         print*," gauge check A: ",
c     1   z1/(qww(0)*max(abs(aww(0)),abs(aww(1)),abs(aww(2)),abs(aww(3))))
            call ztoww(v,zww)
c         z1 = qww(0)*zww(0) - qww(1)*zww(1) - qww(2)*zww(2) - qww(3)*zww(3)
c         print*," gauge check Z: ",
c     1   z1/(qww(0)*max(abs(zww(0)),abs(zww(1)),abs(zww(2)),abs(zww(3))))
                  
            call aatoww(q12(0,j),q34(0,j),v,aaww(0,0,j))
            call aztoww(q12(0,j),q34(0,j),v,azww(0,0,j))
            call aztoww(q34(0,j),q12(0,j),v,zaww(0,0,j))
            call zztoww(q12(0,j),q34(0,j),v,zzww(0,0,j))
            call wwtoww(q12(0,j),q34(0,j),v,wwww6(0,0,j)) ! q12 = W-
            call wwtoww(q34(0,j),q12(0,j),v,wwww5(0,0,j)) ! q12 = W+
C for WV --> e+ nu_e
c NCw tensors for NC process (k=1...4), CCw for CC (k=5,6)
            call WVtoWP(2,q34(0,j),v,NCwpa(0,0,1,j),NCwpz(0,0,1,j)) !emit W- on upper
            call WVtoWP(2,q12(0,j),v,NCwpa(0,0,2,j),NCwpz(0,0,2,j)) !emit W- on lower
            call WVtoWP(1,q34(0,j),v,CCwpa(0,0,1,j),CCwpz(0,0,1,j)) !emit W- on upper
            call WVtoWP(1,q12(0,j),v,CCwpa(0,0,2,j),CCwpz(0,0,2,j)) !emit W- on lower
C     for WV --> mu- nu_mu
            call WVtoWM(2,q34(0,j),v,NCwma(0,0,1,j),NCwmz(0,0,1,j)) !emit W+ on upper
            call WVtoWM(2,q12(0,j),v,NCwma(0,0,2,j),NCwmz(0,0,2,j)) !emit W+ on lower
            call WVtoWM(1,q34(0,j),v,CCwma(0,0,1,j),CCwmz(0,0,1,j)) !emit W+ on upper
            call WVtoWM(1,q12(0,j),v,CCwma(0,0,2,j),CCwmz(0,0,2,j)) !emit W+ on lower
         endif
c     
c-----------------------------
c

c for ZZ->4l precalculate  leptonic tensors
      elseif (bos.eq.22) then
c reset lzs to .true. to force recalculation of virtual contributions in qqzzqq
         do i = 1,4
            lzs(i) = .true.
         enddo
c lepton spinors and Z polarization vectors ! 
c	lepton helicities not fixed 
c 	-> sum over all possible helicites in |M|**2
c
c select helicity: h ... random number for lepton helicity
c			 combination (h=1:4) .. needed only for ZZ

        h = mod(h,4) + 1

	ie = sign(1,2-h)
	iu = (-1)**(h+1)

c...Les Houches interface
        if (lha.and..not.doNLO) then
           helicity(1)= ie
           helicity(2)=-ie
           helicity(3)= iu
           helicity(4)=-iu
        endif
		
         CALL IXXXXX(v(0,1),ZERO ,+ie,-1,lep)	      !e+	
         CALL OXXXXX(v(0,2),ZERO ,-ie, 1,lem)	      !e- 
         CALL IXXXXX(v(0,3),ZERO ,+iu,-1,lup)	      !mu+	
         CALL OXXXXX(v(0,4),ZERO ,-iu, 1,lum)	      !mu- 
	
	 CALL JIOXXX(lep,lem,GZL ,ZMASS,ZWIDTH,ze)     !Zl
	 CALL JIOXXX(lep,lem,GAL ,ZERO ,ZERO  ,ae)      !Al
         CALL JIOXXX(lup,lum,GZL ,ZMASS,ZWIDTH,zu)     !Zl
	 CALL JIOXXX(lup,lum,GAL ,ZERO ,ZERO  ,au)      !Al

         do mu = 0,3
            qe(mu) = v(mu,1)+v(mu,2)
            qu(mu) = v(mu,3)+v(mu,4)
            qvv(mu) = qe(mu) + qu(mu)
         enddo
	 	 
         qe(4) = qe(0)**2-qe(1)**2-qe(2)**2-qe(3)**2
         qu(4) = qu(0)**2-qu(1)**2-qu(2)**2-qu(3)**2

         if (abs(nlo).eq.5 .or. nlo.eq.-4) then     ! determine V/V polarization vectors 
c zet(mu) = ze(mu) - xxez*qe(mu) and zut(mu) = zu(mu) - xxmz*qu(mu)
            qq = qe(0)*qvv(0)-qe(1)*qvv(1)-qe(2)*qvv(2)-qe(3)*qvv(3)
            xxez = dotrc(qvv,ze)/qq
            xxea = dotrc(qvv,ae)/qq
            
	    qq = qu(0)*qvv(0)-qu(1)*qvv(1)-qu(2)*qvv(2)-qu(3)*qvv(3)
            xxuz = dotrc(qvv,zu)/qq            
            xxua = dotrc(qvv,au)/qq
            do mu = 0,3
               zet(mu+1) = ze(mu+1) - xxez*qe(mu)
               zut(mu+1) = zu(mu+1) - xxuz*qu(mu)
               
	       aet(mu+1) = ae(mu+1) - xxea*qe(mu)
               aut(mu+1) = au(mu+1) - xxua*qu(mu)
            enddo
            do mu = 5,6
               zet(mu) = ze(mu)
               zut(mu) = zu(mu)

               aet(mu) = ae(mu)
               aut(mu) = au(mu)
            enddo
         endif

c leptonic tensors
         j = 1
C for AA,AZ,ZA,ZZ -> e+ e- mu+ mu- :
      if (with_kk) then ! Kaluza-Klein scenario
#ifdef WITH_KK
         call vvtozz_KK(q12(0,j),q34(0,j),v,h,aazz(0,0,j),azzz(0,0,j),
     # 					   zazz(0,0,j),zzzz(0,0,j))
c
C for W+W-
         call wwtozz_KK(q12(0,j),q34(0,j),v,h,wwzz5(0,0,j))  ! q12 = W+
         call wwtozz_KK(q34(0,j),q12(0,j),v,h,wwzz6(0,0,j))  ! q12 = W-

C for V1V2 --> e+ e- or  V1V2 -->  mu+ mu- (Vi=A,Z)
c NCl tensors for NC process (k=1...4)

         call VVtoll_KK(2,1,h,q34(0,j),v,aaee(0,0,1,j),azee(0,0,1,j),
     #	 	 zaee(0,0,1,j),zzee(0,0,1,j))    	 !emit V on upper        
	 call VVtoll_KK(1,1,h,q12(0,j),v,aaee(0,0,2,j),azee(0,0,2,j),
     #		 zaee(0,0,2,j),zzee(0,0,2,j))    	 !emit V on lower
         call VVtoll_KK(2,2,h,q34(0,j),v,aauu(0,0,1,j),azuu(0,0,1,j),
     #		 zauu(0,0,1,j),zzuu(0,0,1,j))    	 !emit V on upper        
	 call VVtoll_KK(1,2,h,q12(0,j),v,aauu(0,0,2,j),azuu(0,0,2,j),
     #		zauu(0,0,2,j),zzuu(0,0,2,j))     	!emit V on lower

C for W+W- --> e+ e- or  W+W- -->  mu+ mu-
c  CCl tensors for CC (k=5)

         call WWtoll_KK(2,1,h,q34(0,j),v,CCee(0,0,1,j))     !emit V on upper
         call WWtoll_KK(1,1,h,q12(0,j),v,CCee(0,0,2,j))     !emit V on lower
         call WWtoll_KK(2,2,h,q34(0,j),v,CCuu(0,0,1,j))     !emit V on upper
         call WWtoll_KK(1,2,h,q12(0,j),v,CCuu(0,0,2,j))     !emit V on lower
	 
C for W-W+ --> e+ e- or mu+ mu-
c CCl for CC (k=6)
	
         call WWtoll_KK(1,1,h,q34(0,j),v,CCee6(0,0,1,j))     !emit V on upper
         call WWtoll_KK(2,1,h,q12(0,j),v,CCee6(0,0,2,j))     !emit V on lower
         call WWtoll_KK(1,2,h,q34(0,j),v,CCuu6(0,0,1,j))     !emit V on upper
         call WWtoll_KK(2,2,h,q12(0,j),v,CCuu6(0,0,2,j))     !emit V on lower
	 
	 
c for V-> llll
	
         call vto4l_KK(v,h,azz,zzz)	
#endif	     
c-----------------------------
c
      else ! SM
         call vvtozz(q12(0,j),q34(0,j),v,h,aazz(0,0,j),azzz(0,0,j),
     # 					   zazz(0,0,j),zzzz(0,0,j))
c
C for W+W-
         call wwtozz(q12(0,j),q34(0,j),v,h,wwzz5(0,0,j))  ! q12 = W+
         call wwtozz(q34(0,j),q12(0,j),v,h,wwzz6(0,0,j))  ! q12 = W-

C for V1V2 --> e+ e- or  V1V2 -->  mu+ mu- (Vi=A,Z)
c NCl tensors for NC process (k=1...4)

         call VVtoll(2,1,h,q34(0,j),v,aaee(0,0,1,j),azee(0,0,1,j),
     #	 	 zaee(0,0,1,j),zzee(0,0,1,j))    	 !emit V on upper        
	 call VVtoll(1,1,h,q12(0,j),v,aaee(0,0,2,j),azee(0,0,2,j),
     #		 zaee(0,0,2,j),zzee(0,0,2,j))    	 !emit V on lower
         call VVtoll(2,2,h,q34(0,j),v,aauu(0,0,1,j),azuu(0,0,1,j),
     #		 zauu(0,0,1,j),zzuu(0,0,1,j))    	 !emit V on upper        
	 call VVtoll(1,2,h,q12(0,j),v,aauu(0,0,2,j),azuu(0,0,2,j),
     #		zauu(0,0,2,j),zzuu(0,0,2,j))     	!emit V on lower

C for W+W- --> e+ e- or  W+W- -->  mu+ mu-
c  CCl tensors for CC (k=5)

         call WWtoll(2,1,h,q34(0,j),v,CCee(0,0,1,j))     !emit V on upper
         call WWtoll(1,1,h,q12(0,j),v,CCee(0,0,2,j))     !emit V on lower
         call WWtoll(2,2,h,q34(0,j),v,CCuu(0,0,1,j))     !emit V on upper
         call WWtoll(1,2,h,q12(0,j),v,CCuu(0,0,2,j))     !emit V on lower
	 
C for W-W+ --> e+ e- or mu+ mu-
c CCl for CC (k=6)
	
         call WWtoll(1,1,h,q34(0,j),v,CCee6(0,0,1,j))     !emit V on upper
         call WWtoll(2,1,h,q12(0,j),v,CCee6(0,0,2,j))     !emit V on lower
         call WWtoll(1,2,h,q34(0,j),v,CCuu6(0,0,1,j))     !emit V on upper
         call WWtoll(2,2,h,q12(0,j),v,CCuu6(0,0,2,j))     !emit V on lower
	 
	 
c for V-> llll
	
         call vto4l(v,h,azz,zzz)	
	     
c-----------------------------
c
      endif  !withkk


c for ZZ->2l 2v precalculate  leptonic tensors
      elseif (bos.eq.21) then
c reset lzs to .true. to force recalculation of virtual contributions in qqzzqq
         do i = 1,4
            lzs(i) = .true.
         enddo
c lepton spinors and Z polarization vectors ! 
c	lepton helicities not fixed 
c 		-> sum over all possible helicites in |M|**2
c
c select helicity: h ... random number for lepton helicity
c			 combination (h=1:4), but only h=1 or 3 
c	   	         contribute for ZZ -> 2l 2v (no righthanded neutrinos)

        h = mod(h+2,4)

	ie = sign(1,2-h)

c...Les Houches interface
        if (lha.and..not.doNLO) then
           helicity(1)= ie
           helicity(2)=-ie
           helicity(3)=-1
           helicity(4)= 1
        endif


		
        CALL IXXXXX(v(0,1),ZERO ,+ie,-1,lep)	 !e+   
        CALL OXXXXX(v(0,2),ZERO ,-ie, 1,lem)	 !e- 
        CALL OXXXXX(v(0,3),ZERO ,-1,+1,lum)	!vm	  
        CALL IXXXXX(v(0,4),ZERO ,+1,-1,lup)	!vm~	

        CALL JIOXXX(lep,lem,GZL ,ZMASS,ZWIDTH,ze)      !Ze
        CALL JIOXXX(lep,lem,GAL ,ZERO ,ZERO  ,ae)      !Ae
        CALL JIOXXX(lup,lum,GZN ,ZMASS,ZWIDTH,zu)      !Zv
	
	do i = 1,4
	   au(i) = 0d0
	   zu(i) = -zu(i)
	enddo   			
	do i = 5,6 !add momentum info to A->vv current (au vanishes anyway) 
	   au(i) = zu(i)
	enddo    	
	
        do mu = 0,3
            qe(mu) = v(mu,1)+v(mu,2)
            qu(mu) = v(mu,3)+v(mu,4)
            qvv(mu) = qe(mu) + qu(mu)
        enddo
	 	 
         qe(4) = qe(0)**2-qe(1)**2-qe(2)**2-qe(3)**2
         qu(4) = qu(0)**2-qu(1)**2-qu(2)**2-qu(3)**2

         if (abs(nlo).eq.5 .or. nlo.eq.-4) then     ! determine V/V polarization vectors 
c zet(mu) = ze(mu) - xxez*qe(mu) and zut(mu) = zu(mu) - xxmz*qu(mu)
            qq = qe(0)*qvv(0)-qe(1)*qvv(1)-qe(2)*qvv(2)-qe(3)*qvv(3)
            xxez = dotrc(qvv,ze)/qq
            xxea = dotrc(qvv,ae)/qq
            
	    qq = qu(0)*qvv(0)-qu(1)*qvv(1)-qu(2)*qvv(2)-qu(3)*qvv(3)
            xxuz = dotrc(qvv,zu)/qq            
            xxua = dotrc(qvv,au)/qq
            do mu = 0,3
               zet(mu+1) = ze(mu+1) - xxez*qe(mu)
               zut(mu+1) = zu(mu+1) - xxuz*qu(mu)
               
	       aet(mu+1) = ae(mu+1) - xxea*qe(mu)
               aut(mu+1) = au(mu+1) - xxua*qu(mu)
           enddo
           do mu = 5,6
               zet(mu) = ze(mu)
               zut(mu) = zu(mu)

               aet(mu) = ae(mu)
               aut(mu) = au(mu)
           enddo
         endif

c leptonic tensors
         j = 1
       
      if (with_kk) then ! Kaluza-Klein scenario
#ifdef WITH_KK
C for AA,AZ,ZA,ZZ -> e+ e- vm vm~ :
         call vvtozzn_kk(q12(0,j),q34(0,j),v,h,aazz(0,0,j),azzz(0,0,j),
     # 					    zazz(0,0,j),zzzz(0,0,j))
c
C for W+W-
         call wwtozzn_kk(q12(0,j),q34(0,j),v,h,wwzz5(0,0,j))  ! q12 = W+
         call wwtozzn_kk(q34(0,j),q12(0,j),v,h,wwzz6(0,0,j))  ! q12 = W-
	 
C for V1V2 --> e+ e- or  V1V2 -->  vm vm~ (Vi=A,Z)
c NCl tensors for NC process (k=1...4)

         call VVtoll_kk(2,1,h,q34(0,j),v,aaee(0,0,1,j),azee(0,0,1,j),
     #	 	 zaee(0,0,1,j),zzee(0,0,1,j))    	 !emit V on upper        
	 call VVtoll_kk(1,1,h,q12(0,j),v,aaee(0,0,2,j),azee(0,0,2,j),
     #		 zaee(0,0,2,j),zzee(0,0,2,j))    	 !emit V on lower
         call VVtolln_kk(2,q34(0,j),v,aauu(0,0,1,j),azuu(0,0,1,j),
     #		 zauu(0,0,1,j),zzuu(0,0,1,j))    	 !emit V on upper        
	 call VVtolln_kk(1,q12(0,j),v,aauu(0,0,2,j),azuu(0,0,2,j),
     #		zauu(0,0,2,j),zzuu(0,0,2,j))     	!emit V on lower


C for W+W- --> e+ e- or  W+W- -->  vm vm~
c  CCl tensors for CC (k=5)

         call WWtoll_kk(2,1,h,q34(0,j),v,CCee(0,0,1,j))     !emit V on upper
         call WWtoll_kk(1,1,h,q12(0,j),v,CCee(0,0,2,j))     !emit V on lower
         call WWtolln_kk(2,q34(0,j),v,CCuu(0,0,1,j))     !emit V on upper
         call WWtolln_kk(1,q12(0,j),v,CCuu(0,0,2,j))     !emit V on lower
	 
C for W-W+ --> e+ e- or vm vm~
c CCl for CC (k=6)
	
         call WWtoll_kk(1,1,h,q34(0,j),v,CCee6(0,0,1,j))     !emit V on upper
         call WWtoll_kk(2,1,h,q12(0,j),v,CCee6(0,0,2,j))     !emit V on lower
         call WWtolln_kk(1,q34(0,j),v,CCuu6(0,0,1,j))     !emit V on upper
         call WWtolln_kk(2,q12(0,j),v,CCuu6(0,0,2,j))     !emit V on lower
	 	 
c for V-> llll
	
	call vto4ln_kk(v,h,azz,zzz)	
#endif		
c --------------------------------------------
      else !SM
C for AA,AZ,ZA,ZZ -> e+ e- vm vm~ :
         call vvtozzn(q12(0,j),q34(0,j),v,h,aazz(0,0,j),azzz(0,0,j),
     # 					    zazz(0,0,j),zzzz(0,0,j))
c
C for W+W-
         call wwtozzn(q12(0,j),q34(0,j),v,h,wwzz5(0,0,j))  ! q12 = W+
         call wwtozzn(q34(0,j),q12(0,j),v,h,wwzz6(0,0,j))  ! q12 = W-
	 
C for V1V2 --> e+ e- or  V1V2 -->  vm vm~ (Vi=A,Z)
c NCl tensors for NC process (k=1...4)

         call VVtoll(2,1,h,q34(0,j),v,aaee(0,0,1,j),azee(0,0,1,j),
     #	 	 zaee(0,0,1,j),zzee(0,0,1,j))    	 !emit V on upper        
	 call VVtoll(1,1,h,q12(0,j),v,aaee(0,0,2,j),azee(0,0,2,j),
     #		 zaee(0,0,2,j),zzee(0,0,2,j))    	 !emit V on lower
         call VVtolln(2,q34(0,j),v,aauu(0,0,1,j),azuu(0,0,1,j),
     #		 zauu(0,0,1,j),zzuu(0,0,1,j))    	 !emit V on upper        
	 call VVtolln(1,q12(0,j),v,aauu(0,0,2,j),azuu(0,0,2,j),
     #		zauu(0,0,2,j),zzuu(0,0,2,j))     	!emit V on lower


C for W+W- --> e+ e- or  W+W- -->  vm vm~
c  CCl tensors for CC (k=5)

         call WWtoll(2,1,h,q34(0,j),v,CCee(0,0,1,j))     !emit V on upper
         call WWtoll(1,1,h,q12(0,j),v,CCee(0,0,2,j))     !emit V on lower
         call WWtolln(2,q34(0,j),v,CCuu(0,0,1,j))     !emit V on upper
         call WWtolln(1,q12(0,j),v,CCuu(0,0,2,j))     !emit V on lower
	 
C for W-W+ --> e+ e- or vm vm~
c CCl for CC (k=6)
	
         call WWtoll(1,1,h,q34(0,j),v,CCee6(0,0,1,j))     !emit V on upper
         call WWtoll(2,1,h,q12(0,j),v,CCee6(0,0,2,j))     !emit V on lower
         call WWtolln(1,q34(0,j),v,CCuu6(0,0,1,j))     !emit V on upper
         call WWtolln(2,q12(0,j),v,CCuu6(0,0,2,j))     !emit V on lower
	 	 
c for V-> llll
	
	call vto4ln(v,h,azz,zzz)	
		
c --------------------------------------------
        endif

#ifdef WITH_NLO 
      elseif (nlo.eq.1 .and. bos.eq.2) then
         call BCD_fill(p(0,1),p(0,2),                       !beam
     1                 p(0,3),p(0,4),                       !jet
     2                 v(0,1),v(0,2))                       !decay
#endif
      endif
c
c
c ------------------------------------------------------------------------
c
c
c if desired, print output information
c
      
       if ( ldebug ) then

 10      format( " p(", i1, ") = ", 4(f10.3, 2x) )
 20      format( " v(", i1, ") = ", 4(f10.3, 2x) )
c
         write(6,*) " "
         print*," vector boson decay products momenta "
         do i = 1, n_v
            write(6,20) i, v(0,i), v(1,i), v(2,i), v(3,i)
         end do
         print*," parton momenta "
         do i = 1, n_p
            write(6,10) i, p(0,i), p(1,i), p(2,i), p(3,i)
         end do
c
         write(6,*) " "
         write(6,*) "xi(1), xi(2) =", xi
         write(6,*) "mu_f =",sqrt(mufsq(1,1)),sqrt(mufsq(2,1))
       end if
    
c
c call PDF subroutine in order to determine parton
c distributions in the incoming (anti)protons.
c
      q_sf = sqrt(mufsq(1,1))
      call pdfproton( xi(1), q_sf, pdf(-6,1) )        ! f_a(x1)

      q_sf = sqrt(mufsq(2,1))
      call pdfproton( xi(2), q_sf, pdf(-6,2) )        ! f_a(x2)

c select helicity
      if (bos.eq.2) then
         jsig = min(8*rn(1)+1d0,8.01d0)
c         jsig = 0
      else
         jsig = 0
      endif

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     Now sum over the subprocesses contributing to Z production

      nmax = FL_ZH(fsign,-1)            !reset counter for subprocesses to 0

C*******************  q1 q3 ---> q2 q4  V V  **********************
         
c   physToDiag(ext.momentum label) = Feynman diagram label

      physToDiag(1)=1    !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=3
      physToDiag(3)=2    !physToDiag(3/4) are labels of outgoing quarks.
      physToDiag(4)=4
C NOTE: for call of wbf_zh it is important that p(*,1,*) and p(*,3,*)
c correspond to 1-2 fermion line ALWAYS, i.e physToDiag(1/2)={1,3} and 
c similarly physToDiag(3/4)={2,4} for the 3-4 fermion line
      fsign(1) = 1
      fsign(2) = 1
      fsign(3) = 1
      fsign(4) = 1

      call wbf_zh(xi,p,v,physToDiag,fsign,bos,bosdec, nlo,
     1            pdf,res,nmin,nmax)
     

c      if (ldebug) call resprint2(nmin,nmax,res)
      if (init.eq.1) nproc(1) = nmax

C*******************  q1 qb4 ---> q2 qb3 V V  **********************

      physToDiag(1)=1    !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=4
      physToDiag(3)=2    !physToDiag(3/4) are labels of outgoing quarks.
      physToDiag(4)=3

      fsign(3) = -1
      fsign(4) = -1

      call wbf_zh(xi,p,v,physToDiag,fsign,bos,bosdec, nlo,
     1         pdf,res,nmin,nmax)

c      if (ldebug) call resprint2(nmin,nmax,res)
      if (init.eq.1) nproc(2) = nmax

C*******************  qbar2 q3 ---> qbar1 q4 V V   **********************

      physToDiag(1)=2    !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=3
      physToDiag(3)=1    !physToDiag(3/4) are labels of outgoing quarks.
      physToDiag(4)=4

      fsign(1) = -1
      fsign(2) = -1
      fsign(3) = 1
      fsign(4) = 1
      
      call wbf_zh(xi,p,v,physToDiag,fsign,bos,bosdec, nlo,
     1         pdf,res,nmin,nmax)

c      if (ldebug) call resprint2(nmin,nmax,res)
      if (init.eq.1) nproc(3) = nmax

C*******************  qbar2 qb4 ---> qbar1 qb3 V V  ******************

      physToDiag(1)=2    !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=4
      physToDiag(3)=1    !physToDiag(3/4) are labels of outgoing quarks.
      physToDiag(4)=3

      fsign(1) = -1
      fsign(2) = -1
      fsign(3) = -1 
      fsign(4) = -1
         
      call wbf_zh(xi,p,v,physToDiag,fsign,bos,bosdec, nlo,
     1         pdf,res,nmin,nmax)

c      if (ldebug) call resprint2(nmin,nmax,res)
      if (init.eq.1) nproc(4) = nmax
 
c*****************  end of process evaluation  *********************
      if (init.eq.1) then
         init = init+1
         if (lwarn) print 199," proc #s for Z/Hjj are ",nproc
 199     format(a,4i5)
      endif

      m2s = 0
      DO IRES = 1,NMAX
         m2s = m2s + RES(IRES)
      ENDDO


c...Les Houches interface - the most propable subprocess   
      if (lha.and..not.doNLO) then
         i=0
         weight=0.d0
         rnumb=RandomNumber()
         do while((i.le.nmax).and.(weight.le.rnumb*m2s))
            i=i+1
            weight=weight+res(i)
            iprocess=i
         enddo
      endif


c      DO IRES = nproc(1)+1,nproc(3)
c         m2s = m2s + RES(IRES)
c      ENDDO
    
      if (bos.eq.2) then
         if (jsig.eq.0) then
            m2s_wbfvv = m2s     !*3  ! factor 3 for electrons and muons and tau
         else
            m2s_wbfvv = m2s*8d0   ! factor 8 for random helicity summation

            tauchi(1)=jsig5
            tauchi(2)=jsig5
         endif
      elseif (bos.eq.6) then   
         if (rn(1).gt.0.5d0) then
            tauchi(1)= 1
         else
            tauchi(1)=-1
         endif
         tauchi(2)= -1*tauchi(1)

         m2s_wbfvv = m2s

      elseif (bos.eq.22) then ! qq->ZZqq -> 4l
         m2s_wbfvv = m2s*4d0   ! factor 4 for random helicity summation
      elseif (bos.eq.21) then ! qq->ZZqq -> 2l 2v
         m2s_wbfvv = m2s*2d0   ! factor 2 for random helicity summation
      else
         m2s_wbfvv = m2s
      endif

      if (lha.and..not.doNLO) then
         if (bos.eq.2) then
            if (jsig.ne.0) then                  
               helicity(2)= jsig5 !particle is at the 2nd place
               helicity(1)=-jsig5 
            endif
         endif
      endif

      RETURN
      END

c******************************************************************
c
c   begin subroutine wbf_zh
c
c*****************************************************************
      subroutine wbf_zh(xi,p,v,physToDiag,fsign,bos, bosdec, nlo,
     1                  pdf,res,nmin,nmax)
      implicit none
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
      real*8 p(0:3,max_p), v(0:3,max_v), xi(nx)
      real*8 pdf(-6:6,2), res(maxnumsubproc)
      integer physToDiag(4), fsign(4+max_v), bos, h,nlo, nmin, nmax
c      
      common /hcount / h
c
c wbf_zh calls the amplitude square routines 
c             qqzqq    for qq-->qqZ    for bos=2      and
c             qqhqq    for qq-->qqH    for bos=6
c             qqwwqq   for qq-->qqWW    for bos=34.or.43
c             qqzzqq   for qq-->qqZZ    for bos=22 or 21
c
c  INPUT:  p(0:3,4)        external physical parton momenta
c          v(0:3,nv)       Z decay momenta, for Higgs production only
c                          the sum q(mu) = v(mu,1)+...+v(mu,nv) is needed
c          physToDiag(4)   physToDiag(ext.mom. label) = Feynman diagram label
c          fsign           sign factors for amplitude calls; see qqZqq
c          nlo             nlo = 0: calculate LO only
c                          nlo = 1: calculate full NLO (Born + interference)
c	   h		   specify lepton helicity combination
c  OUTPUT:
c          uucc(ID)   output in format as in qqZqq(j), but corresponding
c          etc.            to res(*) in m2s_qqZqq
c     or   res(k)   
c
c  In and Output
c    nmin, nmax            range of process numbers for this call
c
      real*8 N                              ! color factors
      parameter(N=3d0)

c Note: Factor 9 from color sum included in qqbqq
c
c alfas, scales etc
#include "VBFNLO/utilities/scales.inc"

c local variables
      real*8 uucc, uuss, ddcc, ddss, udsc, ducs
      real*8 pbar(0:3,4+max_v), dotrr, polcol
      double precision NCmatrixelt(0:1,0:1)
      double precision CCmatrixelt(0:1)
      integer iflav(4), diagToPhys(4), FL_ZH
      external dotrr, FL_ZH

      logical ChargedCurrent, sametype, oneAntiparticle
      logical ldebug
      integer bosdec
c      parameter (ldebug=.false., bosdec=3)!3 for H->WW
c      parameter (ldebug=.false., bosdec=2) !2 for H->ZZ
      integer i,if1,if2,icc1,j,k,mu

     

      uucc = 0d0
      uuss = 0d0
      ddcc = 0d0
      ddss = 0d0
      udsc = 0d0
      ducs = 0d0

      nmin = nmax+1
      do i = 1,4
         diagToPhys(physToDiag(i)) = i
      enddo
      polcol = 1d0/(4d0*N**2*xi(1)*xi(2))
         
c get the real emission amplitude squared, store it in uucs(-1,1) etc.
      do mu = 0,3
         do i = 1,4
            pbar(mu,physToDiag(i))=p(mu,i)
         enddo
      enddo

      if (bos.eq.2) then
         do mu = 0,3             ! kinematics for Z-->l+l- decay
            pbar(mu,5) = v(mu,2) ! ebar
            pbar(mu,6) = v(mu,1) ! e
         enddo
         fsign(5) = -1
         fsign(6) = 1
         call qqzqq(pbar,fsign, nlo,
     1              uucc,uuss,ddcc,ddss,udsc,ducs)         
      elseif (bos.eq.6) then
         if (bosdec.eq.3) then
            do mu = 0,3         ! kinematics for H-->WW -->4 lepton decay
               pbar(mu,5) = v(mu,2) ! l+
               pbar(mu,6) = v(mu,1) ! nu
               pbar(mu,7) = v(mu,3) ! nubar
               pbar(mu,8) = v(mu,4) ! l-
            enddo
            fsign(5) = -1
            fsign(6) = 1
            fsign(7) = -1
            fsign(8) = 1
         else
            do mu = 0,3
               pbar(mu,5) = 0       ! dummy momentum
               pbar(mu,6) = 0
               do i = 1,n_v
                  pbar(mu,6) = pbar(mu,6) + v(mu,i)   ! Higgs momentum
               enddo
            enddo
            fsign(5) = 0
            fsign(6) = 1
         endif                  
         call qqhqq(pbar,fsign, nlo, bosdec,
     1              uucc,uuss,ddcc,ddss,udsc,ducs)
      elseif (bos.eq.34 .or. bos.eq.43) then         ! W+W- to 4 leptons
         do mu = 0,3             ! kinematics for H-->WW -->4 lepton decay
            pbar(mu,5) = v(mu,2) ! l+
            pbar(mu,6) = v(mu,1) ! nu
            pbar(mu,7) = v(mu,3) ! nubar
            pbar(mu,8) = v(mu,4) ! l-
         enddo
         fsign(5) = -1
         fsign(6) = 1
         fsign(7) = -1
         fsign(8) = 1
c         call qqhqq(pbar,fsign, nlo, bosdec,
c     1              uucc,uuss,ddcc,ddss,udsc,ducs)
         if (lwarn) then
            call qqwwqq_comp(pbar,fsign, nlo, bosdec,
     1                       uucc,uuss,ddcc,ddss,udsc,ducs)
         endif
         call qqwwqq(pbar,fsign, nlo, 1,
     1                    uucc,uuss,ddcc,ddss,udsc,ducs)
c         call qqwwqq_comp(pbar,fsign, nlo, bosdec,
c     1                    uucc,uuss,ddcc,ddss,udsc,ducs)
      elseif (bos.eq.22) then         ! ZZ to 4 leptons
         do mu = 0,3             ! kinematics for H-->ZZ -->4 lepton decay
            pbar(mu,5) = v(mu,1) ! l+
            pbar(mu,6) = v(mu,2) ! l-
            pbar(mu,7) = v(mu,3) ! l"+
            pbar(mu,8) = v(mu,4) ! l"-
         enddo
         fsign(5) = -1
         fsign(6) = 1
         fsign(7) = -1
         fsign(8) = 1
         if (lwarn) then
c            call qqzzqq_comp(pbar,fsign, nlo,  bos,
c     1                       uucc,uuss,ddcc,ddss,udsc,ducs)
         endif
         call qqzzqq(pbar,fsign, nlo, 1,bos,
     1                    uucc,uuss,ddcc,ddss,udsc,ducs)

c        call qqzzqq_comp(pbar,fsign, nlo, bos,
c     1                    uucc,uuss,ddcc,ddss,udsc,ducs)
      elseif (bos.eq.21) then         ! ZZ to 2 leptons + 2 neutrinos
         do mu = 0,3             ! kinematics for H-->ZZ-->2l 2v decay
            pbar(mu,5) = v(mu,1) ! l+
            pbar(mu,6) = v(mu,2) ! l-
            pbar(mu,7) = v(mu,3) ! v
            pbar(mu,8) = v(mu,4) ! v~
         enddo
         fsign(5) = -1
         fsign(6) =  1
         fsign(7) =  1
         fsign(8) = -1
         if (lwarn) then
c            call qqzzqq_comp(pbar,fsign, nlo,  bos,
c     1                       uucc,uuss,ddcc,ddss,udsc,ducs)
         endif
        
	call qqzzqq(pbar,fsign, nlo, 1,bos,
     1                    uucc,uuss,ddcc,ddss,udsc,ducs) 

c        call qqzzqq_comp(pbar,fsign, nlo, bos,
c     1                    uucc,uuss,ddcc,ddss,udsc,ducs)
      else
         print*," Invalid entry BOS = ",bos
         print*," in call of wbf_zh. Must be 2,6, 34/43, 22 or 21"
         stop
      endif
      NCmatrixelt(0,0)=uucc
      NCmatrixelt(0,1)=uuss
      NCmatrixelt(1,0)=ddcc
      NCmatrixelt(1,1)=ddss
      CCmatrixelt(0)=udsc
      CCmatrixelt(1)=ducs

      do if1=1,nfl
         do if2=1,nfl
            iflav(1)=if1*fsign(physToDiag(1))
            iflav(3)=if1*fsign(physToDiag(3))
            iflav(2)=if2*fsign(physToDiag(2))
            iflav(4)=if2*fsign(physToDiag(4))
            k=FL_ZH(iflav,1)
            if (ldebug) 
     &        print*,k,":",iflav(1),iflav(2),"-->",iflav(3),iflav(4)
            res(k)=pdf(sign1*iflav(1),1)
     &            *pdf(sign2*iflav(2),2)
     &            *NCmatrixelt(mod(if1,2),mod(if2,2))*polcol
C Now check if there is a CC contribution for this choice of initial state
C flavors (i,j). First: check whether initial uu or dd, not ud
            sametype=(mod(if1,2)).eq.(mod(if2,2))
            oneAntiparticle=
     &        ((fsign(physToDiag(1))*fsign(physToDiag(2))).eq.-1) 
c true if only one particle is an antiparticle
            ChargedCurrent=(oneAntiparticle.and.sametype) .or.
     &              ( (.not.oneAntiparticle).and.(.not.sametype) )
            if ( ChargedCurrent .and. if1.le.4 .and. if2.le.4 ) then   
c change 1<-->2 and 3<-->4 in outgoing quark flavors
               iflav(3)=(if1+2*mod(if1,2)-1)*fsign(physToDiag(3))
               iflav(4)=(if2+2*mod(if2,2)-1)*fsign(physToDiag(4))
               icc1 = abs(iflav(diagtophys(1)))
               k=FL_ZH(iflav,1)
               if (ldebug) 
     &           print*,k,":",iflav(1),iflav(2),"-->",iflav(3),iflav(4)
               res(k)=pdf(sign1*iflav(1),1)
     &               *pdf(sign2*iflav(2),2)
     &               *CCmatrixelt(mod(icc1,2))*polcol
            endif                      ! endif(CC)
         enddo
      enddo 

      nmax = FL_ZH(iflav,-2)

      end
c******************************************************************
c
c   end subroutine wbf_zh
c
c*****************************************************************



c******************************************************************
c
c   begin function FL_ZH
c
c*****************************************************************
      integer function FL_ZH(iflav,colstruc)
      implicit none
      integer iflav(4),colstruc    ! input for color and flavor assignment
c  fill hepup color and flavor assignments for WBF processes with
c  two quark lines only

c  id1,2 are flavor identifiers for incoming quarks
c  id3,4 are flavor identifiers for outgoing quarks
c  colstruc = is irrelevant here
c
c  color structure of these interactions is trivial
c  and colstruc was included only to make this program 
c  more consistent with the others.

c  Note that colstruc is also a flag:
c  if colstruc=-1, then we are resetting.

c  The color structure is always the same:
c  color always flows from id1->id3 and id2->id4
c  no gluons are involved.

#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
      
      integer lkup
      common /localkup/ lkup(numParticles,7,maxNumSubProc)
 
      integer listposition
      integer numresets
      save listposition
      save numresets

      data numresets /-1/
      data listposition /0/

      if (colstruc.eq.-1) then  ! we are resetting
c       write(*,*) "we are resetting"
        listposition=0
        numresets=numresets+1
      else if (colstruc.eq.-2) then
c       don"t do anything, just return the number of subprocesses.
      else
c increment the counter regardless of whether or not it"s the 1st time through.
        listposition=listposition+1

c       we fill info for this subprocess,if required
        if(numresets.eq.0) then
          call fillColoredPartons_ZH(iflav,
     1                               listposition+numdecay)
        endif       ! numresets.eq.0
      endif         
      FL_ZH=listposition
      end
c******************************************************************************
c
c   end function FL_ZH
c
c******************************************************************************


c*****************************************************************************
c
c    begin  subroutine fillColoredPartons_ZH
c
c*****************************************************************************
      subroutine fillColoredPartons_ZH(iflav,listposition)
c  assigns values to the variables in the common block localHEPUP
c  in particular, this subroutine assigns values to those variables that 
c  will be stored in the lookup tables generated by writeHEPUPtable.  
c  As the name suggests, this routine only stores the information for the 
c  colored partons.  Particles without color will be dealt with in the 
c  subroutine fillColorless.

      implicit none
c  include files
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
#include "VBFNLO/utilities/process.inc"

c  input parameters 
      integer iflav(4),id1,id2,id3,id4,listposition   

c  local variables
      integer i

c 4 quark flavors and subprocess number      
      id1=iflav(1)
      id2=iflav(2)
      id3=iflav(3)
      id4=iflav(4)

   
      select case(process)
      case(Hjj_WW, Hjj_ZZ_ll, Hjj_ZZ_lnu, WPWMjj, ZZjj_ll, ZZjj_lnu) 
         lnup(listposition)=numParticles+2
      case(Hjj_AA, Hjj_mu, Hjj_tau, Hjj_bbar, Zjj_l, Zjj_nu, WPjj, WMjj) 
         lnup(listposition)=numParticles-1
      case(Hjj)             
         lnup(listposition)=numParticles-3
      end select


      listup(1,listposition)=-1   !incoming quarks
      listup(2,listposition)=-1
      listup(3,listposition)=1    !outgoing quarks
      listup(4,listposition)=1
 
      lidup(1,listposition)=id1     ! the 4 quarks
      lidup(2,listposition)=id2
      lidup(3,listposition)=id3
      lidup(4,listposition)=id4

      do i=3,n_p
        lmothup(1,i,listposition)=1
        lmothup(2,i,listposition)=2
      enddo

      if (id1.gt.0) then
        licolup(1,1,listposition)=501
        licolup(1,3,listposition)=501
        licolup(2,1,listposition)=0
        licolup(2,3,listposition)=0
      else
        licolup(1,1,listposition)=0
        licolup(1,3,listposition)=0
        licolup(2,1,listposition)=501
        licolup(2,3,listposition)=501
      endif

      if (id2.gt.0) then
        licolup(1,2,listposition)=502
        licolup(1,4,listposition)=502
        licolup(2,2,listposition)=0
        licolup(2,4,listposition)=0
      else
        licolup(1,2,listposition)=0
        licolup(1,4,listposition)=0
        licolup(2,2,listposition)=502
        licolup(2,4,listposition)=502
      endif

      end  ! fillColoredPartons_ZH
c*****************************************************************************
c
c    end subroutine fillColoredPartons_ZH
c
c*****************************************************************************

      subroutine resprint2(nmin,nmax,res)
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
      integer nmin,nmax,k,ic
      save ic
      real*8 resold
      common /bresprint/ resold(16)
      real*8 xa,xb,res(maxnumsubproc),fac
      if (nmin.eq.1) ic=1
      xa = 0
      xb = 0
      if (nmin.le.190) then 
         fac = 4*9
      else
         fac = 4d0*24d0
      endif
      do k = nmin,nmax
         xa = xa + res(k)
      enddo
      print*," res([",nmin,",",nmax,"])",xa,
     1         xa/(resold(ic)+resold(ic+1))*fac
      ic = ic+2
      end

c --------------------------------------------------------------------------

      subroutine vtoww_reset
      implicit none
#include "tensor.inc"
      complex*16 zero
      parameter (zero=(0d0,0d0))
      integer j,jj,mu,nu
      do mu = 0,3
         aww(mu) = zero
         zww(mu) = zero
         do nu = 0,3
            do j = 1,3
               aaww(mu,nu,j) = zero
               azww(mu,nu,j) = zero
               zaww(mu,nu,j) = zero
               zzww(mu,nu,j) = zero
               wwww5(mu,nu,j) = zero
               wwww6(mu,nu,j) = zero
               do jj = 1,2
                  NCwpa(mu,nu,jj,j) = zero 
                  NCwpz(mu,nu,jj,j) = zero
                  CCwpa(mu,nu,jj,j) = zero 
                  CCwpz(mu,nu,jj,j) = zero
                  NCwma(mu,nu,jj,j) = zero 
                  NCwmz(mu,nu,jj,j) = zero
                  CCwma(mu,nu,jj,j) = zero 
                  CCwmz(mu,nu,jj,j) = zero 
               enddo
            enddo
         enddo
      enddo
      print*," v(v) to WW tensors initialized to ",zero 
      return
      end

c --------------------------------------------------------------------------

      subroutine vtozz_reset
      implicit none
#include "tensorz.inc"
      complex*16 zero
      parameter (zero=(0d0,0d0))
      integer j,jj,mu,nu
      do mu = 0,3
         azz(mu) = zero
         zzz(mu) = zero
         do nu = 0,3
            do j = 1,3

               wwzz5(mu,nu,j) = zero
               wwzz6(mu,nu,j) = zero
	       aazz(mu,nu,j) = zero
	       azzz(mu,nu,j) = zero
	       zazz(mu,nu,j) = zero
	       zzzz(mu,nu,j) = zero
	       
	       do jj = 1,2
	       
                  aaee(mu,nu,jj,j) = zero
                  azee(mu,nu,jj,j) = zero
                  zaee(mu,nu,jj,j) = zero
                  zzee(mu,nu,jj,j) = zero
                  aauu(mu,nu,jj,j) = zero
                  azuu(mu,nu,jj,j) = zero
                  zauu(mu,nu,jj,j) = zero
                  zzuu(mu,nu,jj,j) = zero
                  CCee(mu,nu,jj,j) = zero
                  CCuu(mu,nu,jj,j) = zero 
                  CCee6(mu,nu,jj,j) = zero 
                  CCuu6(mu,nu,jj,j) = zero
	       
	       enddo            !jj

            enddo               !j
         enddo                  !nu
      enddo                     !mu
      
      print*," v(v) to ZZ tensors initialized to ",zero 
      return
      end


