dnl most stuff in here is shamelessly adapted from Herwig++ or ThePEG

dnl some switches I think are useful

AC_DEFUN([VBFNLO_CHECK_KK],
[
AC_MSG_CHECKING([whether KK should be included])
AC_ARG_ENABLE(kk,
        AC_HELP_STRING([--enable-kk],[--enable-kk to enable simulation of Kaluza-Klein resonances]),
        [],
        [enable_kk=no]
        )
AC_MSG_RESULT([$enable_kk])
AM_CONDITIONAL(WITH_KK,[test "x$enable_kk" = "xyes"])

if test "x$enable_kk" == "xyes" ; then
VBFNLO_CHECK_GSL
fi

])

AC_DEFUN([VBFNLO_CHECK_NLO],
[
AC_MSG_CHECKING([whether NLO should be enabled])
AC_ARG_ENABLE(NLO,
        AC_HELP_STRING([--enable-NLO],[--disable-NLO to disable next-to-leading order QCD]),
        [],
        [enable_NLO=yes]
        )
AC_MSG_RESULT([$enable_NLO])
AM_CONDITIONAL(WITH_NLO,[test "x$enable_NLO" = "xyes"])
])

dnl which processes to enable

AC_DEFUN([VBFNLO_ENABLE_PROCESSES],
[
AC_MSG_CHECKING([for processes to include])

AC_ARG_ENABLE(processes,
        AC_HELP_STRING([--enable-processes=LIST],[Comma-separated list of processes to enable. Options are (vbf ggf diboson triboson hjjj top qcdwwjj qcdvjj)]),
        [],
        [enable_processes=all]
        )
if test "x$enable_processes" = "xall"; then
   all=yes
fi
AC_MSG_RESULT([$enable_processes])

if test ! "$all"; then
   oldIFS="$IFS"
   IFS=","
   for i in $enable_processes; do
       declare $i=yes
   done
   IFS="$oldIFS"
fi

AM_CONDITIONAL(WITH_DIBOSON,[test "$diboson" -o "$all"])
AM_CONDITIONAL(WITH_GGF,[test "$ggf" -o "$all"])
AM_CONDITIONAL(WITH_QCDWW,[test "$qcdwwjj" -o "$all"])
AM_CONDITIONAL(WITH_TOP,[test "$top" -o "$all"])
AM_CONDITIONAL(WITH_TRIBOSON,[test "$triboson" -o "$all"])
AM_CONDITIONAL(WITH_QCDV,[test "$qcdvjj" -o "$all"])
AM_CONDITIONAL(WITH_VBF,[test "$vbf" -o "$all"])
AM_CONDITIONAL(WITH_HJJJ,[test "$hjjj" -o "$all"])
])

dnl ##### ROOT #####
AC_DEFUN([VBFNLO_CHECK_ROOT],[

ROOTPATH=""
ROOTLIBPATH=""
ROOTLIBS=""
ROOTINCLUDE=""

AC_MSG_CHECKING([for ROOT])


AC_ARG_WITH(root,
        AC_HELP_STRING([--with-root=DIR],[location of ROOT installation]),
        [],
        [with_root=no])

if test "x$with_root" = "xno"; then
  AC_MSG_RESULT([not required])
else

  ROOTPATH="$with_root"

  if test -f $ROOTPATH/bin/root-config; then

    AC_MSG_RESULT([$ROOTPATH])

    AC_MSG_CHECKING([for ROOTLIBS])
    if test -z "$ROOTLIBS"; then
      ROOTLIBS=`$ROOTPATH/bin/root-config --libs`
      ROOTLIBPATH=`$ROOTPATH/bin/root-config --libdir`
    fi
    AC_MSG_RESULT([$ROOTLIBS])

    AC_MSG_CHECKING([for ROOTINCLUDE])
    if test -z "$ROOTINCLUDE"; then
      ROOTINCLUDE="`$ROOTPATH/bin/root-config --cflags` -Wno-long-long"
    fi
    AC_MSG_RESULT([$ROOTINCLUDE])


    oldLIBS="$LIBS"
    oldLDFLAGS="$LDFLAGS"
    oldCPPFLAGS="$CPPFLAGS"
    LIBS="$LIBS $ROOTLIBS"
    LDFLAGS="$LDFLAGS $CLHEPLDFLAGS"
    CPPFLAGS="$CPPFLAGS $ROOTINCLUDE"

    AC_MSG_CHECKING([that ROOT works])
    AC_LANG_PUSH([C++])
    AC_LINK_IFELSE([AC_LANG_PROGRAM([[#include <TCanvas.h>]],
    	[[TCanvas c("c1", "");]])],[AC_MSG_RESULT([yes])],[AC_MSG_RESULT([no]) 
     	AC_MSG_ERROR([Use '--with-root=' to set the path to your ROOT installation.\
	If it doesn't work anyhow, you eventually have to set the ROOTSYS environment variable.])
    ])
    AC_LANG_POP([C++])

    LIBS="$oldLIBS"
    LDFLAGS="$oldLDFLAGS"
    CPPFLAGS="$oldCPPFLAGS"

    AC_SUBST(ROOTLIBS)
    AC_SUBST(ROOTLIBPATH)
    AC_SUBST(ROOTPATH)
    AC_SUBST(ROOTINCLUDE)

  else
    AC_MSG_ERROR([root-config not found...aborting])    
  fi

fi
AM_CONDITIONAL(WITH_ROOT, test ! x"$with_root" = "xno")

])

dnl ###### GSL ######
AC_DEFUN([VBFNLO_CHECK_GSL],
[
AC_MSG_CHECKING([for gsl location])
GSLINCLUDE=""
GSLLIBS=""

AC_ARG_WITH(gsl,
        AC_HELP_STRING([--with-gsl=DIR],[location of gsl installation @<:@default=system libs@:>@]),
        [],
	[with_gsl=system])

if test "x$with_gsl" = "xno"; then
AC_MSG_ERROR([libgsl is required. Please install the GNU scientific library and header files.])
fi

if test "x$with_gsl" = "xsystem"; then
	AC_MSG_RESULT([in system libraries])
	oldlibs="$LIBS"
	AC_CHECK_LIB(m,main)
	AC_CHECK_LIB(gslcblas,main)
	AC_CHECK_LIB(gsl,main,[],
			[
			AC_MSG_ERROR([Cannot find libgsl. Please install the GNU scientific library and header files or use --with-gsl=.])
			]
		     )
	GSLLIBS="$LIBS"
	LIBS=$oldlibs
else
	if test "`uname -m`" = "x86_64" -a -e "$with_gsl/lib64/libgsl.a" -a -d "$with_gsl/include/gsl"; then
		AC_MSG_RESULT([found in $with_gsl])
		GSLLIBS="-L$with_gsl/lib64 -R$with_gsl/lib64 -lgslcblas -lgsl"
		GSLINCLUDE="-I$with_gsl/include"
	elif test -e "$with_gsl/lib/libgsl.a" -a -d "$with_gsl/include/gsl"; then
		AC_MSG_RESULT([found in $with_gsl])
		GSLLIBS="-L$with_gsl/lib -R$with_gsl/lib -lgslcblas -lgsl"
		GSLINCLUDE="-I$with_gsl/include"
	else
		AC_MSG_RESULT([not found])
		AC_MSG_ERROR([Can't find $with_gsl/lib/libgsl.a or the headers in $with_gsl/include])
	fi
fi

AC_SUBST(GSLINCLUDE)
AC_SUBST(GSLLIBS)
])

dnl #### LHAPDF ######
AC_DEFUN([VBFNLO_CHECK_LHAPDF],
[
AC_MSG_CHECKING([for LHAPDF])
HAS_LHAPDF="yes"
LHAPDF_DIR=""
LHAPDF_LIBDIR=""
AC_ARG_WITH(LHAPDF,[  --with-LHAPDF=/path/to/LHAPDF or --without-LHAPDF to use internal PDF sets], [if test -n "$with_LHAPDF" -a "x$with_LHAPDF" != "xyes" -a "x$with_LHAPDF" != "xno"; then LHAPDF_DIR="$with_LHAPDF"; elif test "x$with_LHAPDF" = "xno"; then HAS_LHAPDF="no"; fi])

if test -n "$LHAPDF_DIR"; then
	if test "`uname -m`" = "x86_64" -a -d "$LHAPDF_DIR/lib64" -a -e "$LHAPDF_DIR/lib64/libLHAPDF.la" ; then
		LHAPDF_LIBDIR=$LHAPDF_DIR/lib64
	else
		LHAPDF_LIBDIR=$LHAPDF_DIR/lib
	fi
fi

LHAPDF_LDFLAGS=""
if test -n "$LHAPDF_LIBDIR"; then
  if test -e $LHAPDF_LIBDIR/libLHAPDF.so -o -e $LHAPDF_LIBDIR/libLHAPDF.dylib
  then
    LHAPDF_LDFLAGS="-L$LHAPDF_LIBDIR"
  else
    HAS_LHAPDF="no"
  fi
fi

LHAPDF_LIBS="-lLHAPDF"

oldLIB="$LIBS"
oldLDFLAGS="$LDFLAGS"

if test "$HAS_LHAPDF" == "yes"; then
dnl Now lets see if the libraries work properly
  LIBS="$LIBS $LHAPDF_LIBS"
  LDFLAGS="$LDFLAGS $LHAPDF_LDFLAGS"
  AC_LANG_PUSH([C++])
  AC_LINK_IFELSE([AC_LANG_PROGRAM([[extern "C" { void initpdf_(int&); }]],
                                  [[int i = 1; initpdf_(i);]])], ,
                                  HAS_LHAPDF="no")
  AC_LANG_POP([C++])
fi

LIBS="$oldLIB"
LDFLAGS="$oldLDFLAGS"

if test "$HAS_LHAPDF" == "yes"; then
  AC_MSG_RESULT([yes])
elif test "x$with_LHAPDF" == "xno" -o "x$with_LHAPDF" == "x"; then
  AC_MSG_RESULT([not required])
else
  AC_MSG_ERROR([LHAPDF was requested but the library was not found.])
fi
AC_SUBST(LHAPDF_DIR)
AC_SUBST(LHAPDF_LIBS)
AC_SUBST(LHAPDF_LDFLAGS)
AM_CONDITIONAL([WITH_LHAPDF], [test "x$HAS_LHAPDF" == "xyes"])
])

dnl #### check, which fortran compiler we are using #####
AC_DEFUN([VBFNLO_CHECK_FC],
[
  AC_MSG_CHECKING([which Fortran compiler we are using])
  VBFFC="unrecognized, assuming g77 behaviour"
  AM_FCFLAGS="$AM_FCFLAGS -fno-automatic -ffixed-line-length-none"

  check_g77=`echo $FC | grep g77`
  check_gfortran=`echo $FC | grep gfortran`
  check_ifort=`echo $FC | grep ifort`

  if test -n "$check_g77" ; then
     VBFFC="g77"
  elif test -n "$check_gfortran" ; then
     VBFFC="gfortran"
  elif test -n "$check_ifort" ; then
     VBFFC="ifort"
     AM_FCFLAGS="$AM_FCFLAGS -extend_source"
  fi

  AC_MSG_RESULT([$VBFFC])

])

dnl check for MadGraph comparison mode and set flags
AC_DEFUN([VBFNLO_CHECK_MADGRAPH],
[
AC_MSG_CHECKING([whether MadGraph comparisons should be included])
AC_ARG_ENABLE(madgraph,
        AC_HELP_STRING([--enable-madgraph],[--enable-madgraph to include code for MadGraph comparisons]),
        [],
        [enable_madgraph=no]
        )
AC_MSG_RESULT([$enable_madgraph])
AM_CONDITIONAL(WITH_MADGRAPH,[test "x$enable_madgraph" = "xyes"])
])


dnl check for debugging mode and set compiler flags

AC_DEFUN([VBFNLO_SET_FLAGS],
[
AC_MSG_CHECKING([for debugging mode])
AC_ARG_ENABLE(debug,
        AC_HELP_STRING([--enable-debug],[--enable-debug to compile with debug symbols]),
        [],
        [enable_debug=no]
        )
AC_MSG_RESULT([$enable_debug])
AM_CONDITIONAL(VBFNLO_DEBUG,[test "x$enable_debug" = "xyes"])


if test "x$enable_debug" == "xyes" ; then
  AM_FCFLAGS="$AM_FCFLAGS -g $APPLE_GFORTRAN_FIXES -I\$(top_builddir)/include"
  AM_CPPFLAGS="$AM_CPPFLAGS -I\$(top_builddir)/include \$(GSLINCLUDE)"
  AM_CXXFLAGS="$AM_CXXFLAGS -g -fno-inline"
else
  AM_FCFLAGS="$AM_FCFLAGS $APPLE_GFORTRAN_FIXES -I\$(top_builddir)/include"
  AM_CPPFLAGS="$AM_CPPFLAGS -I\$(top_builddir)/include \$(GSLINCLUDE) -DNDEBUG" 
fi

  AC_SUBST(AM_CPPFLAGS)
  AC_SUBST(AM_CXXFLAGS)
  AC_SUBST(AM_FCFLAGS)
  AC_SUBST(AM_LDFLAGS)

  F77=$FC
  AM_FFLAGS=$AM_FCFLAGS
  AC_SUBST(F77)
  AC_SUBST(AM_FFLAGS)

])

AC_DEFUN([VBFNLO_SUMMARY],
[
if test "x$with_LHAPDF" == "x" ; then
  with_LHAPDF="no"
fi
if test "x$with_gsl" == "x" ; then
  with_gsl="not required"
fi
echo "=================================================="
echo ""
echo "  $PACKAGE_STRING configuration"
echo ""
echo "  Processes included:"
echo "  $enable_processes"
echo ""
echo "  NLO QCD           $enable_NLO"
echo "  KK resonances     $enable_kk"
echo "  Debug symbols     $enable_debug"
if test "x$enable_madgraph" = "xyes" ; then
echo "  MadGraph code     included"
fi
echo ""
echo "  LHAPDF            $with_LHAPDF"
echo "  ROOT              $with_root"
echo "  GSL               $with_gsl"
echo ""
echo "=================================================="
])

