c
c phase space generator for pp --> V1 + V2 + V3 + 1 Jet
c                                  V1 --> l1 l2, V2 --> l3 l4, V3 --> l5 l6
c
c       Vera Hankele, <vera@particle.uni-karlsruhe.de>
c       Initial version: June 2006
c       Last modified: January 2007

      SUBROUTINE DEF_PS_ZZZj(bos1,bos2,bos3)

      IMPLICIT NONE

#include "VBFNLO/utilities//global.inc"

      INTEGER bos(3), i, bos1, bos2, bos3

c set up variables in common block BLIPSI for call of LIPSVVVJ

      REAL*8 RM2, RMG, RM2MIN, RM2MAX, S, PTJMIN
      REAL*8 RM2MINX, RM2MAXX, M2MIN, RM2MINY, RM2MAXY
      REAL*8 RM2MAX1, RM2MAX2
      COMMON /TRIBOSBLIPSI/ RM2(3), RMG(3), RM2MIN(3), RM2MAX(3),
     1                RM2MINX, RM2MAXX, RM2MINY, RM2MAXY, M2MIN,
     2                S, PTJMIN, RM2MAX1, RM2MAX2

      DOUBLE PRECISION CLR, XM2, XMG, B, V, A
      COMMON /BKOPOU/ CLR(4,5,-1:1), XM2(6), XMG(6),
     1                B(6,6,6), V(4,5), A(4,5)

      s = ecm**2

      bos(1) = bos1        ! Z = 2
      bos(2) = bos2        ! Z = 2
      bos(3) = bos3        ! Z = 2

      do i = 1,3
         rm2(i) = xm2(bos(i))        ! mass squared
         rmg(i) = xmg(bos(i))        ! mass * decay width
         if (bos(i).eq.2) then
            rm2min(i) = 1.0d-3
            rm2max(i) = 5000.0**2
         else
            rm2min(i) = 1.0d-3         ! minimal/maximal value of mass squared
            rm2max(i) = s/2.        ! for tan mapping
         endif
      enddo

      rm2miny = 1.0d-3
      rm2maxy = s

      rm2minx = 1.0d-3
      rm2maxx = s/2.   ! for intermediate particle X -> V1 V2

c      rm2max1 = (sqrt(xm2(6)-30d0*xmg(6)))**2 !Dieters proposal
c      rm2max2 = (sqrt(xm2(6)+30d0*xmg(6)))**2 !Dieters proposal

      rm2max1 = ((xm2(6)-15d0*xmg(6))/sqrt(xm2(6)) )**2
      rm2max2 = ((xm2(6)+15d0*xmg(6))/sqrt(xm2(6)) )**2

c cutoff for numerical purposes

      m2min = 0.01d0**2
      ptjmin = 20.0d0

 10   format ( " pTjmin = ", 5(f6.1,2x) )
 60   format ( " machine energy = ", f10.1, "  GeV" )
 70   format ( " integration range for boson ",i3," = ",f10.3,
     1         "  to ",f12.3," GeV" )
 72   format ( " boson has mass ",f8.2," GeV and width ",
     1         f10.4," GeV" )
      write(6,*)
      write(6,*) "--------------------PHASE SPACE--------------------"
      write(6,*)
      do i = 1,3
         write(6,72) sqrt(rm2(i)),rmg(i)/sqrt(rm2(i))
         write(6,70) bos(i),sqrt(rm2min(i)),sqrt(rm2max(i))
      enddo
      write(6,*)
      write(6,10) ptjmin
      write(6,60) sqrt(s)
      write(6,*)
      write(6,*) "---------------------------------------------------"
      write(6,*)

      end

C**********************  LIPSVVVJ   ****************************************
C
      SUBROUTINE LIPSZZZJ(N,RD,RN,
     1                   K1,K2,V,Pjet,
     2                   X1,X2,W, NW)

      IMPLICIT NONE

#include "VBFNLO/utilities//global.inc"
c
      INTEGER N,NRANDL, NW
      REAL*8 TPI
      PARAMETER (nrandl=18, TPI=2D0*PI )
      REAL*8 RD(nrandl),RN
      REAL*8 K1(0:3), K2(0:3), V(0:3,6), X1, X2, W, K(0:3)
      REAL*8 Pjet(0:3)

C Phase space generator for the production of 3 bosons (+ 1 jet)
C at hadron colliders
C
C  INPUT:   RD         random number array (between 0 and 1)
C           RN         additional random number which will not be driven by
C                      VEGAS
C           NW         NW = 1 corresponds to Narrow Width Approximation,
C                      NW = 0 means Full calculation
C
C  OUTPUT:  K1,K2      4-vectors for 2 incoming partons
C           X1,X2      corresponding values of Feynman x
C           V(0:3,6)   4-vectors for the 4 decay leptons, assumed massless
C                      Q(*,1) = V(*,1) + V(*,2)
C                      Q(*,2) = V(*,3) + V(*,4)
C                      Q(*,3) = V(*,5) + V(*,6)
C           Pjet(0:3)  4-vector for the jet, assumed massless
C           W          the weight of the phase space point
C


C  INPUT INFORMATION on the phase space region is transferred via the
C    common block /BLIPSI/

      REAL*8 RM2, RMG, RM2MIN, RM2MAX, S, PTC
      REAL*8 RM2MINX, RM2MAXX, RM2MINY, RM2MAXY, M2MIN
      REAL*8 RM2MAX1, RM2MAX2
      COMMON /TRIBOSBLIPSI/ RM2(3), RMG(3), RM2MIN(3), RM2MAX(3),
     1                RM2MINX, RM2MAXX, RM2MINY, RM2MAXY, M2MIN,
     2                S, PTC, RM2MAX1, RM2MAX2
C
C   RM2            resonance mass squared
C   RMG            resonance total width * mass
C   RM2_MIN;_MAX   min & max values of the inv. mass**2 of resonance
C   PTJMIN         minimal PT value of the jet

      DOUBLE PRECISION CLR, XM2, XMG, B, VE, A
      COMMON /BKOPOU/ CLR(4,5,-1:1), XM2(6), XMG(6),
     1                B(6,6,6), VE(4,5), A(4,5)

C  local variables

      INTEGER I, J, II, JJ, MU, ic

      REAL*8 Y, Y2, PT, PHI
      REAL*8 xmin1, xmax1, delx1, tanx1, xval1
      REAL*8 xmin2, xmax2, delx2, tanx2, xval2
      REAL*8 xmin3, xmax3, delx3, tanx3, xval3
      REAL*8 xminx, xmaxx, delxx, tanxx, xvalx
      REAL*8 xminy, xmaxy, delxy, tanxy, xvaly
      REAL*8 coshy2, sinhy2, temp, test
      REAL*8 y_border, tau, ql(0:4,2), q(0:4,3)  
      REAL*8 pX(0:4), phiX, costhetaX, abs_pX, sinthetaX
      REAL*8 pY(0:4), phiY, costhetaY, abs_pY, sinthetaY
      REAL*8 phi2, costheta2, sintheta2
      REAL*8 phi3, costheta3, sintheta3
      REAL*8 phi4, costheta4, sintheta4
      REAL*8 pTmax, pTmin, pL, y2_border, ET
      REAL*8 abs_p, abs_p2, abs_p3, abs_p4
      REAL*8 C, m2, mw

      DOUBLE PRECISION xxwidth, xxm, xywidth, xym
      DOUBLE PRECISION dotrr

      PARAMETER ( xxwidth = 10d1, xxm = 45d1 )
      PARAMETER ( xywidth = 10d1, xym = 45d1 )

      EXTERNAL dotrr

      LOGICAL ldebug
      PARAMETER (ldebug=.false.)

      LOGICAL RESONANCE, TWOBODYDECAY
      EXTERNAL RESONANCE, TWOBODYDECAY

c--------------------------------------------------------------------

c---------------------------------------------------------------------

      if (ldebug) print*," enter LIPSWWZ "

      W = 1d0             !init of weightfactor

      if (NW.eq.1) then
c q^2_Z in narrow width approximation
         q(4,3) = rm2(3)
         W = W * rmg(3)/2.0d0

c pX^2 of intermediate particle X -> Z + Z out of rd(1)
         rm2maxx = s - q(4,3)
         if (.not. Resonance(xxm**2, xxm*xxwidth, rm2minx, rm2maxx,
     1                              rd(1), W, pX(4))) return

c q^2_Y generated out of rd(nrandl-2)
         if (N.eq.1) then
            rm2miny = pX(4) + q(4,3)
            if (.not. Resonance(xym**2, xym*xywidth, rm2miny, rm2maxy,
     1                              rd(nrandl-2), W, pY(4))) return
         endif

c q^2 of Z in narrow width approximation
         q(4,1) = rm2(1)
         W = W * rmg(1)/2.0d0

c q^2 of Z in narrow width approximation
         q(4,2) = rm2(2)
         W = W * rmg(2)/2.0d0        
      
      else !NW.ne.1
c q^2_Z generated out of rd(4)
         if (.not. Resonance(rm2(3), rmg(3), rm2min(3), rm2max(3),
     1                              rd(4), W, q(4,3))) return

c pX^2 of intermediate particle X -> Z + Z out of rd(1)
         rm2maxx = s - q(4,3)
         if ( rd(1) .le. 0.1d0 ) then
            if (.not. Resonance(xxm**2, xxm*xxwidth, rm2minx, rm2max1,
     1                              rd(1)/0.1d0, W, pX(4))) return
            W = W/0.1d0
         else if ( rd(1) .le.  0.8d0 ) then
            if (.not. Resonance(xm2(6), xmg(6), rm2max1, rm2max2,
     1                      (rd(1)-0.1d0)/0.7d0, W, pX(4))) return
            W = W/0.7d0
         else
            if (.not. Resonance(xxm**2, xxm*xxwidth, rm2max2, rm2maxx,
     1                      (rd(1)-0.8d0)/0.2d0, W, pX(4))) return
            W = W/0.2d0
         endif

c q^2_Y generated out of rd(nrandl-2)
         if (N.eq.1) then
            rm2miny = pX(4) + q(4,3)
            if (.not. Resonance(xym**2, xym*xywidth, rm2miny, rm2maxy,
     1                              rd(nrandl-2), W, pY(4))) return
         endif

c q^2 of W1 is generated via tan mapping out of rd(2)
         rm2max(1) = pX(4)
         if (rd(2).lt.0.1d0) then
            W=W/0.1d0
            m2=max(sqrt(xm2(6))-sqrt(rm2(1)),0.0d0)
            mw=rmg(1)*10.0d0*m2
            m2 = m2**2
            if (.not.Resonance(m2, mw, rm2min(1), rm2(1)-16d0*rmg(1),
     1                              rd(2)/0.1d0, W, q(4,1))) RETURN 
         else
            W=W/0.9d0
            if (.not.Resonance(rm2(1), rmg(1), rm2(1)-16d0*rmg(1),
     1           rm2max(1), (rd(2)-0.1d0)/0.9d0, W, q(4,1))) RETURN 
         endif

c q^2 of W2 is generated via tan mapping out of rd(3)
         rm2max(2) = pX(4)-q(4,1)
         if (rd(3).lt.0.1d0) then
            W=W/0.1d0
            m2=max(sqrt(xm2(6))-sqrt(rm2(2)),0.0d0)
            mw=rmg(2)*10.0d0*m2
            m2 = m2**2
            if (.not.Resonance(m2, mw, rm2min(2), rm2(2)-16d0*rmg(2),
     1                              rd(3)/0.1d0, W, q(4,2))) RETURN 
         else
            W=W/0.9d0
            if (.not.Resonance(rm2(2), rmg(2), rm2(2)-16d0*rmg(2),
     1           rm2max(2), (rd(3)-0.1d0)/0.9d0, W, q(4,2))) RETURN 
         endif

      endif !NW

c-------------------------------------------------------------------
c-------------------------------------------------------------------

      if (N.eq.1) then
c generate 2 -> 2 process for k1 k2 -> Pjet q_Y
         call TwoToJetPlusX(rd(5), rn, ecm, pTc, pY(4),
     1              k1(0), k2(0), x1, x2, pY(0), pJet(0), W)
         if (W.eq.0) return

c generate phase space for decay of Y -> X + Z out of rd(17), rd(18)
         if (.not. TwoBodyDecay(rd(nrandl), rd(nrandl-1), pY(0), pY(4),
     1                 pX(4), q(4,3), pX(0), q(0,3), W)) return
      else if (N.eq.0) then
c generate 2 -> 2 process for k1 k2 -> q_Z q_X
         call TwoToTwo(rd(5), rn, ecm, pX(4), q(4,3),
     1              k1(0), k2(0), x1, x2, pX(0), q(0,3), W)
         pjet(0) = 0.0d0
         pjet(1) = 0.0d0
         pjet(2) = 0.0d0
         pjet(3) = 0.0d0
      endif

c generate phase space for decay of X -> W+ + W- out of rd(8), rd(9)
      if (.not. TwoBodyDecay(rd(9), rd(8), pX(0), pX(4), q(4,1), q(4,2),
     1                                    q(0,1),q(0,2),W)) return

c generate phase space for decay of W+ -> l1 + l2 out of rd(10), rd(11)
      call TwoBodyDecay0(rd(11),rd(10),q(0,1),q(4,1),v(0,1),v(0,2),W)

c generate phase space for decay of W- -> l3 + l4 out of rd(12), rd(13)
      call TwoBodyDecay0(rd(13),rd(12),q(0,2),q(4,2),v(0,3),v(0,4),W)

c generate phase space for decay of Z -> l5 + l6 out of rd(14), rd(15)
      call TwoBodyDecay0(rd(15),rd(14),q(0,3),q(4,3),v(0,5),v(0,6),W)

c -----------------------------------------------------

      if (ldebug) then
         print*, "--------- new event -----------"
         print*, ""
         print*, "random numbers"
         print*, rn
         print*, rd(1), rd(2), rd(3)
         print*, rd(4), rd(5), rd(6)
         print*, rd(7), rd(8), rd(9)
         print*, rd(10), rd(11), rd(12)
         print*, rd(13), rd(14), rd(15)
         if (N.eq.1) print*, rd(16), rd(17), rd(18)
         print*, "y     = ", y
         print*, "tau   = ", tau
         print*, "x1    = ", x1
         print*, "x2    = ", x2
         print*, "y2    = ", y2
         print*, "wgt   = ", w
         print*, "---------------------------"

         print*, "--- momenta conservation ? ---"
         do mu=0,3
            test = k1(mu)+k2(mu)-Pjet(mu)-q(mu,1)-q(mu,2)-q(mu,3)
            if (test.gt.1E-9) print*, test
            test = q(mu,1)-v(mu,1)-v(mu,2)
            if (test.gt.1E-9) print*, test
            test = q(mu,2)-v(mu,3)-v(mu,4)
             if (test.gt.1E-9) print*, test
            test = q(mu,3)-v(mu,5)-v(mu,6)
             if (test.gt.1E-9) print*, test
            test = k1(mu)+k2(mu)-Pjet(mu)-v(mu,1)-v(mu,2)
     &             -v(mu,3)-v(mu,4)-v(mu,5)-v(mu,6)
             if (test.gt.1E-9) print*, test
         enddo
         print*,"tau*s = (k1+k2)^2"
         test = tau*s - dotrr(k1,k1)-dotrr(k2,k2)-2*dotrr(k1,k2)
         print*, test
         print*,"---tau*s = (q1 + q2 + q3 + pjet)^2 ?---"
         test = tau*s - dotrr(q(0,1),q(0,1)) - dotrr(q(0,2),q(0,2))
     &     - dotrr(q(0,3),q(0,3)) - 2*dotrr(q(0,1),q(0,2))
     &     - 2*dotrr(q(0,1),q(0,3)) - 2*dotrr(q(0,2),q(0,3))
     &     - dotrr(Pjet(0),Pjet(0)) - 2*dotrr(Pjet(0),q(0,1))
     &     - 2*dotrr(Pjet(0),q(0,2)) - 2*dotrr(Pjet(0),q(0,3))
         print*, test
         print*, "--- mass equal to zero ? ---"
         print*,"p1^2 = ",dotrr(k1(0),k1(0))
         print*,"p2^2 = ",dotrr(k2(0),k2(0))
         print*,"Pjet^2 = ",dotrr(Pjet(0),Pjet(0))
         print*,"v1^2 = ",dotrr(v(0,1),v(0,1))
         print*,"v2^2 = ",dotrr(v(0,2),v(0,2))
         print*,"v3^2 = ",dotrr(v(0,3),v(0,3))
         print*,"v4^2 = ",dotrr(v(0,4),v(0,4))
         print*,"v5^2 = ",dotrr(v(0,5),v(0,5))
         print*,"v6^2 = ",dotrr(v(0,6),v(0,6))
         print*, "-----------------------"

      endif

      return
      end

