c
c phase space generator for pp --> t + q
c
c       Manuel Baehr:     mb@particle.uni-karlsruhe.de
c       Initial version:  2005 March 07
c       Last modified:    2005 March 07
C
C  INPUT:   RD        random number array (between 0 and 1)
C           RN        additional random number which will not be driven by
C                     MONACO = azimuthal angle
C
C  OUTPUT:  K1,K2     4-vectors for 2 incoming partons
C           X1,X2     corresponding values of Feynman x
C           Q(0:4,2)  4-vectors for the vector boson resonances V1 and V2,
C                     the 4th component contains the mass**2
C           D(0:3,4)  4-vectors for the 4 decay leptons, assumed massless
C                     V1(Q(*,1)) --> D(*,1)+D(*,2)
C                     V2(Q(*,2)) --> D(*,3)+D(*,4)
C           P(*,N)    4-vectors for the N jets, assumed massless (except top)
C           W         the weight of the phase space point
C
C
C  INPUT INFORMATION on the phase space region is transferred via the
C    common block /BLIPSI/
C

c      REAL*8 RM2,RMG,RM2MIN,RM2MAX, S,M2MIN, YJMIN,YJMAX,PTJMIN,EJMIN
c      INTEGER INFOJ, iwidth
C           S         center of mass energy**2 of the machine
c      COMMON /BLIPSI/ RM2(0:2),RMG(0:2),RM2MIN(0:2),RM2MAX(0:2),
c     1                S,M2MIN,YJMIN(N1),YJMAX(N1),PTJMIN(N1),EJMIN(N1),
c     2                INFOJ(N1), iwidth(0:2)
C
      subroutine lipstop(rd,rn,k1,k2,p,x1,x2,w)
 
      implicit none

#include "VBFNLO/utilities/global.inc"
c received from global.inc:
c     nrand = phase space dim - 2 - phi
c     ecm   = collider energy


      logical mdebug, lhist, lhist1
      parameter(mdebug=.false.)

      integer hmem
      common /PAWC/ hmem
      real*8            alfas, xmt, alfa, xmz, xmw, sin2w, xmh
      common /bkopin/   alfas, xmt, alfa, xmz, xmw, sin2w, xmh


      integer n1, itn
      double precision tpi
      parameter (n1=max_p-nx, tpi=2D0*PI )
      double precision rd(*),rn,k1(0:3),k2(0:3),p(0:3,n1),x1,x2,w    

c only local variables ---------------------------------
      integer i, mu
      double precision pT, pTmax, pL, ycm, ycm_border, ET, abs_p, msq
      double precision coshycm, sinhycm, temp, test
      double precision taumin, s, y, y_border, tau, phi
c      double precision border, pT, pL, E3, E4, test  !temp variables

      
      s = ecm**2
      msq = xmt**2

      pTmax = (s-msq)/2d0/ecm

c end local variables ----------------------------------
      if (mdebug) then
         print*, "top mass              = ", xmt
         print*, "center of mass energy = ", ecm
         print*, "# outgoing partons    = ", n1
      endif

c generate y, pT, y*, phi from plain distributed rd, rn

      phi = rn * tpi

      pT = rd(1)*pTmax
c---------------------------
c local variables
      ET = sqrt(pT**2+msq)
      temp = (s+msq)/ecm/ET
      ycm_border = log( (temp+sqrt(temp**2-4))/2d0 )
c--------------------------
      ycm = ycm_border*2d0*( rd(2) - 0.5d0 )
c---------------------------
c local variables
      sinhycm = sinh(ycm)
      coshycm = sqrt(sinhycm**2+1)
     
      abs_p = sqrt(ET**2*coshycm**2-xmt**2)
      pL = sinhycm*ET

      tau = (abs_p + coshycm*ET)**2/s
      y_border = log(1/tau)
c--------------------------

      y = y_border*( rd(3) - 0.5d0 )

c generate momenta in lab frame + x1 x2
      
      x1 = sqrt(tau) * exp(y)
      x2 = sqrt(tau) * exp(-y)
      
      k1(0) = 0.5d0 * x1 * sqrt(s) 
      k1(1) = 0d0
      k1(2) = 0d0
      k1(3) = 0.5d0 * x1 * sqrt(s) 

      k2(0) = 0.5d0 * x2 * sqrt(s) 
      k2(1) = 0d0
      k2(2) = 0d0
      k2(3) = - 0.5d0 * x2 * sqrt(s) 
      
c C.M. system momenta:
      p(0,1) = sqrt(pT**2 + pL**2 + msq)
      p(1,1) = pT*cos(phi)
      p(2,1) = pT*sin(phi)
      p(3,1) = pL
      
      p(0,2) = sqrt(pT**2 + pL**2)
      p(1,2) = -p(1,1)
      p(2,2) = -p(2,1)
      p(3,2) = -p(3,1)


c boost in lab frame with -y
      call boostl(-y, p(0,1) )
      call boostl(-y, p(0,2) )

c Flussfaktoren, Phasenraumfaktoren, W-dichten + Jacobian
      w = 1d0/tpi/4d0/s
c      w = w * pT/msq/s
      w = w * pT/abs_p
      w = w * 1/(abs_p + ET*coshycm) 
      w = w * log(1/tau)                !(prop density)^-1 of y
      w = w * pTmax * ycm_border * 2d0  !(prop density)^-1 of pT and ycm
c      w = w * ( (pT**2/msq+1)/(pT**2/msq+sinhycm**2/coshycm**2) - 1 )
      w = w * 3.89379323d8         !conversion into fb
c end -----------------------------------------------------
c ============================= rest passiert im Hauptprogramm    

      if (mdebug) then
         print*, rd(1), rd(2), rd(3)
         print*, "y     = ", y
         print*, "tau   = ", tau
         print*, "ycm   = ", ycm
         print*, "phi   = ", phi
         print*, "x1    = ", x1
         print*, "x2    = ", x2
         print*, "wgt   = ", w
         print*, "---------------------------"

         print*, "--- momenta conservation ? ---"
         do mu=0,3
            test = k1(mu)+k2(mu)-p(mu,1)-p(mu,2)
            print*, test
         enddo
         print*, "-----------------------"

      endif

      return
      end subroutine

