***************************************************************************
*
*  This is the GGFLO main program. 
*
*  It's basically a duplicate of VBFNLO, but without the NLO part 
*  and different initialization routines for process (and couplings?).
*
****************************************************************************

****************************************************************************
*  Last modified: 29.07.2008
****************************************************************************

      PROGRAM ggflo
      implicit none

c include global variables via include-files, e.g. "procID"
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
#include "VBFNLO/utilities/tau_hel.inc"
#include "VBFNLO/utilities/process.inc"

c define local variables

c     variables for the integration loops
      integer iteration, i, seed
      integer*8 Ncall

c     digit characters for numbering the grid output file
      character*1 digit(9)
      data digit /'1','2','3','4','5','6','7','8','9'/
c     the filenames for the grid output
      character*50 file_name_out, file_name_in

c     random number array controlled by monaco
      real*8 rand(max_PS_dim),rn

c     4-momenta of the partons involved in the basic QCD 2->2(+1)+X process.
c     The partons are the incoming quarks/gluons + the 2 tagging jets + 
c     the real emission (for NLO only)
      real*8 p(0:3,max_p,max_kin) 

c     Additional particles like a higgs boson, or any decay products 
c     (also hadronic ones). in short: everything which is not a parton 
c     involved in the basic QCD 2->2(+1)+X process.
      real*8 v(0:3,max_v,max_kin) 

c     Jets after jet-defining function
      real*8 jets(0:7,max_jets,max_kin) 
      integer n_jets(max_kin)
c     Leptons, i.e. electrons, muons, taus? no neutrinos
      real*8 leptons(0:7,max_v,max_kin)
      integer n_leptons(max_kin)
c     photons
      real*8 photons(0:7,max_v,max_kin)
      integer n_photons(max_kin)
c     event weights, jacobi-factors, amplitude-squares
      real*8 weight, dps, m2s(0:3), dsig(0:3)

c     the result of the integration: crossection, error, chisquare
      real*8 chi2
      real*8 NLOxsection(-2:5), NLOsdev(-2:5), NLOchi2(-2:5)

c     other variables
      real*8 x(nx)   ! feynman x parameters
      logical cut_ok ! whether event passes cuts
      real*8 xuz(2,2:3)
      integer gnlo, L, mu
      logical lokt(3)
      common /nlovariable/ gnlo

c     stability flag for Cijs and Dijs
      logical Singular 
      COMMON /GramCheck/ Singular

c     external functions
      logical Cuts     ! general cut-function
      real*8 amplitude ! general squared-amplitude function
      external Cuts, amplitude

      real*8 RandomNumber
      external RandomNumber     

c     debugging and testing variables and switches
      logical showtiming,GramCheck
      parameter (showtiming=.false.)

      Singular = .false. 

      if (showtiming) call mtime

c     first, do all the initialization
      
      call InitGlobalParameters_GGFLO

      call PrintProcInfo
      
      call InitCouplings

      call InitScales

      call InitProcess

      call InitCuts
      
      call InitPhaseSpace
    
      call InitRandomNumbers(seed)

      call InitRandomNumbers(seed)

      call InitHistograms
     
* event output is commented out, because it's buggy and not standardized
*      call InitEventOutput 

c...Les Houches interface
      if (lha.and..not.doNLO) then 
         Select Case(procID)
         CASE DEFAULT
            lha=.true.
            call lha_file(-1)
         END SELECT
      endif

c==========================
c    The LO Calculation
c==========================
    
      if (doBorn) then

         call InitPDFs(0)

         gnlo = 0
         file_name_in  = gridname1(1:INDEX(gridname1,'  ')-1)


         DO iteration = 1, N_iterations1
            Ncall = 2**( N_points(0) - N_iterations1 + iteration )
           
            file_name_out = gridname1(1:INDEX(gridname1,'  ')-1)//
     &                   '.out.'//digit(iteration)
            if ( iteration .eq. 1 ) then
               call monaco_init( PS_dimension, Ncall, seed )
               call monaco_read(file_name_in)
            else
               call monaco_init2( PS_dimension, Ncall )
            end if       
            if (iteration.eq.N_iterations1) then
               call monran_set(1) ! save random number info
            endif

            call ResetWeights

            do i = 1, Ncall
               Singular = .false.
               
c     retrieve a set of random numbers and the corresponding weight
               call monaco_get( rand, weight)

c     transform the random numbers to a physical phase space point
               call phasespace(rand, p, x, v, dps  )

               if ( dps .gt. 0d0 ) then
c     define jets, leptons, photons
                  call defineJets(p, v, 1, jets, n_jets)
                  call DefineLeptons(v, 1, leptons, n_leptons)
                  call DefinePhotons(v, 1, photons, n_photons)

c     apply cuts
                  cut_ok = Cuts(jets, n_jets, leptons, n_leptons, photons, n_photons, 1)
                  if (cut_ok) then
c     call the matrix element code and get an amplitude square
                     m2s(0) = Amplitude(rand, p, x, v, 0)    
                  else
                     dps = 0d0
                     m2s(0) = 0d0
                  end if 
                  dsig(0) = m2s(0)*dps
                  if (cut_ok) then
                     eventweight=dsig(0)*weight
c     in the last iteration : 
                     if (iteration.eq.N_iterations1) then

c     pass event information to the histogram routine
                        call HistogramEvent(weight,dsig(0),dps,
     &                                rand, p(0,1,1), x, v(0,1,1), 
     &                                jets(0,1,1), n_jets(1), 
     &                                leptons(0,1,1), n_leptons(1), 
     &                                photons(0,1,1), n_photons(1), 0)

c     the unweighting of events : 
                        
c                        if (unweighting) then 
                           call Reweight(eventweight) ! reweight                         
c                        endif

                        if (eventweight.gt.0d0) then ! event is to be kept

                           if(lha.and..not.doNLO) call lha_put(p,v) !  Les Houches interface

#ifdef WITH_ROOT  
* event output is commented out, because it's buggy and not standardized
*                          call writerootevent(n_jets(1),jets(0,1,1),n_leptons(1),leptons(0,1,1), eventweight)
*    !                      call writerootevent2(n_p,p(0,1,1),n_v,v(0,1,1), eventweight)
*    !                      call writerootevent3(n_jets(1),jets(0,1,1),n_leptons(1),leptons(0,1,1), eventweight, tauchi(1) )
#endif

                        endif
                     endif
                     
                  endif ! cutok

               else     ! dps > 0d0
                  m2s(0) = 0d0
                  dsig(0) = 0d0
               end if

c     pass weight to monaco
               call monaco_put(rand, weight, dsig(0))

            end do
        
            call monaco_write( file_name_out )

         END DO

         call monaco_result(xsection, sdev, chi2)
     
c     print out the result. 
         print *,""
         print *," result (LO): ",xsection," +- ",sdev," fb  "
     &        ,sdev/xsection*100d0,'%'
         print *,""
      else
         print *," No LO calculation performed."
      endif

c...Les Houches interface
      if(lha.and..not.doNLO) then
         call lha_file(0)
         call lha_file(1)
      endif

      if(.not.doNLO) then 
         print*,'number of events in the last iteration =',ncall
         call reweightinginfo
      endif

      call WriteHistograms

* event output is commented out, because it's buggy and not standardized
*      call CloseEventOutput
     
c print out results in a file also
      open (20,
     &      file="xsection.out",
     &      status='REPLACE')
      write(20,*) xsection, sdev
      write(20,*) NLOxsection(-2), NLOsdev(-2)
      close(20)

      if (showtiming) call mtime
 
c...Les Houches interface
      if (lha.and..not.doNLO) then 
         print*,''
         print*,'created LHA file for the LO calculation'
         print*,''
      endif

  
      STOP
      END
c     END OF GGFLO
    
