***************************************************************************
*
*  This is the VBFNLO main program. 
*
****************************************************************************

****************************************************************************
*  Last modified: 01.04.2008
****************************************************************************

      PROGRAM vbfnlo
      implicit none

c include global variables via include-files, e.g. "procID"
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
#include "VBFNLO/utilities/tau_hel.inc"
#include "VBFNLO/utilities/process.inc"

c define local variables
c--------------------------------------------------------
c     variables for the integration loops
      integer iteration, i, seed
      integer*8 Ncall

c     digit characters for numbering the grid output file
      character*1 digit(9)
      data digit /'1','2','3','4','5','6','7','8','9'/
c     the filenames for the grid output
      character*50 file_name_out, file_name_in

c     random number array controlled by monaco
      real*8 rand(max_PS_dim),rn

c     4-momenta of the partons involved in the basic QCD 2->2(+1)+X process.
c     The partons are the incoming quarks/gluons + the 2 tagging jets + 
c     the real emission (for NLO only)
      real*8 p(0:3,max_p,max_kin) 

c     Additional particles like a higgs boson, or any decay products 
c     (also hadronic ones). in short: everything which is not a parton 
c     involved in the basic QCD 2->2(+1)+X process.
      real*8 v(0:3,max_v,max_kin) 

c     Jets after jet-defining function
      real*8 jets(0:7,max_jets,max_kin) 
      integer n_jets(max_kin)
c     Leptons, i.e. electrons, muons, taus? no neutrinos
      real*8 leptons(0:8,max_v,max_kin)
      integer n_leptons(max_kin)
c     photons
      real*8 photons(0:7,max_v,max_kin)
      integer n_photons(max_kin)
c     event weights, jacobi-factors, amplitude-squares
      real*8 weight, dps, m2s(0:max_kin), dsig(0:max_kin)

c     the result of the integration: crossection, error, chisquare
      real*8 chi2
      real*8 NLOxsection(-2:5), NLOsdev(-2:5), NLOchi2(-2:5)

c     other variables
      real*8 x(nx)   ! feynman x parameters
      logical cut_ok ! whether event passes cuts
      real*8 xuz(2,2:max_kin)
      integer gnlo, L, mu, nd
      logical lokt(max_kin)
      common /nlovariable/ gnlo

c     external functions
      logical Cuts     ! general cut-function
      real*8 amplitude ! general squared-amplitude function
      external Cuts, amplitude

      real*8 RandomNumber
      external RandomNumber     

c     debugging and testing variables and switches
      logical showtiming
      parameter (showtiming=.false.)

      if (showtiming) call mtime

c     first, do all the initialization
    
      call InitGlobalParameters

      call PrintProcInfo
      
      call InitCouplings

      call InitScales

      call InitProcess

      call InitCuts
      
      call InitPhaseSpace
    
      call InitRandomNumbers(seed)

      call InitRandomNumbers(seed)

      call InitHistograms

c...Les Houches interface
      if (lha.and..not.doNLO) then 
         Select Case(procID)
         CASE DEFAULT
            lha=.true.
            call lha_file(-1)
         END SELECT
      endif
      
c==========================
c    The LO Calculation
c==========================
    
      if (doBorn) then

         call InitPDFs(0)

         gnlo = 0
         file_name_in  = gridname1(1:INDEX(gridname1,'  ')-1)


         DO iteration = 1, N_iterations1
            Ncall = 2**( N_points(0) - N_iterations1 + iteration )
           
            file_name_out = gridname1(1:INDEX(gridname1,'  ')-1)//
     &                   '.out.'//digit(iteration)
            if ( iteration .eq. 1 ) then
               call monaco_init( PS_dimension, Ncall, seed )
               call monaco_read(file_name_in)
            else
               call monaco_init2( PS_dimension, Ncall )
            end if       
            if (iteration.eq.N_iterations1) then
               call monran_set(1) ! save random number info
            endif

            call ResetWeights

            do i = 1, Ncall
c     retrieve a set of random numbers and the corresponding weight
               call monaco_get( rand, weight)

c     transform the random numbers to a physical phase space point
               call phasespace(rand, p, x, v, dps  )

               if ( dps .gt. 0d0 ) then
c     define jets, leptons, photons
                  call defineJets(p, v, 1, jets, n_jets)
                  call DefineLeptons(v, 1, leptons, n_leptons)
                  call DefinePhotons(v, 1, photons, n_photons)

c     apply cuts
                  cut_ok = Cuts(jets, n_jets, leptons, n_leptons, 
     1                                       photons, n_photons, 1)
                  if (cut_ok) then
c     calculate fac. and ren. scale for this phase space point
                     call Scales(p, v, jets, n_jets, leptons, 
     1                          n_leptons, photons, n_photons, 1, 0)
c     call the matrix element code and get an amplitude square
                     m2s(0) = Amplitude(rand, p, x, v, 0)   
                  else
                     dps = 0d0
                     m2s(0) = 0d0
                  end if 
                  dsig(0) = m2s(0)*dps

                  if (cut_ok) then
                     eventweight=dsig(0)*weight


c     in the last iteration : 
                     if (iteration.eq.N_iterations1) then

c     pass event information to the histogram routine
                        call HistogramEvent(weight,dsig(0),dps,
     &                                rand, p(0,1,1), x, v(0,1,1), 
     &                                jets(0,1,1), n_jets(1), 
     &                                leptons(0,1,1), n_leptons(1), 
     &                                photons(0,1,1), n_photons(1), 0)

c     the unweighting of events : 
                        
c                        if (unweighting) then 
                           call Reweight(eventweight) ! reweight                         
c                        endif

                        if (eventweight.gt.0d0) then ! event is to be kept

                           if(lha.and..not.doNLO) call lha_put(p,v) !  Les Houches interface

#ifdef WITH_ROOT  
* event output is commented out, because it's buggy and not standardized
*                          call writerootevent(n_jets(1),jets(0,1,1),n_leptons(1),leptons(0,1,1), eventweight)
*    !                      call writerootevent2(n_p,p(0,1,1),n_v,v(0,1,1), eventweight)
*    !                      call writerootevent3(n_jets(1),jets(0,1,1),n_leptons(1),leptons(0,1,1), eventweight, tauchi(1) )
#endif

                        endif
                     endif
                     
                  endif ! cutok

               else     ! dps > 0d0
                  m2s(0) = 0d0
                  dsig(0) = 0d0
               end if

c     pass weight to monaco
               call monaco_put(rand, weight, dsig(0))

            end do
        
            call monaco_write( file_name_out )

         END DO

         call monaco_result(xsection, sdev, chi2)
     
c     print out the result. 
         print *,""
         print *," result (LO): ",xsection," +- ",sdev," fb  "
     &        ,sdev/xsection*100d0,'%'
         print *,""
      else
         print *," No LO calculation performed."
      endif

c...Les Houches interface
      if(lha.and..not.doNLO) then
         call lha_file(0)
         call lha_file(1)
      endif

      if(.not.doNLO) then 
         print*,'number of events in the last iteration =',ncall
         call reweightinginfo
      endif

#ifdef WITH_NLO
c============================
c    The NLO Calculation
c============================     
c  The virtual corrections      
c----------------------------
      if (doVirtuals.and.hasNLO) then

         if (N_iterations1.eq.1) then
            file_name_in  = gridname1(1:INDEX(gridname1,'  ')-1)
         else
            file_name_in  = gridname1(1:INDEX(gridname1,'  ')-1)//
     &           '.out.'//digit(N_iterations1-1)
         endif

         call InitPDFs(1)

         NLOxsection(0) = 0d0
         NLOsdev(0) = 0d0

         do gnlo = 1, NLO_Loops ! loop over different virtual contributions

            Ncall = 2**( N_points(gnlo) )
            file_name_out = gridname1(1:INDEX(gridname1,'  ')-1)//
     &                               '.out.'//digit(1)
            
            call monaco_init( PS_dimension, Ncall, seed )
            call monaco_read(file_name_in)
!            call monran_set(2)  
         
            do i = 1, Ncall
               call monaco_get( rand, weight)
         
               call phasespace(rand, p, x, v, dps  )
               if ( dps .gt. 0d0 ) then
                            
                  call defineJets(p, v, 1, jets, n_jets)
                  call DefineLeptons(v, 1, leptons, n_leptons)
                  call DefinePhotons(v, 1, photons, n_photons)
                  cut_ok = Cuts(jets, n_jets, leptons, n_leptons, photons, n_photons, 1)
           
                  if (cut_ok) then
                     call Scales(p, v, jets, n_jets, leptons, 
     1                          n_leptons, photons, n_photons, 1, 1)
                     m2s(0) = Amplitude(rand, p, x, v, gnlo)             
                  else
                     dps = 0d0
                     m2s(0) = 0d0
                  end if 

                  dsig(0) = m2s(0)*dps

                  call HistogramEvent(weight,dsig(0),dps,
     &                                rand, p(0,1,1), x, v(0,1,1), 
     &                                jets(0,1,1), n_jets(1), 
     &                                leptons(0,1,1), n_leptons(1), 
     &                                photons(0,1,1), n_photons(1), 1)

               else
                  m2s(0) = 0d0
                  dsig(0) = 0d0
               end if

               call monaco_put(rand, weight, dsig(0))
            end do

            call monaco_write(file_name_out)
     
            call monaco_result(NLOxsection(gnlo), 
     &                         NLOsdev(gnlo), NLOchi2(gnlo))

            print *," "
            print *," result for virtual contribution, no. ",gnlo
            print *," sigma = ",NLOxsection(gnlo)," +- ",
     &              NLOsdev(gnlo)," fb  "
            print *," "
            NLOxsection(0) = NLOxsection(0) + NLOxsection(gnlo)
            NLOsdev(0) = sqrt(NLOsdev(0)**2 + NLOsdev(gnlo)**2)

            print *,' total result for virtual contributions so far '

            print *,' sigma = ',NLOxsection(0),' +- ',NLOsdev(0),' fb'
            print *,' '

         enddo

      endif

c----------------------------
c    The real emissions      
c----------------------------
      if (doEmissions.and.hasNLO) then
         call InitPDFs(1)

         call InitRealEmissions

         file_name_in  = gridname2(1:INDEX(gridname2,'  ')-1)

         DO iteration = 1, N_iterations2
            Ncall = 2**( N_points(-1) - N_iterations2 + iteration )
           
            file_name_out = gridname2(1:INDEX(gridname2,'  ')-1)//
     &                   '.out.'//digit(iteration)
            if ( iteration .eq. 1 ) then
               call monaco_init( PS_dimension, Ncall, seed )
               call monaco_read(file_name_in)
            else
               call monaco_init2( PS_dimension, Ncall )
            end if       
           
            do i = 1, Ncall
               call monaco_get( rand, weight)

               call phasespace(rand, p, x, v, dps  )

               if ( dps .gt. 0d0 ) then      
                  call ptilde(p(0,1,1),xuz(1,2),v(0,1,1)) 
                  call defineJets(p, v, n_kin, jets, n_jets)
                  call DefineLeptons(v, n_kin, leptons, n_leptons)
                  call DefinePhotons(v, n_kin, photons, n_photons)
          
                  lokt(1) = Cuts(jets, n_jets, leptons, n_leptons,
     1                                         photons,n_photons, 1)
                  cut_ok = lokt(1)
                  do nd = 2, n_kin
                     lokt(nd) = Cuts(jets, n_jets, leptons, n_leptons,
     1                                           photons, n_photons, nd)
                     cut_ok = cut_ok .or. lokt(nd)
                  enddo
                  if (cut_ok) then
                     call Scales(p, v, jets, n_jets, leptons, 
     1                          n_leptons, photons, n_photons, n_kin, 1)
                     call RE_Amplitude(rand, p, x, v, lokt, xuz, m2s)
                  else
                     dps = 0d0
                     do L = 0,n_kin
                        m2s(L) = 0d0
                     enddo                     
                  end if 

                  do L = 0,n_kin
                     dsig(L) = m2s(L)*dps
                  enddo   

                  if (iteration.eq.N_iterations2) then

                     if (dsig(0).ne.0d0.or.dsig(1).ne.0) then
                        call HistogramEvent(weight,dsig(1),dps,
     &                                rand, p(0,1,1), x, v(0,1,1), 
     &                                jets(0,1,1), n_jets(1), 
     &                                leptons(0,1,1), n_leptons(1), 
     &                                photons(0,1,1), n_photons(1), 1)
                     endif
                     do L = 2,n_kin
                        if (dsig(L).ne.0d0) then
                           call HistogramEvent(weight,dsig(L),dps,
     &                                rand, p(0,1,L), x, v(0,1,L), 
     &                                jets(0,1,L), n_jets(L), 
     &                                leptons(0,1,L), n_leptons(L),
     &                                photons(0,1,1), n_photons(1), 1)

                        endif
                     enddo


                  endif

               else
                  do L = 0,n_kin
                     m2s(L) = 0d0
                     dsig(L) = 0d0
                  enddo
               end if
               

               call monaco_put(rand, weight, dsig(0))
               
            end do
        
            call monaco_write( file_name_out )

         END DO

         call monaco_result(NLOxsection(-1), NLOsdev(-1), NLOchi2(-1))
     
c     print out the result. 
         print *,""
         print *," result for real emissions "
         print *," sigma = ",NLOxsection(-1)," +- ",NLOsdev(-1)," fb"
         print *," "

      endif
      
      if (dovirtuals.and.doEmissions.and.hasNLO) then
         NLOxsection(-2) = NLOxsection(-1)+NLOxsection(0)
         NLOsdev(-2) = sqrt(NLOsdev(-1)**2+NLOsdev(0)**2)
         print *," "
         print *," final result at NLO "         
         print *," sigma = ",NLOxsection(-2)," +- ",NLOsdev(-2)," fb"
     &   ,NLOsdev(-2)/NLOxsection(-2)*100d0,'%'
      endif

#endif

      call WriteHistograms

* event output is commented out, because it's buggy and not standardized
*      call CloseEventOutput
     
c print out results in a file also
      open (20,
     &      file="xsection.out",
     &      status='REPLACE')
      write(20,*) xsection, sdev
      write(20,*) NLOxsection(-2), NLOsdev(-2)
      close(20)

      if (showtiming) call mtime
 
c...Les Houches interface
      if (lha.and..not.doNLO) then 
         print*,''
         print*,'created LHA file for the LO calculation'
         print*,''
      endif
      if(lha.and.doNLO) then
         print*,''
         print*,'No LHA file for the NLO calculation'
         print*,''
      endif
  
      STOP
      END
c     END OF VBFNLO
    

