c*************************************************************************
      FUNCTION Amplitude(rand, p, x, v, nlo)
c*************************************************************************
c     This is the manager of all matrixelement routines and the (only) 
c     interface of those to the main program.  Depending on the process ID
c     the right matrix element is called with the correct calling 
c     convention, which might well be individually different.
c     Once many processes are included in the program, this subroutine
c     will be quite complex, but it should actually be not much more than a
c     big select-case statement.
c
c*************************************************************************
c     INPUT
c     rand      : Array with random numbers driven by monaco. 
c     p         : 4-momenta of the partons involved in the process.
c                 The first 2 entries are the incoming partons.
c     x         : The feynman x of the incoming partons 
c     v         : The 4-momenta of the other particles
c     nlo       : switch for NLO processes
c     
c     OUTPUT
c     Amplitude : The amplitude square as returned by the matrix element code.
c*************************************************************************
      implicit none

#include "global.inc"
#include "process.inc"

      real*8 Amplitude
      real*8 x(nx) 
      real*8 rand(max_PS_dim)
      real*8 p(0:3,max_p,max_kin)
      real*8 v(0:3,max_v,max_kin)
      real*8 rn, rn2(2), xuz(2,2:max_kin), m2s(0:max_kin), m2s_dec,dump
      logical lokt(max_kin)
      integer NLO,i

c     to convert "NLO" to box- and pentagon call conventions
      integer VBFNLOswitch(0:3)
      data VBFNLOswitch / 0, 4,-4,-5 /

c     masses and widths
      double precision XM2s(6),XMGs(6)
      COMMON /BKOPOUshort/ XM2s,XMGs
      Double precision BWNE,BWUD,BWTB,BZNN,BZEE,BZUU,BZDD,BZTT,
     &                 BHWW,BHZZ,BHGG,BHTT,BHBB,BHCC,BHTAU,BHMU,BHGAM
      COMMON /BRANCH/ BWNE,BWUD,BWTB,BZNN,BZEE,BZUU,BZDD,BZTT,
     &                BHWW,BHZZ,BHGG,BHTT,BHBB,BHCC,BHTAU,BHMU,
     &                BHGAM


c     external matrixelement functions
      real*8 m2s_ubtd
      real*8 m2s_wbfvv
      real*8 m2s_wbfw
      real*8 m2s_ggfh
      real*8 m2s_z2j
      real*8 RandomNumber
      real*8 higgs2tau
      double precision m2s_qqVV
      double precision m2s_qqWWZ
      double precision m2s_qqZZW
      double precision m2s_qqWWW

      integer mu
      external m2s_ubtd
      external m2s_wbfvv
      external m2s_wbfw
      external m2s_ggfh
      external m2s_z2j
      external RandomNumber
      external higgs2tau
      external m2s_qqVV
      external m2s_qqWWZ
      external m2s_qqZZW
      external m2s_qqWWW

c     auxiliary 
      real*8 amp, Q(0:3)
      integer lflavr(2)
      real*8 mass2
      external mass2

      SELECT CASE(process)

#ifdef WITH_VBF
**********************
*
*     VBF : Hjj
*
**********************
c     pp -> Hjj
      CASE(Hjj)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
c     pp -> Hjj -> gamma gamma jj
      CASE(Hjj_AA)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
         AMplitude = Amplitude*BHGAM
c     pp -> Hjj -> mu+mu- jj
      CASE(Hjj_mu)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
         Amplitude = Amplitude*BHMU
c     pp -> Hjj -> tau+tau- jj
      CASE(Hjj_tau)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
         Amplitude = Amplitude*BHTAU
c     pp -> Hjj -> b bbar jj
      CASE(Hjj_bbar)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
         Amplitude = AMplitude*BHBB
c     pp -> Hjj -> W W jj
      CASE(Hjj_WW)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
         call m2s_VVsum(v(0,1,1),3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
c     pp -> Hjj -> Z Z jj 
      CASE(Hjj_ZZ_ll)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
         call m2s_VVsum(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec

      CASE(Hjj_ZZ_lnu)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(6,0,nlo,x,p(0,1,1),v(0,1,1),rn)
         call m2s_VVsum(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec  

**********************
*
*     VBF : Hjjj
*
**********************
c     pp -> Hjjj
      CASE(HjjjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> Hjjj -> gamma gamma jjj
      CASE(HjjjLO_AA)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHGAM
c     pp -> Hjjj -> mu+mu- jjj
      CASE(HjjjLO_mu)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHMU
c     pp -> Hjjj -> tau+tau- jjj
      CASE(HjjjLO_tau)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHTAU
c     pp -> Hjjj -> b bbar jjj
      CASE(HjjjLO_bbar)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         Amplitude = AMplitude*BHBB
c     pp -> Hjjj -> W W jjj
      CASE(HjjjLO_WW)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0, 0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         call m2s_VVsum(v(0,1,1),3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
c     pp -> Hjjj -> Z Z jjj
      CASE(HjjjLO_ZZ_ll)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0,0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         call m2s_VVsum(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec

      CASE(HjjjLO_ZZ_lnu)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 6,0,0, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
         call m2s_VVsum(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
#endif

#ifdef WITH_HJJJ
********************
*
*     VBF : Hjjj
*
********************
c     pp -> Hjjj
      CASE(Hjjj)
         rn = RandomNumber()
         call m2s_wbfhj(6, 0, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
c         pp -> Hjjj -> gamma gamma jjj
      CASE(Hjjj_AA)
         rn = RandomNumber()
         call m2s_wbfhj(6, 0, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
         Amplitude = Amplitude*BHGAM
c     pp -> Hjjj -> mu+mu- jjj
      CASE(Hjjj_mu)
         rn = RandomNumber()
         call m2s_wbfhj(6, 0, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
         Amplitude = Amplitude*BHMU
c     pp -> Hjjj -> tau+tau- jjj
      CASE(Hjjj_tau)
         rn = RandomNumber()
         call m2s_wbfhj(6, 0, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
         Amplitude = Amplitude*BHTAU
c     pp -> Hjjj -> b bbar jjj
      CASE(Hjjj_bbar)
         rn = RandomNumber()
         call m2s_wbfhj(6, 0, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
         Amplitude = AMplitude*BHBB
c     pp -> Hjjj -> W W jjj
      CASE(Hjjj_WW)
         rn = RandomNumber()
         call m2s_wbfhj(6, 0, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
         call m2s_VVsum_H3j(v(0,1,1),3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
c     pp -> Hjjj -> Z Z jjj 
      CASE(Hjjj_ZZ_ll)
         rn = RandomNumber()
         call m2s_wbfhj(6, 0, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
         call m2s_VVsum_H3j(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec

      CASE(Hjjj_ZZ_lnu)
         rn = RandomNumber()
         call m2s_wbfhj(6, 0, nlo, x, p(0,1,1), v(0,1,1), rn, m2s(0))
         Amplitude = m2s(0)
         call m2s_VVsum_H3j(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec

*********************
*
*     VBF : Hjjjj
*
*********************
c     pp -> Hjjjj
      CASE(HjjjjLO)
         
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude=m2s(0)
c     pp -> Hjjjj-> gamma gamma jjjj
      CASE(HjjjjLO_AA)
         
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude=m2s(0) 
         Amplitude = Amplitude*BHGAM
c     pp -> Hjjjj -> mu+mu- jjjj
      CASE(HjjjjLO_mu)
         
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHMU
c     pp -> Hjjjj -> tau+tau- jjjj
      CASE(HjjjjLO_tau)
        
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHTAU
c     pp -> Hjjjj -> b bbar jjjj
      CASE(HjjjjLO_bbar)
         
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude=m2s(0)
         Amplitude = Amplitude*BHBB
c     pp -> Hjjjj -> W W jjjj
      CASE(HjjjjLO_WW)
        
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude = m2s(0)
         call m2s_VVsum_H3j(v(0,1,1),3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
c     pp -> Hjjjj -> Z Z jjjj 
      CASE(HjjjjLO_ZZ_ll)
c         
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude = m2s(0)
         call m2s_VVsum_H3j(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec
 
      CASE(HjjjjLO_ZZ_lnu)
c         
         do i=1,n_kin
            lokt(i)=.false.
         enddo
         lokt(1)=.true.
         call m2s_wbfh4j(6, 0, 0, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s(0))
         Amplitude = m2s(0)
         call m2s_VVsum_H3j(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         Amplitude = Amplitude*m2s_dec 
         
#endif

#ifdef WITH_VBF
***********************
*
*     VBF : Wjj, Zjj
*
***********************
c     pp -> Zjj
      CASE(Zjj_l, Zjj_nu) 
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(2,0,nlo,x,p(0,1,1),v(0,1,1),rn)
c     pp -> W+jj
      CASE(WPjj) 
         rn = RandomNumber()
         Amplitude = m2s_wbfw(3,nlo,x,p(0,1,1),v(0,1,1),rn)
c     pp -> W-jj
      CASE(WMjj) 
         rn = RandomNumber()
         Amplitude = m2s_wbfw(4,nlo,x,p(0,1,1),v(0,1,1),rn)

**************************
*
*     VBF : Wjjj, Zjjj
*
**************************
c     pp -> Zjjj
      CASE(ZjjjLO_l, ZjjjLO_nu) 
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 2,0, 0, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> W+jjj
      CASE(WPjjjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfw3j( 3, 0, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> W-jjj
      CASE(WMjjjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfw3j( 4, 0, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)

******************************
*
*     VBF : WWjj, ZZjj, WZjj
*
******************************
c     pp -> WWjj 
      CASE(WPWMjj)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(34,0,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)
c     pp -> ZZjj 
      CASE(ZZjj_ll)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(22,0,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)
      CASE(ZZjj_lnu)
         rn = RandomNumber()
         Amplitude = m2s_wbfvv(21,0,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)
c     pp -> W+Zjj
      CASE(WPZjj)
         rn = RandomNumber()
         Amplitude = m2s_wbfw(32,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)
c     pp -> W-Zjj
      CASE(WMZjj)
         rn = RandomNumber()
         Amplitude = m2s_wbfw(42,VBFNLOswitch(nlo),x,p(0,1,1),v(0,1,1),rn)

**********************************
*
*     VBF : WWjjj, ZZjjj, WZjjj
*
**********************************
c     pp -> WW jjj
      CASE(WPWMjjjLO)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 34,0,  VBFNLOswitch(nlo), lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> ZZ jjj
      CASE(ZZjjjLO_ll)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 22,0,  VBFNLOswitch(nlo), lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
      CASE(ZZjjjLO_lnu)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfzhg( 21,0,  VBFNLOswitch(nlo), lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> W+Z jjj
      CASE(WPZjjjLO)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfw3j( 32,0,  lokt, 
     1          x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
c     pp -> W-Z jjj
      CASE(WMZjjjLO)
         rn = RandomNumber()
         call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_wbfw3j( 42,0, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         Amplitude=m2s(0)
#endif

#ifdef WITH_DIBOSON
*********************
*
*     DIBOSON : WW
*
*********************
c pp -> W+ W- -> ve e+ mu- vm~
      CASE(WW)
         rn = RandomNumber()
         Amplitude = m2s_qqvv( 34, nlo, x, p, v, rn)

**********************
*
*     DIBOSON : WWj
*
**********************
c pp -> W+ W- j -> ve e+ mu- vm~ j
      CASE(WWjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqVVj( 34, 0, lokt,x, p, v, rn, xuz(1,2), m2s)
         Amplitude = m2s(0)
#endif          

#ifdef WITH_TRIBOSON
**************************************
*
*     TRIBOSON : WWZ, ZZW, WWW
*
**************************************	 
c pp -> W+ W- Z -> ve e+ mu- vm~ ta- ta+
      CASE(WWZ)
         rn = RandomNumber()
         Amplitude = m2s_qqWWZ(342,VBFNLOswitch(nlo),x,p,v,rn,narrowwidth)

c pp -> Z Z W+ -> e- e+ mu- mu+ vt ta+
      CASE(ZZWP)
         rn = RandomNumber()
         Amplitude = m2s_qqZZW(223,VBFNLOswitch(nlo),x,p,v,rn,narrowwidth)

c pp -> Z Z W- -> e- e+ mu- mu+ ta- vt~
      CASE(ZZWM)
         rn = RandomNumber()
         Amplitude = m2s_qqZZW(224,VBFNLOswitch(nlo),x,p,v,rn,narrowwidth)

c pp -> W+W-W+ -> ve e+ mu- vmu~ vt ta+
      CASE(WWWP)
         rn = RandomNumber()
         Amplitude = m2s_qqWWW( 343,VBFNLOswitch(nlo),x,p,v,rn,narrowwidth)

c pp -> W+W-W- -> mu- vmu~ ve e+ ta- vt~
      CASE(WWWM)
         rn = RandomNumber()
         Amplitude = m2s_qqWWW( 434,VBFNLOswitch(nlo),x,p,v,rn,narrowwidth)

*****************************************
*
*     TRIBOSON : WWZj, ZZWj, WWWj
*
*****************************************
c pp -> W+ W- Z j -> ve e+ mu- vm~ ta- ta+ j
      CASE(WWZjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqWWZj(342,0,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)
         Amplitude = m2s(0)

c pp -> Z Z W+ j -> e- e+ mu- mu+ vt ta+ j
      CASE(ZZWPjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqZZWj(223,0,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)
         Amplitude = m2s(0)

c pp -> Z Z W- j -> e- e+ mu- mu+ ta- vt~ j
      CASE(ZZWMjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqZZWj(224,0,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)
         Amplitude = m2s(0)

c pp -> W+W-W+j -> ve e+ e- ve~ vmu mu+ j
      CASE(WWWPjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqWWWj(343,0,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)
         Amplitude = m2s(0)

c pp -> W+W-W-j -> mu- vmu~ ve e+ ta- vt~ j
      CASE(WWWMjLO)
         rn = RandomNumber()
         lokt(1) = .true.
         lokt(2) = .false.
         lokt(3) = .false.
         call m2s_qqWWWj(434,0,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)
         Amplitude = m2s(0)
#endif

#ifdef WITH_GGF
**************************************
*
*     GF : Hjj 
*
**************************************
c     GF pp -> Hjj
      CASE(GFHjj)
         Amplitude = m2s_ggfh( x, p(0,1,1),v(0,1,1),rand )
#endif

      CASE DEFAULT
         print *,"Error: Amplitude for process not implemented, ID = ",procID
         print *,"Check process-ID and compiler flags"
         STOP
      END SELECT

      RETURN
      END
       


#ifdef WITH_NLO
c*************************************************************************
      SUBROUTINE RE_Amplitude(rand, p, x, v, lokt, xuz, m2s)
c*************************************************************************
c     Amplitude for real emissions.
c*************************************************************************
      implicit none

#include "global.inc"
#include "process.inc"

      real*8 x(nx) 
      real*8 rand(max_PS_dim)
      real*8 p(0:3,max_p,max_kin)
      real*8 v(0:3,max_v,max_kin)
      real*8 rn, xuz(2,2:max_kin), m2s(0:max_kin), m2s_dec
      logical lokt(max_kin)
      integer L

c     masses and widths
      real*8 XM2s(6),XMGs(6)
      COMMON /BKOPOUshort/ XM2s,XMGs

      Double precision BWNE,BWUD,BWTB,BZNN,BZEE,BZUU,BZDD,BZTT,
     &                 BHWW,BHZZ,BHGG,BHTT,BHBB,BHCC,BHTAU,BHMU,BHGAM
      COMMON /BRANCH/ BWNE,BWUD,BWTB,BZNN,BZEE,BZUU,BZDD,BZTT,
     &                BHWW,BHZZ,BHGG,BHTT,BHBB,BHCC,BHTAU,BHMU,
     &                BHGAM

      real*8 RandomNumber
      external RandomNumber
      SELECT CASE(process)
#ifdef WITH_VBF
**************************************
*
*     VBF : Hjjj, WWjjj, ZZjjj, WZjjj
*
**************************************
c     pp -> Hjjj
      CASE(Hjj)
         rn = RandomNumber()
         call m2s_wbfzhg( 6,0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> Hjjj -> gamma gamma jjj
      CASE(Hjj_AA)
         rn = RandomNumber()
         call m2s_wbfzhg( 6,0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHGAM
         enddo
c     pp -> Hjjj -> mu+mu- jjj
      CASE(Hjj_mu)
         rn = RandomNumber()
         call m2s_wbfzhg( 6,0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHMU
         enddo
c     pp -> Hjjj -> tau+tau- jjj
      CASE(Hjj_tau)
         rn = RandomNumber()
         call m2s_wbfzhg( 6,0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHTAU
         enddo
c     pp -> Hjjj -> b bbar jjj
      CASE(Hjj_bbar)
         rn = RandomNumber()
         call m2s_wbfzhg( 6,0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         do L = 0,3
            m2s(L) = m2s(L)*BHBB
         enddo
c     pp -> Hjjj -> W W jjj
      CASE(Hjj_WW)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         call m2s_VVsum(v(0,1,1),3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
c     pp -> Hjjj -> Z Z jjj
      CASE(Hjj_ZZ_ll)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         call m2s_VVsum(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
      CASE(Hjj_ZZ_lnu)
         rn = RandomNumber()
         call m2s_wbfzhg(6, 0, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
         call m2s_VVsum(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,3
            m2s(L) = m2s(L)*m2s_dec
         enddo
#endif

#ifdef WITH_HJJJ
**************************************
*
*     VBF : Hjjjj
*
**************************************
c     pp -> Hjjjj
      CASE(Hjjj)
         call m2s_wbfh4j(6, 0, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
c     pp -> Hjjjj -> gamma gamma jjjj
      CASE(Hjjj_AA)
         call m2s_wbfh4j(6, 0, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
         do L = 0,n_kin
            m2s(L) = m2s(L)*BHGAM
         enddo
c     pp -> Hjjjj -> mu+mu- jjjj
      CASE(Hjjj_mu)
         call m2s_wbfh4j(6, 0, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
         do L = 0,n_kin
            m2s(L) = m2s(L)*BHMU
         enddo
c     pp -> Hjjjj -> tau+tau- jjjj
      CASE(Hjjj_tau)
         call m2s_wbfh4j(6, 0, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
         do L = 0,n_kin
            m2s(L) = m2s(L)*BHTAU
         enddo
c     pp -> Hjjjj -> b bbar jjjj
      CASE(Hjjj_bbar)
         call m2s_wbfh4j(6, 0, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
         do L = 0,n_kin
            m2s(L) = m2s(L)*BHBB
         enddo
c     pp -> Hjjjj -> W W jjjj
      CASE(Hjjj_WW)
         call m2s_wbfh4j(6, 0, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
         call m2s_VVsum_H3j(v(0,1,1),3,4,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,n_kin
            m2s(L) = m2s(L)*m2s_dec
         enddo
c     pp -> Hjjjj -> Z Z jjjj
      CASE(Hjjj_ZZ_ll)
         call m2s_wbfh4j(6, 0, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
         call m2s_VVsum_H3j(v(0,1,1),2,2,2,2,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,n_kin
            m2s(L) = m2s(L)*m2s_dec
         enddo

      CASE(Hjjj_ZZ_lnu)
         call m2s_wbfh4j(6, 0, 1, lokt, x, p(0,1,1), v(0,1,1), xuz, m2s)
         call m2s_VVsum_H3j(v(0,1,1),2,2,2,1,m2s_dec)
         m2s_dec = m2s_dec/(16.0d0 * pi * xmgs(6))
         do L = 0,n_kin
            m2s(L) = m2s(L)*m2s_dec
         enddo
#endif 

#ifdef WITH_VBF
**************************************
*
*     VBF : Wjjj, Zjjj
*
**************************************
c     pp -> Zjjj 
      CASE(Zjj_l, Zjj_nu) 
         rn = RandomNumber()
         call m2s_wbfzhg( 2,0, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> W+ jjj
      CASE(WPjj) 
         rn = RandomNumber()
         call m2s_wbfw3j( 3, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> W- jjj
      CASE(WMjj)
         rn = RandomNumber()
         call m2s_wbfw3j( 4, 1, lokt,
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)

**************************************
*
*     VBF : WWjjj, ZZjjj, WZjjj
*
**************************************
c     pp -> WW jjj
      CASE(WPWMjj)
         rn = RandomNumber()
         call m2s_wbfzhg( 34,0, 1, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s) 
c     pp -> ZZ jjj
      CASE(ZZjj_ll)
         rn = RandomNumber()
         call m2s_wbfzhg( 22,0, 1, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s) 
      CASE(ZZjj_lnu)
         rn = RandomNumber()
         call m2s_wbfzhg( 21,0, 1, lokt, 
     1                    x, p(0,1,1), v(0,1,1), rn, xuz, m2s) 
c     pp -> W+Z jjj
      CASE(WPZjj) 
         rn = RandomNumber()
         call m2s_wbfw3j( 32, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
c     pp -> W-Z jjj
      CASE(WMZjj) 
         rn = RandomNumber()
         call m2s_wbfw3j( 42, 1, lokt, 
     1        x, p(0,1,1), v(0,1,1), rn, xuz, m2s)
#endif

#ifdef WITH_DIBOSON
**************************************
*
*     DIBOSON : WWj
*
**************************************	 
c pp -> W+ W- (j) -> ve e+ mu- vm~ (j)
      CASE(WW)
         rn = RandomNumber()
         call m2s_qqVVj( 34, 1, lokt, x, p, v, rn, xuz(1,2), m2s)
#endif

#ifdef WITH_TRIBOSON
*****************************************
*
*     TRIBOSON : WWZj, ZZWj, WWWj
*
*****************************************
c pp -> W+ W- Z -> ve e+ mu- vm~ ta- ta+
      CASE(WWZ)
         rn = RandomNumber()
         call m2s_qqWWZj(342,1,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)

c pp -> Z Z W+ -> e- e+ mu- mu+ vt ta+
      CASE(ZZWP)
         rn = RandomNumber()
         call m2s_qqZZWj(223,1,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)

c pp -> Z Z W- -> e- e+ mu- mu+ ta- vt~
      CASE(ZZWM)
         rn = RandomNumber()
         call m2s_qqZZWj(224,1,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)

c pp -> W+ W- W+ -> ve e+ mu- vm~ vt ta+
      CASE(WWWP)
         rn = RandomNumber()
         call m2s_qqWWWj(343,1,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)

c pp -> W- W+ W- -> mu- vm~ ve e+ ta- vt~
      CASE(WWWM)
         rn = RandomNumber()
         call m2s_qqWWWj(434,1,lokt,x,p,v,rn,xuz(1,2),m2s,narrowwidth)

#endif

      CASE DEFAULT
         print *,"Error: Real emissions Amplitude ",
     &           "for process not implemented, ID = ",procID
         STOP
      END SELECT

      RETURN
      END

c with_nlo
#endif
