
****************************************************************************
*   This file contains functions and subroutines for the cuts.
c***************************************************************************
c   LIST OF ALL FUNCTIONS AND SUBROUTINES IN THIS FILE:
c
c     FUNCTION Cuts(jets, njets, v, L)
c     FUNCTION Basic_Cuts(jets, njets, v)
c     FUNCTION VBF_Cuts(jets, njets, v)
c     SUBROUTINE DefineLeptons(v, Lmax, leptons, n_leptons)
c
c***************************************************************************
*   Last modified: 05.04.2007
****************************************************************************
c*************************************************************************
      SUBROUTINE InitCuts
c*************************************************************************
c     Reads in the values for the cuts.
c*************************************************************************
      implicit none
#include "global.inc"
#include "cuts.inc"

      call loadfile("cuts.dat",.false.)

c jet cuts
      call read_real("PT_JET_MIN",ptmin_def,20.0d0)
      call read_real("Y_P_MAX",ymax_part,5d0)
      call read_real("Y_JET_MAX",ymax_def,4.5d0)
      call read_real("RJJ_MIN",rjjmin,0.6d0)

c leptons cuts
      call read_real("RLL_MIN",rll_min,0.0d0)
      call read_real("RLL_MAX",rll_max,50.0d0)
      call read_real("RJL_MIN",rjl_min,0d0)
      call read_real("Y_L_MAX", ymax_l,2.5d0)
      call read_real("PT_L_MIN", ptlmin, 0d0)
      call read_real("MLL_MIN",mllmin,15d0)

c photon cuts
      call read_real("RGG_MIN",rgg_min,0.6d0)
      call read_real("RGG_MAX",rgg_max,50.0d0)
      call read_real("RJG_MIN",rjg_min,0.6d0)
      call read_real("RLG_MIN",rlg_min,0.0d0)
      call read_real("Y_G_MAX",ymax_g,2.5d0)
      call read_real("PT_G_MIN",ptgmin,20d0)

c VBF cuts -  only applied to tagging jets
      call read_real("ETAJJ_MIN",etajjmin, 0d0)
      call read_logical("YSIGN",ysign,.false.)
      call read_logical("LRAPIDGAP",lrapidgap,.false.)
      call read_real("DELY_JL",dely_jlep, 0d0)
      call read_logical("GRAPIDGAP",grapidgap,.false.)
      call read_real("DELY_JG",dely_jphot, 0d0)

      call read_real("MDIJ_MIN",mdijmin, 0d0)
      call read_real("MDIJ_MAX",mdijmax,14000d0)

      call read_logical("JVETO",lveto,.false.)
      call read_real("DELY_JVETO",dely_jveto,0d0)
      call read_real("YMAX_VETO",ymax_veto,0d0)
      call read_real("PTMIN_VETO",ptmin_veto,0d0)

      call closefile
      min_jets=njet_min !minimal number of jets: set in process.F

c print information on cuts
      print *,""
      print *,"         information on cut parameters"
      print *,"-----------------------------------------------"
      print *,""
      print *,"minimum jet pt                       (PT_JET_MIN): ",ptmin_def
      print *,"maximum jet rapidity                  (Y_JET_MAX): ",ymax_def
      print *,"maximum parton pseudorapidity           (Y_P_MAX): ",ymax_part
      print *,"minimum jet-jet R-separation            (RJJ_MIN): ",rjjmin
      print *,"minimum number of jets                 (NJET_MIN):  ",njet_min
      print *,"minimum lepton-lepton R-separation      (RLL_MIN): ",rll_min
      print *,"maximum lepton-lepton R-separation      (RLL_MAX): ",rll_max
      print *,"minimum jet-lepton R-separation         (RJL_MIN): ",rjl_min
      print *,"minimum jet-photon R-separation         (RJG_MIN): ",rjg_min
      print *,"minimum lepton-photon R-separation      (RLG_MIN): ",rlg_min   
      print *,"minimum photon-photon R-separation      (RGG_MIN): ",rgg_min
      print *,"maximum photon-photon R-separation      (RGG_MAX): ",rgg_max
      print *,"maximum lepton rapidity                 (Y_L_MAX): ",ymax_l
      print *,"minimum lepton pt                      (PT_L_MIN): ",ptlmin
      print *,"maximum photon rapidity                 (Y_G_MAX): ",ymax_g
      print *,"minimum photon pt                      (PT_G_MIN): ",ptgmin     
      print *,"minimum jet rapidity separation       (ETAJJ_MIN): ",etajjmin
      print *,"tagging jets in opposite hemispheres      (YSIGN):  ",ysign
      print *,"leptons fall inside rapidity gap      (LRAPIDGAP):  ",lrapidgap
      print *,"min leptons y-dist from tagging jets    (DELY_JL): ",dely_jlep
      print *,"photons fall inside rapidity gap      (GRAPIDGAP):  ",grapidgap
      print *,"min photons y-dist from tagging jets    (DELY_JG): ",dely_jphot
      print *,"dijet min mass cut on tag jets         (MDIJ_MIN): ",mdijmin
      print *,"dijet min mass cut on tag jets         (MDIJ_MAX): ",mdijmax
      print *,"veto criteria for jets                    (JVETO):  ",lveto
      print *,"minimun veto-tag y-dist              (DELY_JVETO): ",dely_jveto
      print *,"maximum |y| for veto jet              (YMAX_VETO): ",ymax_veto
      print *,"minimum pT for veto jet              (PTMIN_VETO): ",ptmin_veto
      print *,"minimal invariant dilepton mass       (MLL_MIN): ",mllmin
      print *,""

      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION Cuts(jets, njets, leptons, nl, photons, nphotons, L)
c*************************************************************************
c     This is the general cut routine. It is intended to be the interface
c     to any basic, specialized or user-defined cut routines. 
c*************************************************************************
c     INPUT 
c     jets : the array with the jet information as it is returned by the 
c            jetdefinition routine.
c     v    : 4-momenta of the particles, which are no jets. 
c     L    : id for momentum set in jets
c
c     OUTPUT
c     Cuts   : .true. or .false. depending on whether the event passes the cuts.
c*************************************************************************

      IMPLICIT NONE
#include "global.inc"
#include "process.inc"

c     input variables
      real*8 jets(0:7,max_jets,max_kin) 
      real*8 leptons(0:8,max_v,max_kin)     
      real*8 photons(0:7,max_v,max_kin)     
      integer L, njets(max_kin), nl(max_kin), nphotons(max_kin)

c     external functions
      logical Basic_Cuts, VBF_Cuts
      external Basic_Cuts, VBF_Cuts

c call of function Basic_Cuts for all processes
      Cuts = Basic_Cuts( jets(0,1,L), njets(L), leptons(0,1,L), nl(L),
     1                                   photons(0,1,1), nphotons(1))

c call of vbf_cuts only for vbf processes
      SELECT CASE(procID)
      CASE(Hjj, Hjj_AA, Hjj_mu, Hjj_tau, Hjj_bbar, Hjj_WW,
     1     Hjj_ZZ_ll, Hjj_ZZ_lnu, Zjj_l, Zjj_nu, WPjj, WMjj,
     2     WPWMjj, ZZjj_ll, ZZjj_lnu, WPZjj, WMZjj)
         if (Cuts) then
            Cuts = VBF_Cuts( jets(0,1,L), njets(L), leptons(0,1,L),
     1                         nl(L), photons(0,1,1), nphotons(1))
         endif
      CASE(GFHjj)
         if (Cuts) then
            Cuts = VBF_Cuts( jets(0,1,L), njets(L), leptons(0,1,L),
     1                         nl(L), photons(0,1,1), nphotons(1))
         endif
      CASE(Hjjj, Hjjj_AA, Hjjj_mu,
     1     Hjjj_tau, Hjjj_bbar, Hjjj_WW, Hjjj_ZZ_ll, Hjjj_ZZ_lnu)
         if (Cuts) then
            Cuts = VBF_Cuts( jets(0,1,L), njets(L), leptons(0,1,L),
     1                         nl(L), photons(0,1,1), nphotons(1))
         endif
      END SELECT

      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION BASIC_CUTS(jets, njets, leptons, nl, photons, nphotons)
c*************************************************************************
c     This routine offers vbf specific cuts, that is cuts for processes 
c     with 2 or more jets and 2 or more leptons.
c*************************************************************************
      IMPLICIT NONE
#include "global.inc"
#include "cuts.inc"

c     input variables
      real*8 jets(0:7,max_jets) 
      real*8 leptons(0:8,max_v)
      real*8 photons(0:7,max_v)
      integer njets, nl, nphotons
      real*8 pt,y,phi
      integer i, j, mu, nveto
      real*8 ylepmax,ylepmin,m2,mjj2,mll,yvetomax,yvetomin
      real*8 yphotmax, yphotmin

      logical pass_cuts

      double precision RSEPS
      external RSEPS

      
      pass_cuts = .true.
      pass_cuts = pass_cuts .and. (njets.ge.min_jets)

c   JET CUTS
c=================
c...jet-jet r-separation
      do i=1,njets-1
         do j = i+1, njets
            pass_cuts=pass_cuts.and.
     &      ( RSEPS(jets(6,i),jets(7,i),jets(6,j),jets(7,j)).gt.rjjmin)
         enddo
      enddo


c   LEPTON CUTS
c=================
c...maximum lepton rapidity and minimum lepton pt
      do i=1, nl
         pass_cuts=pass_cuts.and. 
     &   ( (abs(leptons(6,i)).lt.ymax_l).and.(leptons(5,i).gt.ptlmin) )     
      enddo

c...lepton-lepton r-separation min and max 
      do i=1, nl-1
         do j = i+1, nl
            pass_cuts=pass_cuts.and.
     &      ( RSEPS(leptons(6,i),leptons(7,i),leptons(6,j),leptons(7,j)).gt.rll_min
     &  .and. RSEPS(leptons(6,i),leptons(7,i),leptons(6,j),leptons(7,j)).lt.rll_max)
         enddo
      enddo

c... cut on invariant dilepton mass
      do i = 1,nl-1
         do j = i+1,nl
            if(leptons(8,i)*leptons(8,j).lt.0) then !opposite sign leptons only
               mll = mjj2(leptons(0,i),leptons(0,j))
               pass_cuts=pass_cuts.and.(mll.gt.mllmin**2)
            endif
         enddo
      enddo


c   PHOTON CUTS
c=================
c...maximum photon rapidity and minimum photon pt
      do i=1, nphotons
         pass_cuts=pass_cuts.and. 
     &   ( (abs(photons(6,i)).lt.ymax_g).and.(photons(5,i).gt.ptgmin) )     
      enddo

c...photon-photon r-separation min and max
      do i=1, nphotons-1
         do j = i+1, nphotons
            pass_cuts=pass_cuts.and.
     &      ( RSEPS(photons(6,i),photons(7,i),photons(6,j),photons(7,j)).gt.rgg_min
     &  .and. RSEPS(photons(6,i),photons(7,i),photons(6,j),photons(7,j)).lt.rgg_max)
         enddo
      enddo


c   ADDITIONAL R-SEPARATION CUTS
c=================================
c...jet-lepton r-separation
      do i=1, nl
         do j=1, njets
            pass_cuts=pass_cuts.and.
     &      ( RSEPS(leptons(6,i),leptons(7,i),jets(6,j),jets(7,j)).gt.rjl_min )
         enddo
      enddo

c...jet-photon r-separation
      do i=1, nphotons
         do j=1, njets
            pass_cuts=pass_cuts.and.
     &      ( RSEPS(photons(6,i),photons(7,i),jets(6,j),jets(7,j)).gt.rjg_min )
         enddo
      enddo

c...lepton-photon r-separation
      do i=1, nphotons
         do j=1, nl
            pass_cuts=pass_cuts.and.
     &      ( RSEPS(photons(6,i),photons(7,i),leptons(6,j),leptons(7,j)).gt.rlg_min )
         enddo
      enddo

      
      BASIC_CUTS = pass_cuts
      
      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION VBF_Cuts(jets, njets, leptons, nl, photons, nphotons)
c*************************************************************************
c     This routine offers vbf specific cuts, that is cuts for processes 
c     with 2 or more jets and 2 or more leptons.
c*************************************************************************
      IMPLICIT NONE
#include "global.inc"
#include "cuts.inc"

c     input variables
      real*8 jets(0:7,max_jets) 
      integer njets, nl, nphotons
      real*8 leptons(0:8,max_v)
      real*8 photons(0:7,max_v)
      real*8 pt,y,phi
      integer i, j, mu, nveto
      real*8 ylepmax,ylepmin,m2,mjj2,mll,yvetomax,yvetomin
      real*8 yphotmax, yphotmin

      logical pass_cuts

      double precision RSEPS
      external RSEPS

      logical Basic_Cuts
      external Basic_cuts

      
      pass_cuts = .true.

c...jet-jet rapidity separation
      if (njets.ge.2) then
        pass_cuts=pass_cuts.and.
     &        (abs(jets(6,1)-jets(6,2)).gt.etajjmin)
      endif

c...jets #1 and #2 must have opposite sign rapidity
      if (ysign) then
         if (njets.ge.2) then
            pass_cuts=pass_cuts.and. ( (jets(6,1)*jets(6,2)).le.0d0 )
         endif
      endif

c...leptons fall inside rapidity gap
      if(lrapidgap) then
         if (njets.ge.2) then
            ylepmax = max(jets(6,1),jets(6,2)) - dely_jlep
            ylepmin = min(jets(6,1),jets(6,2)) + dely_jlep
            do i=1,nl
               pass_cuts=pass_cuts.and.(leptons(6,i).lt.ylepmax.and.
     &              leptons(6,i).gt.ylepmin)      
            enddo
         endif
      endif

c...photons fall inside rapidity gap
      if(grapidgap) then
         if (njets.ge.2) then
            yphotmax = max(jets(6,1),jets(6,2)) - dely_jphot
            yphotmin = min(jets(6,1),jets(6,2)) + dely_jphot
            do i=1,nphotons
               pass_cuts=pass_cuts.and.(photons(6,i).lt.yphotmax.and.
     &              photons(6,i).gt.yphotmin)      
            enddo
         endif
      endif

c...  tagging jets invariant dijet mass cut
      if (njets.ge.2) then
         m2=mjj2(jets(0,1),jets(0,2))
         pass_cuts=pass_cuts.and.(m2.ge.mdijmin**2.and.m2.le.mdijmax**2)
      endif

C... veto jet cut      
      nveto=0
      if(lveto) then
         if(njets.ge.3)then
            yvetomax = max(jets(6,1),jets(6,2)) - dely_jveto
            yvetomin = min(jets(6,1),jets(6,2)) + dely_jveto
            do i=3,njets
               if(jets(6,i).le.yvetomax.and.jets(6,i).ge.yvetomin.and.
     &              abs(jets(6,i)).le.ymax_veto.and.
     &              jets(5,i).gt.ptmin_veto) nveto=nveto+1            
            enddo
         pass_cuts=pass_cuts.and.(nveto.eq.0)
         endif
      endif

      
      VBF_cuts = pass_cuts
      
      RETURN
      END

c*************************************************************************
      SUBROUTINE DefineLeptons(v, Lmax, leptons, n_leptons)
c*************************************************************************
c     Similar to "DefineJets" in jetdefinition.f this routine extracts
c     the leptons out of "v" into "leptons". Only e, mu, tau particles are
c     taken. In addition pt, y and phi information is calculated and stored.
c*************************************************************************
      implicit none
#include "global.inc"
      real*8 v(0:3,max_v,max_kin)
      real*8 leptons(0:8,max_v,max_kin)
      integer n_leptons(max_kin), Lmax
      real*8 pt,y,phi
      integer i, l, mu

      do l=1, Lmax

         do i=1, max_v
            do mu=0,8
               leptons(mu,i,l) = 0d0
            enddo
         enddo

         n_leptons(l)=0
c     get leptons out of decay_particles     
         do i=1, n_v
            if ( (abs(particle_IDs(i)).eq.11) ! check if it is a lepton
     &           .or.(abs(particle_IDs(i)).eq.13)
     &           .or.(abs(particle_IDS(i)).eq.15) ! taus!!
     &           ) then
               n_leptons(l)=n_leptons(l)+1
               do mu = 0,3      ! transfer to "leptons"
                  leptons(mu,n_leptons(l),l) = v(mu,i,l)
               enddo
               leptons(8,n_leptons(l),l) = particle_IDs(i)
            endif 
         enddo
c     calculate rapidity, pt, phi-angle for the leptons
         do i=1, n_leptons(l)
            call lego( leptons(0,i,l), pt, y, phi )
            leptons(4,i,l) = 0.0 ! mass  
            leptons(5,i,l) = pt  ! transverse momentum  
            leptons(6,i,l) = y   ! rapidity 
            leptons(7,i,l) = phi ! azimuthal angle
         end do
         
      enddo

      RETURN
      END
     

c*************************************************************************
      SUBROUTINE DefinePhotons(v, Lmax, photons, n_photons)
c*************************************************************************
c     Similar to "DefineJets" in jetdefinition.f this routine extracts
c     the photons out of "v" into "photons". In addition pt, y and phi 
c     information is calculated and stored.
c*************************************************************************
      implicit none
#include "global.inc"
      real*8 v(0:3,max_v,max_kin)
      real*8 photons(0:7,max_v,max_kin)
      integer n_photons(max_kin), Lmax
      real*8 pt,y,phi
      integer i, l, mu

      do l=1, Lmax

         do i=1, max_v
            do mu=0,7
               photons(mu,i,l) = 0d0
            enddo
         enddo

         n_photons(l)=0
c     get photons out of decay_particles     
         do i=1, n_v
            if ( (abs(particle_IDs(i)).eq.22) ! check if it is a photon
     &           ) then
               n_photons(l)=n_photons(l)+1
               do mu = 0,3      ! transfer to "photons"
                  photons(mu,n_photons(l),l) = v(mu,i,l)
               enddo
            endif 
         enddo
c     calculate rapidity, pt, phi-angle for the leptons
         do i=1, n_photons(l)
            call lego(photons(0,i,l), pt, y, phi )
            photons(4,i,l) = 0.0 ! mass  
            photons(5,i,l) = pt ! transverse momentum  
            photons(6,i,l) = y  ! rapidity 
            photons(7,i,l) = phi ! azimuthal angle
         end do
         
      enddo

      RETURN
      END
     
