****************************************************************************
*  This is an include-file of the VBFNLO main program.
*  It contains declarations of global variables.
c***************************************************************************
c  Author(s) : G. Klaemke
c***************************************************************************
*  Last modified: 15.09.2006
****************************************************************************

c the process identification number
      integer procID

c the number of iterations of the integration
      integer N_iterations1, N_iterations2
c the number of points/shots in the last iteration = 2**N_points
c an array is used because there are several loops (LO, virtual contrib., real emissions)
c -1       : real emissions loop
c  0       : LO loop
c  1,2,... : the various virtual contributions
      integer N_points(-1:5)	

c the center of mass energy of the collider
      double precision ecm	

c phasespace dimensionality - this is process dependend!!

      integer*4 PS_dimension

c length of the monaco random number array. (Must always be larger than PS_dimension.)

      integer max_PS_dim
      parameter (max_PS_dim = 30)

c the maximum number of 4-momenta stored in 'partons' 
c here, the partons are the incoming quarks/gluons + the 2 tagging jets
c + the real emission (for NLO only)

      integer max_p
      parameter (max_p=6)

c the maximum number of additional particles like a higgs boson, or any decay products.
c additional/decay particles means: everything which is not a parton involved in 
c the basic QCD 2->2(+1)+X process.

      integer max_v
      parameter (max_v=10)

c number of internal x parameters
      integer nx
      parameter( nx=2 )	

c the maximum number of jets 
      integer max_jets
      parameter (max_jets=max_p+max_v-nx)

c max number of momentum configurations for real emission processes
      integer max_kin
      parameter (max_kin=25)

c actual number of entries in partons
      integer n_p

c actual number of entries in decay_particles
      integer n_v

c actual number of kinematics
      integer n_kin	

c     The particle identification code according to the Monte Carlo particle numbering scheme.
c     reminder :  e- =  11, nue    =  12, mu- =  13, numu    =  14, tau- =  15, nutau  =  16,
c                 e+ = -11, nuebar = -12, mu+ = -13, numubar = -14, tau+ = -15, nutaub = -16,
c             photon = 22,      Z0 = 23,   W+ = 24,       h0 = 25,    A0 = 36,      H+ = 37 
c     in addition, we identify a general parton (quark/gluon) with id=93
      integer particle_IDs(max_v)
		
c file names for input and output of monaco grid information
      character*50 gridname1, gridname2

c     proton/antiproton beams
      integer sign1, sign2

c     is NLO implemented for the process?
      logical hasNLO	

c     switches for born/virtual contribution/real emission
      logical doBorn, doVirtuals, doEmissions	

c     number of loops for different virtual contributions
      integer NLO_Loops

c put everything in one global common block

      common /cglobal/ecm, procID, N_iterations1, N_iterations2, N_points, PS_dimension,
     &	              n_p, n_v, sign1, sign2,
     &	              particle_IDs,
     &	              hasNLO, doBorn, doVirtuals, doEmissions, NLO_Loops,
     &                gridname1, gridname2
	
      common /cglobal2/ n_kin

c distribute finite terms for Born*virtual+real emission = c*|Born|**2 + ....
c with c = -9/2 - pi^2/3 = creal + cvirt
      double precision creal, cvirt, pi, pi2o3p9h
      parameter (pi = 3.141592653589793d0, pi2o3p9h = pi**2/3d0+9d0/2d0)
c      parameter (creal = 0d0, cvirt = -pi2o3p9h-creal)
      parameter (cvirt = pi**2/3d0-7d0, creal = -pi2o3p9h-cvirt) ! for carlo
c      parameter (cvirt = 0d0, creal = -pi2o3p9h-cvirt)

c and for pp -> H jjj.
      double precision crealH3j, cvirtH3j
      parameter (cvirtH3j = pi**2/3d0-8d0, crealH3j = 2d0*pi**2/3d0-13d0/2d0)

c and for pp -> VV (V)
      double precision crealVV, cvirtVV, pi2o3m4
      parameter (pi2o3m4=2d0*(pi**2/3d0-4d0))
      parameter (cvirtVV = 4/3d0*pi**2-8d0,crealVV = pi2o3m4-cvirtVV)
c      parameter (cvirtVV = 2.0d0,crealVV = pi2o3m4-cvirtVV) !for MCFM-check
c       parameter (cvirtVV = 20.0d0,crealVV = 0.0d0) !for veratest

c fix number of external quark flavors to sum over; for CC processes nfl=4
c is hard-wired; nfl=5 gives the option to count initial and final b-quarks 
c This option only exists for Z and H production!
      integer nfl
      parameter (nfl=4)


c qsqAmin is the minimum virtuality for t-channel photon exchange. 
c Subamplitudes will be set to zero if photon virtuality falls below
c qsqAmin
      double precision qsqAmin
      parameter (qsqAmin=4d0)

c flag for warnings in matrixelement routines
      logical lwarn
      parameter (lwarn = .false.)

      integer igauge
      parameter (igauge=2)
c igauge allows to select different ways to insure EW gauge invariance
c in the face of finite W and Z width effects for EW W and Z production:
c  
c   igauge = 1       overall factor scheme
c            2       complex mass scheme, m**2-->m**2-i*m*Gamma
c                    in all propagators
c            3       for ibos=2 only: naive Z Breit Wigner
c           -1       width = 0, for testing purposes only

      logical with_anom 	
      common /anom_switch/ with_anom
      logical with_anomHiggs
      common /anomHiggs/ with_anomHiggs
      logical with_kk 	
      common /kk_switch/ with_kk
