c*************************************************************************
      SUBROUTINE InitGlobalParameters
c*************************************************************************
c     Initialize the global parameters which are defined in "global.inc"
c*************************************************************************

      implicit none
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/scales.inc"
#include "VBFNLO/utilities/lha.inc"
#include "VBFNLO/utilities/koppln.inc"
#include "VBFNLO/utilities/process.inc"
     
c Assign values to the global parameters here.
c Read in values from 'vbfnlo.dat'.

      doBorn=.true.
      doVirtuals=.false.
      doEmissions=.false.
      
      call loadfile("vbfnlo.dat",.true.)
      print *," "
      print *,"       Information on global parameters"
      print *,"-----------------------------------------------"
      call read_int("PROCESS",procID,100)
      narrowwidth = 0
      call read_logical("LOPROCESS_PLUS_JET",LOplusjet,.false.)
      call proc_assignment(procID)
      if (N_lepton.gt.0) then
         call read_IntList("LEPTONS",N_lepton,finallep,finallep_default)
         call lepton_assignment
         call symmetry_factor
      else if (N_lepton.eq.0) then
         symmetryfactor = 1.0d0
      endif

      call read_int("LO_ITERATIONS",N_iterations1,4)
      call read_int("NLO_ITERATIONS",N_iterations2,4)
      call read_logical("LHA_SWITCH",lha,.false.)
      call read_logical("UNWEIGHTING_SWITCH",unweighting,.true.)
      call read_int("PRENEVUNW",prenevunw,1000)
      call read_logical("TAUMASS",taumass,.false.)
      call read_int("LO_POINTS",N_points(0),15)
      call read_int("NLO_POINTS",N_points(-1),15)
      call read_logical("NLO_SWITCH",doNLO,.false.)
      call read_string("LO_GRID",gridname1,"grid2")
      call read_string("NLO_GRID",gridname2,"grid3")
     
      call read_Real("ECM",ecm,14000d0)
      call read_int("BEAM1",sign1,1)
      call read_int("BEAM2",sign2,1)

      call read_int("ID_MUF",id_muf,0)
      call read_int("ID_MUR",id_mur,0)
      call read_Real("MUF_USER",muf_user,100d0)
      call read_Real("MUR_USER",mur_user,100d0)
      call read_Real("XIF",xif,1d0)
      call read_Real("XIR",xir,1d0)

      if (LOplusjet) then
         doNLO = .false.
      endif

#ifdef WITH_NLO
      if (doNLO.and.(.not.LOplusjet)) then
         doVirtuals=.true.
         doEmissions=.true.
      endif
#else
      print *," "
      print *," NLO features are disabled. "
      print *," Processes are available only at LO level."
      if (doNLO.and.(.not.LOplusjet)) then
         print *," "
         print *," Error: selected NLO process is not available."
         stop
      endif
#endif

      call closefile

      RETURN
      END


c*************************************************************************
      SUBROUTINE InitGlobalParameters_GGFLO
c*************************************************************************
c     Initialize the global parameters which are defined in "global.inc"
c*************************************************************************

      implicit none
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/scales.inc"
#include "VBFNLO/utilities/lha.inc"
#include "VBFNLO/utilities/koppln.inc"
#include "VBFNLO/utilities/process.inc"
     
c Assign values to the global parameters here.
c Read in values from 'vbfnlo.dat'.

      doBorn=.true.
      doVirtuals=.false.
      doEmissions=.false.
      
c set procID to 4100 for GGFLO
      procID = 4100

      call loadfile("vbfnlo.dat",.true.)
      print *," "
      print *,"       Information on global parameters"
      print *,"-----------------------------------------------"
      print*,"  PROCESS               = ",procID
      narrowwidth = 0
      call read_logical("LOPROCESS_PLUS_JET",LOplusjet,.false.)
      call proc_assignment(procID)
      if (N_lepton.gt.0) then
         call read_IntList("LEPTONS",N_lepton,finallep,finallep_default)
         call lepton_assignment
         call symmetry_factor
      else if (N_lepton.eq.0) then
         symmetryfactor = 1.0d0
      endif

      call read_int("LO_ITERATIONS",N_iterations1,4)
      call read_int("NLO_ITERATIONS",N_iterations2,4)
      call read_logical("LHA_SWITCH",lha,.false.)
      call read_logical("UNWEIGHTING_SWITCH",unweighting,.true.)
      call read_int("PRENEVUNW",prenevunw,1000)
      call read_logical("TAUMASS",taumass,.false.)
      call read_int("LO_POINTS",N_points(0),15)
      call read_int("NLO_POINTS",N_points(-1),15)
      call read_logical("NLO_SWITCH",doNLO,.false.)
      call read_string("LO_GRID",gridname1,"grid2")
      call read_string("NLO_GRID",gridname2,"grid3")
     
      call read_Real("ECM",ecm,14000d0)
      call read_int("BEAM1",sign1,1)
      call read_int("BEAM2",sign2,1)

      call read_int("ID_MUF",id_muf,0)
      call read_int("ID_MUR",id_mur,0)
      call read_Real("MUF_USER",muf_user,100d0)
      call read_Real("MUR_USER",mur_user,100d0)
      call read_Real("XIF",xif,1d0)
      call read_Real("XIR",xir,1d0)

      if (LOplusjet) then
         doNLO = .false.
      endif

#ifdef WITH_NLO
      if (doNLO.and.(.not.LOplusjet)) then
         doVirtuals=.true.
         doEmissions=.true.
      endif
#else
      print *," "
      print *," NLO features are disabled. "
      print *," Processes are available only at LO level."
      if (doNLO.and.(.not.LOplusjet)) then
         print *," "
         print *," Error: selected NLO process is not available."
         stop
      endif
#endif

      call closefile

      RETURN
      END


