****************************************************************************
* This file contains the subroutines for the histogram handling.
* There is an internal representation of the histograms, that is filled
* by calls of "FillHist". In the end of the program, this data is transvert
* to other histogram representations (e.g. hbook, top-drawer).
* Each histogram has an ID, which is used for the filling.
c***************************************************************************
c   LIST OF ALL FUNCTIONS AND SUBROUTINES IN THIS FILE:
c
c     SUBROUTINE InitHistograms
c     SUBROUTINE HistogramEvent(weight,dsig,dps, rand, p, x, v, jets, n_jets, NLO)
c     SUBROUTINE FillHist(ID, value, weight)
c     SUBROUTINE FillHist2d(ID, X, Y, weight, NLO)
c     SUBROUTINE CreateHist(ID, title, bins, min, max)
c     SUBROUTINE Create2dHist(ID, title, xlabel, ylabel, xbins, ybins, xmin, xmax, ymin, ymax)
c     SUBROUTINE WriteHistograms
c     SUBROUTINE WriteRootHists
c     SUBROUTINE WritePawHists
c     SUBROUTINE WriteTopHists
c     SUBROUTINE WriteGnuHists
c     SUBROUTINE SelectFile
c
c***************************************************************************
c
c      to add a histogram
c     --------------------
c     1: put a "CreateHist" in "InitHistograms". Use unique IDs for each histogram
c     2: put a proper "FillHist" in "HistogramEvents"
c
c***************************************************************************

c*************************************************************************
      SUBROUTINE InitHistograms
c*************************************************************************
c     Set up histograms
c*************************************************************************
      implicit none
#include "hist.inc"
#include "global.inc"
#include "lha.inc"
      integer id, j, k
      real*8 hwid
      character*5 ext

c
c reset histogram contents to zero     
c
      do id = 1,HISTMAX
         nbin(ID) = 100
         hlo(ID) = 0d0
         hup(ID) = 1d0
         htitles(ID) = ""
         do j = 1,BINMAX
            sum(id,j) = 0d0
            square(id,j) = 0d0
            count(id,j) = 0
         enddo
         hstored(ID) = .false.
      enddo

c
c do the same for the two-dimensional histograms
c
      do id = 1, HIST2dMAX
         nbinX(ID) = 100
         nbinY(ID) = 100
         hloX(ID) = 0d0
         hloY(ID) = 0d0
         hupX(ID) = 1d0
         hupY(ID) = 1d0
         h2dtitles(ID) = ""
         xlabels(ID) = ""
         ylabels(ID) = ""
         do j = 1, BINMAXX
            do k = 1, BINMAXY
               sum2d(id,j,k) = 0d0
               square2d(id,j,k) = 0d0
               count2d(id,j,k) = 0
            enddo
         enddo
         h2dstored(ID) = .false.
      enddo

***********************************************************************
***********************************************************************
**                        **
**    HISTOGRAM SETUP     **
**                        **
****************************
****************************


*     call CreateHist(ID, title, #bins, min, max)
      call CreateHist(29, "dS/dpT_j (fb/GeV)",100,0d0,250d0)    !pT of tagging jets
      call CreateHist(30, "dS/dpTmax_j (fb/GeV)",100,0d0,500d0) !pT of tagging jet with higher pT
      call CreateHist(31, "dS/dpTmin_j (fb/GeV)",100,0d0,200d0) !pT of tagging jet with lower pT
      call CreateHist(32, "dS/deta_j (fb)",100,-5d0,5d0)        !y of tagging jets
      call CreateHist(33, "dS/deta_j1 (fb)",100,-5d0,5d0)       !y of tagging jet with higher pT
      call CreateHist(34, "dS/deta_j2 (fb)",100,-5d0,5d0)       !y of tagging jet with lower pT
      call CreateHist(35, "dS/dpTmax_l (fb)",100,0d0,500d0)     !pT_max of all leptons
      call CreateHist(36, "dS/dpTmin_l (fb)",100,0d0,200d0)     !pT_min of all leptons
      call CreateHist(37, "dS/d|eta|max_l (fb/GeV)",100,0d0,5d0)  !|eta|_max of all leptons
      call CreateHist(38, "dS/d|eta|min_l (fb/GeV)",100,0d0,5d0)  !|eta|_min of all leptons

c     example for 2d-hist
      call Create2dHist(10, "d2S/deta_jj dm_jj", "eta_jj", "m_jj", 25, 25, 0d0, 6d0, 0d0, 800d0)

* select the histograms which are stored in a file
      hstored(29) = .true.  
      hstored(30) = .true.
      hstored(31) = .true.
      hstored(32) = .true.
      hstored(33) = .true.
      hstored(34) = .true.
      hstored(35) = .true.
      hstored(36) = .true.
      hstored(37) = .true.
      hstored(38) = .true.

      h2dstored(10) = .true.

***********************************************************************

c     
c select also the NLO histograms to be stored in a file    
c
      if (hasNLO.and.doNLO) then
         do id = 1, histmax/2
            hstored(id+NLOoffset)=hstored(id)
         enddo

         do id = 1, hist2dmax/2
            h2dstored(id+NLOoffset2d)=h2dstored(id)
         enddo
 

      endif
c   
c calculate inverse bin widths for each histogram
c
      do id = 1, histmax
         hwid = hup(id) - hlo(id)
         
         if ( nbin(id).gt.0 .and. hwid.ne.0d0 ) then
            rwid(id) = real( nbin(id) ) / hwid
            dbin(id) = 1/rwid(id)
         else
            dbin(id) = 0d0
            rwid(id) = 0d0
         end if
      end do
c
c the same for the 2d histograms
c
      do id = 1, hist2dmax
         hwid = hupX(id) - hloX(id)
         if ( nbinX(id).gt.0 .and. hwid.ne.0d0 ) then
            rwidX(id) = real( nbinX(id) ) / hwid
            dbinX(id) = 1/rwidX(id)
         else
            dbinX(id) = 0d0
            rwidX(id) = 0d0
         endif
         hwid = hupY(id) - hloY(id)
         if ( nbinY(id).gt.0 .and. hwid.ne.0d0 ) then
            rwidY(id) = real( nbinY(id) ) / hwid
            dbinY(id) = 1/rwidY(id)
         else
            dbinY(id) = 0d0
            rwidY(id) = 0d0
         endif         
      end do

c
c read input file from "vbfnlo.dat"
c
      call loadfile("vbfnlo.dat",.false.)
      call read_logical("ROOT",writeROOT,.false.)
*      call read_logical("HBOOK",writePAW,.false.)
      call read_logical("TOP",writeTOP,.false.)
      call read_logical("GNU",writeGNU,.false.)
      call read_logical("REPLACE",replaceFiles,.false.)
      call read_string("ROOTFILE",rootfile,"histograms")
*      call read_string("PAWFILE",PAWfile,"histograms")
      call read_string("TOPFILE",TOPfile,"histograms")
      call read_string("GNUFILE",GNUfile,"histograms")
      call closefile
      if (.not.replacefiles) then
         ext = ".root"
         call selectfile(rootfile,ext)
*         ext = ".paw"
*         call selectfile(pawfile,ext)
         ext = ".top"
         call selectfile(topfile,ext)
         ext = ".gp"
         call selectfile(gnufile,ext)
      else
        rootfile= rootfile(1:len_trim(rootfile))//".root"
*        pawfile= pawfile(1:index(pawfile," ")-1)//".paw"
        topfile= topfile(1:len_trim(topfile))//".top"        
        gnufile= gnufile(1:len_trim(gnufile))//".gp"        
      endif
     
      RETURN
      END


c*************************************************************************
      SUBROUTINE HistogramEvent(weight,dsig,dps, rand, p, x, v,
     &                           jets, n_jets, leptons, nl, photons, nphotons, NLO)
c*************************************************************************
c     Fill histograms
c*************************************************************************
      implicit none
#include "global.inc"
      real*8 weight, dsig, dps
      integer NLO 
      real*8 p(0:3,max_p), v(0:3,max_v), x(nx),inv(0:3)
      real*8 rand(max_PS_dim)
      real*8 jets(0:7,max_jets), leptons(0:8,max_v), photons(0:7,max_v)
      integer n_jets, nl, nphotons,i
      real*8 pTmaxlep, pTminlep, ymaxlep, yminlep

      real*8 dw
      real*8 deltaR, deltaY, deltaPhi, mjj2, mjj,minv
      
      double precision RSEPS
      
      external RSEPS
      external mjj2

      if ((NLO.ne.0).and.(NLO.ne.1)) return
      
      if (dsig.eq.0d0) return

      dw = dsig * weight

**********************
* pT of tagging jets *
**********************
      if (n_jets.ge.2) then
         call FillHist(29, jets(5,1), dw*0.5d0, NLO)
         call FillHist(29, jets(5,2), dw*0.5d0, NLO)
         call FillHist(30, jets(5,1), dw, NLO)
         call FillHist(31, jets(5,2), dw, NLO)
      endif

***********************
* eta of tagging jets *
***********************
      if (n_jets.ge.2) then
         call FillHist(32, jets(6,1), dw*0.5d0, NLO)
         call FillHist(32, jets(6,2), dw*0.5d0, NLO)
         call FillHist(33, jets(6,1), dw, NLO)
         call FillHist(34, jets(6,2), dw, NLO)
      endif

**********************
* pT of leptons      *
**********************
      pTmaxlep = 0
      do i = 1, nl
         pTmaxlep = max(pTmaxlep,leptons(5,i))
      enddo
      pTminlep = 1.0d6
      do i = 1, nl
         pTminlep = min(pTminlep,leptons(5,i))
      enddo
      if (nl.ge.2) then
         call FillHist(35, pTmaxlep, dw, NLO)
         call FillHist(36, pTminlep, dw, NLO)
      endif

*************************
* |eta| of leptons      *
*************************
      ymaxlep = 0
      do i = 1, nl
         ymaxlep = max(ymaxlep,abs(leptons(6,i)))
      enddo
      yminlep = 1.0d6
      do i = 1, nl
         yminlep = min(yminlep,abs(leptons(6,i)))
      enddo
      if (nl.ge.2) then
         call FillHist(37, ymaxlep, dw, NLO)
         call FillHist(38, yminlep, dw, NLO)
      endif
      
****************************
* example for 2d-histogram *
****************************
      if (n_jets.ge.2) then
         deltay=abs(jets(6,1)-jets(6,2))
         mjj=sqrt(mjj2(jets(0,1),jets(0,2)))
         call FillHist2d(10,deltay,mjj, dw, NLO)
      endif

      RETURN
      END


c*************************************************************************
      SUBROUTINE FillHist(ID, value, weight, NLO)
c*************************************************************************
c     Fill value in histogram. 
c     Smear over 2 bins if value is within fractio drb of bin boundary
c*************************************************************************
      implicit none
#include "hist.inc"
      Integer ID, NLO
      real*8 value, weight
      Integer hist_in, BIN, binl, binh
      real*8 X,Y 
      real*8 xbin, yl, yh
      real*8 drb, tdrbi, rat
      parameter (drb=0.49d0, tdrbi = 0.5d0/drb)

      hist_in = ID
      if (NLO.eq.1) hist_in=hist_in+NLOoffset
      x = value
      y = weight*rwid(hist_in)
      
      xbin = (X-HLO(hist_in)) * RWID(hist_in)
      if (xbin.gt.drb) then
         bin = min( INT(xbin), nbin(hist_in) ) 
         if ( (xbin-bin).lt.drb ) then ! x in low end of bin
            binl = bin+1
            binh = bin+2
            rat = (xbin-bin)*tdrbi + 0.5
            yh = y*rat
            yl = y-yh
            bin = 0
         elseif( (xbin-bin).lt.(1.0-drb) ) then ! x in main part of bin
            bin = bin+2
C            elseif( (xbin-bin).lt.1.0 ) then   ! x in high part of bin
         elseif( (xbin-bin).lt.1.0.and.xbin.lt.nbin(hist_in) ) then ! x in high part of bin
            binl = bin+2
            binh = bin+3
            rat = (xbin-bin-1.0)*tdrbi + 0.5
            yh = y*rat
            yl = y-yh
            bin = 0
         else                   ! x more than drp above upper bin
            bin = nbin(hist_in)+2
         endif
      elseif(xbin.lt.-drb) then
         bin = 1
      else                      ! xbin in [-drb,drb]
         binl = 1
         binh = 2
         rat = xbin * tdrbi + 0.5 ! in [0,1]
         yh = y*rat
         yl = y-yh
         bin = 0
      endif
      if (hist_in.eq.-1) then
         print*," "
         print*," input to hf1dp: x,y = ",value,weight
         print*," bin = ",bin
         if (bin.eq.0) then
            print*," bin_or. = ",min( INT(xbin), nbin(hist_in) ),
     1           " binl,h = ",binl,binh
            print*," xbin = ",xbin," rat = ",rat
            print*," yl = ",yl," yh = ",yh
         endif
      endif
      if (bin.eq.0) then
         SUM(HIST_IN,BINL) = SUM(HIST_IN,BINL) + YL
         SQUARE(HIST_in,BINL) = SQUARE(HIST_in,BINL) + YL*YL
         COUNT(HIST_in,BINL) = COUNT(HIST_in,BINL) + rat
         SUM(HIST_IN,BINH) = SUM(HIST_IN,BINH) + YH
         SQUARE(HIST_in,BINH) = SQUARE(HIST_in,BINH) + YH*YH
         COUNT(HIST_in,BINH) = COUNT(HIST_in,BINH) + 1d0-rat
      else
         SUM(HIST_IN,BIN) = SUM(HIST_IN,BIN) + Y
         SQUARE(HIST_in,BIN) = SQUARE(HIST_in,BIN) + Y*Y
         COUNT(HIST_in,BIN) = COUNT(HIST_in,BIN) + 1
      endif
      
      RETURN
      END


c*************************************************************************
      SUBROUTINE FillHist2d(ID, X, Y, weight, NLO)
c*************************************************************************
c     Fill value in 2d-histogram. 
c     Smear over bins if value is within fractio drb of bin boundary
c*************************************************************************
      implicit none
#include "hist.inc"
      Integer ID, NLO
      real*8 X, Y, weight
      Integer hist_in, BINX, binlx, binhx, BINY, binly, binhy
      real*8 xb, yb, wll, whl, wlh, whh, w, wh, wl, w_h, w_l
      real*8 drb, tdrbi, ratx, raty
      parameter (drb=0.49d0, tdrbi = 0.5d0/drb)

      hist_in = ID
      if (NLO.eq.1) hist_in=hist_in+NLOoffset2d
      w=weight*rwidX(hist_in)*rwidY(hist_in)
      xb = (X-HLOX(hist_in)) * RWIDX(hist_in)
      yb = (Y-HLOY(hist_in)) * RWIDY(hist_in)

c     X-direction
      if (xb.gt.drb) then
         binx = min( INT(xb), nbinX(hist_in) ) 
         if ( (xb-binx).lt.drb ) then ! x in low end of bin    
            binlx = binx+1
            binhx = binx+2
            ratx = (xb-binx)*tdrbi + 0.5
            wh = w*ratx
            wl = w-wh
            binx = 0
         elseif( (xb-binx).lt.(1.0-drb) ) then ! x in main part of bin
            binx = binx+2
         elseif( (xb-binx).lt.1.0.and.xb.lt.nbinX(hist_in) ) then ! x in high part of bin
            binlx = binx+2
            binhx = binx+3
            ratx = (xb-binx-1.0)*tdrbi + 0.5
            wh = w*ratx
            wl = w-wh
            binx = 0
         else                   ! x more than drp above upper bin
            binx = nbinX(hist_in)+2
         endif
      elseif(xb.lt.-drb) then
         binx = 1
      else
         binlx = 1
         binhx = 2
         ratx = xb * tdrbi + 0.5 ! in [0,1]
         wh = w*ratx
         wl = w-wh
         binx = 0
      endif
c     Y-direction
      if (yb.gt.drb) then
         biny = min( INT(yb), nbinY(hist_in) ) 
         if ( (yb-biny).lt.drb ) then ! y in low end of bin    
            binly = biny+1
            binhy = biny+2
            raty = (yb-biny)*tdrbi + 0.5
            whh = wh*raty
            whl = wh-whh
            wlh = wl*raty
            wll = wl-wlh
            w_h = w*raty
            w_l = w-w_h
            biny = 0
         elseif( (yb-biny).lt.(1.0-drb) ) then ! y in main part of bin
            biny = biny+2
         elseif( (yb-biny).lt.1.0.and.yb.lt.nbinY(hist_in) ) then ! y in high part of bin
            binly = biny+2
            binhy = biny+3
            raty = (yb-biny-1.0)*tdrbi + 0.5
            whh = wh*raty
            whl = wh-whh
            wlh = wl*raty
            wll = wl-wlh
            w_h = w*raty
            w_l = w-w_h
            biny = 0
         else                   ! y more than drp above upper bin
            biny = nbinY(hist_in)+2
         endif
      elseif(yb.lt.-drb) then
         biny = 1
      else
         binly = 1
         binhy = 2
         raty = yb * tdrbi + 0.5 ! in [0,1]
         whh = wh*raty
         whl = wh-whh
         wlh = wl*raty
         wll = wl-wlh
         w_h = w*raty
         w_l = w-w_h
         biny = 0
      endif
      if (binx.eq.0) then
         if (biny.eq.0) then
            SUM2d(HIST_IN,BINLX,BINLY) = SUM2d(HIST_IN,BINLX,BINLY) + WLL
            SQUARE2d(HIST_in,BINLX,BINLY) = SQUARE2d(HIST_in,BINLX,BINLY) + WLL*WLL
            COUNT2d(HIST_in,BINLX,BINLY) = COUNT2d(HIST_in,BINLX,BINLY) + ratx*raty

            SUM2d(HIST_IN,BINHX,BINLY) = SUM2d(HIST_IN,BINHX,BINLY) + WHL
            SQUARE2d(HIST_in,BINHX,BINLY) = SQUARE2d(HIST_in,BINHX,BINLY) + WHL*WHL
            COUNT2d(HIST_in,BINHX,BINLY) = COUNT2d(HIST_in,BINHX,BINLY) + (1d0-ratx)*raty

            SUM2d(HIST_IN,BINLX,BINHY) = SUM2d(HIST_IN,BINLX,BINHY) + WLH
            SQUARE2d(HIST_in,BINLX,BINHY) = SQUARE2d(HIST_in,BINLX,BINHY) + WLH*WLH
            COUNT2d(HIST_in,BINLX,BINHY) = COUNT2d(HIST_in,BINLX,BINHY) + ratx*(1d0-raty)

            SUM2d(HIST_IN,BINHX,BINHY) = SUM2d(HIST_IN,BINHX,BINHY) + WHH
            SQUARE2d(HIST_in,BINHX,BINHY) = SQUARE2d(HIST_in,BINHX,BINHY) + WHH*WHH
            COUNT2d(HIST_in,BINHX,BINHY) = COUNT2d(HIST_in,BINHX,BINHY) + (1d0-ratx)*(1d0-raty)
         else
            SUM2d(HIST_IN,BINLX,BINY) = SUM2d(HIST_IN,BINLX,BINY) + WL
            SQUARE2d(HIST_in,BINLX,BINY) = SQUARE2d(HIST_in,BINLX,BINY) + WL*WL
            COUNT2d(HIST_in,BINLX,BINY) = COUNT2d(HIST_in,BINLX,BINY) + ratx

            SUM2d(HIST_IN,BINHX,BINY) = SUM2d(HIST_IN,BINHX,BINY) + WH
            SQUARE2d(HIST_in,BINHX,BINY) = SQUARE2d(HIST_in,BINHX,BINY) + WH*WH
            COUNT2d(HIST_in,BINHX,BINY) = COUNT2d(HIST_in,BINHX,BINY) + (1d0-ratx)
         endif
      else
         if (biny.eq.0) then
            SUM2d(HIST_IN,BINX,BINLY) = SUM2d(HIST_IN,BINX,BINLY) + W_L
            SQUARE2d(HIST_in,BINX,BINLY) = SQUARE2d(HIST_in,BINX,BINLY) + W_L*W_L
            COUNT2d(HIST_in,BINX,BINLY) = COUNT2d(HIST_in,BINX,BINLY) + raty

            SUM2d(HIST_IN,BINX,BINHY) = SUM2d(HIST_IN,BINX,BINHY) + W_H
            SQUARE2d(HIST_in,BINX,BINHY) = SQUARE2d(HIST_in,BINX,BINHY) + W_H*W_H
            COUNT2d(HIST_in,BINX,BINHY) = COUNT2d(HIST_in,BINX,BINHY) + (1d0-raty)
         else
            SUM2d(HIST_IN,BINX,BINY) = SUM2d(HIST_IN,BINX,BINY) + W
            SQUARE2d(HIST_in,BINX,BINY) = SQUARE2d(HIST_in,BINX,BINY) + W*W
            COUNT2d(HIST_in,BINX,BINY) = COUNT2d(HIST_in,BINX,BINY) + 1
         endif
      endif

      RETURN
      END

c*************************************************************************
      SUBROUTINE CreateHist(ID, title, bins, min, max)
c*************************************************************************
c     Set up the histogram for the internal representation.
c*************************************************************************
      implicit none
#include "hist.inc"
      integer ID
      character*(*) title
      integer bins
      real*8 min, max

      nbin(ID) = bins
      hlo(ID) = min
      hup(ID) = max
      htitles(ID) = title

      nbin(ID+NLOoffset) = bins
      hlo(ID+NLOoffset) = min
      hup(ID+NLOoffset) = max
c      htitles(ID+NLOoffset) = title(1:index(title,"  ")-1)//" (NLO)"
      write(htitles(ID+NLOoffset),*) title," (NLO)"
      htitles(ID+NLOoffset) = htitles(ID+NLOoffset)(1:len_trim(htitles(ID+NLOoffset)))
     
      RETURN
      END

c*************************************************************************
      SUBROUTINE Create2dHist(ID, title, xlabel, ylabel, xbins, ybins, xmin, xmax, ymin, ymax)
c*************************************************************************
c     Set up a 2d-histogram for the internal representation.
c*************************************************************************
      implicit none
#include "hist.inc"
      integer ID
      character*(*) title, xlabel, ylabel
      integer xbins, ybins
      real*8 xmin, xmax, ymin, ymax

      nbinX(ID) = xbins
      nbinY(ID) = ybins
      hloX(ID) = xmin
      hupX(ID) = xmax
      hloY(ID) = ymin
      hupY(ID) = ymax      
      h2dtitles(ID) = title
      xlabels(ID) = xlabel
      ylabels(ID) = ylabel

      nbinX(ID+NLOoffset2d) = xbins
      hloX(ID+NLOoffset2d) = xmin
      hupX(ID+NLOoffset2d) = xmax
      nbinY(ID+NLOoffset2d) = ybins
      hloY(ID+NLOoffset2d) = ymin
      hupY(ID+NLOoffset2d) = ymax      
      write(h2dtitles(ID+NLOoffset2d),*) title," (NLO)"
      h2dtitles(ID+NLOoffset2d) = h2dtitles(ID+NLOoffset2d)(1:len_trim(h2dtitles(ID+NLOoffset2d)))
     
      RETURN
      END



c*************************************************************************
      SUBROUTINE WriteHistograms
c*************************************************************************
c     write out histograms in various formats
c*************************************************************************
      implicit none
#include "hist.inc"      
#ifdef WITH_ROOT
      if (writeRoot) call WriteRootHists
#endif

* #ifdef WITH_CERN
*      if (writePaw) call WritePawHists
* #endif

      if (writeTop) call WriteTopHists

      if (writegnu) call WriteGnuHists

      RETURN
      END

#ifdef WITH_ROOT
c*************************************************************************
      SUBROUTINE WriteRootHists
c*************************************************************************
c     Transfers the histogram data to the Root format
c*************************************************************************
      implicit none
#include "hist.inc"
      integer ID,histdp,bin
      double precision x,y
      
      call openrootsession(rootfile)
      DO ID = 1,HISTMAX
         IF (hstored(ID)) THEN
           
            call createroothisto(ID, htitles(ID),nbin(ID), hlo(ID), hup(ID))
            histdp = ID 
            do bin = 1,nbin(ID)+2
               if (count(histdp,bin).gt.1d-3) then
                  x = hlo(ID) + dbin(ID)*(bin-1.5d0)
                  y = sum(histdp,bin)
                  call fillroothisto(ID, x, y)
                  
               endif
            enddo
         ENDIF
      ENDDO

      call closerootsession
      print *," created root file for histograms : ", rootfile
      
      RETURN
      END
#endif

c$$$#ifdef WITH_CERN
c$$$c*************************************************************************
c$$$      SUBROUTINE WritePawHists
c$$$c*************************************************************************
c$$$c     Transfers the histogram data to the HBOOK format
c$$$c*************************************************************************
c$$$      implicit none
c$$$#include "hist.inc"
c$$$      integer ID,histdp,bin
c$$$      real*4 x,y, lo, up
c$$$      integer  nhmem
c$$$      parameter( nhmem=262144 )
c$$$      real  hmem(nhmem)
c$$$      common /pawc/ hmem
c$$$
c$$$      call hlimit( nhmem )
c$$$
c$$$      DO ID = 1,HISTMAX
c$$$         IF (hstored(ID)) THEN
c$$$            lo = hlo(id)
c$$$            up = hup(id)
c$$$            call hbook1( ID,htitles(ID), nbin(id), lo, up, 0.0)
c$$$            call hidopt( ID, "inte" )
c$$$            histdp = ID 
c$$$            do bin = 1,nbin(ID)+2
c$$$               if (count(histdp,bin).gt.1d-3) then
c$$$                  x = lo + dbin(ID)*(real(bin)-1.5)
c$$$                  y = sum(histdp,bin)
c$$$                  call hf1(ID,x,y)
c$$$               endif
c$$$            enddo
c$$$         ENDIF
c$$$      ENDDO
c$$$
c$$$      call hrput( 0, pawfile, "N" )
c$$$       print *," created paw file for histograms : ", pawfile
c$$$       
c$$$
c$$$      RETURN
c$$$      END
c$$$#endif      


c*************************************************************************
      SUBROUTINE WriteTopHists
c*************************************************************************
c     Transfers the histogram data to the top-drawer format
c*************************************************************************
      implicit none
#include "hist.inc"
      integer ID,histdp,bin, ibin, i1
      double precision x,y
      double precision delx,xres(0:1000,2), xmin,xmax,ymin,ymax,ymaxK,yminK

      OPEN(UNIT=20,FILE=topfile,STATUS="REPLACE")

      write (20,"(A)") " set size 11 by 8.5 "

      DO ID = 1,HISTMAX
         IF (hstored(ID)) THEN
            write (20,*)
            write (20,"(a)") " new frame "

            write (20,"(A)") " set intensity 4 "
            write (20,"(A)") " set font duplex " 
            write (20,"(A)") " set tick size .05 "
            write (20,"(A)") " set title size 2.0 "
            
            delx = (hup(id)-hlo(id))/(real(nbin(id))+1e-10)
            XRES(0,1) = hlo(id) - DELX/2. ! stores underflow
            XRES(1,1) = hlo(id) + DELX/2.
            DO  IBIN = 2,nbin(id)+1
               XRES(IBIN,1) = XRES(IBIN-1,1) + DELX
            END DO
            xmin = hlo(id)
            xmax = hup(id)
            ymin = 1d33
            ymax = -1d33
            yminK = 0
            ymaxK = 2
            do ibin = 0,nbin(id)+1  !ibin=nbin+1 for overflow
               xres(ibin,2) = sum(id,ibin+1)               
               ymin = min( ymin, xres(ibin,2) )
               ymax = max( ymax, xres(ibin,2) )              
            enddo
            do i1 = 0,nbin(id)+1
               if (xres(i1,2).eq.0.) xres(i1,2) = 1e-20
            end do
            write (20,"(A,i3,a)") " title 2.0 7.5 ""Fig.",
     1           id,": "//htitles(id)
            write (20,"(A)") " set window  x 1.2 to 5.2 y 3 to 7.0 "
            write (20,202) " set limits x ",xmin," to ",xmax,
     1                             " y ",ymin," to ",ymax
 201        format(2(a,f8.2),2(a,f12.2))
 202        format(2(a,f8.2),2(a,f12.5)) 
            write(20,*) " ( "
            DO  IBIN = 1,NBIN(ID)
               WRITE (20,"(2g20.6)")  XRES(ibin,1), xres(ibin,2)
            END DO
            write (20,"(A)") " join solid "
                       
         ENDIF
      ENDDO

      CLOSE(20)

      print *," created top-drawer file for histograms : ", topfile
       

      RETURN
      END


c*************************************************************************
      SUBROUTINE WriteGnuHists
c*************************************************************************
c     Transfers the histogram data to the gnuplot format
c*************************************************************************
      implicit none
#include "hist.inc"
      integer ID,histdp,bin, ibin, i1, jbin
      double precision x,y
      double precision delx,xres(0:1000,2), xmin,xmax,ymin,ymax,ymaxK,yminK
      double precision dely

      OPEN(UNIT=20,FILE=gnufile,STATUS="REPLACE")

      write (20,"(A)") 'set output "'//
     &gnufile(1:len_trim(gnufile)-3)//'.ps"'
      write (20,"(A)") "set terminal postscript color"
      write (20,"(A)") "set style data lines"
      write (20,"(A)") "set key off"

c     the 1d-histograms
      DO ID = 1,HISTMAX
         IF (hstored(ID)) THEN
            write (20,*)
            write (20,*) "#",ID,": ",
     &           htitles(id)(1:len_trim(htitles(id)))
            write (20,"(A)") 'set title "'//
     &           htitles(id)(1:len_trim(htitles(id)))//'"'
            write (20,"(A)") 'plot "-" '
            
            delx = (hup(id)-hlo(id))/(real(nbin(id))+1e-10)
            XRES(0,1) = hlo(id) - DELX/2. ! stores underflow
            XRES(1,1) = hlo(id) + DELX/2.
            DO  IBIN = 2,nbin(id)+1
               XRES(IBIN,1) = XRES(IBIN-1,1) + DELX
            END DO
            xmin = hlo(id)
            xmax = hup(id)
            ymin = 1d33
            ymax = -1d33
            yminK = 0
            ymaxK = 2
            do ibin = 0,nbin(id)+1  !ibin=nbin+1 for overflow
               xres(ibin,2) = sum(id,ibin+1)               
               ymin = min( ymin, xres(ibin,2) )
               ymax = max( ymax, xres(ibin,2) )              
            enddo
            do i1 = 0,nbin(id)+1
               if (xres(i1,2).eq.0.) xres(i1,2) = 1e-20
            end do

            DO  IBIN = 1,NBIN(ID)
               WRITE (20,"(2g20.6)")  XRES(ibin,1), xres(ibin,2)
            END DO
            
            write (20,"(A)") "e"
          ENDIF
      ENDDO

c     the 2d-histograms
      DO ID = 1, HIST2dMAX
         IF (h2dstored(ID)) THEN
            write (20,*)
            write (20,*) "#",ID,": ",
     &           h2dtitles(id)(1:len_trim(h2dtitles(id)))
            write (20,"(A)") "set view map"
            write (20,"(A)") "set size 3.9/5.0,3.4/3.0"
            write (20,"(A)") "unset surface"
            write (20,"(A)") "set style data pm3d"
            write (20,"(A)") "set style function pm3d"
            write (20,"(A)") "set pm3d implicit at b"            
            write (20,"(A)") 'set title "'//
     &           h2dtitles(id)(1:len_trim(h2dtitles(id)))//'"'
            write (20,"(A)") 'set xlabel "'//
     &           xlabels(id)(1:len_trim(xlabels(id)))//'"'
            write (20,"(A)") 'set ylabel "'//
     &           ylabels(id)(1:len_trim(ylabels(id)))//'"'
c            write (20,"(A)") "set palette rgbformulae 7, 5, 15 negative"
            write (20,"(A)") "set palette rgbformulae 30,31,32 negative"

            write (20,"(A)") 'splot "-" '
            
            delx = (hupx(id)-hlox(id))/(real(nbinx(id))+1e-10)
            dely = (hupy(id)-hloy(id))/(real(nbiny(id))+1e-10)
            
            DO IBIN = 1, NBINX(ID)
               DO JBIN = 1, NBINY(ID)
                   WRITE (20,"(3g20.6)")  hlox(id)+delx*(IBIN-0.5d0), 
     &                    hloy(id)+dely*(JBIN-0.5d0), sum2d(id,ibin,jbin)

               ENDDO
               write (20,"(A)") " "
            ENDDO
            
            write (20,"(A)") "e"
         ENDIF
      ENDDO

      CLOSE(20)
      print *," created GNUplot file for histograms : ", gnufile
      
      RETURN
      END



ccccccccccccccccccccc begin subroutine SelectFile ccccccccccccccc
c
c  This subroutine selects a file name which does not exist yet
c  Terrance Figy <terrance@pheno.physics.wisc.edu
c  August 17, 2001  
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine SelectFile(name,extens)
      implicit none
      character*50 name,lname,lname1
      character*1 digit(0:9)
      character*5 extens
      integer ii,i,j
      logical FileAlreadyExists
      data DIGIT /"0","1","2","3","4","5","6","7","8","9"/
 
      lname1 = name
      do 20 ii = 1,99
     
         if((ii.ge.1).and.(ii.le.9)) then
            
            lname = 
     &           lname1(1:len_trim(lname1))//"."//DIGIT(ii)//extens
            inquire(file=lname,exist=FileAlreadyExists)
            if(FileAlreadyExists) then
               goto 20
            else
               name = lname(1:index(lname," ")-1)
               return
            endif
         elseif(ii.gt.9) then
            i = (ii/10)
            j = mod(ii,10)
            lname = lname1(1:len_trim(lname1))//"."//
     &              DIGIT(i)//DIGIT(j)//extens
            inquire(file=lname,exist=FileAlreadyExists)
            if(FileAlreadyExists) then
               goto 20
            else
               name = lname(1:len_trim(lname))
               return
            endif
         endif

         name = name(1:len_trim(name))
         
 20      continue
         write(*,*) "-------- fatal crash in SelectFile ----------"
         stop
         end
