
C*********************************************************************

C...  LHAGLUE Interface to LHAPDF library of modern parton
C...   density functions (PDF) with uncertainties
C...
C...Authors for v3: Dimitri Bourilkov, Craig Group, Mike Whalley
C...
C...Author for v1 and v2: Dimitri Bourilkov  bourilkov@mailaps.org
C...                      University of Florida
C...
C...HERWIG interface by Dimitri Bourilkov and Craig Group
C...
C...New numbering scheme and upgrade for LHAPDF v2.1
C...by Dimitri Bourilkov and Mike Whalley
C...
C...For more information, or when you cite this interface, currently
C...the official reference is:
C...D.Bourilkov, "Study of Parton Density Function Uncertainties with
C...LHAPDF and PYTHIA at LHC", hep-ph/0305126.
C...
C...The official LHAPDF page is:
C...
C...   http://durpdg.dur.ac.uk/lhapdf/index.html 
C...
C...The interface contains two subroutines (similar to PDFLIB).
C...It can be used seamlessly by Monte Carlo generators 
C...interfaced to PDFLIB or in stand-alone mode.
C...
C...    For initialization (called once)
C...
C...    PDFSET(PARM,VALUE)
C...
C...    For the proton structure function
C...
C...    STRUCTM(X,Q,UPV,DNV,USEA,DSEA,STR,CHM,BOT,TOP,GLU)
C...
C...This interface can be invoked in 3 ways depending
C...on the value of PARM(1) provided by the user when
C...calling PDFSET(PARM,VALUE):
C...
C...    For PYTHIA:         PARM(1).EQ.'NPTYPE'
C...      (this is set automatically by PYTHIA)
C...
C...    For HERWIG:         PARM(1).EQ.'HWLHAPDF'
C...      (set by the USER e.g. in the main program like this:
C...          AUTPDF(1) = 'HWLHAPDF'
C...          AUTPDF(2) = 'HWLHAPDF'                         )
C...
C...    For Stand-alone:    PARM(1).EQ.'DEFAULT'
C...      (can be used for PDF studies or when interfacing
C...       new generators)
C...
C...The LHAPDF set/member is selected depending on the value of:
C...
C...        PYTHIA:   ABS(MSTP(51))
C...        HERWIG:   ABS(INT(VALUE(1)))
C...   STAND-ALONE:   ABS(INT(VALUE(1)))
C...
C...
C...        CONTROL switches:
C...       ==================
C...
C...     THE LOCATION OF THE LHAPDF LIBRARY HAS TO BE SPECIFIED
C...     AS DESCRIBED BELOW (the rest is optional)
C...
C...     if the user does nothing, sensible defaults
C...     are active; to change the behaviour, the corresponding
C...     values of PARM() should be set to the values given below
C...
C...  Location of the LHAPDF library of PDFs (pathname) is given by PARM(20)
C...  Special value:
C...
C...     PARM(20).EQ.'DEFAULT' => default = subdir ../data/PDFsets w.r.t.
C...                              the current directory
C...
C...  Extrapolation of PDFs outside LHAPDF validity range given by
C...  [Xmin,Xmax] and [Q2min,Q2max]; DEFAULT => PDFs "frozen" at the
C...  boundaries
C...
C...      PARM(18).EQ.'EXTRAPOLATE' => Extrapolate PDFs on OWN RISK
C...                          WARNING: Crazy values can be returned
C...
C...  Printout of initialization information in PDFSET (by default)
C...
C...      PARM(19).EQ.'SILENT' => No printout (silent mode)
C...
C...
C...v3.1  26-Apr-2004  New numbering scheme, updated for LHAPDF v2
C...v3.0  23-Jan-2004  HERWIG interface added
C...v2.0  20-Sep-2003  PDFLIB style adopted
C...v1.0  05-Mar-2003  First working version from PYTHIA to LHAPDF v1
C...
C...interface to LHAPDF library

C*********************************************************************

C...PDFSET
C...Initialization for use of parton distributions
C... according to the LHAPDF interface.
C...
C...v3.0  23-Jan-2004  HERWIG interface added
C...v3.1  26-Apr-2004  New numbering scheme
C...
C...interface to LHAPDF library

      SUBROUTINE PDFSET(PARM,VALUE)

C...Double precision and integer declarations.
      IMPLICIT DOUBLE PRECISION(A-H, O-Z)
      IMPLICIT INTEGER(I-N)
C...Commonblocks.
      COMMON/PYDAT1/MSTU(200),PARU(200),MSTJ(200),PARJ(200)
      SAVE /PYDAT1/
      COMMON/PYPARS/MSTP(200),PARP(200),MSTI(200),PARI(200)
      SAVE /PYPARS/
C...Interface to LHAPDFLIB.
      CHARACTER*172 LHANAME
      INTEGER LHASET, LHAMEMB
      COMMON/LHAPDF/LHANAME, LHASET, LHAMEMB
      SAVE /LHAPDF/
      DOUBLE PRECISION QCDLHA4, QCDLHA5
      INTEGER NFLLHA
      COMMON/LHAPDFR/QCDLHA4, QCDLHA5, NFLLHA
      SAVE /LHAPDFR/
      CHARACTER*232 LHAPATH
      COMMON/LHAPDFC/LHAPATH
      SAVE /LHAPDFC/
      INTEGER LHAEXTRP
      COMMON/LHAPDFE/LHAEXTRP
      SAVE /LHAPDFE/
C...Interface to PDFLIB.
      COMMON/W50511/ NPTYPEPDFL,NGROUPPDFL,NSETPDFL,MODEPDFL,
     >               NFLPDFL,LOPDFL,TMASPDFL
      SAVE /W50511/
      DOUBLE PRECISION TMASPDFL
C...Interface to PDFLIB.
      COMMON/W50512/QCDL4,QCDL5
      SAVE /W50512/
      DOUBLE PRECISION QCDL4,QCDL5
C...Interface to PDFLIB.
      COMMON/W50513/XMIN,XMAX,Q2MIN,Q2MAX
      SAVE /W50513/
      DOUBLE PRECISION XMIN,XMAX,Q2MIN,Q2MAX
C...Local arrays and character variables (NOT USED here DB)
      CHARACTER*100 PARM(20)
      DOUBLE PRECISION VALUE(20)
      INTEGER LHAPATHLEN
      INTEGER LHAINPUT
      INTEGER LHASELECT
      INTEGER LHAPRINT
      INTEGER LHAONCE
      INTEGER LHASILENT
      SAVE LHAONCE
      DATA LHAONCE/0/
 
C...Init
      LHAEXTRP = 0
      IF(PARM(18).EQ.'EXTRAPOLATE') THEN  ! Extrapolate PDFs on own risk
         LHAEXTRP = 1
      ENDIF
      LHASILENT = 0
      IF(PARM(19).EQ.'SILENT') THEN       !  No printout (silent MODE)
         LHASILENT = 1
      ENDIF
      IF(PARM(1).EQ.'NPTYPE') THEN        !  PYTHIA
         LHAPRINT = MSTU(11)
         LHAINPUT = ABS(MSTP(51))
         IF(LHASILENT .NE. 1) 
     >    PRINT *,'==== PYTHIA WILL USE LHAPDF ===='
      ELSEIF(PARM(1).EQ.'HWLHAPDF') THEN  !  HERWIG
         LHAINPUT = ABS(INT(VALUE(1)))
         IF(LHAONCE.EQ.LHAINPUT) RETURN
         IF(LHASILENT .NE. 1) 
     >    PRINT *,'==== HERWIG WILL USE LHAPDF ===='
         LHAPRINT = 6
         LHAONCE = LHAINPUT
      ELSEIF(PARM(1).EQ.'DEFAULT') THEN  !  Stand-alone
         LHAINPUT = ABS(INT(VALUE(1)))
         IF(LHAONCE.EQ.LHAINPUT) RETURN
         IF(LHASILENT .NE. 1) 
     >    PRINT *,'==== STAND-ALONE LHAGLUE MODE TO USE LHAPDF ===='
         LHAPRINT = 6
         LHAONCE = LHAINPUT
      ELSE
         PRINT *,'== UNKNOWN LHAPDF INTERFACE CALL! STOP EXECUTION! =='
         STOP
      ENDIF
C...Initialize parton distributions: LHAPDFLIB.
        print*,parm(20)
        IF(PARM(20).EQ.'DEFAULT') THEN ! Use default
         LHAPATH='../data/PDFsets'
        ELSE
         LHAPATH=PARM(20)
        ENDIF
        LHAPATHLEN=INDEX(LHAPATH,' ')-1
        LHASET = LHAINPUT
        XMIN = 1.0D-6      ! X_min for current PDF set
        XMAX = 1.0D0       ! X_max for current PDF set
        Q2MIN = 1.0D0**2   ! Q**2_min scale for current PDF set [GeV]
        Q2MAX = 1.0D5**2   ! Q**2_max scale for current PDF set [GeV]
C...CTEQ Family
        IF((LHAINPUT .GE. 10000) .AND. (LHAINPUT .LE. 19999)) THEN
	  Q2MAX = 1.0D08
          IF((LHAINPUT .GE. 10000) .AND. (LHAINPUT .LE. 10040)) THEN
           LHASET = 10000
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/cteq6.LHpdf'
	   Q2MIN = 1.69D0
          ELSEIF((LHAINPUT .GE. 10041) .AND. (LHAINPUT .LE. 10041)) THEN
           LHASET = 10041
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/cteq6l.LHpdf'
	   Q2MIN = 1.69D0
          ELSEIF((LHAINPUT .GE. 10042) .AND. (LHAINPUT .LE. 10042)) THEN
           LHASET = 10042
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/cteq6ll.LHpdf'
	   Q2MIN = 1.69D0
          ELSEIF((LHAINPUT .GE. 10050) .AND. (LHAINPUT .LE. 10090)) THEN
           LHASET = 10050
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/cteq6mE.LHgrid'
	   Q2MIN = 1.69D0
          ELSEIF((LHAINPUT .GE. 10100) .AND. (LHAINPUT .LE. 10140)) THEN
           LHASET = 10100
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/cteq61.LHpdf'
	   Q2MIN = 1.69D0
          ELSEIF((LHAINPUT .GE. 10150) .AND. (LHAINPUT .LE. 10190)) THEN
           LHASET = 10150
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/cteq61.LHgrid'
	   Q2MIN = 1.69D0
          ELSEIF((LHAINPUT .GE. 19050) .AND. (LHAINPUT .LE. 19050)) THEN
           LHASET = 19050
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/cteq5m.LHgrid'
           XMIN=1.0D-5
          ELSEIF((LHAINPUT .GE. 19051) .AND. (LHAINPUT .LE. 19051)) THEN
           LHASET = 19051
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/cteq5m1.LHgrid'
           XMIN=1.0D-5
          ELSEIF((LHAINPUT .GE. 19060) .AND. (LHAINPUT .LE. 19060)) THEN
           LHASET = 19060
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/cteq5d.LHgrid'
           XMIN=1.0D-5
          ELSEIF((LHAINPUT .GE. 19070) .AND. (LHAINPUT .LE. 19070)) THEN
           LHASET = 19070
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/cteq5l.LHgrid'
           XMIN=1.0D-5
          ELSEIF((LHAINPUT .GE. 19150) .AND. (LHAINPUT .LE. 19150)) THEN
           LHASET = 19150
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/cteq4m.LHgrid'
	   Q2MIN = 2.56D0
           XMIN=1.0D-5
          ELSEIF((LHAINPUT .GE. 19160) .AND. (LHAINPUT .LE. 19160)) THEN
           LHASET = 19160
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/cteq4d.LHgrid'
	   Q2MIN = 2.56D0
           XMIN=1.0D-5
          ELSEIF((LHAINPUT .GE. 19170) .AND. (LHAINPUT .LE. 19170)) THEN
           LHASET = 19170
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/cteq4l.LHgrid'
	   Q2MIN = 2.56D0
           XMIN=1.0D-5
          ELSE
           WRITE(LHAPRINT,5150)  LHASET
           STOP
          ENDIF          
C...MRST Family
        ELSEIF((LHAINPUT .GE. 20000) .AND. (LHAINPUT .LE. 29999)) THEN
          Q2MIN = 1.25D0
	  Q2MAX = 1.0D07
	  XMIN = 1.0D-5
          IF((LHAINPUT .GE. 20000) .AND. (LHAINPUT .LE. 20004)) THEN
           LHASET = 20000
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/MRST2001nlo.LHpdf'
          ELSEIF((LHAINPUT .GE. 20050) .AND. (LHAINPUT .LE. 20054)) THEN
           LHASET = 20050
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/MRST2001nlo.LHgrid'
          ELSEIF((LHAINPUT .GE. 20060) .AND. (LHAINPUT .LE. 20061)) THEN
           LHASET = 20060
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/MRST2001lo.LHgrid'
          ELSEIF((LHAINPUT .GE. 20070) .AND. (LHAINPUT .LE. 20074)) THEN
           LHASET = 20070
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/MRST2001nnlo.LHgrid'
          ELSEIF((LHAINPUT .GE. 20100) .AND. (LHAINPUT .LE. 20130)) THEN
           LHASET = 20100
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/MRST2001E.LHpdf'
          ELSEIF((LHAINPUT .GE. 20150) .AND. (LHAINPUT .LE. 20180)) THEN
           LHASET = 20150
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/MRST2001E.LHgrid'
          ELSEIF((LHAINPUT .GE. 20200) .AND. (LHAINPUT .LE. 20201)) THEN
           LHASET = 20200
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/MRST2002nlo.LHpdf'
          ELSEIF((LHAINPUT .GE. 20250) .AND. (LHAINPUT .LE. 20251)) THEN
           LHASET = 20250
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/MRST2002nlo.LHgrid'
          ELSEIF((LHAINPUT .GE. 20270) .AND. (LHAINPUT .LE. 20271)) THEN
           LHASET = 20270
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/MRST2002nnlo.LHgrid'
          ELSEIF((LHAINPUT .GE. 20300) .AND. (LHAINPUT .LE. 20301)) THEN
           LHASET = 20300
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/MRST2003cnlo.LHpdf'
           Q2MIN = 10.0D0
 	   XMIN = 1.0D-3
          ELSEIF((LHAINPUT .GE. 20350) .AND. (LHAINPUT .LE. 20351)) THEN
           LHASET = 20350
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/MRST2003cnlo.LHgrid'
           Q2MIN = 10.0D0
 	   XMIN = 1.0D-3
          ELSEIF((LHAINPUT .GE. 20370) .AND. (LHAINPUT .LE. 20371)) THEN
           LHASET = 20370
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/MRST2003cnnlo.LHgrid'
           Q2MIN = 7.0D0
 	   XMIN = 1.0D-3
          ELSEIF((LHAINPUT .GE. 29000) .AND. (LHAINPUT .LE. 29003)) THEN
           LHASET = 29000
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/MRST98.LHpdf'
          ELSE
           WRITE(LHAPRINT,5150)  LHASET
           STOP
          ENDIF          
C...Fermi Family
        ELSEIF((LHAINPUT .GE. 30000) .AND. (LHAINPUT .LE. 39999)) THEN
          IF((LHAINPUT .GE. 30100) .AND. (LHAINPUT .LE. 30200)) THEN
           LHASET = 30100
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/Fermi2002_100.LHpdf'
          ELSEIF((LHAINPUT .GE. 31000) .AND. (LHAINPUT .LE. 32000)) THEN
           LHASET = 31000
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/Fermi2002_1000.LHpdf'
          ELSE
           WRITE(LHAPRINT,5150)  LHASET
           STOP
          ENDIF          
C...Alekhin Family
        ELSEIF((LHAINPUT .GE. 40000) .AND. (LHAINPUT .LE. 49999)) THEN
          IF((LHAINPUT .GE. 40100) .AND. (LHAINPUT .LE. 40200)) THEN
           LHASET = 40100
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/Alekhin_100.LHpdf'
          ELSEIF((LHAINPUT .GE. 41000) .AND. (LHAINPUT .LE. 41999)) THEN
           LHASET = 41000
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/Alekhin_1000.LHpdf'
          ELSEIF((LHAINPUT .GE. 40350) .AND. (LHAINPUT .LE. 40365)) THEN
           LHASET = 40350
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/a02_lo_v.LHgrid'
           XMIN = 1.0D-7
	   Q2MIN = 0.8D0
	   Q2MAX = 2.0D08
          ELSEIF((LHAINPUT .GE. 40450) .AND. (LHAINPUT .LE. 40465)) THEN
           LHASET = 40450
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/a02_nlo_v.LHgrid'
           XMIN = 1.0D-7
	   Q2MIN = 0.8D0
	   Q2MAX = 2.0D08
          ELSEIF((LHAINPUT .GE. 40550) .AND. (LHAINPUT .LE. 40565)) THEN
           LHASET = 40550 
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/a02_nnlo_v.LHgrid'
           XMIN = 1.0D-7
	   Q2MIN = 0.8D0
	   Q2MAX = 2.0D08
          ELSE
           WRITE(LHAPRINT,5150)  LHASET
           STOP
          ENDIF          
C...Botje Family
        ELSEIF((LHAINPUT .GE. 50000) .AND. (LHAINPUT .LE. 59999)) THEN
          IF((LHAINPUT .GE. 50100) .AND. (LHAINPUT .LE. 50200)) THEN
           LHASET = 50100
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/Botje_100.LHpdf'
          ELSEIF((LHAINPUT .GE. 51000) .AND. (LHAINPUT .LE. 51999)) THEN
           LHASET = 51000
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/Botje_1000.LHpdf'
          ELSE
           WRITE(LHAPRINT,5150)  LHASET
           STOP
          ENDIF          
C...ZEUS Family
        ELSEIF((LHAINPUT .GE. 60000) .AND. (LHAINPUT .LE. 69999)) THEN
	  Q2MIN = 0.3D0
	  Q2MAX = 2.0D05
          IF((LHAINPUT .GE. 60000) .AND. (LHAINPUT .LE. 60022)) THEN
           LHASET = 60000
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/ZEUS2002_TR.LHpdf'
          ELSEIF((LHAINPUT .GE. 60100) .AND. (LHAINPUT .LE. 60122)) THEN
           LHASET = 60100
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/ZEUS2002_ZM.LHpdf'
          ELSEIF((LHAINPUT .GE. 60200) .AND. (LHAINPUT .LE. 60222)) THEN
           LHASET = 60200
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/ZEUS2002_FF.LHpdf'
          ELSE
           WRITE(LHAPRINT,5150)  LHASET
           STOP
          ENDIF          
C...H1 Family
        ELSEIF((LHAINPUT .GE. 70000) .AND. (LHAINPUT .LE. 79999)) THEN
	  Q2MIN = 1.5D0
	  Q2MAX = 3.5D04
	  XMIN = 5.7D-5
          IF((LHAINPUT .GE. 70050) .AND. (LHAINPUT .LE. 70050)) THEN
           LHASET = 70050
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/H12000ms.LHgrid'
          ELSEIF((LHAINPUT .GE. 70051) .AND. (LHAINPUT .LE. 70070)) THEN
           LHASET = 70050
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/H12000msE.LHgrid'
          ELSEIF((LHAINPUT .GE. 70150) .AND. (LHAINPUT .LE. 70150)) THEN
           LHASET = 70150
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/H12000dis.LHgrid'
          ELSEIF((LHAINPUT .GE. 70151) .AND. (LHAINPUT .LE. 70170)) THEN
           LHASET = 70150
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/H12000disE.LHgrid'
          ELSEIF((LHAINPUT .GE. 70250) .AND. (LHAINPUT .LE. 70250)) THEN
           LHASET = 70250
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/H12000lo.LHgrid'
          ELSEIF((LHAINPUT .GE. 70251) .AND. (LHAINPUT .LE. 70270)) THEN
           LHASET = 70250
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/H12000loE.LHgrid'
          ELSEIF((LHAINPUT .GE. 70350) .AND. (LHAINPUT .LE. 70350)) THEN
           LHASET = 70350
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/H12000lo2.LHgrid'
          ELSEIF((LHAINPUT .GE. 70351) .AND. (LHAINPUT .LE. 70370)) THEN
           LHASET = 70350
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/H12000lo2E.LHgrid'
          ELSE
           WRITE(LHAPRINT,5150)  LHASET
           STOP
          ENDIF          
C...GRV Family
        ELSEIF((LHAINPUT .GE. 80000) .AND. (LHAINPUT .LE. 89999)) THEN
	  Q2MIN = 0.8D0
	  Q2MAX = 2.0D06
	  XMIN = 1.0D-9
          IF((LHAINPUT .GE. 80050) .AND. (LHAINPUT .LE. 80051)) THEN
           LHASET = 80050
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/GRV98nlo.LHgrid'
          ELSEIF((LHAINPUT .GE. 80060) .AND. (LHAINPUT .LE. 80060)) THEN
           LHASET = 80060
           LHANAME=LHAPATH(1:LHAPATHLEN)//'/GRV98lo.LHgrid'
          ELSE
           WRITE(LHAPRINT,5150)  LHASET
           STOP
          ENDIF          
C...Unknown Family ?! Giving up
        ELSE
           WRITE(LHAPRINT,5150)  LHASET
           STOP
        ENDIF

        LHAMEMB=LHAINPUT-LHASET
        CALL INITPDFSET(LHANAME)
        CALL NUMBERPDF(LHAALLMEM)
        IF(LHASILENT .NE. 1) THEN
         WRITE(LHAPRINT,5151)
         WRITE(LHAPRINT,5152) LHANAME
         WRITE(LHAPRINT,5153) LHAALLMEM
         WRITE(LHAPRINT,5154)
        ENDIF
        IF ((LHAMEMB.LT.0) .OR. (LHAMEMB.GT.LHAALLMEM)) THEN
           WRITE(LHAPRINT,5155)  LHAMEMB
           WRITE(LHAPRINT,5156)  LHAALLMEM
           STOP
        ENDIF

        CALL INITPDF(LHAMEMB)

        QMZ = 91.1876D0
        alphasLHA = alphasPDF(QMZ)
        IF(LHASILENT .NE. 1) 
     >   WRITE(LHAPRINT,5158) alphasLHA

        NPTYPEPDFL = 1      ! Proton PDFs
        NFLPDFL = 4
        ALAMBDA = 0.192D0
        QCDLHA4 = ALAMBDA
        QCDLHA5 = ALAMBDA
        IF(PARM(1).EQ.'NPTYPE') THEN        !  PYTHIA
          QCDL4 = ALAMBDA
          QCDL5 = ALAMBDA
        ENDIF

C...Formats for initialization information.
 5150 FORMAT(1X,'WRONG LHAPDF set number =',I12,' given! STOP EXE!')
 5151 FORMAT(1X,'==============================================')
 5152 FORMAT(1X,'PDFset name ',A80)
 5153 FORMAT(1X,'with ',I10,' members')
 5154 FORMAT(1X,'====  initialized. ===========================')
 5155 FORMAT(1X,'LHAPDF problem => YOU asked for member = ',I10)
 5156 FORMAT(1X,'Valid range is: 0 - ',I10,' Execution stopped.')
 5157 FORMAT(1X,'Number of flavors for PDF is:',I4)
 5158 FORMAT(1X,'Strong coupling at Mz for PDF is:',F9.5)

      RETURN
      END
 
C*********************************************************************
 
C...STRUCTM
C...Gives parton distributions according to the LHAPDF interface.
C...Two evolution codes used:
C...  EVLCTEQ for CTEQ PDF sets
C...  QCDNUM  for Other PDF sets
C...
C...Author: Dimitri Bourilkov  bourilkov@mailaps.org
C...
C...v3.0  23-Jan-2004
C...
C...interface to LHAPDF library

      SUBROUTINE STRUCTM(X,Q,UPV,DNV,USEA,DSEA,STR,CHM,BOT,TOP,GLU)

C...Double precision and integer declarations.
      IMPLICIT DOUBLE PRECISION(A-H, O-Z)
      IMPLICIT INTEGER(I-N)
C...Commonblocks.
      COMMON/PYDAT1/MSTU(200),PARU(200),MSTJ(200),PARJ(200)
      SAVE /PYDAT1/
C...Interface to LHAPDFLIB.
      CHARACTER*172 LHANAME
      INTEGER LHASET, LHAMEMB
      COMMON/LHAPDF/LHANAME, LHASET, LHAMEMB
      SAVE /LHAPDF/
      DOUBLE PRECISION QCDLHA4, QCDLHA5
      INTEGER NFLLHA
      COMMON/LHAPDFR/QCDLHA4, QCDLHA5, NFLLHA
      SAVE /LHAPDFR/
      INTEGER LHAEXTRP
      COMMON/LHAPDFE/LHAEXTRP
      SAVE /LHAPDFE/
C...Interface to PDFLIB.
      COMMON/W50513/XMIN,XMAX,Q2MIN,Q2MAX
      SAVE /W50513/
      DOUBLE PRECISION XMIN,XMAX,Q2MIN,Q2MAX
C...Local variables
      DOUBLE PRECISION UPV,DNV,USEA,DSEA,STR,CHM,BOT,TOP,GLU
      DOUBLE PRECISION X,Q,F(-6:6)
 
C...Range of validity e.g. 10^-6 < x < 1, Q2MIN < Q^2 extended by
C...freezing x*f(x,Q2) at borders.
      IF(LHAEXTRP .NE. 1) THEN    ! safe mode == "freeze"
       XIN=MAX(XMIN,MIN(1D0,X))
       Q=SQRT(MAX(0D0,Q2MIN,MIN(Q2MAX,Q**2)))
      ELSE                        ! adventurous mode == OWN RISK !
       XIN=X
      ENDIF

      CALL EVOLVEPDF(XIN,Q,F)

      GLU = F(0)
      DSEA = F(-1)
      DNV = F(1) - DSEA
      USEA = F(-2)
      UPV = F(2) - USEA
      STR = F(3)
      CHM = F(4)
      BOT = F(5)
      TOP = F(6)

      RETURN
      END

      SUBROUTINE STRUCTP(XX,QQ2,P2,IP2,UPV,DNV,USEA,DSEA,STR,CHM,
     &BOT,TOP,GLU)
C...Double precision and integer declarations.
      IMPLICIT DOUBLE PRECISION(A-H, O-Z)
      IMPLICIT INTEGER(I-N)
C...Commonblocks.
      COMMON/PYDAT1/MSTU(200),PARU(200),MSTJ(200),PARJ(200)
      SAVE /PYDAT1/
C...Interface to LHAPDFLIB.
      CHARACTER*172 LHANAME
      INTEGER LHASET, LHAMEMB
      COMMON/LHAPDF/LHANAME, LHASET, LHAMEMB
      SAVE /LHAPDF/
      DOUBLE PRECISION QCDLHA4, QCDLHA5
      INTEGER NFLLHA
      COMMON/LHAPDFR/QCDLHA4, QCDLHA5, NFLLHA
      SAVE /LHAPDFR/
      INTEGER LHAEXTRP
      COMMON/LHAPDFE/LHAEXTRP
      SAVE /LHAPDFE/
C...Interface to PDFLIB.
      COMMON/W50513/XMIN,XMAX,Q2MIN,Q2MAX
      SAVE /W50513/
      DOUBLE PRECISION XMIN,XMAX,Q2MIN,Q2MAX
 
C...Stop program if this routine is ever called.
      WRITE(MSTU(11),5000)
      STOP

 5000 FORMAT('Error: LHAPDF does not include photon structure'/
     &'functions yet. Execution stopped!')
      END
