c*************************************************************************
      SUBROUTINE ptilde(p, xuz, v)
c*************************************************************************
c     This is the manager of all ptilde subroutines and the (only) 
c     interface of those to the main program.  Depending on the process ID
c     the right ptilde routine is called.
c
c*************************************************************************
c     INPUT
c     p(mu,1:np,1) : np parton momenta for real emission graph
c     v(mu,1:nv,1) : nv momenta of other particles for real emission graph
c     
c     OUTPUT
c     p(mu,1:np,j) : np parton momenta in the ptilde kinematic
c     v(mu,1:nv,j) : nv momenta of other particles in the ptilde kinematic
c                    (j = 2, ..., n_kin)
c     xuz          : These are the z, u, x_ij,a of the Catani-Seymour algorithm
c*************************************************************************

      implicit none

#include "global.inc"
#include "process.inc"

      real*8 p(0:3,max_p,max_kin)
      real*8 v(0:3,max_v,max_kin)
      real*8 xuz(2,2:max_kin)

      SELECT CASE(procID)

c VBF: Hjj, Vjj, VVjj
      CASE(Hjj, Hjj_AA, Hjj_mu, Hjj_tau, Hjj_bbar, Hjj_WW, Hjj_ZZ_ll,
     1     Hjj_ZZ_lnu, Zjj_l, Zjj_nu, WPjj, WMjj, WPWMjj, ZZjj_ll,
     2     ZZjj_lnu, WPZjj, WMZjj)
          call VBFptildemake(3,p(0,1,1),xuz,v(0,1,1))

c VBF: Hjjj
      CASE(Hjjj,Hjjj_AA, Hjjj_mu, Hjjj_tau, Hjjj_bbar, Hjjj_WW, 
     1         Hjjj_ZZ_ll,Hjjj_ZZ_lnu)
          call HJJJptildemake(n_kin,p(0,1,1),xuz(1,2),v(0,1,1))

c DIBSON, TRIBSON
      CASE( WW, WWZ, ZZWP, ZZWM, WWWP, WWWM )
          call DIBOSptildemake(3,p(0,1,1),xuz(1,2),v(0,1,1))

      CASE DEFAULT
         print *,"Error: ptilde routine for process "//
     &            "not implemented, ID = ",procID
         STOP
      END SELECT

      return
      end

c************************************************************************
      subroutine VBFptildemake(N,p,xuz,v)
c*************************************************************************
c
c assign ptilde momenta in notation of Catani&Seymour
c
c  input:  p(mu,1:np,1)      np parton momenta for real emission graph
c  output:                   np-1 parton momenta for Born amplitude
c          p(mu,1:np-1,2)        index=2: emission off 1-2 line
c          p(mu,1:np-1,3)        index=3: emission off 3-4 line
c
c          xuz(1,index)      x_ija etc
c          xuz(2,index)      u or z
c
c at the same time, fill the legoplot variables for the external momenta
c
c       Dieter Zeppenfeld, <dieter@particle.uni-karlsruhe.de>
c       Initial version:  November 2002
c       Last modified: September, 2008
c*************************************************************************

      implicit none

#include "global.inc"

      integer mu, a, i, j, N, ii, jj, k
      real*8 p(0:3,max_p,max_kin), xuz(2,2:3), v(0:3,max_v,max_kin)
      real*8 omx, pipj, pipa, pjpa
      real*8 tpi
      parameter (tpi=2d0*3.141592653589793d0)
      logical ldebug
      parameter (ldebug=.false.)

c  parton np is the potentially soft/collinear one
      do ii = 2,N
         if (ii.eq.2) then        ! set mom id for radiation off 1,2 line
            a = 1
            i = 5
            j = 3
         elseif (ii.eq.3) then
            a = 2
            i = 5
            j = 4
         else
            stop
         endif
         pipj = p(0,i,1)*p(0,j,1)-p(1,i,1)*p(1,j,1)-
     1          p(2,i,1)*p(2,j,1)-p(3,i,1)*p(3,j,1)
         pipa = p(0,i,1)*p(0,a,1)-p(1,i,1)*p(1,a,1)-
     1          p(2,i,1)*p(2,a,1)-p(3,i,1)*p(3,a,1)
         pjpa = p(0,j,1)*p(0,a,1)-p(1,j,1)*p(1,a,1)-
     1          p(2,j,1)*p(2,a,1)-p(3,j,1)*p(3,a,1)
         omx = pipj/(pipa+pjpa)
         xuz(1,ii) = 1d0 - omx
         xuz(2,ii) = pipa/(pipa+pjpa)
         if (ldebug) print*," assign momenta # ",a,j,3-a,7-j
         do mu = 0,3
            p(mu,5,ii) = 0
            p(mu,a,ii) = xuz(1,ii)*p(mu,a,1)
            p(mu,j,ii) = p(mu,j,1)+p(mu,i,1)-omx*p(mu,a,1)
            p(mu,3-a,ii)=p(mu,3-a,1)
            p(mu,7-j,ii)=p(mu,7-j,1)
         enddo

         if (ldebug) then
check momentum conservation
            do mu = 0,3
               omx = p(mu,n_p,1)
               do jj = 1,n_p-1
                  if(jj.le.2) omx = omx-p(mu,jj,1)+p(mu,jj,ii)
                  if(jj.gt.2) omx = omx+p(mu,jj,1)-p(mu,jj,ii)
               enddo
               print*," mom.cons. for mu=",mu,": sum= ",omx
            enddo
check masslessness of ptilde_a and ptilde_j
            print*," p_a^2 = ",p(0,a,ii)**2-p(1,a,ii)**2-
     1                         p(2,a,ii)**2-p(3,a,ii)**2
            print*," p_j^2 = ",p(0,j,ii)**2-p(1,j,ii)**2-
     1                         p(2,j,ii)**2-p(3,j,ii)**2
         endif
      enddo

      do i = 1, n_v
         do k = 1, n_kin
            do mu = 0,3
               v(mu,i,k) = v(mu,i,1)
            enddo
         enddo
      enddo

      if ( ldebug ) then
 10      format( " p(", i1, ") = ", 4(f10.3, 2x) )
c
         do j = 1,N
            if (j.eq.1) print*," (n+1) parton momenta "
            if (j.eq.2) print*," n-parton momenta for upper NLO corr."
            if (j.eq.3) print*," n-parton momenta for lower NLO corr."
            do i = 1, n_p-min(j-1,1)
               write(6,10) i, p(0,i,j), p(1,i,j), p(2,i,j), p(3,i,j)
            end do
         enddo
      end if
      end

c********************** subroutine DIBOSptildemake *************************
c
c    this subroutine assigns ptilde momenta to the partons and the final
c    state leptons in notation of Catani and Seymour.
c
c INPUT    p(mu,1:np,1)      np parton momenta for real emission graph
c          v(mu,1:nv,1)      nv lepton momenta for real emission graph
c
c OUTPUT   np-1 parton momenta for Born amplitude
c          p(mu,1:np-1,2)    index=2: emission off 1-2 line
c          p(mu,1:np-1,3)    index=3: emission off 3-4 line

c          lepton momenta for born amplitude
c          v(mu,1:nv,2)      index=2: emission off 1-2 line
c          v(mu,1:nv,3)      index=3: emission off 3-4 line
c
c          xuz               x_ija in the subtraction terms
c
c**************************************************************************

      subroutine DIBOSptildemake(N,p,xuz,v)

      implicit none

#include "global.inc"

      integer mu, a, i, j, N, ii, jj, b
      real*8 p(0:3,max_p,max_kin), xuz, v(0:3,max_v,max_kin)
      real*8 omx, pipa, pipb, papb
      real*8 vK(1:n_v), vKtilde(1:n_v), KKtilde

      real*8 tpi
      real*8 K(0:4), Ktilde(0:4)
      double precision dotrr, temp(0:3), temp2(0:3)
      parameter (tpi=2d0*3.141592653589793)

      logical ldebug
      parameter (ldebug=.false.)

      external dotrr

      pipa = dotrr(p(0,3,1),p(0,1,1))
      pipb = dotrr(p(0,3,1),p(0,2,1))
      papb = dotrr(p(0,1,1),p(0,2,1))

      omx = (pipa + pipb)/papb
      xuz = 1d0 - omx

      do mu = 0,3
         K(mu) = p(mu,1,1) + p(mu,2,1) - p(mu,3,1)
      enddo
      K(4) = dotrr(K,K)

      do ii = 2,N
         if (ii.eq.2) then        ! set mom id for radiation off 1,2 line
            a = 1
            b = 2
            i = 3
         elseif (ii.eq.3) then
            a = 2
            b = 1
            i = 3
         else
            stop
         endif

         do mu = 0,3
            Ktilde(mu) = xuz*p(mu,a,1) + p(mu,b,1)
         enddo

         Ktilde(4) = dotrr(Ktilde,Ktilde)
         KKtilde = dotrr(K,Ktilde)

         if (ldebug) then
            print*," K^2/Ktilde^2 = 1 ? ", K(4)/Ktilde(4)
         endif

         if (ldebug) print*," assign tilde momenta for parton # ",a,i
         do mu = 0,3
            p(mu,i,ii) = 0.0d0
            p(mu,a,ii) = xuz*p(mu,a,1)
            p(mu,b,ii) = p(mu,b,1)
         enddo

         if (ldebug) print*," assign tilde momenta for all leptons: "

         do j = 1,n_v
            vK(j) = dotrr(v(0,j,1),K(0))
            vKtilde(j) = dotrr(v(0,j,1),Ktilde(0))

            do mu = 0,3
               v(mu,j,ii) = v(mu,j,1) - 2d0*(vK(j) + vKtilde(j))/
     &            (K(4) + Ktilde(4) + 2d0*KKtilde)*(K(mu) + Ktilde(mu))
     &            +2d0*vK(j)/K(4) * Ktilde(mu)
            enddo
         enddo

c check if ktilde_i*ktilde_j = k_i*k_j 
         if (ldebug) then
            do j = 1,n_v-1
               do mu = j+1,n_v
                  print*,"momentum configuration ",ii,": "
                  print*," vtilde",j,"*vtilde",mu,
     &                   "/v",j,"*v",mu," = 1 ?", 
     &            dotrr(v(0,j,ii),v(0,mu,ii))
     &            /dotrr( v(0,j,1),v(0,mu,1))
               enddo
            enddo
        endif

c check if v_i^2 = vtile_i^2
        if (ldebug) then
            do j =1,n_v
               if(dotrr(v(0,j,1),v(0,j,1)).ge.1d-7.or.
     &            dotrr(v(0,j,ii),v(0,j,ii)).ge.1d-7) then
               print*," v",j,"^2 = ", dotrr(v(0,j,1),v(0,j,1))
               print*,"momentum configuration ",ii,": "
               print*," vtilde",j,"^2 = ", dotrr(v(0,j,ii),v(0,j,ii))
               endif
            enddo
        endif

c check if (v1+v2)^2 = (vtilde1+vtilde2)^2
        if (ldebug) then
            print*,"momentum configuration ",ii,": "
            do mu = 0,3
               temp(mu) = v(mu,1,1)+v(mu,2,1)
               temp2(mu)= v(mu,1,ii)+v(mu,2,ii)
            enddo
            print*," (v1 + v2)^2/(vtilde1 + vtilde2)^2 = "
     &                  ,dotrr(temp,temp)/dotrr(temp2,temp2)
            do mu = 0,3
               temp(mu) = v(mu,3,1)+v(mu,4,1)
               temp2(mu)= v(mu,3,ii)+v(mu,4,ii)
            enddo
            print*," (v3 + v4)^2/(vtilde3 + vtilde4)^2 = "
     &                  ,dotrr(temp,temp)/dotrr(temp2,temp2)
        endif

c check momentum conservation
         if (ldebug) then
            do mu = 0,3
               temp(mu) = p(mu,a,1) + p(mu,b,1) - p(mu,i,1)
     &               - v(mu,1,1) - v(mu,2,1) - v(mu,3,1) - v(mu,4,1)
               print*," mom.cons. for original momentum configuration "
               print*," mu=",mu,": sum= ",temp(mu)
               temp(mu) = p(mu,a,ii) + p(mu,b,ii)
     &               - v(mu,1,ii) - v(mu,2,ii) - v(mu,3,ii) - v(mu,4,ii)
               print*," mom.cons. for new mom. configuration "
               print*,"mu=",mu,": sum= ",temp(mu)
            enddo

check masslessness of ptilde_a and ptilde_j
            print*," p_a^2 = ",p(0,a,ii)**2-p(1,a,ii)**2-
     1                         p(2,a,ii)**2-p(3,a,ii)**2
            print*," p_i^2 = ",p(0,i,ii)**2-p(1,i,ii)**2-
     1                         p(2,i,ii)**2-p(3,i,ii)**2
            do  j = 1 , n_v
               print*,"j = ",j, " v_j^2 = ",dotrr(v(0,j,ii),v(0,j,ii))
            enddo
         endif

      enddo

      if ( ldebug ) then
 10      format( " p(", i1, ") = ", 4(f10.3, 2x) )
c
         do j = 1,N
            if (j.eq.1) print*," (n+1) parton momenta "
            if (j.eq.2) print*," n-parton momenta for upper NLO corr."
            if (j.eq.3) print*," n-parton momenta for lower NLO corr."
            do i = 1, 3-min(j-1,1)
               write(6,10) i, p(0,i,j), p(1,i,j), p(2,i,j), p(3,i,j)
            enddo
         enddo

      end if
     
      return
      end


c*************************************************************************
      subroutine HJJJptildemake(N,p,xuz,v)
c*************************************************************************
c
c assign ptilde momenta in notation of Catani&Seymour
c
c  input:  p(mu,1:np,1)      np parton momenta for real emission graph
c          v(mu,nv,1)
c  output:                   np-1 parton momenta for Born amplitude
c          p(mu,1:np,nd)     nd is the dipole number
c
c          xuz(1,nd)      x_ija,y_ijk etc
c          xuz(2,nd)      u or z
c
c at the same time, fill the legoplot variables for the external momenta
c
c       Dieter Zeppenfeld, <dieter@pheno.physics.wisc.edu>
c       Initial version:  November 2002
c       Last modified: March 11, 2005
c*************************************************************************

      implicit none

#include "global.inc"

      integer mu,N,ii,jj,j,i
      real*8 p(0:3,max_p,1:max_kin),xuz(2,2:max_kin),v(0:3,max_v,1:max_kin)

      integer N1,npart,ic
      integer kp(2:max_kin),jp(2:max_kin),ip(2:max_kin),nd
      real*8 tpi
      parameter (tpi=2d0*3.141592653589793)
      logical ldebug
      parameter (ldebug=.false.)
      real*8 ph(0:3)            ! 4-momentum of higgs
c     reconstructed from decay momentum

      N1=n_p-nx
      if (N.eq.1) then
         npart = N1-1
      else
         npart = N1
      endif
c
c  set superfluous variables to zero
      do ic = npart+1,N1
         do ii = 1,max_kin
            do mu = 0,3
               p(mu,nx+ic,ii) = 0
            enddo
         enddo
      enddo
c
c  put call to transform subroutine here
c     map ip,kp,jp to nd
c     loop over dipole configurations.
c     D^56,4 in the notation of campbell and ellis
c
      if(N.ge.2) then 
c     fill ip,kp,jp arrays
c
         call create_ikj(ip,kp,jp)   
c
c           
         do nd = 2,N
            call transform(nd,p,v,xuz,ip(nd),kp(nd),jp(nd))
         enddo

      endif


      if ( ldebug ) then
 10      format( ' p(', i1, ') = ', 4(f10.3, 2x) )
c
         do j = 1,N
            if (j.eq.1) print*,' (n+1) parton momenta '
            if (j.ge.2) print*,' n-parton momenta for',j,
     &   'momentum config'
            do i = 1, n_p-min(j-1,1)
               write(6,10) i, p(0,i,j), p(1,i,j), p(2,i,j), p(3,i,j)
            end do

         enddo         
      endif
      end


ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine create_ikj(i,k,j)
      implicit none
#include "VBFNLO/utilities/global.inc"
      integer i(2:max_kin),k(2:max_kin),j(2:max_kin)
      integer a,b,c(4)


c     map from CS notation to CE notation
c
      a = 1                     !initial state particle
      b = 2                     !initial state particle
      c(1) = 3
      c(2) = 4
      c(3) = 5
      c(4) = 6
c     maps to momentum configuration id
c     Please take note of this mapping! It's hard coded.
c     "upper-line dipoles"
      i(2) = c(1)
      k(2) = c(4)
      j(2) = c(3)
c
      i(3) = c(1)
      k(3) = c(3)
      j(3) = c(4)
c
      i(4) = c(3)
      k(4) = c(4)
      j(4) = c(1)
c
      i(5) = c(1)
      k(5) = c(4)
      j(5) = a
c
      i(6) = c(1)
      k(6) = c(3)
      j(6) = a
c      
      i(7) = c(3)
      k(7) = c(4)
      j(7) = a
c
      i(8) = a
      k(8) = c(4)
      j(8) = c(1)
c
      i(9) = a
      k(9) = c(3)
      j(9) = c(1)
c
      i(10) = a
      k(10) = c(4)
      j(10) = c(3)
c
      i(11) = a
      k(11) = c(3)
      j(11) = c(4)
c
c     "lower-line dipoles"
      i(12) = c(2)
      k(12) = c(4)
      j(12) = c(3)
c
      i(13) = c(2)
      k(13) = c(3)
      j(13) = c(4)
c
      i(14) = c(3)
      k(14) = c(4)
      j(14) = c(2)
c
      i(15) = c(2)
      k(15) = c(4)
      j(15) = b
c
      i(16) = c(2)
      k(16) = c(3)
      j(16) = b
c      
      i(17) = c(3)
      k(17) = c(4)
      j(17) = b
c
      i(18) = b
      k(18) = c(4)
      j(18) = c(2)
c
      i(19) = b
      k(19) = c(3)
      j(19) = c(2)
c
      i(20) = b
      k(20) = c(4)
      j(20) = c(3)
c
      i(21) = b
      k(21) = c(3)
      j(21) = c(4)
c     new dipoles for gg->q qb q qb
c     
      i(22) = b
      k(22) = c(2)
      j(22) = c(4)
c
      i(23) = a
      k(23) = c(1)
      j(23) = c(3)
c
      i(24) = a
      k(24) = c(1)
      j(24) = c(4)

      i(25) = b
      k(25) = c(2)
      j(25) = c(3)
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc




c****************************************************************************


c Terrance Figy <terrance@physics.wisc.edu>
c modified last on 17 April 2005
c Takes p1+p2 -> p3+p4 + ...px + .. qp(np+nv)
c to q1+q2 -> q2+q3 + ...qx + .. q(np+nv-1)
c This subroutine transforms p(0:3,np+nv) to q(0:3,np+nv)
c q(mu,np+nv) remains ordered after removal of emitted parton kp    
c
c 
      subroutine transform(nd,pmom,lmom,xuz,ip,kp,jp)
      implicit none
c     do interface with Dieter notation here
c     input: pmom(mu,np,1) parton momenta
c            lmom(mu,nv,1) decay momenta
c     pi is the emitter, pk is the emitted, and pj is the spectator
c     output:pmom(mu,np,nd)
c            lmom(mu,nv,nd)      
c            xuz(2,2:max_kin)
c     local variables:
c     p(mu,1:np+nv) This is filled by pmom and lmom.
c     q(mu,1:np+nv) Returned
c     xu(1) is x or y 
c     xu(2) is u or z
c     Declare variable used in subroutine
c
#include "VBFNLO/utilities/global.inc"
c     
c     you better put mxpt in global.inc
c     np is the number of partons and nv is the number of decay leptons
      integer ip,kp,jp,mu,ic,ipart,nd,j
      real*8 pmom(0:3,max_p,max_kin),lmom(0:3,max_v,max_kin),xuz(2,2:max_kin)
      real*8 p(0:3,max_p+max_v),q(0:3,max_p+max_v),xu(2),sum(0:3)
      real*8 omy,y,x,omx,z,u
      real*8 sik,sjk,sij

      real*8 dotrr
      external dotrr

      real*8 k(0:3),kt(0:3),ks(0:3)
      real*8 kDk,ksDks,pDks,pDk,vDks,vDk
      real*8 one,two            !these can be put into global
      parameter(one=1.0d0,two=2.0d0)
      logical ldebug,ldebug1
      parameter(ldebug=.false.,ldebug1=.false.)
c
c
      if(ldebug1) print*,'i,k,j=',ip,kp,jp
c     fill p(mu,1:np+nv)
      do ic =1,n_p
         do mu=0,3
            p(mu,ic) = pmom(mu,ic,1)
         enddo
      enddo
      do ic =1,n_v
         do mu=0,3
            p(mu,ic+n_p) = lmom(mu,ic,1)
         enddo
      enddo
c
      if(ldebug1) then
         
         do j=1,n_p+n_v
            print*,'p',j,'=',(p(mu,j),mu=0,3)
         enddo
         if(ldebug) then
            do j=1,n_p
               print*,'pmom(',j,',1)=',(pmom(mu,j,1),mu=0,3)
            enddo
            do j=1,n_v
               print*,'lmom(',j,',1)=',(lmom(mu,j,1),mu=0,3)
            enddo
         endif
      endif
c
      do ic=1,n_p+n_v
         do mu=0,3
            q(mu,ic)=0d0
         enddo
      enddo
c
c      if(ldebug) print*,'before selection'
c      if(ldebug) print*,'i,k,j=',ip,kp,jp
      if(ldebug) call checkmomcon(p)
      
c     dot products maybe write a function dot for this
c      sik = p(0,ip)*p(0,kp) -p(1,ip)*p(1,kp)-
c     $     p(2,ip)*p(2,kp)-p(3,ip)*p(3,kp)
c      sjk = p(0,jp)*p(0,kp) -p(1,jp)*p(1,kp)-
c     $     p(2,jp)*p(2,kp)-p(3,jp)*p(3,kp)
c      sij = p(0,ip)*p(0,jp) -p(1,ip)*p(1,jp)-
c     $     p(2,ip)*p(2,jp)-p(3,ip)*p(3,jp)
c
      sik = dotrr(p(0,ip),p(0,kp))
      sjk = dotrr(p(0,jp),p(0,kp))
      sij = dotrr(p(0,ip),p(0,jp))
c
      if(ldebug) print*,'sij,sjk,sij=',sij,sjk,sij
c
c     Final-final
      
      if(ip.gt.2 .and. jp.gt.2) then
         if(ldebug1) print*,'final-final'
        y = sik/(sik+sjk+sij)
        omy = one - y
        z = sij/(sij+sjk)
c
        xu(1) = y
        xu(2) = z
c
        if(ldebug) then
           if(y.lt.0d0) print*,'y in ff is lt 0'
           if(z.lt.0d0) print*,'z in ff is lt 0'
        endif
c    
        if(ldebug) print*,'x=,',xu
        ipart=1
        do ic=1,n_p+n_v
           do mu =0,3
              if(ic.eq.jp) then
                 q(mu,ipart) = p(mu,jp)/omy
              elseif(ic.eq.ip) then
                 q(mu,ipart) = p(mu,ip) + p(mu,kp)-y*p(mu,jp)/omy
              elseif(ic.eq.kp) then
                 goto 20
              else
                 q(mu,ipart) = p(mu,ic)
              endif
           enddo
           ipart = ipart +1
 20        continue
        enddo
         if(ldebug) call checkmomcon(q)
         goto 60
c
c     Initial-final
      elseif(((ip.le.2) .and. (jp.gt.2))) then
         if(ldebug1) print*,'initial-final'
         omx = sjk/(sij+sik)
         x = one - omx
         u = sik/(sik+sij)
c         
         xu(1) = x           
         xu(2) = u              !1-z for fi case       
c
         if(ldebug) then
            if(x.lt.0d0) then
               print*,'x in if is lt 0'
               print*,'x=,',x
               pause
            endif
            if(u.lt.0d0) then
               print*,'u in if is lt 0'
               print*,'u=,',u
               pause
            endif
         endif
c
         if(ldebug) print*,'x=,',xu
         ipart = 1
         do ic=1,n_p+n_v
            do mu =0,3
               if(ic.eq.ip) then
                  q(mu,ipart) = x*p(mu,ip)
               elseif(ic.eq.jp) then
                  q(mu,ipart) = p(mu,jp) + p(mu,kp)-omx*p(mu,ip)
               elseif(ic.eq.kp) then
                  goto 30
               else
                  q(mu,ipart) = p(mu,ic)
               endif
            enddo
            ipart=ipart+1
 30         continue
         enddo
         if(ldebug) call checkmomcon(q)
         goto 60
c
c     Final - initial 
c     
      elseif(((ip.gt.2) .and. (jp.le.2))) then
         if(ldebug1) print*,'final-initial'
         omx = sik/(sij+sjk)
         x = one - omx
         z = sij/(sjk+sij)
         
         xu(1) = x           !=x for fi
         xu(2) = z           !=z for fi
c
         if(ldebug) then
            if(x.lt.0d0) then
               print*,'x in fi is lt 0'
               print*,'x=,',x
               pause
            endif
            if(z.lt.0d0) then
               print*,'z in fi is lt 0'
               print*,'z=,',z
               pause
            endif
         endif
c
         if(ldebug) print*,'x=,',xu
         ipart = 1
         do ic=1,n_p+n_v
            do mu =0,3
               if(ic.eq.jp) then
                  q(mu,ipart) = x*p(mu,jp)
               elseif(ic.eq.ip) then
                  q(mu,ipart) = p(mu,ip) + p(mu,kp)-omx*p(mu,jp)
               elseif(ic.eq.kp) then
                  goto 40
               else
                  q(mu,ipart) = p(mu,ic)
               endif
            enddo
            ipart=ipart+1
 40         continue
         enddo
         if(ldebug) call checkmomcon(q)
         goto 60       
c                  
c     Initial-initial 
      elseif((ip.le.2).and.(jp.le.2)) then
         if(ldebug1) print*,'initial-initial'
         omx = (sik+sjk)/sij
         x = one - omx
         xu(1) = x
         xu(2) = 0.0d0
         if(x.lt.0d0) print*,'x in ii is lt 0'
         if(ldebug) print*,'x=,',xu
c     
c     see 5.5 of Catani-seymour
         do mu = 0,3
            q(mu,ip) = x*p(mu,ip)
            q(mu,jp) = p(mu,jp)
            k(mu) = p(mu,ip)+p(mu,jp)-p(mu,kp)
            kt(mu) = q(mu,ip)+p(mu,jp)
            ks(mu) = k(mu)+kt(mu)
         enddo
         kDk=k(0)**2-k(1)**2-k(2)**2-k(3)**2
         ksDks=ks(0)**2-ks(1)**2-ks(2)**2-ks(3)**2
c     
         ipart = 3
         do ic = 3,n_p+n_v        !transform all final state particles
            if(ic.eq.kp) then 
               goto 50
            else
               pDks = p(0,ic)*ks(0)-p(1,ic)*ks(1) -
     $              p(2,ic)*ks(2)- p(3,ic)*ks(3)
               pDk =  p(0,ic)*k(0)-p(1,ic)*k(1) -
     $              p(2,ic)*k(2)- p(3,ic)*k(3)
               do mu =0,3
                  q(mu,ipart) = p(mu,ic) - 
     $                 two*pDks/ksDks*ks(mu)+two*pDk/kDk*kt(mu)
               enddo
               ipart = ipart + 1
            endif
 50         continue
         enddo   
         if(ldebug) call checkmomcon(q)
         goto 60
      else
         print*,'program will stop'
         stop       
      endif
c
 60   continue

      do mu=0,3
      if(q(mu,n_p+n_v).gt.0d0) then
         print*,'Last entry in q(mu,np+nv) not zero'
         print*,'program error'
         pause
      endif
      enddo
c     last entry in q is zero vector 
c     The higgs decay is the last nv entries in q. 
c     The first np entries are the partons.
c     fill pmom and lmom
      
      do j = 1,n_p+n_v-1
         do mu =0,3
            if(j.le.n_p-1) then
               pmom(mu,j,nd) = q(mu,j)
            else
               lmom(mu,j-n_p+1,nd) = q(mu,j)
            endif
         enddo
      enddo
      do mu=0,3
         pmom(mu,n_p,nd) = q(mu,n_p+n_v) !last entry is zero vector 
      enddo

      if(ldebug1) then
         
         do j=1,n_p+n_v           
            print*,'q(',j,')=',(q(mu,j),mu=0,3)
         enddo
c     print*,'q(',mu,np+nv,')=',q(mu,np+nv)
         if(ldebug) then
            do j=1,n_p
               print*,'pmom(',j,nd,')=',(pmom(mu,j,nd),mu=0,3)
            enddo
            do j=1,n_v
               print*,'lmom(',j,nd,')=',(lmom(mu,j,nd),mu=0,3)
            enddo
         endif
      endif
c     fill xuz
      xuz(1,nd) = xu(1)
      xuz(2,nd) = xu(2)
      if(ldebug1) then
         print*,'xuz(1,',nd,')',xuz(1,nd)
         print*,'xuz(2,',nd,')',xuz(2,nd)
         print*,'-----------------------'
c         pause
      endif
c
      return
      end
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                
c     checkmomcon(p) this subr checks that momentum is conserved.
c                              and that partons are massless
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine checkmomcon(p)
      implicit none
#include "VBFNLO/utilities/global.inc"
      real*8 p(0:3,max_p+max_v),ptot(0:3)
      integer mu,j

c      First 2 in list are initial.

      do mu = 0,3
         ptot(mu) = p(mu,1)+p(mu,2)
      enddo
c     
      print*,'pa + pb =',ptot
      do j = 3,8
         do mu = 0,3
            ptot(mu) = ptot(mu) - p(mu,j)
         enddo
      enddo
      print*,'pa + pb - sum(p_i,i=3..8)=',(ptot(mu),mu=0,3)
      
c      print*,'tot mom=',(p(mu,1)+p(mu,2)-
c     $     p(mu,3)-p(mu,4)-p(mu,5)-p(mu,6)-
c     $     p(mu,7)-p(mu,8),mu=0,3)
      
c     check that partons are massless
      do j=1,n_p+n_v
         print*,'psq(',j,')',p(0,j)**2-p(1,j)**2-p(2,j)**2-p(3,j)**2
      enddo

      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

