***********************************************************************
      FUNCTION A4G_Loop(i1,i2,i3,i4,mass)
*----------------------------------------------------------------------
c modified: Michael Kubocz
c date    : 14.08.06
*----------------------------------------------------------------------
c   i1,i2,i3,i4 gives the permutation of the 4 gluons
c   input gluon momenta and polarization vectors 
c   are stored via the needed dot products in q_dot_ep,
c   ep_dot_ep and q_dot_q and explicitly in qi and eps

c   Normalization: this subroutine gives the pentagon contributions
c   without a factor 4*i*mt the QCD couplings and Yukawa coupling h_t 
c   (gamma_5). Thus, for the full amplitude multiply by an overall
c   normalization factor of 4 mt h_t alpha_s**2 c_i .

c                     * -- i1    
c                   /   \
c           i2 -- *      \
c                 |       * -- A^0
c           i3 -- *      /
c                   \   /
c                     * -- i4

c   When summing the two Feynman graphs with opposite fermion flow
c   below. The color factors c_i is defined in the CP even NP paper
c   eq.3.43
***********************************************************************

      IMPLICIT NONE

#include "ggf_process.inc"
#include "d_coefficients.inc"
#include "ggf_dotproducts.inc"

      complex*16 A4G_Loop

c   four vectors
      real*8 eps,qi
      COMMON /bh4g_mom_eps/ qi(0:3,4),eps(0:3,4)

      real*8 ALFAS,XMT,ALFA,XMZ,XMW,SIN2W,XMH
      COMMON /BKOPIN/ ALFAS,XMT,ALFA,XMZ,XMW,SIN2W,XMH

c   explicit momenta for call of eps tensor
      real*8  q1(0:3),q2(0:3),q3(0:3),q4(0:3),
     &        mu1(0:3),mu2(0:3),mu3(0:3),mu4(0:3)

c   Passarino-Veltman form factors
      complex*16 C0_1M,C0L,D0_123,D0_234,D0_1p2,D0_2p3,D0_3p4,
     &           D1i_123(3),D1i_234(3),D1i_1p2(3),D1i_2p3(3),D1i_3p4(3),
     &           Dij123(24),Dij234(24),Dij1p2(24),Dij2p3(24), 
     &           Dij3p4(24),E0(12),E0L,E1i(4,12),E1iL(4)

c   local dot products 
      real*8 q1sq,q2sq,q3sq,q4sq,mass
      real*8 q12sq,q34sq,q1234sq
      real*8 q1q2,q1q3,q1q4,q2q3,q2q4,q3q4
      real*8 mu1q1,mu1q2,mu1q3,mu1q4,mu2q1,mu2q2,mu2q3,mu2q4,mu3q1,
     &       mu3q2,mu3q3,mu3q4,mu4q1,mu4q2,mu4q3,mu4q4
      real*8 mu1mu2, mu1mu3, mu1mu4, mu2mu3, mu2mu4, mu3mu4
      
c   reduce variables
      real*8 c5mu1a,c5mu2a,c5mu3a,c5mu4a,t6,t7,t8
      real*8 z11,z12,z13,z14,z21,z22,z23,z24, 
     &       z31,z32,z33,z34,z41,z42,z43,z44
      complex*16 res5cd2,res5e2d2,rr5e1a,res5e0
      complex*16 ans,ans1,ans2,ans3,ans4,ans5
      real*8 rans,rans1,rans2,rans3,rans4,rans5,rans6

c   other variables
      integer i1,i2,i3,i4
      integer i,j,mu
      integer id(4)

      integer init,ie0(4,4,4,4)
      data init/0/, ie0/256*0/
      save ie0,init
     
      complex*16 zz
      real*8 dotrr,epsrrrr,xx

      EXTERNAL dotrr,epsrrrr,C0_1M

      logical ldebug
      parameter (ldebug = .false.)

      if (mass.eq.m_top) then
         do i=1,12
            E0(i)=E0t(i)
            do j=1,4
               E1i(j,i)=E1it(j,i)
            enddo
         enddo
      endif
      if (mass.eq.m_bottom) then
         do i=1,12
            E0(i)=E0b(i)
            do j=1,4
               E1i(j,i)=E1ib(j,i)
            enddo
         enddo
      endif

      if (ldebug) then
         print*,' '
         print*,' enter function pent5(',i1,i2,i3,i4,'), ie0 = ',ie0(i1,i2,i3,i4)
         do i = 1,4
            q1(i) = 0
            q2(i) = 0
            q3(i) = 0
            q4(i) = 0
         enddo
         id(1) = i1
         id(2) = i2
         id(3) = i3
         id(4) = i4
c         print*,' input '
c			print*,' '
c         print*,' qi = ',qi
c			print*,' '
c         print*,' eps= ',eps
         do i = 1,4
            do j = 1,4
               print*,' q.q ',i,j,q_dot_q(i,j) !-
c     1                 dotrr(qi(0,i),qi(0,j))
            enddo
         enddo
         do i = 1,4
            do j = 1,4
               print*,' q.e ',i,j,q_dot_ep(i,j) !-
c     1                 dotrr(qi(0,i),eps(0,j))
               q_dot_ep(i,j) = dotrr(qi(0,i),eps(0,j))
            enddo
         enddo
         do i = 1,4
            do j = 1,4
               print*,' e.e ',i,j,ep_dot_ep(i,j) !-
c     1                 dotrr(eps(0,i),eps(0,j))
            enddo
         enddo
c         print*,' '
c         print*,' q_dot_q = ',q_dot_q
c			print*,' '
c         print*,' q_dot_ep = ',q_dot_ep
c			print*,' '
c         print*,' ep_dot_ep = ',ep_dot_ep
      endif

      if (init.eq.0) then   
         init = 1
         ie0(4,2,1,3) = 1   ! this is the map from i1,i2,i3,i4 
         ie0(2,1,3,4) = 2   ! to the index of the E0 variables
         ie0(3,2,1,4) = 3
         ie0(2,1,4,3) = 4
         ie0(4,3,2,1) = 5
         ie0(3,4,2,1) = 6
         ie0(2,3,1,4) = 7
         ie0(3,1,4,2) = 8
         ie0(4,2,3,1) = 9
         ie0(2,4,3,1) = 10
         ie0(3,2,4,1) = 11
         ie0(2,3,4,1) = 12
      endif

      do mu = 0,3
         q1(mu) = qi(mu,i1)
         q2(mu) = qi(mu,i2)
         q3(mu) = qi(mu,i3)
         q4(mu) = qi(mu,i4)

         mu1(mu) = eps(mu,i1)
         mu2(mu) = eps(mu,i2)
         mu3(mu) = eps(mu,i3)
         mu4(mu) = eps(mu,i4)
      enddo

      q1q2 = q_dot_q(i1,i2)
      q1q3 = q_dot_q(i1,i3)
      q1q4 = q_dot_q(i1,i4)
      q2q3 = q_dot_q(i2,i3)
      q2q4 = q_dot_q(i2,i4)
      q3q4 = q_dot_q(i3,i4)

      mu1q1 = q_dot_ep(i1,i1)
      mu1q2 = q_dot_ep(i2,i1)
      mu1q3 = q_dot_ep(i3,i1)
      mu1q4 = q_dot_ep(i4,i1)
           
      mu2q1 = q_dot_ep(i1,i2)
      mu2q2 = q_dot_ep(i2,i2)
      mu2q3 = q_dot_ep(i3,i2)
      mu2q4 = q_dot_ep(i4,i2)
           
      mu3q1 = q_dot_ep(i1,i3)
      mu3q2 = q_dot_ep(i2,i3)
      mu3q3 = q_dot_ep(i3,i3)
      mu3q4 = q_dot_ep(i4,i3)
           
      mu4q1 = q_dot_ep(i1,i4)
      mu4q2 = q_dot_ep(i2,i4)
      mu4q3 = q_dot_ep(i3,i4)
      mu4q4 = q_dot_ep(i4,i4)

      mu1mu2 = ep_dot_ep(i1,i2)
      mu1mu3 = ep_dot_ep(i1,i3)
      mu1mu4 = ep_dot_ep(i1,i4)
      mu2mu3 = ep_dot_ep(i2,i3)
      mu2mu4 = ep_dot_ep(i2,i4)
      mu3mu4 = ep_dot_ep(i3,i4)

      z11 = ep(i1,i1)
      z12 = ep(i1,i2)
      z13 = ep(i1,i3)
      z14 = ep(i1,i4)

      z21 = ep(i2,i1)
      z22 = ep(i2,i2)
      z23 = ep(i2,i3)
      z24 = ep(i2,i4)

      z31 = ep(i3,i1)
      z32 = ep(i3,i2)
      z33 = ep(i3,i3)
      z34 = ep(i3,i4)

      z41 = ep(i4,i1)
      z42 = ep(i4,i2)
      z43 = ep(i4,i3)
      z44 = ep(i4,i4)

      q1sq=q_dot_q(i1,i1)
      q2sq=q_dot_q(i2,i2)
      q3sq=q_dot_q(i3,i3)
      q4sq=q_dot_q(i4,i4)

      q12sq=2*q1q2
      q34sq=2*q3q4
      q1234sq=2*(q1q2+q1q3+q1q4+q2q3+q2q4+q3q4)

      C0L=C0_1M(mass,q12sq,q34sq,q1234sq)   
      CALL D_1M_ALL(mass,q1sq,q2sq,q3sq,q1q2,q1q3,q2q3,Dij123)
      CALL D_1M_ALL(mass,q2sq,q3sq,q4sq,q2q3,q2q4,q3q4,Dij234)
      CALL D_1M_ALL(mass,2*q1q2,q3sq,q4sq,q1q3+q2q3,q1q4+q2q4,q3q4,Dij1p2)
      CALL D_1M_ALL(mass,q1sq,2*q2q3,q4sq,q1q2+q1q3,q1q4,q2q4+q3q4,Dij2p3)
      CALL D_1M_ALL(mass,q1sq,q2sq,2*q3q4,q1q2,q1q3+q1q4,q2q3+q2q4,Dij3p4)
       
      D0_123=Dij123(dd0)
      D0_234=Dij234(dd0)
      D0_1p2=Dij1p2(dd0)
      D0_2p3=Dij2p3(dd0)
      D0_3p4=Dij3p4(dd0)

      E0L = E0(ie0(i1,i2,i3,i4))
      do i = 1,4
         E1iL(i) = E1i(i,ie0(i1,i2,i3,i4))
      enddo

      if (ldebug) then
check expansion of mui in new basis composed of gluon momenta qi
         do mu = 0,3
            xx = z11*q1(mu) + z12*q2(mu) + z13*q3(mu) + z14*q4(mu)
            print*,' mu1(',mu,'): ',mu1(mu),xx,mu1(mu)/xx
            xx = z21*q1(mu) + z22*q2(mu) + z23*q3(mu) + z24*q4(mu)
            print*,' mu2(',mu,'): ',mu2(mu),xx,mu2(mu)/xx
            xx = z31*q1(mu) + z32*q2(mu) + z33*q3(mu) + z34*q4(mu)
            print*,' mu3(',mu,'): ',mu3(mu),xx,mu3(mu)/xx
            xx = z41*q1(mu) + z42*q2(mu) + z43*q3(mu) + z44*q4(mu)
            print*,' mu4(',mu,'): ',mu4(mu),xx,mu4(mu)/xx
         enddo
         print*,' eps(q) ',epsrrrr(q1,q2,q3,q4)
         print*,' eps(mu)',epsrrrr(mu1,mu2,mu3,mu4)
         print*,' E1i = ',(E1i(i,ie0(i1,i2,i3,i4)),i=1,4),E0l
         print*,' D0s = ',D0_123, D0_234, D0_1p2, D0_2p3, D0_3p4,C0l
check E1i properties: result should be 2
         zz = q1q2*e1il(2)+q1q3*e1il(3)+q1q4*e1il(4)
         print*,' q1.E: ',(d0_234-d0_1p2)/zz
         zz = q1q2*e1il(1)+q2q3*e1il(3)+q2q4*e1il(4)
         print*,' q2.E: ',(d0_1p2-d0_2p3-2*q1q2*e0l)/zz
         zz = q1q3*e1il(1)+q2q3*e1il(2)+q3q4*e1il(4)
         print*,' q3.E: ',(d0_2p3-d0_3p4-2*(q1q3+q2q3)*e0l)/zz
         zz = q1q4*e1il(1)+q2q4*e1il(2)+q3q4*e1il(3)
         print*,' q4.E: ',(d0_3p4-d0_123-2*(q1q4+q2q4+q3q4)*e0l)/zz
      endif

      c5mu1a=-epsrrrr(mu2,mu3,mu4,q2)*q3q4+epsrrrr(mu2,mu3,mu4,q3)*q2q4-
     . epsrrrr(mu2,mu3,mu4,q4)*q2q3-2*epsrrrr(mu2,mu3,q2,q3)*(mu4q1+
     . mu4q2+mu4q3)-epsrrrr(mu2,mu3,q2,q4)*mu4q3+epsrrrr(mu2,mu3,q3,q4)
     . *mu4q2+epsrrrr(mu2,mu4,q2,q3)*mu3q4-2*epsrrrr(mu2,mu4,q2,q4)*
     . (mu3q1+mu3q2)-epsrrrr(mu2,mu4,q3,q4)*mu3q2-epsrrrr(mu2,q2,q3,q4)
     . *mu3mu4-epsrrrr(mu3,mu4,q2,q3)*mu2q4+epsrrrr(mu3,mu4,q2,q4)*mu2q3
     . -2*epsrrrr(mu3,mu4,q3,q4)*mu2q1+epsrrrr(mu3,q2,q3,q4)*mu2mu4-
     . epsrrrr(mu4,q2,q3,q4)*mu2mu3

      c5mu2a=-epsrrrr(mu1,mu3,mu4,q1)*q3q4+epsrrrr(mu1,mu3,mu4,q3)*q1q4-
     . epsrrrr(mu1,mu3,mu4,q4)*q1q3-2*epsrrrr(mu1,mu3,q1,q3)*(mu4q1+
     . mu4q2+mu4q3)-epsrrrr(mu1,mu3,q1,q4)*mu4q3+epsrrrr(mu1,mu3,q3,q4)
     . *mu4q1+epsrrrr(mu1,mu4,q1,q3)*mu3q4-2*epsrrrr(mu1,mu4,q1,q4)*
     . (mu3q1+mu3q2)-epsrrrr(mu1,mu4,q3,q4)*mu3q1-epsrrrr(mu1,q1,q3,q4)
     . *mu3mu4-epsrrrr(mu3,mu4,q1,q3)*mu1q4+epsrrrr(mu3,mu4,q1,q4)*mu1q3
     . +2*epsrrrr(mu3,mu4,q3,q4)*(q2q3*z13+q2q4*z14+q3q4*z14)+
     . epsrrrr(mu3,q1,q3,q4)*mu1mu4-epsrrrr(mu4,q1,q3,q4)*mu1mu3

      c5mu3a=-epsrrrr(mu1,mu2,mu4,q1)*q2q4+epsrrrr(mu1,mu2,mu4,q2)*q1q4-
     . epsrrrr(mu1,mu2,mu4,q4)*q1q2-2*epsrrrr(mu1,mu2,q1,q2)*(mu4q1+
     . mu4q2+mu4q3)-epsrrrr(mu1,mu2,q1,q4)*mu4q2+epsrrrr(mu1,mu2,q2,q4)
     . *mu4q1+epsrrrr(mu1,mu4,q1,q2)*mu2q4+2*epsrrrr(mu1,mu4,q1,q4)*
     . (q2q3*z23+q2q4*z24+q3q4*z24)-epsrrrr(mu1,mu4,q2,q4)*mu2q1-
     . epsrrrr(mu1,q1,q2,q4)*mu2mu4-epsrrrr(mu2,mu4,q1,q2)*mu1q4+
     . epsrrrr(mu2,mu4,q1,q4)*mu1q2+2*epsrrrr(mu2,mu4,q2,q4)*(q2q3*z13+
     . q2q4*z14+q3q4*z14)+epsrrrr(mu2,q1,q2,q4)*mu1mu4-
     . epsrrrr(mu4,q1,q2,q4)*mu1mu2

      c5mu4a=-epsrrrr(mu1,mu2,mu3,q1)*q2q3+epsrrrr(mu1,mu2,mu3,q2)*q1q3-
     . epsrrrr(mu1,mu2,mu3,q3)*q1q2+2*epsrrrr(mu1,mu2,q1,q2)*(-q1q2*
     . z31+q3q4*z34)-epsrrrr(mu1,mu2,q1,q3)*mu3q2+epsrrrr(mu1,mu2,q2,q3)
     . *mu3q1+epsrrrr(mu1,mu3,q1,q2)*mu2q3+2*epsrrrr(mu1,mu3,q1,q3)*
     . (-q1q2*z21+q3q4*z24)-epsrrrr(mu1,mu3,q2,q3)*mu2q1-epsrrrr(mu1,
     . q1,q2,q3)*mu2mu3-epsrrrr(mu2,mu3,q1,q2)*mu1q3+epsrrrr(mu2,mu3,q1
     . ,q3)*mu1q2+2*epsrrrr(mu2,mu3,q2,q3)*(q2q3*z13+q2q4*z14+q3q4*z14)
     . +epsrrrr(mu2,q1,q2,q3)*mu1mu3-epsrrrr(mu3,q1,q2,q3)*mu1mu2

      t6=epsrrrr(mu1,mu2,mu3,mu4)*q1q2+epsrrrr(mu1,mu2,mu3,q1)*mu4q2-
     . epsrrrr(mu1,mu2,mu3,q2)*mu4q1-epsrrrr(mu1,mu2,mu4,q1)*mu3q2+
     . epsrrrr(mu1,mu2,mu4,q2)*mu3q1+epsrrrr(mu1,mu2,q1,q2)*mu3mu4-2*
     . epsrrrr(mu1,mu3,mu4,q1)*mu2q1-epsrrrr(mu1,mu3,mu4,q2)*mu2q1-
     . epsrrrr(mu1,mu3,q1,q2)*mu2mu4+epsrrrr(mu1,mu4,q1,q2)*mu2mu3+
     . epsrrrr(mu2,mu3,mu4,q1)*mu1q2+epsrrrr(mu2,mu3,q1,q2)*mu1mu4-
     . epsrrrr(mu2,mu4,q1,q2)*mu1mu3+epsrrrr(mu3,mu4,q1,q2)*mu1mu2

      t7=epsrrrr(mu1,mu2,mu3,mu4)*q1q4-2*epsrrrr(mu1,mu2,mu3,q1)*(mu4q1+
     . mu4q2+mu4q3)-epsrrrr(mu1,mu2,mu3,q4)*mu4q1+epsrrrr(mu1,mu2,
     . mu4,q1)*mu3q4+epsrrrr(mu1,mu2,mu4,q4)*mu3q1+epsrrrr(mu1,mu2,q1,
     . q4)*mu3mu4-epsrrrr(mu1,mu3,mu4,q1)*mu2q4-epsrrrr(mu1,mu3,mu4,q4)
     . *mu2q1-epsrrrr(mu1,mu3,q1,q4)*mu2mu4+epsrrrr(mu1,mu4,q1,q4)*mu2mu3+
     . epsrrrr(mu2,mu3,mu4,q1)*mu1q4+epsrrrr(mu2,mu3,q1,q4)*mu1mu4-
     . epsrrrr(mu2,mu4,q1,q4)*mu1mu3+epsrrrr(mu3,mu4,q1,q4)*mu1mu2

      t8=epsrrrr(mu1,mu2,mu3,mu4)*q3q4-2*epsrrrr(mu1,mu2,mu3,q3)*(mu4q1+
     . mu4q2+mu4q3)-epsrrrr(mu1,mu2,mu3,q4)*mu4q3+
     . epsrrrr(mu1,mu2,mu4,q3)*mu3q4-2*epsrrrr(mu1,mu2,mu4,q4)*(mu3q1+mu3q2)
     . +epsrrrr(mu1,mu2,q3,q4)*mu3mu4-epsrrrr(mu1,mu3,mu4,q3)*mu2q4+
     . epsrrrr(mu1,mu3,mu4,q4)*mu2q3-epsrrrr(mu1,mu3,q3,q4)*mu2mu4+
     . epsrrrr(mu1,mu4,q3,q4)*mu2mu3+epsrrrr(mu2,mu3,mu4,q3)*mu1q4-
     . epsrrrr(mu2,mu3,mu4,q4)*mu1q3+epsrrrr(mu2,mu3,q3,q4)*mu1mu4-
     . epsrrrr(mu2,mu4,q3,q4)*mu1mu3+epsrrrr(mu3,mu4,q3,q4)*mu1mu2

      if (ldebug) then
         print*,' c5muia = ',c5mu1a, c5mu2a, c5mu3a, c5mu4a
         print*,' t6,t7,t8 = ',t6,t7,t8
      endif
c res5cd2;

      res5cd2=epsrrrr(mu1,mu2,mu3,mu4)*c0l+2*epsrrrr(mu1,mu2,mu3,q1)*
     . (mu4q1*Dij2p3(dd11)+mu4q2*Dij2p3(dd12)+mu4q3*Dij2p3(dd12))+2*
     . epsrrrr(mu1,mu2,mu3,q3)*(mu4q1*Dij1p2(dd11)+mu4q2*Dij1p2(dd11)
     . +mu4q3*Dij1p2(dd12))+2*epsrrrr(mu1,mu2,mu4,q4)*(mu3q1*
     . Dij1p2(dd11)+mu3q2*Dij1p2(dd11)+mu3q4*Dij1p2(dd13))+2*
     . epsrrrr(mu1,mu3,mu4,q1)*(mu2q1*Dij3p4(dd11)+mu2q3*Dij3p4(dd13)
     . +mu2q4*Dij3p4(dd13))+2*epsrrrr(mu2,mu3,mu4,q2)*(mu1q2*
     . Dij3p4(dd12)+mu1q3*Dij3p4(dd13)+mu1q4*Dij3p4(dd13))+2*
     . epsrrrr(mu2,mu3,mu4,q4)*(mu1q2*Dij2p3(dd12)+mu1q3*Dij2p3(dd12)
     . +mu1q4*Dij2p3(dd13))-(d0_1p2*t8+d0_2p3*t7+d0_3p4*t6)

c res5e2d2/2;

      ans5=epsrrrr(mu3,mu4,q3,q4)*(mu2q1*Dij123(dd11)*z14+mu2q1*
     . Dij1p2(dd11)*z11-mu2q1*Dij1p2(dd11)*z12+mu2q1*Dij2p3(dd11)*
     . z12-mu2q1*Dij2p3(dd11)*z13+mu2q1*Dij3p4(dd11)*z13-mu2q1*
     . Dij3p4(dd11)*z14+mu2q3*Dij123(dd13)*z14+mu2q3*Dij1p2(dd12)*
     . z11-mu2q3*Dij1p2(dd12)*z12-mu2q3*Dij234(dd12)*z11+mu2q3*
     . Dij2p3(dd12)*z12-mu2q3*Dij2p3(dd12)*z13+mu2q3*Dij3p4(dd13)*
     . z13-mu2q3*Dij3p4(dd13)*z14+mu2q4*Dij1p2(dd13)*z11-mu2q4*
     . Dij1p2(dd13)*z12-mu2q4*Dij234(dd13)*z11+mu2q4*Dij2p3(dd13)*
     . z12-mu2q4*Dij2p3(dd13)*z13+mu2q4*Dij3p4(dd13)*z13-mu2q4*
     . Dij3p4(dd13)*z14)
      ans4=epsrrrr(mu2,mu4,q2,q4)*(mu3q1*Dij123(dd11)*z14+mu3q1*
     . Dij1p2(dd11)*z11-mu3q1*Dij1p2(dd11)*z12+mu3q1*Dij2p3(dd11)*
     . z12-mu3q1*Dij2p3(dd11)*z13+mu3q1*Dij3p4(dd11)*z13-mu3q1*
     . Dij3p4(dd11)*z14+mu3q2*Dij123(dd12)*z14+mu3q2*Dij1p2(dd11)*
     . z11-mu3q2*Dij1p2(dd11)*z12-mu3q2*Dij234(dd11)*z11+mu3q2*
     . Dij2p3(dd12)*z12-mu3q2*Dij2p3(dd12)*z13+mu3q2*Dij3p4(dd12)*
     . z13-mu3q2*Dij3p4(dd12)*z14+mu3q4*Dij1p2(dd13)*z11-mu3q4*
     . Dij1p2(dd13)*z12-mu3q4*Dij234(dd13)*z11+mu3q4*Dij2p3(dd13)*
     . z12-mu3q4*Dij2p3(dd13)*z13+mu3q4*Dij3p4(dd13)*z13-mu3q4*
     . Dij3p4(dd13)*z14)+ans5
      ans3=epsrrrr(mu2,mu3,q2,q3)*(mu4q1*Dij123(dd11)*z14+mu4q1*
     . Dij1p2(dd11)*z11-mu4q1*Dij1p2(dd11)*z12+mu4q1*Dij2p3(dd11)*
     . z12-mu4q1*Dij2p3(dd11)*z13+mu4q1*Dij3p4(dd11)*z13-mu4q1*
     . Dij3p4(dd11)*z14+mu4q2*Dij123(dd12)*z14+mu4q2*Dij1p2(dd11)*
     . z11-mu4q2*Dij1p2(dd11)*z12-mu4q2*Dij234(dd11)*z11+mu4q2*
     . Dij2p3(dd12)*z12-mu4q2*Dij2p3(dd12)*z13+mu4q2*Dij3p4(dd12)*
     . z13-mu4q2*Dij3p4(dd12)*z14+mu4q3*Dij123(dd13)*z14+mu4q3*
     . Dij1p2(dd12)*z11-mu4q3*Dij1p2(dd12)*z12-mu4q3*Dij234(dd12)*
     . z11+mu4q3*Dij2p3(dd12)*z12-mu4q3*Dij2p3(dd12)*z13+mu4q3*
     . Dij3p4(dd13)*z13-mu4q3*Dij3p4(dd13)*z14)+ans4
      ans2=epsrrrr(mu1,mu4,q1,q4)*(mu3q1*Dij123(dd11)*z24+mu3q1*
     . Dij1p2(dd11)*z21-mu3q1*Dij1p2(dd11)*z22+mu3q1*Dij2p3(dd11)*
     . z22-mu3q1*Dij2p3(dd11)*z23+mu3q1*Dij3p4(dd11)*z23-mu3q1*
     . Dij3p4(dd11)*z24+mu3q2*Dij123(dd12)*z24+mu3q2*Dij1p2(dd11)*
     . z21-mu3q2*Dij1p2(dd11)*z22-mu3q2*Dij234(dd11)*z21+mu3q2*
     . Dij2p3(dd12)*z22-mu3q2*Dij2p3(dd12)*z23+mu3q2*Dij3p4(dd12)*
     . z23-mu3q2*Dij3p4(dd12)*z24+mu3q4*Dij1p2(dd13)*z21-mu3q4*
     . Dij1p2(dd13)*z22-mu3q4*Dij234(dd13)*z21+mu3q4*Dij2p3(dd13)*
     . z22-mu3q4*Dij2p3(dd13)*z23+mu3q4*Dij3p4(dd13)*z23-mu3q4*
     . Dij3p4(dd13)*z24)+ans3
      ans1=epsrrrr(mu1,mu3,q1,q3)*(mu4q1*Dij123(dd11)*z24+mu4q1*
     . Dij1p2(dd11)*z21-mu4q1*Dij1p2(dd11)*z22+mu4q1*Dij2p3(dd11)*
     . z22-mu4q1*Dij2p3(dd11)*z23+mu4q1*Dij3p4(dd11)*z23-mu4q1*
     . Dij3p4(dd11)*z24+mu4q2*Dij123(dd12)*z24+mu4q2*Dij1p2(dd11)*
     . z21-mu4q2*Dij1p2(dd11)*z22-mu4q2*Dij234(dd11)*z21+mu4q2*
     . Dij2p3(dd12)*z22-mu4q2*Dij2p3(dd12)*z23+mu4q2*Dij3p4(dd12)*
     . z23-mu4q2*Dij3p4(dd12)*z24+mu4q3*Dij123(dd13)*z24+mu4q3*
     . Dij1p2(dd12)*z21-mu4q3*Dij1p2(dd12)*z22-mu4q3*Dij234(dd12)*
     . z21+mu4q3*Dij2p3(dd12)*z22-mu4q3*Dij2p3(dd12)*z23+mu4q3*
     . Dij3p4(dd13)*z23-mu4q3*Dij3p4(dd13)*z24)+ans2
      ans=epsrrrr(mu1,mu2,q1,q2)*(mu4q1*Dij123(dd11)*z34+mu4q1*
     . Dij1p2(dd11)*z31-mu4q1*Dij1p2(dd11)*z32+mu4q1*Dij2p3(dd11)*
     . z32-mu4q1*Dij2p3(dd11)*z33+mu4q1*Dij3p4(dd11)*z33-mu4q1*
     . Dij3p4(dd11)*z34+mu4q2*Dij123(dd12)*z34+mu4q2*Dij1p2(dd11)*
     . z31-mu4q2*Dij1p2(dd11)*z32-mu4q2*Dij234(dd11)*z31+mu4q2*
     . Dij2p3(dd12)*z32-mu4q2*Dij2p3(dd12)*z33+mu4q2*Dij3p4(dd12)*
     . z33-mu4q2*Dij3p4(dd12)*z34+mu4q3*Dij123(dd13)*z34+mu4q3*
     . Dij1p2(dd12)*z31-mu4q3*Dij1p2(dd12)*z32-mu4q3*Dij234(dd12)*
     . z31+mu4q3*Dij2p3(dd12)*z32-mu4q3*Dij2p3(dd12)*z33+mu4q3*
     . Dij3p4(dd13)*z33-mu4q3*Dij3p4(dd13)*z34)+ans1

      res5e2d2 = ans*2 + 2*d0_234*(epsrrrr(mu1,mu2,q1,q2)*mu4q1*z31 + 
     . epsrrrr(mu1,mu3,q1,q3)*mu4q1*z21 + epsrrrr(mu1,mu4,q1,q4)*mu3q1*z21 +
     . epsrrrr(mu2,mu3,q2,q3)*mu4q1*z11 + epsrrrr(mu2,mu4,q2,q4)*mu3q1*z11 + 
     . epsrrrr(mu3,mu4,q3,q4)*mu2q1*z11)

C rr5e1a/2;

      ans=c5mu1a*(mu1q2*e1il(2)+mu1q3*e1il(3)+mu1q4*e1il(4))
     . +c5mu2a*(mu2q1*e1il(1)+mu2q3*e1il(3)+mu2q4*e1il(4))+
     . c5mu3a*(mu3q1*e1il(1)+mu3q2*e1il(2)+mu3q4*e1il(4))+
     . c5mu4a*(mu4q1*e1il(1)+mu4q2*e1il(2)+mu4q3*e1il(3))

      rr5e1a = ans*2

c res5e0;

      rans6=epsrrrr(mu4,q1,q2,q4)*(-2*mu1mu2*mu3q1-2*mu1mu2*mu3q2+
     . mu1mu3*mu2q3-mu1q3*mu2mu3)+epsrrrr(mu4,q1,q3,q4)*(-mu1mu2*
     . mu3q2-2*mu1mu3*mu2q1+mu1q2*mu2mu3)+epsrrrr(mu4,q2,q3,q4)*
     . (mu1mu2*mu3q1-mu1mu3*mu2q1)+epsrrrr(q1,q2,q3,q4)*(mu1mu2*
     . mu3mu4-mu1mu3*mu2mu4+mu1mu4*mu2mu3)
      rans5=epsrrrr(mu2,q1,q3,q4)*(-mu1mu3*mu4q2+mu1mu4*mu3q2+mu1q2
     . *mu3mu4)+epsrrrr(mu2,q2,q3,q4)*(mu1mu3*mu4q1-mu1mu4*mu3q1)+
     . epsrrrr(mu3,mu4,q1,q2)*(mu1mu2*q3q4-mu1q3*mu2q4+mu1q4*mu2q3)+
     . epsrrrr(mu3,mu4,q1,q3)*(-mu1mu2*q2q4+mu1q2*mu2q4-2*mu1q4*mu2q1)+
     . epsrrrr(mu3,mu4,q1,q4)*(mu1mu2*q2q3-mu1q2*mu2q3+
     . 2*mu1q3*mu2q1)+epsrrrr(mu3,mu4,q2,q3)*(mu1mu2*q1q4-mu1q4*
     . mu2q1)+epsrrrr(mu3,mu4,q2,q4)*(-mu1mu2*q1q3+mu1q3*mu2q1)+
     . epsrrrr(mu3,mu4,q3,q4)*(mu1mu2*q1q2-mu1q2*mu2q1)+epsrrrr(mu3,
     . q1,q2,q3)*(-2*mu1mu2*mu4q1-2*mu1mu2*mu4q2-2*mu1mu2*
     . mu4q3+mu1mu4*mu2q4-mu1q4*mu2mu4)+epsrrrr(mu3,q1,q2,q4)*(-
     . mu1mu2*mu4q3-mu1mu4*mu2q3+mu1q3*mu2mu4)+epsrrrr(mu3,q1,q3,q4)*
     . (mu1mu2*mu4q2+2*mu1mu4*mu2q1-mu1q2*mu2mu4)+epsrrrr(mu3,q2,q3,q4)*  
     . (-mu1mu2*mu4q1+mu1mu4*mu2q1)+epsrrrr(mu4,q1,q2,q3)*
     . (mu1mu2*mu3q4-mu1mu3*mu2q4+mu1q4*mu2mu3)+rans6
      rans4=epsrrrr(mu2,mu3,q1,q3)*(-mu1mu4*q2q4+2*mu1q2*mu4q1+
     . 2*mu1q2*mu4q2+2*mu1q2*mu4q3+mu1q4*mu4q2)+epsrrrr(mu2,mu3,
     . q1,q4)*(mu1mu4*q2q3+mu1q2*mu4q3-mu1q3*mu4q2)+epsrrrr(mu2,
     . mu3,q2,q3)*(mu1mu4*q1q4-mu1q4*mu4q1)+epsrrrr(mu2,mu3,q2,q4)*
     . (-mu1mu4*q1q3+mu1q3*mu4q1)+epsrrrr(mu2,mu3,q3,q4)*(mu1mu4*
     . q1q2-mu1q2*mu4q1)+epsrrrr(mu2,mu4,q1,q2)*(-mu1mu3*q3q4+mu1q3*
     . mu3q4-2*mu1q4*mu3q1-2*mu1q4*mu3q2)+epsrrrr(mu2,mu4,q1,q3)*
     . (mu1mu3*q2q4-mu1q2*mu3q4-mu1q4*mu3q2)+epsrrrr(mu2,
     . mu4,q1,q4)*(-mu1mu3*q2q3+2*mu1q2*mu3q1+2*mu1q2*mu3q2
     . +mu1q3*mu3q2)+epsrrrr(mu2,mu4,q2,q3)*(-mu1mu3*q1q4+mu1q4*
     . mu3q1)+epsrrrr(mu2,mu4,q2,q4)*(mu1mu3*q1q3-mu1q3*mu3q1)+
     . epsrrrr(mu2,mu4,q3,q4)*(-mu1mu3*q1q2+mu1q2*mu3q1)+epsrrrr(mu2,
     . q1,q2,q3)*(2*mu1mu3*mu4q1+2*mu1mu3*mu4q2+2*mu1mu3*
     . mu4q3-mu1mu4*mu3q4+mu1q4*mu3mu4)+epsrrrr(mu2,q1,q2,q4)*(mu1mu3*
     . mu4q3+2*mu1mu4*mu3q1+2*mu1mu4*mu3q2-mu1q3*mu3mu4)+rans5
      rans3=epsrrrr(mu1,mu4,q2,q4)*(-mu2mu3*q1q3-2*mu2q1*mu3q1-
     . 2*mu2q1*mu3q2+mu2q3*mu3q1)+epsrrrr(mu1,mu4,q3,q4)*(mu2mu3*q1q2-
     . 2*mu2q1*mu3q1-mu2q1*mu3q2)+epsrrrr(mu1,q1,q2,q3)*(-
     . 2*mu2mu3*mu4q1-2*mu2mu3*mu4q2-2*mu2mu3*mu4q3+mu2mu4*
     . mu3q4-mu2q4*mu3mu4)+epsrrrr(mu1,q1,q2,q4)*(-mu2mu3*mu4q3-
     . 2*mu2mu4*mu3q1-2*mu2mu4*mu3q2+mu2q3*mu3mu4)+
     . epsrrrr(mu1,q1,q3,q4)*(mu2mu3*mu4q2-mu2mu4*mu3q2-2*mu2q1*
     . mu3mu4)+epsrrrr(mu1,q2,q3,q4)*(-mu2mu3*mu4q1+mu2mu4*mu3q1-
     . mu2q1*mu3mu4)+epsrrrr(mu2,mu3,mu4,q1)*(mu1q2*q3q4-mu1q3*q2q4+
     . mu1q4*q2q3)+epsrrrr(mu2,mu3,mu4,q2)*(mu1q3*q1q4-mu1q4*q1q3)+
     . epsrrrr(mu2,mu3,mu4,q3)*(-mu1q2*q1q4+mu1q4*q1q2)+
     . epsrrrr(mu2,mu3,mu4,q4)*(mu1q2*q1q3-mu1q3*q1q2)+
     . epsrrrr(mu2,mu3,q1,q2)*(mu1mu4*q3q4-2*mu1q3*mu4q1-2*mu1q3*mu4q2-
     . 2*mu1q3*mu4q3-mu1q4*mu4q3)+rans4
      rans2=epsrrrr(mu1,mu3,q1,q2)*(-mu2mu4*q3q4+2*mu2q3*mu4q1+
     . 2*mu2q3*mu4q2+2*mu2q3*mu4q3+mu2q4*mu4q3)+epsrrrr(mu1,mu3,
     . q1,q3)*(mu2mu4*q2q4-4*mu2q1*mu4q1-4*mu2q1*mu4q2-
     . 4*mu2q1*mu4q3-mu2q4*mu4q2)+epsrrrr(mu1,mu3,q1,q4)*(-mu2mu4*
     . q2q3-2*mu2q1*mu4q3+mu2q3*mu4q2)+epsrrrr(mu1,mu3,q2,q3)*(-
     . mu2mu4*q1q4-2*mu2q1*mu4q1-2*mu2q1*mu4q2-2*mu2q1*
     . mu4q3+mu2q4*mu4q1)+epsrrrr(mu1,mu3,q2,q4)*(mu2mu4*q1q3-
     . mu2q1*mu4q3-mu2q3*mu4q1)+epsrrrr(mu1,mu3,q3,q4)*(-mu2mu4*q1q2+
     . 2*mu2q1*mu4q1+mu2q1*mu4q2)+epsrrrr(mu1,mu4,q1,q2)*(mu2mu3
     . *q3q4-mu2q3*mu3q4+2*mu2q4*mu3q1+2*mu2q4*mu3q2)+
     . epsrrrr(mu1,mu4,q1,q3)*(-mu2mu3*q2q4+2*mu2q1*mu3q4+mu2q4*
     . mu3q2)+epsrrrr(mu1,mu4,q1,q4)*(mu2mu3*q2q3-4*mu2q1*mu3q1-
     . 4*mu2q1*mu3q2-mu2q3*mu3q2)+epsrrrr(mu1,mu4,q2,q3)*(mu2mu3
     . *q1q4+mu2q1*mu3q4-mu2q4*mu3q1)+rans3
      rans1=epsrrrr(mu1,mu2,q1,q2)*(mu3mu4*q3q4-4*mu3q1*mu4q1-4*
     . mu3q1*mu4q2-4*mu3q1*mu4q3-4*mu3q2*mu4q1-4*mu3q2*
     . mu4q2-4*mu3q2*mu4q3-mu3q4*mu4q3)+epsrrrr(mu1,mu2,q1,q3)*(
     . -mu3mu4*q2q4-2*mu3q2*mu4q1-2*mu3q2*mu4q2-2*mu3q2*
     . mu4q3+mu3q4*mu4q2)+epsrrrr(mu1,mu2,q1,q4)*(mu3mu4*q2q3-2*
     . mu3q1*mu4q2-2*mu3q2*mu4q2-mu3q2*mu4q3)+epsrrrr(mu1,mu2,
     . q2,q3)*(mu3mu4*q1q4+2*mu3q1*mu4q1+2*mu3q1*mu4q2+2*
     . mu3q1*mu4q3-mu3q4*mu4q1)+epsrrrr(mu1,mu2,q2,q4)*(-mu3mu4*
     . q1q3+2*mu3q1*mu4q1+mu3q1*mu4q3+2*mu3q2*mu4q1)+epsrrrr(
     . mu1,mu2,q3,q4)*(mu3mu4*q1q2-mu3q1*mu4q2+mu3q2*mu4q1)+
     . epsrrrr(mu1,mu3,mu4,q1)*(-2*mu2q1*q3q4+mu2q3*q2q4-mu2q4*
     . q2q3)+epsrrrr(mu1,mu3,mu4,q2)*(-mu2q1*q3q4-mu2q3*q1q4+
     . mu2q4*q1q3)+epsrrrr(mu1,mu3,mu4,q3)*(2*mu2q1*q1q4+mu2q1*q2q4
     . -mu2q4*q1q2)+epsrrrr(mu1,mu3,mu4,q4)*(-2*mu2q1*q1q3-mu2q1
     . *q2q3+mu2q3*q1q2)+rans2
      rans=epsrrrr(mu1,mu2,mu3,mu4)*(q1q2*q3q4-q1q3*q2q4+q1q4*q2q3
     . )+epsrrrr(mu1,mu2,mu3,q1)*(-2*mu4q1*q2q3-2*mu4q2*q2q3+
     . mu4q2*q3q4-2*mu4q3*q2q3-mu4q3*q2q4)+epsrrrr(mu1,mu2,mu3,
     . q2)*(2*mu4q1*q1q3-mu4q1*q3q4+2*mu4q2*q1q3+2*mu4q3
     . *q1q3+mu4q3*q1q4)+epsrrrr(mu1,mu2,mu3,q3)*(-2*mu4q1*q1q2+
     . mu4q1*q2q4-2*mu4q2*q1q2-mu4q2*q1q4-2*mu4q3*q1q2)+
     . epsrrrr(mu1,mu2,mu3,q4)*(-mu4q1*q2q3+mu4q2*q1q3-mu4q3*q1q2)
     . +epsrrrr(mu1,mu2,mu4,q1)*(-2*mu3q1*q2q4-2*mu3q2*q2q4-
     . mu3q2*q3q4+mu3q4*q2q3)+epsrrrr(mu1,mu2,mu4,q2)*(2*mu3q1*
     . q1q4+mu3q1*q3q4+2*mu3q2*q1q4-mu3q4*q1q3)+epsrrrr(mu1,
     . mu2,mu4,q3)*(-mu3q1*q2q4+mu3q2*q1q4+mu3q4*q1q2)+epsrrrr(
     . mu1,mu2,mu4,q4)*(-2*mu3q1*q1q2+mu3q1*q2q3-2*mu3q2*
     . q1q2-mu3q2*q1q3)+rans1
      res5e0 = rans*E0L

      A4G_Loop = res5cd2 + res5e2d2 + rr5e1a + res5e0

      if (ldebug) then
         print*,' res5cd2  = ', res5cd2 
         print*,' res5e2d2 = ', res5e2d2
         print*,' rr5e1a   = ', rr5e1a
         print*,' res5e0   = ', res5e0
         print*,' pent5    = ', A4G_Loop, A4G_Loop/res5e2d2
      endif
            
      RETURN
      
      END


*----------------------------------------------------------------------
      FUNCTION H4G_Loop(i1,i2,i3,i4,mass)
*----------------------------------------------------------------------
c   This routine calculates the Higgs + 4 gluon massive pentagon loop.
c   The code has been generated with Mathematica. 
c   The integers i1,i2,i3,i4 define the permutation of the gluons. 
c   Gluon momenta, polarization vectors and dot-products are stored in 
c   q_dot_ep, ep_dot_ep and q_dot_q.

c   Normalization: this subroutine gives the pentagon contributions
c   without a factor 4*i*mt the QCD couplings and Yukawa coupling h_t 
c   (gamma_5). Thus, for the full amplitude multiply by an overall
c   normalization factor of 4 mt h_t alpha_s**2 c_i .

c                     * -- i1    
c                   /   \
c           i2 -- *      \
c                 |       * -- H
c           i3 -- *      /
c                   \   /
c                     * -- i4

*----------------------------------------------------------------------
c   author: Gunnar Klaemke
c   date  : 30.08.2006
***********************************************************************

      IMPLICIT NONE

#include "ggf_process.inc"
#include "d_coefficients.inc"
#include "ggf_dotproducts.inc"

      integer i1,i2,i3,i4
      real*8 mass
      complex*16 H4G_Loop

c local dot products and other variables
      complex*16 D0123, D0234, D01p2, D02p3, D03p4
      complex*16 Dij123(24), Dij234(24), Dij1p2(24), 
     1           Dij2p3(24), Dij3p4(24),E0(12),E0L,E1i(4,12),E1iL(4)
      integer ij,kl,i,j,mu
      integer init,ie0(4,4,4,4)
      data init/0/, ie0/256*0/
      save ie0,init
c
c  explicit momenta for call of eps tensor

      real*8 q1sq, q2sq, q3sq, q4sq
      real*8 q1q2, q1q3, q1q4, q2q3, q2q4, q3q4
      real*8 mu1q1, mu1q2, mu1q3, mu1q4,
     1       mu2q1, mu2q2, mu2q3, mu2q4, 
     1       mu3q1, mu3q2, mu3q3, mu3q4,
     1       mu4q1, mu4q2, mu4q3, mu4q4
      real*8 mu1mu2, mu1mu3, mu1mu4, mu2mu3, mu2mu4, mu3mu4
c
c reduce variables
      real*8 z11, z12, z13, z14,   z21, z22, z23, z24, 
     1       z31, z32, z33, z34,   z41, z42, z43, z44
     
      
      complex*16 kk(1200)

      if (mass.eq.m_top) then
         do i=1,12
            E0(i)=E0t(i)
            do j=1,4
               E1i(j,i)=E1it(j,i)
            enddo
         enddo
      endif
      if (mass.eq.m_bottom) then
         do i=1,12
            E0(i)=E0b(i)
            do j=1,4
               E1i(j,i)=E1ib(j,i)
            enddo
         enddo
      endif

      if (init.eq.0) then   
         init = 1
         ie0(4,2,1,3) = 1   ! this is the map from i1,i2,i3,i4 
         ie0(2,1,3,4) = 2   ! to the index of the E0 variables
         ie0(3,2,1,4) = 3
         ie0(2,1,4,3) = 4
         ie0(4,3,2,1) = 5
         ie0(3,4,2,1) = 6
         ie0(2,3,1,4) = 7
         ie0(3,1,4,2) = 8
         ie0(4,2,3,1) = 9
         ie0(2,4,3,1) = 10
         ie0(3,2,4,1) = 11
         ie0(2,3,4,1) = 12
      endif
 
      q1sq=0.0d0
      q2sq=0.0d0
      q3sq=0.0d0
      q4sq=0.0d0

      q1q2 = q_dot_q(i1,i2)
      q1q3 = q_dot_q(i1,i3)
      q1q4 = q_dot_q(i1,i4)
      q2q3 = q_dot_q(i2,i3)
      q2q4 = q_dot_q(i2,i4)
      q3q4 = q_dot_q(i3,i4)

      mu1q1 = q_dot_ep(i1,i1)
      mu1q2 = q_dot_ep(i2,i1)
      mu1q3 = q_dot_ep(i3,i1)
      mu1q4 = q_dot_ep(i4,i1)
           
      mu2q1 = q_dot_ep(i1,i2)
      mu2q2 = q_dot_ep(i2,i2)
      mu2q3 = q_dot_ep(i3,i2)
      mu2q4 = q_dot_ep(i4,i2)
           
      mu3q1 = q_dot_ep(i1,i3)
      mu3q2 = q_dot_ep(i2,i3)
      mu3q3 = q_dot_ep(i3,i3)
      mu3q4 = q_dot_ep(i4,i3)
           
      mu4q1 = q_dot_ep(i1,i4)
      mu4q2 = q_dot_ep(i2,i4)
      mu4q3 = q_dot_ep(i3,i4)
      mu4q4 = q_dot_ep(i4,i4)

      mu1mu2 = ep_dot_ep(i1,i2)
      mu1mu3 = ep_dot_ep(i1,i3)
      mu1mu4 = ep_dot_ep(i1,i4)
      mu2mu3 = ep_dot_ep(i2,i3)
      mu2mu4 = ep_dot_ep(i2,i4)
      mu3mu4 = ep_dot_ep(i3,i4)

      z11 = ep(i1,i1)
      z12 = ep(i1,i2)
      z13 = ep(i1,i3)
      z14 = ep(i1,i4)

      z21 = ep(i2,i1)
      z22 = ep(i2,i2)
      z23 = ep(i2,i3)
      z24 = ep(i2,i4)

      z31 = ep(i3,i1)
      z32 = ep(i3,i2)
      z33 = ep(i3,i3)
      z34 = ep(i3,i4)

      z41 = ep(i4,i1)
      z42 = ep(i4,i2)
      z43 = ep(i4,i3)
      z44 = ep(i4,i4)

      E0L = -1.0*E0(ie0(i1,i2,i3,i4))
      do i = 1,4
         E1iL(i) = -E1i(i,ie0(i1,i2,i3,i4))
      enddo

      CALL D_1M_ALL(mass, q1sq, q2sq, q3sq, q1q2, q1q3, q2q3, Dij123)
      CALL D_1M_ALL(mass, q2sq, q3sq, q4sq, q2q3, q2q4, q3q4, Dij234)
      CALL D_1M_ALL(mass, 2*q1q2, q3sq, q4sq, q1q3+q2q3, q1q4+q2q4, q3q4, Dij1p2)
      CALL D_1M_ALL(mass, q1sq, 2*q2q3, q4sq, q1q2+q1q3, q1q4, q2q4+q3q4, Dij2p3)
      CALL D_1M_ALL(mass, q1sq, q2sq, 2*q3q4, q1q2, q1q3+q1q4, q2q3+q2q4, Dij3p4)
       
      D0123=Dij123(dd0)
      D0234=Dij234(dd0)
      D01p2=Dij1p2(dd0)
      D02p3=Dij2p3(dd0)
      D03p4=Dij3p4(dd0)
  
      
        KK(1)= 2*mu4q1 + mu4q2
        KK(2)= mass**2 - q1q2
        KK(3)= mu2q1*KK(1) + mu2mu4*KK(2)
        KK(4)= -2*mu3q1 - mu3q2
        KK(5)= -mass**2 + q1q2
        KK(6)= mu2q1*KK(4) + mu2mu3*KK(5)
        KK(7)= mu3q2*mu4q1 - mu3q1*mu4q2 + mu3mu4*KK(5)
        KK(8)= -(mu1q2*mu2q1*mu3mu4) + mu1q2*mu2mu4*mu3q1 - 
     &  mu1q2*mu2mu3*mu4q1 + mu1mu3*KK(3) + mu1mu4*KK(6) + 
     &  mu1mu2*KK(7)
        KK(9)= -(mu2q4*mu3q1) + mu2q1*mu3q4 + mu2mu3*q1q4
        KK(10)= -2*mu4q1 - 2*mu4q2 - 2*mu4q3
        KK(11)= mu2q4*mu4q1 - mu2mu4*q1q4 + mu2q1*KK(10)
        KK(12)= 2*mu4q1 + 2*mu4q2 + 2*mu4q3
        KK(13)= -(mu3q4*mu4q1) + mu3mu4*q1q4 + mu3q1*KK(12)
        KK(14)= -(mu1q4*mu2q1*mu3mu4) + mu1q4*mu2mu4*mu3q1 - 
     &  mu1q4*mu2mu3*mu4q1 + mu1mu4*KK(9) + mu1mu3*KK(11) + 
     &  mu1mu2*KK(13)
        KK(15)= mu1q4*mu2q3 - mu1q3*mu2q4
        KK(16)= -2*mu1q4*mu3q1 - 2*mu1q4*mu3q2 + mu1q3*mu3q4
        KK(17)= -2*mu1q3*mu4q1 - 2*mu1q3*mu4q2 - 2*mu1q3*mu4q3 - mu1q4*mu4q3
        KK(18)= -2*q1q2 - 2*q1q3 - 2*q2q3 - q3q4
        KK(19)= 2*mu2q3*mu4q1 + 2*mu2q3*mu4q2 + 2*mu2q3*mu4q3 + 
     &  mu2q4*mu4q3 + mu2mu4*KK(18)
        KK(20)= 2*q1q2 + 2*q1q3 + 2*q2q3 + q3q4
        KK(21)= 2*mu2q4*mu3q1 + 2*mu2q4*mu3q2 - mu2q3*mu3q4 + mu2mu3*KK(20)
        KK(22)= -4*mu4q2 - 4*mu4q3
        KK(23)= -4*mu4q1 - 4*mu4q2 - 4*mu4q3
        KK(24)= -4*mu3q2*mu4q1 - mu3q4*mu4q3 + mu3mu4*KK(20) + 
     &  mu3q2*KK(22) + mu3q1*KK(23)
        KK(25)= mu3mu4*KK(15) + mu2mu4*KK(16) + mu2mu3*KK(17) + 
     &  mu1mu3*KK(19) + mu1mu4*KK(21) + mu1mu2*KK(24)
        KK(26)= -2*mu2q4*mu3q2 + mu2q3*mu3q4
        KK(27)= mu3q2*KK(15) + mu1q2*KK(26)
        KK(28)= -2*mu2q3 - 2*mu2q4
        KK(29)= -(mu1q4*mu2q3) + mu1q3*mu2q4
        KK(30)= -2*mu2q3*mu4q2 - 2*mu2q3*mu4q3 - mu2q4*mu4q3
        KK(31)= mu1q2*mu4q1*KK(28) + mu4q2*KK(29) + mu1q2*KK(30)
        KK(32)= 2*mu1q3 + 4*mu1q4
        KK(33)= 4*mu1q2*mu3q2 - 2*mu1q3*mu3q4 + mu3q2*KK(32)
        KK(34)= 2*mu1q3 + 2*mu1q4
        KK(35)= 2*mu1q3*mu4q3 + mu1q4*mu4q3 + mu4q2*KK(34)
        KK(36)= 4*mu1q2 + 4*mu1q3 + 4*mu1q4
        KK(37)= 4*mu1q3 + 2*mu1q4
        KK(38)= 4*mu4q2 + 4*mu4q3
        KK(39)= 4*mu1q3*mu4q3 + 2*mu1q4*mu4q3 + mu4q1*KK(36) + 
     &  mu4q2*KK(37) + mu1q2*KK(38)
        KK(40)= mu3q4*mu4q3 + mu3q2*KK(38)
        KK(41)= -(mu1q3*mu3q4*mu4q2) + mu4q1*KK(33) + mu3q2*KK(35) + 
     &  mu3q1*KK(39) + mu1q2*KK(40)
        KK(42)= -2*q1q2 - q1q4
        KK(43)= mu1q4*q1q3 + mu1q3*KK(42)
        KK(44)= 2*q1q3 + q1q4
        KK(45)= 2*mu4q2*q1q3 + mu4q3*KK(44)
        KK(46)= -2*q1q2 + q2q4
        KK(47)= 2*q1q3 - q3q4
        KK(48)= -(mu1q4*q2q3) + mu1q3*KK(46) + mu1q2*KK(47)
        KK(49)= -2*mu1q3*mu4q3*q1q2 - mu1q4*mu4q3*q1q2 + 
     &  mu4q2*KK(43) + mu1q2*KK(45) + mu4q1*KK(48)
        KK(50)= mu2q4*q1q3 - mu2q3*q1q4
        KK(51)= -2*q1q3 - q2q3
        KK(52)= 2*q1q4 + q2q4
        KK(53)= -(mu1q2*q3q4) + mu1q4*KK(51) + mu1q3*KK(52)
        KK(54)= mu1q4*mu2q3*q1q2 - mu1q3*mu2q4*q1q2 + mu1q2*KK(50) + 
     &  mu2q1*KK(53)
        KK(55)= 2*q1q2 + q1q3
        KK(56)= -(mu2q3*q1q4) + mu2q4*KK(55)
        KK(57)= 2*q1q2 - q2q3
        KK(58)= mu2q3*q2q4 + mu2q4*KK(57)
        KK(59)= 2*q1q3 + q2q3
        KK(60)= -4*q1q4 - 2*q2q4 - 2*q3q4
        KK(61)= -4*q1q4 - 2*q2q4 - q3q4
        KK(62)= mu3q4*KK(59) + mu3q1*KK(60) + mu3q2*KK(61)
        KK(63)= q1q4*q2q3 - q1q3*q2q4 + q1q2*q3q4
        KK(64)= -(mu2q3*mu3q4*q1q2) + mu3q2*KK(56) + mu3q1*KK(58) + 
     &  mu2q1*KK(62) + mu2mu3*KK(63)
        KK(65)= -2*q1q2 - q1q3
        KK(66)= mu1q3*q1q4 + mu1q4*KK(65)
        KK(67)= -(mu3q4*q1q3) + 2*mu3q2*q1q4
        KK(68)= -2*q1q2 + q2q3
        KK(69)= 2*q1q4 + q3q4
        KK(70)= -(mu1q3*q2q4) + mu1q4*KK(68) + mu1q2*KK(69)
        KK(71)= mu1q3*mu3q4*q1q2 + mu3q2*KK(66) + mu1q2*KK(67) + mu3q1*KK(70)
        KK(72)= -4*q1q2 - 2*q1q3 - 2*q1q4
        KK(73)= -4*q1q2 - 2*q1q3 - q1q4
        KK(74)= mu4q2*KK(72) + mu4q3*KK(73)
        KK(75)= -4*q1q2 + 2*q2q3 + q2q4
        KK(76)= -4*q1q2 + 2*q2q3 + 2*q2q4
        KK(77)= -4*q1q2 - 2*q1q4 + 2*q2q3 - q3q4
        KK(78)= mu4q3*KK(75) + mu4q1*KK(76) + mu4q2*KK(77)
        KK(79)= -4*q1q2 - 2*q1q3 + 2*q2q4 + q3q4
        KK(80)= -(mu3q4*q2q3) + mu3q2*KK(79)
        KK(81)= -(mu3q4*mu4q3*q1q2) + mu3q4*mu4q2*q1q3 + 
     &  mu3mu4*KK(63) + mu3q2*KK(74) + mu3q1*KK(78) + 
     &  mu4q1*KK(80)
        KK(82)= 2*q1q2 + q1q4
        KK(83)= -(mu2q4*q1q3) + mu2q3*KK(82)
        KK(84)= 2*q1q2 - q2q4
        KK(85)= mu2q4*q2q3 + mu2q3*KK(84)
        KK(86)= -(q1q4*q2q3) + q1q3*q2q4 - q1q2*q3q4
        KK(87)= -4*q1q3 - 2*q1q4 - 2*q2q3 - q2q4
        KK(88)= -4*q1q3 - 2*q2q3 + q3q4
        KK(89)= -4*q1q3 - 2*q2q3 + 2*q3q4
        KK(90)= mu4q3*KK(87) + mu4q2*KK(88) + mu4q1*KK(89)
        KK(91)= 2*mu2q3*mu4q3*q1q2 + mu2q4*mu4q3*q1q2 + 
     &  mu4q2*KK(83) + mu4q1*KK(85) + mu2mu4*KK(86) + 
     &  mu2q1*KK(90)
        KK(92)= mu4q1*KK(27) + mu3q1*KK(31) + mu2q1*KK(41) + 
     &  mu2mu3*KK(49) + mu3mu4*KK(54) + mu1mu4*KK(64) + 
     &  mu2mu4*KK(71) + mu1mu2*KK(81) + mu1mu3*KK(91)
        KK(93)= 2*mu1q3*mu3q1*mu4q1 - 2*mu1mu3*mu4q1*q1q3
        KK(94)= 8*mu4q2 + 8*mu4q3
        KK(95)= 16*mu4q1 + 8*mu4q2 + 8*mu4q3
        KK(96)= 16*mu3q2*mu4q1 + 2*mu3q4*mu4q3 + mu3q2*KK(94) + mu3q1*KK(95)
        KK(97)= -8*mu3q1*mu4q1*q1q2 - 8*mu3q2*mu4q1*q1q2
        KK(98)= -8*q1q3 - 8*q2q3
        KK(99)= mu3q1*mu4q1*KK(98) + mu3q2*mu4q1*KK(98)
        KK(100)= -8*q1q4 - 8*q2q4 - 8*q3q4
        KK(101)= mu3q1*mu4q1*KK(100) + mu3q2*mu4q1*KK(100)
        KK(102)= 2*mu2q3*mu3q2*mu4q1 - 2*mu2mu3*mu4q1*q2q3 - 
     &  2*mu2q1*mu3mu4*q3q4 + mu2q1*KK(96) + z22*KK(97) + 
     &  z23*KK(99) + z24*KK(101)
        KK(103)= 2*mu1q2 + 2*mu1q4
        KK(104)= -2*mu1mu2*mu4q1*q1q2 - 2*mu1mu4*mu2q1*q1q4 + 
     &  mu2q1*mu4q1*KK(103)
        KK(105)= 8*q1q3 + 8*q2q3
        KK(106)= 8*q1q4 + 8*q2q4 + 8*q3q4
        KK(107)= 8*mu3q1*mu4q1*q1q2*z21 - 8*mu2q1*mu4q1*q1q2*z31 + 
     &  8*mu4q1*q1q2**2*z22*z31 + 
     &  mu4q1*q1q2*z23*z31*KK(105) + 
     &  mu4q1*q1q2*z24*z31*KK(106)
        KK(108)= 8*q1q3**2 + 16*q1q3*q2q3 + 8*q2q3**2
        KK(109)= 8*q2q4 + 8*q3q4
        KK(110)= 8*q1q4*q2q3 + q1q3*KK(106) + q2q3*KK(109)
        KK(111)= mu2q1*mu4q1*z31*KK(98) + mu3q1*mu4q1*z21*KK(105) + 
     &  mu4q1*q1q2*z22*z31*KK(105) + mu4q1*z23*z31*KK(108) + 
     &  mu4q1*z24*z31*KK(110)
        KK(112)= 16*q2q4 + 16*q3q4
        KK(113)= 8*q1q4**2 + 8*q2q4**2 + 16*q2q4*q3q4 + 8*q3q4**2 + 
     &  q1q4*KK(112)
        KK(114)= mu2q1*mu4q1*z31*KK(100) + mu3q1*mu4q1*z21*KK(106) + 
     &  mu4q1*q1q2*z22*z31*KK(106) + mu4q1*z23*z31*KK(110) + 
     &  mu4q1*z24*z31*KK(113)
        KK(115)= -2*mu1mu4*mu2mu3*q2q3 + 2*mu1mu3*mu2mu4*q2q3 - 
     &  2*mu1mu2*mu3mu4*q2q3 + z21*KK(93) + z11*KK(102) + 
     &  z31*KK(104) + z12*KK(107) + z13*KK(111) + z14*KK(114)
        KK(116)= -8*mu2q1*mu4q1*q1q2*z34 + 8*mu4q1*q1q2**2*z22*z34 + 
     &  mu4q1*q1q2*z23*z34*KK(105) + 
     &  mu4q1*q1q2*z24*z34*KK(106)
        KK(117)= mu2q1*mu4q1*z34*KK(98) + mu4q1*q1q2*z22*z34*KK(105) + 
     &  mu4q1*z23*z34*KK(108) + mu4q1*z24*z34*KK(110)
        KK(118)= -8*mu3q1*mu4q1*q1q2 - 8*mu3q2*mu4q1*q1q2 + 
     &  mu4q1*q1q2*z34*KK(106)
        KK(119)= mu3q1*mu4q1*KK(98) + mu3q2*mu4q1*KK(98) + mu4q1*z34*KK(110)
        KK(120)= mu3q1*mu4q1*KK(100) + mu3q2*mu4q1*KK(100) + mu4q1*z34*KK(113)
        KK(121)= 2*mu2q3*mu3q2*mu4q1 - 2*mu2mu3*mu4q1*q2q3 - 
     &  2*mu2q1*mu3mu4*q3q4 + mu2q1*KK(96) + 
     &  mu2q1*mu4q1*z34*KK(100) + z22*KK(118) + z23*KK(119) + 
     &  z24*KK(120)
        KK(122)= z24*KK(93) + z34*KK(104) + z12*KK(116) + 
     &  z13*KK(117) + z14*KK(121)
        KK(123)= 10*mu4q2 + 8*mu4q3
        KK(124)= 8*mu1q2*mu4q1 + mu1q2*KK(123)
        KK(125)= 2*mu1q2*mu2q4*mu4q2 - 2*mu1mu2*mu4q2*q1q2 - 
     &  2*mu1q2*mu2mu4*q2q4 + mu2q1*KK(124)
        KK(126)= -8*mu4q2*q1q2 - 8*mu4q3*q1q2
        KK(127)= -8*mu1q2*mu4q1*q1q2 + mu1q2*KK(126)
        KK(128)= mu4q2*KK(98) + mu4q3*KK(98)
        KK(129)= mu1q2*mu4q1*KK(98) + mu1q2*KK(128)
        KK(130)= mu4q2*KK(100) + mu4q3*KK(100)
        KK(131)= mu1q2*mu4q1*KK(100) + mu1q2*KK(130)
        KK(132)= 2*mu1q3*mu3q1*mu4q2 - 2*mu1mu3*mu4q2*q1q3 + z34*KK(131)
        KK(133)= -8*mu2q1*mu4q2*q1q2*z34 + 8*mu4q2*q1q2**2*z22*z34 + 
     &  mu4q2*q1q2*z23*z34*KK(105) + 
     &  mu4q2*q1q2*z24*z34*KK(106)
        KK(134)= mu2q1*mu4q2*z34*KK(98) + mu4q2*q1q2*z22*z34*KK(105) + 
     &  mu4q2*z23*z34*KK(108) + mu4q2*z24*z34*KK(110)
        KK(135)= 8*mu3q1*mu4q2 + 8*mu3q2*mu4q2
        KK(136)= -8*mu3q1*mu4q2*q1q2 - 8*mu3q2*mu4q2*q1q2 + 
     &  mu4q2*q1q2*z34*KK(106)
        KK(137)= mu3q1*mu4q2*KK(98) + mu3q2*mu4q2*KK(98) + mu4q2*z34*KK(110)
        KK(138)= mu3q1*mu4q2*KK(100) + mu3q2*mu4q2*KK(100) + mu4q2*z34*KK(113)
        KK(139)= 2*mu2q3*mu3q2*mu4q2 - 2*mu2mu3*mu4q2*q2q3 + 
     &  mu2q1*mu4q2*z34*KK(100) + mu2q1*KK(135) + 
     &  z22*KK(136) + z23*KK(137) + z24*KK(138)
        KK(140)= z34*KK(125) + z22*z34*KK(127) + z23*z34*KK(129) + 
     &  z24*KK(132) + z12*KK(133) + z13*KK(134) + z14*KK(139)
        KK(141)= 8*mu1q3*mu4q1 + 8*mu1q3*mu4q2 + 2*mu1q2*mu4q3 + 8*mu1q3*mu4q3
        KK(142)= 2*mu1q4*mu2q3*mu4q1 + 2*mu1q3*mu2q4*mu4q2 - 
     &  2*mu1mu2*mu4q3*q1q2 - 2*mu1mu4*mu2q3*q1q4 - 
     &  2*mu1q3*mu2mu4*q2q4 + mu2q1*KK(141)
        KK(143)= -8*mu1q3*mu4q1*q1q2 - 8*mu1q3*mu4q2*q1q2 - 8*mu1q3*mu4q3*q1q2
        KK(144)= mu1q3*mu4q1*KK(98) + mu1q3*mu4q2*KK(98) + mu1q3*mu4q3*KK(98)
        KK(145)= mu1q3*mu4q1*KK(100) + mu1q3*mu4q2*KK(100) + 
     &  mu1q3*mu4q3*KK(100)
        KK(146)= 2*mu1q3*mu3q1*mu4q3 - 2*mu1mu3*mu4q3*q1q3 + z34*KK(145)
        KK(147)= -8*mu2q1*mu4q3*q1q2*z34 + 8*mu4q3*q1q2**2*z22*z34 + 
     &  mu4q3*q1q2*z23*z34*KK(105) + 
     &  mu4q3*q1q2*z24*z34*KK(106)
        KK(148)= mu2q1*mu4q3*z34*KK(98) + mu4q3*q1q2*z22*z34*KK(105) + 
     &  mu4q3*z23*z34*KK(108) + mu4q3*z24*z34*KK(110)
        KK(149)= 8*mu2q3*mu4q1 + 8*mu2q3*mu4q2 + 8*mu2q3*mu4q3
        KK(150)= 8*mu2q3*mu4q2 + 10*mu2q3*mu4q3
        KK(151)= 8*mu3q1*mu4q3 + 8*mu3q2*mu4q3
        KK(152)= -8*mu3q1*mu4q3*q1q2 - 8*mu3q2*mu4q3*q1q2 + 
     &  mu4q3*q1q2*z34*KK(106)
        KK(153)= mu3q1*mu4q3*KK(98) + mu3q2*mu4q3*KK(98) + mu4q3*z34*KK(110)
        KK(154)= mu3q1*mu4q3*KK(100) + mu3q2*mu4q3*KK(100) + mu4q3*z34*KK(113)
        KK(155)= 8*mu2q3*mu3q2*mu4q1 + 2*mu2q3*mu3q4*mu4q3 - 
     &  2*mu2mu3*mu4q3*q2q3 - 2*mu2q3*mu3mu4*q3q4 + 
     &  mu2q1*mu4q3*z34*KK(100) + mu3q1*KK(149) + 
     &  mu3q2*KK(150) + mu2q1*KK(151) + z22*KK(152) + 
     &  z23*KK(153) + z24*KK(154)
        KK(156)= z34*KK(142) + z22*z34*KK(143) + z23*z34*KK(144) + 
     &  z24*KK(146) + z12*KK(147) + z13*KK(148) + z14*KK(155)
        KK(157)= 8*mu2q1*mu3q1*mu4q1 + mu3q1*mu4q1*z24*KK(100)
        KK(158)= -8*mu3q1*mu4q1*q1q2*z12*z24 + 
     &  mu3q1*mu4q1*z13*z24*KK(98) + z14*KK(157)
        KK(159)= 16*mu4q2 + 8*mu4q3
        KK(160)= 8*mu1q2*mu3q2*mu4q1 + 8*mu1q2*mu3q1*mu4q2 + 
     &  mu1q2*mu3q2*KK(159)
        KK(161)= 8*mu2q1*mu3q2*mu4q2 + mu3q2*mu4q2*z24*KK(100)
        KK(162)= -8*mu3q2*mu4q2*q1q2*z12*z24 + 
     &  mu3q2*mu4q2*z13*z24*KK(98) + z24*KK(160) + z14*KK(161)
        KK(163)= 8*mu1q3*mu3q1*mu4q3 + 8*mu1q3*mu3q2*mu4q3
        KK(164)= 16*mu1q2*mu4q1 + mu1q2*KK(94)
        KK(165)= 8*mu1q2*mu3q2*mu4q1 + mu3q1*KK(164)
        KK(166)= -8*mu3q2*mu4q1*q1q2 - 8*mu3q1*mu4q2*q1q2
        KK(167)= mu3q2*mu4q1*KK(98) + mu3q1*mu4q2*KK(98)
        KK(168)= 8*mu3q2*mu4q1 + 8*mu3q1*mu4q2
        KK(169)= mu3q2*mu4q1*KK(100) + mu3q1*mu4q2*KK(100)
        KK(170)= mu2q1*KK(168) + z24*KK(169)
        KK(171)= z24*KK(165) + z12*z24*KK(166) + z13*z24*KK(167) + z14*KK(170)
        KK(172)= 16*mu1q3*mu4q1 + 8*mu1q3*mu4q2 + 8*mu1q3*mu4q3
        KK(173)= 8*mu1q3*mu3q2*mu4q1 + mu3q1*KK(172)
        KK(174)= 8*mu2q1*mu3q1*mu4q3 + mu3q1*mu4q3*z24*KK(100)
        KK(175)= -8*mu3q1*mu4q3*q1q2*z12*z24 + 
     &  mu3q1*mu4q3*z13*z24*KK(98) + z24*KK(173) + z14*KK(174)
        KK(176)= 8*mu1q3*mu4q2 + 8*mu1q2*mu4q3
        KK(177)= 16*mu1q3*mu4q2 + 8*mu1q3*mu4q3
        KK(178)= 8*mu1q3*mu3q2*mu4q1 + 8*mu1q2*mu3q2*mu4q3 + 
     &  mu3q1*KK(176) + mu3q2*KK(177)
        KK(179)= 8*mu2q1*mu3q2*mu4q3 + mu3q2*mu4q3*z24*KK(100)
        KK(180)= -8*mu3q2*mu4q3*q1q2*z12*z24 + 
     &  mu3q2*mu4q3*z13*z24*KK(98) + z24*KK(178) + z14*KK(179)
        KK(181)= -8*mu3q1 - 8*mu3q2
        KK(182)= -8*mu4q1 - 8*mu4q2 - 8*mu4q3
        KK(183)= mu1mu4*KK(181) + mu1mu3*KK(182)
        KK(184)= -8*mu2q1*mu3mu4 + mu3mu4*z24*KK(106)
        KK(185)= 8*mu3mu4*q1q2*z12*z24 + mu3mu4*z13*z24*KK(105) + 
     &  z24*KK(183) + z14*KK(184)
        KK(186)= 8*mu2q3*mu3q2*mu4q1 + 8*mu2q3*mu3q1*mu4q2 + 
     &  8*mu2q1*mu3q2*mu4q3
        KK(187)= -8*mu2q1*mu3mu4 - 8*mu2mu4*mu3q1 - 8*mu2mu3*mu4q1
        KK(188)= -8*mu2mu4*mu3q2 - 8*mu2mu3*mu4q2
        KK(189)= -8*mu2q3*mu3mu4 - 8*mu2mu3*mu4q3
        KK(190)= 8*mu2q3*mu3q1*mu4q1 + 8*mu2q1*mu3q1*mu4q3
        KK(191)= -2*mu1q4*mu3q1 - 2*mu1q4*mu3q2
        KK(192)= -2*mu1q3*mu4q1 - 2*mu1q3*mu4q2
        KK(193)= -4*q1q2 - 2*q1q3 - 2*q2q3
        KK(194)= 2*mu2q3*mu4q1 + 2*mu2q3*mu4q2 + mu2mu4*KK(193)
        KK(195)= 4*q1q2 + 2*q1q3 + 2*q2q3
        KK(196)= 2*mu2q4*mu3q1 + 2*mu2q4*mu3q2 + mu2mu3*KK(195)
        KK(197)= -8*mu4q2 - 4*mu4q3
        KK(198)= -8*mu4q1 - 8*mu4q2 - 4*mu4q3
        KK(199)= -8*mu3q2*mu4q1 + mu3mu4*KK(195) + mu3q2*KK(197) + 
     &  mu3q1*KK(198)
        KK(200)= 2*mu1q3*mu4q1 + 2*mu1q3*mu4q2
        KK(201)= -2*mu4q1*q1q3 - 2*mu4q2*q1q3
        KK(202)= mu3q1*KK(200) + mu1mu3*KK(201)
        KK(203)= 16*mu4q1 + 16*mu4q2 + 8*mu4q3
        KK(204)= 16*mu3q2*mu4q1 + 2*mu3q4*mu4q3 + mu3q2*KK(159) + 
     &  mu3q1*KK(203)
        KK(205)= -2*mu4q1*q2q3 - 2*mu4q2*q2q3
        KK(206)= -8*mu4q1*q1q2 - 8*mu4q2*q1q2
        KK(207)= -8*mu3q2*mu4q1*q1q2 - 8*mu3q2*mu4q2*q1q2 + mu3q1*KK(206)
        KK(208)= mu4q1*KK(98) + mu4q2*KK(98)
        KK(209)= mu3q2*mu4q1*KK(98) + mu3q2*mu4q2*KK(98) + mu3q1*KK(208)
        KK(210)= mu4q1*KK(100) + mu4q2*KK(100)
        KK(211)= mu3q2*mu4q1*KK(100) + mu3q2*mu4q2*KK(100) + mu3q1*KK(210)
        KK(212)= 2*mu2q3*mu3q2*mu4q1 + 2*mu2q3*mu3q2*mu4q2 - 
     &  2*mu2q1*mu3mu4*q3q4 + mu2q1*KK(204) + 
     &  mu2mu3*KK(205) + z22*KK(207) + z23*KK(209) + 
     &  z24*KK(211)
        KK(213)= 10*mu1q2 + 2*mu1q4
        KK(214)= mu1q2*KK(123) + mu4q1*KK(213)
        KK(215)= -2*mu4q1*q1q2 - 2*mu4q2*q1q2
        KK(216)= 2*mu1q2*mu2q4*mu4q2 - 2*mu1mu4*mu2q1*q1q4 - 
     &  2*mu1q2*mu2mu4*q2q4 + mu2q1*KK(214) + mu1mu2*KK(215)
        KK(217)= -10*mu1q2 - 2*mu1q4
        KK(218)= -10*mu4q2 - 8*mu4q3
        KK(219)= mu4q1*KK(217) + mu1q2*KK(218)
        KK(220)= 2*mu4q1*q1q2 + 2*mu4q2*q1q2
        KK(221)= -2*mu1q2*mu2q4*mu4q2 + 2*mu1mu4*mu2q1*q1q4 + 
     &  2*mu1q2*mu2mu4*q2q4 + mu2q1*KK(219) + mu1mu2*KK(220)
        KK(222)= 2*mu4q1*q1q3 + 2*mu4q2*q1q3
        KK(223)= 8*mu4q2*q1q2 + 8*mu4q3*q1q2
        KK(224)= 8*mu1q2*mu4q1*q1q2 + mu1q2*KK(223)
        KK(225)= z31*KK(127) + mu3q1*KK(192) + mu1mu3*KK(222) + z32*KK(224)
        KK(226)= mu4q2*KK(105) + mu4q3*KK(105)
        KK(227)= mu1q2*mu4q1*KK(105) + mu1q2*KK(226)
        KK(228)= z31*KK(129) + z32*KK(227)
        KK(229)= mu4q2*KK(106) + mu4q3*KK(106)
        KK(230)= mu1q2*mu4q1*KK(106) + mu1q2*KK(229)
        KK(231)= z31*KK(131) + z32*KK(230)
        KK(232)= mu4q1*KK(105) + mu4q2*KK(105)
        KK(233)= mu4q1*q1q2*KK(105) + mu4q2*q1q2*KK(105)
        KK(234)= mu4q1*q1q2*KK(98) + mu4q2*q1q2*KK(98)
        KK(235)= z31*KK(233) + z32*KK(234)
        KK(236)= mu4q1*KK(108) + mu4q2*KK(108)
        KK(237)= -8*q1q3**2 - 16*q1q3*q2q3 - 8*q2q3**2
        KK(238)= mu4q1*KK(237) + mu4q2*KK(237)
        KK(239)= z31*KK(236) + z32*KK(238)
        KK(240)= mu4q1*KK(110) + mu4q2*KK(110)
        KK(241)= -8*q2q4 - 8*q3q4
        KK(242)= -8*q1q4*q2q3 + q1q3*KK(100) + q2q3*KK(241)
        KK(243)= mu4q1*KK(242) + mu4q2*KK(242)
        KK(244)= z31*KK(240) + z32*KK(243)
        KK(245)= mu2q1*z31*KK(208) + mu2q1*z32*KK(232) + z22*KK(235) + 
     &  z23*KK(239) + z24*KK(244)
        KK(246)= -16*mu4q2 - 8*mu4q3
        KK(247)= -16*mu4q1 - 16*mu4q2 - 8*mu4q3
        KK(248)= -16*mu3q2*mu4q1 - 2*mu3q4*mu4q3 + mu3q2*KK(246) + 
     &  mu3q1*KK(247)
        KK(249)= 2*mu4q1*q2q3 + 2*mu4q2*q2q3
        KK(250)= 8*mu4q1*q1q2 + 8*mu4q2*q1q2
        KK(251)= 8*mu4q1*q1q2**2 + 8*mu4q2*q1q2**2
        KK(252)= -8*mu4q1*q1q2**2 - 8*mu4q2*q1q2**2
        KK(253)= 8*mu3q2*mu4q1*q1q2 + 8*mu3q2*mu4q2*q1q2 + 
     &  mu3q1*KK(250) + z31*KK(251) + z32*KK(252)
        KK(254)= mu3q2*mu4q1*KK(105) + mu3q2*mu4q2*KK(105) + 
     &  mu3q1*KK(232) + z31*KK(233) + z32*KK(234)
        KK(255)= mu4q1*KK(106) + mu4q2*KK(106)
        KK(256)= mu4q1*q1q2*KK(106) + mu4q2*q1q2*KK(106)
        KK(257)= mu4q1*q1q2*KK(100) + mu4q2*q1q2*KK(100)
        KK(258)= mu3q2*mu4q1*KK(106) + mu3q2*mu4q2*KK(106) + 
     &  mu3q1*KK(255) + z31*KK(256) + z32*KK(257)
        KK(259)= -2*mu2q3*mu3q2*mu4q1 - 2*mu2q3*mu3q2*mu4q2 + 
     &  2*mu2q1*mu3mu4*q3q4 + mu2q1*z31*KK(206) + 
     &  mu2q1*KK(248) + mu2mu3*KK(249) + mu2q1*z32*KK(250) + 
     &  z22*KK(253) + z23*KK(254) + z24*KK(258)
        KK(260)= z31*KK(256) + z32*KK(257)
        KK(261)= mu4q1*KK(113) + mu4q2*KK(113)
        KK(262)= -16*q2q4 - 16*q3q4
        KK(263)= -8*q1q4**2 - 8*q2q4**2 - 16*q2q4*q3q4 - 8*q3q4**2 + 
     &  q1q4*KK(262)
        KK(264)= mu4q1*KK(263) + mu4q2*KK(263)
        KK(265)= z31*KK(261) + z32*KK(264)
        KK(266)= mu2q1*z31*KK(210) + z23*KK(244) + mu2q1*z32*KK(255) + 
     &  z22*KK(260) + z24*KK(265)
        KK(267)= mu2mu4*KK(191) + mu2mu3*KK(192) + mu1mu3*KK(194) + 
     &  mu1mu4*KK(196) + mu1mu2*KK(199) + z21*KK(202) + 
     &  z11*KK(212) + z31*KK(216) + z32*KK(221) + 
     &  z22*KK(225) + z23*KK(228) + z24*KK(231) + 
     &  z13*KK(245) + z12*KK(259) + z14*KK(266)
        KK(268)= -2*q1q3 - 2*q2q3
        KK(269)= 2*mu2q3*mu4q3 + mu2mu4*KK(268)
        KK(270)= 2*q1q3 + 2*q2q3
        KK(271)= -4*mu3q1*mu4q3 - 4*mu3q2*mu4q3 + mu3mu4*KK(270)
        KK(272)= 2*mu1q3*mu3q1*mu4q3 - 2*mu1mu3*mu4q3*q1q3
        KK(273)= -8*mu3q1*mu4q3*q1q2 - 8*mu3q2*mu4q3*q1q2
        KK(274)= mu3q1*mu4q3*KK(98) + mu3q2*mu4q3*KK(98)
        KK(275)= mu3q1*mu4q3*KK(100) + mu3q2*mu4q3*KK(100)
        KK(276)= 8*mu2q3*mu3q2*mu4q1 + 2*mu2q3*mu3q4*mu4q3 - 
     &  2*mu2mu3*mu4q3*q2q3 - 2*mu2q3*mu3mu4*q3q4 + 
     &  mu3q1*KK(149) + mu3q2*KK(150) + mu2q1*KK(151) + 
     &  z22*KK(273) + z23*KK(274) + z24*KK(275)
        KK(277)= -8*mu1q3*mu4q1 - 8*mu1q3*mu4q2 - 2*mu1q2*mu4q3 - 
     &  8*mu1q3*mu4q3
        KK(278)= -2*mu1q4*mu2q3*mu4q1 - 2*mu1q3*mu2q4*mu4q2 + 
     &  2*mu1mu2*mu4q3*q1q2 + 2*mu1mu4*mu2q3*q1q4 + 
     &  2*mu1q3*mu2mu4*q2q4 + mu2q1*KK(277)
        KK(279)= 8*mu1q3*mu4q1*q1q2 + 8*mu1q3*mu4q2*q1q2 + 8*mu1q3*mu4q3*q1q2
        KK(280)= -2*mu1q3*mu3q1*mu4q3 + 2*mu1mu3*mu4q3*q1q3 + 
     &  z31*KK(143) + z32*KK(279)
        KK(281)= mu1q3*mu4q1*KK(105) + mu1q3*mu4q2*KK(105) + 
     &  mu1q3*mu4q3*KK(105)
        KK(282)= z31*KK(144) + z32*KK(281)
        KK(283)= mu1q3*mu4q1*KK(106) + mu1q3*mu4q2*KK(106) + 
     &  mu1q3*mu4q3*KK(106)
        KK(284)= z31*KK(145) + z32*KK(283)
        KK(285)= mu4q3*q1q2*z32*KK(98) + mu4q3*q1q2*z31*KK(105)
        KK(286)= mu4q3*z31*KK(108) + mu4q3*z32*KK(237)
        KK(287)= mu4q3*z31*KK(110) + mu4q3*z32*KK(242)
        KK(288)= mu2q1*mu4q3*z31*KK(98) + mu2q1*mu4q3*z32*KK(105) + 
     &  z22*KK(285) + z23*KK(286) + z24*KK(287)
        KK(289)= -8*mu2q3*mu4q2 - 10*mu2q3*mu4q3
        KK(290)= -8*mu2q3*mu4q1 - 8*mu2q3*mu4q2 - 8*mu2q3*mu4q3
        KK(291)= -8*mu3q1*mu4q3 - 8*mu3q2*mu4q3
        KK(292)= 8*mu3q1*mu4q3*q1q2 + 8*mu3q2*mu4q3*q1q2 + 
     &  8*mu4q3*q1q2**2*z31 - 8*mu4q3*q1q2**2*z32
        KK(293)= mu4q3*q1q2*z32*KK(98) + mu3q1*mu4q3*KK(105) + 
     &  mu3q2*mu4q3*KK(105) + mu4q3*q1q2*z31*KK(105)
        KK(294)= mu4q3*q1q2*z32*KK(100) + mu3q1*mu4q3*KK(106) + 
     &  mu3q2*mu4q3*KK(106) + mu4q3*q1q2*z31*KK(106)
        KK(295)= -8*mu2q3*mu3q2*mu4q1 - 2*mu2q3*mu3q4*mu4q3 + 
     &  2*mu2mu3*mu4q3*q2q3 + 2*mu2q3*mu3mu4*q3q4 - 
     &  8*mu2q1*mu4q3*q1q2*z31 + 8*mu2q1*mu4q3*q1q2*z32 + 
     &  mu3q2*KK(289) + mu3q1*KK(290) + mu2q1*KK(291) + 
     &  z22*KK(292) + z23*KK(293) + z24*KK(294)
        KK(296)= mu4q3*q1q2*z32*KK(100) + mu4q3*q1q2*z31*KK(106)
        KK(297)= mu4q3*z31*KK(113) + mu4q3*z32*KK(263)
        KK(298)= mu2q1*mu4q3*z31*KK(100) + mu2q1*mu4q3*z32*KK(106) + 
     &  z23*KK(287) + z22*KK(296) + z24*KK(297)
        KK(299)= -2*mu1q3*mu2mu3*mu4q3 + z31*KK(142) + 
     &  mu1mu3*KK(269) + mu1mu4*mu2mu3*KK(270) + 
     &  mu1mu2*KK(271) + z21*KK(272) + z11*KK(276) + 
     &  z32*KK(278) + z22*KK(280) + z23*KK(282) + 
     &  z24*KK(284) + z13*KK(288) + z12*KK(295) + z14*KK(298)
        KK(300)= -2*q1q4 - 2*q2q4 - 2*q3q4
        KK(301)= 2*q1q4 + 2*q2q4 + 2*q3q4
        KK(302)= 2*mu2q4*mu3q4 + mu2mu3*KK(301)
        KK(303)= -4*mu3q4*mu4q1 - 4*mu3q4*mu4q2 - 4*mu3q4*mu4q3 + 
     &  mu3mu4*KK(301)
        KK(304)= 8*mu2q4*mu4q2 + 8*mu2q4*mu4q3
        KK(305)= 8*mu2q4*mu4q1 + 8*mu2q4*mu4q2 + 8*mu2q4*mu4q3
        KK(306)= 8*mu2q4*mu3q2*mu4q1 + 2*mu2q4*mu3q4*mu4q3 - 
     &  2*mu2q4*mu3mu4*q3q4 + mu3q2*KK(304) + mu3q1*KK(305)
        KK(307)= -8*mu2q4*mu4q2 - 8*mu2q4*mu4q3
        KK(308)= -8*mu2q4*mu4q1 - 8*mu2q4*mu4q2 - 8*mu2q4*mu4q3
        KK(309)= -8*mu2q4*mu3q2*mu4q1 - 2*mu2q4*mu3q4*mu4q3 + 
     &  2*mu2q4*mu3mu4*q3q4 + mu3q2*KK(307) + mu3q1*KK(308)
        KK(310)= 8*mu1q4*mu4q1 + 8*mu1q4*mu4q2 + 8*mu1q4*mu4q3
        KK(311)= 2*mu1q4*mu2q4*mu4q1 + 2*mu1q4*mu2q4*mu4q2 - 
     &  2*mu1mu4*mu2q4*q1q4 - 2*mu1q4*mu2mu4*q2q4 + 
     &  mu2q1*KK(310)
        KK(312)= -8*mu1q4*mu4q1 - 8*mu1q4*mu4q2 - 8*mu1q4*mu4q3
        KK(313)= -2*mu1q4*mu2q4*mu4q1 - 2*mu1q4*mu2q4*mu4q2 + 
     &  2*mu1mu4*mu2q4*q1q4 + 2*mu1q4*mu2mu4*q2q4 + 
     &  mu2q1*KK(312)
        KK(314)= -8*mu1q4*mu4q1*q1q2 - 8*mu1q4*mu4q2*q1q2 - 8*mu1q4*mu4q3*q1q2
        KK(315)= 8*mu1q4*mu4q1*q1q2 + 8*mu1q4*mu4q2*q1q2 + 8*mu1q4*mu4q3*q1q2
        KK(316)= z31*KK(314) + z32*KK(315)
        KK(317)= mu1q4*mu4q1*KK(98) + mu1q4*mu4q2*KK(98) + mu1q4*mu4q3*KK(98)
        KK(318)= mu1q4*mu4q1*KK(105) + mu1q4*mu4q2*KK(105) + 
     &  mu1q4*mu4q3*KK(105)
        KK(319)= z31*KK(317) + z32*KK(318)
        KK(320)= mu1q4*mu4q1*KK(100) + mu1q4*mu4q2*KK(100) + 
     &  mu1q4*mu4q3*KK(100)
        KK(321)= mu1q4*mu4q1*KK(106) + mu1q4*mu4q2*KK(106) + 
     &  mu1q4*mu4q3*KK(106)
        KK(322)= z31*KK(320) + z32*KK(321)
        KK(323)= -2*mu1q4*mu2mu4*mu3q4 + mu1mu3*mu2mu4*KK(300) + 
     &  mu1mu4*KK(302) + mu1mu2*KK(303) + z11*KK(306) + 
     &  z12*KK(309) + z31*KK(311) + z32*KK(313) + 
     &  z22*KK(316) + z23*KK(319) + z24*KK(322)
        KK(324)= -4*mu4q1 - 4*mu4q2
        KK(325)= -4*mu3q2*mu4q1 - 4*mu3q2*mu4q2 + mu3q1*KK(324)
        KK(326)= 8*mu4q1 + 8*mu4q2
        KK(327)= 8*mu3q2*mu4q1 + 8*mu3q2*mu4q2 + mu3q1*KK(326)
        KK(328)= 16*mu1q2*mu4q1 + mu1q2*KK(159)
        KK(329)= 16*mu1q2*mu3q2*mu4q1 + mu1q2*mu3q2*KK(159) + mu3q1*KK(328)
        KK(330)= -16*mu1q2*mu4q1 + mu1q2*KK(246)
        KK(331)= -16*mu1q2*mu3q2*mu4q1 + mu1q2*mu3q2*KK(246) + mu3q1*KK(330)
        KK(332)= -8*mu4q1 - 8*mu4q2
        KK(333)= -8*mu3q2*mu4q1 - 8*mu3q2*mu4q2 + mu3q1*KK(332)
        KK(334)= 8*mu3q2*mu4q1*q1q2 + 8*mu3q2*mu4q2*q1q2 + mu3q1*KK(250)
        KK(335)= z21*KK(207) + mu2q1*KK(333) + z22*KK(334)
        KK(336)= mu3q2*mu4q1*KK(105) + mu3q2*mu4q2*KK(105) + mu3q1*KK(232)
        KK(337)= z21*KK(209) + z22*KK(336)
        KK(338)= mu3q2*mu4q1*KK(106) + mu3q2*mu4q2*KK(106) + mu3q1*KK(255)
        KK(339)= z21*KK(211) + z22*KK(338)
        KK(340)= mu1mu2*KK(325) + mu2q1*z11*KK(327) + z21*KK(329) + 
     &  z22*KK(331) + z12*KK(335) + z13*KK(337) + z14*KK(339)
        KK(341)= -8*mu1q3*mu3q1*mu4q3 - 8*mu1q3*mu3q2*mu4q3
        KK(342)= z21*KK(163) + z22*KK(341)
        KK(343)= 8*mu1q4*mu3q4*mu4q1 + 8*mu1q4*mu3q4*mu4q2 + 
     &  8*mu1q4*mu3q4*mu4q3
        KK(344)= -8*mu1q4*mu3q4*mu4q1 - 8*mu1q4*mu3q4*mu4q2 - 
     &  8*mu1q4*mu3q4*mu4q3
        KK(345)= z21*KK(343) + z22*KK(344)
        KK(346)= -4*mu3q1*mu4q3 - 4*mu3q2*mu4q3
        KK(347)= 16*mu1q3*mu4q1 + 16*mu1q3*mu4q2 + 8*mu1q2*mu4q3 + 
     &  8*mu1q3*mu4q3
        KK(348)= 16*mu1q3*mu3q2*mu4q1 + 8*mu1q2*mu3q2*mu4q3 + 
     &  mu3q2*KK(177) + mu3q1*KK(347)
        KK(349)= -16*mu1q3*mu4q2 - 8*mu1q3*mu4q3
        KK(350)= -16*mu1q3*mu4q1 - 16*mu1q3*mu4q2 - 8*mu1q2*mu4q3 - 
     &  8*mu1q3*mu4q3
        KK(351)= -16*mu1q3*mu3q2*mu4q1 - 8*mu1q2*mu3q2*mu4q3 + 
     &  mu3q2*KK(349) + mu3q1*KK(350)
        KK(352)= 8*mu3q1*mu4q3*q1q2 + 8*mu3q2*mu4q3*q1q2
        KK(353)= z21*KK(273) + mu2q1*KK(291) + z22*KK(352)
        KK(354)= mu3q1*mu4q3*KK(105) + mu3q2*mu4q3*KK(105)
        KK(355)= z21*KK(274) + z22*KK(354)
        KK(356)= mu3q1*mu4q3*KK(106) + mu3q2*mu4q3*KK(106)
        KK(357)= z21*KK(275) + z22*KK(356)
        KK(358)= mu2q1*z11*KK(151) + mu1mu2*KK(346) + z21*KK(348) + 
     &  z22*KK(351) + z12*KK(353) + z13*KK(355) + z14*KK(357)
        KK(359)= -4*mu3q4*mu4q1 - 4*mu3q4*mu4q2
        KK(360)= 8*mu3q4*mu4q1 + 8*mu3q4*mu4q2
        KK(361)= 16*mu1q4*mu3q2 + 8*mu1q2*mu3q4
        KK(362)= 16*mu1q4*mu4q2 + 8*mu1q4*mu4q3
        KK(363)= 16*mu1q4*mu4q1 + 16*mu1q4*mu4q2 + 8*mu1q4*mu4q3
        KK(364)= 8*mu3q4*mu4q2 + 8*mu3q4*mu4q3
        KK(365)= mu4q1*KK(361) + mu3q2*KK(362) + mu3q1*KK(363) + mu1q2*KK(364)
        KK(366)= -16*mu1q4*mu3q2 - 8*mu1q2*mu3q4
        KK(367)= -16*mu1q4*mu4q2 - 8*mu1q4*mu4q3
        KK(368)= -16*mu1q4*mu4q1 - 16*mu1q4*mu4q2 - 8*mu1q4*mu4q3
        KK(369)= -8*mu3q4*mu4q2 - 8*mu3q4*mu4q3
        KK(370)= mu4q1*KK(366) + mu3q2*KK(367) + mu3q1*KK(368) + mu1q2*KK(369)
        KK(371)= -8*mu3q4*mu4q1 - 8*mu3q4*mu4q2
        KK(372)= -8*mu3q4*mu4q1*q1q2 - 8*mu3q4*mu4q2*q1q2
        KK(373)= 8*mu3q4*mu4q1*q1q2 + 8*mu3q4*mu4q2*q1q2
        KK(374)= mu2q1*KK(371) + z21*KK(372) + z22*KK(373)
        KK(375)= mu3q4*mu4q1*KK(98) + mu3q4*mu4q2*KK(98)
        KK(376)= mu3q4*mu4q1*KK(105) + mu3q4*mu4q2*KK(105)
        KK(377)= z21*KK(375) + z22*KK(376)
        KK(378)= mu3q4*mu4q1*KK(100) + mu3q4*mu4q2*KK(100)
        KK(379)= mu3q4*mu4q1*KK(106) + mu3q4*mu4q2*KK(106)
        KK(380)= z21*KK(378) + z22*KK(379)
        KK(381)= mu1mu2*KK(359) + mu2q1*z11*KK(360) + z21*KK(365) + 
     &  z22*KK(370) + z12*KK(374) + z13*KK(377) + z14*KK(380)
        KK(382)= 8*mu1q3*mu3q4*mu4q1 + 8*mu1q3*mu3q4*mu4q2 + 
     &  8*mu1q4*mu3q1*mu4q3 + 8*mu1q4*mu3q2*mu4q3 + 
     &  8*mu1q3*mu3q4*mu4q3
        KK(383)= -8*mu1q3*mu3q4*mu4q1 - 8*mu1q3*mu3q4*mu4q2 - 
     &  8*mu1q4*mu3q1*mu4q3 - 8*mu1q4*mu3q2*mu4q3 - 
     &  8*mu1q3*mu3q4*mu4q3
        KK(384)= -8*mu2q1*mu3q4*mu4q3 - 8*mu3q4*mu4q3*q1q2*z21 + 
     &  8*mu3q4*mu4q3*q1q2*z22
        KK(385)= mu3q4*mu4q3*z21*KK(98) + mu3q4*mu4q3*z22*KK(105)
        KK(386)= mu3q4*mu4q3*z21*KK(100) + mu3q4*mu4q3*z22*KK(106)
        KK(387)= -4*mu1mu2*mu3q4*mu4q3 + 8*mu2q1*mu3q4*mu4q3*z11 + 
     &  z21*KK(382) + z22*KK(383) + z12*KK(384) + 
     &  z13*KK(385) + z14*KK(386)
        KK(388)= 8*mu3q1 + 8*mu3q2
        KK(389)= 8*mu4q1 + 8*mu4q2 + 8*mu4q3
        KK(390)= mu1mu4*KK(388) + mu1mu3*KK(389)
        KK(391)= 8*mu2q1*mu3mu4 + 8*mu3mu4*q1q2*z21 - 8*mu3mu4*q1q2*z22
        KK(392)= mu3mu4*z22*KK(98) + mu3mu4*z21*KK(105)
        KK(393)= mu3mu4*z22*KK(100) + mu3mu4*z21*KK(106)
        KK(394)= 4*mu1mu2*mu3mu4 - 8*mu2q1*mu3mu4*z11 + z21*KK(183) + 
     &  z22*KK(390) + z12*KK(391) + z13*KK(392) + z14*KK(393)
        KK(395)= mu2q1*z11*KK(327) + mu2q1*z12*KK(333)
        KK(396)= 8*mu2q3*mu3q4*mu4q1 + 8*mu2q3*mu3q4*mu4q2 + 
     &  8*mu2q4*mu3q1*mu4q3 + 8*mu2q4*mu3q2*mu4q3 + 
     &  8*mu2q1*mu3q4*mu4q3
        KK(397)= -8*mu2q3*mu3q4*mu4q1 - 8*mu2q3*mu3q4*mu4q2 - 
     &  8*mu2q4*mu3q1*mu4q3 - 8*mu2q4*mu3q2*mu4q3 - 
     &  8*mu2q1*mu3q4*mu4q3
        KK(398)= z11*KK(396) + z12*KK(397)
        KK(399)= -8*mu2q1*mu3mu4 + mu2mu4*KK(181) + mu2mu3*KK(332)
        KK(400)= 8*mu2q1*mu3mu4 + mu2mu3*KK(326) + mu2mu4*KK(388)
        KK(401)= z11*KK(399) + z12*KK(400)
        KK(402)= 8*mu2q3*mu3mu4 + 8*mu2mu3*mu4q3
        KK(403)= z11*KK(189) + z12*KK(402)
        KK(404)= -8*mu2q4*mu3mu4 - 8*mu2mu4*mu3q4
        KK(405)= 8*mu2q4*mu3mu4 + 8*mu2mu4*mu3q4
        KK(406)= z11*KK(404) + z12*KK(405)
        KK(407)= 8*mu2q3*mu4q1 + 8*mu2q3*mu4q2
        KK(408)= 8*mu2q3*mu3q2*mu4q1 + 8*mu2q3*mu3q2*mu4q2 + 
     &  mu2q1*KK(151) + mu3q1*KK(407)
        KK(409)= -8*mu2q3*mu4q1 - 8*mu2q3*mu4q2
        KK(410)= -8*mu2q3*mu3q2*mu4q1 - 8*mu2q3*mu3q2*mu4q2 + 
     &  mu2q1*KK(291) + mu3q1*KK(409)
        KK(411)= z11*KK(408) + z12*KK(410)
        KK(412)= 8*mu2q4*mu4q1 + 8*mu2q4*mu4q2
        KK(413)= 8*mu2q4*mu3q2*mu4q1 + 8*mu2q4*mu3q2*mu4q2 + 
     &  mu2q1*KK(360) + mu3q1*KK(412)
        KK(414)= -8*mu2q4*mu4q1 - 8*mu2q4*mu4q2
        KK(415)= -8*mu2q4*mu3q2*mu4q1 - 8*mu2q4*mu3q2*mu4q2 + 
     &  mu2q1*KK(371) + mu3q1*KK(414)
        KK(416)= z11*KK(413) + z12*KK(415)
        KK(417)= 8*mu2q3*mu3q1*mu4q3 + 8*mu2q3*mu3q2*mu4q3
        KK(418)= -8*mu2q3*mu3q1*mu4q3 - 8*mu2q3*mu3q2*mu4q3
        KK(419)= z11*KK(417) + z12*KK(418)
        KK(420)= 8*mu2q4*mu3q4*mu4q1 + 8*mu2q4*mu3q4*mu4q2
        KK(421)= -8*mu2q4*mu3q4*mu4q1 - 8*mu2q4*mu3q4*mu4q2
        KK(422)= z11*KK(420) + z12*KK(421)
        KK(423)= 8*mu2q3*mu3q4*mu4q3*z11 - 8*mu2q3*mu3q4*mu4q3*z12
        KK(424)= 8*mu2q4*mu3q4*mu4q3*z11 - 8*mu2q4*mu3q4*mu4q3*z12
        KK(425)= 16*mu1q2*mu4q1 - 2*mu1q3*mu4q2 + mu1q2*KK(94)
        KK(426)= 8*mu1q2*mu3q2*mu4q1 + 2*mu1mu3*mu4q2*q1q3 + mu3q1*KK(425)
        KK(427)= -8*mu3q1*mu4q2 - 8*mu3q2*mu4q2
        KK(428)= 8*mu3q1*mu4q2*q1q2 + 8*mu3q2*mu4q2*q1q2
        KK(429)= mu3q1*mu4q2*KK(105) + mu3q2*mu4q2*KK(105)
        KK(430)= mu3q1*mu4q2*KK(106) + mu3q2*mu4q2*KK(106)
        KK(431)= -2*mu2q3*mu3q2*mu4q2 + 2*mu2mu3*mu4q2*q2q3 + 
     &  mu2q1*KK(427) + z22*KK(428) + z23*KK(429) + 
     &  z24*KK(430)
        KK(432)= -8*mu1q2*mu4q1 + mu1q2*KK(218)
        KK(433)= -2*mu1q2*mu2q4*mu4q2 + 2*mu1mu2*mu4q2*q1q2 + 
     &  2*mu1q2*mu2mu4*q2q4 + mu2q1*KK(432)
        KK(434)= mu4q2*q1q2*z22*z31*KK(98) + mu2q1*mu4q2*z31*KK(105) + 
     &  z21*KK(167) + mu4q2*z23*z31*KK(237) + 
     &  mu4q2*z24*z31*KK(242)
        KK(435)= 8*mu2q1*mu4q2*q1q2*z31 - 8*mu4q2*q1q2**2*z22*z31 + 
     &  mu4q2*q1q2*z23*z31*KK(98) + 
     &  mu4q2*q1q2*z24*z31*KK(100) + z21*KK(166)
        KK(436)= mu4q2*q1q2*z22*z31*KK(100) + 
     &  mu2q1*mu4q2*z31*KK(106) + z21*KK(169) + 
     &  mu4q2*z23*z31*KK(242) + mu4q2*z24*z31*KK(263)
        KK(437)= -2*mu1mu4*mu2mu3*q2q3 + 2*mu1mu3*mu2mu4*q2q3 - 
     &  2*mu1mu2*mu3mu4*q2q3 + z22*z31*KK(224) + 
     &  z23*z31*KK(227) + z24*z31*KK(230) + z21*KK(426) + 
     &  z11*KK(431) + z31*KK(433) + z13*KK(434) + 
     &  z12*KK(435) + z14*KK(436)
        KK(438)= 16*mu1q3*mu4q1 + 8*mu1q3*mu4q2 + 6*mu1q3*mu4q3
        KK(439)= 8*mu1q3*mu3q2*mu4q1 + 2*mu1mu3*mu4q3*q1q3 + mu3q1*KK(438)
        KK(440)= -16*mu2q3*mu4q1 - 8*mu2q3*mu4q2 - 8*mu2q3*mu4q3
        KK(441)= -8*mu2q3*mu3q2*mu4q1 - 2*mu2q3*mu3q4*mu4q3 + 
     &  2*mu2mu3*mu4q3*q2q3 + 2*mu2q3*mu3mu4*q3q4 + 
     &  mu3q2*KK(289) + mu2q1*KK(291) + z22*KK(352) + 
     &  z23*KK(354) + z24*KK(356) + mu3q1*KK(440)
        KK(442)= mu3q1*mu4q3*z21*KK(98) + mu4q3*q1q2*z22*z31*KK(98) + 
     &  mu2q1*mu4q3*z31*KK(105) + mu4q3*z23*z31*KK(237) + 
     &  mu4q3*z24*z31*KK(242)
        KK(443)= -8*mu3q1*mu4q3*q1q2*z21 + 8*mu2q1*mu4q3*q1q2*z31 - 
     &  8*mu4q3*q1q2**2*z22*z31 + mu4q3*q1q2*z23*z31*KK(98) + 
     &  mu4q3*q1q2*z24*z31*KK(100)
        KK(444)= mu3q1*mu4q3*z21*KK(100) + 
     &  mu4q3*q1q2*z22*z31*KK(100) + 
     &  mu2q1*mu4q3*z31*KK(106) + mu4q3*z23*z31*KK(242) + 
     &  mu4q3*z24*z31*KK(263)
        KK(445)= -2*mu1mu4*mu2mu3*q2q3 + 2*mu1mu3*mu2mu4*q2q3 - 
     &  2*mu1mu2*mu3mu4*q2q3 + z31*KK(278) + 
     &  z22*z31*KK(279) + z23*z31*KK(281) + z24*z31*KK(283) + 
     &  z21*KK(439) + z11*KK(441) + z13*KK(442) + 
     &  z12*KK(443) + z14*KK(444)
        KK(446)= -2*q2q4 - 2*q3q4
        KK(447)= 2*q2q4 + 2*q3q4
        KK(448)= -16*mu2q4*mu4q1 - 8*mu2q4*mu4q2 - 8*mu2q4*mu4q3
        KK(449)= -8*mu2q4*mu3q2*mu4q1 - 2*mu2q4*mu3q4*mu4q3 + 
     &  2*mu2q4*mu3mu4*q3q4 + mu3q2*KK(307) + mu3q1*KK(448)
        KK(450)= 16*mu1q4*mu4q1 + 8*mu1q4*mu4q2 + 8*mu1q4*mu4q3
        KK(451)= 8*mu1q4*mu3q2*mu4q1 + mu3q1*KK(450)
        KK(452)= -8*mu3q4*mu4q1*q1q2*z12*z21 + 
     &  mu3q4*mu4q1*z13*z21*KK(98) + 
     &  mu3q4*mu4q1*z14*z21*KK(100) + z31*KK(313) + 
     &  z22*z31*KK(315) + z23*z31*KK(318) + z24*z31*KK(321) + 
     &  mu1mu4*mu2mu3*KK(446) + mu1mu2*mu3mu4*KK(446) + 
     &  mu1mu3*mu2mu4*KK(447) + z11*KK(449) + z21*KK(451)
        KK(453)= -8*mu1q2*mu3q2*mu4q1 - 8*mu1q2*mu3q1*mu4q2 + 
     &  mu1q2*mu3q2*KK(246)
        KK(454)= 8*mu3q2*mu4q2*q1q2*z12*z21 + 
     &  mu3q2*mu4q2*z13*z21*KK(105) + 
     &  mu3q2*mu4q2*z14*z21*KK(106) + z21*KK(453)
        KK(455)= 8*mu2q3*mu3q1*mu4q3*z11 + z21*KK(341)
        KK(456)= 8*mu2q4*mu3q4*mu4q1*z11 + z21*KK(344)
        KK(457)= 8*mu2q3*mu3q2*mu4q1 + 8*mu2q3*mu3q1*mu4q2
        KK(458)= -8*mu1q3*mu4q2 - 8*mu1q2*mu4q3
        KK(459)= -8*mu1q3*mu3q2*mu4q1 - 8*mu1q2*mu3q2*mu4q3 + 
     &  mu3q2*KK(349) + mu3q1*KK(458)
        KK(460)= 8*mu3q2*mu4q3*q1q2*z12*z21 + 
     &  mu3q2*mu4q3*z13*z21*KK(105) + 
     &  mu3q2*mu4q3*z14*z21*KK(106) + z11*KK(457) + 
     &  z21*KK(459)
        KK(461)= 8*mu2q4*mu3q2*mu4q1 + 8*mu2q4*mu3q1*mu4q2
        KK(462)= -8*mu1q4*mu3q2 - 8*mu1q2*mu3q4
        KK(463)= -8*mu1q4*mu3q1*mu4q2 + mu3q2*KK(367) + 
     &  mu1q2*KK(369) + mu4q1*KK(462)
        KK(464)= 8*mu3q4*mu4q2*q1q2*z12*z21 + 
     &  mu3q4*mu4q2*z13*z21*KK(105) + 
     &  mu3q4*mu4q2*z14*z21*KK(106) + z11*KK(461) + 
     &  z21*KK(463)
        KK(465)= 8*mu2q3*mu3q4*mu4q1 + 8*mu2q4*mu3q1*mu4q3
        KK(466)= 8*mu3q4*mu4q3*q1q2*z12*z21 + 
     &  mu3q4*mu4q3*z13*z21*KK(105) + 
     &  mu3q4*mu4q3*z14*z21*KK(106) + z21*KK(383) + 
     &  z11*KK(465)
        KK(467)= -8*mu2mu4*mu3q1 - 8*mu2mu3*mu4q1
        KK(468)= -8*mu3mu4*q1q2*z12*z21 + mu3mu4*z13*z21*KK(98) + 
     &  mu3mu4*z14*z21*KK(100) + z21*KK(390) + z11*KK(467)
        KK(469)= -8*mu2q3*mu3q4*mu4q2 - 8*mu2q4*mu3q2*mu4q3
        KK(470)= 8*mu2mu4*mu3q2 + 8*mu2mu3*mu4q2
        KK(471)= -2*mu1q3*mu3q1*mu4q1 + 2*mu1mu3*mu4q1*q1q3
        KK(472)= -2*mu1q2 - 2*mu1q4
        KK(473)= 2*mu1mu2*mu4q1*q1q2 + 2*mu1mu4*mu2q1*q1q4 + 
     &  mu2q1*mu4q1*KK(472)
        KK(474)= -8*mu4q2 - 8*mu4q3
        KK(475)= -16*mu4q1 - 8*mu4q2 - 8*mu4q3
        KK(476)= -16*mu3q2*mu4q1 - 2*mu3q4*mu4q3 + mu3q2*KK(474) + 
     &  mu3q1*KK(475)
        KK(477)= 8*mu3q1*mu4q1*q1q2 + 8*mu3q2*mu4q1*q1q2 + 
     &  mu4q1*q1q2*z33*KK(98) + mu4q1*q1q2*z32*KK(105)
        KK(478)= mu3q1*mu4q1*KK(105) + mu3q2*mu4q1*KK(105) + 
     &  mu4q1*z32*KK(108) + mu4q1*z33*KK(237)
        KK(479)= mu3q1*mu4q1*KK(106) + mu3q2*mu4q1*KK(106) + 
     &  mu4q1*z32*KK(110) + mu4q1*z33*KK(242)
        KK(480)= -2*mu2q3*mu3q2*mu4q1 + 2*mu2mu3*mu4q1*q2q3 + 
     &  2*mu2q1*mu3mu4*q3q4 + mu2q1*mu4q1*z32*KK(98) + 
     &  mu2q1*mu4q1*z33*KK(105) + mu2q1*KK(476) + 
     &  z22*KK(477) + z23*KK(478) + z24*KK(479)
        KK(481)= -8*mu3q1*mu4q1*q1q2 - 8*mu3q2*mu4q1*q1q2 + 
     &  8*mu4q1*q1q2**2*z32 - 8*mu4q1*q1q2**2*z33
        KK(482)= mu3q1*mu4q1*KK(98) + mu3q2*mu4q1*KK(98) + 
     &  mu4q1*q1q2*z33*KK(98) + mu4q1*q1q2*z32*KK(105)
        KK(483)= mu3q1*mu4q1*KK(100) + mu3q2*mu4q1*KK(100) + 
     &  mu4q1*q1q2*z33*KK(100) + mu4q1*q1q2*z32*KK(106)
        KK(484)= 2*mu2q3*mu3q2*mu4q1 - 2*mu2mu3*mu4q1*q2q3 - 
     &  2*mu2q1*mu3mu4*q3q4 - 8*mu2q1*mu4q1*q1q2*z32 + 
     &  8*mu2q1*mu4q1*q1q2*z33 + mu2q1*KK(96) + z22*KK(481) + 
     &  z23*KK(482) + z24*KK(483)
        KK(485)= mu4q1*z32*KK(110) + mu4q1*z33*KK(242)
        KK(486)= mu4q1*q1q2*z33*KK(100) + mu4q1*q1q2*z32*KK(106)
        KK(487)= mu4q1*z32*KK(113) + mu4q1*z33*KK(263)
        KK(488)= mu2q1*mu4q1*z32*KK(100) + mu2q1*mu4q1*z33*KK(106) + 
     &  z23*KK(485) + z22*KK(486) + z24*KK(487)
        KK(489)= -2*mu1mu3*mu2q1*mu4q1 + 2*mu1mu2*mu3q1*mu4q1 + 
     &  z22*KK(93) + z32*KK(104) + z23*KK(471) + 
     &  z33*KK(473) + z13*KK(480) + z12*KK(484) + z14*KK(488)
        KK(490)= -2*mu1q2*mu2q4 - 2*mu1q3*mu2q4
        KK(491)= 2*mu1q2*mu3q4 + 2*mu1q3*mu3q4
        KK(492)= -2*mu4q2 - 2*mu4q3
        KK(493)= 2*mu4q2 + 2*mu4q3
        KK(494)= -4*mu1q2 - 4*mu1q3
        KK(495)= -4*mu1q3*mu4q2 - 4*mu1q3*mu4q3 + mu1q2*KK(22) + mu4q1*KK(494)
        KK(496)= 8*mu1q2 + 8*mu1q3
        KK(497)= 10*mu4q2 + 10*mu4q3
        KK(498)= 8*mu1q3*mu4q2 + 8*mu1q3*mu4q3 + mu4q1*KK(496) + mu1q2*KK(497)
        KK(499)= -2*mu4q2*q1q2 - 2*mu4q3*q1q2
        KK(500)= -2*mu1q2*q2q4 - 2*mu1q3*q2q4
        KK(501)= 2*mu1q4*mu2q3*mu4q1 + 2*mu1q2*mu2q4*mu4q2 + 
     &  2*mu1q3*mu2q4*mu4q2 - 2*mu1mu4*mu2q3*q1q4 + 
     &  mu2q1*KK(498) + mu1mu2*KK(499) + mu2mu4*KK(500)
        KK(502)= -8*mu1q2 - 8*mu1q3
        KK(503)= -10*mu4q2 - 10*mu4q3
        KK(504)= -8*mu1q3*mu4q2 - 8*mu1q3*mu4q3 + mu4q1*KK(502) + 
     &  mu1q2*KK(503)
        KK(505)= 2*mu4q2*q1q2 + 2*mu4q3*q1q2
        KK(506)= 2*mu1q2*q2q4 + 2*mu1q3*q2q4
        KK(507)= -2*mu1q4*mu2q3*mu4q1 - 2*mu1q2*mu2q4*mu4q2 - 
     &  2*mu1q3*mu2q4*mu4q2 + 2*mu1mu4*mu2q3*q1q4 + 
     &  mu2q1*KK(504) + mu1mu2*KK(505) + mu2mu4*KK(506)
        KK(508)= 2*mu1q3*mu4q2 + 2*mu1q3*mu4q3
        KK(509)= -2*mu4q2*q1q3 - 2*mu4q3*q1q3
        KK(510)= -8*mu1q2*q1q2 - 8*mu1q3*q1q2
        KK(511)= -8*mu1q3*mu4q2*q1q2 - 8*mu1q3*mu4q3*q1q2 + 
     &  mu1q2*KK(126) + mu4q1*KK(510)
        KK(512)= 8*mu1q2*q1q2 + 8*mu1q3*q1q2
        KK(513)= 8*mu1q3*mu4q2*q1q2 + 8*mu1q3*mu4q3*q1q2 + 
     &  mu1q2*KK(223) + mu4q1*KK(512)
        KK(514)= mu3q1*KK(508) + mu1mu3*KK(509) + z32*KK(511) + z33*KK(513)
        KK(515)= -2*mu1q3*mu4q2 - 2*mu1q3*mu4q3
        KK(516)= 2*mu4q2*q1q3 + 2*mu4q3*q1q3
        KK(517)= mu1q2*KK(98) + mu1q3*KK(98)
        KK(518)= mu1q3*mu4q2*KK(98) + mu1q3*mu4q3*KK(98) + 
     &  mu1q2*KK(128) + mu4q1*KK(517)
        KK(519)= mu1q2*KK(105) + mu1q3*KK(105)
        KK(520)= mu1q3*mu4q2*KK(105) + mu1q3*mu4q3*KK(105) + 
     &  mu1q2*KK(226) + mu4q1*KK(519)
        KK(521)= mu3q1*KK(515) + mu1mu3*KK(516) + z32*KK(518) + z33*KK(520)
        KK(522)= mu1q2*KK(100) + mu1q3*KK(100)
        KK(523)= mu1q3*mu4q2*KK(100) + mu1q3*mu4q3*KK(100) + 
     &  mu1q2*KK(130) + mu4q1*KK(522)
        KK(524)= mu1q2*KK(106) + mu1q3*KK(106)
        KK(525)= mu1q3*mu4q2*KK(106) + mu1q3*mu4q3*KK(106) + 
     &  mu1q2*KK(229) + mu4q1*KK(524)
        KK(526)= z32*KK(523) + z33*KK(525)
        KK(527)= mu3q1*KK(474) + mu3q2*KK(474)
        KK(528)= -10*mu2q3*mu4q2 - 10*mu2q3*mu4q3
        KK(529)= 2*mu4q2*q2q3 + 2*mu4q3*q2q3
        KK(530)= mu4q2*q1q2*KK(105) + mu4q3*q1q2*KK(105)
        KK(531)= mu4q2*q1q2*KK(98) + mu4q3*q1q2*KK(98)
        KK(532)= mu3q1*KK(223) + mu3q2*KK(223) + z32*KK(530) + z33*KK(531)
        KK(533)= mu4q2*KK(108) + mu4q3*KK(108)
        KK(534)= mu4q2*KK(237) + mu4q3*KK(237)
        KK(535)= mu3q1*KK(226) + mu3q2*KK(226) + z32*KK(533) + z33*KK(534)
        KK(536)= mu4q2*KK(110) + mu4q3*KK(110)
        KK(537)= mu4q2*KK(242) + mu4q3*KK(242)
        KK(538)= mu3q1*KK(229) + mu3q2*KK(229) + z32*KK(536) + z33*KK(537)
        KK(539)= -8*mu2q3*mu3q2*mu4q1 - 2*mu2q3*mu3q4*mu4q3 + 
     &  2*mu2q3*mu3mu4*q3q4 + mu2q1*z32*KK(128) + 
     &  mu2q1*z33*KK(226) + mu3q1*KK(290) + mu2q1*KK(527) + 
     &  mu3q2*KK(528) + mu2mu3*KK(529) + z22*KK(532) + 
     &  z23*KK(535) + z24*KK(538)
        KK(540)= 10*mu2q3*mu4q2 + 10*mu2q3*mu4q3
        KK(541)= mu3q1*KK(94) + mu3q2*KK(94)
        KK(542)= -2*mu4q2*q2q3 - 2*mu4q3*q2q3
        KK(543)= 8*mu4q2*q1q2**2 + 8*mu4q3*q1q2**2
        KK(544)= -8*mu4q2*q1q2**2 - 8*mu4q3*q1q2**2
        KK(545)= mu3q1*KK(126) + mu3q2*KK(126) + z32*KK(543) + z33*KK(544)
        KK(546)= mu3q1*KK(128) + mu3q2*KK(128) + z32*KK(530) + z33*KK(531)
        KK(547)= mu4q2*q1q2*KK(106) + mu4q3*q1q2*KK(106)
        KK(548)= mu4q2*q1q2*KK(100) + mu4q3*q1q2*KK(100)
        KK(549)= mu3q1*KK(130) + mu3q2*KK(130) + z32*KK(547) + z33*KK(548)
        KK(550)= 8*mu2q3*mu3q2*mu4q1 + 2*mu2q3*mu3q4*mu4q3 - 
     &  2*mu2q3*mu3mu4*q3q4 + mu2q1*z32*KK(126) + 
     &  mu3q1*KK(149) + mu2q1*z33*KK(223) + mu3q2*KK(540) + 
     &  mu2q1*KK(541) + mu2mu3*KK(542) + z22*KK(545) + 
     &  z23*KK(546) + z24*KK(549)
        KK(551)= z32*KK(536) + z33*KK(537)
        KK(552)= z32*KK(547) + z33*KK(548)
        KK(553)= mu4q2*KK(113) + mu4q3*KK(113)
        KK(554)= mu4q2*KK(263) + mu4q3*KK(263)
        KK(555)= z32*KK(553) + z33*KK(554)
        KK(556)= mu2q1*z32*KK(130) + mu2q1*z33*KK(229) + z23*KK(551) + 
     &  z22*KK(552) + z24*KK(555)
        KK(557)= mu3mu4*KK(490) + mu2mu4*KK(491) + 
     &  mu1mu3*mu2q1*KK(492) + mu1mu2*mu3q1*KK(493) + 
     &  mu2mu3*KK(495) + z32*KK(501) + z33*KK(507) + 
     &  z22*KK(514) + z23*KK(521) + z24*KK(526) + 
     &  z13*KK(539) + z12*KK(550) + z14*KK(556)
        KK(558)= -4*mu1q4*mu4q1 - 4*mu1q4*mu4q2 - 4*mu1q4*mu4q3
        KK(559)= z32*KK(314) + z33*KK(315)
        KK(560)= z32*KK(317) + z33*KK(318)
        KK(561)= z32*KK(320) + z33*KK(321)
        KK(562)= -2*mu1q4*mu2q4*mu3mu4 + 2*mu1q4*mu2mu4*mu3q4 + 
     &  z12*KK(306) + z13*KK(309) + z32*KK(311) + 
     &  z33*KK(313) + mu2mu3*KK(558) + z22*KK(559) + 
     &  z23*KK(560) + z24*KK(561)
        KK(563)= 8*mu2q1*mu3q1*mu4q1 - 8*mu3q1*mu4q1*q1q2*z22 + 
     &  8*mu3q1*mu4q1*q1q2*z23
        KK(564)= -8*mu2q1*mu3q1*mu4q1 + mu3q1*mu4q1*z22*KK(98) + 
     &  mu3q1*mu4q1*z23*KK(105)
        KK(565)= mu3q1*mu4q1*z22*KK(100) + mu3q1*mu4q1*z23*KK(106)
        KK(566)= z12*KK(563) + z13*KK(564) + z14*KK(565)
        KK(567)= -4*mu1q3*mu4q2 - 4*mu1q3*mu4q3 + mu1q2*KK(22)
        KK(568)= 8*mu1q2*mu3q2 + 8*mu1q3*mu3q2
        KK(569)= 16*mu4q2 + 16*mu4q3
        KK(570)= 16*mu1q3*mu4q2 + 16*mu1q3*mu4q3
        KK(571)= 8*mu1q3*mu4q2 + 8*mu1q3*mu4q3 + mu1q2*KK(94)
        KK(572)= mu4q1*KK(568) + mu1q2*mu3q2*KK(569) + mu3q2*KK(570) + 
     &  mu3q1*KK(571)
        KK(573)= -8*mu1q2*mu3q2 - 8*mu1q3*mu3q2
        KK(574)= -16*mu4q2 - 16*mu4q3
        KK(575)= -16*mu1q3*mu4q2 - 16*mu1q3*mu4q3
        KK(576)= -8*mu1q3*mu4q2 - 8*mu1q3*mu4q3 + mu1q2*KK(474)
        KK(577)= mu4q1*KK(573) + mu1q2*mu3q2*KK(574) + mu3q2*KK(575) + 
     &  mu3q1*KK(576)
        KK(578)= mu2q1*mu3q2*KK(94) + mu3q2*z22*KK(126) + mu3q2*z23*KK(223)
        KK(579)= mu3q2*z22*KK(128) + mu3q2*z23*KK(226) + mu2q1*mu3q2*KK(474)
        KK(580)= mu3q2*z22*KK(130) + mu3q2*z23*KK(229)
        KK(581)= mu2mu3*KK(567) + z22*KK(572) + z23*KK(577) + 
     &  z12*KK(578) + z13*KK(579) + z14*KK(580)
        KK(582)= z22*KK(343) + z23*KK(344)
        KK(583)= 16*mu1q2 + 16*mu1q3
        KK(584)= 8*mu1q3*mu4q2 + 8*mu1q3*mu4q3 + mu1q2*KK(94) + mu4q1*KK(583)
        KK(585)= mu4q1*KK(568) + mu3q1*KK(584)
        KK(586)= -16*mu1q2 - 16*mu1q3
        KK(587)= -8*mu1q3*mu4q2 - 8*mu1q3*mu4q3 + mu1q2*KK(474) + 
     &  mu4q1*KK(586)
        KK(588)= mu4q1*KK(573) + mu3q1*KK(587)
        KK(589)= 8*mu3q2*mu4q1 + mu3q1*KK(94)
        KK(590)= -8*mu3q2*mu4q1*q1q2 + mu3q1*KK(126)
        KK(591)= 8*mu3q2*mu4q1*q1q2 + mu3q1*KK(223)
        KK(592)= mu2q1*KK(589) + z22*KK(590) + z23*KK(591)
        KK(593)= -8*mu3q2*mu4q1 + mu3q1*KK(474)
        KK(594)= mu3q2*mu4q1*KK(98) + mu3q1*KK(128)
        KK(595)= mu3q2*mu4q1*KK(105) + mu3q1*KK(226)
        KK(596)= mu2q1*KK(593) + z22*KK(594) + z23*KK(595)
        KK(597)= mu3q2*mu4q1*KK(100) + mu3q1*KK(130)
        KK(598)= mu3q2*mu4q1*KK(106) + mu3q1*KK(229)
        KK(599)= z22*KK(597) + z23*KK(598)
        KK(600)= mu2mu3*mu4q1*KK(494) + z22*KK(585) + z23*KK(588) + 
     &  z12*KK(592) + z13*KK(596) + z14*KK(599)
        KK(601)= -16*mu1q4*mu4q1 - 8*mu1q4*mu4q2 - 8*mu1q4*mu4q3
        KK(602)= -8*mu1q4*mu3q2*mu4q1 + mu3q1*KK(601)
        KK(603)= 8*mu2q1*mu3q4*mu4q1 - 8*mu3q4*mu4q1*q1q2*z22 + 
     &  8*mu3q4*mu4q1*q1q2*z23
        KK(604)= -8*mu2q1*mu3q4*mu4q1 + mu3q4*mu4q1*z22*KK(98) + 
     &  mu3q4*mu4q1*z23*KK(105)
        KK(605)= mu3q4*mu4q1*z22*KK(100) + mu3q4*mu4q1*z23*KK(106)
        KK(606)= -4*mu1q4*mu2mu3*mu4q1 + z22*KK(451) + z23*KK(602) + 
     &  z12*KK(603) + z13*KK(604) + z14*KK(605)
        KK(607)= -4*mu1q4*mu4q2 - 4*mu1q4*mu4q3
        KK(608)= 8*mu1q4*mu3q2 + 8*mu1q2*mu3q4 + 8*mu1q3*mu3q4
        KK(609)= 8*mu1q4*mu4q2 + 8*mu1q4*mu4q3
        KK(610)= 16*mu1q4*mu4q2 + 16*mu1q4*mu4q3
        KK(611)= 8*mu1q3*mu3q4*mu4q2 + 8*mu1q3*mu3q4*mu4q3 + 
     &  mu1q2*KK(364) + mu4q1*KK(608) + mu3q1*KK(609) + 
     &  mu3q2*KK(610)
        KK(612)= -8*mu1q4*mu3q2 - 8*mu1q2*mu3q4 - 8*mu1q3*mu3q4
        KK(613)= -16*mu1q4*mu4q2 - 16*mu1q4*mu4q3
        KK(614)= -8*mu1q4*mu4q2 - 8*mu1q4*mu4q3
        KK(615)= -8*mu1q3*mu3q4*mu4q2 - 8*mu1q3*mu3q4*mu4q3 + 
     &  mu1q2*KK(369) + mu4q1*KK(612) + mu3q2*KK(613) + 
     &  mu3q1*KK(614)
        KK(616)= -8*mu3q4*mu4q2*q1q2 - 8*mu3q4*mu4q3*q1q2
        KK(617)= 8*mu3q4*mu4q2*q1q2 + 8*mu3q4*mu4q3*q1q2
        KK(618)= mu2q1*KK(364) + z22*KK(616) + z23*KK(617)
        KK(619)= mu3q4*mu4q2*KK(98) + mu3q4*mu4q3*KK(98)
        KK(620)= mu3q4*mu4q2*KK(105) + mu3q4*mu4q3*KK(105)
        KK(621)= mu2q1*KK(369) + z22*KK(619) + z23*KK(620)
        KK(622)= mu3q4*mu4q2*KK(100) + mu3q4*mu4q3*KK(100)
        KK(623)= mu3q4*mu4q2*KK(106) + mu3q4*mu4q3*KK(106)
        KK(624)= z22*KK(622) + z23*KK(623)
        KK(625)= mu2mu3*KK(607) + z22*KK(611) + z23*KK(615) + 
     &  z12*KK(618) + z13*KK(621) + z14*KK(624)
        KK(626)= -8*mu2q1*mu3mu4 + 8*mu3mu4*q1q2*z22 - 8*mu3mu4*q1q2*z23
        KK(627)= 8*mu2q1*mu3mu4 + mu3mu4*z23*KK(98) + mu3mu4*z22*KK(105)
        KK(628)= mu3mu4*z23*KK(100) + mu3mu4*z22*KK(106)
        KK(629)= 4*mu1mu4*mu2mu3 + z22*KK(183) + z23*KK(390) + 
     &  z12*KK(626) + z13*KK(627) + z14*KK(628)
        KK(630)= 8*mu2q1*mu3q1*mu4q1*z12 - 8*mu2q1*mu3q1*mu4q1*z13
        KK(631)= 8*mu2q4*mu3q2 + 8*mu2q3*mu3q4
        KK(632)= mu3q1*KK(304) + mu2q1*KK(364) + mu4q1*KK(631)
        KK(633)= -8*mu2q4*mu3q2 - 8*mu2q3*mu3q4
        KK(634)= mu3q1*KK(307) + mu2q1*KK(369) + mu4q1*KK(633)
        KK(635)= z12*KK(632) + z13*KK(634)
        KK(636)= 8*mu2q1*mu3mu4 + 8*mu2mu4*mu3q1 + 8*mu2mu3*mu4q1
        KK(637)= z12*KK(187) + z13*KK(636)
        KK(638)= -8*mu2q3*mu3mu4 - 8*mu2mu4*mu3q2 + mu2mu3*KK(474)
        KK(639)= 8*mu2q3*mu3mu4 + 8*mu2mu4*mu3q2 + mu2mu3*KK(94)
        KK(640)= z12*KK(638) + z13*KK(639)
        KK(641)= z12*KK(404) + z13*KK(405)
        KK(642)= 8*mu2q3*mu4q2 + 8*mu2q3*mu4q3
        KK(643)= -8*mu2q3*mu4q2 - 8*mu2q3*mu4q3
        KK(644)= mu3q2*z12*KK(642) + mu3q2*z13*KK(643)
        KK(645)= 8*mu2q3*mu3q1*mu4q1 + mu2q1*KK(589)
        KK(646)= -8*mu2q3*mu3q1*mu4q1 + mu2q1*KK(593)
        KK(647)= z12*KK(645) + z13*KK(646)
        KK(648)= 8*mu2q4*mu3q1*mu4q1 + 8*mu2q1*mu3q4*mu4q1
        KK(649)= -8*mu2q4*mu3q1*mu4q1 - 8*mu2q1*mu3q4*mu4q1
        KK(650)= z12*KK(648) + z13*KK(649)
        KK(651)= 8*mu2q3*mu3q2*mu4q1 + mu2q1*mu3q2*KK(94) + mu3q1*KK(642)
        KK(652)= -8*mu2q3*mu3q2*mu4q1 + mu2q1*mu3q2*KK(474) + mu3q1*KK(643)
        KK(653)= z12*KK(651) + z13*KK(652)
        KK(654)= 8*mu2q4*mu3q4*mu4q1*z12 - 8*mu2q4*mu3q4*mu4q1*z13
        KK(655)= 8*mu2q3*mu3q4*mu4q2 + 8*mu2q3*mu3q4*mu4q3 + mu3q2*KK(304)
        KK(656)= -8*mu2q3*mu3q4*mu4q2 - 8*mu2q3*mu3q4*mu4q3 + mu3q2*KK(307)
        KK(657)= z12*KK(655) + z13*KK(656)
        KK(658)= 8*mu2q4*mu3q4*mu4q2 + 8*mu2q4*mu3q4*mu4q3
        KK(659)= -8*mu2q4*mu3q4*mu4q2 - 8*mu2q4*mu3q4*mu4q3
        KK(660)= z12*KK(658) + z13*KK(659)
        KK(661)= -8*mu3q1*mu4q1*q1q2 - 8*mu3q2*mu4q1*q1q2 + 
     &  mu4q1*q1q2*z34*KK(98) + mu4q1*q1q2*z33*KK(105)
        KK(662)= mu3q1*mu4q1*KK(98) + mu3q2*mu4q1*KK(98) + 
     &  mu4q1*z33*KK(108) + mu4q1*z34*KK(237)
        KK(663)= mu3q1*mu4q1*KK(100) + mu3q2*mu4q1*KK(100) + 
     &  mu4q1*z33*KK(110) + mu4q1*z34*KK(242)
        KK(664)= 2*mu2q3*mu3q2*mu4q1 - 2*mu2mu3*mu4q1*q2q3 - 
     &  2*mu2q1*mu3mu4*q3q4 + mu2q1*KK(96) + 
     &  mu2q1*mu4q1*z33*KK(98) + mu2q1*mu4q1*z34*KK(105) + 
     &  z22*KK(661) + z23*KK(662) + z24*KK(663)
        KK(665)= 8*mu4q1*q1q2**2*z33 - 8*mu4q1*q1q2**2*z34
        KK(666)= mu4q1*q1q2*z34*KK(98) + mu4q1*q1q2*z33*KK(105)
        KK(667)= mu4q1*q1q2*z34*KK(100) + mu4q1*q1q2*z33*KK(106)
        KK(668)= -8*mu2q1*mu4q1*q1q2*z33 + 8*mu2q1*mu4q1*q1q2*z34 + 
     &  z22*KK(665) + z23*KK(666) + z24*KK(667)
        KK(669)= mu3q1*mu4q1*KK(105) + mu3q2*mu4q1*KK(105) + 
     &  mu4q1*z33*KK(110) + mu4q1*z34*KK(242)
        KK(670)= 8*mu3q1*mu4q1*q1q2 + 8*mu3q2*mu4q1*q1q2 + 
     &  mu4q1*q1q2*z34*KK(100) + mu4q1*q1q2*z33*KK(106)
        KK(671)= mu3q1*mu4q1*KK(106) + mu3q2*mu4q1*KK(106) + 
     &  mu4q1*z33*KK(113) + mu4q1*z34*KK(263)
        KK(672)= -2*mu2q3*mu3q2*mu4q1 + 2*mu2mu3*mu4q1*q2q3 + 
     &  2*mu2q1*mu3mu4*q3q4 + mu2q1*mu4q1*z33*KK(100) + 
     &  mu2q1*mu4q1*z34*KK(106) + mu2q1*KK(476) + 
     &  z23*KK(669) + z22*KK(670) + z24*KK(671)
        KK(673)= -2*mu1mu4*mu2q1*mu3q1 + 2*mu1mu3*mu2q1*mu4q1 + 
     &  z23*KK(93) + z33*KK(104) + z24*KK(471) + 
     &  z34*KK(473) + z13*KK(664) + z12*KK(668) + z14*KK(672)
        KK(674)= z33*KK(127) + z34*KK(224)
        KK(675)= 2*mu1q3*mu3q1*mu4q2 - 2*mu1mu3*mu4q2*q1q3 + 
     &  z33*KK(129) + z34*KK(227)
        KK(676)= -2*mu1q3*mu3q1*mu4q2 + 2*mu1mu3*mu4q2*q1q3 + 
     &  z33*KK(131) + z34*KK(230)
        KK(677)= -8*mu3q1*mu4q2*q1q2 - 8*mu3q2*mu4q2*q1q2 + 
     &  mu4q2*q1q2*z34*KK(98) + mu4q2*q1q2*z33*KK(105)
        KK(678)= mu3q1*mu4q2*KK(98) + mu3q2*mu4q2*KK(98) + 
     &  mu4q2*z33*KK(108) + mu4q2*z34*KK(237)
        KK(679)= mu3q1*mu4q2*KK(100) + mu3q2*mu4q2*KK(100) + 
     &  mu4q2*z33*KK(110) + mu4q2*z34*KK(242)
        KK(680)= 2*mu2q3*mu3q2*mu4q2 - 2*mu2mu3*mu4q2*q2q3 + 
     &  mu2q1*mu4q2*z33*KK(98) + mu2q1*mu4q2*z34*KK(105) + 
     &  mu2q1*KK(135) + z22*KK(677) + z23*KK(678) + 
     &  z24*KK(679)
        KK(681)= 8*mu4q2*q1q2**2*z33 - 8*mu4q2*q1q2**2*z34
        KK(682)= mu4q2*q1q2*z34*KK(98) + mu4q2*q1q2*z33*KK(105)
        KK(683)= mu4q2*q1q2*z34*KK(100) + mu4q2*q1q2*z33*KK(106)
        KK(684)= -8*mu2q1*mu4q2*q1q2*z33 + 8*mu2q1*mu4q2*q1q2*z34 + 
     &  z22*KK(681) + z23*KK(682) + z24*KK(683)
        KK(685)= mu3q1*mu4q2*KK(105) + mu3q2*mu4q2*KK(105) + 
     &  mu4q2*z33*KK(110) + mu4q2*z34*KK(242)
        KK(686)= 8*mu3q1*mu4q2*q1q2 + 8*mu3q2*mu4q2*q1q2 + 
     &  mu4q2*q1q2*z34*KK(100) + mu4q2*q1q2*z33*KK(106)
        KK(687)= mu3q1*mu4q2*KK(106) + mu3q2*mu4q2*KK(106) + 
     &  mu4q2*z33*KK(113) + mu4q2*z34*KK(263)
        KK(688)= -2*mu2q3*mu3q2*mu4q2 + 2*mu2mu3*mu4q2*q2q3 + 
     &  mu2q1*mu4q2*z33*KK(100) + mu2q1*mu4q2*z34*KK(106) + 
     &  mu2q1*KK(427) + z23*KK(685) + z22*KK(686) + 
     &  z24*KK(687)
        KK(689)= -4*mu1q2*mu2q1*mu3mu4 - 2*mu1q2*mu2mu4*mu3q2 + 
     &  2*mu1q2*mu2mu3*mu4q2 + z33*KK(125) + z34*KK(433) + 
     &  z22*KK(674) + z23*KK(675) + z24*KK(676) + 
     &  z13*KK(680) + z12*KK(684) + z14*KK(688)
        KK(690)= -4*mu1q3 - 4*mu1q4
        KK(691)= -2*mu1q3 - 2*mu1q4
        KK(692)= 2*mu2q3 + 2*mu2q4
        KK(693)= 2*mu1q4*mu2q3 + 2*mu1q4*mu2q4
        KK(694)= 2*mu1q3*mu2q4 + 2*mu1q4*mu2q4
        KK(695)= 8*mu1q3 + 8*mu1q4
        KK(696)= 2*mu1q2*mu4q3 + 8*mu1q3*mu4q3 + 8*mu1q4*mu4q3 + 
     &  mu4q1*KK(695) + mu4q2*KK(695)
        KK(697)= -2*mu2q3*q1q4 - 2*mu2q4*q1q4
        KK(698)= -2*mu1q3*q2q4 - 2*mu1q4*q2q4
        KK(699)= -2*mu1mu2*mu4q3*q1q2 + mu4q1*KK(693) + 
     &  mu4q2*KK(694) + mu2q1*KK(696) + mu1mu4*KK(697) + 
     &  mu2mu4*KK(698)
        KK(700)= -2*mu1q4*mu2q3 - 2*mu1q4*mu2q4
        KK(701)= -2*mu1q3*mu2q4 - 2*mu1q4*mu2q4
        KK(702)= -8*mu1q3 - 8*mu1q4
        KK(703)= -2*mu1q2*mu4q3 - 8*mu1q3*mu4q3 - 8*mu1q4*mu4q3 + 
     &  mu4q1*KK(702) + mu4q2*KK(702)
        KK(704)= 2*mu2q3*q1q4 + 2*mu2q4*q1q4
        KK(705)= 2*mu1q3*q2q4 + 2*mu1q4*q2q4
        KK(706)= 2*mu1mu2*mu4q3*q1q2 + mu4q1*KK(700) + mu4q2*KK(701) + 
     &  mu2q1*KK(703) + mu1mu4*KK(704) + mu2mu4*KK(705)
        KK(707)= -8*mu1q3*q1q2 - 8*mu1q4*q1q2
        KK(708)= -8*mu1q3*mu4q3*q1q2 - 8*mu1q4*mu4q3*q1q2 + 
     &  mu4q1*KK(707) + mu4q2*KK(707)
        KK(709)= 8*mu1q3*q1q2 + 8*mu1q4*q1q2
        KK(710)= 8*mu1q3*mu4q3*q1q2 + 8*mu1q4*mu4q3*q1q2 + 
     &  mu4q1*KK(709) + mu4q2*KK(709)
        KK(711)= z33*KK(708) + z34*KK(710)
        KK(712)= mu1q3*KK(98) + mu1q4*KK(98)
        KK(713)= mu1q3*mu4q3*KK(98) + mu1q4*mu4q3*KK(98) + 
     &  mu4q1*KK(712) + mu4q2*KK(712)
        KK(714)= mu1q3*KK(105) + mu1q4*KK(105)
        KK(715)= mu1q3*mu4q3*KK(105) + mu1q4*mu4q3*KK(105) + 
     &  mu4q1*KK(714) + mu4q2*KK(714)
        KK(716)= 2*mu1q3*mu3q1*mu4q3 - 2*mu1mu3*mu4q3*q1q3 + 
     &  z33*KK(713) + z34*KK(715)
        KK(717)= mu1q3*KK(100) + mu1q4*KK(100)
        KK(718)= mu1q3*mu4q3*KK(100) + mu1q4*mu4q3*KK(100) + 
     &  mu4q1*KK(717) + mu4q2*KK(717)
        KK(719)= mu1q3*KK(106) + mu1q4*KK(106)
        KK(720)= mu1q3*mu4q3*KK(106) + mu1q4*mu4q3*KK(106) + 
     &  mu4q1*KK(719) + mu4q2*KK(719)
        KK(721)= -2*mu1q3*mu3q1*mu4q3 + 2*mu1mu3*mu4q3*q1q3 + 
     &  z33*KK(718) + z34*KK(720)
        KK(722)= 8*mu2q3 + 8*mu2q4
        KK(723)= 8*mu2q3*mu4q3 + 8*mu2q4*mu4q3 + mu4q1*KK(722) + mu4q2*KK(722)
        KK(724)= 10*mu2q3*mu4q3 + 8*mu2q4*mu4q3 + mu4q2*KK(722)
        KK(725)= -2*mu2q3*q3q4 - 2*mu2q4*q3q4
        KK(726)= -8*mu3q1*mu4q3*q1q2 - 8*mu3q2*mu4q3*q1q2 + 
     &  mu4q3*q1q2*z34*KK(98) + mu4q3*q1q2*z33*KK(105)
        KK(727)= mu3q1*mu4q3*KK(98) + mu3q2*mu4q3*KK(98) + 
     &  mu4q3*z33*KK(108) + mu4q3*z34*KK(237)
        KK(728)= mu3q1*mu4q3*KK(100) + mu3q2*mu4q3*KK(100) + 
     &  mu4q3*z33*KK(110) + mu4q3*z34*KK(242)
        KK(729)= 2*mu2q3*mu3q4*mu4q3 + 2*mu2q4*mu3q4*mu4q3 - 
     &  2*mu2mu3*mu4q3*q2q3 + mu2q1*mu4q3*z33*KK(98) + 
     &  mu2q1*mu4q3*z34*KK(105) + mu2q1*KK(151) + 
     &  mu3q2*mu4q1*KK(722) + mu3q1*KK(723) + mu3q2*KK(724) + 
     &  mu3mu4*KK(725) + z22*KK(726) + z23*KK(727) + 
     &  z24*KK(728)
        KK(730)= 8*mu4q3*q1q2**2*z33 - 8*mu4q3*q1q2**2*z34
        KK(731)= mu4q3*q1q2*z34*KK(98) + mu4q3*q1q2*z33*KK(105)
        KK(732)= mu4q3*q1q2*z34*KK(100) + mu4q3*q1q2*z33*KK(106)
        KK(733)= -8*mu2q1*mu4q3*q1q2*z33 + 8*mu2q1*mu4q3*q1q2*z34 + 
     &  z22*KK(730) + z23*KK(731) + z24*KK(732)
        KK(734)= -8*mu2q3 - 8*mu2q4
        KK(735)= -10*mu2q3*mu4q3 - 8*mu2q4*mu4q3 + mu4q2*KK(734)
        KK(736)= -8*mu2q3*mu4q3 - 8*mu2q4*mu4q3 + mu4q1*KK(734) + 
     &  mu4q2*KK(734)
        KK(737)= 2*mu2q3*q3q4 + 2*mu2q4*q3q4
        KK(738)= mu3q1*mu4q3*KK(105) + mu3q2*mu4q3*KK(105) + 
     &  mu4q3*z33*KK(110) + mu4q3*z34*KK(242)
        KK(739)= 8*mu3q1*mu4q3*q1q2 + 8*mu3q2*mu4q3*q1q2 + 
     &  mu4q3*q1q2*z34*KK(100) + mu4q3*q1q2*z33*KK(106)
        KK(740)= mu3q1*mu4q3*KK(106) + mu3q2*mu4q3*KK(106) + 
     &  mu4q3*z33*KK(113) + mu4q3*z34*KK(263)
        KK(741)= -2*mu2q3*mu3q4*mu4q3 - 2*mu2q4*mu3q4*mu4q3 + 
     &  2*mu2mu3*mu4q3*q2q3 + mu2q1*mu4q3*z33*KK(100) + 
     &  mu2q1*mu4q3*z34*KK(106) + mu2q1*KK(291) + 
     &  mu3q2*mu4q1*KK(734) + mu3q2*KK(735) + mu3q1*KK(736) + 
     &  mu3mu4*KK(737) + z23*KK(738) + z22*KK(739) + 
     &  z24*KK(740)
        KK(742)= mu1mu4*mu3q1*KK(28) + mu2mu3*mu4q2*KK(34) + 
     &  mu2q1*mu3mu4*KK(690) + mu2mu4*mu3q2*KK(691) + 
     &  mu1mu3*mu4q1*KK(692) + z33*KK(699) + z34*KK(706) + 
     &  z22*KK(711) + z23*KK(716) + z24*KK(721) + 
     &  z13*KK(729) + z12*KK(733) + z14*KK(741)
        KK(743)= -8*mu3q1*mu4q1*q1q2*z23 + 8*mu3q1*mu4q1*q1q2*z24
        KK(744)= 8*mu2q1*mu3q1*mu4q1 + mu3q1*mu4q1*z23*KK(98) + 
     &  mu3q1*mu4q1*z24*KK(105)
        KK(745)= -8*mu2q1*mu3q1*mu4q1 + mu3q1*mu4q1*z23*KK(100) + 
     &  mu3q1*mu4q1*z24*KK(106)
        KK(746)= z12*KK(743) + z13*KK(744) + z14*KK(745)
        KK(747)= -8*mu3q2*mu4q2*q1q2*z23 + 8*mu3q2*mu4q2*q1q2*z24
        KK(748)= 8*mu2q1*mu3q2*mu4q2 + mu3q2*mu4q2*z23*KK(98) + 
     &  mu3q2*mu4q2*z24*KK(105)
        KK(749)= -8*mu2q1*mu3q2*mu4q2 + mu3q2*mu4q2*z23*KK(100) + 
     &  mu3q2*mu4q2*z24*KK(106)
        KK(750)= z23*KK(160) + z24*KK(453) + z12*KK(747) + 
     &  z13*KK(748) + z14*KK(749)
        KK(751)= -4*mu2q3 - 4*mu2q4
        KK(752)= -4*mu1q4*mu2q3 - 4*mu1q4*mu2q4 + mu1q3*KK(751)
        KK(753)= 8*mu1q3*mu3q4 + 8*mu1q4*mu3q4
        KK(754)= 8*mu1q3*mu4q3 + 8*mu1q4*mu4q3
        KK(755)= 8*mu1q3*mu3q4*mu4q3 + 8*mu1q4*mu3q4*mu4q3 + 
     &  mu4q1*KK(753) + mu4q2*KK(753) + mu3q1*KK(754) + 
     &  mu3q2*KK(754)
        KK(756)= -8*mu1q3*mu3q4 - 8*mu1q4*mu3q4
        KK(757)= -8*mu1q3*mu4q3 - 8*mu1q4*mu4q3
        KK(758)= -8*mu1q3*mu3q4*mu4q3 - 8*mu1q4*mu3q4*mu4q3 + 
     &  mu4q1*KK(756) + mu4q2*KK(756) + mu3q1*KK(757) + 
     &  mu3q2*KK(757)
        KK(759)= -8*mu3q4*mu4q3*q1q2*z23 + 8*mu3q4*mu4q3*q1q2*z24
        KK(760)= 8*mu2q1*mu3q4*mu4q3 + mu3q4*mu4q3*z23*KK(98) + 
     &  mu3q4*mu4q3*z24*KK(105)
        KK(761)= -8*mu2q1*mu3q4*mu4q3 + mu3q4*mu4q3*z23*KK(100) + 
     &  mu3q4*mu4q3*z24*KK(106)
        KK(762)= 2*mu1mu4*mu2mu3*q3q4 - 2*mu1mu3*mu2mu4*q3q4 + 
     &  2*mu1mu2*mu3mu4*q3q4 + mu3mu4*KK(752) + z23*KK(755) + 
     &  z24*KK(758) + z12*KK(759) + z13*KK(760) + z14*KK(761)
        KK(763)= -16*mu1q2*mu4q1 + mu1q2*KK(474)
        KK(764)= -8*mu1q2*mu3q2*mu4q1 + mu3q1*KK(763)
        KK(765)= 8*mu3q2*mu4q1*q1q2 + 8*mu3q1*mu4q2*q1q2
        KK(766)= z23*KK(166) + z24*KK(765)
        KK(767)= mu3q2*mu4q1*KK(105) + mu3q1*mu4q2*KK(105)
        KK(768)= z23*KK(167) + mu2q1*KK(168) + z24*KK(767)
        KK(769)= -8*mu3q2*mu4q1 - 8*mu3q1*mu4q2
        KK(770)= mu3q2*mu4q1*KK(106) + mu3q1*mu4q2*KK(106)
        KK(771)= z23*KK(169) + mu2q1*KK(769) + z24*KK(770)
        KK(772)= -4*mu1q2*mu2q1*mu3mu4 + 2*mu1mu4*mu2mu3*q1q2 - 
     &  2*mu1mu3*mu2mu4*q1q2 + 2*mu1mu2*mu3mu4*q1q2 + 
     &  z23*KK(165) + z24*KK(764) + z12*KK(766) + 
     &  z13*KK(768) + z14*KK(771)
        KK(773)= -2*q1q3 - 2*q1q4
        KK(774)= 2*q1q3 + 2*q1q4
        KK(775)= 16*mu1q3 + 16*mu1q4
        KK(776)= 8*mu1q3*mu4q3 + 8*mu1q4*mu4q3 + mu4q2*KK(695) + mu4q1*KK(775)
        KK(777)= mu3q2*mu4q1*KK(695) + mu3q1*KK(776)
        KK(778)= -16*mu1q3 - 16*mu1q4
        KK(779)= -8*mu1q3*mu4q3 - 8*mu1q4*mu4q3 + mu4q2*KK(702) + 
     &  mu4q1*KK(778)
        KK(780)= mu3q2*mu4q1*KK(702) + mu3q1*KK(779)
        KK(781)= -8*mu3q4*mu4q1*q1q2 - 8*mu3q1*mu4q3*q1q2
        KK(782)= 8*mu3q4*mu4q1*q1q2 + 8*mu3q1*mu4q3*q1q2
        KK(783)= z23*KK(781) + z24*KK(782)
        KK(784)= 8*mu3q4*mu4q1 + 8*mu3q1*mu4q3
        KK(785)= mu3q4*mu4q1*KK(98) + mu3q1*mu4q3*KK(98)
        KK(786)= mu3q4*mu4q1*KK(105) + mu3q1*mu4q3*KK(105)
        KK(787)= mu2q1*KK(784) + z23*KK(785) + z24*KK(786)
        KK(788)= -8*mu3q4*mu4q1 - 8*mu3q1*mu4q3
        KK(789)= mu3q4*mu4q1*KK(100) + mu3q1*mu4q3*KK(100)
        KK(790)= mu3q4*mu4q1*KK(106) + mu3q1*mu4q3*KK(106)
        KK(791)= mu2q1*KK(788) + z23*KK(789) + z24*KK(790)
        KK(792)= mu2q1*mu3mu4*KK(690) + mu1mu3*mu2mu4*KK(773) + 
     &  mu1mu4*mu2mu3*KK(774) + mu1mu2*mu3mu4*KK(774) + 
     &  z23*KK(777) + z24*KK(780) + z12*KK(783) + 
     &  z13*KK(787) + z14*KK(791)
        KK(793)= -2*q2q3 - 2*q2q4
        KK(794)= 2*q2q3 + 2*q2q4
        KK(795)= 8*mu1q2*mu3q4 + mu3q2*KK(695)
        KK(796)= 8*mu1q2*mu4q3 + mu4q2*KK(695)
        KK(797)= 8*mu1q3*mu4q3 + 8*mu1q4*mu4q3 + mu4q2*KK(775)
        KK(798)= 8*mu3q4*mu4q2 + 8*mu3q2*mu4q3 + 8*mu3q4*mu4q3
        KK(799)= mu4q1*KK(795) + mu3q1*KK(796) + mu3q2*KK(797) + mu1q2*KK(798)
        KK(800)= -8*mu1q2*mu3q4 + mu3q2*KK(702)
        KK(801)= -8*mu1q2*mu4q3 + mu4q2*KK(702)
        KK(802)= -8*mu1q3*mu4q3 - 8*mu1q4*mu4q3 + mu4q2*KK(778)
        KK(803)= -8*mu3q4*mu4q2 - 8*mu3q2*mu4q3 - 8*mu3q4*mu4q3
        KK(804)= mu4q1*KK(800) + mu3q1*KK(801) + mu3q2*KK(802) + mu1q2*KK(803)
        KK(805)= -8*mu3q4*mu4q2*q1q2 - 8*mu3q2*mu4q3*q1q2
        KK(806)= 8*mu3q4*mu4q2*q1q2 + 8*mu3q2*mu4q3*q1q2
        KK(807)= z23*KK(805) + z24*KK(806)
        KK(808)= 8*mu3q4*mu4q2 + 8*mu3q2*mu4q3
        KK(809)= mu3q4*mu4q2*KK(98) + mu3q2*mu4q3*KK(98)
        KK(810)= mu3q4*mu4q2*KK(105) + mu3q2*mu4q3*KK(105)
        KK(811)= mu2q1*KK(808) + z23*KK(809) + z24*KK(810)
        KK(812)= -8*mu3q4*mu4q2 - 8*mu3q2*mu4q3
        KK(813)= mu3q4*mu4q2*KK(100) + mu3q2*mu4q3*KK(100)
        KK(814)= mu3q4*mu4q2*KK(106) + mu3q2*mu4q3*KK(106)
        KK(815)= mu2q1*KK(812) + z23*KK(813) + z24*KK(814)
        KK(816)= mu1q2*mu3mu4*KK(751) + mu1mu3*mu2mu4*KK(793) + 
     &  mu1mu4*mu2mu3*KK(794) + mu1mu2*mu3mu4*KK(794) + 
     &  z23*KK(799) + z24*KK(804) + z12*KK(807) + 
     &  z13*KK(811) + z14*KK(815)
        KK(817)= 8*mu3mu4*q1q2*z23 - 8*mu3mu4*q1q2*z24
        KK(818)= -8*mu2q1*mu3mu4 + mu3mu4*z24*KK(98) + mu3mu4*z23*KK(105)
        KK(819)= 8*mu2q1*mu3mu4 + mu3mu4*z24*KK(100) + mu3mu4*z23*KK(106)
        KK(820)= -4*mu1mu4*mu2mu3 + 4*mu1mu3*mu2mu4 + z23*KK(183) + 
     &  z24*KK(390) + z12*KK(817) + z13*KK(818) + z14*KK(819)
        KK(821)= 8*mu2q1*mu3q1*mu4q1*z13 - 8*mu2q1*mu3q1*mu4q1*z14
        KK(822)= mu3q2*mu4q1*KK(722) + mu3q1*mu4q2*KK(722) + mu2q1*KK(808)
        KK(823)= mu3q2*mu4q1*KK(734) + mu3q1*mu4q2*KK(734) + mu2q1*KK(812)
        KK(824)= z13*KK(822) + z14*KK(823)
        KK(825)= z13*KK(187) + z14*KK(636)
        KK(826)= z13*KK(188) + z14*KK(470)
        KK(827)= -8*mu2mu4*mu3q4 - 8*mu2mu3*mu4q3 + mu3mu4*KK(734)
        KK(828)= 8*mu2mu4*mu3q4 + 8*mu2mu3*mu4q3 + mu3mu4*KK(722)
        KK(829)= z13*KK(827) + z14*KK(828)
        KK(830)= 8*mu2q3*mu3q4*mu4q3 + 8*mu2q4*mu3q4*mu4q3
        KK(831)= -8*mu2q3*mu3q4*mu4q3 - 8*mu2q4*mu3q4*mu4q3
        KK(832)= z13*KK(830) + z14*KK(831)
        KK(833)= mu2q1*z13*KK(168) + mu2q1*z14*KK(769)
        KK(834)= mu3q1*mu4q1*KK(722) + mu2q1*KK(784)
        KK(835)= mu3q1*mu4q1*KK(734) + mu2q1*KK(788)
        KK(836)= z13*KK(834) + z14*KK(835)
        KK(837)= 8*mu2q1*mu3q2*mu4q2*z13 - 8*mu2q1*mu3q2*mu4q2*z14
        KK(838)= 8*mu2q3*mu3q4 + 8*mu2q4*mu3q4
        KK(839)= 8*mu2q3*mu4q3 + 8*mu2q4*mu4q3
        KK(840)= 8*mu2q1*mu3q4*mu4q3 + mu4q1*KK(838) + mu3q1*KK(839)
        KK(841)= -8*mu2q3*mu3q4 - 8*mu2q4*mu3q4
        KK(842)= -8*mu2q3*mu4q3 - 8*mu2q4*mu4q3
        KK(843)= -8*mu2q1*mu3q4*mu4q3 + mu4q1*KK(841) + mu3q1*KK(842)
        KK(844)= z13*KK(840) + z14*KK(843)
        KK(845)= mu3q2*mu4q2*z13*KK(722) + mu3q2*mu4q2*z14*KK(734)
        KK(846)= mu4q2*KK(838) + mu3q2*KK(839)
        KK(847)= mu4q2*KK(841) + mu3q2*KK(842)
        KK(848)= z13*KK(846) + z14*KK(847)
        KK(849)= 8*mu1q2 + 8*mu1q3 + 8*mu1q4
        KK(850)= 4*mu1q3*mu4q3 + 2*mu1q4*mu4q3 + mu4q2*KK(37) + 
     &  mu1q2*KK(38) + mu4q1*KK(849)
        KK(851)= mu4q1*KK(33) + mu3q1*KK(850)
        KK(852)= -2*mu1q3*q1q2 + 2*mu1q2*q1q3
        KK(853)= -2*mu1q4*q1q2 + 2*mu1q2*q1q4
        KK(854)= -2*mu1q4*q1q3 + 2*mu1q3*q1q4
        KK(855)= -4*q1q2 - 2*q1q3
        KK(856)= -4*q1q2 - 2*q1q4
        KK(857)= -8*q1q2 + 2*q2q3 + 2*q2q4
        KK(858)= -4*mu4q3*q1q2 + mu4q2*KK(856) + mu4q1*KK(857)
        KK(859)= mu3q2*mu4q1*KK(855) + mu3q1*KK(858)
        KK(860)= -8*q1q4 - 2*q2q4 - 2*q3q4
        KK(861)= 2*mu3q4*q1q3 - 4*mu3q2*q1q4 + mu3q1*KK(860)
        KK(862)= 2*mu2q4*mu3q1*q1q2 + mu2q1*KK(861)
        KK(863)= -4*q1q3 - 2*q1q4
        KK(864)= -8*q1q3 - 2*q2q3 + 2*q3q4
        KK(865)= -4*mu4q2*q1q3 + mu4q3*KK(863) + mu4q1*KK(864)
        KK(866)= 2*mu2q3*mu4q1*q1q2 + mu2q1*KK(865)
        KK(867)= -4*mu1q3*mu3q1*mu4q1*q1q2 + 4*mu1mu3*mu4q1*q1q2*q1q3
        KK(868)= -4*q1q3 - 4*q2q3
        KK(869)= 4*q1q3**2 + 4*q1q3*q2q3
        KK(870)= mu1q3*mu3q1*mu4q1*KK(868) + mu1mu3*mu4q1*KK(869)
        KK(871)= -4*q1q4 - 4*q2q4 - 4*q3q4
        KK(872)= 4*q1q4 + 4*q2q4 + 4*q3q4
        KK(873)= mu1q3*mu3q1*mu4q1*KK(871) + mu1mu3*mu4q1*q1q3*KK(872)
        KK(874)= -4*mu1q2*q1q2 - 4*mu1q4*q1q2
        KK(875)= 4*mu1mu2*mu4q1*q1q2**2 + 4*mu1mu4*mu2q1*q1q2*q1q4 + 
     &  mu2q1*mu4q1*KK(874)
        KK(876)= mu1q2*KK(868) + mu1q4*KK(868)
        KK(877)= 4*q1q3 + 4*q2q3
        KK(878)= 4*q1q3*q1q4 + 4*q1q4*q2q3
        KK(879)= mu2q1*mu4q1*KK(876) + mu1mu2*mu4q1*q1q2*KK(877) + 
     &  mu1mu4*mu2q1*KK(878)
        KK(880)= mu1q2*KK(871) + mu1q4*KK(871)
        KK(881)= 4*q2q4 + 4*q3q4
        KK(882)= 4*q1q4**2 + q1q4*KK(881)
        KK(883)= mu1mu2*mu4q1*q1q2*KK(872) + mu2q1*mu4q1*KK(880) + 
     &  mu1mu4*mu2q1*KK(882)
        KK(884)= -16*mu4q2*q1q2 - 16*mu4q3*q1q2
        KK(885)= -32*mu4q1*q1q2 - 16*mu4q2*q1q2 - 16*mu4q3*q1q2
        KK(886)= -32*mu3q2*mu4q1*q1q2 - 4*mu3q4*mu4q3*q1q2 + 
     &  mu3q2*KK(884) + mu3q1*KK(885)
        KK(887)= 16*q1q3 + 16*q2q3
        KK(888)= 16*q1q4 + 16*q2q4 + 16*q3q4
        KK(889)= -16*q1q3 - 16*q2q3
        KK(890)= -16*q1q3**2 - 32*q1q3*q2q3 - 16*q2q3**2
        KK(891)= -16*q1q4 - 16*q2q4 - 16*q3q4
        KK(892)= -16*q1q4*q2q3 + q2q3*KK(262) + q1q3*KK(891)
        KK(893)= mu3q1*mu4q1*q1q2*KK(887) + mu3q2*mu4q1*q1q2*KK(887) + 
     &  mu4q1*q1q2**2*z32*KK(889) + mu4q1*q1q2*z33*KK(890) + 
     &  mu4q1*q1q2*z34*KK(892)
        KK(894)= 16*mu3q1*mu4q1*q1q2**2 + 16*mu3q2*mu4q1*q1q2**2 - 
     &  16*mu4q1*q1q2**3*z32 + mu4q1*q1q2**2*z33*KK(889) + 
     &  mu4q1*q1q2**2*z34*KK(891)
        KK(895)= -32*q2q4 - 32*q3q4
        KK(896)= -16*q1q4**2 - 16*q2q4**2 - 32*q2q4*q3q4 - 
     &  16*q3q4**2 + q1q4*KK(895)
        KK(897)= mu3q1*mu4q1*q1q2*KK(888) + mu3q2*mu4q1*q1q2*KK(888) + 
     &  mu4q1*q1q2**2*z32*KK(891) + mu4q1*q1q2*z33*KK(892) + 
     &  mu4q1*q1q2*z34*KK(896)
        KK(898)= -4*mu2q3*mu3q2*mu4q1*q1q2 + 
     &  4*mu2mu3*mu4q1*q1q2*q2q3 + 4*mu2q1*mu3mu4*q1q2*q3q4 + 
     &  16*mu2q1*mu4q1*q1q2**2*z32 + mu2q1*KK(886) + 
     &  mu2q1*mu4q1*q1q2*z33*KK(887) + 
     &  mu2q1*mu4q1*q1q2*z34*KK(888) + z23*KK(893) + 
     &  z22*KK(894) + z24*KK(897)
        KK(899)= mu4q2*KK(891) + mu4q3*KK(891)
        KK(900)= -32*q1q4 - 32*q2q4 - 32*q3q4
        KK(901)= mu4q2*KK(891) + mu4q3*KK(891) + mu4q1*KK(900)
        KK(902)= mu3q4*mu4q3*KK(871) + mu3q2*KK(899) + 
     &  mu3q2*mu4q1*KK(900) + mu3q1*KK(901)
        KK(903)= 4*q1q4*q3q4 + 4*q2q4*q3q4 + 4*q3q4**2
        KK(904)= 4*q1q4*q2q3 + q2q3*KK(881)
        KK(905)= 16*q1q4*q2q3 + q2q3*KK(112) + q1q3*KK(888)
        KK(906)= 32*q2q4 + 32*q3q4
        KK(907)= 16*q1q4**2 + 16*q2q4**2 + 32*q2q4*q3q4 + 16*q3q4**2 + 
     &  q1q4*KK(906)
        KK(908)= -16*q2q4**2 - 32*q2q4*q3q4 - 16*q3q4**2
        KK(909)= -16*q1q4**2*q2q3 + q1q4*q2q3*KK(895) + q1q3*KK(896) + 
     &  q2q3*KK(908)
        KK(910)= -48*q2q4 - 48*q3q4
        KK(911)= -48*q2q4**2 - 96*q2q4*q3q4 - 48*q3q4**2
        KK(912)= -16*q1q4**3 - 16*q2q4**3 - 48*q2q4**2*q3q4 - 
     &  48*q2q4*q3q4**2 - 16*q3q4**3 + q1q4**2*KK(910) + 
     &  q1q4*KK(911)
        KK(913)= mu4q1*q1q2*z32*KK(896) + mu3q1*mu4q1*KK(907) + 
     &  mu3q2*mu4q1*KK(907) + mu4q1*z33*KK(909) + 
     &  mu4q1*z34*KK(912)
        KK(914)= -32*q1q4*q2q3 + q2q3*KK(895)
        KK(915)= -16*q1q4*q2q3**2 + q2q3**2*KK(262) + 
     &  q1q3**2*KK(891) + q1q3*KK(914)
        KK(916)= mu4q1*q1q2*z32*KK(892) + mu3q1*mu4q1*KK(905) + 
     &  mu3q2*mu4q1*KK(905) + mu4q1*z34*KK(909) + 
     &  mu4q1*z33*KK(915)
        KK(917)= mu2q3*mu3q2*mu4q1*KK(871) + 
     &  mu2q1*mu4q1*q1q2*z32*KK(888) + z22*KK(897) + 
     &  mu2q1*KK(902) + mu2q1*mu3mu4*KK(903) + 
     &  mu2mu3*mu4q1*KK(904) + mu2q1*mu4q1*z33*KK(905) + 
     &  mu2q1*mu4q1*z34*KK(907) + z24*KK(913) + z23*KK(916)
        KK(918)= mu4q2*KK(889) + mu4q3*KK(889)
        KK(919)= -32*q1q3 - 32*q2q3
        KK(920)= mu4q2*KK(889) + mu4q3*KK(889) + mu4q1*KK(919)
        KK(921)= mu3q4*mu4q3*KK(868) + mu3q2*KK(918) + 
     &  mu3q2*mu4q1*KK(919) + mu3q1*KK(920)
        KK(922)= 4*q1q3*q2q3 + 4*q2q3**2
        KK(923)= 4*q1q3*q3q4 + 4*q2q3*q3q4
        KK(924)= 16*q1q3**2 + 32*q1q3*q2q3 + 16*q2q3**2
        KK(925)= -16*q1q3**3 - 48*q1q3**2*q2q3 - 48*q1q3*q2q3**2 - 16*q2q3**3
        KK(926)= mu4q1*q1q2*z32*KK(890) + mu4q1*z34*KK(915) + 
     &  mu3q1*mu4q1*KK(924) + mu3q2*mu4q1*KK(924) + 
     &  mu4q1*z33*KK(925)
        KK(927)= mu2q3*mu3q2*mu4q1*KK(868) + 
     &  mu2q1*mu4q1*q1q2*z32*KK(887) + z22*KK(893) + 
     &  mu2q1*mu4q1*z34*KK(905) + z24*KK(916) + 
     &  mu2q1*KK(921) + mu2mu3*mu4q1*KK(922) + 
     &  mu2q1*mu3mu4*KK(923) + mu2q1*mu4q1*z33*KK(924) + 
     &  z23*KK(926)
        KK(928)= mu1q2*mu3q1*mu4q1*KK(28) + mu2q1*KK(851) + 
     &  mu2mu3*mu4q1*KK(852) + mu2mu4*mu3q1*KK(853) + 
     &  mu2q1*mu3mu4*KK(854) + mu1mu2*KK(859) + 
     &  mu1mu4*KK(862) + mu1mu3*KK(866) + z22*KK(867) + 
     &  z23*KK(870) + z24*KK(873) + z32*KK(875) + 
     &  z33*KK(879) + z34*KK(883) + z12*KK(898) + 
     &  z14*KK(917) + z13*KK(927)
        KK(929)= 4*mu2q3 - 2*mu2q4
        KK(930)= -2*mu2q3 + 4*mu2q4
        KK(931)= 4*mu2q3 + 4*mu2q4
        KK(932)= 4*mu2q3*mu4q3 + 2*mu2q4*mu4q3 + mu4q2*KK(931)
        KK(933)= -2*mu2q3*mu3q4*mu4q2 + mu3q2*KK(932)
        KK(934)= 20*mu1q2*mu3q2 + 4*mu1q4*mu3q2
        KK(935)= 20*mu4q2 + 16*mu4q3
        KK(936)= 16*mu1q2*mu4q1 + 4*mu1q3*mu4q2 + mu1q2*KK(935)
        KK(937)= 24*mu4q2 + 20*mu4q3
        KK(938)= 4*mu3q4*mu4q3 + mu3q2*KK(937)
        KK(939)= mu3q2*mu4q2*KK(34) + mu4q1*KK(934) + mu3q1*KK(936) + 
     &  mu1q2*KK(938)
        KK(940)= -4*q1q3 - 2*q2q3
        KK(941)= 2*mu2q3*mu4q2*q1q2 + mu2q1*mu4q2*KK(940)
        KK(942)= -4*q1q4 - 2*q2q4
        KK(943)= 2*mu2q4*mu3q2*q1q2 + mu2q1*mu3q2*KK(942)
        KK(944)= -8*q1q2 - 2*q1q3 - 2*q1q4
        KK(945)= -4*mu4q3*q1q2 + mu4q2*KK(944)
        KK(946)= -4*q1q2 + 2*q2q3
        KK(947)= -4*q1q2 + 2*q2q4
        KK(948)= mu3q2*KK(945) + mu3q1*mu4q2*KK(946) + mu3q2*mu4q1*KK(947)
        KK(949)= 2*q1q4 - 4*q2q4 - 2*q3q4
        KK(950)= 2*mu3q4*q2q3 + mu3q2*KK(949)
        KK(951)= -2*mu1q4*mu3q2*q1q2 - 4*mu1q2*mu3q1*q2q4 + mu1q2*KK(950)
        KK(952)= -2*mu2q4*q2q3 + 2*mu2q3*q2q4
        KK(953)= -4*mu1q2*mu2q1*q3q4 + mu1q2*KK(952)
        KK(954)= -4*q2q3 - 2*q2q4
        KK(955)= 2*q1q3 - 4*q2q3 + 2*q3q4
        KK(956)= mu4q3*KK(954) + mu4q2*KK(955)
        KK(957)= -2*mu1q3*mu4q2*q1q2 - 4*mu1q2*mu4q1*q2q3 + mu1q2*KK(956)
        KK(958)= -20*mu4q2*q1q2 - 16*mu4q3*q1q2
        KK(959)= -16*mu1q2*mu4q1*q1q2 + mu1q2*KK(958)
        KK(960)= -4*mu1q2*mu2q4*mu4q2*q1q2 + 4*mu1mu2*mu4q2*q1q2**2 + 
     &  4*mu1q2*mu2mu4*q1q2*q2q4 + mu2q1*KK(959)
        KK(961)= -20*q1q3 - 20*q2q3
        KK(962)= mu4q3*KK(889) + mu4q2*KK(961)
        KK(963)= mu1q2*mu4q1*KK(889) + mu1q2*KK(962)
        KK(964)= 4*q1q3*q2q4 + 4*q2q3*q2q4
        KK(965)= mu1q2*mu2q4*mu4q2*KK(868) + 
     &  mu1mu2*mu4q2*q1q2*KK(877) + mu2q1*KK(963) + 
     &  mu1q2*mu2mu4*KK(964)
        KK(966)= -20*q1q4 - 20*q2q4 - 20*q3q4
        KK(967)= mu4q3*KK(891) + mu4q2*KK(966)
        KK(968)= mu1q2*mu4q1*KK(891) + mu1q2*KK(967)
        KK(969)= 4*q1q4*q2q4 + 4*q2q4**2 + 4*q2q4*q3q4
        KK(970)= mu1q2*mu2q4*mu4q2*KK(871) + 
     &  mu1mu2*mu4q2*q1q2*KK(872) + mu2q1*KK(968) + 
     &  mu1q2*mu2mu4*KK(969)
        KK(971)= 16*mu4q2*q1q2**2 + 16*mu4q3*q1q2**2
        KK(972)= 16*mu1q2*mu4q1*q1q2**2 + mu1q2*KK(971)
        KK(973)= mu4q2*q1q2*KK(887) + mu4q3*q1q2*KK(887)
        KK(974)= mu1q2*mu4q1*q1q2*KK(887) + mu1q2*KK(973)
        KK(975)= mu4q2*q1q2*KK(888) + mu4q3*q1q2*KK(888)
        KK(976)= mu1q2*mu4q1*q1q2*KK(888) + mu1q2*KK(975)
        KK(977)= -4*mu1q3*mu3q1*mu4q2*q1q2 + 
     &  4*mu1mu3*mu4q2*q1q2*q1q3 + z32*KK(972) + 
     &  z33*KK(974) + z34*KK(976)
        KK(978)= mu4q2*KK(924) + mu4q3*KK(924)
        KK(979)= mu1q2*mu4q1*KK(924) + mu1q2*KK(978)
        KK(980)= mu4q2*KK(905) + mu4q3*KK(905)
        KK(981)= mu1q2*mu4q1*KK(905) + mu1q2*KK(980)
        KK(982)= mu1q3*mu3q1*mu4q2*KK(868) + mu1mu3*mu4q2*KK(869) + 
     &  z32*KK(974) + z33*KK(979) + z34*KK(981)
        KK(983)= mu4q2*KK(907) + mu4q3*KK(907)
        KK(984)= mu1q2*mu4q1*KK(907) + mu1q2*KK(983)
        KK(985)= mu1q3*mu3q1*mu4q2*KK(871) + 
     &  mu1mu3*mu4q2*q1q3*KK(872) + z32*KK(976) + 
     &  z33*KK(981) + z34*KK(984)
        KK(986)= -16*mu3q1*mu4q2*q1q2 - 16*mu3q2*mu4q2*q1q2
        KK(987)= mu3q1*mu4q2*q1q2*KK(887) + mu3q2*mu4q2*q1q2*KK(887) + 
     &  mu4q2*q1q2**2*z32*KK(889) + mu4q2*q1q2*z33*KK(890) + 
     &  mu4q2*q1q2*z34*KK(892)
        KK(988)= 16*mu3q1*mu4q2*q1q2**2 + 16*mu3q2*mu4q2*q1q2**2 - 
     &  16*mu4q2*q1q2**3*z32 + mu4q2*q1q2**2*z33*KK(889) + 
     &  mu4q2*q1q2**2*z34*KK(891)
        KK(989)= mu3q1*mu4q2*q1q2*KK(888) + mu3q2*mu4q2*q1q2*KK(888) + 
     &  mu4q2*q1q2**2*z32*KK(891) + mu4q2*q1q2*z33*KK(892) + 
     &  mu4q2*q1q2*z34*KK(896)
        KK(990)= -4*mu2q3*mu3q2*mu4q2*q1q2 + 
     &  4*mu2mu3*mu4q2*q1q2*q2q3 + 
     &  16*mu2q1*mu4q2*q1q2**2*z32 + 
     &  mu2q1*mu4q2*q1q2*z33*KK(887) + 
     &  mu2q1*mu4q2*q1q2*z34*KK(888) + mu2q1*KK(986) + 
     &  z23*KK(987) + z22*KK(988) + z24*KK(989)
        KK(991)= mu3q1*mu4q2*KK(891) + mu3q2*mu4q2*KK(891)
        KK(992)= mu4q2*q1q2*z32*KK(896) + mu3q1*mu4q2*KK(907) + 
     &  mu3q2*mu4q2*KK(907) + mu4q2*z33*KK(909) + 
     &  mu4q2*z34*KK(912)
        KK(993)= mu4q2*q1q2*z32*KK(892) + mu3q1*mu4q2*KK(905) + 
     &  mu3q2*mu4q2*KK(905) + mu4q2*z34*KK(909) + 
     &  mu4q2*z33*KK(915)
        KK(994)= mu2q3*mu3q2*mu4q2*KK(871) + 
     &  mu2q1*mu4q2*q1q2*z32*KK(888) + mu2mu3*mu4q2*KK(904) + 
     &  mu2q1*mu4q2*z33*KK(905) + mu2q1*mu4q2*z34*KK(907) + 
     &  z22*KK(989) + mu2q1*KK(991) + z24*KK(992) + 
     &  z23*KK(993)
        KK(995)= mu3q1*mu4q2*KK(889) + mu3q2*mu4q2*KK(889)
        KK(996)= mu4q2*q1q2*z32*KK(890) + mu4q2*z34*KK(915) + 
     &  mu3q1*mu4q2*KK(924) + mu3q2*mu4q2*KK(924) + 
     &  mu4q2*z33*KK(925)
        KK(997)= mu2q3*mu3q2*mu4q2*KK(868) + 
     &  mu2q1*mu4q2*q1q2*z32*KK(887) + 
     &  mu2q1*mu4q2*z34*KK(905) + mu2mu3*mu4q2*KK(922) + 
     &  mu2q1*mu4q2*z33*KK(924) + z22*KK(987) + z24*KK(993) + 
     &  mu2q1*KK(995) + z23*KK(996)
        KK(998)= mu1q2*mu3q2*mu4q1*KK(929) + 
     &  mu1q2*mu3q1*mu4q2*KK(930) + mu1q2*KK(933) + 
     &  mu2q1*KK(939) + mu1mu3*KK(941) + mu1mu4*KK(943) + 
     &  mu1mu2*KK(948) + mu2mu4*KK(951) + mu3mu4*KK(953) + 
     &  mu2mu3*KK(957) + z32*KK(960) + z33*KK(965) + 
     &  z34*KK(970) + z22*KK(977) + z23*KK(982) + 
     &  z24*KK(985) + z12*KK(990) + z14*KK(994) + z13*KK(997)
        KK(999)= 4*mu1q3*mu2q3 + 4*mu1q4*mu2q3
        KK(1000)= -2*mu1q3*mu2q3*mu3q4 + mu3q2*KK(999)
        KK(1001)= -2*mu1q2*mu2q3*mu4q3 + 4*mu1q3*mu2q3*mu4q3 + 
     &  2*mu1q4*mu2q3*mu4q3 + mu1q3*mu4q2*KK(931) + 
     &  mu4q1*KK(999)
        KK(1002)= 16*mu1q3*mu4q2 + 18*mu1q3*mu4q3
        KK(1003)= 16*mu1q3*mu4q1 + 16*mu1q3*mu4q2 + 4*mu1q2*mu4q3 + 
     &  20*mu1q3*mu4q3
        KK(1004)= 16*mu1q3*mu3q2*mu4q1 + 4*mu1q2*mu3q2*mu4q3 + 
     &  4*mu1q3*mu3q4*mu4q3 + mu3q2*KK(1002) + mu3q1*KK(1003)
        KK(1005)= 4*mu2q3*mu4q3 + 2*mu2q4*mu4q3
        KK(1006)= mu1q3*mu4q2*KK(931) + mu1q3*KK(1005)
        KK(1007)= mu3q2*mu4q3*KK(855) + mu3q1*mu4q3*KK(946)
        KK(1008)= -4*q1q4 - 2*q3q4
        KK(1009)= 2*mu2q3*mu3q4*q1q3 - 4*mu2q3*mu3q2*q1q4 + 
     &  mu2q3*mu3q1*KK(1008)
        KK(1010)= -4*q2q4 - 2*q3q4
        KK(1011)= 2*mu1q3*mu3q4*q2q3 - 4*mu1q3*mu3q1*q2q4 + 
     &  mu1q3*mu3q2*KK(1010)
        KK(1012)= 2*q1q4 + 2*q2q4
        KK(1013)= -2*mu2q4*q2q3 + mu2q3*KK(1012)
        KK(1014)= -2*mu1q4*mu2q3*q1q3 - 4*mu1q3*mu2q1*q3q4 + mu1q3*KK(1013)
        KK(1015)= 2*q1q2 - 4*q1q3 - 2*q1q4
        KK(1016)= -4*q1q3 + 2*q3q4
        KK(1017)= -4*mu2q3*mu4q2*q1q3 + mu2q1*mu4q3*KK(940) + 
     &  mu2q3*mu4q3*KK(1015) + mu2q3*mu4q1*KK(1016)
        KK(1018)= -2*q1q2 - 4*q2q3 - 2*q2q4
        KK(1019)= -4*q2q3 + 2*q3q4
        KK(1020)= 2*mu1q2*mu4q3*q1q3 - 4*mu1q3*mu4q1*q2q3 + 
     &  mu1q3*mu4q3*KK(1018) + mu1q3*mu4q2*KK(1019)
        KK(1021)= -16*mu1q3*mu4q1*q1q2 - 16*mu1q3*mu4q2*q1q2 - 
     &  4*mu1q2*mu4q3*q1q2 - 16*mu1q3*mu4q3*q1q2
        KK(1022)= -4*mu1q4*mu2q3*mu4q1*q1q2 - 
     &  4*mu1q3*mu2q4*mu4q2*q1q2 + 4*mu1mu2*mu4q3*q1q2**2 + 
     &  4*mu1mu4*mu2q3*q1q2*q1q4 + 4*mu1q3*mu2mu4*q1q2*q2q4 + 
     &  mu2q1*KK(1021)
        KK(1023)= mu1q2*mu4q3*KK(868) + mu1q3*mu4q1*KK(889) + 
     &  mu1q3*mu4q2*KK(889) + mu1q3*mu4q3*KK(889)
        KK(1024)= mu1q4*mu2q3*mu4q1*KK(868) + 
     &  mu1q3*mu2q4*mu4q2*KK(868) + 
     &  mu1mu2*mu4q3*q1q2*KK(877) + mu1mu4*mu2q3*KK(878) + 
     &  mu1q3*mu2mu4*KK(964) + mu2q1*KK(1023)
        KK(1025)= mu1q2*mu4q3*KK(871) + mu1q3*mu4q1*KK(891) + 
     &  mu1q3*mu4q2*KK(891) + mu1q3*mu4q3*KK(891)
        KK(1026)= mu1q4*mu2q3*mu4q1*KK(871) + 
     &  mu1q3*mu2q4*mu4q2*KK(871) + 
     &  mu1mu2*mu4q3*q1q2*KK(872) + mu1mu4*mu2q3*KK(882) + 
     &  mu1q3*mu2mu4*KK(969) + mu2q1*KK(1025)
        KK(1027)= 16*mu1q3*mu4q1*q1q2**2 + 16*mu1q3*mu4q2*q1q2**2 + 
     &  16*mu1q3*mu4q3*q1q2**2
        KK(1028)= mu1q3*mu4q1*q1q2*KK(887) + mu1q3*mu4q2*q1q2*KK(887) + 
     &  mu1q3*mu4q3*q1q2*KK(887)
        KK(1029)= mu1q3*mu4q1*q1q2*KK(888) + mu1q3*mu4q2*q1q2*KK(888) + 
     &  mu1q3*mu4q3*q1q2*KK(888)
        KK(1030)= -4*mu1q3*mu3q1*mu4q3*q1q2 + 
     &  4*mu1mu3*mu4q3*q1q2*q1q3 + z32*KK(1027) + 
     &  z33*KK(1028) + z34*KK(1029)
        KK(1031)= mu1q3*mu4q1*KK(924) + mu1q3*mu4q2*KK(924) + 
     &  mu1q3*mu4q3*KK(924)
        KK(1032)= mu1q3*mu4q1*KK(905) + mu1q3*mu4q2*KK(905) + 
     &  mu1q3*mu4q3*KK(905)
        KK(1033)= mu1q3*mu3q1*mu4q3*KK(868) + mu1mu3*mu4q3*KK(869) + 
     &  z32*KK(1028) + z33*KK(1031) + z34*KK(1032)
        KK(1034)= mu1q3*mu4q1*KK(907) + mu1q3*mu4q2*KK(907) + 
     &  mu1q3*mu4q3*KK(907)
        KK(1035)= mu1q3*mu3q1*mu4q3*KK(871) + 
     &  mu1mu3*mu4q3*q1q3*KK(872) + z32*KK(1029) + 
     &  z33*KK(1032) + z34*KK(1034)
        KK(1036)= -16*mu2q3*mu4q2*q1q2 - 20*mu2q3*mu4q3*q1q2
        KK(1037)= -16*mu2q3*mu4q1*q1q2 - 16*mu2q3*mu4q2*q1q2 - 
     &  16*mu2q3*mu4q3*q1q2
        KK(1038)= -16*mu3q1*mu4q3*q1q2 - 16*mu3q2*mu4q3*q1q2
        KK(1039)= mu3q1*mu4q3*q1q2*KK(887) + mu3q2*mu4q3*q1q2*KK(887) + 
     &  mu4q3*q1q2**2*z32*KK(889) + mu4q3*q1q2*z33*KK(890) + 
     &  mu4q3*q1q2*z34*KK(892)
        KK(1040)= 16*mu3q1*mu4q3*q1q2**2 + 16*mu3q2*mu4q3*q1q2**2 - 
     &  16*mu4q3*q1q2**3*z32 + mu4q3*q1q2**2*z33*KK(889) + 
     &  mu4q3*q1q2**2*z34*KK(891)
        KK(1041)= mu3q1*mu4q3*q1q2*KK(888) + mu3q2*mu4q3*q1q2*KK(888) + 
     &  mu4q3*q1q2**2*z32*KK(891) + mu4q3*q1q2*z33*KK(892) + 
     &  mu4q3*q1q2*z34*KK(896)
        KK(1042)= -16*mu2q3*mu3q2*mu4q1*q1q2 - 
     &  4*mu2q3*mu3q4*mu4q3*q1q2 + 4*mu2mu3*mu4q3*q1q2*q2q3 + 
     &  4*mu2q3*mu3mu4*q1q2*q3q4 + 
     &  16*mu2q1*mu4q3*q1q2**2*z32 + 
     &  mu2q1*mu4q3*q1q2*z33*KK(887) + 
     &  mu2q1*mu4q3*q1q2*z34*KK(888) + mu3q2*KK(1036) + 
     &  mu3q1*KK(1037) + mu2q1*KK(1038) + z23*KK(1039) + 
     &  z22*KK(1040) + z24*KK(1041)
        KK(1043)= mu2q3*mu4q2*KK(891) + mu2q3*mu4q3*KK(966)
        KK(1044)= mu2q3*mu4q1*KK(891) + mu2q3*mu4q2*KK(891) + 
     &  mu2q3*mu4q3*KK(891)
        KK(1045)= mu3q1*mu4q3*KK(891) + mu3q2*mu4q3*KK(891)
        KK(1046)= mu4q3*q1q2*z32*KK(896) + mu3q1*mu4q3*KK(907) + 
     &  mu3q2*mu4q3*KK(907) + mu4q3*z33*KK(909) + 
     &  mu4q3*z34*KK(912)
        KK(1047)= mu4q3*q1q2*z32*KK(892) + mu3q1*mu4q3*KK(905) + 
     &  mu3q2*mu4q3*KK(905) + mu4q3*z34*KK(909) + 
     &  mu4q3*z33*KK(915)
        KK(1048)= mu2q3*mu3q4*mu4q3*KK(871) + 
     &  mu2q1*mu4q3*q1q2*z32*KK(888) + 
     &  mu2q3*mu3q2*mu4q1*KK(891) + mu2q3*mu3mu4*KK(903) + 
     &  mu2mu3*mu4q3*KK(904) + mu2q1*mu4q3*z33*KK(905) + 
     &  mu2q1*mu4q3*z34*KK(907) + z22*KK(1041) + 
     &  mu3q2*KK(1043) + mu3q1*KK(1044) + mu2q1*KK(1045) + 
     &  z24*KK(1046) + z23*KK(1047)
        KK(1049)= mu2q3*mu4q2*KK(889) + mu2q3*mu4q3*KK(961)
        KK(1050)= mu2q3*mu4q1*KK(889) + mu2q3*mu4q2*KK(889) + 
     &  mu2q3*mu4q3*KK(889)
        KK(1051)= mu3q1*mu4q3*KK(889) + mu3q2*mu4q3*KK(889)
        KK(1052)= mu4q3*q1q2*z32*KK(890) + mu4q3*z34*KK(915) + 
     &  mu3q1*mu4q3*KK(924) + mu3q2*mu4q3*KK(924) + 
     &  mu4q3*z33*KK(925)
        KK(1053)= mu2q3*mu3q4*mu4q3*KK(868) + 
     &  mu2q1*mu4q3*q1q2*z32*KK(887) + 
     &  mu2q3*mu3q2*mu4q1*KK(889) + mu2q1*mu4q3*z34*KK(905) + 
     &  mu2mu3*mu4q3*KK(922) + mu2q3*mu3mu4*KK(923) + 
     &  mu2q1*mu4q3*z33*KK(924) + z22*KK(1039) + 
     &  z24*KK(1047) + mu3q2*KK(1049) + mu3q1*KK(1050) + 
     &  mu2q1*KK(1051) + z23*KK(1052)
        KK(1054)= -2*mu1q3*mu2q3*mu3q4*mu4q2 + mu4q1*KK(1000) + 
     &  mu3q1*KK(1001) + mu2q1*KK(1004) + mu3q2*KK(1006) + 
     &  mu1mu2*KK(1007) + mu1mu4*KK(1009) + mu2mu4*KK(1011) + 
     &  mu3mu4*KK(1014) + mu1mu3*KK(1017) + mu2mu3*KK(1020) + 
     &  z32*KK(1022) + z33*KK(1024) + z34*KK(1026) + 
     &  z22*KK(1030) + z23*KK(1033) + z24*KK(1035) + 
     &  z12*KK(1042) + z14*KK(1048) + z13*KK(1053)
        KK(1055)= 4*mu1q4*mu2q3 + 4*mu1q4*mu2q4
        KK(1056)= -2*mu1q2*mu2q4*mu3q4 - 2*mu1q3*mu2q4*mu3q4 + mu3q2*KK(1055)
        KK(1057)= 4*mu1q4*mu2q3*mu4q3 + 2*mu1q4*mu2q4*mu4q3 + mu4q2*KK(1055)
        KK(1058)= 4*mu1q3*mu2q4 + 4*mu1q4*mu2q4
        KK(1059)= 4*mu1q3*mu2q4*mu4q3 + 2*mu1q4*mu2q4*mu4q3 + 
     &  mu4q1*KK(1058) + mu4q2*KK(1058)
        KK(1060)= 16*mu1q4*mu3q2 + 4*mu1q2*mu3q4 + 4*mu1q4*mu3q4
        KK(1061)= 16*mu1q4*mu4q1 + 16*mu1q4*mu4q2 + 16*mu1q4*mu4q3
        KK(1062)= 4*mu3q4*mu4q2 + 4*mu3q4*mu4q3
        KK(1063)= 2*mu1q4*mu3q4*mu4q2 + 4*mu1q4*mu3q4*mu4q3 + 
     &  mu3q2*KK(610) + mu4q1*KK(1060) + mu3q1*KK(1061) + 
     &  mu1q2*KK(1062)
        KK(1064)= -4*mu3q4*mu4q3*q1q2 + mu3q4*mu4q2*KK(856) + 
     &  mu3q4*mu4q1*KK(947)
        KK(1065)= 2*q1q2 + 2*q1q3
        KK(1066)= -4*mu2q4*mu3q2*q1q4 + mu2q1*mu3q4*KK(942) + 
     &  mu2q4*mu3q1*KK(1008) + mu2q4*mu3q4*KK(1065)
        KK(1067)= -2*q1q2 + 2*q2q3
        KK(1068)= 2*mu1q2*mu3q4*q1q4 - 4*mu1q4*mu3q1*q2q4 + 
     &  mu1q4*mu3q2*KK(1010) + mu1q4*mu3q4*KK(1067)
        KK(1069)= 2*mu1q3*mu2q4*q1q4 + 2*mu1q4*mu2q3*q2q4 - 
     &  4*mu1q4*mu2q1*q3q4 + mu1q4*mu2q4*KK(268)
        KK(1070)= -4*mu2q4*mu4q2*q1q3 + mu2q4*mu4q3*KK(863) + 
     &  mu2q4*mu4q1*KK(1016)
        KK(1071)= -4*mu1q4*mu4q1*q2q3 + mu1q4*mu4q3*KK(954) + 
     &  mu1q4*mu4q2*KK(1019)
        KK(1072)= -16*mu2q4*mu4q2*q1q2 - 16*mu2q4*mu4q3*q1q2
        KK(1073)= -16*mu2q4*mu4q1*q1q2 - 16*mu2q4*mu4q2*q1q2 - 
     &  16*mu2q4*mu4q3*q1q2
        KK(1074)= -16*mu2q4*mu3q2*mu4q1*q1q2 - 
     &  4*mu2q4*mu3q4*mu4q3*q1q2 + 4*mu2q4*mu3mu4*q1q2*q3q4 + 
     &  mu3q2*KK(1072) + mu3q1*KK(1073)
        KK(1075)= mu2q4*mu4q2*KK(889) + mu2q4*mu4q3*KK(889)
        KK(1076)= mu2q4*mu4q1*KK(889) + mu2q4*mu4q2*KK(889) + 
     &  mu2q4*mu4q3*KK(889)
        KK(1077)= mu2q4*mu3q4*mu4q3*KK(868) + 
     &  mu2q4*mu3q2*mu4q1*KK(889) + mu2q4*mu3mu4*KK(923) + 
     &  mu3q2*KK(1075) + mu3q1*KK(1076)
        KK(1078)= mu2q4*mu4q2*KK(891) + mu2q4*mu4q3*KK(891)
        KK(1079)= mu2q4*mu4q1*KK(891) + mu2q4*mu4q2*KK(891) + 
     &  mu2q4*mu4q3*KK(891)
        KK(1080)= mu2q4*mu3q4*mu4q3*KK(871) + 
     &  mu2q4*mu3q2*mu4q1*KK(891) + mu2q4*mu3mu4*KK(903) + 
     &  mu3q2*KK(1078) + mu3q1*KK(1079)
        KK(1081)= -16*mu1q4*mu4q1*q1q2 - 16*mu1q4*mu4q2*q1q2 - 
     &  16*mu1q4*mu4q3*q1q2
        KK(1082)= -4*mu1q4*mu2q4*mu4q1*q1q2 - 
     &  4*mu1q4*mu2q4*mu4q2*q1q2 + 4*mu1mu4*mu2q4*q1q2*q1q4 + 
     &  4*mu1q4*mu2mu4*q1q2*q2q4 + mu2q1*KK(1081)
        KK(1083)= mu1q4*mu4q1*KK(889) + mu1q4*mu4q2*KK(889) + 
     &  mu1q4*mu4q3*KK(889)
        KK(1084)= mu1q4*mu2q4*mu4q1*KK(868) + 
     &  mu1q4*mu2q4*mu4q2*KK(868) + mu1mu4*mu2q4*KK(878) + 
     &  mu1q4*mu2mu4*KK(964) + mu2q1*KK(1083)
        KK(1085)= mu1q4*mu4q1*KK(891) + mu1q4*mu4q2*KK(891) + 
     &  mu1q4*mu4q3*KK(891)
        KK(1086)= mu1q4*mu2q4*mu4q1*KK(871) + 
     &  mu1q4*mu2q4*mu4q2*KK(871) + mu1mu4*mu2q4*KK(882) + 
     &  mu1q4*mu2mu4*KK(969) + mu2q1*KK(1085)
        KK(1087)= 16*mu1q4*mu4q1*q1q2**2 + 16*mu1q4*mu4q2*q1q2**2 + 
     &  16*mu1q4*mu4q3*q1q2**2
        KK(1088)= mu1q4*mu4q1*q1q2*KK(887) + mu1q4*mu4q2*q1q2*KK(887) + 
     &  mu1q4*mu4q3*q1q2*KK(887)
        KK(1089)= mu1q4*mu4q1*q1q2*KK(888) + mu1q4*mu4q2*q1q2*KK(888) + 
     &  mu1q4*mu4q3*q1q2*KK(888)
        KK(1090)= z32*KK(1087) + z33*KK(1088) + z34*KK(1089)
        KK(1091)= mu1q4*mu4q1*KK(924) + mu1q4*mu4q2*KK(924) + 
     &  mu1q4*mu4q3*KK(924)
        KK(1092)= mu1q4*mu4q1*KK(905) + mu1q4*mu4q2*KK(905) + 
     &  mu1q4*mu4q3*KK(905)
        KK(1093)= z32*KK(1088) + z33*KK(1091) + z34*KK(1092)
        KK(1094)= mu1q4*mu4q1*KK(907) + mu1q4*mu4q2*KK(907) + 
     &  mu1q4*mu4q3*KK(907)
        KK(1095)= z32*KK(1089) + z33*KK(1092) + z34*KK(1094)
        KK(1096)= -2*mu1q4*mu2q3*mu3q4*mu4q2 + mu4q1*KK(1056) + 
     &  mu3q2*KK(1057) + mu3q1*KK(1059) + mu2q1*KK(1063) + 
     &  mu1mu2*KK(1064) + mu1mu4*KK(1066) + mu2mu4*KK(1068) + 
     &  mu3mu4*KK(1069) + mu1mu3*KK(1070) + mu2mu3*KK(1071) + 
     &  z12*KK(1074) + z13*KK(1077) + z14*KK(1080) + 
     &  z32*KK(1082) + z33*KK(1084) + z34*KK(1086) + 
     &  z22*KK(1090) + z23*KK(1093) + z24*KK(1095)
        KK(1097)= 8*mu2q1*mu3q1*mu4q1*z14*Dij123(dd31) + 
     &  8*mu2q1*mu3q2*mu4q2*z14*Dij123(dd36) + 
     &  8*mu2q3*mu3q1*mu4q3*z14*Dij123(dd37) + 
     &  8*mu2q3*mu3q2*mu4q2*z14*Dij123(dd38) + 
     &  8*mu2q3*mu3q2*mu4q3*z14*Dij123(dd39) - 
     &  8*mu2q3*mu3q2*mu4q2*z11*Dij234(dd34) - 
     &  8*mu2q4*mu3q2*mu4q2*z11*Dij234(dd35) - 
     &  8*mu2q3*mu3q2*mu4q3*z11*Dij234(dd36) - 
     &  8*mu2q4*mu3q4*mu4q2*z11*Dij234(dd37) - 
     &  8*mu2q3*mu3q4*mu4q3*z11*Dij234(dd38) - 
     &  8*mu2q4*mu3q4*mu4q3*z11*Dij234(dd39) + D03p4*KK(8) + 
     &  D02p3*KK(14) + D01p2*KK(25) + E0L*KK(92) + 
     &  D0234*KK(115) + Dij123(dd11)*KK(122) + 
     &  Dij123(dd12)*KK(140) + Dij123(dd13)*KK(156) + 
     &  Dij123(dd21)*KK(158) + Dij123(dd22)*KK(162) + 
     &  z24*Dij123(dd23)*KK(163) + 
     &  mu2q1*z14*Dij123(dd34)*KK(168) + 
     &  Dij123(dd24)*KK(171) + Dij123(dd25)*KK(175) + 
     &  Dij123(dd26)*KK(180) + Dij123(dd27)*KK(185) + 
     &  z14*Dij123(dd310)*KK(186) + 
     &  z14*Dij123(dd311)*KK(187) + 
     &  z14*Dij123(dd312)*KK(188) + 
     &  z14*Dij123(dd313)*KK(189) + 
     &  z14*Dij123(dd35)*KK(190) + Dij1p2(dd11)*KK(267) + 
     &  Dij1p2(dd12)*KK(299) + Dij1p2(dd13)*KK(323) + 
     &  Dij1p2(dd21)*KK(340) + Dij1p2(dd22)*KK(342) + 
     &  Dij1p2(dd23)*KK(345) + Dij1p2(dd24)*KK(358) + 
     &  Dij1p2(dd25)*KK(381) + Dij1p2(dd26)*KK(387) + 
     &  Dij1p2(dd27)*KK(394) + Dij1p2(dd31)*KK(395) + 
     &  Dij1p2(dd310)*KK(398) + Dij1p2(dd311)*KK(401) + 
     &  z11*Dij234(dd312)*KK(402) + Dij1p2(dd312)*KK(403) + 
     &  z11*Dij234(dd313)*KK(405) + Dij1p2(dd313)*KK(406) + 
     &  Dij1p2(dd34)*KK(411) + Dij1p2(dd35)*KK(416) + 
     &  Dij1p2(dd36)*KK(419) + Dij1p2(dd37)*KK(422) + 
     &  Dij1p2(dd38)*KK(423) + Dij1p2(dd39)*KK(424) + 
     &  Dij234(dd11)*KK(437) + Dij234(dd12)*KK(445) + 
     &  Dij234(dd13)*KK(452) + Dij234(dd21)*KK(454) + 
     &  Dij234(dd22)*KK(455) + Dij234(dd23)*KK(456) + 
     &  Dij234(dd24)*KK(460) + Dij234(dd25)*KK(464) + 
     &  Dij234(dd26)*KK(466) + Dij234(dd27)*KK(468) + 
     &  z11*Dij234(dd310)*KK(469) + 
     &  z11*Dij234(dd311)*KK(470) + Dij2p3(dd11)*KK(489) + 
     &  Dij2p3(dd12)*KK(557) + Dij2p3(dd13)*KK(562) + 
     &  Dij2p3(dd21)*KK(566) + Dij2p3(dd22)*KK(581) + 
     &  Dij2p3(dd23)*KK(582) + Dij2p3(dd24)*KK(600) + 
     &  Dij2p3(dd25)*KK(606) + Dij2p3(dd26)*KK(625) + 
     &  Dij2p3(dd27)*KK(629) + Dij2p3(dd31)*KK(630) + 
     &  Dij2p3(dd310)*KK(635) + Dij2p3(dd311)*KK(637) + 
     &  Dij2p3(dd312)*KK(640) + Dij2p3(dd313)*KK(641) + 
     &  Dij2p3(dd32)*KK(644) + Dij2p3(dd34)*KK(647) + 
     &  Dij2p3(dd35)*KK(650) + Dij2p3(dd36)*KK(653) + 
     &  Dij2p3(dd37)*KK(654) + Dij2p3(dd38)*KK(657) + 
     &  Dij2p3(dd39)*KK(660) + Dij3p4(dd11)*KK(673) + 
     &  Dij3p4(dd12)*KK(689) + Dij3p4(dd13)*KK(742) + 
     &  Dij3p4(dd21)*KK(746) + Dij3p4(dd22)*KK(750) + 
     &  Dij3p4(dd23)*KK(762) + Dij3p4(dd24)*KK(772) + 
     &  Dij3p4(dd25)*KK(792) + Dij3p4(dd26)*KK(816) + 
     &  Dij3p4(dd27)*KK(820) + Dij3p4(dd31)*KK(821) + 
     &  Dij3p4(dd310)*KK(824) + Dij3p4(dd311)*KK(825) + 
     &  Dij3p4(dd312)*KK(826) + Dij3p4(dd313)*KK(829) + 
     &  Dij3p4(dd33)*KK(832) + Dij3p4(dd34)*KK(833) + 
     &  Dij3p4(dd35)*KK(836) + Dij3p4(dd36)*KK(837) + 
     &  Dij3p4(dd37)*KK(844) + Dij3p4(dd38)*KK(845) + 
     &  Dij3p4(dd39)*KK(848) + E1iL(1)*KK(928) + 
     &  E1iL(2)*KK(998) + E1iL(3)*KK(1054) + E1iL(4)*KK(1096)
        H4G_Loop=KK(1097)

      RETURN
      END

***********************************************************************
      FUNCTION A4G_LoopDD(i1,i2,i3,i4,m)
*----------------------------------------------------------------------
c author: Michael Kubocz
c date    : 01.08.08
*----------------------------------------------------------------------
c   This pentagon subroutine uses the Denner-Dittmaier tensor decomposition.      
c   i1,i2,i3,i4 gives the permutation of the 4 gluons
c   input gluon momenta and polarization vectors 
c   are stored via the needed dot products in q_dot_ep,
c   ep_dot_ep and q_dot_q and explicitly in qi and eps

c   Normalization: this subroutine gives the pentagon contributions
c   without a factor 4*i*mt the QCD couplings and Yukawa coupling h_t 
c   (gamma_5). Thus, for the full amplitude multiply by an overall
c   normalization factor of 4 mt h_t alpha_s**2 c_i .

c                     * -- i1    
c                   /   \
c           i2 -- *      \
c                 |       * -- A^0
c           i3 -- *      /
c                   \   /
c                     * -- i4

c   When summing the two Feynman graphs with opposite fermion flow
c   below. The color factors c_i is defined in the CP even NP paper
c   eq.3.43
***********************************************************************

      IMPLICIT NONE

#include "ggf_process.inc"
#include "d_coefficients.inc"
#include "ggf_dotproducts.inc"

      complex*16 A4G_LoopDD

c   four vectors
      real*8 eps,qi,m
      COMMON /bh4g_mom_eps/ qi(0:3,4),eps(0:3,4)

      real*8 ALFAS,XMT,ALFA,XMZ,XMW,SIN2W,XMH
      COMMON /BKOPIN/ ALFAS,XMT,ALFA,XMZ,XMW,SIN2W,XMH

c   explicit momenta for call of eps tensor
      real*8  q1(0:3),q2(0:3),q3(0:3),q4(0:3),
     &        mu1(0:3),mu2(0:3),mu3(0:3),mu4(0:3)

c   Passarino-Veltman/Denner-Dittmaier form factors
      complex*16 C0_1M,C034,
     &           D0_1M,D0234,Dij(3,13), 
     &           E0L,Eij(4,46)

c   local dot products 
      real*8 q1sq,q2sq,q3sq,q4sq,q12sq,q34sq,q1234sq,
     &       q1q2,q1q3,q1q4,q2q3,q2q4,q3q4,
     &       q1J1,q2J1,q3J1,q4J1,q1J2,q2J2,q3J2,q4J2,
     &       q1J3,q2J3,q3J3,q4J3,q1J4,q2J4,q3J4,q4J4,
     &       J1J2,J1J3,J1J4,J2J3,J2J4,J3J4
      
c   other variables
      integer i1,i2,i3,i4
      integer i,j,mu
      integer id(4)

      integer init,ie0(4,4,4,4)
      data init/0/, ie0/256*0/
      save ie0,init

      complex*16 coeffEmu,coeffEmunu,coeffD0,coeffDmu,coeffE0C0,coeffE0

      real*8 dotrr,epsrrrr

      EXTERNAL dotrr,epsrrrr,C0_1M,D0_1M

      if (init.eq.0) then   
         init = 1
         ie0(4,2,1,3) = 1   ! this is the map from i1,i2,i3,i4 
         ie0(2,1,3,4) = 2   ! to the index of the E0 variables
         ie0(3,2,1,4) = 3
         ie0(2,1,4,3) = 4
         ie0(4,3,2,1) = 5
         ie0(3,4,2,1) = 6
         ie0(2,3,1,4) = 7
         ie0(3,1,4,2) = 8
         ie0(4,2,3,1) = 9
         ie0(2,4,3,1) = 10
         ie0(3,2,4,1) = 11
         ie0(2,3,4,1) = 12
      endif

      if (m.eq.m_top) then
         E0L=-E0t(ie0(i1,i2,i3,i4))
         do i=1,4
            do j=1,46 
               Eij(i,j)=DDEijtop(i,j,ie0(i1,i2,i3,i4))
            enddo
         enddo  
      endif
      if (m.eq.m_bottom) then
         E0L =-E0b(ie0(i1,i2,i3,i4))
         do i=1,4
            do j=1,46 
               Eij(i,j)=DDEijbottom(i,j,ie0(i1,i2,i3,i4))
            enddo
         enddo   
      endif

      do mu = 0,3
         q1(mu) = qi(mu,i1)
         q2(mu) = qi(mu,i2)
         q3(mu) = qi(mu,i3)
         q4(mu) = qi(mu,i4)

         mu1(mu) = eps(mu,i1)
         mu2(mu) = eps(mu,i2)
         mu3(mu) = eps(mu,i3)
         mu4(mu) = eps(mu,i4)
      enddo

      q1q2 = q_dot_q(i1,i2)
      q1q3 = q_dot_q(i1,i3)
      q1q4 = q_dot_q(i1,i4)
      q2q3 = q_dot_q(i2,i3)
      q2q4 = q_dot_q(i2,i4)
      q3q4 = q_dot_q(i3,i4)

      q1J1 = q_dot_ep(i1,i1)
      q2J1 = q_dot_ep(i2,i1)
      q3J1 = q_dot_ep(i3,i1)
      q4J1 = q_dot_ep(i4,i1)

      q1J2 = q_dot_ep(i1,i2)
      q2J2 = q_dot_ep(i2,i2)
      q3J2 = q_dot_ep(i3,i2)
      q4J2 = q_dot_ep(i4,i2)

      q1J3 = q_dot_ep(i1,i3)
      q2J3 = q_dot_ep(i2,i3)
      q3J3 = q_dot_ep(i3,i3)
      q4J3 = q_dot_ep(i4,i3)

      q1J4 = q_dot_ep(i1,i4)
      q2J4 = q_dot_ep(i2,i4)
      q3J4 = q_dot_ep(i3,i4)
      q4J4 = q_dot_ep(i4,i4)

      J1J2 = ep_dot_ep(i1,i2)
      J1J3 = ep_dot_ep(i1,i3)
      J1J4 = ep_dot_ep(i1,i4)
      J2J3 = ep_dot_ep(i2,i3)
      J2J4 = ep_dot_ep(i2,i4)
      J3J4 = ep_dot_ep(i3,i4)

      q1sq=q_dot_q(i1,i1)
      q2sq=q_dot_q(i2,i2)
      q3sq=q_dot_q(i3,i3)
      q4sq=q_dot_q(i4,i4)

      q12sq=2d0*q1q2
      q34sq=2d0*q3q4
      q1234sq=2*(q1q2+q1q3+q1q4+q2q3+q2q4+q3q4)

      C034=C0_1M(m,q3sq,q4sq,q34sq) 

      D0234 = D0_1M(m,q2sq,q3sq,q4sq,q2q3,q2q4,q3q4)
      CALL D_1M_ALL2(m,q2sq,q3sq,q4sq,q2q3,q2q4,q3q4,Dij)

      coeffD0=-4*m*((q1q2+q1q4-2*q1sq+2*q2q3+q2sq+q3q4+q3sq)*
     &    epsrrrr(mu1,mu2,mu3,mu4)-(q2J4+2*q3J4)*epsrrrr(mu1,mu2,mu3,q1)-
     &   q1J4*epsrrrr(mu1,mu2,mu3,q2)-2*(q2J4+q3J4)*epsrrrr(mu1,mu2,mu3,q3)-
     &   (q1J4+q3J4)*epsrrrr(mu1,mu2,mu3,q4)+
     &   (-q2J3+q4J3)*epsrrrr(mu1,mu2,mu4,q1)+q1J3*epsrrrr(mu1,mu2,mu4,q2)+
     &   q4J3*epsrrrr(mu1,mu2,mu4,q3)+(q1J3-2*q2J3)*epsrrrr(mu1,mu2,mu4,q4)+
     &   J3J4*(epsrrrr(mu1,mu2,q1,q2)+epsrrrr(mu1,mu2,q1,q4)+
     &      epsrrrr(mu1,mu2,q3,q4))-q1J2*epsrrrr(mu1,mu3,mu4,q2)-
     &   q4J2*(epsrrrr(mu1,mu3,mu4,q1)+epsrrrr(mu1,mu3,mu4,q3))+
     &   (-q1J2+q3J2)*epsrrrr(mu1,mu3,mu4,q4)-
     &   J2J4*(epsrrrr(mu1,mu3,q1,q2)+epsrrrr(mu1,mu3,q1,q4)+
     &      epsrrrr(mu1,mu3,q3,q4))+
     &   J2J3*(epsrrrr(mu1,mu4,q1,q2)+epsrrrr(mu1,mu4,q1,q4)+
     &      epsrrrr(mu1,mu4,q3,q4))+(q2J1+q4J1)*epsrrrr(mu2,mu3,mu4,q1)+
     &   q4J1*epsrrrr(mu2,mu3,mu4,q3)-q3J1*epsrrrr(mu2,mu3,mu4,q4)+
     &   J1J4*(epsrrrr(mu2,mu3,q1,q2)+epsrrrr(mu2,mu3,q1,q4)+
     &      epsrrrr(mu2,mu3,q3,q4))-
     &   J1J3*(epsrrrr(mu2,mu4,q1,q2)+epsrrrr(mu2,mu4,q1,q4)+
     &      epsrrrr(mu2,mu4,q3,q4))+
     &   J1J2*(epsrrrr(mu3,mu4,q1,q2)+epsrrrr(mu3,mu4,q1,q4)+
     &      epsrrrr(mu3,mu4,q3,q4)))*D0234

      coeffDmu=8*m*(-2*q1q2*Dij(1,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q2q3*Dij(1,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q2sq*Dij(1,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q3*Dij(1,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q2q3*Dij(1,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q3sq*Dij(1,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q4*Dij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q2q4*Dij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q3q4*Dij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)+
     &   q2J4*Dij(1,1)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q3J4*Dij(1,2)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q2J4*Dij(1,1)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q3J4*Dij(1,2)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q2J3*Dij(1,1)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q4J3*Dij(1,3)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q3J2*Dij(1,2)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q4J2*Dij(1,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   (q2J1*Dij(1,1)+q3J1*Dij(1,2)+q4J1*Dij(1,3))*epsrrrr(mu2,mu3,mu4,q2)+
     &   (q2J1*Dij(1,1)+q3J1*Dij(1,2)+q4J1*Dij(1,3))*epsrrrr(mu2,mu3,mu4,q4))

      coeffE0C0=4*m*(-C034+E0L*m**4)*epsrrrr(mu1,mu2,mu3,mu4)

      coeffE0=-4*E0L*m*((m**4+2*q1q2**2+q1sq**2+q1q4*q2q3-q1q3*q2q4+q1q4*q2sq+
     &      q1sq*(3*q1q2+2*q1q3+q1q4+2*q2q3+q2sq+q3q4+q3sq)+
     &      q1q2*(2*q1q3+2*q1q4+2*q2q3+q2sq+q3q4+q3sq))*
     &    epsrrrr(mu1,mu2,mu3,mu4)-4*q1J4*q1q2*epsrrrr(mu1,mu2,mu3,q1)-
     &   2*q1J4*q1sq*epsrrrr(mu1,mu2,mu3,q1)-
     &   2*q1q2*q2J4*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q1q3*q2J4*epsrrrr(mu1,mu2,mu3,q1)-q1sq*q2J4*epsrrrr(mu1,mu2,mu3,q1)-
     &   2*q1J4*q2q3*epsrrrr(mu1,mu2,mu3,q1)-
     &   2*q1J4*q2sq*epsrrrr(mu1,mu2,mu3,q1)-q2J4*q2sq*epsrrrr(mu1,mu2,mu3,q1)-
     &   4*q1q2*q3J4*epsrrrr(mu1,mu2,mu3,q1)-
     &   2*q1sq*q3J4*epsrrrr(mu1,mu2,mu3,q1)-
     &   2*q2q3*q3J4*epsrrrr(mu1,mu2,mu3,q1)-q2q4*q3J4*epsrrrr(mu1,mu2,mu3,q1)-
     &   2*q2sq*q3J4*epsrrrr(mu1,mu2,mu3,q1)+q2J4*q3q4*epsrrrr(mu1,mu2,mu3,q1)+
     &   q2J4*q3sq*epsrrrr(mu1,mu2,mu3,q1)-2*q1J4*q1q2*epsrrrr(mu1,mu2,mu3,q2)-
     &   q1J4*q1sq*epsrrrr(mu1,mu2,mu3,q2)+2*q1q3*q2J4*epsrrrr(mu1,mu2,mu3,q2)-
     &   2*q1J4*q2q3*epsrrrr(mu1,mu2,mu3,q2)-q1J4*q2sq*epsrrrr(mu1,mu2,mu3,q2)+
     &   2*q1q3*q3J4*epsrrrr(mu1,mu2,mu3,q2)+q1q4*q3J4*epsrrrr(mu1,mu2,mu3,q2)-
     &   q1J4*q3q4*epsrrrr(mu1,mu2,mu3,q2)-q1J4*q3sq*epsrrrr(mu1,mu2,mu3,q2)-
     &   2*q1J4*q1q2*epsrrrr(mu1,mu2,mu3,q3)-
     &   2*q1J4*q1sq*epsrrrr(mu1,mu2,mu3,q3)-
     &   2*q1q2*q2J4*epsrrrr(mu1,mu2,mu3,q3)-q1q4*q2J4*epsrrrr(mu1,mu2,mu3,q3)-
     &   2*q1sq*q2J4*epsrrrr(mu1,mu2,mu3,q3)+q1J4*q2q4*epsrrrr(mu1,mu2,mu3,q3)-
     &   2*q1q2*q3J4*epsrrrr(mu1,mu2,mu3,q3)-
     &   2*q1sq*q3J4*epsrrrr(mu1,mu2,mu3,q3)-
     &   2*q1J4*q1q2*epsrrrr(mu1,mu2,mu3,q4)-q1J4*q1sq*epsrrrr(mu1,mu2,mu3,q4)+
     &   q1q3*q2J4*epsrrrr(mu1,mu2,mu3,q4)-q1J4*q2q3*epsrrrr(mu1,mu2,mu3,q4)-
     &   q1J4*q2sq*epsrrrr(mu1,mu2,mu3,q4)-q1q2*q3J4*epsrrrr(mu1,mu2,mu3,q4)-
     &   q1sq*q3J4*epsrrrr(mu1,mu2,mu3,q4)-2*q1q2*q2J3*epsrrrr(mu1,mu2,mu4,q1)-
     &   2*q1q3*q2J3*epsrrrr(mu1,mu2,mu4,q1)-q1sq*q2J3*epsrrrr(mu1,mu2,mu4,q1)-
     &   2*q2J3*q2q3*epsrrrr(mu1,mu2,mu4,q1)-
     &   2*q1J3*q2q4*epsrrrr(mu1,mu2,mu4,q1)-
     &   2*q2J3*q2q4*epsrrrr(mu1,mu2,mu4,q1)-q2J3*q2sq*epsrrrr(mu1,mu2,mu4,q1)-
     &   q2J3*q3q4*epsrrrr(mu1,mu2,mu4,q1)-q2J3*q3sq*epsrrrr(mu1,mu2,mu4,q1)+
     &   2*q1q2*q4J3*epsrrrr(mu1,mu2,mu4,q1)+q1sq*q4J3*epsrrrr(mu1,mu2,mu4,q1)+
     &   q2q3*q4J3*epsrrrr(mu1,mu2,mu4,q1)+q2sq*q4J3*epsrrrr(mu1,mu2,mu4,q1)+
     &   2*q1J3*q1q2*epsrrrr(mu1,mu2,mu4,q2)+
     &   2*q1J3*q1q3*epsrrrr(mu1,mu2,mu4,q2)+
     &   2*q1J3*q1q4*epsrrrr(mu1,mu2,mu4,q2)+q1J3*q1sq*epsrrrr(mu1,mu2,mu4,q2)+
     &   2*q1q4*q2J3*epsrrrr(mu1,mu2,mu4,q2)+
     &   2*q1J3*q2q3*epsrrrr(mu1,mu2,mu4,q2)+q1J3*q2sq*epsrrrr(mu1,mu2,mu4,q2)+
     &   q1J3*q3q4*epsrrrr(mu1,mu2,mu4,q2)+q1J3*q3sq*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1q3*q4J3*epsrrrr(mu1,mu2,mu4,q2)+q1q4*q2J3*epsrrrr(mu1,mu2,mu4,q3)-
     &   q1J3*q2q4*epsrrrr(mu1,mu2,mu4,q3)+q1q2*q4J3*epsrrrr(mu1,mu2,mu4,q3)+
     &   q1sq*q4J3*epsrrrr(mu1,mu2,mu4,q3)-q1J3*q1sq*epsrrrr(mu1,mu2,mu4,q4)-
     &   2*q1q2*q2J3*epsrrrr(mu1,mu2,mu4,q4)-q1q3*q2J3*epsrrrr(mu1,mu2,mu4,q4)-
     &   2*q1sq*q2J3*epsrrrr(mu1,mu2,mu4,q4)+q1J3*q2q3*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1J3*q2sq*epsrrrr(mu1,mu2,mu4,q4)-4*q1J3*q1J4*epsrrrr(mu1,mu2,q1,q2)+
     &   2*J3J4*q1q2*epsrrrr(mu1,mu2,q1,q2)+2*J3J4*q1q3*epsrrrr(mu1,mu2,q1,q2)+
     &   J3J4*q1sq*epsrrrr(mu1,mu2,q1,q2)-4*q1J4*q2J3*epsrrrr(mu1,mu2,q1,q2)-
     &   4*q1J3*q2J4*epsrrrr(mu1,mu2,q1,q2)-4*q2J3*q2J4*epsrrrr(mu1,mu2,q1,q2)+
     &   2*J3J4*q2q3*epsrrrr(mu1,mu2,q1,q2)+J3J4*q2sq*epsrrrr(mu1,mu2,q1,q2)-
     &   4*q1J3*q3J4*epsrrrr(mu1,mu2,q1,q2)-4*q2J3*q3J4*epsrrrr(mu1,mu2,q1,q2)+
     &   J3J4*q3q4*epsrrrr(mu1,mu2,q1,q2)+J3J4*q3sq*epsrrrr(mu1,mu2,q1,q2)-
     &   q3J4*q4J3*epsrrrr(mu1,mu2,q1,q2)-2*q1J4*q2J3*epsrrrr(mu1,mu2,q1,q3)-
     &   2*q2J3*q2J4*epsrrrr(mu1,mu2,q1,q3)-J3J4*q2q4*epsrrrr(mu1,mu2,q1,q3)-
     &   2*q2J3*q3J4*epsrrrr(mu1,mu2,q1,q3)+q2J4*q4J3*epsrrrr(mu1,mu2,q1,q3)+
     &   2*J3J4*q1q2*epsrrrr(mu1,mu2,q1,q4)+J3J4*q1sq*epsrrrr(mu1,mu2,q1,q4)-
     &   2*q1J3*q2J4*epsrrrr(mu1,mu2,q1,q4)-2*q2J3*q2J4*epsrrrr(mu1,mu2,q1,q4)+
     &   J3J4*q2q3*epsrrrr(mu1,mu2,q1,q4)+J3J4*q2sq*epsrrrr(mu1,mu2,q1,q4)-
     &   q2J3*q3J4*epsrrrr(mu1,mu2,q1,q4)+2*q1J3*q1J4*epsrrrr(mu1,mu2,q2,q3)+
     &   J3J4*q1q4*epsrrrr(mu1,mu2,q2,q3)+2*q1J3*q2J4*epsrrrr(mu1,mu2,q2,q3)+
     &   2*q1J3*q3J4*epsrrrr(mu1,mu2,q2,q3)-q1J4*q4J3*epsrrrr(mu1,mu2,q2,q3)+
     &   2*q1J3*q1J4*epsrrrr(mu1,mu2,q2,q4)-J3J4*q1q3*epsrrrr(mu1,mu2,q2,q4)+
     &   2*q1J4*q2J3*epsrrrr(mu1,mu2,q2,q4)+q1J3*q3J4*epsrrrr(mu1,mu2,q2,q4)+
     &   J3J4*q1q2*epsrrrr(mu1,mu2,q3,q4)+J3J4*q1sq*epsrrrr(mu1,mu2,q3,q4)+
     &   q1J4*q2J3*epsrrrr(mu1,mu2,q3,q4)-q1J3*q2J4*epsrrrr(mu1,mu2,q3,q4)-
     &   4*q1J2*q1q2*epsrrrr(mu1,mu3,mu4,q1)-
     &   4*q1J2*q1q3*epsrrrr(mu1,mu3,mu4,q1)-
     &   2*q1J2*q1sq*epsrrrr(mu1,mu3,mu4,q1)-
     &   4*q1J2*q2q3*epsrrrr(mu1,mu3,mu4,q1)-
     &   2*q1J2*q2sq*epsrrrr(mu1,mu3,mu4,q1)+q2q4*q3J2*epsrrrr(mu1,mu3,mu4,q1)-
     &   2*q1J2*q3q4*epsrrrr(mu1,mu3,mu4,q1)-
     &   2*q1J2*q3sq*epsrrrr(mu1,mu3,mu4,q1)-
     &   2*q1q2*q4J2*epsrrrr(mu1,mu3,mu4,q1)-q1sq*q4J2*epsrrrr(mu1,mu3,mu4,q1)-
     &   q2q3*q4J2*epsrrrr(mu1,mu3,mu4,q1)-q2sq*q4J2*epsrrrr(mu1,mu3,mu4,q1)-
     &   2*q1J2*q1q2*epsrrrr(mu1,mu3,mu4,q2)-
     &   2*q1J2*q1q3*epsrrrr(mu1,mu3,mu4,q2)-q1J2*q1sq*epsrrrr(mu1,mu3,mu4,q2)-
     &   2*q1J2*q2q3*epsrrrr(mu1,mu3,mu4,q2)-q1J2*q2sq*epsrrrr(mu1,mu3,mu4,q2)-
     &   q1q4*q3J2*epsrrrr(mu1,mu3,mu4,q2)-q1J2*q3q4*epsrrrr(mu1,mu3,mu4,q2)-
     &   q1J2*q3sq*epsrrrr(mu1,mu3,mu4,q2)+q1q3*q4J2*epsrrrr(mu1,mu3,mu4,q2)+
     &   2*q1J2*q1q4*epsrrrr(mu1,mu3,mu4,q3)+q1J2*q2q4*epsrrrr(mu1,mu3,mu4,q3)-
     &   q1q2*q4J2*epsrrrr(mu1,mu3,mu4,q3)-q1sq*q4J2*epsrrrr(mu1,mu3,mu4,q3)-
     &   2*q1J2*q1q2*epsrrrr(mu1,mu3,mu4,q4)-
     &   2*q1J2*q1q3*epsrrrr(mu1,mu3,mu4,q4)-q1J2*q1sq*epsrrrr(mu1,mu3,mu4,q4)-
     &   q1J2*q2q3*epsrrrr(mu1,mu3,mu4,q4)-q1J2*q2sq*epsrrrr(mu1,mu3,mu4,q4)+
     &   q1q2*q3J2*epsrrrr(mu1,mu3,mu4,q4)+q1sq*q3J2*epsrrrr(mu1,mu3,mu4,q4)-
     &   2*J2J4*q1q2*epsrrrr(mu1,mu3,q1,q2)-2*J2J4*q1q3*epsrrrr(mu1,mu3,q1,q2)-
     &   J2J4*q1sq*epsrrrr(mu1,mu3,q1,q2)-2*J2J4*q2q3*epsrrrr(mu1,mu3,q1,q2)-
     &   J2J4*q2sq*epsrrrr(mu1,mu3,q1,q2)+2*q1J4*q3J2*epsrrrr(mu1,mu3,q1,q2)+
     &   2*q2J4*q3J2*epsrrrr(mu1,mu3,q1,q2)+2*q3J2*q3J4*epsrrrr(mu1,mu3,q1,q2)-
     &   J2J4*q3q4*epsrrrr(mu1,mu3,q1,q2)-J2J4*q3sq*epsrrrr(mu1,mu3,q1,q2)+
     &   q3J4*q4J2*epsrrrr(mu1,mu3,q1,q2)-4*q1J2*q1J4*epsrrrr(mu1,mu3,q1,q3)-
     &   4*q1J2*q2J4*epsrrrr(mu1,mu3,q1,q3)+J2J4*q2q4*epsrrrr(mu1,mu3,q1,q3)-
     &   4*q1J2*q3J4*epsrrrr(mu1,mu3,q1,q3)-q2J4*q4J2*epsrrrr(mu1,mu3,q1,q3)-
     &   2*J2J4*q1q2*epsrrrr(mu1,mu3,q1,q4)-J2J4*q1sq*epsrrrr(mu1,mu3,q1,q4)-
     &   J2J4*q2q3*epsrrrr(mu1,mu3,q1,q4)-J2J4*q2sq*epsrrrr(mu1,mu3,q1,q4)+
     &   q2J4*q3J2*epsrrrr(mu1,mu3,q1,q4)-2*q1J2*q3J4*epsrrrr(mu1,mu3,q1,q4)-
     &   2*q1J2*q1J4*epsrrrr(mu1,mu3,q2,q3)-J2J4*q1q4*epsrrrr(mu1,mu3,q2,q3)-
     &   2*q1J2*q2J4*epsrrrr(mu1,mu3,q2,q3)-2*q1J2*q3J4*epsrrrr(mu1,mu3,q2,q3)+
     &   q1J4*q4J2*epsrrrr(mu1,mu3,q2,q3)+J2J4*q1q3*epsrrrr(mu1,mu3,q2,q4)-
     &   q1J4*q3J2*epsrrrr(mu1,mu3,q2,q4)-q1J2*q3J4*epsrrrr(mu1,mu3,q2,q4)+
     &   2*q1J2*q1J4*epsrrrr(mu1,mu3,q3,q4)-J2J4*q1q2*epsrrrr(mu1,mu3,q3,q4)-
     &   J2J4*q1sq*epsrrrr(mu1,mu3,q3,q4)+q1J2*q2J4*epsrrrr(mu1,mu3,q3,q4)+
     &   2*J2J3*q1q2*epsrrrr(mu1,mu4,q1,q2)+2*J2J3*q1q3*epsrrrr(mu1,mu4,q1,q2)+
     &   J2J3*q1sq*epsrrrr(mu1,mu4,q1,q2)+2*J2J3*q2q3*epsrrrr(mu1,mu4,q1,q2)+
     &   J2J3*q2sq*epsrrrr(mu1,mu4,q1,q2)+J2J3*q3q4*epsrrrr(mu1,mu4,q1,q2)+
     &   J2J3*q3sq*epsrrrr(mu1,mu4,q1,q2)+2*q1J3*q4J2*epsrrrr(mu1,mu4,q1,q2)+
     &   2*q2J3*q4J2*epsrrrr(mu1,mu4,q1,q2)-q3J2*q4J3*epsrrrr(mu1,mu4,q1,q2)-
     &   J2J3*q2q4*epsrrrr(mu1,mu4,q1,q3)+q2J3*q4J2*epsrrrr(mu1,mu4,q1,q3)+
     &   2*q1J2*q4J3*epsrrrr(mu1,mu4,q1,q3)-4*q1J2*q1J3*epsrrrr(mu1,mu4,q1,q4)+
     &   2*J2J3*q1q2*epsrrrr(mu1,mu4,q1,q4)+J2J3*q1sq*epsrrrr(mu1,mu4,q1,q4)-
     &   4*q1J2*q2J3*epsrrrr(mu1,mu4,q1,q4)+J2J3*q2q3*epsrrrr(mu1,mu4,q1,q4)+
     &   J2J3*q2sq*epsrrrr(mu1,mu4,q1,q4)-q2J3*q3J2*epsrrrr(mu1,mu4,q1,q4)+
     &   J2J3*q1q4*epsrrrr(mu1,mu4,q2,q3)-q1J3*q4J2*epsrrrr(mu1,mu4,q2,q3)+
     &   q1J2*q4J3*epsrrrr(mu1,mu4,q2,q3)-2*q1J2*q1J3*epsrrrr(mu1,mu4,q2,q4)-
     &   J2J3*q1q3*epsrrrr(mu1,mu4,q2,q4)-2*q1J2*q2J3*epsrrrr(mu1,mu4,q2,q4)+
     &   q1J3*q3J2*epsrrrr(mu1,mu4,q2,q4)-2*q1J2*q1J3*epsrrrr(mu1,mu4,q3,q4)+
     &   J2J3*q1q2*epsrrrr(mu1,mu4,q3,q4)+J2J3*q1sq*epsrrrr(mu1,mu4,q3,q4)-
     &   q1J2*q2J3*epsrrrr(mu1,mu4,q3,q4)-2*J2J3*q1J4*epsrrrr(mu1,q1,q2,q3)-
     &   2*J2J3*q2J4*epsrrrr(mu1,q1,q2,q3)-2*J2J3*q3J4*epsrrrr(mu1,q1,q2,q3)-
     &   J3J4*q4J2*epsrrrr(mu1,q1,q2,q3)+J2J4*q4J3*epsrrrr(mu1,q1,q2,q3)-
     &   2*J2J4*q1J3*epsrrrr(mu1,q1,q2,q4)-2*J2J4*q2J3*epsrrrr(mu1,q1,q2,q4)+
     &   J3J4*q3J2*epsrrrr(mu1,q1,q2,q4)-J2J3*q3J4*epsrrrr(mu1,q1,q2,q4)-
     &   2*J3J4*q1J2*epsrrrr(mu1,q1,q3,q4)-J2J4*q2J3*epsrrrr(mu1,q1,q3,q4)+
     &   J2J3*q2J4*epsrrrr(mu1,q1,q3,q4)-J3J4*q1J2*epsrrrr(mu1,q2,q3,q4)+
     &   J2J4*q1J3*epsrrrr(mu1,q2,q3,q4)-J2J3*q1J4*epsrrrr(mu1,q2,q3,q4)+
     &   2*q1q2*q2J1*epsrrrr(mu2,mu3,mu4,q1)+
     &   2*q1q3*q2J1*epsrrrr(mu2,mu3,mu4,q1)+q1sq*q2J1*epsrrrr(mu2,mu3,mu4,q1)+
     &   2*q2J1*q2q3*epsrrrr(mu2,mu3,mu4,q1)+q2J1*q2sq*epsrrrr(mu2,mu3,mu4,q1)-
     &   q2q4*q3J1*epsrrrr(mu2,mu3,mu4,q1)+q2J1*q3q4*epsrrrr(mu2,mu3,mu4,q1)+
     &   q2J1*q3sq*epsrrrr(mu2,mu3,mu4,q1)+2*q1q2*q4J1*epsrrrr(mu2,mu3,mu4,q1)+
     &   q1sq*q4J1*epsrrrr(mu2,mu3,mu4,q1)+q2q3*q4J1*epsrrrr(mu2,mu3,mu4,q1)+
     &   q2sq*q4J1*epsrrrr(mu2,mu3,mu4,q1)+q1q4*q3J1*epsrrrr(mu2,mu3,mu4,q2)-
     &   q1q3*q4J1*epsrrrr(mu2,mu3,mu4,q2)-q1q4*q2J1*epsrrrr(mu2,mu3,mu4,q3)+
     &   q1q2*q4J1*epsrrrr(mu2,mu3,mu4,q3)+q1sq*q4J1*epsrrrr(mu2,mu3,mu4,q3)+
     &   q1q3*q2J1*epsrrrr(mu2,mu3,mu4,q4)-q1q2*q3J1*epsrrrr(mu2,mu3,mu4,q4)-
     &   q1sq*q3J1*epsrrrr(mu2,mu3,mu4,q4)+2*J1J4*q1q2*epsrrrr(mu2,mu3,q1,q2)+
     &   2*J1J4*q1q3*epsrrrr(mu2,mu3,q1,q2)+J1J4*q1sq*epsrrrr(mu2,mu3,q1,q2)+
     &   2*J1J4*q2q3*epsrrrr(mu2,mu3,q1,q2)+J1J4*q2sq*epsrrrr(mu2,mu3,q1,q2)-
     &   2*q1J4*q3J1*epsrrrr(mu2,mu3,q1,q2)-2*q2J4*q3J1*epsrrrr(mu2,mu3,q1,q2)-
     &   2*q3J1*q3J4*epsrrrr(mu2,mu3,q1,q2)+J1J4*q3q4*epsrrrr(mu2,mu3,q1,q2)+
     &   J1J4*q3sq*epsrrrr(mu2,mu3,q1,q2)-q3J4*q4J1*epsrrrr(mu2,mu3,q1,q2)+
     &   2*q1J4*q2J1*epsrrrr(mu2,mu3,q1,q3)+2*q2J1*q2J4*epsrrrr(mu2,mu3,q1,q3)-
     &   J1J4*q2q4*epsrrrr(mu2,mu3,q1,q3)+2*q2J1*q3J4*epsrrrr(mu2,mu3,q1,q3)+
     &   q2J4*q4J1*epsrrrr(mu2,mu3,q1,q3)+2*J1J4*q1q2*epsrrrr(mu2,mu3,q1,q4)+
     &   J1J4*q1sq*epsrrrr(mu2,mu3,q1,q4)+J1J4*q2q3*epsrrrr(mu2,mu3,q1,q4)+
     &   J1J4*q2sq*epsrrrr(mu2,mu3,q1,q4)-q2J4*q3J1*epsrrrr(mu2,mu3,q1,q4)+
     &   q2J1*q3J4*epsrrrr(mu2,mu3,q1,q4)+J1J4*q1q4*epsrrrr(mu2,mu3,q2,q3)-
     &   q1J4*q4J1*epsrrrr(mu2,mu3,q2,q3)-J1J4*q1q3*epsrrrr(mu2,mu3,q2,q4)+
     &   q1J4*q3J1*epsrrrr(mu2,mu3,q2,q4)+J1J4*q1q2*epsrrrr(mu2,mu3,q3,q4)+
     &   J1J4*q1sq*epsrrrr(mu2,mu3,q3,q4)-q1J4*q2J1*epsrrrr(mu2,mu3,q3,q4)-
     &   2*J1J3*q1q2*epsrrrr(mu2,mu4,q1,q2)-2*J1J3*q1q3*epsrrrr(mu2,mu4,q1,q2)-
     &   J1J3*q1sq*epsrrrr(mu2,mu4,q1,q2)-2*J1J3*q2q3*epsrrrr(mu2,mu4,q1,q2)-
     &   J1J3*q2sq*epsrrrr(mu2,mu4,q1,q2)-J1J3*q3q4*epsrrrr(mu2,mu4,q1,q2)-
     &   J1J3*q3sq*epsrrrr(mu2,mu4,q1,q2)-2*q1J3*q4J1*epsrrrr(mu2,mu4,q1,q2)-
     &   2*q2J3*q4J1*epsrrrr(mu2,mu4,q1,q2)+q3J1*q4J3*epsrrrr(mu2,mu4,q1,q2)+
     &   J1J3*q2q4*epsrrrr(mu2,mu4,q1,q3)-q2J3*q4J1*epsrrrr(mu2,mu4,q1,q3)-
     &   q2J1*q4J3*epsrrrr(mu2,mu4,q1,q3)-2*J1J3*q1q2*epsrrrr(mu2,mu4,q1,q4)-
     &   J1J3*q1sq*epsrrrr(mu2,mu4,q1,q4)+2*q1J3*q2J1*epsrrrr(mu2,mu4,q1,q4)+
     &   2*q2J1*q2J3*epsrrrr(mu2,mu4,q1,q4)-J1J3*q2q3*epsrrrr(mu2,mu4,q1,q4)-
     &   J1J3*q2sq*epsrrrr(mu2,mu4,q1,q4)+q2J3*q3J1*epsrrrr(mu2,mu4,q1,q4)-
     &   J1J3*q1q4*epsrrrr(mu2,mu4,q2,q3)+q1J3*q4J1*epsrrrr(mu2,mu4,q2,q3)+
     &   J1J3*q1q3*epsrrrr(mu2,mu4,q2,q4)-q1J3*q3J1*epsrrrr(mu2,mu4,q2,q4)-
     &   J1J3*q1q2*epsrrrr(mu2,mu4,q3,q4)-J1J3*q1sq*epsrrrr(mu2,mu4,q3,q4)+
     &   q1J3*q2J1*epsrrrr(mu2,mu4,q3,q4)+2*J1J3*q1J4*epsrrrr(mu2,q1,q2,q3)+
     &   2*J1J3*q2J4*epsrrrr(mu2,q1,q2,q3)+2*J1J3*q3J4*epsrrrr(mu2,q1,q2,q3)+
     &   J3J4*q4J1*epsrrrr(mu2,q1,q2,q3)-J1J4*q4J3*epsrrrr(mu2,q1,q2,q3)+
     &   2*J1J4*q1J3*epsrrrr(mu2,q1,q2,q4)+2*J1J4*q2J3*epsrrrr(mu2,q1,q2,q4)-
     &   J3J4*q3J1*epsrrrr(mu2,q1,q2,q4)+J1J3*q3J4*epsrrrr(mu2,q1,q2,q4)+
     &   J3J4*q2J1*epsrrrr(mu2,q1,q3,q4)+J1J4*q2J3*epsrrrr(mu2,q1,q3,q4)-
     &   J1J3*q2J4*epsrrrr(mu2,q1,q3,q4)-J1J4*q1J3*epsrrrr(mu2,q2,q3,q4)+
     &   J1J3*q1J4*epsrrrr(mu2,q2,q3,q4)+2*J1J2*q1q2*epsrrrr(mu3,mu4,q1,q2)+
     &   2*J1J2*q1q3*epsrrrr(mu3,mu4,q1,q2)+J1J2*q1sq*epsrrrr(mu3,mu4,q1,q2)+
     &   2*J1J2*q2q3*epsrrrr(mu3,mu4,q1,q2)+J1J2*q2sq*epsrrrr(mu3,mu4,q1,q2)+
     &   J1J2*q3q4*epsrrrr(mu3,mu4,q1,q2)+J1J2*q3sq*epsrrrr(mu3,mu4,q1,q2)+
     &   q3J2*q4J1*epsrrrr(mu3,mu4,q1,q2)-q3J1*q4J2*epsrrrr(mu3,mu4,q1,q2)-
     &   J1J2*q2q4*epsrrrr(mu3,mu4,q1,q3)-2*q1J2*q4J1*epsrrrr(mu3,mu4,q1,q3)+
     &   q2J1*q4J2*epsrrrr(mu3,mu4,q1,q3)+2*J1J2*q1q2*epsrrrr(mu3,mu4,q1,q4)+
     &   J1J2*q1sq*epsrrrr(mu3,mu4,q1,q4)+J1J2*q2q3*epsrrrr(mu3,mu4,q1,q4)+
     &   J1J2*q2sq*epsrrrr(mu3,mu4,q1,q4)+2*q1J2*q3J1*epsrrrr(mu3,mu4,q1,q4)-
     &   q2J1*q3J2*epsrrrr(mu3,mu4,q1,q4)+J1J2*q1q4*epsrrrr(mu3,mu4,q2,q3)-
     &   q1J2*q4J1*epsrrrr(mu3,mu4,q2,q3)-J1J2*q1q3*epsrrrr(mu3,mu4,q2,q4)+
     &   q1J2*q3J1*epsrrrr(mu3,mu4,q2,q4)+J1J2*q1q2*epsrrrr(mu3,mu4,q3,q4)+
     &   J1J2*q1sq*epsrrrr(mu3,mu4,q3,q4)-q1J2*q2J1*epsrrrr(mu3,mu4,q3,q4)-
     &   2*J1J2*q1J4*epsrrrr(mu3,q1,q2,q3)-2*J1J2*q2J4*epsrrrr(mu3,q1,q2,q3)-
     &   2*J1J2*q3J4*epsrrrr(mu3,q1,q2,q3)-J2J4*q4J1*epsrrrr(mu3,q1,q2,q3)+
     &   J1J4*q4J2*epsrrrr(mu3,q1,q2,q3)+J2J4*q3J1*epsrrrr(mu3,q1,q2,q4)-
     &   J1J4*q3J2*epsrrrr(mu3,q1,q2,q4)-J1J2*q3J4*epsrrrr(mu3,q1,q2,q4)+
     &   2*J1J4*q1J2*epsrrrr(mu3,q1,q3,q4)-J2J4*q2J1*epsrrrr(mu3,q1,q3,q4)+
     &   J1J2*q2J4*epsrrrr(mu3,q1,q3,q4)+J1J4*q1J2*epsrrrr(mu3,q2,q3,q4)-
     &   J1J2*q1J4*epsrrrr(mu3,q2,q3,q4)+J2J3*q4J1*epsrrrr(mu4,q1,q2,q3)-
     &   J1J3*q4J2*epsrrrr(mu4,q1,q2,q3)+J1J2*q4J3*epsrrrr(mu4,q1,q2,q3)-
     &   2*J1J2*q1J3*epsrrrr(mu4,q1,q2,q4)-2*J1J2*q2J3*epsrrrr(mu4,q1,q2,q4)-
     &   J2J3*q3J1*epsrrrr(mu4,q1,q2,q4)+J1J3*q3J2*epsrrrr(mu4,q1,q2,q4)-
     &   2*J1J3*q1J2*epsrrrr(mu4,q1,q3,q4)+J2J3*q2J1*epsrrrr(mu4,q1,q3,q4)-
     &   J1J2*q2J3*epsrrrr(mu4,q1,q3,q4)-J1J3*q1J2*epsrrrr(mu4,q2,q3,q4)+
     &   J1J2*q1J3*epsrrrr(mu4,q2,q3,q4)+
     &   (J1J4*J2J3-J1J3*J2J4+J1J2*J3J4)*epsrrrr(q1,q2,q3,q4))

      coeffEmu=8*m*(-(q1q2**2*Eij(1,1)*epsrrrr(mu1,mu2,mu3,mu4))-
     &   q1q2*q1q3*Eij(1,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q1q4*Eij(1,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   4*q1q2*q1sq*Eij(1,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   3*q1q3*q1sq*Eij(1,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4*q1sq*Eij(1,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1sq**2*Eij(1,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1sq*q2q3*Eij(1,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q2sq*Eij(1,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q3q4*Eij(1,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q3sq*Eij(1,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   3*q1q2**2*Eij(1,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q2*q1q3*Eij(1,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q1q4*Eij(1,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q2*q1sq*Eij(1,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   3*q1q2*q2q3*Eij(1,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q2q3*Eij(1,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q2*q2sq*Eij(1,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4*q2sq*Eij(1,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q2sq*Eij(1,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q3q4*Eij(1,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q3sq*Eij(1,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   3*q1q2*q1q3*Eij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q3**2*Eij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q3*q1q4*Eij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q3*q1sq*Eij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q2q3*Eij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q3*q2q3*Eij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4*q2q3*Eij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q2q3*Eij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q3*q2sq*Eij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q3*q3q4*Eij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q3sq*Eij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q3*q3sq*Eij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q3sq*Eij(1,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   3*q1q2*q1q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q3*q1q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4**2*Eij(1,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q4*q1sq*Eij(1,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q4*q2q3*Eij(1,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q2q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4*q2q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q2q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4*q2sq*Eij(1,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q3q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4*q3q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q3q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4*q3sq*Eij(1,4)*epsrrrr(mu1,mu2,mu3,mu4)+
     &   4*q1J4*q1q2*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q1)+
     &   3*q1J4*q1sq*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1q2*q2J4*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q1)-
     &   q1q3*q2J4*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1sq*q2J4*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1J4*q2q3*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1J4*q2sq*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q1q2*q3J4*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q1sq*q3J4*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q1J4*q1q2*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q1)+
     &   3*q1q2*q2J4*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1sq*q2J4*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q1J4*q2sq*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q2J4*q2sq*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q1q2*q3J4*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q2sq*q3J4*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q1J4*q1q3*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1q3*q2J4*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q1J4*q2q3*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q2J4*q2q3*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q1q2*q3J4*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q1q3*q3J4*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1sq*q3J4*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q1)+
     &   3*q2q3*q3J4*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q2sq*q3J4*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q1)-
     &   q2J4*q3sq*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q1J4*q1q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1q4*q2J4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q1J4*q2q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q2J4*q2q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q1q4*q3J4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q1)+
     &   2*q2q4*q3J4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q1)-
     &   q2J4*q3q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1J4*q1q2*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q2)+
     &   q1J4*q1sq*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q2)+
     &   q1J4*q1q2*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q2)-
     &   q1q3*q2J4*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q2)+
     &   q1J4*q2q3*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q2)+
     &   q1J4*q2sq*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q2)+
     &   q1J4*q1q3*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q2)+
     &   q1J4*q2q3*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q2)-
     &   q1q3*q3J4*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q2)+
     &   q1J4*q3sq*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q2)+
     &   q1J4*q1q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q2)+
     &   q1J4*q2q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q2)+
     &   q1J4*q3q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q2)+
     &   q1J4*q1q2*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q3)+
     &   3*q1J4*q1sq*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q3)+
     &   2*q1sq*q2J4*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q3)+
     &   2*q1sq*q3J4*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q3)+
     &   2*q1J4*q1q2*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q3)+
     &   3*q1q2*q2J4*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1sq*q2J4*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q3)+
     &   2*q1q2*q3J4*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q3)+
     &   2*q1J4*q1q3*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q3)+
     &   2*q1q3*q2J4*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1q2*q3J4*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q3)+
     &   2*q1q3*q3J4*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1sq*q3J4*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q3)+
     &   2*q1J4*q1q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q3)+
     &   2*q1q4*q2J4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q3)+
     &   2*q1q4*q3J4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1J4*q1q2*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q4)+
     &   q1J4*q1sq*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q4)+
     &   q1sq*q3J4*Eij(1,1)*epsrrrr(mu1,mu2,mu3,q4)+
     &   q1J4*q1q2*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q4)+
     &   q1J4*q2sq*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q4)+
     &   q1q2*q3J4*Eij(1,2)*epsrrrr(mu1,mu2,mu3,q4)+
     &   q1J4*q1q3*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q4)+
     &   q1J4*q2q3*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q4)+
     &   q1q3*q3J4*Eij(1,3)*epsrrrr(mu1,mu2,mu3,q4)+
     &   q1J4*q1q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q4)+
     &   q1J4*q2q4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q4)+
     &   q1q4*q3J4*Eij(1,4)*epsrrrr(mu1,mu2,mu3,q4)+
     &   q1q2*q2J3*Eij(1,1)*epsrrrr(mu1,mu2,mu4,q1)+
     &   q1q3*q2J3*Eij(1,1)*epsrrrr(mu1,mu2,mu4,q1)+
     &   q1sq*q2J3*Eij(1,1)*epsrrrr(mu1,mu2,mu4,q1)+
     &   q1J3*q2q4*Eij(1,1)*epsrrrr(mu1,mu2,mu4,q1)-
     &   q1q2*q4J3*Eij(1,1)*epsrrrr(mu1,mu2,mu4,q1)-
     &   q1sq*q4J3*Eij(1,1)*epsrrrr(mu1,mu2,mu4,q1)+
     &   q1q2*q2J3*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q1)+
     &   q2J3*q2q3*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q1)+
     &   q2J3*q2q4*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q1)+
     &   q2J3*q2sq*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q1)-
     &   q1q2*q4J3*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q1)-
     &   q2sq*q4J3*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q1)+
     &   q1q3*q2J3*Eij(1,3)*epsrrrr(mu1,mu2,mu4,q1)+
     &   q2J3*q2q3*Eij(1,3)*epsrrrr(mu1,mu2,mu4,q1)+
     &   q2J3*q3sq*Eij(1,3)*epsrrrr(mu1,mu2,mu4,q1)-
     &   q1q3*q4J3*Eij(1,3)*epsrrrr(mu1,mu2,mu4,q1)-
     &   q2q3*q4J3*Eij(1,3)*epsrrrr(mu1,mu2,mu4,q1)+
     &   q1q4*q2J3*Eij(1,4)*epsrrrr(mu1,mu2,mu4,q1)+
     &   q2J3*q2q4*Eij(1,4)*epsrrrr(mu1,mu2,mu4,q1)+
     &   q2J3*q3q4*Eij(1,4)*epsrrrr(mu1,mu2,mu4,q1)-
     &   q1q4*q4J3*Eij(1,4)*epsrrrr(mu1,mu2,mu4,q1)-
     &   q1J3*q1q2*Eij(1,1)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1J3*q1q3*Eij(1,1)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1J3*q1q4*Eij(1,1)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1J3*q1sq*Eij(1,1)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1J3*q1q2*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1q4*q2J3*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1J3*q2q3*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1J3*q2sq*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1J3*q1q3*Eij(1,3)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1J3*q2q3*Eij(1,3)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1J3*q3sq*Eij(1,3)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1J3*q1q4*Eij(1,4)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1J3*q2q4*Eij(1,4)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1J3*q3q4*Eij(1,4)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1q4*q4J3*Eij(1,4)*epsrrrr(mu1,mu2,mu4,q2)-
     &   q1sq*q4J3*Eij(1,1)*epsrrrr(mu1,mu2,mu4,q3)-
     &   q1q2*q4J3*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q3)-
     &   q1q3*q4J3*Eij(1,3)*epsrrrr(mu1,mu2,mu4,q3)-
     &   q1q4*q4J3*Eij(1,4)*epsrrrr(mu1,mu2,mu4,q3)+
     &   2*q1J3*q1sq*Eij(1,1)*epsrrrr(mu1,mu2,mu4,q4)+
     &   2*q1sq*q2J3*Eij(1,1)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1J3*q1q2*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q4)+
     &   3*q1q2*q2J3*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1sq*q2J3*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q4)-
     &   q1J3*q2sq*Eij(1,2)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1J3*q1q3*Eij(1,3)*epsrrrr(mu1,mu2,mu4,q4)+
     &   2*q1q3*q2J3*Eij(1,3)*epsrrrr(mu1,mu2,mu4,q4)-
     &   q1J3*q2q3*Eij(1,3)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1J3*q1q4*Eij(1,4)*epsrrrr(mu1,mu2,mu4,q4)+
     &   2*q1q4*q2J3*Eij(1,4)*epsrrrr(mu1,mu2,mu4,q4)-
     &   q1J3*q2q4*Eij(1,4)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1q2*q4J3*Eij(1,4)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1sq*q4J3*Eij(1,4)*epsrrrr(mu1,mu2,mu4,q4)+
     &   4*q1J3*q1J4*Eij(1,1)*epsrrrr(mu1,mu2,q1,q2)-
     &   J3J4*q1q2*Eij(1,1)*epsrrrr(mu1,mu2,q1,q2)-
     &   J3J4*q1q3*Eij(1,1)*epsrrrr(mu1,mu2,q1,q2)-
     &   J3J4*q1sq*Eij(1,1)*epsrrrr(mu1,mu2,q1,q2)+
     &   2*q1J4*q2J3*Eij(1,1)*epsrrrr(mu1,mu2,q1,q2)+
     &   2*q1J3*q2J4*Eij(1,1)*epsrrrr(mu1,mu2,q1,q2)+
     &   2*q1J3*q3J4*Eij(1,1)*epsrrrr(mu1,mu2,q1,q2)-
     &   J3J4*q1q2*Eij(1,2)*epsrrrr(mu1,mu2,q1,q2)+
     &   2*q1J4*q2J3*Eij(1,2)*epsrrrr(mu1,mu2,q1,q2)+
     &   2*q1J3*q2J4*Eij(1,2)*epsrrrr(mu1,mu2,q1,q2)+
     &   4*q2J3*q2J4*Eij(1,2)*epsrrrr(mu1,mu2,q1,q2)-
     &   J3J4*q2q3*Eij(1,2)*epsrrrr(mu1,mu2,q1,q2)-
     &   J3J4*q2sq*Eij(1,2)*epsrrrr(mu1,mu2,q1,q2)+
     &   2*q2J3*q3J4*Eij(1,2)*epsrrrr(mu1,mu2,q1,q2)-
     &   J3J4*q1q3*Eij(1,3)*epsrrrr(mu1,mu2,q1,q2)-
     &   J3J4*q2q3*Eij(1,3)*epsrrrr(mu1,mu2,q1,q2)+
     &   2*q1J3*q3J4*Eij(1,3)*epsrrrr(mu1,mu2,q1,q2)+
     &   2*q2J3*q3J4*Eij(1,3)*epsrrrr(mu1,mu2,q1,q2)-
     &   J3J4*q3sq*Eij(1,3)*epsrrrr(mu1,mu2,q1,q2)-
     &   J3J4*q1q4*Eij(1,4)*epsrrrr(mu1,mu2,q1,q2)-
     &   J3J4*q2q4*Eij(1,4)*epsrrrr(mu1,mu2,q1,q2)-
     &   J3J4*q3q4*Eij(1,4)*epsrrrr(mu1,mu2,q1,q2)+
     &   2*q1J4*q4J3*Eij(1,4)*epsrrrr(mu1,mu2,q1,q2)+
     &   2*q2J4*q4J3*Eij(1,4)*epsrrrr(mu1,mu2,q1,q2)+
     &   2*q3J4*q4J3*Eij(1,4)*epsrrrr(mu1,mu2,q1,q2)+
     &   q1J4*q2J3*Eij(1,1)*epsrrrr(mu1,mu2,q1,q3)+
     &   q2J3*q2J4*Eij(1,2)*epsrrrr(mu1,mu2,q1,q3)+
     &   q2J3*q3J4*Eij(1,3)*epsrrrr(mu1,mu2,q1,q3)-
     &   J3J4*q1q2*Eij(1,1)*epsrrrr(mu1,mu2,q1,q4)-
     &   J3J4*q1sq*Eij(1,1)*epsrrrr(mu1,mu2,q1,q4)+
     &   q1J3*q2J4*Eij(1,1)*epsrrrr(mu1,mu2,q1,q4)-
     &   J3J4*q1q2*Eij(1,2)*epsrrrr(mu1,mu2,q1,q4)+
     &   q2J3*q2J4*Eij(1,2)*epsrrrr(mu1,mu2,q1,q4)-
     &   J3J4*q2sq*Eij(1,2)*epsrrrr(mu1,mu2,q1,q4)-
     &   J3J4*q1q3*Eij(1,3)*epsrrrr(mu1,mu2,q1,q4)-
     &   J3J4*q2q3*Eij(1,3)*epsrrrr(mu1,mu2,q1,q4)-
     &   J3J4*q1q4*Eij(1,4)*epsrrrr(mu1,mu2,q1,q4)-
     &   J3J4*q2q4*Eij(1,4)*epsrrrr(mu1,mu2,q1,q4)+
     &   q2J4*q4J3*Eij(1,4)*epsrrrr(mu1,mu2,q1,q4)-
     &   q1J3*q1J4*Eij(1,1)*epsrrrr(mu1,mu2,q2,q3)-
     &   q1J3*q2J4*Eij(1,2)*epsrrrr(mu1,mu2,q2,q3)-
     &   q1J3*q3J4*Eij(1,3)*epsrrrr(mu1,mu2,q2,q3)-
     &   q1J3*q1J4*Eij(1,1)*epsrrrr(mu1,mu2,q2,q4)-
     &   q1J4*q2J3*Eij(1,2)*epsrrrr(mu1,mu2,q2,q4)-
     &   q1J4*q4J3*Eij(1,4)*epsrrrr(mu1,mu2,q2,q4)-
     &   J3J4*q1sq*Eij(1,1)*epsrrrr(mu1,mu2,q3,q4)-
     &   J3J4*q1q2*Eij(1,2)*epsrrrr(mu1,mu2,q3,q4)-
     &   J3J4*q1q3*Eij(1,3)*epsrrrr(mu1,mu2,q3,q4)-
     &   J3J4*q1q4*Eij(1,4)*epsrrrr(mu1,mu2,q3,q4)+
     &   4*q1J2*q1q2*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q1)+
     &   4*q1J2*q1q3*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q1)+
     &   3*q1J2*q1sq*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q1J2*q2q3*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q2sq*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q3q4*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q3sq*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1q2*q4J2*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1sq*q4J2*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q1J2*q1q2*Eij(1,2)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q1J2*q2q3*Eij(1,2)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q1J2*q2sq*Eij(1,2)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1q2*q4J2*Eij(1,2)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q2sq*q4J2*Eij(1,2)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q1J2*q1q3*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q1J2*q2q3*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q1q2*q3J2*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q1q3*q3J2*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1sq*q3J2*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q2q3*q3J2*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q2sq*q3J2*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q3J2*q3q4*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q1J2*q3sq*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q3J2*q3sq*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1q3*q4J2*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q2q3*q4J2*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q1J2*q1q4*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q1J2*q2q4*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q1J2*q3q4*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q1q2*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q1q3*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1q4*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1sq*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   2*q2q3*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q2q4*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q2sq*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q3q4*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q3sq*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q1q2*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q2)+
     &   q1J2*q1q3*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q2)+
     &   q1J2*q1sq*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q2)+
     &   q1J2*q1q2*Eij(1,2)*epsrrrr(mu1,mu3,mu4,q2)+
     &   q1J2*q2q3*Eij(1,2)*epsrrrr(mu1,mu3,mu4,q2)+
     &   q1J2*q2sq*Eij(1,2)*epsrrrr(mu1,mu3,mu4,q2)+
     &   q1J2*q1q3*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q2)+
     &   q1J2*q2q3*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q2)+
     &   q1J2*q3sq*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q2)+
     &   q1J2*q1q4*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q2)+
     &   q1J2*q2q4*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q2)+
     &   q1J2*q3q4*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q2)-
     &   q1J2*q1q4*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q3)+
     &   q1sq*q4J2*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q3)+
     &   q1q2*q4J2*Eij(1,2)*epsrrrr(mu1,mu3,mu4,q3)-
     &   q1q4*q3J2*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q3)+
     &   q1q3*q4J2*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q3)+
     &   q1J2*q1q2*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q4)+
     &   q1J2*q1q3*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q4)+
     &   q1J2*q1sq*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q4)-
     &   q1sq*q3J2*Eij(1,1)*epsrrrr(mu1,mu3,mu4,q4)+
     &   q1J2*q1q2*Eij(1,2)*epsrrrr(mu1,mu3,mu4,q4)+
     &   q1J2*q2sq*Eij(1,2)*epsrrrr(mu1,mu3,mu4,q4)-
     &   q1q2*q3J2*Eij(1,2)*epsrrrr(mu1,mu3,mu4,q4)+
     &   q1J2*q1q3*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q4)+
     &   q1J2*q2q3*Eij(1,3)*epsrrrr(mu1,mu3,mu4,q4)+
     &   q1J2*q1q4*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q4)+
     &   q1J2*q2q4*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q4)-
     &   q1q4*q3J2*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q4)+
     &   q1q3*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,mu4,q4)+
     &   J2J4*q1q2*Eij(1,1)*epsrrrr(mu1,mu3,q1,q2)+
     &   J2J4*q1q3*Eij(1,1)*epsrrrr(mu1,mu3,q1,q2)+
     &   J2J4*q1sq*Eij(1,1)*epsrrrr(mu1,mu3,q1,q2)-
     &   q1J4*q3J2*Eij(1,1)*epsrrrr(mu1,mu3,q1,q2)+
     &   J2J4*q1q2*Eij(1,2)*epsrrrr(mu1,mu3,q1,q2)+
     &   J2J4*q2q3*Eij(1,2)*epsrrrr(mu1,mu3,q1,q2)+
     &   J2J4*q2sq*Eij(1,2)*epsrrrr(mu1,mu3,q1,q2)-
     &   q2J4*q3J2*Eij(1,2)*epsrrrr(mu1,mu3,q1,q2)+
     &   J2J4*q1q3*Eij(1,3)*epsrrrr(mu1,mu3,q1,q2)+
     &   J2J4*q2q3*Eij(1,3)*epsrrrr(mu1,mu3,q1,q2)-
     &   q3J2*q3J4*Eij(1,3)*epsrrrr(mu1,mu3,q1,q2)+
     &   J2J4*q3sq*Eij(1,3)*epsrrrr(mu1,mu3,q1,q2)+
     &   J2J4*q1q4*Eij(1,4)*epsrrrr(mu1,mu3,q1,q2)+
     &   J2J4*q2q4*Eij(1,4)*epsrrrr(mu1,mu3,q1,q2)+
     &   J2J4*q3q4*Eij(1,4)*epsrrrr(mu1,mu3,q1,q2)+
     &   4*q1J2*q1J4*Eij(1,1)*epsrrrr(mu1,mu3,q1,q3)+
     &   2*q1J2*q2J4*Eij(1,1)*epsrrrr(mu1,mu3,q1,q3)+
     &   2*q1J2*q3J4*Eij(1,1)*epsrrrr(mu1,mu3,q1,q3)+
     &   2*q1J2*q2J4*Eij(1,2)*epsrrrr(mu1,mu3,q1,q3)+
     &   2*q1J4*q3J2*Eij(1,3)*epsrrrr(mu1,mu3,q1,q3)+
     &   2*q2J4*q3J2*Eij(1,3)*epsrrrr(mu1,mu3,q1,q3)+
     &   2*q1J2*q3J4*Eij(1,3)*epsrrrr(mu1,mu3,q1,q3)+
     &   2*q3J2*q3J4*Eij(1,3)*epsrrrr(mu1,mu3,q1,q3)+
     &   2*q1J4*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,q1,q3)+
     &   2*q2J4*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,q1,q3)+
     &   2*q3J4*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,q1,q3)+
     &   J2J4*q1q2*Eij(1,1)*epsrrrr(mu1,mu3,q1,q4)+
     &   J2J4*q1sq*Eij(1,1)*epsrrrr(mu1,mu3,q1,q4)+
     &   q1J2*q3J4*Eij(1,1)*epsrrrr(mu1,mu3,q1,q4)+
     &   J2J4*q1q2*Eij(1,2)*epsrrrr(mu1,mu3,q1,q4)+
     &   J2J4*q2sq*Eij(1,2)*epsrrrr(mu1,mu3,q1,q4)+
     &   J2J4*q1q3*Eij(1,3)*epsrrrr(mu1,mu3,q1,q4)+
     &   J2J4*q2q3*Eij(1,3)*epsrrrr(mu1,mu3,q1,q4)+
     &   q3J2*q3J4*Eij(1,3)*epsrrrr(mu1,mu3,q1,q4)+
     &   J2J4*q1q4*Eij(1,4)*epsrrrr(mu1,mu3,q1,q4)+
     &   J2J4*q2q4*Eij(1,4)*epsrrrr(mu1,mu3,q1,q4)+
     &   q3J4*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,q1,q4)+
     &   q1J2*q1J4*Eij(1,1)*epsrrrr(mu1,mu3,q2,q3)+
     &   q1J2*q2J4*Eij(1,2)*epsrrrr(mu1,mu3,q2,q3)+
     &   q1J2*q3J4*Eij(1,3)*epsrrrr(mu1,mu3,q2,q3)-
     &   q1J2*q1J4*Eij(1,1)*epsrrrr(mu1,mu3,q3,q4)+
     &   J2J4*q1sq*Eij(1,1)*epsrrrr(mu1,mu3,q3,q4)+
     &   J2J4*q1q2*Eij(1,2)*epsrrrr(mu1,mu3,q3,q4)+
     &   J2J4*q1q3*Eij(1,3)*epsrrrr(mu1,mu3,q3,q4)-
     &   q1J4*q3J2*Eij(1,3)*epsrrrr(mu1,mu3,q3,q4)+
     &   J2J4*q1q4*Eij(1,4)*epsrrrr(mu1,mu3,q3,q4)-
     &   q1J4*q4J2*Eij(1,4)*epsrrrr(mu1,mu3,q3,q4)-
     &   J2J3*q1q2*Eij(1,1)*epsrrrr(mu1,mu4,q1,q2)-
     &   J2J3*q1q3*Eij(1,1)*epsrrrr(mu1,mu4,q1,q2)-
     &   J2J3*q1sq*Eij(1,1)*epsrrrr(mu1,mu4,q1,q2)-
     &   q1J3*q4J2*Eij(1,1)*epsrrrr(mu1,mu4,q1,q2)-
     &   J2J3*q1q2*Eij(1,2)*epsrrrr(mu1,mu4,q1,q2)-
     &   J2J3*q2q3*Eij(1,2)*epsrrrr(mu1,mu4,q1,q2)-
     &   J2J3*q2sq*Eij(1,2)*epsrrrr(mu1,mu4,q1,q2)-
     &   q2J3*q4J2*Eij(1,2)*epsrrrr(mu1,mu4,q1,q2)-
     &   J2J3*q1q3*Eij(1,3)*epsrrrr(mu1,mu4,q1,q2)-
     &   J2J3*q2q3*Eij(1,3)*epsrrrr(mu1,mu4,q1,q2)-
     &   J2J3*q3sq*Eij(1,3)*epsrrrr(mu1,mu4,q1,q2)-
     &   J2J3*q1q4*Eij(1,4)*epsrrrr(mu1,mu4,q1,q2)-
     &   J2J3*q2q4*Eij(1,4)*epsrrrr(mu1,mu4,q1,q2)-
     &   J2J3*q3q4*Eij(1,4)*epsrrrr(mu1,mu4,q1,q2)-
     &   q4J2*q4J3*Eij(1,4)*epsrrrr(mu1,mu4,q1,q2)-
     &   q1J2*q4J3*Eij(1,1)*epsrrrr(mu1,mu4,q1,q3)-
     &   q3J2*q4J3*Eij(1,3)*epsrrrr(mu1,mu4,q1,q3)-
     &   q4J2*q4J3*Eij(1,4)*epsrrrr(mu1,mu4,q1,q3)+
     &   4*q1J2*q1J3*Eij(1,1)*epsrrrr(mu1,mu4,q1,q4)-
     &   J2J3*q1q2*Eij(1,1)*epsrrrr(mu1,mu4,q1,q4)-
     &   J2J3*q1sq*Eij(1,1)*epsrrrr(mu1,mu4,q1,q4)+
     &   2*q1J2*q2J3*Eij(1,1)*epsrrrr(mu1,mu4,q1,q4)-
     &   J2J3*q1q2*Eij(1,2)*epsrrrr(mu1,mu4,q1,q4)+
     &   2*q1J2*q2J3*Eij(1,2)*epsrrrr(mu1,mu4,q1,q4)-
     &   J2J3*q2sq*Eij(1,2)*epsrrrr(mu1,mu4,q1,q4)-
     &   J2J3*q1q3*Eij(1,3)*epsrrrr(mu1,mu4,q1,q4)-
     &   J2J3*q2q3*Eij(1,3)*epsrrrr(mu1,mu4,q1,q4)+
     &   2*q1J3*q3J2*Eij(1,3)*epsrrrr(mu1,mu4,q1,q4)+
     &   2*q2J3*q3J2*Eij(1,3)*epsrrrr(mu1,mu4,q1,q4)-
     &   J2J3*q1q4*Eij(1,4)*epsrrrr(mu1,mu4,q1,q4)-
     &   J2J3*q2q4*Eij(1,4)*epsrrrr(mu1,mu4,q1,q4)+
     &   2*q1J3*q4J2*Eij(1,4)*epsrrrr(mu1,mu4,q1,q4)+
     &   2*q2J3*q4J2*Eij(1,4)*epsrrrr(mu1,mu4,q1,q4)+
     &   2*q1J2*q4J3*Eij(1,4)*epsrrrr(mu1,mu4,q1,q4)+
     &   q1J2*q1J3*Eij(1,1)*epsrrrr(mu1,mu4,q2,q4)+
     &   q1J2*q2J3*Eij(1,2)*epsrrrr(mu1,mu4,q2,q4)+
     &   q1J2*q4J3*Eij(1,4)*epsrrrr(mu1,mu4,q2,q4)+
     &   q1J2*q1J3*Eij(1,1)*epsrrrr(mu1,mu4,q3,q4)-
     &   J2J3*q1sq*Eij(1,1)*epsrrrr(mu1,mu4,q3,q4)-
     &   J2J3*q1q2*Eij(1,2)*epsrrrr(mu1,mu4,q3,q4)-
     &   J2J3*q1q3*Eij(1,3)*epsrrrr(mu1,mu4,q3,q4)+
     &   q1J3*q3J2*Eij(1,3)*epsrrrr(mu1,mu4,q3,q4)-
     &   J2J3*q1q4*Eij(1,4)*epsrrrr(mu1,mu4,q3,q4)+
     &   q1J3*q4J2*Eij(1,4)*epsrrrr(mu1,mu4,q3,q4)+
     &   J2J3*q1J4*Eij(1,1)*epsrrrr(mu1,q1,q2,q3)+
     &   J2J3*q2J4*Eij(1,2)*epsrrrr(mu1,q1,q2,q3)+
     &   J2J3*q3J4*Eij(1,3)*epsrrrr(mu1,q1,q2,q3)+
     &   J2J4*q1J3*Eij(1,1)*epsrrrr(mu1,q1,q2,q4)+
     &   J2J4*q2J3*Eij(1,2)*epsrrrr(mu1,q1,q2,q4)+
     &   J2J4*q4J3*Eij(1,4)*epsrrrr(mu1,q1,q2,q4)+
     &   J3J4*q1J2*Eij(1,1)*epsrrrr(mu1,q1,q3,q4)+
     &   J3J4*q3J2*Eij(1,3)*epsrrrr(mu1,q1,q3,q4)+
     &   J3J4*q4J2*Eij(1,4)*epsrrrr(mu1,q1,q3,q4)-
     &   q1q2*q2J1*Eij(1,1)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q1q3*q2J1*Eij(1,1)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q1sq*q2J1*Eij(1,1)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q1q2*q4J1*Eij(1,1)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q1sq*q4J1*Eij(1,1)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q1q2*q2J1*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q2J1*q2q3*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q2J1*q2sq*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q1q2*q4J1*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q2sq*q4J1*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q1q3*q2J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q2J1*q2q3*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q2J1*q3sq*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q1q3*q4J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q2q3*q4J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q1q4*q2J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q2J1*q2q4*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q2J1*q3q4*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q1q4*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q1)-
     &   q2q4*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q1)+
     &   2*q1q2*q2J1*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q2)+
     &   2*q1q3*q2J1*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1sq*q2J1*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q2)+
     &   2*q2J1*q2q3*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2J1*q2sq*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2J1*q3q4*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2J1*q3sq*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q2)+
     &   2*q1q2*q3J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q2)+
     &   2*q1q3*q3J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1sq*q3J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q2)+
     &   2*q2q3*q3J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2sq*q3J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q3J1*q3q4*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q3J1*q3sq*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q2)+
     &   2*q1q2*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q2)+
     &   2*q1q3*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1sq*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q2)+
     &   2*q2q3*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2sq*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q3q4*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q3sq*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q2)-
     &   q1sq*q4J1*Eij(1,1)*epsrrrr(mu2,mu3,mu4,q3)-
     &   q2J1*q2q4*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q3)-
     &   q1q2*q4J1*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q3)-
     &   q2q4*q3J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q3)-
     &   q1q3*q4J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q3)-
     &   q1q4*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q3)-
     &   q2q4*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q3)+
     &   q1sq*q3J1*Eij(1,1)*epsrrrr(mu2,mu3,mu4,q4)+
     &   2*q1q2*q2J1*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1sq*q2J1*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2J1*q2q3*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2J1*q2sq*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1q2*q3J1*Eij(1,2)*epsrrrr(mu2,mu3,mu4,q4)+
     &   2*q1q2*q3J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1q3*q3J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1sq*q3J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2q3*q3J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2sq*q3J1*Eij(1,3)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1q4*q3J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q4)+
     &   2*q1q2*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1sq*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2q3*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2sq*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,mu4,q4)-
     &   J1J4*q1q2*Eij(1,1)*epsrrrr(mu2,mu3,q1,q2)-
     &   J1J4*q1q3*Eij(1,1)*epsrrrr(mu2,mu3,q1,q2)-
     &   J1J4*q1sq*Eij(1,1)*epsrrrr(mu2,mu3,q1,q2)+
     &   q1J4*q3J1*Eij(1,1)*epsrrrr(mu2,mu3,q1,q2)-
     &   J1J4*q1q2*Eij(1,2)*epsrrrr(mu2,mu3,q1,q2)-
     &   J1J4*q2q3*Eij(1,2)*epsrrrr(mu2,mu3,q1,q2)-
     &   J1J4*q2sq*Eij(1,2)*epsrrrr(mu2,mu3,q1,q2)+
     &   q2J4*q3J1*Eij(1,2)*epsrrrr(mu2,mu3,q1,q2)-
     &   J1J4*q1q3*Eij(1,3)*epsrrrr(mu2,mu3,q1,q2)-
     &   J1J4*q2q3*Eij(1,3)*epsrrrr(mu2,mu3,q1,q2)+
     &   q3J1*q3J4*Eij(1,3)*epsrrrr(mu2,mu3,q1,q2)-
     &   J1J4*q3sq*Eij(1,3)*epsrrrr(mu2,mu3,q1,q2)-
     &   J1J4*q1q4*Eij(1,4)*epsrrrr(mu2,mu3,q1,q2)-
     &   J1J4*q2q4*Eij(1,4)*epsrrrr(mu2,mu3,q1,q2)-
     &   J1J4*q3q4*Eij(1,4)*epsrrrr(mu2,mu3,q1,q2)-
     &   q1J4*q2J1*Eij(1,1)*epsrrrr(mu2,mu3,q1,q3)-
     &   q2J1*q2J4*Eij(1,2)*epsrrrr(mu2,mu3,q1,q3)-
     &   q2J1*q3J4*Eij(1,3)*epsrrrr(mu2,mu3,q1,q3)-
     &   J1J4*q1q2*Eij(1,1)*epsrrrr(mu2,mu3,q1,q4)-
     &   J1J4*q1sq*Eij(1,1)*epsrrrr(mu2,mu3,q1,q4)-
     &   J1J4*q1q2*Eij(1,2)*epsrrrr(mu2,mu3,q1,q4)-
     &   J1J4*q2sq*Eij(1,2)*epsrrrr(mu2,mu3,q1,q4)-
     &   J1J4*q1q3*Eij(1,3)*epsrrrr(mu2,mu3,q1,q4)-
     &   J1J4*q2q3*Eij(1,3)*epsrrrr(mu2,mu3,q1,q4)-
     &   J1J4*q1q4*Eij(1,4)*epsrrrr(mu2,mu3,q1,q4)-
     &   J1J4*q2q4*Eij(1,4)*epsrrrr(mu2,mu3,q1,q4)+
     &   2*q1J4*q2J1*Eij(1,2)*epsrrrr(mu2,mu3,q2,q3)+
     &   2*q2J1*q2J4*Eij(1,2)*epsrrrr(mu2,mu3,q2,q3)+
     &   2*q2J1*q3J4*Eij(1,2)*epsrrrr(mu2,mu3,q2,q3)+
     &   2*q1J4*q3J1*Eij(1,3)*epsrrrr(mu2,mu3,q2,q3)+
     &   2*q2J4*q3J1*Eij(1,3)*epsrrrr(mu2,mu3,q2,q3)+
     &   2*q3J1*q3J4*Eij(1,3)*epsrrrr(mu2,mu3,q2,q3)+
     &   2*q1J4*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,q2,q3)+
     &   2*q2J4*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,q2,q3)+
     &   2*q3J4*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,q2,q3)+
     &   q2J1*q3J4*Eij(1,2)*epsrrrr(mu2,mu3,q2,q4)+
     &   q3J1*q3J4*Eij(1,3)*epsrrrr(mu2,mu3,q2,q4)+
     &   q3J4*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,q2,q4)-
     &   J1J4*q1sq*Eij(1,1)*epsrrrr(mu2,mu3,q3,q4)-
     &   J1J4*q1q2*Eij(1,2)*epsrrrr(mu2,mu3,q3,q4)-
     &   q2J1*q2J4*Eij(1,2)*epsrrrr(mu2,mu3,q3,q4)-
     &   J1J4*q1q3*Eij(1,3)*epsrrrr(mu2,mu3,q3,q4)-
     &   q2J4*q3J1*Eij(1,3)*epsrrrr(mu2,mu3,q3,q4)-
     &   J1J4*q1q4*Eij(1,4)*epsrrrr(mu2,mu3,q3,q4)-
     &   q2J4*q4J1*Eij(1,4)*epsrrrr(mu2,mu3,q3,q4)+
     &   J1J3*q1q2*Eij(1,1)*epsrrrr(mu2,mu4,q1,q2)+
     &   J1J3*q1q3*Eij(1,1)*epsrrrr(mu2,mu4,q1,q2)+
     &   J1J3*q1sq*Eij(1,1)*epsrrrr(mu2,mu4,q1,q2)+
     &   q1J3*q4J1*Eij(1,1)*epsrrrr(mu2,mu4,q1,q2)+
     &   J1J3*q1q2*Eij(1,2)*epsrrrr(mu2,mu4,q1,q2)+
     &   J1J3*q2q3*Eij(1,2)*epsrrrr(mu2,mu4,q1,q2)+
     &   J1J3*q2sq*Eij(1,2)*epsrrrr(mu2,mu4,q1,q2)+
     &   q2J3*q4J1*Eij(1,2)*epsrrrr(mu2,mu4,q1,q2)+
     &   J1J3*q1q3*Eij(1,3)*epsrrrr(mu2,mu4,q1,q2)+
     &   J1J3*q2q3*Eij(1,3)*epsrrrr(mu2,mu4,q1,q2)+
     &   J1J3*q3sq*Eij(1,3)*epsrrrr(mu2,mu4,q1,q2)+
     &   J1J3*q1q4*Eij(1,4)*epsrrrr(mu2,mu4,q1,q2)+
     &   J1J3*q2q4*Eij(1,4)*epsrrrr(mu2,mu4,q1,q2)+
     &   J1J3*q3q4*Eij(1,4)*epsrrrr(mu2,mu4,q1,q2)+
     &   q4J1*q4J3*Eij(1,4)*epsrrrr(mu2,mu4,q1,q2)+
     &   J1J3*q1q2*Eij(1,1)*epsrrrr(mu2,mu4,q1,q4)+
     &   J1J3*q1sq*Eij(1,1)*epsrrrr(mu2,mu4,q1,q4)-
     &   q1J3*q2J1*Eij(1,1)*epsrrrr(mu2,mu4,q1,q4)+
     &   J1J3*q1q2*Eij(1,2)*epsrrrr(mu2,mu4,q1,q4)-
     &   q2J1*q2J3*Eij(1,2)*epsrrrr(mu2,mu4,q1,q4)+
     &   J1J3*q2sq*Eij(1,2)*epsrrrr(mu2,mu4,q1,q4)+
     &   J1J3*q1q3*Eij(1,3)*epsrrrr(mu2,mu4,q1,q4)+
     &   J1J3*q2q3*Eij(1,3)*epsrrrr(mu2,mu4,q1,q4)+
     &   J1J3*q1q4*Eij(1,4)*epsrrrr(mu2,mu4,q1,q4)+
     &   J1J3*q2q4*Eij(1,4)*epsrrrr(mu2,mu4,q1,q4)-
     &   q2J1*q4J3*Eij(1,4)*epsrrrr(mu2,mu4,q1,q4)-
     &   q2J1*q4J3*Eij(1,2)*epsrrrr(mu2,mu4,q2,q3)-
     &   q3J1*q4J3*Eij(1,3)*epsrrrr(mu2,mu4,q2,q3)-
     &   q4J1*q4J3*Eij(1,4)*epsrrrr(mu2,mu4,q2,q3)+
     &   2*q1J3*q2J1*Eij(1,2)*epsrrrr(mu2,mu4,q2,q4)+
     &   2*q2J1*q2J3*Eij(1,2)*epsrrrr(mu2,mu4,q2,q4)+
     &   2*q1J3*q3J1*Eij(1,3)*epsrrrr(mu2,mu4,q2,q4)+
     &   2*q2J3*q3J1*Eij(1,3)*epsrrrr(mu2,mu4,q2,q4)+
     &   2*q1J3*q4J1*Eij(1,4)*epsrrrr(mu2,mu4,q2,q4)+
     &   2*q2J3*q4J1*Eij(1,4)*epsrrrr(mu2,mu4,q2,q4)+
     &   J1J3*q1sq*Eij(1,1)*epsrrrr(mu2,mu4,q3,q4)+
     &   J1J3*q1q2*Eij(1,2)*epsrrrr(mu2,mu4,q3,q4)+
     &   q2J1*q2J3*Eij(1,2)*epsrrrr(mu2,mu4,q3,q4)+
     &   J1J3*q1q3*Eij(1,3)*epsrrrr(mu2,mu4,q3,q4)+
     &   q2J3*q3J1*Eij(1,3)*epsrrrr(mu2,mu4,q3,q4)+
     &   J1J3*q1q4*Eij(1,4)*epsrrrr(mu2,mu4,q3,q4)+
     &   q2J3*q4J1*Eij(1,4)*epsrrrr(mu2,mu4,q3,q4)-
     &   J1J3*q1J4*Eij(1,1)*epsrrrr(mu2,q1,q2,q3)-
     &   J1J3*q2J4*Eij(1,2)*epsrrrr(mu2,q1,q2,q3)-
     &   J1J3*q3J4*Eij(1,3)*epsrrrr(mu2,q1,q2,q3)-
     &   J1J4*q1J3*Eij(1,1)*epsrrrr(mu2,q1,q2,q4)-
     &   J1J4*q2J3*Eij(1,2)*epsrrrr(mu2,q1,q2,q4)-
     &   J1J4*q4J3*Eij(1,4)*epsrrrr(mu2,q1,q2,q4)+
     &   J3J4*q2J1*Eij(1,2)*epsrrrr(mu2,q2,q3,q4)+
     &   J3J4*q3J1*Eij(1,3)*epsrrrr(mu2,q2,q3,q4)+
     &   J3J4*q4J1*Eij(1,4)*epsrrrr(mu2,q2,q3,q4)-
     &   J1J2*q1q2*Eij(1,1)*epsrrrr(mu3,mu4,q1,q2)-
     &   J1J2*q1q3*Eij(1,1)*epsrrrr(mu3,mu4,q1,q2)-
     &   J1J2*q1sq*Eij(1,1)*epsrrrr(mu3,mu4,q1,q2)-
     &   J1J2*q1q2*Eij(1,2)*epsrrrr(mu3,mu4,q1,q2)-
     &   J1J2*q2q3*Eij(1,2)*epsrrrr(mu3,mu4,q1,q2)-
     &   J1J2*q2sq*Eij(1,2)*epsrrrr(mu3,mu4,q1,q2)-
     &   J1J2*q1q3*Eij(1,3)*epsrrrr(mu3,mu4,q1,q2)-
     &   J1J2*q2q3*Eij(1,3)*epsrrrr(mu3,mu4,q1,q2)-
     &   J1J2*q3sq*Eij(1,3)*epsrrrr(mu3,mu4,q1,q2)-
     &   J1J2*q1q4*Eij(1,4)*epsrrrr(mu3,mu4,q1,q2)-
     &   J1J2*q2q4*Eij(1,4)*epsrrrr(mu3,mu4,q1,q2)-
     &   J1J2*q3q4*Eij(1,4)*epsrrrr(mu3,mu4,q1,q2)+
     &   q1J2*q4J1*Eij(1,1)*epsrrrr(mu3,mu4,q1,q3)+
     &   q3J2*q4J1*Eij(1,3)*epsrrrr(mu3,mu4,q1,q3)+
     &   q4J1*q4J2*Eij(1,4)*epsrrrr(mu3,mu4,q1,q3)-
     &   J1J2*q1q2*Eij(1,1)*epsrrrr(mu3,mu4,q1,q4)-
     &   J1J2*q1sq*Eij(1,1)*epsrrrr(mu3,mu4,q1,q4)-
     &   q1J2*q3J1*Eij(1,1)*epsrrrr(mu3,mu4,q1,q4)-
     &   J1J2*q1q2*Eij(1,2)*epsrrrr(mu3,mu4,q1,q4)-
     &   J1J2*q2sq*Eij(1,2)*epsrrrr(mu3,mu4,q1,q4)-
     &   J1J2*q1q3*Eij(1,3)*epsrrrr(mu3,mu4,q1,q4)-
     &   J1J2*q2q3*Eij(1,3)*epsrrrr(mu3,mu4,q1,q4)-
     &   q3J1*q3J2*Eij(1,3)*epsrrrr(mu3,mu4,q1,q4)-
     &   J1J2*q1q4*Eij(1,4)*epsrrrr(mu3,mu4,q1,q4)-
     &   J1J2*q2q4*Eij(1,4)*epsrrrr(mu3,mu4,q1,q4)-
     &   q3J1*q4J2*Eij(1,4)*epsrrrr(mu3,mu4,q1,q4)+
     &   q2J1*q4J2*Eij(1,2)*epsrrrr(mu3,mu4,q2,q3)+
     &   q3J1*q4J2*Eij(1,3)*epsrrrr(mu3,mu4,q2,q3)+
     &   q4J1*q4J2*Eij(1,4)*epsrrrr(mu3,mu4,q2,q3)-
     &   q2J1*q3J2*Eij(1,2)*epsrrrr(mu3,mu4,q2,q4)-
     &   q3J1*q3J2*Eij(1,3)*epsrrrr(mu3,mu4,q2,q4)-
     &   q3J2*q4J1*Eij(1,4)*epsrrrr(mu3,mu4,q2,q4)-
     &   J1J2*q1sq*Eij(1,1)*epsrrrr(mu3,mu4,q3,q4)-
     &   J1J2*q1q2*Eij(1,2)*epsrrrr(mu3,mu4,q3,q4)+
     &   2*q1J2*q2J1*Eij(1,2)*epsrrrr(mu3,mu4,q3,q4)-
     &   J1J2*q1q3*Eij(1,3)*epsrrrr(mu3,mu4,q3,q4)+
     &   2*q1J2*q3J1*Eij(1,3)*epsrrrr(mu3,mu4,q3,q4)-
     &   J1J2*q1q4*Eij(1,4)*epsrrrr(mu3,mu4,q3,q4)+
     &   2*q1J2*q4J1*Eij(1,4)*epsrrrr(mu3,mu4,q3,q4)+
     &   J1J2*q1J4*Eij(1,1)*epsrrrr(mu3,q1,q2,q3)+
     &   J1J2*q2J4*Eij(1,2)*epsrrrr(mu3,q1,q2,q3)+
     &   J1J2*q3J4*Eij(1,3)*epsrrrr(mu3,q1,q2,q3)-
     &   J1J4*(q1J2*Eij(1,1)+q3J2*Eij(1,3)+q4J2*Eij(1,4))*
     &    epsrrrr(mu3,q1,q3,q4)-J2J4*q2J1*Eij(1,2)*epsrrrr(mu3,q2,q3,q4)-
     &   J2J4*q3J1*Eij(1,3)*epsrrrr(mu3,q2,q3,q4)-
     &   J2J4*q4J1*Eij(1,4)*epsrrrr(mu3,q2,q3,q4)+
     &   J1J2*q1J3*Eij(1,1)*epsrrrr(mu4,q1,q2,q4)+
     &   J1J2*q2J3*Eij(1,2)*epsrrrr(mu4,q1,q2,q4)+
     &   J1J2*q4J3*Eij(1,4)*epsrrrr(mu4,q1,q2,q4)+
     &   J1J3*(q1J2*Eij(1,1)+q3J2*Eij(1,3)+q4J2*Eij(1,4))*
     &    epsrrrr(mu4,q1,q3,q4)+J2J3*q2J1*Eij(1,2)*epsrrrr(mu4,q2,q3,q4)+
     &   J2J3*q3J1*Eij(1,3)*epsrrrr(mu4,q2,q3,q4)+
     &   J2J3*q4J1*Eij(1,4)*epsrrrr(mu4,q2,q3,q4))

      coeffEmunu=16*m*(-(q1q2*q1sq*Eij(2,1)*epsrrrr(mu1,mu2,mu3,mu4))-
     &   q1q3*q1sq*Eij(2,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq**2*Eij(2,1)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2**2*Eij(2,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q2q3*Eij(2,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q2sq*Eij(2,2)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q3**2*Eij(2,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q3*q2q3*Eij(2,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q3*q3sq*Eij(2,3)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4**2*Eij(2,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4*q2q4*Eij(2,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4*q3q4*Eij(2,4)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2**2*Eij(2,5)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q1q3*Eij(2,5)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q2*q1sq*Eij(2,5)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q2q3*Eij(2,5)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q2sq*Eij(2,5)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q1q3*Eij(2,6)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q3**2*Eij(2,6)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q3*q1sq*Eij(2,6)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q2q3*Eij(2,6)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q3sq*Eij(2,6)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q1q4*Eij(2,7)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q3*q1q4*Eij(2,7)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q4*q1sq*Eij(2,7)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q2q4*Eij(2,7)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*q3q4*Eij(2,7)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q2*q1q3*Eij(2,8)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q2q3*Eij(2,8)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q3*q2q3*Eij(2,8)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q3*q2sq*Eij(2,8)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q3sq*Eij(2,8)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q2*q1q4*Eij(2,9)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4*q2q3*Eij(2,9)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q2q4*Eij(2,9)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4*q2sq*Eij(2,9)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*q3q4*Eij(2,9)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   2*q1q3*q1q4*Eij(2,10)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4*q2q3*Eij(2,10)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q3*q2q4*Eij(2,10)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q3*q3q4*Eij(2,10)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q4*q3sq*Eij(2,10)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q2*Eij(2,11)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1q3*Eij(2,11)*epsrrrr(mu1,mu2,mu3,mu4)-
     &   q1sq*Eij(2,11)*epsrrrr(mu1,mu2,mu3,mu4)+
     &   q1J4*q1q2*Eij(2,1)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1J4*q1sq*Eij(2,1)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1q2*q2J4*Eij(2,2)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q2J4*q2sq*Eij(2,2)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1q3*q3J4*Eij(2,3)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q2q3*q3J4*Eij(2,3)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1J4*q1q2*Eij(2,5)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1q2*q2J4*Eij(2,5)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1sq*q2J4*Eij(2,5)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1J4*q2sq*Eij(2,5)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1J4*q1q3*Eij(2,6)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1J4*q2q3*Eij(2,6)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1q2*q3J4*Eij(2,6)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1sq*q3J4*Eij(2,6)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1J4*q1q4*Eij(2,7)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1J4*q2q4*Eij(2,7)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1q3*q2J4*Eij(2,8)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q2J4*q2q3*Eij(2,8)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1q2*q3J4*Eij(2,8)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q2sq*q3J4*Eij(2,8)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1q4*q2J4*Eij(2,9)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q2J4*q2q4*Eij(2,9)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1q4*q3J4*Eij(2,10)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q2q4*q3J4*Eij(2,10)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1J4*Eij(2,11)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q2J4*Eij(2,11)*epsrrrr(mu1,mu2,mu3,q1)+
     &   q1J4*q1sq*Eij(2,1)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1q2*q2J4*Eij(2,2)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1q3*q3J4*Eij(2,3)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1J4*q1q2*Eij(2,5)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1sq*q2J4*Eij(2,5)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1J4*q1q3*Eij(2,6)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1sq*q3J4*Eij(2,6)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1J4*q1q4*Eij(2,7)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1q3*q2J4*Eij(2,8)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1q2*q3J4*Eij(2,8)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1q4*q2J4*Eij(2,9)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1q4*q3J4*Eij(2,10)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1J4*Eij(2,11)*epsrrrr(mu1,mu2,mu3,q3)+
     &   q1J3*q1sq*Eij(2,1)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1q2*q2J3*Eij(2,2)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1q4*q4J3*Eij(2,4)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1J3*q1q2*Eij(2,5)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1sq*q2J3*Eij(2,5)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1J3*q1q3*Eij(2,6)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1J3*q1q4*Eij(2,7)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1sq*q4J3*Eij(2,7)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1q3*q2J3*Eij(2,8)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1q4*q2J3*Eij(2,9)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1q2*q4J3*Eij(2,9)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1q3*q4J3*Eij(2,10)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1J3*Eij(2,11)*epsrrrr(mu1,mu2,mu4,q4)+
     &   q1J3*q1J4*Eij(2,1)*epsrrrr(mu1,mu2,q1,q2)+
     &   q2J3*q2J4*Eij(2,2)*epsrrrr(mu1,mu2,q1,q2)+
     &   q1J4*q2J3*Eij(2,5)*epsrrrr(mu1,mu2,q1,q2)+
     &   q1J3*q2J4*Eij(2,5)*epsrrrr(mu1,mu2,q1,q2)+
     &   q1J3*q3J4*Eij(2,6)*epsrrrr(mu1,mu2,q1,q2)+
     &   q1J4*q4J3*Eij(2,7)*epsrrrr(mu1,mu2,q1,q2)+
     &   q2J3*q3J4*Eij(2,8)*epsrrrr(mu1,mu2,q1,q2)+
     &   q2J4*q4J3*Eij(2,9)*epsrrrr(mu1,mu2,q1,q2)+
     &   q3J4*q4J3*Eij(2,10)*epsrrrr(mu1,mu2,q1,q2)+
     &   J3J4*Eij(2,11)*epsrrrr(mu1,mu2,q1,q2)+
     &   q1J2*q1q2*Eij(2,1)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q1q3*Eij(2,1)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q1sq*Eij(2,1)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1q3*q3J2*Eij(2,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q2q3*q3J2*Eij(2,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q3J2*q3sq*Eij(2,3)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1q4*q4J2*Eij(2,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q2q4*q4J2*Eij(2,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q3q4*q4J2*Eij(2,4)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q1q2*Eij(2,5)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q2q3*Eij(2,5)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q2sq*Eij(2,5)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q1q3*Eij(2,6)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q2q3*Eij(2,6)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1q2*q3J2*Eij(2,6)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1q3*q3J2*Eij(2,6)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1sq*q3J2*Eij(2,6)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q3sq*Eij(2,6)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q1q4*Eij(2,7)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q2q4*Eij(2,7)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*q3q4*Eij(2,7)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1q2*q4J2*Eij(2,7)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1q3*q4J2*Eij(2,7)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1sq*q4J2*Eij(2,7)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1q2*q3J2*Eij(2,8)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q2q3*q3J2*Eij(2,8)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q2sq*q3J2*Eij(2,8)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1q2*q4J2*Eij(2,9)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q2q3*q4J2*Eij(2,9)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q2sq*q4J2*Eij(2,9)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1q4*q3J2*Eij(2,10)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q2q4*q3J2*Eij(2,10)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q3J2*q3q4*Eij(2,10)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1q3*q4J2*Eij(2,10)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q2q3*q4J2*Eij(2,10)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q3sq*q4J2*Eij(2,10)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q1J2*Eij(2,11)*epsrrrr(mu1,mu3,mu4,q1)+
     &   q3J2*Eij(2,11)*epsrrrr(mu1,mu3,mu4,q1)+
     &   (q3J2*q3J4*Eij(2,3)+q1J4*q3J2*Eij(2,6)+
     &      q1J2*(q1J4*Eij(2,1)+q2J4*Eij(2,5)+q3J4*Eij(2,6))+
     &      q2J4*q3J2*Eij(2,8)+q4J2*
     &       (q1J4*Eij(2,7)+q2J4*Eij(2,9)+q3J4*Eij(2,10))+J2J4*Eij(2,11))*
     &    epsrrrr(mu1,mu3,q1,q3)+
     &   (q4J2*q4J3*Eij(2,4)+q1J3*q4J2*Eij(2,7)+
     &      q1J2*(q1J3*Eij(2,1)+q2J3*Eij(2,5)+q4J3*Eij(2,7))+
     &      q2J3*q4J2*Eij(2,9)+q3J2*
     &       (q1J3*Eij(2,6)+q2J3*Eij(2,8)+q4J3*Eij(2,10))+J2J3*Eij(2,11))*
     &    epsrrrr(mu1,mu4,q1,q4)+q1q2*q2J1*Eij(2,2)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2J1*q2q3*Eij(2,2)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2J1*q2sq*Eij(2,2)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q3*q3J1*Eij(2,3)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2q3*q3J1*Eij(2,3)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q3J1*q3sq*Eij(2,3)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q4*q4J1*Eij(2,4)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2q4*q4J1*Eij(2,4)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q3q4*q4J1*Eij(2,4)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q2*q2J1*Eij(2,5)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q3*q2J1*Eij(2,5)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1sq*q2J1*Eij(2,5)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q2*q3J1*Eij(2,6)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q3*q3J1*Eij(2,6)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1sq*q3J1*Eij(2,6)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q2*q4J1*Eij(2,7)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q3*q4J1*Eij(2,7)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1sq*q4J1*Eij(2,7)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q3*q2J1*Eij(2,8)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2J1*q2q3*Eij(2,8)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q2*q3J1*Eij(2,8)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2q3*q3J1*Eij(2,8)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2sq*q3J1*Eij(2,8)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2J1*q3sq*Eij(2,8)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q4*q2J1*Eij(2,9)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2J1*q2q4*Eij(2,9)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2J1*q3q4*Eij(2,9)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q2*q4J1*Eij(2,9)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2q3*q4J1*Eij(2,9)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2sq*q4J1*Eij(2,9)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q4*q3J1*Eij(2,10)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2q4*q3J1*Eij(2,10)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q3J1*q3q4*Eij(2,10)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q3*q4J1*Eij(2,10)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2q3*q4J1*Eij(2,10)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q3sq*q4J1*Eij(2,10)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q2J1*Eij(2,11)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q3J1*Eij(2,11)*epsrrrr(mu2,mu3,mu4,q2)+
     &   q1q2*q2J1*Eij(2,2)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2J1*q2sq*Eij(2,2)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1q3*q3J1*Eij(2,3)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2q3*q3J1*Eij(2,3)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1q4*q4J1*Eij(2,4)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2q4*q4J1*Eij(2,4)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1q2*q2J1*Eij(2,5)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1sq*q2J1*Eij(2,5)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1q2*q3J1*Eij(2,6)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1sq*q3J1*Eij(2,6)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1q2*q4J1*Eij(2,7)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1sq*q4J1*Eij(2,7)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1q3*q2J1*Eij(2,8)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2J1*q2q3*Eij(2,8)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1q2*q3J1*Eij(2,8)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2sq*q3J1*Eij(2,8)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1q4*q2J1*Eij(2,9)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2J1*q2q4*Eij(2,9)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1q2*q4J1*Eij(2,9)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2sq*q4J1*Eij(2,9)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1q4*q3J1*Eij(2,10)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2q4*q3J1*Eij(2,10)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q1q3*q4J1*Eij(2,10)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2q3*q4J1*Eij(2,10)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2J1*Eij(2,11)*epsrrrr(mu2,mu3,mu4,q4)+
     &   q2J1*q2J4*Eij(2,2)*epsrrrr(mu2,mu3,q2,q3)+
     &   q3J1*q3J4*Eij(2,3)*epsrrrr(mu2,mu3,q2,q3)+
     &   q1J4*q2J1*Eij(2,5)*epsrrrr(mu2,mu3,q2,q3)+
     &   q1J4*q3J1*Eij(2,6)*epsrrrr(mu2,mu3,q2,q3)+
     &   q1J4*q4J1*Eij(2,7)*epsrrrr(mu2,mu3,q2,q3)+
     &   q2J4*q3J1*Eij(2,8)*epsrrrr(mu2,mu3,q2,q3)+
     &   q2J1*q3J4*Eij(2,8)*epsrrrr(mu2,mu3,q2,q3)+
     &   q2J4*q4J1*Eij(2,9)*epsrrrr(mu2,mu3,q2,q3)+
     &   q3J4*q4J1*Eij(2,10)*epsrrrr(mu2,mu3,q2,q3)+
     &   J1J4*Eij(2,11)*epsrrrr(mu2,mu3,q2,q3)+
     &   q2J1*q2J3*Eij(2,2)*epsrrrr(mu2,mu4,q2,q4)+
     &   q4J1*q4J3*Eij(2,4)*epsrrrr(mu2,mu4,q2,q4)+
     &   q1J3*q2J1*Eij(2,5)*epsrrrr(mu2,mu4,q2,q4)+
     &   q1J3*q3J1*Eij(2,6)*epsrrrr(mu2,mu4,q2,q4)+
     &   q1J3*q4J1*Eij(2,7)*epsrrrr(mu2,mu4,q2,q4)+
     &   q2J3*q3J1*Eij(2,8)*epsrrrr(mu2,mu4,q2,q4)+
     &   q2J3*q4J1*Eij(2,9)*epsrrrr(mu2,mu4,q2,q4)+
     &   q2J1*q4J3*Eij(2,9)*epsrrrr(mu2,mu4,q2,q4)+
     &   q3J1*q4J3*Eij(2,10)*epsrrrr(mu2,mu4,q2,q4)+
     &   J1J3*Eij(2,11)*epsrrrr(mu2,mu4,q2,q4)+
     &   q3J1*q3J2*Eij(2,3)*epsrrrr(mu3,mu4,q3,q4)+
     &   q4J1*q4J2*Eij(2,4)*epsrrrr(mu3,mu4,q3,q4)+
     &   q1J2*q2J1*Eij(2,5)*epsrrrr(mu3,mu4,q3,q4)+
     &   q1J2*q3J1*Eij(2,6)*epsrrrr(mu3,mu4,q3,q4)+
     &   q1J2*q4J1*Eij(2,7)*epsrrrr(mu3,mu4,q3,q4)+
     &   q2J1*q3J2*Eij(2,8)*epsrrrr(mu3,mu4,q3,q4)+
     &   q2J1*q4J2*Eij(2,9)*epsrrrr(mu3,mu4,q3,q4)+
     &   q3J2*q4J1*Eij(2,10)*epsrrrr(mu3,mu4,q3,q4)+
     &   q3J1*q4J2*Eij(2,10)*epsrrrr(mu3,mu4,q3,q4)+
     &   J1J2*Eij(2,11)*epsrrrr(mu3,mu4,q3,q4))

      A4G_LoopDD=(coeffD0+coeffDmu+coeffE0C0+coeffE0+coeffEmu+coeffEmunu)/(4d0*m)

      RETURN
      END

***********************************************************************
      FUNCTION H4G_LoopDD(i1,i2,i3,i4,m)
*----------------------------------------------------------------------
c author: Michael Kubocz
c date    : 03.08.08
*----------------------------------------------------------------------
c   This pentagon subroutine uses the Denner-Dittmaier tensor decomposition.      
c   i1,i2,i3,i4 gives the permutation of the 4 gluons
c   input gluon momenta and polarization vectors 
c   are stored via the needed dot products in q_dot_ep,
c   ep_dot_ep and q_dot_q and explicitly in qi and eps

c   Normalization: this subroutine gives the pentagon contributions
c   without a factor 4*i*mt the QCD couplings and Yukawa coupling h_t.
c   Thus, for the full amplitude multiply by an overall
c   normalization factor of 4 mt h_t alpha_s**2 c_i .

c                     * -- i1    
c                   /   \
c           i2 -- *      \
c                 |       * -- H^0
c           i3 -- *      /
c                   \   /
c                     * -- i4

c   When summing the two Feynman graphs with opposite fermion flow
c   below. The color factors c_i is defined in the CP even NP paper
c   eq.3.43
***********************************************************************

      IMPLICIT NONE

#include "ggf_process.inc"
#include "d_coefficients.inc"
#include "ggf_dotproducts.inc"

      complex*16 H4G_LoopDD

c   four vectors
      real*8 eps,qi,m
      COMMON /bh4g_mom_eps/ qi(0:3,4),eps(0:3,4)

      real*8 ALFAS,XMT,ALFA,XMZ,XMW,SIN2W,XMH
      COMMON /BKOPIN/ ALFAS,XMT,ALFA,XMZ,XMW,SIN2W,XMH

c   explicit momenta for call of eps tensor
      real*8  q1(0:3),q2(0:3),q3(0:3),q4(0:3),
     &        mu1(0:3),mu2(0:3),mu3(0:3),mu4(0:3)

c   Tensor oefficients
      complex*16 C0_1M,C034,
     &           D0_1M,D0234,Dij(3,13),  
     &           E0L,Eij(4,46)

      complex*16 HEmunurhosigma,HEmunurho,HEmunu,HEmu,HE0,HDmunu,HDmu,HD0

c   local dot products 
      real*8 q1sq,q2sq,q3sq,q4sq,q12sq,q34sq,q1234sq,
     &       q1q2,q1q3,q1q4,q2q3,q2q4,q3q4,
     &       q1J1,q2J1,q3J1,q4J1,q1J2,q2J2,q3J2,q4J2,
     &       q1J3,q2J3,q3J3,q4J3,q1J4,q2J4,q3J4,q4J4,
     &       J1J2,J1J3,J1J4,J2J3,J2J4,J3J4
      
c   other variables
      integer i1,i2,i3,i4
      integer i,j,mu
      integer id(4)

      integer init,ie0(4,4,4,4)
      data init/0/, ie0/256*0/
      save ie0,init

      real*8 dotrr

      EXTERNAL dotrr,C0_1M,D0_1M

      if (init.eq.0) then   
         init = 1
         ie0(4,2,1,3) = 1   ! this is the map from i1,i2,i3,i4 
         ie0(2,1,3,4) = 2   ! to the index of the E0 variables
         ie0(3,2,1,4) = 3
         ie0(2,1,4,3) = 4
         ie0(4,3,2,1) = 5
         ie0(3,4,2,1) = 6
         ie0(2,3,1,4) = 7
         ie0(3,1,4,2) = 8
         ie0(4,2,3,1) = 9
         ie0(2,4,3,1) = 10
         ie0(3,2,4,1) = 11
         ie0(2,3,4,1) = 12
      endif

      if (m.eq.m_top) then
         E0L=-E0t(ie0(i1,i2,i3,i4))
         do i=1,4
           do j=1,46 
               Eij(i,j)=DDEijtop(i,j,ie0(i1,i2,i3,i4))
            enddo
         enddo  
      endif
      if (m.eq.m_bottom) then
         E0L =-E0b(ie0(i1,i2,i3,i4))
         do i=1,4
            do j=1,46 
               Eij(i,j)=DDEijbottom(i,j,ie0(i1,i2,i3,i4))
            enddo
         enddo   
      endif

      q1q2 = q_dot_q(i1,i2)
      q1q3 = q_dot_q(i1,i3)
      q1q4 = q_dot_q(i1,i4)
      q2q3 = q_dot_q(i2,i3)
      q2q4 = q_dot_q(i2,i4)
      q3q4 = q_dot_q(i3,i4)

      q1J1 = q_dot_ep(i1,i1)
      q2J1 = q_dot_ep(i2,i1)
      q3J1 = q_dot_ep(i3,i1)
      q4J1 = q_dot_ep(i4,i1)

      q1J2 = q_dot_ep(i1,i2)
      q2J2 = q_dot_ep(i2,i2)
      q3J2 = q_dot_ep(i3,i2)
      q4J2 = q_dot_ep(i4,i2)

      q1J3 = q_dot_ep(i1,i3)
      q2J3 = q_dot_ep(i2,i3)
      q3J3 = q_dot_ep(i3,i3)
      q4J3 = q_dot_ep(i4,i3)

      q1J4 = q_dot_ep(i1,i4)
      q2J4 = q_dot_ep(i2,i4)
      q3J4 = q_dot_ep(i3,i4)
      q4J4 = q_dot_ep(i4,i4)

      J1J2 = ep_dot_ep(i1,i2)
      J1J3 = ep_dot_ep(i1,i3)
      J1J4 = ep_dot_ep(i1,i4)
      J2J3 = ep_dot_ep(i2,i3)
      J2J4 = ep_dot_ep(i2,i4)
      J3J4 = ep_dot_ep(i3,i4)

      q1sq=q_dot_q(i1,i1)
      q2sq=q_dot_q(i2,i2)
      q3sq=q_dot_q(i3,i3)
      q4sq=q_dot_q(i4,i4)

      C034=-C0_1M(m,q3sq,q4sq,2d0*q3q4) 

      D0234=D0_1M(m,q2sq,q3sq,q4sq,q2q3,q2q4,q3q4)
      CALL D_1M_ALL2(m,q2sq,q3sq,q4sq,q2q3,q2q4,q3q4,Dij)

      Dij(2,7)=-Dij(2,7)
      Dij(3,11)=-Dij(3,11)
      Dij(3,12)=-Dij(3,12)
      Dij(3,13)=-Dij(3,13)

      HEmunurhosigma=64*m*(q2J1*q2J2*q2J3*q2J4*Eij(4,2)+q3J1*q3J2*q3J3*q3J4*Eij(4,3)+
     &    q1J2*q1J3*q1J4*q2J1*Eij(4,5)+q1J2*q1J3*q1J4*q3J1*Eij(4,6)+
     &    q1J4*q2J1*q2J2*q2J3*Eij(4,8)+q1J3*q2J1*q2J2*q2J4*Eij(4,8)+
     &    q1J2*q2J1*q2J3*q2J4*Eij(4,8)+q2J2*q2J3*q2J4*q3J1*Eij(4,9)+
     &    q2J1*q2J3*q2J4*q3J2*Eij(4,9)+q2J1*q2J2*q2J4*q3J3*Eij(4,9)+
     &    q2J1*q2J2*q2J3*q3J4*Eij(4,9)+q2J1*q2J3*q2J4*q4J2*Eij(4,10)+
     &    q2J1*q2J2*q2J4*q4J3*Eij(4,10)+q2J1*q2J2*q2J3*q4J4*Eij(4,10)+
     &    q1J4*q3J1*q3J2*q3J3*Eij(4,11)+q1J3*q3J1*q3J2*q3J4*Eij(4,11)+
     &    q1J2*q3J1*q3J3*q3J4*Eij(4,11)+q2J4*q3J1*q3J2*q3J3*Eij(4,12)+
     &    q2J3*q3J1*q3J2*q3J4*Eij(4,12)+q2J2*q3J1*q3J3*q3J4*Eij(4,12)+
     &    q2J1*q3J2*q3J3*q3J4*Eij(4,12)+q3J1*q3J3*q3J4*q4J2*Eij(4,13)+
     &    q3J1*q3J2*q3J4*q4J3*Eij(4,13)+q3J1*q3J2*q3J3*q4J4*Eij(4,13)+
     &    q2J1*q4J2*q4J3*q4J4*Eij(4,15)+q3J1*q4J2*q4J3*q4J4*Eij(4,16)+
     &    q1J3*q1J4*q2J1*q2J2*Eij(4,17)+q1J2*q1J4*q2J1*q2J3*Eij(4,17)+
     &    q1J2*q1J3*q2J1*q2J4*Eij(4,17)+q1J3*q1J4*q3J1*q3J2*Eij(4,18)+
     &    q1J2*q1J4*q3J1*q3J3*Eij(4,18)+q1J2*q1J3*q3J1*q3J4*Eij(4,18)+
     &    q2J3*q2J4*q3J1*q3J2*Eij(4,20)+q2J2*q2J4*q3J1*q3J3*Eij(4,20)+
     &    q2J1*q2J4*q3J2*q3J3*Eij(4,20)+q2J2*q2J3*q3J1*q3J4*Eij(4,20)+
     &    q2J1*q2J3*q3J2*q3J4*Eij(4,20)+q2J1*q2J2*q3J3*q3J4*Eij(4,20)+
     &    q2J1*q2J4*q4J2*q4J3*Eij(4,21)+q2J1*q2J3*q4J2*q4J4*Eij(4,21)+
     &    q2J1*q2J2*q4J3*q4J4*Eij(4,21)+q3J1*q3J4*q4J2*q4J3*Eij(4,22)+
     &    q3J1*q3J3*q4J2*q4J4*Eij(4,22)+q3J1*q3J2*q4J3*q4J4*Eij(4,22)+
     &    q1J3*q1J4*q2J2*q3J1*Eij(4,23)+q1J2*q1J4*q2J3*q3J1*Eij(4,23)+
     &    q1J2*q1J3*q2J4*q3J1*Eij(4,23)+q1J3*q1J4*q2J1*q3J2*Eij(4,23)+
     &    q1J2*q1J4*q2J1*q3J3*Eij(4,23)+q1J2*q1J3*q2J1*q3J4*Eij(4,23)+
     &    q1J3*q1J4*q2J1*q4J2*Eij(4,24)+q1J2*q1J4*q2J1*q4J3*Eij(4,24)+
     &    q1J2*q1J3*q2J1*q4J4*Eij(4,24)+q1J3*q1J4*q3J1*q4J2*Eij(4,25)+
     &    q1J2*q1J4*q3J1*q4J3*Eij(4,25)+q1J2*q1J3*q3J1*q4J4*Eij(4,25)+
     &    q1J4*q2J2*q2J3*q3J1*Eij(4,26)+q1J3*q2J2*q2J4*q3J1*Eij(4,26)+
     &    q1J2*q2J3*q2J4*q3J1*Eij(4,26)+q1J4*q2J1*q2J3*q3J2*Eij(4,26)+
     &    q1J3*q2J1*q2J4*q3J2*Eij(4,26)+q1J4*q2J1*q2J2*q3J3*Eij(4,26)+
     &    q1J2*q2J1*q2J4*q3J3*Eij(4,26)+q1J3*q2J1*q2J2*q3J4*Eij(4,26)+
     &    q1J2*q2J1*q2J3*q3J4*Eij(4,26)+q1J4*q2J1*q2J3*q4J2*Eij(4,27)+
     &    q1J3*q2J1*q2J4*q4J2*Eij(4,27)+q1J4*q2J1*q2J2*q4J3*Eij(4,27)+
     &    q1J2*q2J1*q2J4*q4J3*Eij(4,27)+q1J3*q2J1*q2J2*q4J4*Eij(4,27)+
     &    q1J2*q2J1*q2J3*q4J4*Eij(4,27)+q2J3*q2J4*q3J1*q4J2*Eij(4,28)+
     &    q2J1*q2J4*q3J3*q4J2*Eij(4,28)+q2J1*q2J3*q3J4*q4J2*Eij(4,28)+
     &    q2J2*q2J4*q3J1*q4J3*Eij(4,28)+q2J1*q2J4*q3J2*q4J3*Eij(4,28)+
     &    q2J1*q2J2*q3J4*q4J3*Eij(4,28)+q2J2*q2J3*q3J1*q4J4*Eij(4,28)+
     &    q2J1*q2J3*q3J2*q4J4*Eij(4,28)+q2J1*q2J2*q3J3*q4J4*Eij(4,28)+
     &    q1J4*q2J3*q3J1*q3J2*Eij(4,29)+q1J3*q2J4*q3J1*q3J2*Eij(4,29)+
     &    q1J4*q2J2*q3J1*q3J3*Eij(4,29)+q1J2*q2J4*q3J1*q3J3*Eij(4,29)+
     &    q1J4*q2J1*q3J2*q3J3*Eij(4,29)+q1J3*q2J2*q3J1*q3J4*Eij(4,29)+
     &    q1J2*q2J3*q3J1*q3J4*Eij(4,29)+q1J3*q2J1*q3J2*q3J4*Eij(4,29)+
     &    q1J2*q2J1*q3J3*q3J4*Eij(4,29)+q1J4*q3J1*q3J3*q4J2*Eij(4,30)+
     &    q1J3*q3J1*q3J4*q4J2*Eij(4,30)+q1J4*q3J1*q3J2*q4J3*Eij(4,30)+
     &    q1J2*q3J1*q3J4*q4J3*Eij(4,30)+q1J3*q3J1*q3J2*q4J4*Eij(4,30)+
     &    q1J2*q3J1*q3J3*q4J4*Eij(4,30)+q2J4*q3J1*q3J3*q4J2*Eij(4,31)+
     &    q2J3*q3J1*q3J4*q4J2*Eij(4,31)+q2J1*q3J3*q3J4*q4J2*Eij(4,31)+
     &    q2J4*q3J1*q3J2*q4J3*Eij(4,31)+q2J2*q3J1*q3J4*q4J3*Eij(4,31)+
     &    q2J1*q3J2*q3J4*q4J3*Eij(4,31)+q2J3*q3J1*q3J2*q4J4*Eij(4,31)+
     &    q2J2*q3J1*q3J3*q4J4*Eij(4,31)+q2J1*q3J2*q3J3*q4J4*Eij(4,31)+
     &    q1J4*q2J1*q4J2*q4J3*Eij(4,32)+q1J3*q2J1*q4J2*q4J4*Eij(4,32)+
     &    q1J2*q2J1*q4J3*q4J4*Eij(4,32)+q1J4*q3J1*q4J2*q4J3*Eij(4,33)+
     &    q1J3*q3J1*q4J2*q4J4*Eij(4,33)+q1J2*q3J1*q4J3*q4J4*Eij(4,33)+
     &    q2J4*q3J1*q4J2*q4J3*Eij(4,34)+q2J1*q3J4*q4J2*q4J3*Eij(4,34)+
     &    q2J3*q3J1*q4J2*q4J4*Eij(4,34)+q2J1*q3J3*q4J2*q4J4*Eij(4,34)+
     &    q2J2*q3J1*q4J3*q4J4*Eij(4,34)+q2J1*q3J2*q4J3*q4J4*Eij(4,34)+
     &    q1J4*q2J3*q3J1*q4J2*Eij(4,35)+q1J3*q2J4*q3J1*q4J2*Eij(4,35)+
     &    q1J4*q2J1*q3J3*q4J2*Eij(4,35)+q1J3*q2J1*q3J4*q4J2*Eij(4,35)+
     &    q1J4*q2J2*q3J1*q4J3*Eij(4,35)+q1J2*q2J4*q3J1*q4J3*Eij(4,35)+
     &    q1J4*q2J1*q3J2*q4J3*Eij(4,35)+q1J2*q2J1*q3J4*q4J3*Eij(4,35)+
     &    q1J3*q2J2*q3J1*q4J4*Eij(4,35)+q1J2*q2J3*q3J1*q4J4*Eij(4,35)+
     &    q1J3*q2J1*q3J2*q4J4*Eij(4,35)+q1J2*q2J1*q3J3*q4J4*Eij(4,35)+
     &    J1J4*q1J2*q1J3*Eij(4,36)+J1J3*q1J2*q1J4*Eij(4,36)+
     &    J1J2*q1J3*q1J4*Eij(4,36)+J3J4*q2J1*q2J2*Eij(4,37)+
     &    J2J4*q2J1*q2J3*Eij(4,37)+J1J4*q2J2*q2J3*Eij(4,37)+
     &    J2J3*q2J1*q2J4*Eij(4,37)+J1J3*q2J2*q2J4*Eij(4,37)+
     &    J1J2*q2J3*q2J4*Eij(4,37)+J3J4*q3J1*q3J2*Eij(4,38)+
     &    J2J4*q3J1*q3J3*Eij(4,38)+J1J4*q3J2*q3J3*Eij(4,38)+
     &    J2J3*q3J1*q3J4*Eij(4,38)+J1J3*q3J2*q3J4*Eij(4,38)+
     &    J1J2*q3J3*q3J4*Eij(4,38)+J1J4*q4J2*q4J3*Eij(4,39)+
     &    J1J3*q4J2*q4J4*Eij(4,39)+J1J2*q4J3*q4J4*Eij(4,39)+
     &    J3J4*q1J2*q2J1*Eij(4,40)+J2J4*q1J3*q2J1*Eij(4,40)+
     &    J2J3*q1J4*q2J1*Eij(4,40)+J1J4*q1J3*q2J2*Eij(4,40)+
     &    J1J3*q1J4*q2J2*Eij(4,40)+J1J4*q1J2*q2J3*Eij(4,40)+
     &    J1J2*q1J4*q2J3*Eij(4,40)+J1J3*q1J2*q2J4*Eij(4,40)+
     &    J1J2*q1J3*q2J4*Eij(4,40)+J3J4*q1J2*q3J1*Eij(4,41)+
     &    J2J4*q1J3*q3J1*Eij(4,41)+J2J3*q1J4*q3J1*Eij(4,41)+
     &    J1J4*q1J3*q3J2*Eij(4,41)+J1J3*q1J4*q3J2*Eij(4,41)+
     &    J1J4*q1J2*q3J3*Eij(4,41)+J1J2*q1J4*q3J3*Eij(4,41)+
     &    J1J3*q1J2*q3J4*Eij(4,41)+J1J2*q1J3*q3J4*Eij(4,41)+
     &    J1J4*q1J3*q4J2*Eij(4,42)+J1J3*q1J4*q4J2*Eij(4,42)+
     &    J1J4*q1J2*q4J3*Eij(4,42)+J1J2*q1J4*q4J3*Eij(4,42)+
     &    J1J3*q1J2*q4J4*Eij(4,42)+J1J2*q1J3*q4J4*Eij(4,42)+
     &    q1J1*(q1J3*q1J4*q2J2*Eij(4,5)+q1J3*q1J4*q4J2*Eij(4,7)+
     &       q2J2*q2J3*q2J4*Eij(4,8)+q4J2*q4J3*q4J4*Eij(4,14)+
     &       q1J4*q2J2*q2J3*Eij(4,17)+q1J3*q2J2*q2J4*Eij(4,17)+
     &       q1J4*q4J2*q4J3*Eij(4,19)+q1J3*q4J2*q4J4*Eij(4,19)+
     &       q1J4*q2J2*q3J3*Eij(4,23)+q1J3*q2J2*q3J4*Eij(4,23)+
     &       q1J4*q2J3*q4J2*Eij(4,24)+q1J3*q2J4*q4J2*Eij(4,24)+
     &       q1J4*q2J2*q4J3*Eij(4,24)+q1J3*q2J2*q4J4*Eij(4,24)+
     &       q1J4*q3J3*q4J2*Eij(4,25)+q1J3*q3J4*q4J2*Eij(4,25)+
     &       q2J2*q2J4*q3J3*Eij(4,26)+q2J2*q2J3*q3J4*Eij(4,26)+
     &       q2J3*q2J4*q4J2*Eij(4,27)+q2J2*q2J4*q4J3*Eij(4,27)+
     &       q2J2*q2J3*q4J4*Eij(4,27)+q2J2*q3J3*q3J4*Eij(4,29)+
     &       q3J3*q3J4*q4J2*Eij(4,30)+q2J4*q4J2*q4J3*Eij(4,32)+
     &       q2J3*q4J2*q4J4*Eij(4,32)+q2J2*q4J3*q4J4*Eij(4,32)+
     &       q3J4*q4J2*q4J3*Eij(4,33)+q3J3*q4J2*q4J4*Eij(4,33)+
     &       q2J4*q3J3*q4J2*Eij(4,35)+q2J3*q3J4*q4J2*Eij(4,35)+
     &       q2J2*q3J4*q4J3*Eij(4,35)+q2J2*q3J3*q4J4*Eij(4,35)+
     &       J2J4*q1J3*Eij(4,36)+J2J3*q1J4*Eij(4,36)+
     &       q1J2*(q1J4*q3J3*Eij(4,6)+q1J4*q4J3*Eij(4,7)+
     &          q1J3*(q1J4*Eij(4,1)+q2J4*Eij(4,5)+q3J4*Eij(4,6)+
     &             q4J4*Eij(4,7))+q3J3*q3J4*Eij(4,18)+q4J3*q4J4*Eij(4,19)+
     &          q2J4*q3J3*Eij(4,23)+q2J4*q4J3*Eij(4,24)+
     &          q2J3*(q1J4*Eij(4,5)+q2J4*Eij(4,17)+q3J4*Eij(4,23)+
     &             q4J4*Eij(4,24))+q3J4*q4J3*Eij(4,25)+
     &          q3J3*q4J4*Eij(4,25)+J3J4*Eij(4,36))+J3J4*q2J2*Eij(4,40)+
     &       J2J4*q2J3*Eij(4,40)+J2J3*q2J4*Eij(4,40)+J2J4*q3J3*Eij(4,41)+
     &       J2J3*q3J4*Eij(4,41)+
     &       q3J2*(q1J4*q2J3*Eij(4,23)+q1J4*q4J3*Eij(4,25)+
     &          q1J3*(q1J4*Eij(4,6)+q3J4*Eij(4,18)+q2J4*Eij(4,23)+
     &             q4J4*Eij(4,25))+q2J3*q2J4*Eij(4,26)+
     &          q2J3*q3J4*Eij(4,29)+q3J4*q4J3*Eij(4,30)+
     &          q3J3*(q3J4*Eij(4,11)+q1J4*Eij(4,18)+q2J4*Eij(4,29)+
     &             q4J4*Eij(4,30))+q4J3*q4J4*Eij(4,33)+
     &          q2J4*q4J3*Eij(4,35)+q2J3*q4J4*Eij(4,35)+J3J4*Eij(4,41))+
     &       J3J4*q4J2*Eij(4,42)+J2J4*q4J3*Eij(4,42)+J2J3*q4J4*Eij(4,42))+
     &    J3J4*q2J2*q3J1*Eij(4,43)+J2J4*q2J3*q3J1*Eij(4,43)+
     &    J2J3*q2J4*q3J1*Eij(4,43)+J3J4*q2J1*q3J2*Eij(4,43)+
     &    J1J4*q2J3*q3J2*Eij(4,43)+J1J3*q2J4*q3J2*Eij(4,43)+
     &    J2J4*q2J1*q3J3*Eij(4,43)+J1J4*q2J2*q3J3*Eij(4,43)+
     &    J1J2*q2J4*q3J3*Eij(4,43)+J2J3*q2J1*q3J4*Eij(4,43)+
     &    J1J3*q2J2*q3J4*Eij(4,43)+J1J2*q2J3*q3J4*Eij(4,43)+
     &    J3J4*q2J1*q4J2*Eij(4,44)+J1J4*q2J3*q4J2*Eij(4,44)+
     &    J1J3*q2J4*q4J2*Eij(4,44)+J2J4*q2J1*q4J3*Eij(4,44)+
     &    J1J4*q2J2*q4J3*Eij(4,44)+J1J2*q2J4*q4J3*Eij(4,44)+
     &    J2J3*q2J1*q4J4*Eij(4,44)+J1J3*q2J2*q4J4*Eij(4,44)+
     &    J1J2*q2J3*q4J4*Eij(4,44)+J3J4*q3J1*q4J2*Eij(4,45)+
     &    J1J4*q3J3*q4J2*Eij(4,45)+J1J3*q3J4*q4J2*Eij(4,45)+
     &    J2J4*q3J1*q4J3*Eij(4,45)+J1J4*q3J2*q4J3*Eij(4,45)+
     &    J1J2*q3J4*q4J3*Eij(4,45)+J2J3*q3J1*q4J4*Eij(4,45)+
     &    J1J3*q3J2*q4J4*Eij(4,45)+J1J2*q3J3*q4J4*Eij(4,45)+
     &    q4J1*(q4J2*q4J3*q4J4*Eij(4,4)+q1J2*q1J3*q1J4*Eij(4,7)+
     &       q1J4*q4J2*q4J3*Eij(4,14)+q1J3*q4J2*q4J4*Eij(4,14)+
     &       q1J2*q4J3*q4J4*Eij(4,14)+q2J4*q4J2*q4J3*Eij(4,15)+
     &       q2J3*q4J2*q4J4*Eij(4,15)+q3J4*q4J2*q4J3*Eij(4,16)+
     &       q3J3*q4J2*q4J4*Eij(4,16)+q1J3*q1J4*q4J2*Eij(4,19)+
     &       q1J2*q1J4*q4J3*Eij(4,19)+q1J2*q1J3*q4J4*Eij(4,19)+
     &       q2J3*q2J4*q4J2*Eij(4,21)+q3J3*q3J4*q4J2*Eij(4,22)+
     &       q1J2*q1J4*q2J3*Eij(4,24)+q1J2*q1J3*q2J4*Eij(4,24)+
     &       q1J2*q1J4*q3J3*Eij(4,25)+q1J2*q1J3*q3J4*Eij(4,25)+
     &       q1J2*q2J3*q2J4*Eij(4,27)+q1J2*q3J3*q3J4*Eij(4,30)+
     &       q1J4*q2J3*q4J2*Eij(4,32)+q1J3*q2J4*q4J2*Eij(4,32)+
     &       q1J2*q2J4*q4J3*Eij(4,32)+q1J2*q2J3*q4J4*Eij(4,32)+
     &       q1J4*q3J3*q4J2*Eij(4,33)+q1J3*q3J4*q4J2*Eij(4,33)+
     &       q1J2*q3J4*q4J3*Eij(4,33)+q1J2*q3J3*q4J4*Eij(4,33)+
     &       q2J4*q3J3*q4J2*Eij(4,34)+q2J3*q3J4*q4J2*Eij(4,34)+
     &       q1J2*q2J4*q3J3*Eij(4,35)+q1J2*q2J3*q3J4*Eij(4,35)+
     &       J3J4*q4J2*Eij(4,39)+J2J4*q4J3*Eij(4,39)+J2J3*q4J4*Eij(4,39)+
     &       J3J4*q1J2*Eij(4,42)+J2J4*q1J3*Eij(4,42)+J2J3*q1J4*Eij(4,42)+
     &       J2J4*q2J3*Eij(4,44)+J2J3*q2J4*Eij(4,44)+
     &       q2J2*(q1J3*q1J4*Eij(4,24)+q1J3*q2J4*Eij(4,27)+
     &          q2J4*q3J3*Eij(4,28)+
     &          q2J3*(q2J4*Eij(4,10)+q4J4*Eij(4,21)+q1J4*Eij(4,27)+
     &             q3J4*Eij(4,28))+q3J3*q3J4*Eij(4,31)+
     &          q1J3*q4J4*Eij(4,32)+q3J3*q4J4*Eij(4,34)+
     &          q4J3*(q4J4*Eij(4,15)+q2J4*Eij(4,21)+q1J4*Eij(4,32)+
     &             q3J4*Eij(4,34))+q1J4*q3J3*Eij(4,35)+
     &          q1J3*q3J4*Eij(4,35)+J3J4*Eij(4,44))+J2J4*q3J3*Eij(4,45)+
     &       J2J3*q3J4*Eij(4,45)+
     &       q3J2*(q1J3*q1J4*Eij(4,25)+q2J3*q2J4*Eij(4,28)+
     &          q1J3*q3J4*Eij(4,30)+q2J3*q3J4*Eij(4,31)+
     &          q3J3*(q3J4*Eij(4,13)+q4J4*Eij(4,22)+q1J4*Eij(4,30)+
     &             q2J4*Eij(4,31))+q1J3*q4J4*Eij(4,33)+
     &          q2J3*q4J4*Eij(4,34)+
     &          q4J3*(q4J4*Eij(4,16)+q3J4*Eij(4,22)+q1J4*Eij(4,33)+
     &             q2J4*Eij(4,34))+q1J4*q2J3*Eij(4,35)+
     &          q1J3*q2J4*Eij(4,35)+J3J4*Eij(4,45)))+J1J4*J2J3*Eij(4,46)+
     &    J1J3*J2J4*Eij(4,46)+J1J2*J3J4*Eij(4,46))


      HEmunurho=32*m*(7*q1J1*q1J2*q1J3*q1J4*Eij(3,1)-J3J4*q1J1*q1J2*q1sq*Eij(3,1)-
     &    J2J3*q1J1*q1J4*q1sq*Eij(3,1)-J1J2*q1J3*q1J4*q1sq*Eij(3,1)-
     &    J1J2*q1q2*q2J3*q2J4*Eij(3,2)+5*q2J1*q2J2*q2J3*q2J4*Eij(3,2)-
     &    J3J4*q2J1*q2J2*q2sq*Eij(3,2)-J2J3*q2J1*q2J4*q2sq*Eij(3,2)-
     &    J2J3*q1q3*q3J1*q3J4*Eij(3,3)-J2J3*q2q3*q3J1*q3J4*Eij(3,3)-
     &    J1J2*q1q3*q3J3*q3J4*Eij(3,3)+3*q3J1*q3J2*q3J3*q3J4*Eij(3,3)-
     &    J3J4*q3J1*q3J2*q3sq*Eij(3,3)-J3J4*q1q4*q4J1*q4J2*Eij(3,4)-
     &    J3J4*q2q4*q4J1*q4J2*Eij(3,4)-J3J4*q3q4*q4J1*q4J2*Eij(3,4)+
     &    2*q1J4*q4J1*q4J2*q4J3*Eij(3,4)+2*q2J4*q4J1*q4J2*q4J3*Eij(3,4)+
     &    2*q3J4*q4J1*q4J2*q4J3*Eij(3,4)-J2J3*q1q4*q4J1*q4J4*Eij(3,4)-
     &    J2J3*q2q4*q4J1*q4J4*Eij(3,4)-J1J2*q1q4*q4J3*q4J4*Eij(3,4)+
     &    q4J1*q4J2*q4J3*q4J4*Eij(3,4)-J1J2*q1J3*q1J4*q1q2*Eij(3,5)+
     &    6*q1J2*q1J3*q1J4*q2J1*Eij(3,5)-J3J4*q1J2*q1sq*q2J1*Eij(3,5)-
     &    J2J3*q1J4*q1sq*q2J1*Eij(3,5)-J3J4*q1J1*q1sq*q2J2*Eij(3,5)-
     &    J1J2*q1J4*q1sq*q2J3*Eij(3,5)-J2J3*q1J1*q1sq*q2J4*Eij(3,5)-
     &    J1J2*q1J3*q1sq*q2J4*Eij(3,5)-J3J4*q1J1*q1J2*q2sq*Eij(3,5)-
     &    J2J3*q1J1*q1J4*q2sq*Eij(3,5)-
     &    J3J4*q1J1*q1J2*q1q2*(Eij(3,1)+Eij(3,5))-
     &    J2J3*q1J1*q1J4*q1q2*(Eij(3,1)+Eij(3,5))+
     &    q1J1*q1J3*q1J4*q2J2*(Eij(3,1)+5*Eij(3,5))+
     &    q1J1*q1J2*q1J4*q2J3*(2*Eij(3,1)+5*Eij(3,5))+
     &    q1J1*q1J2*q1J3*q2J4*(2*Eij(3,1)+5*Eij(3,5))-
     &    J2J3*q1J1*q1J4*q1q3*Eij(3,6)-J1J2*q1J3*q1J4*q1q3*Eij(3,6)-
     &    J2J3*q1J1*q1J4*q2q3*Eij(3,6)+6*q1J2*q1J3*q1J4*q3J1*Eij(3,6)-
     &    J3J4*q1J2*q1sq*q3J1*Eij(3,6)-J2J3*q1J4*q1sq*q3J1*Eij(3,6)+
     &    5*q1J1*q1J3*q1J4*q3J2*Eij(3,6)-J3J4*q1J1*q1sq*q3J2*Eij(3,6)-
     &    J1J2*q1J4*q1sq*q3J3*Eij(3,6)-J2J3*q1J1*q1sq*q3J4*Eij(3,6)-
     &    J1J2*q1J3*q1sq*q3J4*Eij(3,6)-J3J4*q1J1*q1J2*q3sq*Eij(3,6)-
     &    J3J4*q1J1*q1J2*q1q3*(Eij(3,1)+Eij(3,6))-
     &    J3J4*q1J1*q1J2*q2q3*(Eij(3,5)+Eij(3,6))+
     &    q1J1*q1J2*q1J4*q3J3*(Eij(3,1)+5*Eij(3,6))+
     &    q1J1*q1J2*q1J3*q3J4*(2*Eij(3,1)+5*Eij(3,6))-
     &    J3J4*q1J1*q1J2*q1q4*Eij(3,7)-J2J3*q1J1*q1J4*q1q4*Eij(3,7)-
     &    J1J2*q1J3*q1J4*q1q4*Eij(3,7)-J3J4*q1J1*q1J2*q2q4*Eij(3,7)-
     &    J2J3*q1J1*q1J4*q2q4*Eij(3,7)-J3J4*q1J1*q1J2*q3q4*Eij(3,7)+
     &    6*q1J2*q1J3*q1J4*q4J1*Eij(3,7)-J3J4*q1J2*q1sq*q4J1*Eij(3,7)-
     &    J2J3*q1J4*q1sq*q4J1*Eij(3,7)+5*q1J1*q1J3*q1J4*q4J2*Eij(3,7)-
     &    J3J4*q1J1*q1sq*q4J2*Eij(3,7)+5*q1J1*q1J2*q1J4*q4J3*Eij(3,7)-
     &    J1J2*q1J4*q1sq*q4J3*Eij(3,7)-J2J3*q1J1*q1sq*q4J4*Eij(3,7)-
     &    J1J2*q1J3*q1sq*q4J4*Eij(3,7)+
     &    q1J1*q1J2*q1J3*q4J4*(Eij(3,1)+5*Eij(3,7))-
     &    J3J4*q1sq*q2J1*q2J2*Eij(3,8)-J1J2*q1J4*q1q2*q2J3*Eij(3,8)-
     &    J1J2*q1J3*q1q2*q2J4*Eij(3,8)-J2J3*q1sq*q2J1*q2J4*Eij(3,8)-
     &    J1J2*q1sq*q2J3*q2J4*Eij(3,8)-J3J4*q1J2*q2J1*q2sq*Eij(3,8)-
     &    J2J3*q1J4*q2J1*q2sq*Eij(3,8)-J3J4*q1J1*q2J2*q2sq*Eij(3,8)-
     &    J2J3*q1J1*q2J4*q2sq*Eij(3,8)-
     &    J3J4*q1q2*q2J1*q2J2*(Eij(3,2)+Eij(3,8))-
     &    J2J3*q1q2*q2J1*q2J4*(Eij(3,2)+Eij(3,8))-
     &    J3J4*q1J2*q1q2*q2J1*(Eij(3,5)+Eij(3,8))-
     &    J2J3*q1J4*q1q2*q2J1*(Eij(3,5)+Eij(3,8))-
     &    J3J4*q1J1*q1q2*q2J2*(Eij(3,5)+Eij(3,8))+
     &    3*q1J1*q1J4*q2J2*q2J3*(Eij(3,5)+Eij(3,8))-
     &    J2J3*q1J1*q1q2*q2J4*(Eij(3,5)+Eij(3,8))+
     &    3*q1J1*q1J3*q2J2*q2J4*(Eij(3,5)+Eij(3,8))+
     &    2*q1J2*q2J1*q2J3*q2J4*(Eij(3,2)+2*Eij(3,8))+
     &    2*q1J2*q1J4*q2J1*q2J3*(Eij(3,5)+2*Eij(3,8))+
     &    2*q1J2*q1J3*q2J1*q2J4*(Eij(3,5)+2*Eij(3,8))+
     &    q1J4*q2J1*q2J2*q2J3*(2*Eij(3,2)+3*Eij(3,8))+
     &    q1J3*q2J1*q2J2*q2J4*(2*Eij(3,2)+3*Eij(3,8))+
     &    q1J1*q1J2*q2J3*q2J4*(4*Eij(3,5)+3*Eij(3,8))+
     &    q1J3*q1J4*q2J1*q2J2*(Eij(3,5)+4*Eij(3,8))+
     &    q1J1*q2J2*q2J3*q2J4*(Eij(3,2)+5*Eij(3,8))-
     &    J2J3*q1q3*q2J1*q2J4*Eij(3,9)-J1J2*q1q3*q2J3*q2J4*Eij(3,9)-
     &    J2J3*q2J1*q2J4*q2q3*Eij(3,9)+5*q2J2*q2J3*q2J4*q3J1*Eij(3,9)-
     &    J3J4*q2J2*q2sq*q3J1*Eij(3,9)-J2J3*q2J4*q2sq*q3J1*Eij(3,9)+
     &    4*q2J1*q2J3*q2J4*q3J2*Eij(3,9)-J3J4*q2J1*q2sq*q3J2*Eij(3,9)-
     &    J1J2*q1q2*q2J4*q3J3*Eij(3,9)-J1J2*q1q2*q2J3*q3J4*Eij(3,9)-
     &    J2J3*q2J1*q2sq*q3J4*Eij(3,9)-J3J4*q2J1*q2J2*q3sq*Eij(3,9)-
     &    J3J4*q2J1*q2J2*q2q3*(Eij(3,2)+Eij(3,9))-
     &    J3J4*q1q3*q2J1*q2J2*(Eij(3,8)+Eij(3,9))+
     &    q2J1*q2J2*q2J4*q3J3*(Eij(3,2)+3*Eij(3,9))+
     &    q2J1*q2J2*q2J3*q3J4*(2*Eij(3,2)+3*Eij(3,9))-
     &    J3J4*q1q4*q2J1*q2J2*Eij(3,10)-J2J3*q1q4*q2J1*q2J4*Eij(3,10)-
     &    J1J2*q1q4*q2J3*q2J4*Eij(3,10)-J3J4*q2J1*q2J2*q2q4*Eij(3,10)-
     &    J2J3*q2J1*q2J4*q2q4*Eij(3,10)-J3J4*q2J1*q2J2*q3q4*Eij(3,10)+
     &    5*q2J2*q2J3*q2J4*q4J1*Eij(3,10)-J3J4*q2J2*q2sq*q4J1*Eij(3,10)-
     &    J2J3*q2J4*q2sq*q4J1*Eij(3,10)+4*q2J1*q2J3*q2J4*q4J2*Eij(3,10)-
     &    J3J4*q2J1*q2sq*q4J2*Eij(3,10)-J1J2*q1q2*q2J4*q4J3*Eij(3,10)+
     &    3*q2J1*q2J2*q2J4*q4J3*Eij(3,10)-J1J2*q1q2*q2J3*q4J4*Eij(3,10)-
     &    J2J3*q2J1*q2sq*q4J4*Eij(3,10)+
     &    q2J1*q2J2*q2J3*q4J4*(Eij(3,2)+3*Eij(3,10))-
     &    J2J3*q1J4*q1q3*q3J1*Eij(3,11)-J2J3*q1J4*q2q3*q3J1*Eij(3,11)+
     &    4*q1J3*q1J4*q3J1*q3J2*Eij(3,11)-J3J4*q1sq*q3J1*q3J2*Eij(3,11)-
     &    J1J2*q1J4*q1q3*q3J3*Eij(3,11)-J2J3*q1J1*q1q3*q3J4*Eij(3,11)-
     &    J1J2*q1J3*q1q3*q3J4*Eij(3,11)-J2J3*q1J1*q2q3*q3J4*Eij(3,11)-
     &    J2J3*q1sq*q3J1*q3J4*Eij(3,11)-J1J2*q1sq*q3J3*q3J4*Eij(3,11)-
     &    J3J4*q1J2*q3J1*q3sq*Eij(3,11)-J3J4*q1J1*q3J2*q3sq*Eij(3,11)-
     &    J3J4*q1q3*q3J1*q3J2*(Eij(3,3)+Eij(3,11))+
     &    2*q1J3*q3J1*q3J2*q3J4*(Eij(3,3)+Eij(3,11))+
     &    q1J4*q3J1*q3J2*q3J3*(2*Eij(3,3)+Eij(3,11))-
     &    J3J4*q1J2*q1q3*q3J1*(Eij(3,6)+Eij(3,11))-
     &    J3J4*q1J1*q1q3*q3J2*(Eij(3,6)+Eij(3,11))+
     &    3*q1J1*q1J2*q3J3*q3J4*(Eij(3,6)+Eij(3,11))+
     &    2*q1J2*q1J3*q3J1*q3J4*(Eij(3,6)+2*Eij(3,11))+
     &    q1J1*q3J2*q3J3*q3J4*(Eij(3,3)+3*Eij(3,11))+
     &    q1J2*q3J1*q3J3*q3J4*(2*Eij(3,3)+3*Eij(3,11))+
     &    q1J1*q1J4*q3J2*q3J3*(Eij(3,6)+3*Eij(3,11))+
     &    q1J1*q1J3*q3J2*q3J4*(2*Eij(3,6)+3*Eij(3,11))+
     &    q1J2*q1J4*q3J1*q3J3*(Eij(3,6)+4*Eij(3,11))-
     &    J2J3*q1q3*q2J4*q3J1*Eij(3,12)-J2J3*q2J4*q2q3*q3J1*Eij(3,12)+
     &    4*q2J3*q2J4*q3J1*q3J2*Eij(3,12)-J3J4*q2sq*q3J1*q3J2*Eij(3,12)-
     &    J1J2*q1q3*q2J4*q3J3*Eij(3,12)-J2J3*q1q3*q2J1*q3J4*Eij(3,12)-
     &    J1J2*q1q3*q2J3*q3J4*Eij(3,12)-J2J3*q2J1*q2q3*q3J4*Eij(3,12)-
     &    J2J3*q2sq*q3J1*q3J4*Eij(3,12)-J1J2*q1q2*q3J3*q3J4*Eij(3,12)+
     &    3*q2J1*q3J2*q3J3*q3J4*Eij(3,12)-J3J4*q2J2*q3J1*q3sq*Eij(3,12)-
     &    J3J4*q2J1*q3J2*q3sq*Eij(3,12)-
     &    J3J4*q2q3*q3J1*q3J2*(Eij(3,3)+Eij(3,12))+
     &    2*q2J3*q3J1*q3J2*q3J4*(Eij(3,3)+Eij(3,12))+
     &    q2J4*q3J1*q3J2*q3J3*(2*Eij(3,3)+Eij(3,12))-
     &    J3J4*q2J2*q2q3*q3J1*(Eij(3,9)+Eij(3,12))-
     &    J3J4*q2J1*q2q3*q3J2*(Eij(3,9)+Eij(3,12))+
     &    2*q2J1*q2J3*q3J2*q3J4*(Eij(3,9)+Eij(3,12))+
     &    q2J1*q2J2*q3J3*q3J4*(3*Eij(3,9)+Eij(3,12))-
     &    J3J4*q1q2*q3J1*q3J2*(Eij(3,11)+Eij(3,12))+
     &    2*q1J4*q2J3*q3J1*q3J2*(Eij(3,11)+Eij(3,12))+
     &    2*q1J3*q2J4*q3J1*q3J2*(Eij(3,11)+Eij(3,12))-
     &    J2J3*q1q2*q3J1*q3J4*(Eij(3,11)+Eij(3,12))+
     &    q2J1*q2J4*q3J2*q3J3*(Eij(3,9)+2*Eij(3,12))+
     &    q2J2*q3J1*q3J3*q3J4*(Eij(3,3)+3*Eij(3,12))+
     &    q2J2*q2J4*q3J1*q3J3*(Eij(3,9)+3*Eij(3,12))+
     &    q2J2*q2J3*q3J1*q3J4*(2*Eij(3,9)+3*Eij(3,12))-
     &    J3J4*q1q4*q3J1*q3J2*Eij(3,13)-J3J4*q2q4*q3J1*q3J2*Eij(3,13)-
     &    J2J3*q1q4*q3J1*q3J4*Eij(3,13)-J2J3*q2q4*q3J1*q3J4*Eij(3,13)-
     &    J1J2*q1q4*q3J3*q3J4*Eij(3,13)-J3J4*q3J1*q3J2*q3q4*Eij(3,13)-
     &    J2J3*q1q3*q3J4*q4J1*Eij(3,13)-J2J3*q2q3*q3J4*q4J1*Eij(3,13)+
     &    3*q3J2*q3J3*q3J4*q4J1*Eij(3,13)-J3J4*q3J2*q3sq*q4J1*Eij(3,13)+
     &    3*q3J1*q3J3*q3J4*q4J2*Eij(3,13)-J3J4*q3J1*q3sq*q4J2*Eij(3,13)+
     &    2*q1J4*q3J1*q3J2*q4J3*Eij(3,13)+2*q2J4*q3J1*q3J2*q4J3*Eij(3,13)-
     &    J1J2*q1q3*q3J4*q4J3*Eij(3,13)+2*q3J1*q3J2*q3J4*q4J3*Eij(3,13)-
     &    J2J3*q1q3*q3J1*q4J4*Eij(3,13)-J2J3*q2q3*q3J1*q4J4*Eij(3,13)-
     &    J1J2*q1q3*q3J3*q4J4*Eij(3,13)+q3J1*q3J2*q4J3*q4J4*Eij(3,13)+
     &    q3J1*q3J2*q3J3*q4J4*(Eij(3,3)+Eij(3,13))+
     &    q1J3*q3J1*q3J2*q4J4*(Eij(3,11)+2*Eij(3,13))+
     &    q2J3*q3J1*q3J2*q4J4*(Eij(3,12)+2*Eij(3,13))-
     &    J3J4*q1J2*q1q4*q4J1*Eij(3,14)-J2J3*q1J4*q1q4*q4J1*Eij(3,14)-
     &    J3J4*q1J2*q2q4*q4J1*Eij(3,14)-J2J3*q1J4*q2q4*q4J1*Eij(3,14)-
     &    J3J4*q1J2*q3q4*q4J1*Eij(3,14)-J3J4*q1J1*q1q4*q4J2*Eij(3,14)-
     &    J3J4*q1J1*q2q4*q4J2*Eij(3,14)-J3J4*q1J1*q3q4*q4J2*Eij(3,14)+
     &    4*q1J3*q1J4*q4J1*q4J2*Eij(3,14)-J3J4*q1sq*q4J1*q4J2*Eij(3,14)-
     &    J1J2*q1J4*q1q4*q4J3*Eij(3,14)+4*q1J2*q1J4*q4J1*q4J3*Eij(3,14)+
     &    3*q1J1*q1J4*q4J2*q4J3*Eij(3,14)-J2J3*q1J1*q1q4*q4J4*Eij(3,14)-
     &    J1J2*q1J3*q1q4*q4J4*Eij(3,14)-J2J3*q1J1*q2q4*q4J4*Eij(3,14)-
     &    J2J3*q1sq*q4J1*q4J4*Eij(3,14)-J1J2*q1sq*q4J3*q4J4*Eij(3,14)+
     &    q1J1*q4J2*q4J3*q4J4*(Eij(3,4)+Eij(3,14))+
     &    q1J3*q4J1*q4J2*q4J4*(2*Eij(3,4)+Eij(3,14))+
     &    q1J2*q4J1*q4J3*q4J4*(2*Eij(3,4)+Eij(3,14))+
     &    q1J1*q1J3*q4J2*q4J4*(Eij(3,7)+3*Eij(3,14))+
     &    q1J1*q1J2*q4J3*q4J4*(Eij(3,7)+3*Eij(3,14))+
     &    q1J2*q1J3*q4J1*q4J4*(Eij(3,7)+4*Eij(3,14))-
     &    J3J4*q1q4*q2J2*q4J1*Eij(3,15)-J2J3*q1q4*q2J4*q4J1*Eij(3,15)-
     &    J3J4*q2J2*q2q4*q4J1*Eij(3,15)-J2J3*q2J4*q2q4*q4J1*Eij(3,15)-
     &    J3J4*q2J2*q3q4*q4J1*Eij(3,15)-J3J4*q1q4*q2J1*q4J2*Eij(3,15)-
     &    J3J4*q2J1*q2q4*q4J2*Eij(3,15)-J3J4*q2J1*q3q4*q4J2*Eij(3,15)+
     &    4*q2J3*q2J4*q4J1*q4J2*Eij(3,15)-J3J4*q2sq*q4J1*q4J2*Eij(3,15)-
     &    J1J2*q1q4*q2J4*q4J3*Eij(3,15)+3*q2J2*q2J4*q4J1*q4J3*Eij(3,15)+
     &    2*q1J4*q2J1*q4J2*q4J3*Eij(3,15)+2*q2J1*q2J4*q4J2*q4J3*Eij(3,15)+
     &    2*q2J1*q3J4*q4J2*q4J3*Eij(3,15)-J2J3*q1q4*q2J1*q4J4*Eij(3,15)-
     &    J1J2*q1q4*q2J3*q4J4*Eij(3,15)-J2J3*q2J1*q2q4*q4J4*Eij(3,15)-
     &    J2J3*q2sq*q4J1*q4J4*Eij(3,15)-J1J2*q1q2*q4J3*q4J4*Eij(3,15)+
     &    q2J1*q4J2*q4J3*q4J4*Eij(3,15)+
     &    q2J2*q4J1*q4J3*q4J4*(Eij(3,4)+Eij(3,15))+
     &    q2J3*q4J1*q4J2*q4J4*(2*Eij(3,4)+Eij(3,15))+
     &    q2J1*q2J2*q4J3*q4J4*(Eij(3,10)+Eij(3,15))-
     &    J3J4*q1q2*q4J1*q4J2*(Eij(3,14)+Eij(3,15))+
     &    2*q1J4*q2J3*q4J1*q4J2*(Eij(3,14)+Eij(3,15))+
     &    2*q1J3*q2J4*q4J1*q4J2*(Eij(3,14)+Eij(3,15))+
     &    2*q1J2*q2J4*q4J1*q4J3*(Eij(3,14)+Eij(3,15))-
     &    J2J3*q1q2*q4J1*q4J4*(Eij(3,14)+Eij(3,15))+
     &    q1J1*q2J4*q4J2*q4J3*(2*Eij(3,14)+Eij(3,15))+
     &    q2J1*q2J3*q4J2*q4J4*(Eij(3,10)+2*Eij(3,15))+
     &    q1J4*q2J2*q4J1*q4J3*(Eij(3,14)+2*Eij(3,15))+
     &    q2J2*q2J3*q4J1*q4J4*(Eij(3,10)+3*Eij(3,15))-
     &    J3J4*q1q4*q3J2*q4J1*Eij(3,16)-J3J4*q2q4*q3J2*q4J1*Eij(3,16)-
     &    J2J3*q1q4*q3J4*q4J1*Eij(3,16)-J2J3*q2q4*q3J4*q4J1*Eij(3,16)-
     &    J3J4*q3J2*q3q4*q4J1*Eij(3,16)-J3J4*q1q4*q3J1*q4J2*Eij(3,16)-
     &    J3J4*q2q4*q3J1*q4J2*Eij(3,16)-J3J4*q3J1*q3q4*q4J2*Eij(3,16)+
     &    3*q3J3*q3J4*q4J1*q4J2*Eij(3,16)-J3J4*q3sq*q4J1*q4J2*Eij(3,16)-
     &    J1J2*q1q4*q3J4*q4J3*Eij(3,16)+2*q1J4*q3J2*q4J1*q4J3*Eij(3,16)+
     &    2*q2J4*q3J2*q4J1*q4J3*Eij(3,16)+2*q3J2*q3J4*q4J1*q4J3*Eij(3,16)+
     &    2*q1J4*q3J1*q4J2*q4J3*Eij(3,16)+2*q2J4*q3J1*q4J2*q4J3*Eij(3,16)+
     &    2*q3J1*q3J4*q4J2*q4J3*Eij(3,16)-J2J3*q1q4*q3J1*q4J4*Eij(3,16)-
     &    J2J3*q2q4*q3J1*q4J4*Eij(3,16)-J1J2*q1q4*q3J3*q4J4*Eij(3,16)-
     &    J2J3*q1q3*q4J1*q4J4*Eij(3,16)-J2J3*q2q3*q4J1*q4J4*Eij(3,16)-
     &    J1J2*q1q3*q4J3*q4J4*Eij(3,16)+q3J2*q4J1*q4J3*q4J4*Eij(3,16)+
     &    q3J1*q4J2*q4J3*q4J4*Eij(3,16)+
     &    q3J3*q4J1*q4J2*q4J4*(Eij(3,4)+Eij(3,16))+
     &    q3J2*q3J3*q4J1*q4J4*(Eij(3,13)+Eij(3,16))+
     &    q3J1*q3J3*q4J2*q4J4*(Eij(3,13)+Eij(3,16))-
     &    J3J4*q1q3*q4J1*q4J2*(Eij(3,14)+Eij(3,16))+
     &    2*q1J3*q3J4*q4J1*q4J2*(Eij(3,14)+Eij(3,16))+
     &    2*q1J2*q3J4*q4J1*q4J3*(Eij(3,14)+Eij(3,16))+
     &    q1J1*q3J4*q4J2*q4J3*(2*Eij(3,14)+Eij(3,16))-
     &    J3J4*q2q3*q4J1*q4J2*(Eij(3,15)+Eij(3,16))+
     &    2*q2J3*q3J4*q4J1*q4J2*(Eij(3,15)+Eij(3,16))+
     &    q2J2*q3J4*q4J1*q4J3*(2*Eij(3,15)+Eij(3,16))+
     &    q1J4*q3J3*q4J1*q4J2*(Eij(3,14)+2*Eij(3,16))+
     &    q2J4*q3J3*q4J1*q4J2*(Eij(3,15)+2*Eij(3,16))-
     &    J2J3*q1J4*q1q3*q2J1*Eij(3,17)-J1J2*q1J4*q1q3*q2J3*Eij(3,17)-
     &    J2J3*q1J1*q1q3*q2J4*Eij(3,17)-J1J2*q1J3*q1q3*q2J4*Eij(3,17)-
     &    J2J3*q1J4*q2J1*q2q3*Eij(3,17)-J2J3*q1J1*q2J4*q2q3*Eij(3,17)-
     &    J3J4*q1sq*q2J2*q3J1*Eij(3,17)-J2J3*q1sq*q2J4*q3J1*Eij(3,17)-
     &    J3J4*q1J2*q2sq*q3J1*Eij(3,17)-J2J3*q1J4*q2sq*q3J1*Eij(3,17)+
     &    4*q1J3*q1J4*q2J1*q3J2*Eij(3,17)-J3J4*q1sq*q2J1*q3J2*Eij(3,17)-
     &    J3J4*q1J1*q2sq*q3J2*Eij(3,17)-J1J2*q1J4*q1q2*q3J3*Eij(3,17)-
     &    J1J2*q1sq*q2J4*q3J3*Eij(3,17)-J1J2*q1J3*q1q2*q3J4*Eij(3,17)-
     &    J2J3*q1sq*q2J1*q3J4*Eij(3,17)-J1J2*q1sq*q2J3*q3J4*Eij(3,17)-
     &    J2J3*q1J1*q2sq*q3J4*Eij(3,17)-J3J4*q1J2*q2J1*q3sq*Eij(3,17)-
     &    J3J4*q1J1*q2J2*q3sq*Eij(3,17)-
     &    J3J4*q1J2*q1q3*q2J1*(Eij(3,5)+Eij(3,17))-
     &    J3J4*q1J1*q1q3*q2J2*(Eij(3,5)+Eij(3,17))-
     &    J3J4*q1J2*q1q2*q3J1*(Eij(3,6)+Eij(3,17))-
     &    J2J3*q1J4*q1q2*q3J1*(Eij(3,6)+Eij(3,17))-
     &    J3J4*q1J1*q1q2*q3J2*(Eij(3,6)+Eij(3,17))-
     &    J2J3*q1J1*q1q2*q3J4*(Eij(3,6)+Eij(3,17))-
     &    J3J4*q1J2*q2J1*q2q3*(Eij(3,8)+Eij(3,17))-
     &    J3J4*q1J1*q2J2*q2q3*(Eij(3,8)+Eij(3,17))-
     &    J3J4*q1q2*q2J2*q3J1*(Eij(3,9)+Eij(3,17))-
     &    J2J3*q1q2*q2J4*q3J1*(Eij(3,9)+Eij(3,17))-
     &    J3J4*q1q2*q2J1*q3J2*(Eij(3,9)+Eij(3,17))+
     &    2*q1J4*q2J1*q2J3*q3J2*(Eij(3,9)+Eij(3,17))+
     &    2*q1J3*q2J1*q2J4*q3J2*(Eij(3,9)+Eij(3,17))-
     &    J2J3*q1q2*q2J1*q3J4*(Eij(3,9)+Eij(3,17))+
     &    2*q1J2*q2J1*q2J3*q3J4*(Eij(3,8)+Eij(3,9)+Eij(3,17))+
     &    q1J4*q2J1*q2J2*q3J3*(Eij(3,8)+2*Eij(3,9)+Eij(3,17))+
     &    q1J3*q2J1*q2J2*q3J4*(2*(Eij(3,8)+Eij(3,9))+Eij(3,17))-
     &    J3J4*q1J2*q2q3*q3J1*(Eij(3,11)+Eij(3,17))-
     &    J3J4*q1J1*q2q3*q3J2*(Eij(3,11)+Eij(3,17))-
     &    J3J4*q1q3*q2J2*q3J1*(Eij(3,12)+Eij(3,17))-
     &    J3J4*q1q3*q2J1*q3J2*(Eij(3,12)+Eij(3,17))+
     &    2*q1J3*q2J1*q3J2*q3J4*(Eij(3,12)+Eij(3,17))+
     &    2*q1J2*q2J3*q3J1*q3J4*(Eij(3,11)+Eij(3,12)+Eij(3,17))+
     &    q1J1*q2J4*q3J2*q3J3*(2*Eij(3,11)+Eij(3,12)+Eij(3,17))+
     &    q1J4*q2J1*q3J2*q3J3*(2*Eij(3,12)+Eij(3,17))+
     &    q1J4*q2J2*q3J1*q3J3*(Eij(3,11)+2*Eij(3,12)+Eij(3,17))+
     &    q1J2*q2J4*q3J1*q3J3*(2*Eij(3,11)+2*Eij(3,12)+Eij(3,17))+
     &    2*q1J2*q1J3*q2J1*q3J4*(Eij(3,5)+2*Eij(3,17))+
     &    2*q1J2*q1J4*q2J3*q3J1*(Eij(3,6)+2*Eij(3,17))+
     &    2*q1J2*q1J3*q2J4*q3J1*(Eij(3,6)+2*Eij(3,17))+
     &    2*q1J2*q2J3*q2J4*q3J1*(Eij(3,9)+2*Eij(3,17))+
     &    q1J2*q2J1*q2J4*q3J3*(Eij(3,8)+2*Eij(3,9)+2*Eij(3,17))+
     &    q1J1*q2J3*q3J2*q3J4*(2*Eij(3,11)+Eij(3,12)+2*Eij(3,17))+
     &    q1J1*q1J4*q2J2*q3J3*(Eij(3,5)+Eij(3,6)+3*Eij(3,17))+
     &    q1J1*q1J3*q2J2*q3J4*(2*Eij(3,5)+Eij(3,6)+3*Eij(3,17))+
     &    q1J1*q1J4*q2J3*q3J2*(2*Eij(3,6)+3*Eij(3,17))+
     &    q1J1*q1J3*q2J4*q3J2*(2*Eij(3,6)+3*Eij(3,17))+
     &    q1J1*q1J2*q2J4*q3J3*(Eij(3,5)+2*Eij(3,6)+3*Eij(3,17))+
     &    q1J1*q1J2*q2J3*q3J4*(2*(Eij(3,5)+Eij(3,6))+3*Eij(3,17))+
     &    q1J1*q2J2*q2J4*q3J3*(Eij(3,8)+Eij(3,9)+3*Eij(3,17))+
     &    q1J1*q2J2*q2J3*q3J4*(2*Eij(3,8)+Eij(3,9)+3*Eij(3,17))+
     &    q1J4*q2J2*q2J3*q3J1*(2*Eij(3,9)+3*Eij(3,17))+
     &    q1J3*q2J2*q2J4*q3J1*(2*Eij(3,9)+3*Eij(3,17))+
     &    q1J1*q2J2*q3J3*q3J4*(Eij(3,11)+Eij(3,12)+3*Eij(3,17))+
     &    q1J2*q2J1*q3J3*q3J4*(2*Eij(3,12)+3*Eij(3,17))+
     &    q1J2*q1J4*q2J1*q3J3*(Eij(3,5)+4*Eij(3,17))+
     &    q1J3*q1J4*q2J2*q3J1*(Eij(3,6)+4*Eij(3,17))+
     &    q1J1*q2J3*q2J4*q3J2*(Eij(3,9)+4*Eij(3,17))+
     &    q1J3*q2J2*q3J1*q3J4*(Eij(3,11)+2*(Eij(3,12)+Eij(3,17)))-
     &    J3J4*q1J2*q1q4*q2J1*Eij(3,18)-J2J3*q1J4*q1q4*q2J1*Eij(3,18)-
     &    J3J4*q1J1*q1q4*q2J2*Eij(3,18)-J1J2*q1J4*q1q4*q2J3*Eij(3,18)-
     &    J2J3*q1J1*q1q4*q2J4*Eij(3,18)-J1J2*q1J3*q1q4*q2J4*Eij(3,18)-
     &    J3J4*q1J2*q2J1*q2q4*Eij(3,18)-J2J3*q1J4*q2J1*q2q4*Eij(3,18)-
     &    J3J4*q1J1*q2J2*q2q4*Eij(3,18)-J2J3*q1J1*q2J4*q2q4*Eij(3,18)-
     &    J3J4*q1J2*q2J1*q3q4*Eij(3,18)-J3J4*q1J1*q2J2*q3q4*Eij(3,18)-
     &    J3J4*q1sq*q2J2*q4J1*Eij(3,18)-J2J3*q1sq*q2J4*q4J1*Eij(3,18)-
     &    J3J4*q1J2*q2sq*q4J1*Eij(3,18)-J2J3*q1J4*q2sq*q4J1*Eij(3,18)+
     &    4*q1J3*q1J4*q2J1*q4J2*Eij(3,18)-J3J4*q1sq*q2J1*q4J2*Eij(3,18)-
     &    J3J4*q1J1*q2sq*q4J2*Eij(3,18)-J1J2*q1J4*q1q2*q4J3*Eij(3,18)+
     &    4*q1J2*q1J4*q2J1*q4J3*Eij(3,18)-J1J2*q1sq*q2J4*q4J3*Eij(3,18)-
     &    J1J2*q1J3*q1q2*q4J4*Eij(3,18)-J2J3*q1sq*q2J1*q4J4*Eij(3,18)-
     &    J1J2*q1sq*q2J3*q4J4*Eij(3,18)-J2J3*q1J1*q2sq*q4J4*Eij(3,18)-
     &    J3J4*q1J2*q1q2*q4J1*(Eij(3,7)+Eij(3,18))-
     &    J2J3*q1J4*q1q2*q4J1*(Eij(3,7)+Eij(3,18))-
     &    J3J4*q1J1*q1q2*q4J2*(Eij(3,7)+Eij(3,18))-
     &    J2J3*q1J1*q1q2*q4J4*(Eij(3,7)+Eij(3,18))-
     &    J3J4*q1q2*q2J2*q4J1*(Eij(3,10)+Eij(3,18))-
     &    J2J3*q1q2*q2J4*q4J1*(Eij(3,10)+Eij(3,18))-
     &    J3J4*q1q2*q2J1*q4J2*(Eij(3,10)+Eij(3,18))+
     &    2*q1J4*q2J1*q2J3*q4J2*(Eij(3,10)+Eij(3,18))+
     &    2*q1J3*q2J1*q2J4*q4J2*(Eij(3,10)+Eij(3,18))+
     &    2*q1J2*q2J1*q2J4*q4J3*(Eij(3,10)+Eij(3,18))-
     &    J2J3*q1q2*q2J1*q4J4*(Eij(3,10)+Eij(3,18))+
     &    q1J4*q2J1*q2J2*q4J3*(2*Eij(3,10)+Eij(3,18))+
     &    q1J3*q2J1*q2J2*q4J4*(Eij(3,8)+2*Eij(3,10)+Eij(3,18))+
     &    q1J1*q2J2*q4J3*q4J4*(Eij(3,14)+Eij(3,15)+Eij(3,18))+
     &    q1J1*q2J3*q4J2*q4J4*(2*Eij(3,14)+Eij(3,15)+Eij(3,18))+
     &    q1J3*q2J1*q4J2*q4J4*(2*Eij(3,15)+Eij(3,18))+
     &    q1J2*q2J1*q4J3*q4J4*(2*Eij(3,15)+Eij(3,18))+
     &    q1J3*q2J2*q4J1*q4J4*(Eij(3,14)+2*Eij(3,15)+Eij(3,18))+
     &    q1J2*q2J3*q4J1*q4J4*(2*Eij(3,14)+2*Eij(3,15)+Eij(3,18))+
     &    2*q1J2*q1J4*q2J3*q4J1*(Eij(3,7)+2*Eij(3,18))+
     &    2*q1J2*q1J3*q2J4*q4J1*(Eij(3,7)+2*Eij(3,18))+
     &    2*q1J2*q2J3*q2J4*q4J1*(Eij(3,10)+2*Eij(3,18))+
     &    q1J2*q2J1*q2J3*q4J4*(Eij(3,8)+2*Eij(3,10)+2*Eij(3,18))+
     &    q1J1*q1J4*q2J2*q4J3*(Eij(3,7)+3*Eij(3,18))+
     &    q1J1*q1J3*q2J2*q4J4*(Eij(3,5)+Eij(3,7)+3*Eij(3,18))+
     &    q1J1*q1J4*q2J3*q4J2*(2*Eij(3,7)+3*Eij(3,18))+
     &    q1J1*q1J3*q2J4*q4J2*(2*Eij(3,7)+3*Eij(3,18))+
     &    q1J1*q1J2*q2J4*q4J3*(2*Eij(3,7)+3*Eij(3,18))+
     &    q1J1*q1J2*q2J3*q4J4*(Eij(3,5)+2*Eij(3,7)+3*Eij(3,18))+
     &    q1J1*q2J2*q2J4*q4J3*(Eij(3,10)+3*Eij(3,18))+
     &    q1J1*q2J2*q2J3*q4J4*(Eij(3,8)+Eij(3,10)+3*Eij(3,18))+
     &    q1J4*q2J2*q2J3*q4J1*(2*Eij(3,10)+3*Eij(3,18))+
     &    q1J3*q2J2*q2J4*q4J1*(2*Eij(3,10)+3*Eij(3,18))+
     &    q1J2*q1J3*q2J1*q4J4*(Eij(3,5)+4*Eij(3,18))+
     &    q1J3*q1J4*q2J2*q4J1*(Eij(3,7)+4*Eij(3,18))+
     &    q1J1*q2J3*q2J4*q4J2*(Eij(3,10)+4*Eij(3,18))-
     &    J3J4*q1J2*q1q4*q3J1*Eij(3,19)-J2J3*q1J4*q1q4*q3J1*Eij(3,19)-
     &    J3J4*q1J2*q2q4*q3J1*Eij(3,19)-J2J3*q1J4*q2q4*q3J1*Eij(3,19)-
     &    J3J4*q1J1*q1q4*q3J2*Eij(3,19)-J3J4*q1J1*q2q4*q3J2*Eij(3,19)-
     &    J1J2*q1J4*q1q4*q3J3*Eij(3,19)-J2J3*q1J1*q1q4*q3J4*Eij(3,19)-
     &    J1J2*q1J3*q1q4*q3J4*Eij(3,19)-J2J3*q1J1*q2q4*q3J4*Eij(3,19)-
     &    J3J4*q1J2*q3J1*q3q4*Eij(3,19)-J3J4*q1J1*q3J2*q3q4*Eij(3,19)-
     &    J2J3*q1J4*q1q3*q4J1*Eij(3,19)-J2J3*q1J4*q2q3*q4J1*Eij(3,19)+
     &    4*q1J3*q1J4*q3J2*q4J1*Eij(3,19)-J3J4*q1sq*q3J2*q4J1*Eij(3,19)-
     &    J2J3*q1sq*q3J4*q4J1*Eij(3,19)-J3J4*q1J2*q3sq*q4J1*Eij(3,19)+
     &    4*q1J3*q1J4*q3J1*q4J2*Eij(3,19)-J3J4*q1sq*q3J1*q4J2*Eij(3,19)-
     &    J3J4*q1J1*q3sq*q4J2*Eij(3,19)-J1J2*q1J4*q1q3*q4J3*Eij(3,19)+
     &    4*q1J2*q1J4*q3J1*q4J3*Eij(3,19)+3*q1J1*q1J4*q3J2*q4J3*Eij(3,19)-
     &    J1J2*q1sq*q3J4*q4J3*Eij(3,19)-J2J3*q1J1*q1q3*q4J4*Eij(3,19)-
     &    J1J2*q1J3*q1q3*q4J4*Eij(3,19)-J2J3*q1J1*q2q3*q4J4*Eij(3,19)-
     &    J2J3*q1sq*q3J1*q4J4*Eij(3,19)-J1J2*q1sq*q3J3*q4J4*Eij(3,19)-
     &    J3J4*q1J2*q1q3*q4J1*(Eij(3,7)+Eij(3,19))-
     &    J3J4*q1J1*q1q3*q4J2*(Eij(3,7)+Eij(3,19))-
     &    J3J4*q1q3*q3J2*q4J1*(Eij(3,13)+Eij(3,19))+
     &    2*q1J3*q3J2*q3J4*q4J1*(Eij(3,13)+Eij(3,19))-
     &    J3J4*q1q3*q3J1*q4J2*(Eij(3,13)+Eij(3,19))+
     &    2*q1J3*q3J1*q3J4*q4J2*(Eij(3,13)+Eij(3,19))+
     &    2*q1J2*q3J1*q3J4*q4J3*(Eij(3,13)+Eij(3,19))+
     &    q1J1*q3J2*q3J3*q4J4*(Eij(3,11)+Eij(3,13)+Eij(3,19))+
     &    q1J4*q3J2*q3J3*q4J1*(2*Eij(3,13)+Eij(3,19))+
     &    q1J4*q3J1*q3J3*q4J2*(2*Eij(3,13)+Eij(3,19))+
     &    q1J2*q3J1*q3J3*q4J4*(Eij(3,11)+2*Eij(3,13)+Eij(3,19))+
     &    q1J1*q3J2*q4J3*q4J4*(Eij(3,16)+Eij(3,19))+
     &    q1J1*q3J3*q4J2*q4J4*(Eij(3,14)+Eij(3,16)+Eij(3,19))+
     &    q1J3*q3J2*q4J1*q4J4*(2*Eij(3,16)+Eij(3,19))+
     &    q1J3*q3J1*q4J2*q4J4*(2*Eij(3,16)+Eij(3,19))+
     &    q1J2*q3J1*q4J3*q4J4*(2*Eij(3,16)+Eij(3,19))+
     &    q1J2*q3J3*q4J1*q4J4*(Eij(3,14)+2*Eij(3,16)+Eij(3,19))-
     &    J3J4*q1J2*q2q3*q4J1*(Eij(3,18)+Eij(3,19))-
     &    J3J4*q1J1*q2q3*q4J2*(Eij(3,18)+Eij(3,19))+
     &    2*q1J2*q1J3*q3J4*q4J1*(Eij(3,7)+2*Eij(3,19))+
     &    q1J1*q3J2*q3J4*q4J3*(Eij(3,13)+2*Eij(3,19))+
     &    q1J1*q1J3*q3J2*q4J4*(Eij(3,6)+3*Eij(3,19))+
     &    q1J1*q1J4*q3J3*q4J2*(Eij(3,7)+3*Eij(3,19))+
     &    q1J1*q1J2*q3J3*q4J4*(Eij(3,6)+Eij(3,7)+3*Eij(3,19))+
     &    q1J1*q1J3*q3J4*q4J2*(2*Eij(3,7)+3*Eij(3,19))+
     &    q1J1*q1J2*q3J4*q4J3*(2*Eij(3,7)+3*Eij(3,19))+
     &    q1J1*q3J3*q3J4*q4J2*(Eij(3,13)+3*Eij(3,19))+
     &    q1J2*q3J3*q3J4*q4J1*(2*Eij(3,13)+3*Eij(3,19))+
     &    q1J2*q1J3*q3J1*q4J4*(Eij(3,6)+4*Eij(3,19))+
     &    q1J2*q1J4*q3J3*q4J1*(Eij(3,7)+4*Eij(3,19))-
     &    J3J4*q1q4*q2J2*q3J1*Eij(3,20)-J2J3*q1q4*q2J4*q3J1*Eij(3,20)-
     &    J3J4*q2J2*q2q4*q3J1*Eij(3,20)-J2J3*q2J4*q2q4*q3J1*Eij(3,20)-
     &    J3J4*q1q4*q2J1*q3J2*Eij(3,20)-J3J4*q2J1*q2q4*q3J2*Eij(3,20)-
     &    J1J2*q1q4*q2J4*q3J3*Eij(3,20)-J2J3*q1q4*q2J1*q3J4*Eij(3,20)-
     &    J1J2*q1q4*q2J3*q3J4*Eij(3,20)-J2J3*q2J1*q2q4*q3J4*Eij(3,20)-
     &    J3J4*q2J2*q3J1*q3q4*Eij(3,20)-J3J4*q2J1*q3J2*q3q4*Eij(3,20)-
     &    J2J3*q1q3*q2J4*q4J1*Eij(3,20)-J2J3*q2J4*q2q3*q4J1*Eij(3,20)+
     &    4*q2J3*q2J4*q3J2*q4J1*Eij(3,20)-J3J4*q2sq*q3J2*q4J1*Eij(3,20)-
     &    J2J3*q2sq*q3J4*q4J1*Eij(3,20)-J3J4*q2J2*q3sq*q4J1*Eij(3,20)+
     &    4*q2J3*q2J4*q3J1*q4J2*Eij(3,20)-J3J4*q2sq*q3J1*q4J2*Eij(3,20)+
     &    3*q2J1*q3J3*q3J4*q4J2*Eij(3,20)-J3J4*q2J1*q3sq*q4J2*Eij(3,20)-
     &    J1J2*q1q3*q2J4*q4J3*Eij(3,20)+3*q2J2*q2J4*q3J1*q4J3*Eij(3,20)+
     &    2*q1J4*q2J1*q3J2*q4J3*Eij(3,20)+2*q2J1*q2J4*q3J2*q4J3*Eij(3,20)-
     &    J1J2*q1q2*q3J4*q4J3*Eij(3,20)+2*q2J1*q3J2*q3J4*q4J3*Eij(3,20)-
     &    J2J3*q1q3*q2J1*q4J4*Eij(3,20)-J1J2*q1q3*q2J3*q4J4*Eij(3,20)-
     &    J2J3*q2J1*q2q3*q4J4*Eij(3,20)-J2J3*q2sq*q3J1*q4J4*Eij(3,20)-
     &    J1J2*q1q2*q3J3*q4J4*Eij(3,20)+q2J1*q3J2*q4J3*q4J4*Eij(3,20)-
     &    J3J4*q2J2*q2q3*q4J1*(Eij(3,10)+Eij(3,20))-
     &    J3J4*q2J1*q2q3*q4J2*(Eij(3,10)+Eij(3,20))+
     &    2*q2J1*q2J3*q3J4*q4J2*(Eij(3,10)+Eij(3,20))+
     &    q2J1*q2J2*q3J3*q4J4*(Eij(3,9)+Eij(3,10)+Eij(3,20))+
     &    q2J1*q2J2*q3J4*q4J3*(2*Eij(3,10)+Eij(3,20))+
     &    q2J1*q3J2*q3J3*q4J4*(Eij(3,12)+Eij(3,20))-
     &    J3J4*q2q3*q3J2*q4J1*(Eij(3,13)+Eij(3,20))+
     &    2*q2J3*q3J2*q3J4*q4J1*(Eij(3,13)+Eij(3,20))-
     &    J3J4*q2q3*q3J1*q4J2*(Eij(3,13)+Eij(3,20))+
     &    2*q2J3*q3J1*q3J4*q4J2*(Eij(3,13)+Eij(3,20))+
     &    q2J2*q3J1*q3J3*q4J4*(Eij(3,12)+Eij(3,13)+Eij(3,20))+
     &    q2J4*q3J2*q3J3*q4J1*(2*Eij(3,13)+Eij(3,20))+
     &    q2J4*q3J1*q3J3*q4J2*(2*Eij(3,13)+Eij(3,20))+
     &    q2J1*q3J3*q4J2*q4J4*(Eij(3,15)+Eij(3,20))+
     &    q2J2*q3J1*q4J3*q4J4*(Eij(3,16)+Eij(3,20))+
     &    q2J2*q3J3*q4J1*q4J4*(Eij(3,15)+Eij(3,16)+Eij(3,20))+
     &    q2J3*q3J2*q4J1*q4J4*(2*Eij(3,16)+Eij(3,20))+
     &    q2J3*q3J1*q4J2*q4J4*(2*Eij(3,16)+Eij(3,20))-
     &    J3J4*q1q3*q2J2*q4J1*(Eij(3,18)+Eij(3,20))-
     &    J3J4*q1q3*q2J1*q4J2*(Eij(3,18)+Eij(3,20))+
     &    2*q1J3*q2J1*q3J4*q4J2*(Eij(3,18)+Eij(3,20))+
     &    2*q1J2*q2J1*q3J4*q4J3*(Eij(3,18)+Eij(3,20))-
     &    J3J4*q1q2*q3J2*q4J1*(Eij(3,19)+Eij(3,20))+
     &    2*q1J4*q2J3*q3J2*q4J1*(Eij(3,19)+Eij(3,20))+
     &    2*q1J3*q2J4*q3J2*q4J1*(Eij(3,19)+Eij(3,20))-
     &    J2J3*q1q2*q3J4*q4J1*(Eij(3,19)+Eij(3,20))-
     &    J3J4*q1q2*q3J1*q4J2*(Eij(3,19)+Eij(3,20))+
     &    2*q1J4*q2J3*q3J1*q4J2*(Eij(3,19)+Eij(3,20))+
     &    2*q1J3*q2J4*q3J1*q4J2*(Eij(3,19)+Eij(3,20))+
     &    2*q1J2*q2J4*q3J1*q4J3*(Eij(3,19)+Eij(3,20))-
     &    J2J3*q1q2*q3J1*q4J4*(Eij(3,19)+Eij(3,20))+
     &    2*q1J2*q2J3*q3J4*q4J1*(Eij(3,18)+Eij(3,19)+Eij(3,20))+
     &    q1J1*q2J2*q3J3*q4J4*(Eij(3,17)+Eij(3,18)+Eij(3,19)+Eij(3,20))+
     &    q1J1*q2J2*q3J4*q4J3*(2*Eij(3,18)+Eij(3,19)+Eij(3,20))+
     &    q1J1*q2J4*q3J2*q4J3*(2*Eij(3,19)+Eij(3,20))+
     &    q1J1*q2J3*q3J2*q4J4*(Eij(3,17)+2*Eij(3,19)+Eij(3,20))+
     &    q1J1*q2J4*q3J3*q4J2*(Eij(3,18)+2*Eij(3,19)+Eij(3,20))+
     &    q1J1*q2J3*q3J4*q4J2*(2*Eij(3,18)+2*Eij(3,19)+Eij(3,20))+
     &    q2J1*q2J3*q3J2*q4J4*(Eij(3,9)+2*Eij(3,20))+
     &    q2J1*q2J4*q3J3*q4J2*(Eij(3,10)+2*Eij(3,20))+
     &    q2J2*q3J1*q3J4*q4J3*(Eij(3,13)+2*Eij(3,20))+
     &    q1J3*q2J1*q3J2*q4J4*(Eij(3,17)+2*Eij(3,20))+
     &    q1J4*q2J1*q3J3*q4J2*(Eij(3,18)+2*Eij(3,20))+
     &    q1J2*q2J1*q3J3*q4J4*(Eij(3,17)+Eij(3,18)+2*Eij(3,20))+
     &    q1J4*q2J2*q3J1*q4J3*(Eij(3,19)+2*Eij(3,20))+
     &    q1J3*q2J2*q3J1*q4J4*(Eij(3,17)+Eij(3,19)+2*Eij(3,20))+
     &    q1J4*q2J2*q3J3*q4J1*(Eij(3,18)+Eij(3,19)+2*Eij(3,20))+
     &    q1J3*q2J2*q3J4*q4J1*(2*Eij(3,18)+Eij(3,19)+2*Eij(3,20))+
     &    q2J2*q2J3*q3J1*q4J4*(Eij(3,9)+3*Eij(3,20))+
     &    q2J2*q2J4*q3J3*q4J1*(Eij(3,10)+3*Eij(3,20))+
     &    q2J2*q2J3*q3J4*q4J1*(2*Eij(3,10)+3*Eij(3,20))+
     &    q2J2*q3J3*q3J4*q4J1*(Eij(3,13)+3*Eij(3,20))+
     &    q1J2*q2J3*q3J1*q4J4*(Eij(3,17)+2*(Eij(3,19)+Eij(3,20)))+
     &    q1J2*q2J4*q3J3*q4J1*(Eij(3,18)+2*(Eij(3,19)+Eij(3,20)))+
     &    J3J4*q1J1*q1J2*Eij(3,21)+3*J2J4*q1J1*q1J3*Eij(3,21)+
     &    4*J1J4*q1J2*q1J3*Eij(3,21)+J2J3*q1J1*q1J4*Eij(3,21)+
     &    4*J1J3*q1J2*q1J4*Eij(3,21)+2*J1J2*q1J3*q1J4*Eij(3,21)-
     &    J1J4*J2J3*q1sq*Eij(3,21)-2*J1J2*J3J4*q1sq*Eij(3,21)-
     &    J3J4*q1J2*q2J1*Eij(3,21)-J2J3*q1J4*q2J1*Eij(3,21)-
     &    J3J4*q1J1*q2J2*Eij(3,21)-J2J3*q1J1*q2J4*Eij(3,21)-
     &    J3J4*q1J2*q3J1*Eij(3,21)-J3J4*q1J1*q3J2*Eij(3,21)+
     &    J3J4*q1J2*q2J1*Eij(3,22)+2*J2J4*q1J3*q2J1*Eij(3,22)+
     &    J2J3*q1J4*q2J1*Eij(3,22)-J3J4*q1J1*q2J2*Eij(3,22)-
     &    J3J4*q2J1*q2J2*Eij(3,22)-J1J2*q1J4*q2J3*Eij(3,22)+
     &    2*J2J4*q2J1*q2J3*Eij(3,22)+3*J1J4*q2J2*q2J3*Eij(3,22)-
     &    J2J3*q1J1*q2J4*Eij(3,22)-J1J2*q1J3*q2J4*Eij(3,22)+
     &    3*J1J3*q2J2*q2J4*Eij(3,22)+4*J1J2*q2J3*q2J4*Eij(3,22)-
     &    J1J4*J2J3*q2sq*Eij(3,22)-J1J2*J3J4*q2sq*Eij(3,22)-
     &    J3J4*q2J2*q3J1*Eij(3,22)-J3J4*q2J1*q3J2*Eij(3,22)+
     &    J2J4*q2J1*q3J3*Eij(3,22)+2*J2J3*q2J1*q3J4*Eij(3,22)+
     &    J2J3*q2J1*q4J4*Eij(3,22)-J1J4*J2J3*q1q2*(Eij(3,21)+Eij(3,22))+
     &    J3J4*q1J1*q2J2*(Eij(3,21)+Eij(3,22))+
     &    2*J1J4*q1J2*q2J3*(Eij(3,21)+Eij(3,22))+
     &    2*J1J2*q1J4*q2J3*(Eij(3,21)+Eij(3,22))+
     &    2*J1J3*q1J2*q2J4*(Eij(3,21)+Eij(3,22))+
     &    2*J1J2*q1J3*q2J4*(Eij(3,21)+Eij(3,22))+
     &    J2J4*q1J1*q2J3*(2*Eij(3,21)+Eij(3,22))+
     &    J2J3*q1J1*q2J4*(2*Eij(3,21)+Eij(3,22))-
     &    J1J2*J3J4*q1q2*(Eij(3,21)+2*Eij(3,22))+
     &    J1J4*q1J3*q2J2*(Eij(3,21)+2*Eij(3,22))+
     &    J1J3*q1J4*q2J2*(Eij(3,21)+2*Eij(3,22))-J1J4*J2J3*q1q3*Eij(3,23)-
     &    J1J4*J2J3*q2q3*Eij(3,23)+J3J4*q1J2*q3J1*Eij(3,23)+
     &    2*J2J4*q1J3*q3J1*Eij(3,23)+J2J3*q1J4*q3J1*Eij(3,23)+
     &    2*J2J4*q2J3*q3J1*Eij(3,23)+J2J3*q2J4*q3J1*Eij(3,23)+
     &    2*J1J4*q1J3*q3J2*Eij(3,23)+2*J1J3*q1J4*q3J2*Eij(3,23)-
     &    J3J4*q2J1*q3J2*Eij(3,23)+2*J1J4*q2J3*q3J2*Eij(3,23)+
     &    2*J1J3*q2J4*q3J2*Eij(3,23)-2*J3J4*q3J1*q3J2*Eij(3,23)-
     &    J1J2*q1J4*q3J3*Eij(3,23)+J2J4*q3J1*q3J3*Eij(3,23)+
     &    J1J4*q3J2*q3J3*Eij(3,23)-J2J3*q1J1*q3J4*Eij(3,23)-
     &    J1J2*q1J3*q3J4*Eij(3,23)-J2J3*q2J1*q3J4*Eij(3,23)+
     &    2*J2J3*q3J1*q3J4*Eij(3,23)+2*J1J3*q3J2*q3J4*Eij(3,23)+
     &    3*J1J2*q3J3*q3J4*Eij(3,23)-J1J2*J3J4*q3sq*Eij(3,23)+
     &    J2J3*q3J1*q4J4*Eij(3,23)+J1J3*q3J2*q4J4*Eij(3,23)+
     &    J2J4*q1J1*q3J3*(Eij(3,21)+Eij(3,23))+
     &    2*J1J3*q1J2*q3J4*(Eij(3,21)+Eij(3,23))+
     &    2*J1J2*q1J3*q3J4*(Eij(3,21)+Eij(3,23))+
     &    J2J3*q1J1*q3J4*(2*Eij(3,21)+Eij(3,23))-
     &    J1J2*J3J4*q2q3*(Eij(3,22)+Eij(3,23))+
     &    J1J4*q2J2*q3J3*(Eij(3,22)+Eij(3,23))+
     &    2*J1J2*q2J3*q3J4*(Eij(3,22)+Eij(3,23))+
     &    J1J3*q2J2*q3J4*(2*Eij(3,22)+Eij(3,23))-
     &    J1J2*J3J4*q1q3*(Eij(3,21)+2*Eij(3,23))+
     &    J1J4*q1J2*q3J3*(Eij(3,21)+2*Eij(3,23))+
     &    J1J2*q1J4*q3J3*(Eij(3,21)+2*Eij(3,23))+
     &    J1J2*q2J4*q3J3*(Eij(3,22)+2*Eij(3,23))-J1J4*J2J3*q1q4*Eij(3,24)-
     &    2*J1J2*J3J4*q1q4*Eij(3,24)-J1J4*J2J3*q2q4*Eij(3,24)-
     &    J1J2*J3J4*q2q4*Eij(3,24)-J1J2*J3J4*q3q4*Eij(3,24)+
     &    J3J4*q1J2*q4J1*Eij(3,24)+2*J2J4*q1J3*q4J1*Eij(3,24)+
     &    J2J3*q1J4*q4J1*Eij(3,24)+2*J2J4*q2J3*q4J1*Eij(3,24)+
     &    J2J3*q2J4*q4J1*Eij(3,24)-J3J4*q3J2*q4J1*Eij(3,24)+
     &    J2J4*q3J3*q4J1*Eij(3,24)+2*J2J3*q3J4*q4J1*Eij(3,24)+
     &    2*J1J4*q1J3*q4J2*Eij(3,24)+2*J1J3*q1J4*q4J2*Eij(3,24)-
     &    J3J4*q2J1*q4J2*Eij(3,24)+2*J1J4*q2J3*q4J2*Eij(3,24)+
     &    2*J1J3*q2J4*q4J2*Eij(3,24)-J3J4*q3J1*q4J2*Eij(3,24)+
     &    J1J4*q3J3*q4J2*Eij(3,24)+2*J1J3*q3J4*q4J2*Eij(3,24)+
     &    J2J4*q1J1*q4J3*Eij(3,24)+2*J1J4*q1J2*q4J3*Eij(3,24)+
     &    J1J2*q1J4*q4J3*Eij(3,24)+J1J4*q2J2*q4J3*Eij(3,24)+
     &    2*J1J2*q2J4*q4J3*Eij(3,24)+2*J1J2*q3J4*q4J3*Eij(3,24)-
     &    J2J3*q1J1*q4J4*Eij(3,24)-J1J2*q1J3*q4J4*Eij(3,24)-
     &    J2J3*q2J1*q4J4*Eij(3,24)+J2J3*q4J1*q4J4*Eij(3,24)+
     &    J1J3*q4J2*q4J4*Eij(3,24)+J1J2*q4J3*q4J4*Eij(3,24)+
     &    J2J3*q1J1*q4J4*(Eij(3,21)+Eij(3,24))+
     &    J1J3*q2J2*q4J4*(Eij(3,22)+Eij(3,24))+
     &    J1J2*q3J3*q4J4*(Eij(3,23)+Eij(3,24))+
     &    J1J3*q1J2*q4J4*(Eij(3,21)+2*Eij(3,24))+
     &    J1J2*q1J3*q4J4*(Eij(3,21)+2*Eij(3,24))+
     &    J1J2*q2J3*q4J4*(Eij(3,22)+2*Eij(3,24)))


      HEmunu=16*m*(18*q1J1*q1J2*q1J3*q1J4*Eij(2,1)-
     &    2*J3J4*q1J1*q1J2*q1q2*Eij(2,1)-2*J2J3*q1J1*q1J4*q1q2*Eij(2,1)-
     &    J1J2*q1J3*q1J4*q1q2*Eij(2,1)-2*J3J4*q1J1*q1J2*q1q3*Eij(2,1)-
     &    J1J4*q1J2*q1J3*q1q4*Eij(2,1)-4*J3J4*q1J1*q1J2*q1sq*Eij(2,1)-
     &    J1J4*q1J2*q1J3*q1sq*Eij(2,1)-4*J2J3*q1J1*q1J4*q1sq*Eij(2,1)-
     &    4*J1J2*q1J3*q1J4*q1sq*Eij(2,1)+J1J4*J2J3*q1sq**2*Eij(2,1)-
     &    J1J3*J2J4*q1sq**2*Eij(2,1)+J1J2*J3J4*q1sq**2*Eij(2,1)-
     &    J2J4*q1J1*q1sq*q2J3*Eij(2,1)-2*J3J4*q1J1*q1J2*q2q3*Eij(2,1)-
     &    J2J3*q1J1*q1J4*q2q3*Eij(2,1)-J2J4*q1J1*q1J3*q2q4*Eij(2,1)-
     &    J3J4*q1J1*q1J2*q2sq*Eij(2,1)-J2J3*q1J1*q1J4*q2sq*Eij(2,1)-
     &    J3J4*q1J1*q1J2*q3q4*Eij(2,1)-J3J4*q1J1*q1J2*q3sq*Eij(2,1)-
     &    J2J4*q1J3*q1sq*q4J1*Eij(2,1)+J2J4*q1J1*q1sq*q4J3*Eij(2,1)-
     &    2*J3J4*q1q2*q2J1*q2J2*Eij(2,2)-2*J3J4*q1q3*q2J1*q2J2*Eij(2,2)-
     &    J3J4*q1sq*q2J1*q2J2*Eij(2,2)-J1J4*q1q4*q2J2*q2J3*Eij(2,2)-
     &    2*J2J3*q1q2*q2J1*q2J4*Eij(2,2)-J2J3*q1sq*q2J1*q2J4*Eij(2,2)-
     &    J1J3*q1q3*q2J2*q2J4*Eij(2,2)-5*J1J2*q1q2*q2J3*q2J4*Eij(2,2)-
     &    J1J2*q1sq*q2J3*q2J4*Eij(2,2)+8*q2J1*q2J2*q2J3*q2J4*Eij(2,2)-
     &    2*J3J4*q2J1*q2J2*q2q3*Eij(2,2)-J2J3*q2J1*q2J4*q2q3*Eij(2,2)-
     &    J2J4*q2J1*q2J3*q2q4*Eij(2,2)+J1J4*J2J3*q1q2*q2sq*Eij(2,2)-
     &    J1J3*J2J4*q1q2*q2sq*Eij(2,2)+J1J2*J3J4*q1q2*q2sq*Eij(2,2)-
     &    2*J3J4*q1J2*q2J1*q2sq*Eij(2,2)-2*J2J3*q1J4*q2J1*q2sq*Eij(2,2)-
     &    J1J4*q1J3*q2J2*q2sq*Eij(2,2)+J1J3*q1J4*q2J2*q2sq*Eij(2,2)-
     &    2*J3J4*q2J1*q2J2*q2sq*Eij(2,2)-J2J4*q2J1*q2J3*q2sq*Eij(2,2)-
     &    J1J3*q1J2*q2J4*q2sq*Eij(2,2)+J1J2*q1J3*q2J4*q2sq*Eij(2,2)-
     &    2*J2J3*q2J1*q2J4*q2sq*Eij(2,2)+J1J3*q1q2*q2J4*q3J2*Eij(2,2)-
     &    2*J2J3*q2J1*q2sq*q3J4*Eij(2,2)-J3J4*q2J1*q2J2*q3q4*Eij(2,2)-
     &    J3J4*q2J1*q2J2*q3sq*Eij(2,2)+J1J4*q1q2*q2J3*q4J2*Eij(2,2)-
     &    J3J4*q2J1*q2sq*q4J2*Eij(2,2)+J2J4*q2J1*q2sq*q4J3*Eij(2,2)-
     &    J2J3*q2J1*q2sq*q4J4*Eij(2,2)-2*J2J3*q1J4*q2q3*q3J1*Eij(2,3)-
     &    2*J3J4*q1q2*q3J1*q3J2*Eij(2,3)-2*J3J4*q1q3*q3J1*q3J2*Eij(2,3)-
     &    J3J4*q1sq*q3J1*q3J2*Eij(2,3)-2*J3J4*q2q3*q3J1*q3J2*Eij(2,3)-
     &    J3J4*q2sq*q3J1*q3J2*Eij(2,3)-J2J4*q2q4*q3J1*q3J3*Eij(2,3)-
     &    J1J4*q1q4*q3J2*q3J3*Eij(2,3)+2*q1J4*q3J1*q3J2*q3J3*Eij(2,3)+
     &    2*q2J4*q3J1*q3J2*q3J3*Eij(2,3)-J1J3*q1J2*q1q3*q3J4*Eij(2,3)-
     &    J1J3*q1J2*q2q3*q3J4*Eij(2,3)+J1J2*q1J3*q2q3*q3J4*Eij(2,3)-
     &    2*J2J3*q1q2*q3J1*q3J4*Eij(2,3)-3*J2J3*q1q3*q3J1*q3J4*Eij(2,3)-
     &    J2J3*q1sq*q3J1*q3J4*Eij(2,3)-3*J2J3*q2q3*q3J1*q3J4*Eij(2,3)-
     &    J2J3*q2sq*q3J1*q3J4*Eij(2,3)+5*q1J3*q3J1*q3J2*q3J4*Eij(2,3)+
     &    5*q2J3*q3J1*q3J2*q3J4*Eij(2,3)-J1J2*q1q2*q3J3*q3J4*Eij(2,3)-
     &    3*J1J2*q1q3*q3J3*q3J4*Eij(2,3)-J1J2*q1sq*q3J3*q3J4*Eij(2,3)+
     &    2*q3J1*q3J2*q3J3*q3J4*Eij(2,3)-J3J4*q3J1*q3J2*q3q4*Eij(2,3)+
     &    J1J4*J2J3*q1q3*q3sq*Eij(2,3)-J1J3*J2J4*q1q3*q3sq*Eij(2,3)+
     &    J1J2*J3J4*q1q3*q3sq*Eij(2,3)-2*J3J4*q1J2*q3J1*q3sq*Eij(2,3)-
     &    J3J4*q2J2*q3J1*q3sq*Eij(2,3)-J2J4*q2J3*q3J1*q3sq*Eij(2,3)+
     &    J2J3*q2J4*q3J1*q3sq*Eij(2,3)-J3J4*q1J1*q3J2*q3sq*Eij(2,3)-
     &    J1J4*q1J3*q3J2*q3sq*Eij(2,3)+J1J3*q1J4*q3J2*q3sq*Eij(2,3)-
     &    J3J4*q3J1*q3J2*q3sq*Eij(2,3)-J2J4*q1q3*q3J3*q4J1*Eij(2,3)-
     &    J3J4*q1q3*q3J1*q4J2*Eij(2,3)-J3J4*q2q3*q3J1*q4J2*Eij(2,3)+
     &    J1J4*q1q3*q3J3*q4J2*Eij(2,3)+J2J4*q1q3*q3J1*q4J3*Eij(2,3)+
     &    J2J4*q2q3*q3J1*q4J3*Eij(2,3)+q3J1*q3J2*q3J4*q4J3*Eij(2,3)-
     &    J2J3*q2q3*q3J1*q4J4*Eij(2,3)+q3J1*q3J2*q3J3*q4J4*Eij(2,3)+
     &    J1J4*J2J3*q1q4**2*Eij(2,4)-J1J3*J2J4*q1q4**2*Eij(2,4)+
     &    J1J2*J3J4*q1q4**2*Eij(2,4)+J1J4*J2J3*q1q4*q2q4*Eij(2,4)-
     &    J1J3*J2J4*q1q4*q2q4*Eij(2,4)+J1J2*J3J4*q1q4*q2q4*Eij(2,4)+
     &    J1J4*J2J3*q1q4*q3q4*Eij(2,4)-J1J3*J2J4*q1q4*q3q4*Eij(2,4)+
     &    J1J2*J3J4*q1q4*q3q4*Eij(2,4)-2*J3J4*q1J2*q1q4*q4J1*Eij(2,4)-
     &    2*J2J3*q1J4*q1q4*q4J1*Eij(2,4)-J3J4*q1q4*q2J2*q4J1*Eij(2,4)-
     &    J2J3*q1q4*q2J4*q4J1*Eij(2,4)-2*J3J4*q1J2*q2q4*q4J1*Eij(2,4)-
     &    2*J2J3*q1J4*q2q4*q4J1*Eij(2,4)-J3J4*q2J2*q2q4*q4J1*Eij(2,4)-
     &    J2J4*q2J3*q2q4*q4J1*Eij(2,4)-J2J3*q2J4*q2q4*q4J1*Eij(2,4)-
     &    2*J2J3*q1q4*q3J4*q4J1*Eij(2,4)-2*J2J3*q2q4*q3J4*q4J1*Eij(2,4)-
     &    2*J3J4*q1J2*q3q4*q4J1*Eij(2,4)-J3J4*q2J2*q3q4*q4J1*Eij(2,4)-
     &    J2J4*q2J3*q3q4*q4J1*Eij(2,4)+J2J3*q2J4*q3q4*q4J1*Eij(2,4)+
     &    J1J3*q1J4*q1q4*q4J2*Eij(2,4)-J1J4*q1J3*q2q4*q4J2*Eij(2,4)+
     &    J1J3*q1J4*q2q4*q4J2*Eij(2,4)-J3J4*q1J1*q3q4*q4J2*Eij(2,4)-
     &    J1J4*q1J3*q3q4*q4J2*Eij(2,4)+J1J3*q1J4*q3q4*q4J2*Eij(2,4)-
     &    2*J3J4*q1q2*q4J1*q4J2*Eij(2,4)-2*J3J4*q1q3*q4J1*q4J2*Eij(2,4)-
     &    J3J4*q1q4*q4J1*q4J2*Eij(2,4)-J3J4*q1sq*q4J1*q4J2*Eij(2,4)-
     &    2*J3J4*q2q3*q4J1*q4J2*Eij(2,4)-J3J4*q2q4*q4J1*q4J2*Eij(2,4)-
     &    J3J4*q2sq*q4J1*q4J2*Eij(2,4)+4*q1J3*q3J4*q4J1*q4J2*Eij(2,4)+
     &    4*q2J3*q3J4*q4J1*q4J2*Eij(2,4)+2*q3J3*q3J4*q4J1*q4J2*Eij(2,4)-
     &    J3J4*q3q4*q4J1*q4J2*Eij(2,4)-J3J4*q3sq*q4J1*q4J2*Eij(2,4)-
     &    2*J1J2*q1J4*q1q4*q4J3*Eij(2,4)-2*J1J2*q1q4*q2J4*q4J3*Eij(2,4)-
     &    2*J1J2*q1q4*q3J4*q4J3*Eij(2,4)+4*q1J2*q2J4*q4J1*q4J3*Eij(2,4)+
     &    2*q2J2*q2J4*q4J1*q4J3*Eij(2,4)+2*q1J1*q1J4*q4J2*q4J3*Eij(2,4)+
     &    q1J4*q4J1*q4J2*q4J3*Eij(2,4)+q2J4*q4J1*q4J2*q4J3*Eij(2,4)+
     &    q3J4*q4J1*q4J2*q4J3*Eij(2,4)-J1J3*q1J2*q1q4*q4J4*Eij(2,4)-
     &    J1J3*q1J2*q2q4*q4J4*Eij(2,4)+J1J2*q1J3*q2q4*q4J4*Eij(2,4)+
     &    J1J3*q1q4*q3J2*q4J4*Eij(2,4)-2*J2J3*q1q2*q4J1*q4J4*Eij(2,4)-
     &    J2J3*q1q4*q4J1*q4J4*Eij(2,4)-J2J3*q1sq*q4J1*q4J4*Eij(2,4)-
     &    J2J3*q2q3*q4J1*q4J4*Eij(2,4)-J2J3*q2q4*q4J1*q4J4*Eij(2,4)-
     &    J2J3*q2sq*q4J1*q4J4*Eij(2,4)-J1J3*q1q3*q4J2*q4J4*Eij(2,4)+
     &    2*q1J3*q4J1*q4J2*q4J4*Eij(2,4)+2*q2J3*q4J1*q4J2*q4J4*Eij(2,4)+
     &    q3J3*q4J1*q4J2*q4J4*Eij(2,4)-J1J2*q1q2*q4J3*q4J4*Eij(2,4)-
     &    J1J2*q1q4*q4J3*q4J4*Eij(2,4)-J1J2*q1sq*q4J3*q4J4*Eij(2,4)+
     &    2*q1J2*q4J1*q4J3*q4J4*Eij(2,4)+q2J2*q4J1*q4J3*q4J4*Eij(2,4)+
     &    q1J1*q4J2*q4J3*q4J4*Eij(2,4)+
     &    J1J2*q1J3*q1J4*q1q2*(Eij(2,1)-3*Eij(2,5))+
     &    J1J4*J2J3*q1q2*q1sq*Eij(2,5)-J1J3*J2J4*q1q2*q1sq*Eij(2,5)+
     &    J1J2*J3J4*q1q2*q1sq*Eij(2,5)-2*J3J4*q1J2*q1q2*q2J1*Eij(2,5)-
     &    2*J2J3*q1J4*q1q2*q2J1*Eij(2,5)-2*J3J4*q1J2*q1q3*q2J1*Eij(2,5)-
     &    3*J3J4*q1J2*q1sq*q2J1*Eij(2,5)-3*J2J3*q1J4*q1sq*q2J1*Eij(2,5)-
     &    2*J3J4*q1J1*q1q2*q2J2*Eij(2,5)-2*J3J4*q1J1*q1q3*q2J2*Eij(2,5)-
     &    J1J3*q1J4*q1q3*q2J2*Eij(2,5)-J1J4*q1J3*q1q4*q2J2*Eij(2,5)-
     &    J3J4*q1J1*q1sq*q2J2*Eij(2,5)-J1J4*q1J3*q1sq*q2J2*Eij(2,5)+
     &    J1J3*q1J4*q1sq*q2J2*Eij(2,5)-J3J4*q1sq*q2J1*q2J2*Eij(2,5)-
     &    J1J2*q1J4*q1q2*q2J3*Eij(2,5)-J1J4*q1J2*q1q4*q2J3*Eij(2,5)-
     &    J1J2*q1J4*q1sq*q2J3*Eij(2,5)-J2J4*q1sq*q2J1*q2J3*Eij(2,5)-
     &    2*J2J3*q1J1*q1q2*q2J4*Eij(2,5)-J1J2*q1J3*q1q2*q2J4*Eij(2,5)-
     &    J1J3*q1J2*q1q3*q2J4*Eij(2,5)-J2J3*q1J1*q1sq*q2J4*Eij(2,5)-
     &    J1J3*q1J2*q1sq*q2J4*Eij(2,5)-J1J2*q1J3*q1sq*q2J4*Eij(2,5)-
     &    J2J3*q1sq*q2J1*q2J4*Eij(2,5)-4*J1J2*q1sq*q2J3*q2J4*Eij(2,5)+
     &    J1J3*q1J2*q1J4*q2q3*Eij(2,5)-2*J3J4*q1J2*q2J1*q2q3*Eij(2,5)-
     &    J2J3*q1J4*q2J1*q2q3*Eij(2,5)-2*J3J4*q1J1*q2J2*q2q3*Eij(2,5)-
     &    J2J3*q1J1*q2J4*q2q3*Eij(2,5)-J2J4*q1J3*q2J1*q2q4*Eij(2,5)-
     &    J2J4*q1J1*q2J3*q2q4*Eij(2,5)-3*J3J4*q1J1*q1J2*q2sq*Eij(2,5)-
     &    J1J4*q1J2*q1J3*q2sq*Eij(2,5)-3*J2J3*q1J1*q1J4*q2sq*Eij(2,5)+
     &    J1J2*q1J3*q1J4*q2sq*Eij(2,5)+J1J4*J2J3*q1sq*q2sq*Eij(2,5)-
     &    J1J3*J2J4*q1sq*q2sq*Eij(2,5)+J1J2*J3J4*q1sq*q2sq*Eij(2,5)-
     &    J3J4*q1J2*q2J1*q2sq*Eij(2,5)-J2J3*q1J4*q2J1*q2sq*Eij(2,5)-
     &    J3J4*q1J1*q2J2*q2sq*Eij(2,5)-J2J4*q1J1*q2J3*q2sq*Eij(2,5)-
     &    J2J3*q1J1*q2J4*q2sq*Eij(2,5)+J1J3*q1sq*q2J4*q3J2*Eij(2,5)-
     &    2*J2J3*q1sq*q2J1*q3J4*Eij(2,5)-J3J4*q1J2*q2J1*q3q4*Eij(2,5)-
     &    J3J4*q1J1*q2J2*q3q4*Eij(2,5)-J3J4*q1J2*q2J1*q3sq*Eij(2,5)-
     &    J3J4*q1J1*q2J2*q3sq*Eij(2,5)-J2J4*q1J3*q1q2*q4J1*Eij(2,5)-
     &    J3J4*q1sq*q2J1*q4J2*Eij(2,5)+J1J4*q1sq*q2J3*q4J2*Eij(2,5)+
     &    J2J4*q1sq*q2J1*q4J3*Eij(2,5)+J2J4*q1J1*q2sq*q4J3*Eij(2,5)-
     &    J2J3*q1sq*q2J1*q4J4*Eij(2,5)-
     &    3*J3J4*q1J1*q1J2*q1q2*(Eij(2,1)+Eij(2,5))-
     &    J1J4*q1J2*q1J3*q1q2*(Eij(2,1)+Eij(2,5))-
     &    3*J2J3*q1J1*q1J4*q1q2*(Eij(2,1)+Eij(2,5))+
     &    J1J4*J2J3*q1q2*q1sq*(Eij(2,1)+Eij(2,5))-
     &    J1J3*J2J4*q1q2*q1sq*(Eij(2,1)+Eij(2,5))+
     &    J1J2*J3J4*q1q2*q1sq*(Eij(2,1)+Eij(2,5))-
     &    J3J4*q1J1*q1sq*q2J2*(Eij(2,1)+Eij(2,5))-
     &    J2J4*q1J1*q1q2*q2J3*(Eij(2,1)+Eij(2,5))-
     &    2*J1J2*q1J4*q1sq*q2J3*(Eij(2,1)+Eij(2,5))-
     &    J2J3*q1J1*q1sq*q2J4*(Eij(2,1)+Eij(2,5))+
     &    J2J4*q1J1*q1q2*q4J3*(Eij(2,1)+Eij(2,5))-
     &    J1J2*q1J3*q1sq*q2J4*(2*Eij(2,1)+Eij(2,5))+
     &    J1J4*J2J3*q1q2**2*(Eij(2,2)+Eij(2,5))-
     &    J1J3*J2J4*q1q2**2*(Eij(2,2)+Eij(2,5))+
     &    J1J2*J3J4*q1q2**2*(Eij(2,2)+Eij(2,5))-
     &    2*J3J4*q1J2*q1q2*q2J1*(Eij(2,2)+Eij(2,5))-
     &    2*J2J3*q1J4*q1q2*q2J1*(Eij(2,2)+Eij(2,5))-
     &    J1J4*q1J3*q1q2*q2J2*(Eij(2,2)+Eij(2,5))+
     &    J1J3*q1J4*q1q2*q2J2*(Eij(2,2)+Eij(2,5))+
     &    4*q1J3*q1J4*q2J1*q2J2*(Eij(2,2)+Eij(2,5))-
     &    J3J4*q1q2*q2J1*q2J2*(Eij(2,2)+Eij(2,5))-
     &    2*J1J2*q1J4*q1q2*q2J3*(Eij(2,2)+Eij(2,5))-
     &    J2J4*q1q2*q2J1*q2J3*(Eij(2,2)+Eij(2,5))-
     &    J1J3*q1J2*q1q2*q2J4*(Eij(2,2)+Eij(2,5))-
     &    J1J2*q1J3*q1q2*q2J4*(Eij(2,2)+Eij(2,5))-
     &    J2J3*q1q2*q2J1*q2J4*(Eij(2,2)+Eij(2,5))-
     &    J3J4*q1J1*q2J2*q2sq*(Eij(2,2)+Eij(2,5))-
     &    J2J3*q1J1*q2J4*q2sq*(Eij(2,2)+Eij(2,5))-
     &    2*J2J3*q1q2*q2J1*q3J4*(Eij(2,2)+Eij(2,5))-
     &    J3J4*q1q2*q2J1*q4J2*(Eij(2,2)+Eij(2,5))+
     &    J2J4*q1q2*q2J1*q4J3*(Eij(2,2)+Eij(2,5))-
     &    J2J3*q1q2*q2J1*q4J4*(Eij(2,2)+Eij(2,5))+
     &    2*q1J4*q2J1*q2J2*q2J3*(3*Eij(2,2)+Eij(2,5))+
     &    2*q1J3*q2J1*q2J2*q2J4*(3*Eij(2,2)+Eij(2,5))-
     &    J3J4*q1J1*q1q2*q2J2*(Eij(2,1)+Eij(2,2)+2*Eij(2,5))-
     &    J2J3*q1J1*q1q2*q2J4*(Eij(2,1)+Eij(2,2)+2*Eij(2,5))+
     &    2*q1J1*q1J2*q1J4*q2J3*(5*Eij(2,1)+4*Eij(2,5))+
     &    2*q1J1*q1J2*q1J3*q2J4*(5*Eij(2,1)+4*Eij(2,5))+
     &    q1J2*q2J1*q2J3*q2J4*(9*Eij(2,2)+4*Eij(2,5))+
     &    2*q1J1*q1J2*q2J3*q2J4*(2*Eij(2,1)+Eij(2,2)+6*Eij(2,5))+
     &    q1J1*q1J3*q1J4*q2J2*(5*Eij(2,1)+8*Eij(2,5))+
     &    q1J1*q2J2*q2J3*q2J4*(5*Eij(2,2)+8*Eij(2,5))+
     &    q1J1*q1J4*q2J2*q2J3*(2*Eij(2,1)+2*Eij(2,2)+9*Eij(2,5))+
     &    q1J1*q1J3*q2J2*q2J4*(2*Eij(2,1)+2*Eij(2,2)+9*Eij(2,5))+
     &    q1J2*q1J4*q2J1*q2J3*(4*Eij(2,2)+9*Eij(2,5))+
     &    q1J2*q1J3*q2J1*q2J4*(4*Eij(2,2)+9*Eij(2,5))+
     &    q1J2*q1J3*q1J4*q2J1*(Eij(2,1)+12*Eij(2,5))-
     &    3*J2J3*q1J1*q1J4*q1q3*Eij(2,6)-3*J1J2*q1J3*q1J4*q1q3*Eij(2,6)+
     &    J1J4*J2J3*q1q3*q1sq*Eij(2,6)-J1J3*J2J4*q1q3*q1sq*Eij(2,6)+
     &    J1J2*J3J4*q1q3*q1sq*Eij(2,6)-3*J2J3*q1J1*q1J4*q2q3*Eij(2,6)+
     &    J1J2*q1J3*q1J4*q2q3*Eij(2,6)-2*J3J4*q1J2*q1q2*q3J1*Eij(2,6)-
     &    2*J2J3*q1J4*q1q2*q3J1*Eij(2,6)-2*J3J4*q1J2*q1q3*q3J1*Eij(2,6)-
     &    3*J3J4*q1J2*q1sq*q3J1*Eij(2,6)-J2J3*q1J4*q1sq*q3J1*Eij(2,6)-
     &    J3J4*q1sq*q2J2*q3J1*Eij(2,6)-J2J4*q1sq*q2J3*q3J1*Eij(2,6)-
     &    2*J3J4*q1J2*q2q3*q3J1*Eij(2,6)-J2J3*q1J4*q2q3*q3J1*Eij(2,6)-
     &    J2J4*q1J3*q2q4*q3J1*Eij(2,6)-J3J4*q1J2*q2sq*q3J1*Eij(2,6)-
     &    J2J3*q1J4*q2sq*q3J1*Eij(2,6)+8*q1J1*q1J3*q1J4*q3J2*Eij(2,6)-
     &    2*J3J4*q1J1*q1q2*q3J2*Eij(2,6)-2*J3J4*q1J1*q1q3*q3J2*Eij(2,6)-
     &    J1J3*q1J4*q1q3*q3J2*Eij(2,6)-J1J4*q1J3*q1q4*q3J2*Eij(2,6)-
     &    2*J3J4*q1J1*q1sq*q3J2*Eij(2,6)-J1J4*q1J3*q1sq*q3J2*Eij(2,6)-
     &    2*J3J4*q1J1*q2q3*q3J2*Eij(2,6)-J3J4*q1J1*q2sq*q3J2*Eij(2,6)-
     &    J1J2*q1J4*q1q2*q3J3*Eij(2,6)-J1J4*q1J2*q1q4*q3J3*Eij(2,6)-
     &    J1J2*q1J4*q1sq*q3J3*Eij(2,6)-J2J4*q1J1*q2q4*q3J3*Eij(2,6)-
     &    2*J2J3*q1J1*q1q2*q3J4*Eij(2,6)-J1J2*q1J3*q1q2*q3J4*Eij(2,6)-
     &    J1J3*q1J2*q1q3*q3J4*Eij(2,6)-J2J3*q1J1*q1sq*q3J4*Eij(2,6)-
     &    J1J3*q1J2*q1sq*q3J4*Eij(2,6)-J1J2*q1J3*q1sq*q3J4*Eij(2,6)-
     &    J2J3*q1J1*q2q3*q3J4*Eij(2,6)-J2J3*q1J1*q2sq*q3J4*Eij(2,6)-
     &    3*J2J3*q1sq*q3J1*q3J4*Eij(2,6)+J1J3*q1sq*q3J2*q3J4*Eij(2,6)-
     &    3*J1J2*q1sq*q3J3*q3J4*Eij(2,6)-J3J4*q1J2*q3J1*q3q4*Eij(2,6)-
     &    J3J4*q1J1*q3J2*q3q4*Eij(2,6)-3*J3J4*q1J1*q1J2*q3sq*Eij(2,6)-
     &    J1J4*q1J2*q1J3*q3sq*Eij(2,6)+J1J3*q1J2*q1J4*q3sq*Eij(2,6)+
     &    J1J4*J2J3*q1sq*q3sq*Eij(2,6)-J1J3*J2J4*q1sq*q3sq*Eij(2,6)+
     &    J1J2*J3J4*q1sq*q3sq*Eij(2,6)-J2J4*q1J1*q2J3*q3sq*Eij(2,6)+
     &    J2J3*q1J1*q2J4*q3sq*Eij(2,6)-J3J4*q1J2*q3J1*q3sq*Eij(2,6)-
     &    J3J4*q1J1*q3J2*q3sq*Eij(2,6)-J2J4*q1J3*q1q3*q4J1*Eij(2,6)-
     &    J2J4*q1sq*q3J3*q4J1*Eij(2,6)-J3J4*q1sq*q3J1*q4J2*Eij(2,6)+
     &    J1J4*q1sq*q3J3*q4J2*Eij(2,6)+J2J4*q1J1*q1q3*q4J3*Eij(2,6)+
     &    J2J4*q1J1*q2q3*q4J3*Eij(2,6)+J2J4*q1sq*q3J1*q4J3*Eij(2,6)-
     &    3*J3J4*q1J1*q1J2*q1q3*(Eij(2,1)+Eij(2,6))-
     &    J1J4*q1J2*q1J3*q1q3*(Eij(2,1)+Eij(2,6))+
     &    J1J4*J2J3*q1q3*q1sq*(Eij(2,1)+Eij(2,6))-
     &    J1J3*J2J4*q1q3*q1sq*(Eij(2,1)+Eij(2,6))+
     &    J1J2*J3J4*q1q3*q1sq*(Eij(2,1)+Eij(2,6))-
     &    J2J4*q1J1*q1q3*q2J3*(Eij(2,1)+Eij(2,6))+
     &    J1J3*q1J4*q1sq*q3J2*(Eij(2,1)+Eij(2,6))-
     &    J2J3*q1J1*q1sq*q3J4*(2*Eij(2,1)+Eij(2,6))-
     &    J1J2*q1J3*q1sq*q3J4*(2*Eij(2,1)+Eij(2,6))+
     &    J1J4*J2J3*q1q3**2*(Eij(2,3)+Eij(2,6))-
     &    J1J3*J2J4*q1q3**2*(Eij(2,3)+Eij(2,6))+
     &    J1J2*J3J4*q1q3**2*(Eij(2,3)+Eij(2,6))-
     &    2*J3J4*q1J2*q1q3*q3J1*(Eij(2,3)+Eij(2,6))-
     &    J3J4*q1q3*q2J2*q3J1*(Eij(2,3)+Eij(2,6))-
     &    J2J4*q1q3*q2J3*q3J1*(Eij(2,3)+Eij(2,6))-
     &    J3J4*q1J1*q1q3*q3J2*(Eij(2,3)+Eij(2,6))-
     &    J1J4*q1J3*q1q3*q3J2*(Eij(2,3)+Eij(2,6))+
     &    4*q1J2*q1J4*q3J1*q3J3*(Eij(2,3)+Eij(2,6))-
     &    J2J3*q1J4*q1q3*q3J1*(2*Eij(2,3)+Eij(2,6))-
     &    J1J2*q1J4*q1q3*q3J3*(2*Eij(2,3)+Eij(2,6))+
     &    2*q1J2*q3J1*q3J3*q3J4*(3*Eij(2,3)+Eij(2,6))+
     &    q1J3*q1J4*q3J1*q3J2*(4*Eij(2,3)+Eij(2,6))+
     &    q1J4*q2J3*q3J1*q3J2*(4*Eij(2,3)+Eij(2,6))-
     &    3*J3J4*q1J1*q1J2*q2q3*(Eij(2,5)+Eij(2,6))-
     &    J1J4*q1J2*q1J3*q2q3*(Eij(2,5)+Eij(2,6))+
     &    J1J4*J2J3*q1sq*q2q3*(Eij(2,5)+Eij(2,6))-
     &    J1J3*J2J4*q1sq*q2q3*(Eij(2,5)+Eij(2,6))+
     &    J1J2*J3J4*q1sq*q2q3*(Eij(2,5)+Eij(2,6))-
     &    J2J4*q1J1*q2J3*q2q3*(Eij(2,5)+Eij(2,6))-
     &    J2J3*q1sq*q2J4*q3J1*(Eij(2,5)+Eij(2,6))-
     &    2*J1J2*q1sq*q2J3*q3J4*(Eij(2,5)+Eij(2,6))-
     &    J2J3*q1J4*q1sq*q3J1*(Eij(2,1)+2*Eij(2,6))-
     &    J1J2*q1J4*q1sq*q3J3*(Eij(2,1)+2*Eij(2,6))+
     &    J1J3*q1J4*q1q3*q3J2*(Eij(2,3)+2*Eij(2,6))-
     &    J2J3*q1J1*q1q3*q3J4*(Eij(2,3)+2*Eij(2,6))-
     &    J1J2*q1J3*q1q3*q3J4*(Eij(2,3)+2*Eij(2,6))-
     &    J2J3*q1J1*q2q3*q3J4*(Eij(2,3)+2*Eij(2,6))+
     &    q1J1*q3J2*q3J3*q3J4*(3*Eij(2,3)+2*Eij(2,6))-
     &    J1J2*q1sq*q2J4*q3J3*(Eij(2,5)+2*Eij(2,6))+
     &    2*q1J1*q1J3*q3J2*q3J4*(Eij(2,3)+3*Eij(2,6))+
     &    q1J1*q1J4*q3J2*q3J3*(2*Eij(2,3)+3*Eij(2,6))+
     &    2*q1J1*q1J2*q1J3*q3J4*(5*Eij(2,1)+4*Eij(2,6))+
     &    q1J1*q1J2*q1J4*q3J3*(5*Eij(2,1)+8*Eij(2,6))+
     &    q1J1*q1J2*q3J3*q3J4*(2*Eij(2,1)+2*Eij(2,3)+9*Eij(2,6))+
     &    q1J2*q1J3*q3J1*q3J4*(4*Eij(2,3)+9*Eij(2,6))+
     &    q1J2*q1J3*q1J4*q3J1*(Eij(2,1)+12*Eij(2,6))+
     &    J1J4*q1J3*q1sq*q4J2*(Eij(2,1)-Eij(2,7))-
     &    J1J4*q1J3*q1q4*q4J2*(Eij(2,4)-Eij(2,7))-
     &    3*J3J4*q1J1*q1J2*q1q4*Eij(2,7)-J1J4*q1J2*q1J3*q1q4*Eij(2,7)-
     &    3*J2J3*q1J1*q1J4*q1q4*Eij(2,7)-3*J1J2*q1J3*q1J4*q1q4*Eij(2,7)+
     &    2*J1J4*J2J3*q1q4*q1sq*Eij(2,7)-2*J1J3*J2J4*q1q4*q1sq*Eij(2,7)+
     &    2*J1J2*J3J4*q1q4*q1sq*Eij(2,7)-J2J4*q1J1*q1q4*q2J3*Eij(2,7)-
     &    3*J3J4*q1J1*q1J2*q2q4*Eij(2,7)-J1J4*q1J2*q1J3*q2q4*Eij(2,7)-
     &    3*J2J3*q1J1*q1J4*q2q4*Eij(2,7)+J1J2*q1J3*q1J4*q2q4*Eij(2,7)+
     &    J1J4*J2J3*q1sq*q2q4*Eij(2,7)-J1J3*J2J4*q1sq*q2q4*Eij(2,7)+
     &    J1J2*J3J4*q1sq*q2q4*Eij(2,7)-J2J4*q1J1*q2J3*q2q4*Eij(2,7)-
     &    3*J3J4*q1J1*q1J2*q3q4*Eij(2,7)-J1J4*q1J2*q1J3*q3q4*Eij(2,7)+
     &    J1J3*q1J2*q1J4*q3q4*Eij(2,7)+J1J4*J2J3*q1sq*q3q4*Eij(2,7)-
     &    J1J3*J2J4*q1sq*q3q4*Eij(2,7)+J1J2*J3J4*q1sq*q3q4*Eij(2,7)-
     &    J2J4*q1J1*q2J3*q3q4*Eij(2,7)+J2J3*q1J1*q2J4*q3q4*Eij(2,7)-
     &    2*J3J4*q1J2*q1q2*q4J1*Eij(2,7)-2*J2J3*q1J4*q1q2*q4J1*Eij(2,7)-
     &    2*J3J4*q1J2*q1q3*q4J1*Eij(2,7)-J2J4*q1J3*q1q4*q4J1*Eij(2,7)-
     &    3*J3J4*q1J2*q1sq*q4J1*Eij(2,7)-3*J2J3*q1J4*q1sq*q4J1*Eij(2,7)-
     &    J3J4*q1sq*q2J2*q4J1*Eij(2,7)-J2J3*q1sq*q2J4*q4J1*Eij(2,7)-
     &    2*J3J4*q1J2*q2q3*q4J1*Eij(2,7)-J2J3*q1J4*q2q3*q4J1*Eij(2,7)-
     &    J2J4*q1J3*q2q4*q4J1*Eij(2,7)-J3J4*q1J2*q2sq*q4J1*Eij(2,7)-
     &    J2J3*q1J4*q2sq*q4J1*Eij(2,7)-2*J2J3*q1sq*q3J4*q4J1*Eij(2,7)-
     &    J3J4*q1J2*q3q4*q4J1*Eij(2,7)-J3J4*q1J2*q3sq*q4J1*Eij(2,7)+
     &    8*q1J1*q1J3*q1J4*q4J2*Eij(2,7)-2*J3J4*q1J1*q1q2*q4J2*Eij(2,7)-
     &    2*J3J4*q1J1*q1q3*q4J2*Eij(2,7)-J1J3*q1J4*q1q3*q4J2*Eij(2,7)-
     &    J1J4*q1J3*q1q4*q4J2*Eij(2,7)-J3J4*q1J1*q1sq*q4J2*Eij(2,7)+
     &    J1J3*q1J4*q1sq*q4J2*Eij(2,7)-2*J3J4*q1J1*q2q3*q4J2*Eij(2,7)-
     &    J3J4*q1J1*q2sq*q4J2*Eij(2,7)-J3J4*q1J1*q3q4*q4J2*Eij(2,7)-
     &    J3J4*q1J1*q3sq*q4J2*Eij(2,7)-J3J4*q1sq*q4J1*q4J2*Eij(2,7)+
     &    8*q1J1*q1J2*q1J4*q4J3*Eij(2,7)-J1J2*q1J4*q1q2*q4J3*Eij(2,7)+
     &    J2J4*q1J1*q1q4*q4J3*Eij(2,7)-J1J4*q1J2*q1q4*q4J3*Eij(2,7)-
     &    3*J1J2*q1J4*q1sq*q4J3*Eij(2,7)-2*J1J2*q1sq*q2J4*q4J3*Eij(2,7)-
     &    2*J1J2*q1sq*q3J4*q4J3*Eij(2,7)+J2J4*q1q2*q4J1*q4J3*Eij(2,7)+
     &    J1J4*q1sq*q4J2*q4J3*Eij(2,7)-2*J2J3*q1J1*q1q2*q4J4*Eij(2,7)-
     &    J1J2*q1J3*q1q2*q4J4*Eij(2,7)-J1J3*q1J2*q1q3*q4J4*Eij(2,7)-
     &    J2J3*q1J1*q1sq*q4J4*Eij(2,7)-J1J3*q1J2*q1sq*q4J4*Eij(2,7)-
     &    J1J2*q1J3*q1sq*q4J4*Eij(2,7)-J2J3*q1J1*q2q3*q4J4*Eij(2,7)-
     &    J2J3*q1J1*q2sq*q4J4*Eij(2,7)+J1J3*q1sq*q3J2*q4J4*Eij(2,7)-
     &    J2J3*q1sq*q4J1*q4J4*Eij(2,7)-J1J2*q1sq*q4J3*q4J4*Eij(2,7)-
     &    J3J4*q1J1*q1sq*q4J2*(Eij(2,1)+Eij(2,7))-
     &    J2J3*q1J1*q1sq*q4J4*(Eij(2,1)+Eij(2,7))-
     &    J1J2*q1J3*q1sq*q4J4*(Eij(2,1)+Eij(2,7))-
     &    J3J4*q1J1*q1q4*q4J2*(Eij(2,4)+Eij(2,7))-
     &    J3J4*q1J1*q2q4*q4J2*(Eij(2,4)+Eij(2,7))-
     &    J2J3*q1J1*q1q4*q4J4*(Eij(2,4)+Eij(2,7))-
     &    J1J2*q1J3*q1q4*q4J4*(Eij(2,4)+Eij(2,7))-
     &    J2J3*q1J1*q2q4*q4J4*(Eij(2,4)+Eij(2,7))+
     &    4*q1J2*q1J3*q4J1*q4J4*(Eij(2,4)+Eij(2,7))+
     &    q1J1*q2J4*q4J2*q4J3*(2*Eij(2,4)+Eij(2,7))+
     &    q1J1*q3J4*q4J2*q4J3*(2*Eij(2,4)+Eij(2,7))+
     &    q1J3*q1J4*q4J1*q4J2*(4*Eij(2,4)+Eij(2,7))+
     &    q1J3*q2J4*q4J1*q4J2*(4*Eij(2,4)+Eij(2,7))+
     &    q1J2*q1J4*q4J1*q4J3*(4*Eij(2,4)+Eij(2,7))+
     &    q1J2*q3J4*q4J1*q4J3*(4*Eij(2,4)+Eij(2,7))-
     &    J2J4*q1sq*q2J3*q4J1*(Eij(2,5)+Eij(2,7))-
     &    J2J3*q1sq*q3J1*q4J4*(Eij(2,6)+Eij(2,7))-
     &    J1J2*q1sq*q3J3*q4J4*(Eij(2,6)+Eij(2,7))-
     &    J1J2*q1sq*q2J3*q4J4*(Eij(2,5)+2*Eij(2,7))+
     &    q1J1*q1J3*q4J2*q4J4*(2*Eij(2,4)+3*Eij(2,7))+
     &    q1J1*q1J2*q4J3*q4J4*(2*Eij(2,4)+3*Eij(2,7))+
     &    q1J1*q1J2*q1J3*q4J4*(5*Eij(2,1)+8*Eij(2,7))+
     &    q1J2*q1J3*q1J4*q4J1*(Eij(2,1)+12*Eij(2,7))+
     &    J2J3*q2J1*q2J4*q2q3*(Eij(2,2)-Eij(2,8))-
     &    J2J3*q2J4*q2q3*q3J1*(Eij(2,3)-Eij(2,8))+
     &    J2J3*q1q3*q2J1*q2J4*(Eij(2,5)-Eij(2,8))+
     &    J2J3*q1J1*q1q3*q2J4*(Eij(2,1)-Eij(2,6)-Eij(2,8))+
     &    J2J3*q1J1*q2J4*q2q3*(Eij(2,5)-Eij(2,6)-Eij(2,8))-
     &    2*J2J3*q1J4*q1q3*q2J1*Eij(2,8)-J1J3*q1J2*q1q3*q2J4*Eij(2,8)-
     &    4*J1J2*q1q3*q2J3*q2J4*Eij(2,8)-2*J2J3*q1J4*q2J1*q2q3*Eij(2,8)-
     &    J1J3*q1J2*q2J4*q2q3*Eij(2,8)+J1J2*q1J3*q2J4*q2q3*Eij(2,8)+
     &    J1J4*J2J3*q1q3*q2sq*Eij(2,8)-J1J3*J2J4*q1q3*q2sq*Eij(2,8)+
     &    J1J2*J3J4*q1q3*q2sq*Eij(2,8)-2*J3J4*q1q2*q2J2*q3J1*Eij(2,8)-
     &    2*J3J4*q1q3*q2J2*q3J1*Eij(2,8)-J3J4*q1sq*q2J2*q3J1*Eij(2,8)-
     &    2*J2J3*q1q2*q2J4*q3J1*Eij(2,8)-J2J3*q1sq*q2J4*q3J1*Eij(2,8)+
     &    8*q2J2*q2J3*q2J4*q3J1*Eij(2,8)-2*J3J4*q2J2*q2q3*q3J1*Eij(2,8)-
     &    J2J3*q2J4*q2q3*q3J1*Eij(2,8)-J2J4*q2J3*q2q4*q3J1*Eij(2,8)-
     &    2*J3J4*q1J2*q2sq*q3J1*Eij(2,8)-2*J2J3*q1J4*q2sq*q3J1*Eij(2,8)-
     &    2*J3J4*q2J2*q2sq*q3J1*Eij(2,8)-J2J4*q2J3*q2sq*q3J1*Eij(2,8)-
     &    2*J2J3*q2J4*q2sq*q3J1*Eij(2,8)+4*q1J3*q1J4*q2J1*q3J2*Eij(2,8)-
     &    2*J3J4*q1q2*q2J1*q3J2*Eij(2,8)-2*J3J4*q1q3*q2J1*q3J2*Eij(2,8)-
     &    J3J4*q1sq*q2J1*q3J2*Eij(2,8)-J1J4*q1q4*q2J3*q3J2*Eij(2,8)+
     &    4*q1J3*q2J1*q2J4*q3J2*Eij(2,8)-2*J3J4*q2J1*q2q3*q3J2*Eij(2,8)-
     &    J3J4*q1J1*q2sq*q3J2*Eij(2,8)-J1J4*q1J3*q2sq*q3J2*Eij(2,8)+
     &    J1J3*q1J4*q2sq*q3J2*Eij(2,8)-J3J4*q2J1*q2sq*q3J2*Eij(2,8)-
     &    J1J4*q1q4*q2J2*q3J3*Eij(2,8)-J1J2*q1q2*q2J4*q3J3*Eij(2,8)-
     &    J1J2*q1sq*q2J4*q3J3*Eij(2,8)-J2J4*q2J1*q2q4*q3J3*Eij(2,8)+
     &    2*q1J4*q2J1*q3J2*q3J3*Eij(2,8)+2*q2J1*q2J4*q3J2*q3J3*Eij(2,8)-
     &    2*J2J3*q1q2*q2J1*q3J4*Eij(2,8)-2*J2J3*q1q3*q2J1*q3J4*Eij(2,8)-
     &    J2J3*q1sq*q2J1*q3J4*Eij(2,8)-J1J3*q1q3*q2J2*q3J4*Eij(2,8)-
     &    J1J2*q1q2*q2J3*q3J4*Eij(2,8)-J1J2*q1sq*q2J3*q3J4*Eij(2,8)-
     &    3*J2J3*q2J1*q2q3*q3J4*Eij(2,8)-J1J3*q1J2*q2sq*q3J4*Eij(2,8)+
     &    J1J2*q1J3*q2sq*q3J4*Eij(2,8)-J2J3*q2J1*q2sq*q3J4*Eij(2,8)-
     &    2*J2J3*q2sq*q3J1*q3J4*Eij(2,8)+J1J3*q1q2*q3J2*q3J4*Eij(2,8)+
     &    4*q1J3*q2J1*q3J2*q3J4*Eij(2,8)+5*q2J1*q2J3*q3J2*q3J4*Eij(2,8)-
     &    3*J1J2*q1q2*q3J3*q3J4*Eij(2,8)+2*q2J1*q3J2*q3J3*q3J4*Eij(2,8)-
     &    J3J4*q2J2*q3J1*q3q4*Eij(2,8)-J3J4*q2J1*q3J2*q3q4*Eij(2,8)+
     &    J1J4*J2J3*q1q2*q3sq*Eij(2,8)-J1J3*J2J4*q1q2*q3sq*Eij(2,8)+
     &    J1J2*J3J4*q1q2*q3sq*Eij(2,8)-2*J3J4*q1J2*q2J1*q3sq*Eij(2,8)-
     &    J1J4*q1J3*q2J2*q3sq*Eij(2,8)+J1J3*q1J4*q2J2*q3sq*Eij(2,8)-
     &    J3J4*q2J1*q2J2*q3sq*Eij(2,8)-J2J4*q2J1*q2J3*q3sq*Eij(2,8)+
     &    J2J3*q2J1*q2J4*q3sq*Eij(2,8)-J3J4*q2J2*q3J1*q3sq*Eij(2,8)-
     &    J3J4*q2J1*q3J2*q3sq*Eij(2,8)-J2J4*q1q2*q3J3*q4J1*Eij(2,8)-
     &    J3J4*q1q3*q2J1*q4J2*Eij(2,8)+J1J4*q1q3*q2J3*q4J2*Eij(2,8)-
     &    J3J4*q2J1*q2q3*q4J2*Eij(2,8)-J3J4*q2sq*q3J1*q4J2*Eij(2,8)+
     &    J1J4*q1q2*q3J3*q4J2*Eij(2,8)+J2J4*q1q3*q2J1*q4J3*Eij(2,8)+
     &    J2J4*q2J1*q2q3*q4J3*Eij(2,8)+J2J4*q2sq*q3J1*q4J3*Eij(2,8)+
     &    q2J1*q3J2*q3J4*q4J3*Eij(2,8)-J2J3*q1q3*q2J1*q4J4*Eij(2,8)-
     &    J2J3*q2J1*q2q3*q4J4*Eij(2,8)-J2J3*q2sq*q3J1*q4J4*Eij(2,8)+
     &    2*q1J3*q2J1*q3J2*q4J4*Eij(2,8)+q2J1*q3J2*q3J3*q4J4*Eij(2,8)+
     &    J1J4*J2J3*q1q2*q2q3*(Eij(2,2)+Eij(2,8))-
     &    J1J3*J2J4*q1q2*q2q3*(Eij(2,2)+Eij(2,8))+
     &    J1J2*J3J4*q1q2*q2q3*(Eij(2,2)+Eij(2,8))-
     &    2*J3J4*q1J2*q2J1*q2q3*(Eij(2,2)+Eij(2,8))-
     &    J1J4*q1J3*q2J2*q2q3*(Eij(2,2)+Eij(2,8))+
     &    J1J3*q1J4*q2J2*q2q3*(Eij(2,2)+Eij(2,8))-
     &    J3J4*q2J1*q2J2*q2q3*(Eij(2,2)+Eij(2,8))-
     &    J2J4*q2J1*q2J3*q2q3*(Eij(2,2)+Eij(2,8))-
     &    2*J1J2*q1q2*q2J3*q3J4*(Eij(2,2)+Eij(2,8))+
     &    2*q2J1*q2J2*q2J3*q3J4*(3*Eij(2,2)+Eij(2,8))+
     &    J1J4*J2J3*q1q3*q2q3*(Eij(2,3)+Eij(2,8))-
     &    J1J3*J2J4*q1q3*q2q3*(Eij(2,3)+Eij(2,8))+
     &    J1J2*J3J4*q1q3*q2q3*(Eij(2,3)+Eij(2,8))-
     &    2*J3J4*q1J2*q2q3*q3J1*(Eij(2,3)+Eij(2,8))-
     &    J3J4*q2J2*q2q3*q3J1*(Eij(2,3)+Eij(2,8))-
     &    J2J4*q2J3*q2q3*q3J1*(Eij(2,3)+Eij(2,8))-
     &    J3J4*q1J1*q2q3*q3J2*(Eij(2,3)+Eij(2,8))-
     &    J1J4*q1J3*q2q3*q3J2*(Eij(2,3)+Eij(2,8))+
     &    J1J3*q1J4*q2q3*q3J2*(Eij(2,3)+Eij(2,8))-
     &    2*J1J2*q1q3*q2J3*q3J4*(Eij(2,3)+Eij(2,8))-
     &    J1J2*q1q3*q2J4*q3J3*(2*Eij(2,3)+Eij(2,8))+
     &    q1J3*q2J4*q3J1*q3J2*(4*Eij(2,3)+Eij(2,8))+
     &    q2J3*q2J4*q3J1*q3J2*(4*Eij(2,3)+Eij(2,8))+
     &    J1J4*J2J3*q1q2*q1q3*(Eij(2,5)+Eij(2,8))-
     &    J1J3*J2J4*q1q2*q1q3*(Eij(2,5)+Eij(2,8))+
     &    J1J2*J3J4*q1q2*q1q3*(Eij(2,5)+Eij(2,8))-
     &    2*J3J4*q1J2*q1q3*q2J1*(Eij(2,5)+Eij(2,8))-
     &    J1J4*q1J3*q1q3*q2J2*(Eij(2,5)+Eij(2,8))+
     &    J1J3*q1J4*q1q3*q2J2*(Eij(2,5)+Eij(2,8))-
     &    J3J4*q1q3*q2J1*q2J2*(Eij(2,5)+Eij(2,8))-
     &    J2J4*q1q3*q2J1*q2J3*(Eij(2,5)+Eij(2,8))+
     &    2*q1J3*q2J1*q2J2*q3J4*(2*Eij(2,2)+Eij(2,5)+Eij(2,8))-
     &    J2J3*q1J1*q2sq*q3J4*(2*Eij(2,5)+Eij(2,8))-
     &    J2J3*q1q3*q2J4*q3J1*(Eij(2,3)-Eij(2,6)+Eij(2,8))-
     &    J1J2*q1J3*q1q2*q3J4*(2*Eij(2,5)-Eij(2,6)+Eij(2,8))+
     &    J1J4*J2J3*q1q2*q1q3*(Eij(2,6)+Eij(2,8))-
     &    J1J3*J2J4*q1q2*q1q3*(Eij(2,6)+Eij(2,8))+
     &    J1J2*J3J4*q1q2*q1q3*(Eij(2,6)+Eij(2,8))-
     &    2*J1J2*q1J4*q1q3*q2J3*(Eij(2,6)+Eij(2,8))-
     &    2*J3J4*q1J2*q1q2*q3J1*(Eij(2,6)+Eij(2,8))-
     &    J3J4*q1q2*q2J2*q3J1*(Eij(2,6)+Eij(2,8))-
     &    J2J4*q1q2*q2J3*q3J1*(Eij(2,6)+Eij(2,8))-
     &    J3J4*q1J1*q1q2*q3J2*(Eij(2,6)+Eij(2,8))-
     &    J1J4*q1J3*q1q2*q3J2*(Eij(2,6)+Eij(2,8))-
     &    J1J3*q1J2*q1q2*q3J4*(Eij(2,6)+Eij(2,8))-
     &    J3J4*q1J1*q2J2*q3sq*(Eij(2,6)+Eij(2,8))-
     &    J3J4*q1q2*q3J1*q4J2*(Eij(2,6)+Eij(2,8))+
     &    J2J4*q1q2*q3J1*q4J3*(Eij(2,6)+Eij(2,8))-
     &    J2J3*q1q2*q2J4*q3J1*(Eij(2,2)+Eij(2,6)+Eij(2,8))+
     &    4*q1J2*q2J3*q3J1*q3J4*(Eij(2,3)+Eij(2,6)+Eij(2,8))+
     &    2*q1J2*q2J4*q3J1*q3J3*(2*Eij(2,3)+Eij(2,6)+Eij(2,8))+
     &    J1J3*q1J4*q1q2*q3J2*(Eij(2,5)+Eij(2,6)+Eij(2,8))-
     &    J3J4*q1J1*q1q3*q2J2*(Eij(2,1)+Eij(2,5)+Eij(2,6)+Eij(2,8))-
     &    J3J4*q1J1*q2J2*q2q3*(Eij(2,2)+Eij(2,5)+Eij(2,6)+Eij(2,8))-
     &    J2J3*q1J1*q1q2*q3J4*(2*Eij(2,1)+2*Eij(2,5)+Eij(2,6)+Eij(2,8))-
     &    J1J2*q1J3*q1q3*q2J4*(2*Eij(2,6)+Eij(2,8))+
     &    4*q1J2*q1J4*q2J3*q3J1*(2*Eij(2,6)+Eij(2,8))+
     &    q1J1*q2J4*q3J2*q3J3*(2*Eij(2,3)+2*Eij(2,6)+Eij(2,8))+
     &    2*q1J1*q1J3*q2J4*q3J2*(3*Eij(2,6)+Eij(2,8))+
     &    2*q1J1*q1J2*q2J3*q3J4*(2*Eij(2,1)+3*Eij(2,5)+3*Eij(2,6)+
     &       Eij(2,8))-J1J2*q1q2*q2J4*q3J3*(Eij(2,2)+2*Eij(2,8))+
     &    q2J1*q2J2*q2J4*q3J3*(3*Eij(2,2)+2*Eij(2,8))+
     &    q2J2*q3J1*q3J3*q3J4*(3*Eij(2,3)+2*Eij(2,8))-
     &    J1J2*q1J4*q1q2*q3J3*(Eij(2,5)+2*Eij(2,8))+
     &    q1J4*q2J1*q2J2*q3J3*(2*Eij(2,2)+Eij(2,5)+2*Eij(2,8))+
     &    4*q1J2*q2J3*q2J4*q3J1*(Eij(2,6)+2*Eij(2,8))+
     &    q1J4*q2J2*q3J1*q3J3*(2*Eij(2,3)+Eij(2,6)+2*Eij(2,8))+
     &    q1J1*q1J4*q2J2*q3J3*(Eij(2,1)+3*Eij(2,5)+3*Eij(2,6)+
     &       2*Eij(2,8))+q1J1*q1J3*q2J2*q3J4*
     &     (2*Eij(2,1)+6*Eij(2,5)+3*Eij(2,6)+2*Eij(2,8))+
     &    q1J1*q2J3*q3J2*q3J4*(2*Eij(2,3)+5*Eij(2,6)+2*Eij(2,8))+
     &    q1J1*q1J4*q2J3*q3J2*(Eij(2,1)+6*Eij(2,6)+2*Eij(2,8))+
     &    q1J1*q1J2*q2J4*q3J3*(2*Eij(2,1)+3*Eij(2,5)+6*Eij(2,6)+
     &       2*Eij(2,8))+q2J1*q2J2*q3J3*q3J4*(2*Eij(2,2)+3*Eij(2,8))+
     &    2*q2J2*q2J3*q3J1*q3J4*(Eij(2,3)+3*Eij(2,8))+
     &    q2J2*q2J4*q3J1*q3J3*(2*Eij(2,3)+3*Eij(2,8))+
     &    2*q1J4*q2J2*q2J3*q3J1*(Eij(2,6)+3*Eij(2,8))-
     &    J2J3*q1q2*q3J1*q3J4*(2*Eij(2,6)+3*Eij(2,8))+
     &    q1J1*q2J2*q2J4*q3J3*(Eij(2,2)+3*Eij(2,5)+2*Eij(2,6)+
     &       3*Eij(2,8))+q1J1*q2J2*q2J3*q3J4*
     &     (2*Eij(2,2)+6*Eij(2,5)+2*Eij(2,6)+3*Eij(2,8))+
     &    q2J1*q2J3*q2J4*q3J2*(Eij(2,2)+4*Eij(2,8))+
     &    q1J4*q2J1*q2J3*q3J2*(Eij(2,5)+4*Eij(2,8))+
     &    q1J2*q1J4*q2J1*q3J3*(4*Eij(2,5)+Eij(2,6)+4*Eij(2,8))+
     &    q1J2*q1J3*q2J1*q3J4*(8*Eij(2,5)+Eij(2,6)+4*Eij(2,8))+
     &    q1J2*q1J3*q2J4*q3J1*(Eij(2,5)+8*Eij(2,6)+4*Eij(2,8))+
     &    q1J2*q2J1*q2J4*q3J3*(2*Eij(2,2)+2*Eij(2,5)+5*Eij(2,8))+
     &    q1J2*q2J1*q2J3*q3J4*(4*Eij(2,2)+4*Eij(2,5)+5*Eij(2,8))+
     &    q1J3*q2J2*q3J1*q3J4*(2*Eij(2,3)+2*Eij(2,6)+5*Eij(2,8))+
     &    q1J2*q2J1*q3J3*q3J4*(Eij(2,3)+2*Eij(2,5)+6*Eij(2,8))+
     &    q1J3*q2J2*q2J4*q3J1*(Eij(2,2)+2*Eij(2,6)+6*Eij(2,8))-
     &    J2J3*q1J4*q1q2*q3J1*(Eij(2,5)+2*(Eij(2,6)+Eij(2,8)))+
     &    q1J1*q2J2*q3J3*q3J4*(Eij(2,3)+2*Eij(2,5)+
     &       3*(Eij(2,6)+Eij(2,8)))+
     &    q1J3*q1J4*q2J2*q3J1*(Eij(2,5)+4*(Eij(2,6)+Eij(2,8)))+
     &    q1J1*q2J3*q2J4*q3J2*(Eij(2,5)+4*(Eij(2,6)+Eij(2,8)))+
     &    J1J4*q1J3*q1q2*q4J2*(Eij(2,5)-Eij(2,7)-Eij(2,9))+
     &    J1J4*J2J3*q1q2*q1q4*Eij(2,9)-J1J3*J2J4*q1q2*q1q4*Eij(2,9)+
     &    J1J2*J3J4*q1q2*q1q4*Eij(2,9)-2*J3J4*q1J2*q1q4*q2J1*Eij(2,9)-
     &    2*J2J3*q1J4*q1q4*q2J1*Eij(2,9)-J1J4*q1J3*q1q4*q2J2*Eij(2,9)+
     &    J1J3*q1J4*q1q4*q2J2*Eij(2,9)-J3J4*q1q4*q2J1*q2J2*Eij(2,9)-
     &    J2J4*q1q4*q2J1*q2J3*Eij(2,9)-J1J3*q1J2*q1q4*q2J4*Eij(2,9)-
     &    J2J3*q1q4*q2J1*q2J4*Eij(2,9)-4*J1J2*q1q4*q2J3*q2J4*Eij(2,9)+
     &    J1J4*J2J3*q1q2*q2q4*Eij(2,9)-J1J3*J2J4*q1q2*q2q4*Eij(2,9)+
     &    J1J2*J3J4*q1q2*q2q4*Eij(2,9)-2*J3J4*q1J2*q2J1*q2q4*Eij(2,9)-
     &    2*J2J3*q1J4*q2J1*q2q4*Eij(2,9)-J1J4*q1J3*q2J2*q2q4*Eij(2,9)+
     &    J1J3*q1J4*q2J2*q2q4*Eij(2,9)-J3J4*q2J1*q2J2*q2q4*Eij(2,9)-
     &    J2J4*q2J1*q2J3*q2q4*Eij(2,9)-J1J3*q1J2*q2J4*q2q4*Eij(2,9)+
     &    J1J2*q1J3*q2J4*q2q4*Eij(2,9)-J2J3*q2J1*q2J4*q2q4*Eij(2,9)+
     &    J1J4*J2J3*q1q4*q2sq*Eij(2,9)-J1J3*J2J4*q1q4*q2sq*Eij(2,9)+
     &    J1J2*J3J4*q1q4*q2sq*Eij(2,9)+J1J3*q1q4*q2J4*q3J2*Eij(2,9)-
     &    2*J2J3*q1q4*q2J1*q3J4*Eij(2,9)-2*J2J3*q2J1*q2q4*q3J4*Eij(2,9)+
     &    J1J4*J2J3*q1q2*q3q4*Eij(2,9)-J1J3*J2J4*q1q2*q3q4*Eij(2,9)+
     &    J1J2*J3J4*q1q2*q3q4*Eij(2,9)-2*J3J4*q1J2*q2J1*q3q4*Eij(2,9)-
     &    J1J4*q1J3*q2J2*q3q4*Eij(2,9)+J1J3*q1J4*q2J2*q3q4*Eij(2,9)-
     &    J3J4*q2J1*q2J2*q3q4*Eij(2,9)-J2J4*q2J1*q2J3*q3q4*Eij(2,9)+
     &    J2J3*q2J1*q2J4*q3q4*Eij(2,9)-2*J3J4*q1q2*q2J2*q4J1*Eij(2,9)-
     &    2*J3J4*q1q3*q2J2*q4J1*Eij(2,9)-J3J4*q1sq*q2J2*q4J1*Eij(2,9)-
     &    2*J2J3*q1q2*q2J4*q4J1*Eij(2,9)-J2J3*q1sq*q2J4*q4J1*Eij(2,9)+
     &    8*q2J2*q2J3*q2J4*q4J1*Eij(2,9)-2*J3J4*q2J2*q2q3*q4J1*Eij(2,9)-
     &    J2J3*q2J4*q2q3*q4J1*Eij(2,9)-J2J4*q2J3*q2q4*q4J1*Eij(2,9)-
     &    2*J3J4*q1J2*q2sq*q4J1*Eij(2,9)-2*J2J3*q1J4*q2sq*q4J1*Eij(2,9)-
     &    2*J3J4*q2J2*q2sq*q4J1*Eij(2,9)-J2J4*q2J3*q2sq*q4J1*Eij(2,9)-
     &    2*J2J3*q2J4*q2sq*q4J1*Eij(2,9)-2*J2J3*q2sq*q3J4*q4J1*Eij(2,9)-
     &    J3J4*q2J2*q3q4*q4J1*Eij(2,9)-J3J4*q2J2*q3sq*q4J1*Eij(2,9)+
     &    4*q1J3*q1J4*q2J1*q4J2*Eij(2,9)-2*J3J4*q1q2*q2J1*q4J2*Eij(2,9)-
     &    2*J3J4*q1q3*q2J1*q4J2*Eij(2,9)-J3J4*q1q4*q2J1*q4J2*Eij(2,9)-
     &    J3J4*q1sq*q2J1*q4J2*Eij(2,9)+4*q1J4*q2J1*q2J3*q4J2*Eij(2,9)-
     &    J1J3*q1q3*q2J4*q4J2*Eij(2,9)-2*J3J4*q2J1*q2q3*q4J2*Eij(2,9)-
     &    J3J4*q2J1*q2q4*q4J2*Eij(2,9)-J1J4*q1J3*q2sq*q4J2*Eij(2,9)+
     &    J1J3*q1J4*q2sq*q4J2*Eij(2,9)-J3J4*q2J1*q2sq*q4J2*Eij(2,9)+
     &    2*q1J4*q2J1*q3J3*q4J2*Eij(2,9)+4*q1J3*q2J1*q3J4*q4J2*Eij(2,9)+
     &    4*q2J1*q2J3*q3J4*q4J2*Eij(2,9)+2*q2J1*q3J3*q3J4*q4J2*Eij(2,9)-
     &    J3J4*q2J1*q3q4*q4J2*Eij(2,9)-J3J4*q2J1*q3sq*q4J2*Eij(2,9)-
     &    J3J4*q2sq*q4J1*q4J2*Eij(2,9)-2*J1J2*q1J4*q1q2*q4J3*Eij(2,9)+
     &    J2J4*q1q4*q2J1*q4J3*Eij(2,9)-J1J4*q1q4*q2J2*q4J3*Eij(2,9)+
     &    2*q1J4*q2J1*q2J2*q4J3*Eij(2,9)-3*J1J2*q1q2*q2J4*q4J3*Eij(2,9)-
     &    J1J2*q1sq*q2J4*q4J3*Eij(2,9)+5*q1J2*q2J1*q2J4*q4J3*Eij(2,9)+
     &    2*q2J1*q2J2*q2J4*q4J3*Eij(2,9)-2*J1J2*q1q2*q3J4*q4J3*Eij(2,9)+
     &    J2J4*q2sq*q4J1*q4J3*Eij(2,9)+J1J4*q1q2*q4J2*q4J3*Eij(2,9)+
     &    q2J1*q2J4*q4J2*q4J3*Eij(2,9)+q2J1*q3J4*q4J2*q4J3*Eij(2,9)-
     &    2*J2J3*q1q2*q2J1*q4J4*Eij(2,9)-J2J3*q1q4*q2J1*q4J4*Eij(2,9)-
     &    J2J3*q1sq*q2J1*q4J4*Eij(2,9)-J1J3*q1q3*q2J2*q4J4*Eij(2,9)-
     &    J1J2*q1q2*q2J3*q4J4*Eij(2,9)-J1J2*q1sq*q2J3*q4J4*Eij(2,9)-
     &    J2J3*q2J1*q2q3*q4J4*Eij(2,9)-J2J3*q2J1*q2q4*q4J4*Eij(2,9)-
     &    J1J3*q1J2*q2sq*q4J4*Eij(2,9)+J1J2*q1J3*q2sq*q4J4*Eij(2,9)-
     &    J2J3*q2J1*q2sq*q4J4*Eij(2,9)+J1J3*q1q2*q3J2*q4J4*Eij(2,9)-
     &    J2J3*q2sq*q4J1*q4J4*Eij(2,9)+2*q1J3*q2J1*q4J2*q4J4*Eij(2,9)+
     &    2*q2J1*q2J3*q4J2*q4J4*Eij(2,9)+q2J1*q3J3*q4J2*q4J4*Eij(2,9)-
     &    J1J2*q1q2*q4J3*q4J4*Eij(2,9)+q2J1*q2J2*q4J3*q4J4*Eij(2,9)-
     &    J2J4*q1q4*q2J3*q4J1*(Eij(2,4)+Eij(2,9))+
     &    q1J4*q2J2*q4J1*q4J3*(2*Eij(2,4)+Eij(2,9))+
     &    q2J2*q3J4*q4J1*q4J3*(2*Eij(2,4)+Eij(2,9))-
     &    J1J2*q1q4*q2J3*q4J4*(2*Eij(2,4)+Eij(2,9))+
     &    q1J4*q2J3*q4J1*q4J2*(4*Eij(2,4)+Eij(2,9))+
     &    q2J3*q2J4*q4J1*q4J2*(4*Eij(2,4)+Eij(2,9))-
     &    J3J4*q1J1*q2sq*q4J2*(Eij(2,5)+Eij(2,9))-
     &    J2J3*q1J1*q2sq*q4J4*(Eij(2,5)+Eij(2,9))-
     &    J1J2*q1J3*q1q2*q4J4*(Eij(2,5)-Eij(2,7)+Eij(2,9))+
     &    J1J4*J2J3*q1q2*q1q4*(Eij(2,7)+Eij(2,9))-
     &    J1J3*J2J4*q1q2*q1q4*(Eij(2,7)+Eij(2,9))+
     &    J1J2*J3J4*q1q2*q1q4*(Eij(2,7)+Eij(2,9))-
     &    J3J4*q1J1*q1q4*q2J2*(Eij(2,7)+Eij(2,9))-
     &    2*J1J2*q1J4*q1q4*q2J3*(Eij(2,7)+Eij(2,9))-
     &    J2J3*q1J1*q1q4*q2J4*(Eij(2,7)+Eij(2,9))-
     &    J3J4*q1J1*q2J2*q2q4*(Eij(2,7)+Eij(2,9))-
     &    J2J3*q1J1*q2J4*q2q4*(Eij(2,7)+Eij(2,9))-
     &    J3J4*q1J1*q2J2*q3q4*(Eij(2,7)+Eij(2,9))-
     &    2*J3J4*q1J2*q1q2*q4J1*(Eij(2,7)+Eij(2,9))-
     &    2*J2J3*q1J4*q1q2*q4J1*(Eij(2,7)+Eij(2,9))-
     &    J3J4*q1q2*q2J2*q4J1*(Eij(2,7)+Eij(2,9))-
     &    J2J3*q1q2*q2J4*q4J1*(Eij(2,7)+Eij(2,9))-
     &    2*J2J3*q1q2*q3J4*q4J1*(Eij(2,7)+Eij(2,9))+
     &    J1J3*q1J4*q1q2*q4J2*(Eij(2,7)+Eij(2,9))-
     &    J3J4*q1q2*q4J1*q4J2*(Eij(2,7)+Eij(2,9))-
     &    J1J3*q1J2*q1q2*q4J4*(Eij(2,7)+Eij(2,9))-
     &    J2J3*q1q2*q4J1*q4J4*(Eij(2,7)+Eij(2,9))-
     &    J2J4*q1q2*q2J3*q4J1*(Eij(2,2)+Eij(2,7)+Eij(2,9))+
     &    q1J1*q2J2*q4J3*q4J4*(Eij(2,4)+Eij(2,7)+Eij(2,9))+
     &    2*q1J2*q2J3*q4J1*q4J4*(2*Eij(2,4)+Eij(2,7)+Eij(2,9))-
     &    J3J4*q1J1*q1q2*q4J2*(Eij(2,1)+Eij(2,5)+Eij(2,7)+Eij(2,9))-
     &    J2J3*q1J1*q1q2*q4J4*(Eij(2,1)+Eij(2,5)+Eij(2,7)+Eij(2,9))-
     &    J1J2*q1J3*q1q4*q2J4*(2*Eij(2,7)+Eij(2,9))+
     &    4*q1J2*q1J3*q2J4*q4J1*(2*Eij(2,7)+Eij(2,9))+
     &    q1J1*q2J3*q4J2*q4J4*(2*Eij(2,4)+2*Eij(2,7)+Eij(2,9))+
     &    2*q1J1*q1J4*q2J3*q4J2*(3*Eij(2,7)+Eij(2,9))+
     &    2*q1J1*q1J2*q2J4*q4J3*(3*Eij(2,7)+Eij(2,9))-
     &    J1J2*q1q2*q3J3*q4J4*(Eij(2,8)+Eij(2,9))+
     &    q2J1*q2J2*q3J3*q4J4*(Eij(2,2)+Eij(2,8)+Eij(2,9))-
     &    J2J3*q1q2*q3J1*q4J4*(Eij(2,6)+Eij(2,8)+Eij(2,9))+
     &    q2J1*q2J3*q3J2*q4J4*(2*Eij(2,8)+Eij(2,9))+
     &    q2J1*q2J2*q3J4*q4J3*(Eij(2,2)+2*Eij(2,9))-
     &    J1J2*q1q2*q2J3*q4J4*(Eij(2,2)+2*Eij(2,9))+
     &    q2J1*q2J2*q2J3*q4J4*(3*Eij(2,2)+2*Eij(2,9))+
     &    q1J2*q2J1*q4J3*q4J4*(Eij(2,4)+2*Eij(2,9))+
     &    q1J3*q2J1*q2J2*q4J4*(2*Eij(2,2)+Eij(2,5)+2*Eij(2,9))+
     &    4*q1J2*q2J3*q2J4*q4J1*(Eij(2,7)+2*Eij(2,9))+
     &    q1J3*q2J2*q4J1*q4J4*(2*Eij(2,4)+Eij(2,7)+2*Eij(2,9))+
     &    q1J1*q1J4*q2J2*q4J3*(3*Eij(2,7)+2*Eij(2,9))+
     &    q1J1*q1J3*q2J2*q4J4*(Eij(2,1)+3*Eij(2,5)+3*Eij(2,7)+
     &       2*Eij(2,9))+q1J1*q1J3*q2J4*q4J2*
     &     (Eij(2,1)+6*Eij(2,7)+2*Eij(2,9))+
     &    q1J1*q1J2*q2J3*q4J4*(2*Eij(2,1)+3*Eij(2,5)+6*Eij(2,7)+
     &       2*Eij(2,9))+q2J1*q2J4*q3J3*q4J2*(Eij(2,8)+2*Eij(2,9))+
     &    q2J2*q2J3*q4J1*q4J4*(2*Eij(2,4)+3*Eij(2,9))+
     &    2*q1J3*q2J2*q2J4*q4J1*(Eij(2,7)+3*Eij(2,9))+
     &    q1J1*q2J2*q2J4*q4J3*(2*Eij(2,7)+3*Eij(2,9))+
     &    q1J1*q2J2*q2J3*q4J4*(Eij(2,2)+3*Eij(2,5)+2*Eij(2,7)+
     &       3*Eij(2,9))+q2J1*q2J3*q2J4*q4J2*(Eij(2,2)+4*Eij(2,9))+
     &    q1J3*q2J1*q2J4*q4J2*(Eij(2,5)+4*Eij(2,9))+
     &    q1J2*q1J4*q2J1*q4J3*(Eij(2,7)+4*Eij(2,9))+
     &    q1J2*q1J3*q2J1*q4J4*(4*Eij(2,5)+Eij(2,7)+4*Eij(2,9))+
     &    q1J2*q1J4*q2J3*q4J1*(Eij(2,5)+8*Eij(2,7)+4*Eij(2,9))+
     &    q1J2*q2J1*q2J3*q4J4*(2*Eij(2,2)+2*Eij(2,5)+5*Eij(2,9))+
     &    q1J4*q2J2*q2J3*q4J1*(Eij(2,2)+2*Eij(2,7)+6*Eij(2,9))+
     &    q1J3*q1J4*q2J2*q4J1*(Eij(2,5)+4*(Eij(2,7)+Eij(2,9)))+
     &    q1J1*q2J3*q2J4*q4J2*(Eij(2,5)+4*(Eij(2,7)+Eij(2,9)))+
     &    J1J4*J2J3*q1q3*q1q4*Eij(2,10)-J1J3*J2J4*q1q3*q1q4*Eij(2,10)+
     &    J1J2*J3J4*q1q3*q1q4*Eij(2,10)+J1J4*J2J3*q1q3*q2q4*Eij(2,10)-
     &    J1J3*J2J4*q1q3*q2q4*Eij(2,10)+J1J2*J3J4*q1q3*q2q4*Eij(2,10)-
     &    2*J3J4*q1J2*q1q4*q3J1*Eij(2,10)-J3J4*q1q4*q2J2*q3J1*Eij(2,10)-
     &    J2J4*q1q4*q2J3*q3J1*Eij(2,10)-2*J3J4*q1J2*q2q4*q3J1*Eij(2,10)-
     &    2*J2J3*q1J4*q2q4*q3J1*Eij(2,10)-J3J4*q2J2*q2q4*q3J1*Eij(2,10)-
     &    J2J4*q2J3*q2q4*q3J1*Eij(2,10)-J2J3*q2J4*q2q4*q3J1*Eij(2,10)-
     &    J3J4*q1J1*q1q4*q3J2*Eij(2,10)-J1J4*q1J3*q1q4*q3J2*Eij(2,10)-
     &    J3J4*q1J1*q2q4*q3J2*Eij(2,10)-J1J4*q1J3*q2q4*q3J2*Eij(2,10)+
     &    J1J3*q1J4*q2q4*q3J2*Eij(2,10)-J1J3*q1J2*q1q4*q3J4*Eij(2,10)-
     &    J1J3*q1J2*q2q4*q3J4*Eij(2,10)+J1J2*q1J3*q2q4*q3J4*Eij(2,10)-
     &    3*J2J3*q1q4*q3J1*q3J4*Eij(2,10)-2*J2J3*q2q4*q3J1*q3J4*Eij(2,10)+
     &    J1J3*q1q4*q3J2*q3J4*Eij(2,10)-3*J1J2*q1q4*q3J3*q3J4*Eij(2,10)+
     &    J1J4*J2J3*q1q3*q3q4*Eij(2,10)-J1J3*J2J4*q1q3*q3q4*Eij(2,10)+
     &    J1J2*J3J4*q1q3*q3q4*Eij(2,10)-2*J3J4*q1J2*q3J1*q3q4*Eij(2,10)-
     &    J3J4*q2J2*q3J1*q3q4*Eij(2,10)-J2J4*q2J3*q3J1*q3q4*Eij(2,10)+
     &    J2J3*q2J4*q3J1*q3q4*Eij(2,10)-J3J4*q1J1*q3J2*q3q4*Eij(2,10)-
     &    J1J4*q1J3*q3J2*q3q4*Eij(2,10)+J1J3*q1J4*q3J2*q3q4*Eij(2,10)+
     &    J1J4*J2J3*q1q4*q3sq*Eij(2,10)-J1J3*J2J4*q1q4*q3sq*Eij(2,10)+
     &    J1J2*J3J4*q1q4*q3sq*Eij(2,10)-2*J2J3*q1J4*q1q3*q4J1*Eij(2,10)-
     &    2*J2J3*q1J4*q2q3*q4J1*Eij(2,10)-2*J3J4*q1q2*q3J2*q4J1*Eij(2,10)-
     &    2*J3J4*q1q3*q3J2*q4J1*Eij(2,10)-J3J4*q1sq*q3J2*q4J1*Eij(2,10)+
     &    4*q1J3*q2J4*q3J2*q4J1*Eij(2,10)-2*J3J4*q2q3*q3J2*q4J1*Eij(2,10)-
     &    J3J4*q2sq*q3J2*q4J1*Eij(2,10)-J2J4*q1q4*q3J3*q4J1*Eij(2,10)-
     &    J2J4*q2q4*q3J3*q4J1*Eij(2,10)+2*q2J4*q3J2*q3J3*q4J1*Eij(2,10)-
     &    2*J2J3*q1q2*q3J4*q4J1*Eij(2,10)-2*J2J3*q1q3*q3J4*q4J1*Eij(2,10)-
     &    J2J3*q1sq*q3J4*q4J1*Eij(2,10)-3*J2J3*q2q3*q3J4*q4J1*Eij(2,10)-
     &    J2J3*q2sq*q3J4*q4J1*Eij(2,10)+4*q1J3*q3J2*q3J4*q4J1*Eij(2,10)+
     &    5*q2J3*q3J2*q3J4*q4J1*Eij(2,10)+2*q3J2*q3J3*q3J4*q4J1*Eij(2,10)-
     &    J3J4*q3J2*q3q4*q4J1*Eij(2,10)-2*J3J4*q1J2*q3sq*q4J1*Eij(2,10)-
     &    J3J4*q2J2*q3sq*q4J1*Eij(2,10)-J2J4*q2J3*q3sq*q4J1*Eij(2,10)+
     &    J2J3*q2J4*q3sq*q4J1*Eij(2,10)-J3J4*q3J2*q3sq*q4J1*Eij(2,10)-
     &    2*J3J4*q1q2*q3J1*q4J2*Eij(2,10)-2*J3J4*q1q3*q3J1*q4J2*Eij(2,10)-
     &    J3J4*q1q4*q3J1*q4J2*Eij(2,10)-J3J4*q1sq*q3J1*q4J2*Eij(2,10)+
     &    4*q1J4*q2J3*q3J1*q4J2*Eij(2,10)-2*J3J4*q2q3*q3J1*q4J2*Eij(2,10)-
     &    J3J4*q2q4*q3J1*q4J2*Eij(2,10)-J3J4*q2sq*q3J1*q4J2*Eij(2,10)+
     &    2*q1J4*q3J1*q3J3*q4J2*Eij(2,10)-J1J3*q1q3*q3J4*q4J2*Eij(2,10)+
     &    5*q1J3*q3J1*q3J4*q4J2*Eij(2,10)+4*q2J3*q3J1*q3J4*q4J2*Eij(2,10)+
     &    2*q3J1*q3J3*q3J4*q4J2*Eij(2,10)-J3J4*q3J1*q3q4*q4J2*Eij(2,10)-
     &    J3J4*q1J1*q3sq*q4J2*Eij(2,10)-J1J4*q1J3*q3sq*q4J2*Eij(2,10)+
     &    J1J3*q1J4*q3sq*q4J2*Eij(2,10)-J3J4*q3J1*q3sq*q4J2*Eij(2,10)-
     &    J3J4*q1q3*q4J1*q4J2*Eij(2,10)-J3J4*q2q3*q4J1*q4J2*Eij(2,10)-
     &    2*J1J2*q1J4*q1q3*q4J3*Eij(2,10)-2*J1J2*q1q3*q2J4*q4J3*Eij(2,10)+
     &    4*q1J2*q1J4*q3J1*q4J3*Eij(2,10)+J2J4*q1q4*q3J1*q4J3*Eij(2,10)+
     &    2*q1J4*q2J2*q3J1*q4J3*Eij(2,10)+4*q1J2*q2J4*q3J1*q4J3*Eij(2,10)+
     &    2*q2J2*q2J4*q3J1*q4J3*Eij(2,10)+2*q1J1*q1J4*q3J2*q4J3*Eij(2,10)-
     &    J1J4*q1q4*q3J2*q4J3*Eij(2,10)+2*q1J1*q2J4*q3J2*q4J3*Eij(2,10)-
     &    J1J2*q1q2*q3J4*q4J3*Eij(2,10)-2*J1J2*q1q3*q3J4*q4J3*Eij(2,10)-
     &    J1J2*q1sq*q3J4*q4J3*Eij(2,10)+J2J4*q2q3*q4J1*q4J3*Eij(2,10)+
     &    q1J4*q3J2*q4J1*q4J3*Eij(2,10)+q3J2*q3J4*q4J1*q4J3*Eij(2,10)+
     &    J1J4*q1q3*q4J2*q4J3*Eij(2,10)+q2J4*q3J1*q4J2*q4J3*Eij(2,10)+
     &    q3J1*q3J4*q4J2*q4J3*Eij(2,10)-J1J3*q1J2*q1q3*q4J4*Eij(2,10)-
     &    J1J3*q1J2*q2q3*q4J4*Eij(2,10)+J1J2*q1J3*q2q3*q4J4*Eij(2,10)-
     &    2*J2J3*q1q2*q3J1*q4J4*Eij(2,10)-J2J3*q1sq*q3J1*q4J4*Eij(2,10)-
     &    J2J3*q2q3*q3J1*q4J4*Eij(2,10)-J2J3*q2q4*q3J1*q4J4*Eij(2,10)-
     &    J2J3*q2sq*q3J1*q4J4*Eij(2,10)-J1J2*q1q2*q3J3*q4J4*Eij(2,10)-
     &    J1J2*q1sq*q3J3*q4J4*Eij(2,10)-J2J3*q1q3*q4J1*q4J4*Eij(2,10)-
     &    J2J3*q2q3*q4J1*q4J4*Eij(2,10)+2*q1J3*q3J2*q4J1*q4J4*Eij(2,10)+
     &    q3J2*q3J3*q4J1*q4J4*Eij(2,10)+2*q2J3*q3J1*q4J2*q4J4*Eij(2,10)+
     &    q3J1*q3J3*q4J2*q4J4*Eij(2,10)-J1J2*q1q3*q4J3*q4J4*Eij(2,10)+
     &    2*q1J2*q3J1*q4J3*q4J4*Eij(2,10)+q2J2*q3J1*q4J3*q4J4*Eij(2,10)+
     &    q1J1*q3J2*q4J3*q4J4*Eij(2,10)-
     &    J2J3*q1q3*q3J1*q4J4*(Eij(2,3)+Eij(2,10))-
     &    J1J2*q1q3*q3J3*q4J4*(Eij(2,3)+Eij(2,10))+
     &    q1J3*q3J1*q3J2*q4J4*(2*Eij(2,3)+Eij(2,10))+
     &    q2J3*q3J1*q3J2*q4J4*(2*Eij(2,3)+Eij(2,10))-
     &    J2J3*q1q4*q3J1*q4J4*(Eij(2,4)+Eij(2,10))-
     &    J1J2*q1q4*q3J3*q4J4*(Eij(2,4)+Eij(2,10))+
     &    q1J4*q3J3*q4J1*q4J2*(2*Eij(2,4)+Eij(2,10))+
     &    q2J4*q3J3*q4J1*q4J2*(2*Eij(2,4)+Eij(2,10))-
     &    J2J3*q1J1*q1q3*q4J4*(Eij(2,6)+Eij(2,10))-
     &    J1J2*q1J3*q1q3*q4J4*(Eij(2,6)+Eij(2,10))-
     &    J2J3*q1J1*q2q3*q4J4*(Eij(2,6)+Eij(2,10))+
     &    q1J1*q3J2*q3J3*q4J4*(Eij(2,3)+Eij(2,6)+Eij(2,10))-
     &    J2J3*q1q3*q2J4*q4J1*(-Eij(2,7)+Eij(2,10))+
     &    J1J4*J2J3*q1q3*q1q4*(Eij(2,7)+Eij(2,10))-
     &    J1J3*J2J4*q1q3*q1q4*(Eij(2,7)+Eij(2,10))+
     &    J1J2*J3J4*q1q3*q1q4*(Eij(2,7)+Eij(2,10))+
     &    J1J3*q1J4*q1q4*q3J2*(Eij(2,7)+Eij(2,10))-
     &    2*J3J4*q1J2*q1q3*q4J1*(Eij(2,7)+Eij(2,10))-
     &    J3J4*q1q3*q2J2*q4J1*(Eij(2,7)+Eij(2,10))+
     &    J1J3*q1J4*q1q3*q4J2*(Eij(2,7)+Eij(2,10))+
     &    q1J1*q3J3*q4J2*q4J4*(Eij(2,4)+Eij(2,7)+Eij(2,10))-
     &    J1J4*q1J3*q1q3*q4J2*(-Eij(2,6)+Eij(2,7)+Eij(2,10))-
     &    J3J4*q1J1*q1q3*q4J2*(Eij(2,6)+Eij(2,7)+Eij(2,10))-
     &    J2J3*q1J1*q1q4*q3J4*(2*Eij(2,7)+Eij(2,10))-
     &    J1J2*q1J3*q1q4*q3J4*(2*Eij(2,7)+Eij(2,10))-
     &    J2J3*q1J1*q2q4*q3J4*(2*Eij(2,7)+Eij(2,10))+
     &    4*q1J2*q1J3*q3J4*q4J1*(2*Eij(2,7)+Eij(2,10))+
     &    2*q1J1*q1J3*q3J4*q4J2*(3*Eij(2,7)+Eij(2,10))+
     &    q2J2*q3J1*q3J3*q4J4*(Eij(2,3)+Eij(2,8)+Eij(2,10))-
     &    J2J4*q1q3*q2J3*q4J1*(Eij(2,7)+Eij(2,8)+Eij(2,10))-
     &    J2J3*q2J4*q2q3*q4J1*(-Eij(2,9)+Eij(2,10))+
     &    J1J4*J2J3*q1q4*q2q3*(Eij(2,9)+Eij(2,10))-
     &    J1J3*J2J4*q1q4*q2q3*(Eij(2,9)+Eij(2,10))+
     &    J1J2*J3J4*q1q4*q2q3*(Eij(2,9)+Eij(2,10))-
     &    J2J3*q1q4*q2J4*q3J1*(Eij(2,9)+Eij(2,10))-
     &    2*J1J2*q1q4*q2J3*q3J4*(Eij(2,9)+Eij(2,10))-
     &    2*J3J4*q1J2*q2q3*q4J1*(Eij(2,9)+Eij(2,10))-
     &    J3J4*q2J2*q2q3*q4J1*(Eij(2,9)+Eij(2,10))-
     &    J2J4*q2J3*q2q3*q4J1*(Eij(2,9)+Eij(2,10))-
     &    J1J4*q1J3*q2q3*q4J2*(Eij(2,9)+Eij(2,10))+
     &    J1J3*q1J4*q2q3*q4J2*(Eij(2,9)+Eij(2,10))+
     &    q2J2*q3J3*q4J1*q4J4*(Eij(2,4)+Eij(2,9)+Eij(2,10))-
     &    J3J4*q1J1*q2q3*q4J2*(Eij(2,6)+Eij(2,9)+Eij(2,10))+
     &    4*q1J2*q2J3*q3J4*q4J1*(Eij(2,7)+Eij(2,9)+Eij(2,10))+
     &    2*q1J1*q2J3*q3J4*q4J2*(2*Eij(2,7)+Eij(2,9)+Eij(2,10))+
     &    q1J1*q2J2*q3J3*q4J4*(Eij(2,5)+Eij(2,6)+Eij(2,7)+Eij(2,8)+
     &       Eij(2,9)+Eij(2,10))+
     &    2*q1J3*q2J2*q3J4*q4J1*(Eij(2,7)+2*Eij(2,9)+Eij(2,10))+
     &    2*q2J2*q2J3*q3J4*q4J1*(3*Eij(2,9)+Eij(2,10))+
     &    q1J2*q2J1*q3J4*q4J3*(Eij(2,5)+4*Eij(2,9)+Eij(2,10))+
     &    q1J1*q2J2*q3J4*q4J3*(Eij(2,5)+2*(Eij(2,7)+Eij(2,9))+
     &       Eij(2,10))+q1J2*q2J1*q3J3*q4J4*
     &     (Eij(2,5)+2*(Eij(2,8)+Eij(2,9))+Eij(2,10))+
     &    q1J4*q3J2*q3J3*q4J1*(Eij(2,3)+2*Eij(2,10))+
     &    q2J4*q3J1*q3J3*q4J2*(Eij(2,3)+2*Eij(2,10))+
     &    q2J3*q3J2*q4J1*q4J4*(Eij(2,4)+2*Eij(2,10))+
     &    q1J3*q3J1*q4J2*q4J4*(Eij(2,4)+2*Eij(2,10))+
     &    q1J1*q3J2*q3J4*q4J3*(Eij(2,6)+2*Eij(2,10))+
     &    q1J2*q3J1*q3J3*q4J4*(2*Eij(2,3)+Eij(2,6)+2*Eij(2,10))+
     &    q1J1*q1J3*q3J2*q4J4*(3*Eij(2,6)+2*Eij(2,10))-
     &    J2J3*q1J4*q1q4*q3J1*(Eij(2,7)+2*Eij(2,10))-
     &    J1J2*q1J4*q1q4*q3J3*(Eij(2,7)+2*Eij(2,10))+
     &    q1J2*q3J3*q4J1*q4J4*(2*Eij(2,4)+Eij(2,7)+2*Eij(2,10))+
     &    q1J1*q1J4*q3J3*q4J2*(3*Eij(2,7)+2*Eij(2,10))+
     &    q1J1*q1J2*q3J4*q4J3*(Eij(2,1)+6*Eij(2,7)+2*Eij(2,10))+
     &    q1J1*q1J2*q3J3*q4J4*(Eij(2,1)+3*(Eij(2,6)+Eij(2,7))+
     &       2*Eij(2,10))+q2J2*q3J1*q3J4*q4J3*(Eij(2,8)+2*Eij(2,10))-
     &    J1J2*q1q3*q2J3*q4J4*(Eij(2,8)+2*Eij(2,10))+
     &    2*q1J2*q2J3*q3J1*q4J4*(Eij(2,6)+Eij(2,8)+2*Eij(2,10))+
     &    q1J1*q2J3*q3J2*q4J4*(2*Eij(2,6)+Eij(2,7)+Eij(2,8)+
     &       2*Eij(2,10))+q2J2*q2J3*q3J1*q4J4*(3*Eij(2,8)+2*Eij(2,10))-
     &    J1J2*q1q4*q2J4*q3J3*(Eij(2,9)+2*Eij(2,10))+
     &    2*q1J2*q2J4*q3J3*q4J1*(Eij(2,7)+Eij(2,9)+2*Eij(2,10))+
     &    q1J1*q2J4*q3J3*q4J2*(Eij(2,6)+2*Eij(2,7)+Eij(2,9)+
     &       2*Eij(2,10))+q1J3*q2J2*q3J1*q4J4*
     &     (Eij(2,6)+2*Eij(2,8)+Eij(2,9)+2*Eij(2,10))+
     &    q1J4*q2J2*q3J3*q4J1*(Eij(2,7)+Eij(2,8)+2*Eij(2,9)+
     &       2*Eij(2,10))+q2J2*q2J4*q3J3*q4J1*(3*Eij(2,9)+2*Eij(2,10))+
     &    2*q1J2*q3J3*q3J4*q4J1*(Eij(2,7)+3*Eij(2,10))+
     &    q1J1*q3J3*q3J4*q4J2*(2*Eij(2,7)+3*Eij(2,10))+
     &    q2J2*q3J3*q3J4*q4J1*(2*Eij(2,9)+3*Eij(2,10))+
     &    q1J3*q1J4*q3J2*q4J1*(Eij(2,6)+4*Eij(2,10))+
     &    q1J2*q3J1*q3J4*q4J3*(Eij(2,6)+4*Eij(2,10))+
     &    q1J3*q1J4*q3J1*q4J2*(Eij(2,7)+4*Eij(2,10))+
     &    q1J2*q1J3*q3J1*q4J4*(4*Eij(2,6)+Eij(2,7)+4*Eij(2,10))+
     &    q1J2*q1J4*q3J3*q4J1*(Eij(2,6)+4*Eij(2,7)+4*Eij(2,10))+
     &    q2J3*q2J4*q3J1*q4J2*(Eij(2,8)+4*Eij(2,10))+
     &    q1J4*q2J3*q3J2*q4J1*(Eij(2,7)+Eij(2,8)+4*Eij(2,10))+
     &    q2J3*q2J4*q3J2*q4J1*(Eij(2,9)+4*Eij(2,10))+
     &    q1J3*q2J4*q3J1*q4J2*(Eij(2,6)+Eij(2,9)+4*Eij(2,10))-
     &    J3J4*q1J1*q1J2*Eij(2,11)+2*J2J4*q1J1*q1J3*Eij(2,11)+
     &    3*J1J4*q1J2*q1J3*Eij(2,11)-J2J3*q1J1*q1J4*Eij(2,11)+
     &    4*J1J3*q1J2*q1J4*Eij(2,11)+J1J2*q1J3*q1J4*Eij(2,11)-
     &    J1J4*J2J3*q1q2*Eij(2,11)-J1J3*J2J4*q1q2*Eij(2,11)-
     &    2*J1J2*J3J4*q1q2*Eij(2,11)+J1J4*J2J3*q1q3*Eij(2,11)-
     &    2*J1J3*J2J4*q1q3*Eij(2,11)-J1J2*J3J4*q1q3*Eij(2,11)-
     &    J1J4*J2J3*q1q4*Eij(2,11)-J1J3*J2J4*q1sq*Eij(2,11)-
     &    J1J2*J3J4*q1sq*Eij(2,11)-J3J4*q1J2*q2J1*Eij(2,11)-
     &    2*J2J3*q1J4*q2J1*Eij(2,11)-J3J4*q1J1*q2J2*Eij(2,11)+
     &    J1J4*q1J3*q2J2*Eij(2,11)+3*J1J3*q1J4*q2J2*Eij(2,11)-
     &    J3J4*q2J1*q2J2*Eij(2,11)+J2J4*q1J1*q2J3*Eij(2,11)+
     &    4*J1J4*q1J2*q2J3*Eij(2,11)+2*J1J2*q1J4*q2J3*Eij(2,11)-
     &    J2J4*q2J1*q2J3*Eij(2,11)+2*J1J4*q2J2*q2J3*Eij(2,11)+
     &    3*J1J3*q1J2*q2J4*Eij(2,11)+3*J1J2*q1J3*q2J4*Eij(2,11)-
     &    J2J3*q2J1*q2J4*Eij(2,11)+2*J1J3*q2J2*q2J4*Eij(2,11)+
     &    4*J1J2*q2J3*q2J4*Eij(2,11)-J1J4*J2J3*q2q3*Eij(2,11)-
     &    2*J1J2*J3J4*q2q3*Eij(2,11)-J1J3*J2J4*q2q4*Eij(2,11)-
     &    J1J4*J2J3*q2sq*Eij(2,11)-J1J2*J3J4*q2sq*Eij(2,11)-
     &    2*J3J4*q1J2*q3J1*Eij(2,11)+J2J4*q1J3*q3J1*Eij(2,11)-
     &    J2J3*q1J4*q3J1*Eij(2,11)-J3J4*q2J2*q3J1*Eij(2,11)-
     &    J2J4*q2J3*q3J1*Eij(2,11)+J2J3*q2J4*q3J1*Eij(2,11)-
     &    J3J4*q1J1*q3J2*Eij(2,11)-J1J4*q1J3*q3J2*Eij(2,11)+
     &    2*J1J3*q1J4*q3J2*Eij(2,11)+J1J4*q2J3*q3J2*Eij(2,11)+
     &    J2J4*q1J1*q3J3*Eij(2,11)+2*J1J4*q1J2*q3J3*Eij(2,11)+
     &    J1J2*q1J4*q3J3*Eij(2,11)+J1J4*q2J2*q3J3*Eij(2,11)+
     &    2*J1J2*q2J4*q3J3*Eij(2,11)+4*J1J3*q1J2*q3J4*Eij(2,11)+
     &    2*J1J2*q1J3*q3J4*Eij(2,11)-2*J2J3*q2J1*q3J4*Eij(2,11)+
     &    2*J1J3*q2J2*q3J4*Eij(2,11)+4*J1J2*q2J3*q3J4*Eij(2,11)+
     &    2*J1J2*q3J3*q3J4*Eij(2,11)-J1J2*J3J4*q3q4*Eij(2,11)-
     &    J1J2*J3J4*q3sq*Eij(2,11)-J2J4*q1J3*q4J1*Eij(2,11)+
     &    J2J3*q1J4*q4J1*Eij(2,11)-J3J4*q1J1*q4J2*Eij(2,11)+
     &    J1J4*q1J3*q4J2*Eij(2,11)-J3J4*q2J1*q4J2*Eij(2,11)+
     &    J1J3*q2J4*q4J2*Eij(2,11)+J2J4*q1J1*q4J3*Eij(2,11)+
     &    J2J4*q2J1*q4J3*Eij(2,11)+J1J2*q3J4*q4J3*Eij(2,11)+
     &    2*J1J3*q1J2*q4J4*Eij(2,11)+J1J2*q1J3*q4J4*Eij(2,11)-
     &    J2J3*q2J1*q4J4*Eij(2,11)+J1J3*q2J2*q4J4*Eij(2,11)+
     &    2*J1J2*q2J3*q4J4*Eij(2,11)+J1J2*q3J3*q4J4*Eij(2,11))


      HEmu=8*m*(-4*J1J4*q1J2*q1J3*q1q2*Eij(1,1)+J1J4*J2J3*q1q2**2*Eij(1,1)-
     &    J1J3*J2J4*q1q2**2*Eij(1,1)+J1J2*J3J4*q1q2**2*Eij(1,1)-
     &    4*J1J4*q1J2*q1J3*q1q3*Eij(1,1)+J1J4*J2J3*q1q2*q1q3*Eij(1,1)-
     &    J1J3*J2J4*q1q2*q1q3*Eij(1,1)+J1J2*J3J4*q1q2*q1q3*Eij(1,1)-
     &    4*J1J4*q1J2*q1J3*q1q4*Eij(1,1)+J1J4*J2J3*q1q2*q1q4*Eij(1,1)-
     &    J1J3*J2J4*q1q2*q1q4*Eij(1,1)+J1J2*J3J4*q1q2*q1q4*Eij(1,1)-
     &    3*J1J4*q1J2*q1J3*q1sq*Eij(1,1)-5*J1J2*q1J3*q1J4*q1sq*Eij(1,1)+
     &    4*J1J4*J2J3*q1q2*q1sq*Eij(1,1)-4*J1J3*J2J4*q1q2*q1sq*Eij(1,1)+
     &    4*J1J2*J3J4*q1q2*q1sq*Eij(1,1)+3*J1J4*J2J3*q1q3*q1sq*Eij(1,1)-
     &    3*J1J3*J2J4*q1q3*q1sq*Eij(1,1)+3*J1J2*J3J4*q1q3*q1sq*Eij(1,1)+
     &    J1J4*J2J3*q1q4*q1sq*Eij(1,1)-J1J3*J2J4*q1q4*q1sq*Eij(1,1)+
     &    J1J2*J3J4*q1q4*q1sq*Eij(1,1)+2*J1J4*J2J3*q1sq**2*Eij(1,1)-
     &    2*J1J3*J2J4*q1sq**2*Eij(1,1)+2*J1J2*J3J4*q1sq**2*Eij(1,1)-
     &    J1J4*q1J3*q1q2*q2J2*Eij(1,1)+J1J3*q1J4*q1q2*q2J2*Eij(1,1)-
     &    J1J4*q1J3*q1q3*q2J2*Eij(1,1)-J1J4*q1J3*q1q4*q2J2*Eij(1,1)-
     &    J1J4*q1J3*q1sq*q2J2*Eij(1,1)+J1J3*q1J4*q1sq*q2J2*Eij(1,1)-
     &    J1J4*q1J2*q1q2*q2J3*Eij(1,1)-J1J2*q1J4*q1q2*q2J3*Eij(1,1)-
     &    J1J4*q1J2*q1q3*q2J3*Eij(1,1)-2*J1J4*q1J2*q1q4*q2J3*Eij(1,1)-
     &    J1J4*q1J2*q1sq*q2J3*Eij(1,1)-5*J1J2*q1J4*q1sq*q2J3*Eij(1,1)-
     &    J1J3*q1J2*q1q2*q2J4*Eij(1,1)-J1J2*q1J3*q1q2*q2J4*Eij(1,1)-
     &    J1J3*q1J2*q1q3*q2J4*Eij(1,1)-J1J2*q1J3*q1q3*q2J4*Eij(1,1)-
     &    J1J2*q1J3*q1q4*q2J4*Eij(1,1)-J1J3*q1J2*q1sq*q2J4*Eij(1,1)-
     &    5*J1J2*q1J3*q1sq*q2J4*Eij(1,1)-4*J1J2*q1sq*q2J3*q2J4*Eij(1,1)-
     &    2*J1J4*q1J2*q1J3*q2q3*Eij(1,1)+J1J3*q1J2*q1J4*q2q3*Eij(1,1)+
     &    J1J2*q1J3*q1J4*q2q3*Eij(1,1)+2*J1J4*J2J3*q1sq*q2q3*Eij(1,1)-
     &    2*J1J3*J2J4*q1sq*q2q3*Eij(1,1)+2*J1J2*J3J4*q1sq*q2q3*Eij(1,1)-
     &    J1J4*q1J2*q1J3*q2q4*Eij(1,1)+J1J2*q1J3*q1J4*q2q4*Eij(1,1)-
     &    J1J4*q1J2*q1J3*q2sq*Eij(1,1)+J1J2*q1J3*q1J4*q2sq*Eij(1,1)+
     &    J1J4*J2J3*q1sq*q2sq*Eij(1,1)-J1J3*J2J4*q1sq*q2sq*Eij(1,1)+
     &    J1J2*J3J4*q1sq*q2sq*Eij(1,1)+4*q1J2*q1J3*q1J4*q3J1*Eij(1,1)-
     &    J2J3*q1J4*q1q2*q3J1*Eij(1,1)+J3J4*q1J2*q1q4*q3J1*Eij(1,1)-
     &    3*J2J3*q1J4*q1sq*q3J1*Eij(1,1)+q1J3*q1J4*q2J2*q3J1*Eij(1,1)+
     &    q1J2*q1J4*q2J3*q3J1*Eij(1,1)+2*q1J2*q1J3*q2J4*q3J1*Eij(1,1)-
     &    2*J2J3*q1sq*q2J4*q3J1*Eij(1,1)+J1J3*q1J4*q1q2*q3J2*Eij(1,1)+
     &    3*J1J3*q1J4*q1sq*q3J2*Eij(1,1)+2*J1J3*q1sq*q2J4*q3J2*Eij(1,1)-
     &    J1J2*q1J4*q1q2*q3J3*Eij(1,1)-J1J4*q1J2*q1q4*q3J3*Eij(1,1)-
     &    3*J1J2*q1J4*q1sq*q3J3*Eij(1,1)-2*J1J2*q1sq*q2J4*q3J3*Eij(1,1)-
     &    2*J1J3*q1J2*q1q2*q3J4*Eij(1,1)-2*J1J3*q1J2*q1q3*q3J4*Eij(1,1)-
     &    J1J3*q1J2*q1q4*q3J4*Eij(1,1)-2*J1J3*q1J2*q1sq*q3J4*Eij(1,1)-
     &    4*J1J2*q1J3*q1sq*q3J4*Eij(1,1)-4*J1J2*q1sq*q2J3*q3J4*Eij(1,1)+
     &    2*q1J2*q1J3*q3J1*q3J4*Eij(1,1)-2*J2J3*q1sq*q3J1*q3J4*Eij(1,1)+
     &    2*J1J3*q1sq*q3J2*q3J4*Eij(1,1)-2*J1J2*q1sq*q3J3*q3J4*Eij(1,1)-
     &    J1J4*q1J2*q1J3*q3q4*Eij(1,1)+J1J3*q1J2*q1J4*q3q4*Eij(1,1)+
     &    J1J4*J2J3*q1sq*q3q4*Eij(1,1)-J1J3*J2J4*q1sq*q3q4*Eij(1,1)+
     &    J1J2*J3J4*q1sq*q3q4*Eij(1,1)-J1J4*q1J2*q1J3*q3sq*Eij(1,1)+
     &    J1J3*q1J2*q1J4*q3sq*Eij(1,1)+J1J4*J2J3*q1sq*q3sq*Eij(1,1)-
     &    J1J3*J2J4*q1sq*q3sq*Eij(1,1)+J1J2*J3J4*q1sq*q3sq*Eij(1,1)+
     &    4*q1J2*q1J3*q1J4*q4J1*Eij(1,1)-J3J4*q1J2*q1q2*q4J1*Eij(1,1)-
     &    J2J3*q1J4*q1q2*q4J1*Eij(1,1)-J3J4*q1J2*q1q3*q4J1*Eij(1,1)-
     &    J3J4*q1J2*q1sq*q4J1*Eij(1,1)-2*J2J4*q1J3*q1sq*q4J1*Eij(1,1)-
     &    J2J3*q1J4*q1sq*q4J1*Eij(1,1)+q1J3*q1J4*q2J2*q4J1*Eij(1,1)+
     &    2*q1J2*q1J4*q2J3*q4J1*Eij(1,1)-2*J2J4*q1sq*q2J3*q4J1*Eij(1,1)+
     &    q1J2*q1J3*q2J4*q4J1*Eij(1,1)+J3J4*q1sq*q3J2*q4J1*Eij(1,1)+
     &    q1J2*q1J4*q3J3*q4J1*Eij(1,1)-J2J4*q1sq*q3J3*q4J1*Eij(1,1)+
     &    q1J2*q1J3*q3J4*q4J1*Eij(1,1)-J2J3*q1sq*q3J4*q4J1*Eij(1,1)+
     &    J1J3*q1J4*q1q2*q4J2*Eij(1,1)+2*J1J4*q1J3*q1sq*q4J2*Eij(1,1)+
     &    J1J3*q1J4*q1sq*q4J2*Eij(1,1)+2*J1J4*q1sq*q2J3*q4J2*Eij(1,1)-
     &    J3J4*q1sq*q3J1*q4J2*Eij(1,1)+J1J4*q1sq*q3J3*q4J2*Eij(1,1)+
     &    J1J3*q1sq*q3J4*q4J2*Eij(1,1)+J1J4*q1J2*q1q2*q4J3*Eij(1,1)-
     &    J1J2*q1J4*q1q2*q4J3*Eij(1,1)+J1J4*q1J2*q1q3*q4J3*Eij(1,1)+
     &    J1J4*q1J2*q1sq*q4J3*Eij(1,1)-J1J2*q1J4*q1sq*q4J3*Eij(1,1)-
     &    q1J2*q1J4*q3J1*q4J3*Eij(1,1)+J2J4*q1sq*q3J1*q4J3*Eij(1,1)-
     &    J1J4*q1sq*q3J2*q4J3*Eij(1,1)-J1J2*q1sq*q3J4*q4J3*Eij(1,1)-
     &    J1J3*q1J2*q1q2*q4J4*Eij(1,1)-J1J3*q1J2*q1q3*q4J4*Eij(1,1)-
     &    J1J3*q1J2*q1sq*q4J4*Eij(1,1)-2*J1J2*q1J3*q1sq*q4J4*Eij(1,1)-
     &    2*J1J2*q1sq*q2J3*q4J4*Eij(1,1)+q1J2*q1J3*q3J1*q4J4*Eij(1,1)-
     &    J2J3*q1sq*q3J1*q4J4*Eij(1,1)+J1J3*q1sq*q3J2*q4J4*Eij(1,1)-
     &    J1J2*q1sq*q3J3*q4J4*Eij(1,1)-2*J1J4*q1J2*q1J3*q1q2*Eij(1,2)-
     &    2*J1J2*q1J3*q1J4*q1q2*Eij(1,2)+3*J1J4*J2J3*q1q2**2*Eij(1,2)-
     &    3*J1J3*J2J4*q1q2**2*Eij(1,2)+3*J1J2*J3J4*q1q2**2*Eij(1,2)+
     &    2*J1J4*J2J3*q1q2*q1q3*Eij(1,2)-2*J1J3*J2J4*q1q2*q1q3*Eij(1,2)+
     &    2*J1J2*J3J4*q1q2*q1q3*Eij(1,2)+J1J4*J2J3*q1q2*q1q4*Eij(1,2)-
     &    J1J3*J2J4*q1q2*q1q4*Eij(1,2)+J1J2*J3J4*q1q2*q1q4*Eij(1,2)+
     &    2*J1J4*J2J3*q1q2*q1sq*Eij(1,2)-2*J1J3*J2J4*q1q2*q1sq*Eij(1,2)+
     &    2*J1J2*J3J4*q1q2*q1sq*Eij(1,2)-3*J1J4*q1J3*q1q2*q2J2*Eij(1,2)+
     &    3*J1J3*q1J4*q1q2*q2J2*Eij(1,2)-2*J1J4*q1J3*q1q3*q2J2*Eij(1,2)-
     &    2*J1J4*q1J3*q1q4*q2J2*Eij(1,2)-J1J4*q1J3*q1sq*q2J2*Eij(1,2)+
     &    J1J3*q1J4*q1sq*q2J2*Eij(1,2)-J1J4*q1J2*q1q2*q2J3*Eij(1,2)-
     &    5*J1J2*q1J4*q1q2*q2J3*Eij(1,2)-2*J1J4*q1J2*q1q4*q2J3*Eij(1,2)-
     &    2*J1J2*q1J4*q1sq*q2J3*Eij(1,2)-3*J1J4*q1q4*q2J2*q2J3*Eij(1,2)-
     &    3*J1J3*q1J2*q1q2*q2J4*Eij(1,2)-3*J1J2*q1J3*q1q2*q2J4*Eij(1,2)-
     &    2*J1J3*q1J2*q1q3*q2J4*Eij(1,2)-J1J3*q1J2*q1sq*q2J4*Eij(1,2)-
     &    J1J2*q1J3*q1sq*q2J4*Eij(1,2)-3*J1J3*q1q3*q2J2*q2J4*Eij(1,2)-
     &    8*J1J2*q1q2*q2J3*q2J4*Eij(1,2)-J1J2*q1q3*q2J3*q2J4*Eij(1,2)-
     &    J1J2*q1q4*q2J3*q2J4*Eij(1,2)-4*J1J2*q1sq*q2J3*q2J4*Eij(1,2)-
     &    2*J1J4*q1J2*q1J3*q2q3*Eij(1,2)+2*J1J3*q1J2*q1J4*q2q3*Eij(1,2)+
     &    3*J1J4*J2J3*q1q2*q2q3*Eij(1,2)-3*J1J3*J2J4*q1q2*q2q3*Eij(1,2)+
     &    3*J1J2*J3J4*q1q2*q2q3*Eij(1,2)+J1J4*J2J3*q1sq*q2q3*Eij(1,2)-
     &    J1J3*J2J4*q1sq*q2q3*Eij(1,2)+J1J2*J3J4*q1sq*q2q3*Eij(1,2)-
     &    3*J1J4*q1J3*q2J2*q2q3*Eij(1,2)+3*J1J3*q1J4*q2J2*q2q3*Eij(1,2)-
     &    J1J4*q1J2*q2J3*q2q3*Eij(1,2)+J1J2*q1J4*q2J3*q2q3*Eij(1,2)-
     &    J1J4*q1J2*q2J3*q2q4*Eij(1,2)+J1J2*q1J4*q2J3*q2q4*Eij(1,2)-
     &    2*J1J4*q1J2*q1J3*q2sq*Eij(1,2)+2*J1J2*q1J3*q1J4*q2sq*Eij(1,2)+
     &    2*J1J4*J2J3*q1q2*q2sq*Eij(1,2)-2*J1J3*J2J4*q1q2*q2sq*Eij(1,2)+
     &    2*J1J2*J3J4*q1q2*q2sq*Eij(1,2)+J1J4*J2J3*q1q4*q2sq*Eij(1,2)-
     &    J1J3*J2J4*q1q4*q2sq*Eij(1,2)+J1J2*J3J4*q1q4*q2sq*Eij(1,2)+
     &    J1J4*J2J3*q1sq*q2sq*Eij(1,2)-J1J3*J2J4*q1sq*q2sq*Eij(1,2)+
     &    J1J2*J3J4*q1sq*q2sq*Eij(1,2)-2*J1J4*q1J3*q2J2*q2sq*Eij(1,2)+
     &    2*J1J3*q1J4*q2J2*q2sq*Eij(1,2)-J1J4*q1J2*q2J3*q2sq*Eij(1,2)+
     &    J1J2*q1J4*q2J3*q2sq*Eij(1,2)-2*J1J3*q1J2*q2J4*q2sq*Eij(1,2)+
     &    2*J1J2*q1J3*q2J4*q2sq*Eij(1,2)-2*J2J3*q1J4*q1q2*q3J1*Eij(1,2)+
     &    2*q1J3*q1J4*q2J2*q3J1*Eij(1,2)+J3J4*q1q4*q2J2*q3J1*Eij(1,2)+
     &    2*q1J2*q1J3*q2J4*q3J1*Eij(1,2)-3*J2J3*q1q2*q2J4*q3J1*Eij(1,2)-
     &    J2J3*q1sq*q2J4*q3J1*Eij(1,2)+3*q1J3*q2J2*q2J4*q3J1*Eij(1,2)+
     &    q1J2*q2J3*q2J4*q3J1*Eij(1,2)+2*J1J3*q1J4*q1q2*q3J2*Eij(1,2)+
     &    3*J1J3*q1q2*q2J4*q3J2*Eij(1,2)+J1J3*q1sq*q2J4*q3J2*Eij(1,2)-
     &    2*J1J2*q1J4*q1q2*q3J3*Eij(1,2)-J1J4*q1q4*q2J2*q3J3*Eij(1,2)-
     &    3*J1J2*q1q2*q2J4*q3J3*Eij(1,2)-J1J2*q1sq*q2J4*q3J3*Eij(1,2)-
     &    2*J1J3*q1J2*q1q2*q3J4*Eij(1,2)-2*J1J2*q1J3*q1q2*q3J4*Eij(1,2)-
     &    2*J1J3*q1q3*q2J2*q3J4*Eij(1,2)-J1J3*q1q4*q2J2*q3J4*Eij(1,2)-
     &    6*J1J2*q1q2*q2J3*q3J4*Eij(1,2)-2*J1J2*q1sq*q2J3*q3J4*Eij(1,2)-
     &    2*J1J3*q1J2*q2sq*q3J4*Eij(1,2)+2*J1J2*q1J3*q2sq*q3J4*Eij(1,2)-
     &    2*J2J3*q1q2*q3J1*q3J4*Eij(1,2)+2*q1J3*q2J2*q3J1*q3J4*Eij(1,2)+
     &    2*J1J3*q1q2*q3J2*q3J4*Eij(1,2)-2*J1J2*q1q2*q3J3*q3J4*Eij(1,2)+
     &    J1J4*J2J3*q1q2*q3q4*Eij(1,2)-J1J3*J2J4*q1q2*q3q4*Eij(1,2)+
     &    J1J2*J3J4*q1q2*q3q4*Eij(1,2)-J1J4*q1J3*q2J2*q3q4*Eij(1,2)+
     &    J1J3*q1J4*q2J2*q3q4*Eij(1,2)+J1J4*J2J3*q1q2*q3sq*Eij(1,2)-
     &    J1J3*J2J4*q1q2*q3sq*Eij(1,2)+J1J2*J3J4*q1q2*q3sq*Eij(1,2)-
     &    J1J4*q1J3*q2J2*q3sq*Eij(1,2)+J1J3*q1J4*q2J2*q3sq*Eij(1,2)-
     &    J3J4*q1J2*q1q2*q4J1*Eij(1,2)-J2J4*q1J3*q1q2*q4J1*Eij(1,2)-
     &    J2J3*q1J4*q1q2*q4J1*Eij(1,2)+2*q1J3*q1J4*q2J2*q4J1*Eij(1,2)-
     &    J3J4*q1q3*q2J2*q4J1*Eij(1,2)+2*q1J2*q1J4*q2J3*q4J1*Eij(1,2)-
     &    3*J2J4*q1q2*q2J3*q4J1*Eij(1,2)-J2J4*q1sq*q2J3*q4J1*Eij(1,2)+
     &    3*q1J4*q2J2*q2J3*q4J1*Eij(1,2)+q1J2*q2J3*q2J4*q4J1*Eij(1,2)-
     &    J3J4*q1J2*q2sq*q4J1*Eij(1,2)+J2J4*q1J3*q2sq*q4J1*Eij(1,2)-
     &    J2J3*q1J4*q2sq*q4J1*Eij(1,2)+J3J4*q1q2*q3J2*q4J1*Eij(1,2)-
     &    J2J4*q1q2*q3J3*q4J1*Eij(1,2)+q1J4*q2J2*q3J3*q4J1*Eij(1,2)-
     &    J2J3*q1q2*q3J4*q4J1*Eij(1,2)+q1J3*q2J2*q3J4*q4J1*Eij(1,2)+
     &    J1J4*q1J3*q1q2*q4J2*Eij(1,2)+J1J3*q1J4*q1q2*q4J2*Eij(1,2)+
     &    3*J1J4*q1q2*q2J3*q4J2*Eij(1,2)+J1J4*q1sq*q2J3*q4J2*Eij(1,2)-
     &    J1J4*q1J3*q2sq*q4J2*Eij(1,2)+J1J3*q1J4*q2sq*q4J2*Eij(1,2)-
     &    J3J4*q1q2*q3J1*q4J2*Eij(1,2)+J1J4*q1q2*q3J3*q4J2*Eij(1,2)+
     &    J1J3*q1q2*q3J4*q4J2*Eij(1,2)+J1J4*q1J2*q1q2*q4J3*Eij(1,2)-
     &    J1J2*q1J4*q1q2*q4J3*Eij(1,2)+J1J4*q1q3*q2J2*q4J3*Eij(1,2)+
     &    J1J4*q1J2*q2sq*q4J3*Eij(1,2)-J1J2*q1J4*q2sq*q4J3*Eij(1,2)+
     &    J2J4*q1q2*q3J1*q4J3*Eij(1,2)-q1J4*q2J2*q3J1*q4J3*Eij(1,2)-
     &    J1J4*q1q2*q3J2*q4J3*Eij(1,2)-J1J2*q1q2*q3J4*q4J3*Eij(1,2)-
     &    J1J3*q1J2*q1q2*q4J4*Eij(1,2)-J1J2*q1J3*q1q2*q4J4*Eij(1,2)-
     &    J1J3*q1q3*q2J2*q4J4*Eij(1,2)-3*J1J2*q1q2*q2J3*q4J4*Eij(1,2)-
     &    J1J2*q1sq*q2J3*q4J4*Eij(1,2)-J1J3*q1J2*q2sq*q4J4*Eij(1,2)+
     &    J1J2*q1J3*q2sq*q4J4*Eij(1,2)-J2J3*q1q2*q3J1*q4J4*Eij(1,2)+
     &    q1J3*q2J2*q3J1*q4J4*Eij(1,2)+J1J3*q1q2*q3J2*q4J4*Eij(1,2)-
     &    J1J2*q1q2*q3J3*q4J4*Eij(1,2)-2*J1J4*q1J2*q1J3*q1q3*Eij(1,3)-
     &    2*J1J2*q1J3*q1J4*q1q3*Eij(1,3)+3*J1J4*J2J3*q1q2*q1q3*Eij(1,3)-
     &    3*J1J3*J2J4*q1q2*q1q3*Eij(1,3)+3*J1J2*J3J4*q1q2*q1q3*Eij(1,3)+
     &    2*J1J4*J2J3*q1q3**2*Eij(1,3)-2*J1J3*J2J4*q1q3**2*Eij(1,3)+
     &    2*J1J2*J3J4*q1q3**2*Eij(1,3)+J1J4*J2J3*q1q3*q1q4*Eij(1,3)-
     &    J1J3*J2J4*q1q3*q1q4*Eij(1,3)+J1J2*J3J4*q1q3*q1q4*Eij(1,3)+
     &    2*J1J4*J2J3*q1q3*q1sq*Eij(1,3)-2*J1J3*J2J4*q1q3*q1sq*Eij(1,3)+
     &    2*J1J2*J3J4*q1q3*q1sq*Eij(1,3)-J1J4*q1J3*q1q3*q2J2*Eij(1,3)+
     &    J1J3*q1J4*q1q3*q2J2*Eij(1,3)-J1J4*q1J2*q1q3*q2J3*Eij(1,3)-
     &    3*J1J2*q1J4*q1q3*q2J3*Eij(1,3)-J1J3*q1J2*q1q3*q2J4*Eij(1,3)-
     &    3*J1J2*q1J3*q1q3*q2J4*Eij(1,3)-4*J1J2*q1q3*q2J3*q2J4*Eij(1,3)-
     &    2*J1J4*q1J2*q1J3*q2q3*Eij(1,3)+2*J1J2*q1J3*q1J4*q2q3*Eij(1,3)+
     &    J1J4*J2J3*q1q2*q2q3*Eij(1,3)-J1J3*J2J4*q1q2*q2q3*Eij(1,3)+
     &    J1J2*J3J4*q1q2*q2q3*Eij(1,3)+2*J1J4*J2J3*q1q3*q2q3*Eij(1,3)-
     &    2*J1J3*J2J4*q1q3*q2q3*Eij(1,3)+2*J1J2*J3J4*q1q3*q2q3*Eij(1,3)+
     &    J1J4*J2J3*q1q4*q2q3*Eij(1,3)-J1J3*J2J4*q1q4*q2q3*Eij(1,3)+
     &    J1J2*J3J4*q1q4*q2q3*Eij(1,3)+J1J4*J2J3*q1sq*q2q3*Eij(1,3)-
     &    J1J3*J2J4*q1sq*q2q3*Eij(1,3)+J1J2*J3J4*q1sq*q2q3*Eij(1,3)-
     &    J1J4*q1J3*q2J2*q2q3*Eij(1,3)+J1J3*q1J4*q2J2*q2q3*Eij(1,3)-
     &    J1J4*q1J2*q2J3*q2q3*Eij(1,3)+J1J2*q1J4*q2J3*q2q3*Eij(1,3)-
     &    J1J3*q1J2*q2J4*q2q3*Eij(1,3)+J1J2*q1J3*q2J4*q2q3*Eij(1,3)+
     &    J1J4*J2J3*q1q3*q2sq*Eij(1,3)-J1J3*J2J4*q1q3*q2sq*Eij(1,3)+
     &    J1J2*J3J4*q1q3*q2sq*Eij(1,3)+8*q1J2*q1J3*q1J4*q3J1*Eij(1,3)-
     &    4*J3J4*q1J2*q1q2*q3J1*Eij(1,3)-4*J2J3*q1J4*q1q2*q3J1*Eij(1,3)-
     &    4*J3J4*q1J2*q1q3*q3J1*Eij(1,3)-2*J2J3*q1J4*q1q3*q3J1*Eij(1,3)-
     &    2*J3J4*q1J2*q1sq*q3J1*Eij(1,3)-2*J2J3*q1J4*q1sq*q3J1*Eij(1,3)+
     &    4*q1J3*q1J4*q2J2*q3J1*Eij(1,3)-2*J3J4*q1q2*q2J2*q3J1*Eij(1,3)-
     &    2*J3J4*q1q3*q2J2*q3J1*Eij(1,3)-J3J4*q1sq*q2J2*q3J1*Eij(1,3)+
     &    8*q1J2*q1J4*q2J3*q3J1*Eij(1,3)-2*J2J4*q1q2*q2J3*q3J1*Eij(1,3)-
     &    2*J2J4*q1q3*q2J3*q3J1*Eij(1,3)-J2J4*q1sq*q2J3*q3J1*Eij(1,3)+
     &    4*q1J4*q2J2*q2J3*q3J1*Eij(1,3)+8*q1J2*q1J3*q2J4*q3J1*Eij(1,3)-
     &    2*J2J3*q1q2*q2J4*q3J1*Eij(1,3)-J2J3*q1sq*q2J4*q3J1*Eij(1,3)+
     &    4*q1J3*q2J2*q2J4*q3J1*Eij(1,3)+8*q1J2*q2J3*q2J4*q3J1*Eij(1,3)+
     &    4*q2J2*q2J3*q2J4*q3J1*Eij(1,3)-4*J3J4*q1J2*q2q3*q3J1*Eij(1,3)-
     &    2*J2J3*q1J4*q2q3*q3J1*Eij(1,3)-2*J3J4*q2J2*q2q3*q3J1*Eij(1,3)-
     &    2*J2J4*q2J3*q2q3*q3J1*Eij(1,3)-2*J2J4*q1J3*q2q4*q3J1*Eij(1,3)-
     &    2*J2J4*q2J3*q2q4*q3J1*Eij(1,3)-2*J3J4*q1J2*q2sq*q3J1*Eij(1,3)-
     &    2*J2J3*q1J4*q2sq*q3J1*Eij(1,3)-J3J4*q2J2*q2sq*q3J1*Eij(1,3)-
     &    J2J4*q2J3*q2sq*q3J1*Eij(1,3)-J2J3*q2J4*q2sq*q3J1*Eij(1,3)-
     &    2*J1J4*q1J3*q1q2*q3J2*Eij(1,3)+2*J1J3*q1J4*q1q2*q3J2*Eij(1,3)-
     &    2*J1J4*q1J3*q1q3*q3J2*Eij(1,3)+2*J1J3*q1J4*q1q3*q3J2*Eij(1,3)-
     &    2*J1J4*q1J3*q1q4*q3J2*Eij(1,3)-J1J4*q1J3*q1sq*q3J2*Eij(1,3)+
     &    J1J3*q1J4*q1sq*q3J2*Eij(1,3)-2*J1J4*q1q4*q2J3*q3J2*Eij(1,3)-
     &    2*J1J4*q1J3*q2q3*q3J2*Eij(1,3)+2*J1J3*q1J4*q2q3*q3J2*Eij(1,3)-
     &    J1J4*q1J3*q2sq*q3J2*Eij(1,3)+J1J3*q1J4*q2sq*q3J2*Eij(1,3)+
     &    2*q1J3*q1J4*q3J1*q3J2*Eij(1,3)+J3J4*q1q4*q3J1*q3J2*Eij(1,3)+
     &    2*q1J4*q2J3*q3J1*q3J2*Eij(1,3)+2*q1J3*q2J4*q3J1*q3J2*Eij(1,3)+
     &    2*q2J3*q2J4*q3J1*q3J2*Eij(1,3)+J3J4*q2q4*q3J1*q3J2*Eij(1,3)-
     &    2*J1J2*q1J4*q1q2*q3J3*Eij(1,3)-2*J1J2*q1J4*q1q3*q3J3*Eij(1,3)-
     &    2*J1J4*q1J2*q1q4*q3J3*Eij(1,3)-2*J1J2*q1J4*q1sq*q3J3*Eij(1,3)-
     &    J1J4*q1q4*q2J2*q3J3*Eij(1,3)-2*J1J2*q1q2*q2J4*q3J3*Eij(1,3)-
     &    2*J1J2*q1q3*q2J4*q3J3*Eij(1,3)-J1J2*q1q4*q2J4*q3J3*Eij(1,3)-
     &    2*J1J2*q1sq*q2J4*q3J3*Eij(1,3)-J1J4*q1J2*q2q4*q3J3*Eij(1,3)+
     &    J1J2*q1J4*q2q4*q3J3*Eij(1,3)+4*q1J2*q1J4*q3J1*q3J3*Eij(1,3)+
     &    2*q1J4*q2J2*q3J1*q3J3*Eij(1,3)+4*q1J2*q2J4*q3J1*q3J3*Eij(1,3)+
     &    2*q2J2*q2J4*q3J1*q3J3*Eij(1,3)-J2J4*q2q4*q3J1*q3J3*Eij(1,3)-
     &    J1J4*q1q4*q3J2*q3J3*Eij(1,3)-2*J1J3*q1J2*q1q2*q3J4*Eij(1,3)-
     &    4*J1J3*q1J2*q1q3*q3J4*Eij(1,3)-2*J1J2*q1J3*q1q3*q3J4*Eij(1,3)-
     &    J1J3*q1J2*q1sq*q3J4*Eij(1,3)-J1J2*q1J3*q1sq*q3J4*Eij(1,3)-
     &    J1J3*q1q3*q2J2*q3J4*Eij(1,3)-2*J1J2*q1q2*q2J3*q3J4*Eij(1,3)-
     &    5*J1J2*q1q3*q2J3*q3J4*Eij(1,3)-2*J1J2*q1sq*q2J3*q3J4*Eij(1,3)-
     &    3*J1J3*q1J2*q2q3*q3J4*Eij(1,3)+3*J1J2*q1J3*q2q3*q3J4*Eij(1,3)-
     &    J1J3*q1J2*q2sq*q3J4*Eij(1,3)+J1J2*q1J3*q2sq*q3J4*Eij(1,3)+
     &    10*q1J2*q1J3*q3J1*q3J4*Eij(1,3)-5*J2J3*q1q2*q3J1*q3J4*Eij(1,3)-
     &    2*J2J3*q1q3*q3J1*q3J4*Eij(1,3)-3*J2J3*q1sq*q3J1*q3J4*Eij(1,3)+
     &    5*q1J3*q2J2*q3J1*q3J4*Eij(1,3)+9*q1J2*q2J3*q3J1*q3J4*Eij(1,3)+
     &    4*q2J2*q2J3*q3J1*q3J4*Eij(1,3)-2*J2J3*q2q3*q3J1*q3J4*Eij(1,3)-
     &    J2J3*q2q4*q3J1*q3J4*Eij(1,3)-2*J2J3*q2sq*q3J1*q3J4*Eij(1,3)+
     &    J1J3*q1q2*q3J2*q3J4*Eij(1,3)-J1J3*q1q4*q3J2*q3J4*Eij(1,3)+
     &    J1J3*q1sq*q3J2*q3J4*Eij(1,3)+2*q1J3*q3J1*q3J2*q3J4*Eij(1,3)+
     &    2*q2J3*q3J1*q3J2*q3J4*Eij(1,3)-3*J1J2*q1q2*q3J3*q3J4*Eij(1,3)-
     &    2*J1J2*q1q3*q3J3*q3J4*Eij(1,3)-3*J1J2*q1sq*q3J3*q3J4*Eij(1,3)+
     &    4*q1J2*q3J1*q3J3*q3J4*Eij(1,3)+2*q2J2*q3J1*q3J3*q3J4*Eij(1,3)+
     &    J1J4*J2J3*q1q3*q3q4*Eij(1,3)-J1J3*J2J4*q1q3*q3q4*Eij(1,3)+
     &    J1J2*J3J4*q1q3*q3q4*Eij(1,3)-2*J3J4*q1J2*q3J1*q3q4*Eij(1,3)-
     &    J3J4*q2J2*q3J1*q3q4*Eij(1,3)-J2J4*q2J3*q3J1*q3q4*Eij(1,3)+
     &    J2J3*q2J4*q3J1*q3q4*Eij(1,3)-J1J4*q1J3*q3J2*q3q4*Eij(1,3)+
     &    J1J3*q1J4*q3J2*q3q4*Eij(1,3)-2*J1J4*q1J2*q1J3*q3sq*Eij(1,3)+
     &    2*J1J3*q1J2*q1J4*q3sq*Eij(1,3)+J1J4*J2J3*q1q2*q3sq*Eij(1,3)-
     &    J1J3*J2J4*q1q2*q3sq*Eij(1,3)+J1J2*J3J4*q1q2*q3sq*Eij(1,3)+
     &    J1J4*J2J3*q1q3*q3sq*Eij(1,3)-J1J3*J2J4*q1q3*q3sq*Eij(1,3)+
     &    J1J2*J3J4*q1q3*q3sq*Eij(1,3)+J1J4*J2J3*q1sq*q3sq*Eij(1,3)-
     &    J1J3*J2J4*q1sq*q3sq*Eij(1,3)+J1J2*J3J4*q1sq*q3sq*Eij(1,3)-
     &    J1J4*q1J3*q2J2*q3sq*Eij(1,3)+J1J3*q1J4*q2J2*q3sq*Eij(1,3)-
     &    J1J4*q1J2*q2J3*q3sq*Eij(1,3)+J1J2*q1J4*q2J3*q3sq*Eij(1,3)+
     &    J1J3*q1J2*q2J4*q3sq*Eij(1,3)-J1J2*q1J3*q2J4*q3sq*Eij(1,3)-
     &    2*J3J4*q1J2*q3J1*q3sq*Eij(1,3)-J3J4*q2J2*q3J1*q3sq*Eij(1,3)-
     &    J2J4*q2J3*q3J1*q3sq*Eij(1,3)+J2J3*q2J4*q3J1*q3sq*Eij(1,3)-
     &    J1J4*q1J3*q3J2*q3sq*Eij(1,3)+J1J3*q1J4*q3J2*q3sq*Eij(1,3)-
     &    J3J4*q1J2*q1q3*q4J1*Eij(1,3)-J2J4*q1J3*q1q3*q4J1*Eij(1,3)-
     &    J2J3*q1J4*q1q3*q4J1*Eij(1,3)-2*J2J4*q1q3*q2J3*q4J1*Eij(1,3)-
     &    J3J4*q1J2*q2q3*q4J1*Eij(1,3)+J2J4*q1J3*q2q3*q4J1*Eij(1,3)-
     &    J2J3*q1J4*q2q3*q4J1*Eij(1,3)+2*q1J3*q1J4*q3J2*q4J1*Eij(1,3)+
     &    2*q1J4*q2J3*q3J2*q4J1*Eij(1,3)+2*q1J2*q1J4*q3J3*q4J1*Eij(1,3)-
     &    J2J4*q1q2*q3J3*q4J1*Eij(1,3)-J2J4*q1q3*q3J3*q4J1*Eij(1,3)-
     &    J2J4*q1sq*q3J3*q4J1*Eij(1,3)+q1J4*q2J2*q3J3*q4J1*Eij(1,3)+
     &    q1J2*q2J4*q3J3*q4J1*Eij(1,3)+q1J4*q3J2*q3J3*q4J1*Eij(1,3)-
     &    J2J3*q1q3*q3J4*q4J1*Eij(1,3)+q1J3*q3J2*q3J4*q4J1*Eij(1,3)+
     &    J1J4*q1J3*q1q3*q4J2*Eij(1,3)+J1J3*q1J4*q1q3*q4J2*Eij(1,3)+
     &    2*J1J4*q1q3*q2J3*q4J2*Eij(1,3)-J1J4*q1J3*q2q3*q4J2*Eij(1,3)+
     &    J1J3*q1J4*q2q3*q4J2*Eij(1,3)-2*J3J4*q1q2*q3J1*q4J2*Eij(1,3)-
     &    J3J4*q1q3*q3J1*q4J2*Eij(1,3)-J3J4*q1sq*q3J1*q4J2*Eij(1,3)+
     &    2*q1J3*q2J4*q3J1*q4J2*Eij(1,3)+2*q2J3*q2J4*q3J1*q4J2*Eij(1,3)-
     &    J3J4*q2q3*q3J1*q4J2*Eij(1,3)-J3J4*q2sq*q3J1*q4J2*Eij(1,3)+
     &    J1J4*q1q2*q3J3*q4J2*Eij(1,3)+J1J4*q1q3*q3J3*q4J2*Eij(1,3)+
     &    J1J4*q1sq*q3J3*q4J2*Eij(1,3)+q2J4*q3J1*q3J3*q4J2*Eij(1,3)+
     &    J1J3*q1q3*q3J4*q4J2*Eij(1,3)+q2J3*q3J1*q3J4*q4J2*Eij(1,3)+
     &    J1J4*q1J2*q1q3*q4J3*Eij(1,3)-J1J2*q1J4*q1q3*q4J3*Eij(1,3)+
     &    J1J4*q1J2*q2q3*q4J3*Eij(1,3)-J1J2*q1J4*q2q3*q4J3*Eij(1,3)+
     &    2*J2J4*q1q2*q3J1*q4J3*Eij(1,3)+J2J4*q1q3*q3J1*q4J3*Eij(1,3)+
     &    J2J4*q1sq*q3J1*q4J3*Eij(1,3)+J2J4*q2q3*q3J1*q4J3*Eij(1,3)+
     &    J2J4*q2sq*q3J1*q4J3*Eij(1,3)-q1J4*q3J1*q3J2*q4J3*Eij(1,3)-
     &    q2J4*q3J1*q3J2*q4J3*Eij(1,3)-J1J2*q1q3*q3J4*q4J3*Eij(1,3)+
     &    2*q1J2*q3J1*q3J4*q4J3*Eij(1,3)+q2J2*q3J1*q3J4*q4J3*Eij(1,3)-
     &    J1J3*q1J2*q1q3*q4J4*Eij(1,3)-J1J2*q1J3*q1q3*q4J4*Eij(1,3)-
     &    2*J1J2*q1q3*q2J3*q4J4*Eij(1,3)-J1J3*q1J2*q2q3*q4J4*Eij(1,3)+
     &    J1J2*q1J3*q2q3*q4J4*Eij(1,3)+4*q1J2*q1J3*q3J1*q4J4*Eij(1,3)-
     &    2*J2J3*q1q2*q3J1*q4J4*Eij(1,3)-J2J3*q1q3*q3J1*q4J4*Eij(1,3)-
     &    J2J3*q1sq*q3J1*q4J4*Eij(1,3)+2*q1J3*q2J2*q3J1*q4J4*Eij(1,3)+
     &    4*q1J2*q2J3*q3J1*q4J4*Eij(1,3)+2*q2J2*q2J3*q3J1*q4J4*Eij(1,3)-
     &    J2J3*q2q3*q3J1*q4J4*Eij(1,3)-J2J3*q2sq*q3J1*q4J4*Eij(1,3)+
     &    q1J3*q3J1*q3J2*q4J4*Eij(1,3)+q2J3*q3J1*q3J2*q4J4*Eij(1,3)-
     &    J1J2*q1q2*q3J3*q4J4*Eij(1,3)-J1J2*q1q3*q3J3*q4J4*Eij(1,3)-
     &    J1J2*q1sq*q3J3*q4J4*Eij(1,3)+2*q1J2*q3J1*q3J3*q4J4*Eij(1,3)+
     &    q2J2*q3J1*q3J3*q4J4*Eij(1,3)-2*J1J4*q1J2*q1J3*q1q4*Eij(1,4)-
     &    2*J1J2*q1J3*q1J4*q1q4*Eij(1,4)+3*J1J4*J2J3*q1q2*q1q4*Eij(1,4)-
     &    3*J1J3*J2J4*q1q2*q1q4*Eij(1,4)+3*J1J2*J3J4*q1q2*q1q4*Eij(1,4)+
     &    2*J1J4*J2J3*q1q3*q1q4*Eij(1,4)-2*J1J3*J2J4*q1q3*q1q4*Eij(1,4)+
     &    2*J1J2*J3J4*q1q3*q1q4*Eij(1,4)+J1J4*J2J3*q1q4**2*Eij(1,4)-
     &    J1J3*J2J4*q1q4**2*Eij(1,4)+J1J2*J3J4*q1q4**2*Eij(1,4)+
     &    2*J1J4*J2J3*q1q4*q1sq*Eij(1,4)-2*J1J3*J2J4*q1q4*q1sq*Eij(1,4)+
     &    2*J1J2*J3J4*q1q4*q1sq*Eij(1,4)-J1J4*q1J3*q1q4*q2J2*Eij(1,4)+
     &    J1J3*q1J4*q1q4*q2J2*Eij(1,4)-J1J4*q1J2*q1q4*q2J3*Eij(1,4)-
     &    3*J1J2*q1J4*q1q4*q2J3*Eij(1,4)-J1J3*q1J2*q1q4*q2J4*Eij(1,4)-
     &    3*J1J2*q1J3*q1q4*q2J4*Eij(1,4)-4*J1J2*q1q4*q2J3*q2J4*Eij(1,4)+
     &    2*J1J4*J2J3*q1q4*q2q3*Eij(1,4)-2*J1J3*J2J4*q1q4*q2q3*Eij(1,4)+
     &    2*J1J2*J3J4*q1q4*q2q3*Eij(1,4)-2*J1J4*q1J2*q1J3*q2q4*Eij(1,4)+
     &    2*J1J2*q1J3*q1J4*q2q4*Eij(1,4)+J1J4*J2J3*q1q2*q2q4*Eij(1,4)-
     &    J1J3*J2J4*q1q2*q2q4*Eij(1,4)+J1J2*J3J4*q1q2*q2q4*Eij(1,4)+
     &    J1J4*J2J3*q1q4*q2q4*Eij(1,4)-J1J3*J2J4*q1q4*q2q4*Eij(1,4)+
     &    J1J2*J3J4*q1q4*q2q4*Eij(1,4)+J1J4*J2J3*q1sq*q2q4*Eij(1,4)-
     &    J1J3*J2J4*q1sq*q2q4*Eij(1,4)+J1J2*J3J4*q1sq*q2q4*Eij(1,4)-
     &    J1J4*q1J3*q2J2*q2q4*Eij(1,4)+J1J3*q1J4*q2J2*q2q4*Eij(1,4)-
     &    J1J4*q1J2*q2J3*q2q4*Eij(1,4)+J1J2*q1J4*q2J3*q2q4*Eij(1,4)-
     &    J1J3*q1J2*q2J4*q2q4*Eij(1,4)+J1J2*q1J3*q2J4*q2q4*Eij(1,4)+
     &    J1J4*J2J3*q1q4*q2sq*Eij(1,4)-J1J3*J2J4*q1q4*q2sq*Eij(1,4)+
     &    J1J2*J3J4*q1q4*q2sq*Eij(1,4)-2*J2J3*q1J4*q1q4*q3J1*Eij(1,4)-
     &    2*J2J3*q1q4*q2J4*q3J1*Eij(1,4)+2*J1J3*q1J4*q1q4*q3J2*Eij(1,4)+
     &    2*J1J3*q1q4*q2J4*q3J2*Eij(1,4)-2*J1J2*q1J4*q1q4*q3J3*Eij(1,4)-
     &    2*J1J2*q1q4*q2J4*q3J3*Eij(1,4)-2*J1J3*q1J2*q1q4*q3J4*Eij(1,4)-
     &    2*J1J2*q1J3*q1q4*q3J4*Eij(1,4)-4*J1J2*q1q4*q2J3*q3J4*Eij(1,4)-
     &    2*J1J3*q1J2*q2q4*q3J4*Eij(1,4)+2*J1J2*q1J3*q2q4*q3J4*Eij(1,4)-
     &    2*J2J3*q1q4*q3J1*q3J4*Eij(1,4)+2*J1J3*q1q4*q3J2*q3J4*Eij(1,4)-
     &    2*J1J2*q1q4*q3J3*q3J4*Eij(1,4)-2*J1J4*q1J2*q1J3*q3q4*Eij(1,4)+
     &    2*J1J3*q1J2*q1J4*q3q4*Eij(1,4)+J1J4*J2J3*q1q2*q3q4*Eij(1,4)-
     &    J1J3*J2J4*q1q2*q3q4*Eij(1,4)+J1J2*J3J4*q1q2*q3q4*Eij(1,4)+
     &    J1J4*J2J3*q1q4*q3q4*Eij(1,4)-J1J3*J2J4*q1q4*q3q4*Eij(1,4)+
     &    J1J2*J3J4*q1q4*q3q4*Eij(1,4)+J1J4*J2J3*q1sq*q3q4*Eij(1,4)-
     &    J1J3*J2J4*q1sq*q3q4*Eij(1,4)+J1J2*J3J4*q1sq*q3q4*Eij(1,4)-
     &    J1J4*q1J3*q2J2*q3q4*Eij(1,4)+J1J3*q1J4*q2J2*q3q4*Eij(1,4)-
     &    J1J4*q1J2*q2J3*q3q4*Eij(1,4)+J1J2*q1J4*q2J3*q3q4*Eij(1,4)+
     &    J1J3*q1J2*q2J4*q3q4*Eij(1,4)-J1J2*q1J3*q2J4*q3q4*Eij(1,4)+
     &    J1J4*J2J3*q1q4*q3sq*Eij(1,4)-J1J3*J2J4*q1q4*q3sq*Eij(1,4)+
     &    J1J2*J3J4*q1q4*q3sq*Eij(1,4)+8*q1J2*q1J3*q1J4*q4J1*Eij(1,4)-
     &    4*J3J4*q1J2*q1q2*q4J1*Eij(1,4)-4*J2J3*q1J4*q1q2*q4J1*Eij(1,4)-
     &    4*J3J4*q1J2*q1q3*q4J1*Eij(1,4)-J3J4*q1J2*q1q4*q4J1*Eij(1,4)-
     &    J2J4*q1J3*q1q4*q4J1*Eij(1,4)-J2J3*q1J4*q1q4*q4J1*Eij(1,4)-
     &    2*J3J4*q1J2*q1sq*q4J1*Eij(1,4)-2*J2J3*q1J4*q1sq*q4J1*Eij(1,4)+
     &    4*q1J3*q1J4*q2J2*q4J1*Eij(1,4)-2*J3J4*q1q2*q2J2*q4J1*Eij(1,4)-
     &    2*J3J4*q1q3*q2J2*q4J1*Eij(1,4)-J3J4*q1sq*q2J2*q4J1*Eij(1,4)+
     &    8*q1J2*q1J4*q2J3*q4J1*Eij(1,4)-2*J2J4*q1q2*q2J3*q4J1*Eij(1,4)-
     &    2*J2J4*q1q3*q2J3*q4J1*Eij(1,4)-2*J2J4*q1q4*q2J3*q4J1*Eij(1,4)-
     &    J2J4*q1sq*q2J3*q4J1*Eij(1,4)+4*q1J4*q2J2*q2J3*q4J1*Eij(1,4)+
     &    8*q1J2*q1J3*q2J4*q4J1*Eij(1,4)-2*J2J3*q1q2*q2J4*q4J1*Eij(1,4)+
     &    2*J2J3*q1q3*q2J4*q4J1*Eij(1,4)-J2J3*q1sq*q2J4*q4J1*Eij(1,4)+
     &    4*q1J3*q2J2*q2J4*q4J1*Eij(1,4)+8*q1J2*q2J3*q2J4*q4J1*Eij(1,4)+
     &    4*q2J2*q2J3*q2J4*q4J1*Eij(1,4)-4*J3J4*q1J2*q2q3*q4J1*Eij(1,4)-
     &    2*J2J3*q1J4*q2q3*q4J1*Eij(1,4)-2*J3J4*q2J2*q2q3*q4J1*Eij(1,4)-
     &    2*J2J4*q2J3*q2q3*q4J1*Eij(1,4)-J3J4*q1J2*q2q4*q4J1*Eij(1,4)-
     &    J2J4*q1J3*q2q4*q4J1*Eij(1,4)-J2J3*q1J4*q2q4*q4J1*Eij(1,4)-
     &    2*J2J4*q2J3*q2q4*q4J1*Eij(1,4)-2*J3J4*q1J2*q2sq*q4J1*Eij(1,4)-
     &    2*J2J3*q1J4*q2sq*q4J1*Eij(1,4)-J3J4*q2J2*q2sq*q4J1*Eij(1,4)-
     &    J2J4*q2J3*q2sq*q4J1*Eij(1,4)-J2J3*q2J4*q2sq*q4J1*Eij(1,4)+
     &    J3J4*q1q4*q3J2*q4J1*Eij(1,4)+2*q1J4*q2J3*q3J2*q4J1*Eij(1,4)+
     &    2*q2J3*q2J4*q3J2*q4J1*Eij(1,4)+J3J4*q2q4*q3J2*q4J1*Eij(1,4)+
     &    4*q1J2*q1J4*q3J3*q4J1*Eij(1,4)-J2J4*q1q4*q3J3*q4J1*Eij(1,4)+
     &    2*q1J4*q2J2*q3J3*q4J1*Eij(1,4)+4*q1J2*q2J4*q3J3*q4J1*Eij(1,4)+
     &    2*q2J2*q2J4*q3J3*q4J1*Eij(1,4)-J2J4*q2q4*q3J3*q4J1*Eij(1,4)+
     &    8*q1J2*q1J3*q3J4*q4J1*Eij(1,4)-4*J2J3*q1q2*q3J4*q4J1*Eij(1,4)-
     &    J2J3*q1q4*q3J4*q4J1*Eij(1,4)-2*J2J3*q1sq*q3J4*q4J1*Eij(1,4)+
     &    4*q1J3*q2J2*q3J4*q4J1*Eij(1,4)+8*q1J2*q2J3*q3J4*q4J1*Eij(1,4)+
     &    4*q2J2*q2J3*q3J4*q4J1*Eij(1,4)-2*J2J3*q2q3*q3J4*q4J1*Eij(1,4)-
     &    J2J3*q2q4*q3J4*q4J1*Eij(1,4)-2*J2J3*q2sq*q3J4*q4J1*Eij(1,4)+
     &    2*q2J3*q3J2*q3J4*q4J1*Eij(1,4)+4*q1J2*q3J3*q3J4*q4J1*Eij(1,4)+
     &    2*q2J2*q3J3*q3J4*q4J1*Eij(1,4)-2*J3J4*q1J2*q3q4*q4J1*Eij(1,4)-
     &    J3J4*q2J2*q3q4*q4J1*Eij(1,4)-J2J4*q2J3*q3q4*q4J1*Eij(1,4)+
     &    J2J3*q2J4*q3q4*q4J1*Eij(1,4)-2*J3J4*q1J2*q3sq*q4J1*Eij(1,4)-
     &    J3J4*q2J2*q3sq*q4J1*Eij(1,4)-J2J4*q2J3*q3sq*q4J1*Eij(1,4)+
     &    J2J3*q2J4*q3sq*q4J1*Eij(1,4)-2*J1J4*q1J3*q1q2*q4J2*Eij(1,4)+
     &    2*J1J3*q1J4*q1q2*q4J2*Eij(1,4)-2*J1J4*q1J3*q1q3*q4J2*Eij(1,4)-
     &    J1J4*q1J3*q1q4*q4J2*Eij(1,4)+J1J3*q1J4*q1q4*q4J2*Eij(1,4)-
     &    J1J4*q1J3*q1sq*q4J2*Eij(1,4)+J1J3*q1J4*q1sq*q4J2*Eij(1,4)-
     &    2*J1J3*q1q3*q2J4*q4J2*Eij(1,4)-2*J1J4*q1J3*q2q3*q4J2*Eij(1,4)+
     &    2*J1J3*q1J4*q2q3*q4J2*Eij(1,4)-J1J4*q1J3*q2q4*q4J2*Eij(1,4)+
     &    J1J3*q1J4*q2q4*q4J2*Eij(1,4)-J1J4*q1J3*q2sq*q4J2*Eij(1,4)+
     &    J1J3*q1J4*q2sq*q4J2*Eij(1,4)+2*q1J3*q1J4*q3J1*q4J2*Eij(1,4)+
     &    2*q1J3*q2J4*q3J1*q4J2*Eij(1,4)-2*J1J3*q1q3*q3J4*q4J2*Eij(1,4)+
     &    2*q1J3*q3J1*q3J4*q4J2*Eij(1,4)-J1J4*q1J3*q3q4*q4J2*Eij(1,4)+
     &    J1J3*q1J4*q3q4*q4J2*Eij(1,4)-J1J4*q1J3*q3sq*q4J2*Eij(1,4)+
     &    J1J3*q1J4*q3sq*q4J2*Eij(1,4)+2*q1J3*q1J4*q4J1*q4J2*Eij(1,4)-
     &    2*J3J4*q1q2*q4J1*q4J2*Eij(1,4)-J3J4*q1q3*q4J1*q4J2*Eij(1,4)-
     &    J3J4*q1sq*q4J1*q4J2*Eij(1,4)+2*q1J4*q2J3*q4J1*q4J2*Eij(1,4)+
     &    2*q1J3*q2J4*q4J1*q4J2*Eij(1,4)+2*q2J3*q2J4*q4J1*q4J2*Eij(1,4)-
     &    J3J4*q2q3*q4J1*q4J2*Eij(1,4)-J3J4*q2sq*q4J1*q4J2*Eij(1,4)+
     &    q1J4*q3J3*q4J1*q4J2*Eij(1,4)+q2J4*q3J3*q4J1*q4J2*Eij(1,4)+
     &    q1J3*q3J4*q4J1*q4J2*Eij(1,4)+q2J3*q3J4*q4J1*q4J2*Eij(1,4)-
     &    2*J1J2*q1J4*q1q2*q4J3*Eij(1,4)-J1J4*q1J2*q1q4*q4J3*Eij(1,4)-
     &    J1J2*q1J4*q1q4*q4J3*Eij(1,4)-2*J1J2*q1J4*q1sq*q4J3*Eij(1,4)-
     &    J1J4*q1q4*q2J2*q4J3*Eij(1,4)-2*J1J2*q1q2*q2J4*q4J3*Eij(1,4)-
     &    J1J2*q1q4*q2J4*q4J3*Eij(1,4)-2*J1J2*q1sq*q2J4*q4J3*Eij(1,4)+
     &    J2J4*q1q4*q3J1*q4J3*Eij(1,4)-J1J4*q1q4*q3J2*q4J3*Eij(1,4)-
     &    2*J1J2*q1q2*q3J4*q4J3*Eij(1,4)-J1J2*q1q4*q3J4*q4J3*Eij(1,4)-
     &    2*J1J2*q1sq*q3J4*q4J3*Eij(1,4)+2*q1J2*q1J4*q4J1*q4J3*Eij(1,4)+
     &    J2J4*q1q2*q4J1*q4J3*Eij(1,4)+q1J4*q2J2*q4J1*q4J3*Eij(1,4)+
     &    q1J2*q2J4*q4J1*q4J3*Eij(1,4)+J2J4*q2q3*q4J1*q4J3*Eij(1,4)+
     &    J2J4*q2sq*q4J1*q4J3*Eij(1,4)-q2J4*q3J2*q4J1*q4J3*Eij(1,4)+
     &    2*q1J2*q3J4*q4J1*q4J3*Eij(1,4)+q2J2*q3J4*q4J1*q4J3*Eij(1,4)+
     &    J1J4*q1q2*q4J2*q4J3*Eij(1,4)+J1J4*q1q3*q4J2*q4J3*Eij(1,4)+
     &    J1J4*q1sq*q4J2*q4J3*Eij(1,4)-q1J4*q3J1*q4J2*q4J3*Eij(1,4)-
     &    2*J1J3*q1J2*q1q2*q4J4*Eij(1,4)-2*J1J3*q1J2*q1q3*q4J4*Eij(1,4)-
     &    J1J3*q1J2*q1q4*q4J4*Eij(1,4)-J1J2*q1J3*q1q4*q4J4*Eij(1,4)-
     &    J1J3*q1J2*q1sq*q4J4*Eij(1,4)-J1J2*q1J3*q1sq*q4J4*Eij(1,4)-
     &    J1J3*q1q3*q2J2*q4J4*Eij(1,4)-2*J1J2*q1q2*q2J3*q4J4*Eij(1,4)-
     &    J1J2*q1q3*q2J3*q4J4*Eij(1,4)-2*J1J2*q1q4*q2J3*q4J4*Eij(1,4)-
     &    2*J1J2*q1sq*q2J3*q4J4*Eij(1,4)-J1J3*q1J2*q2q3*q4J4*Eij(1,4)+
     &    J1J2*q1J3*q2q3*q4J4*Eij(1,4)-J1J3*q1J2*q2q4*q4J4*Eij(1,4)+
     &    J1J2*q1J3*q2q4*q4J4*Eij(1,4)-J1J3*q1J2*q2sq*q4J4*Eij(1,4)+
     &    J1J2*q1J3*q2sq*q4J4*Eij(1,4)+2*q1J2*q1J3*q3J1*q4J4*Eij(1,4)-
     &    J2J3*q1q2*q3J1*q4J4*Eij(1,4)-J2J3*q1q4*q3J1*q4J4*Eij(1,4)-
     &    J2J3*q1sq*q3J1*q4J4*Eij(1,4)+q1J3*q2J2*q3J1*q4J4*Eij(1,4)+
     &    q1J2*q2J3*q3J1*q4J4*Eij(1,4)+J1J3*q1q2*q3J2*q4J4*Eij(1,4)+
     &    J1J3*q1q4*q3J2*q4J4*Eij(1,4)+J1J3*q1sq*q3J2*q4J4*Eij(1,4)-
     &    J1J2*q1q2*q3J3*q4J4*Eij(1,4)-J1J2*q1q4*q3J3*q4J4*Eij(1,4)-
     &    J1J2*q1sq*q3J3*q4J4*Eij(1,4)+4*q1J2*q1J3*q4J1*q4J4*Eij(1,4)-
     &    2*J2J3*q1q2*q4J1*q4J4*Eij(1,4)-J2J3*q1sq*q4J1*q4J4*Eij(1,4)+
     &    2*q1J3*q2J2*q4J1*q4J4*Eij(1,4)+4*q1J2*q2J3*q4J1*q4J4*Eij(1,4)+
     &    2*q2J2*q2J3*q4J1*q4J4*Eij(1,4)-J2J3*q2q3*q4J1*q4J4*Eij(1,4)-
     &    J2J3*q2sq*q4J1*q4J4*Eij(1,4)+q2J3*q3J2*q4J1*q4J4*Eij(1,4)+
     &    2*q1J2*q3J3*q4J1*q4J4*Eij(1,4)+q2J2*q3J3*q4J1*q4J4*Eij(1,4)-
     &    J1J3*q1q3*q4J2*q4J4*Eij(1,4)+q1J3*q3J1*q4J2*q4J4*Eij(1,4)-
     &    J1J2*q1q2*q4J3*q4J4*Eij(1,4)-J1J2*q1sq*q4J3*q4J4*Eij(1,4)+
     &    q1J1*(-8*J2J3*q1J4*q1q2*Eij(1,1)-5*J2J3*q1J4*q1sq*Eij(1,1)-
     &       3*J2J4*q1q2*q2J3*Eij(1,1)-3*J2J4*q1q3*q2J3*Eij(1,1)-
     &       2*J2J4*q1sq*q2J3*Eij(1,1)-3*J2J3*q1q2*q2J4*Eij(1,1)+
     &       3*J2J3*q1q3*q2J4*Eij(1,1)-2*J2J3*q1sq*q2J4*Eij(1,1)-
     &       3*J2J3*q1J4*q2q3*Eij(1,1)-2*J2J4*q2J3*q2q3*Eij(1,1)-
     &       3*J2J4*q1J3*q2q4*Eij(1,1)-2*J2J4*q2J3*q2q4*Eij(1,1)-
     &       3*J2J3*q1J4*q2sq*Eij(1,1)-J2J4*q2J3*q2sq*Eij(1,1)-
     &       J2J3*q2J4*q2sq*Eij(1,1)+3*q1J4*q2J3*q3J2*Eij(1,1)+
     &       2*q2J3*q2J4*q3J2*Eij(1,1)+J3J4*q2q4*q3J2*Eij(1,1)-
     &       J2J4*q2q4*q3J3*Eij(1,1)-6*J2J3*q1q2*q3J4*Eij(1,1)-
     &       4*J2J3*q1sq*q3J4*Eij(1,1)-2*J2J3*q2q3*q3J4*Eij(1,1)-
     &       J2J3*q2q4*q3J4*Eij(1,1)-2*J2J3*q2sq*q3J4*Eij(1,1)+
     &       2*q2J3*q3J2*q3J4*Eij(1,1)-J2J4*q2J3*q3q4*Eij(1,1)+
     &       J2J3*q2J4*q3q4*Eij(1,1)-J2J4*q2J3*q3sq*Eij(1,1)+
     &       J2J3*q2J4*q3sq*Eij(1,1)-3*J3J4*q1q2*q4J2*Eij(1,1)-
     &       2*J3J4*q1sq*q4J2*Eij(1,1)+3*q1J3*q2J4*q4J2*Eij(1,1)+
     &       2*q2J3*q2J4*q4J2*Eij(1,1)-J3J4*q2q3*q4J2*Eij(1,1)-
     &       J3J4*q2sq*q4J2*Eij(1,1)+q2J4*q3J3*q4J2*Eij(1,1)+
     &       q2J3*q3J4*q4J2*Eij(1,1)+3*J2J4*q1q2*q4J3*Eij(1,1)+
     &       2*J2J4*q1sq*q4J3*Eij(1,1)+J2J4*q2q3*q4J3*Eij(1,1)+
     &       J2J4*q2sq*q4J3*Eij(1,1)-q2J4*q3J2*q4J3*Eij(1,1)-
     &       3*J2J3*q1q2*q4J4*Eij(1,1)-2*J2J3*q1sq*q4J4*Eij(1,1)-
     &       J2J3*q2q3*q4J4*Eij(1,1)-J2J3*q2sq*q4J4*Eij(1,1)+
     &       q2J3*q3J2*q4J4*Eij(1,1)-2*J2J3*q1J4*q1q2*Eij(1,2)-
     &       J2J4*q1q2*q2J3*Eij(1,2)-3*J2J3*q1q2*q2J4*Eij(1,2)-
     &       J2J3*q1sq*q2J4*Eij(1,2)-J2J4*q2J3*q2q3*Eij(1,2)-
     &       J2J4*q2J3*q2q4*Eij(1,2)-2*J2J3*q1J4*q2sq*Eij(1,2)-
     &       J2J4*q2J3*q2sq*Eij(1,2)-2*J2J3*q2J4*q2sq*Eij(1,2)+
     &       q2J3*q2J4*q3J2*Eij(1,2)-2*J2J3*q1q2*q3J4*Eij(1,2)-
     &       2*J2J3*q2sq*q3J4*Eij(1,2)-J3J4*q1q2*q4J2*Eij(1,2)+
     &       q2J3*q2J4*q4J2*Eij(1,2)-J3J4*q2sq*q4J2*Eij(1,2)+
     &       J2J4*q1q2*q4J3*Eij(1,2)+J2J4*q2sq*q4J3*Eij(1,2)-
     &       J2J3*q1q2*q4J4*Eij(1,2)-J2J3*q2sq*q4J4*Eij(1,2)-
     &       2*J2J3*q1J4*q1q3*Eij(1,3)-J2J4*q1q3*q2J3*Eij(1,3)-
     &       J2J3*q1q3*q2J4*Eij(1,3)-2*J2J3*q1J4*q2q3*Eij(1,3)-
     &       J2J4*q2J3*q2q3*Eij(1,3)-J2J3*q2J4*q2q3*Eij(1,3)+
     &       4*q1J3*q1J4*q3J2*Eij(1,3)-2*J3J4*q1q2*q3J2*Eij(1,3)-
     &       2*J3J4*q1q3*q3J2*Eij(1,3)-J3J4*q1sq*q3J2*Eij(1,3)+
     &       4*q1J4*q2J3*q3J2*Eij(1,3)+4*q1J3*q2J4*q3J2*Eij(1,3)+
     &       4*q2J3*q2J4*q3J2*Eij(1,3)-2*J3J4*q2q3*q3J2*Eij(1,3)-
     &       J3J4*q2sq*q3J2*Eij(1,3)-J2J4*q2q4*q3J3*Eij(1,3)+
     &       2*q1J4*q3J2*q3J3*Eij(1,3)+2*q2J4*q3J2*q3J3*Eij(1,3)-
     &       2*J2J3*q1q2*q3J4*Eij(1,3)-2*J2J3*q1q3*q3J4*Eij(1,3)-
     &       J2J3*q1sq*q3J4*Eij(1,3)-3*J2J3*q2q3*q3J4*Eij(1,3)-
     &       J2J3*q2sq*q3J4*Eij(1,3)+4*q1J3*q3J2*q3J4*Eij(1,3)+
     &       5*q2J3*q3J2*q3J4*Eij(1,3)+2*q3J2*q3J3*q3J4*Eij(1,3)-
     &       J3J4*q3J2*q3q4*Eij(1,3)-J2J4*q2J3*q3sq*Eij(1,3)+
     &       J2J3*q2J4*q3sq*Eij(1,3)-J3J4*q3J2*q3sq*Eij(1,3)-
     &       J3J4*q1q3*q4J2*Eij(1,3)-J3J4*q2q3*q4J2*Eij(1,3)+
     &       q2J4*q3J3*q4J2*Eij(1,3)+J2J4*q1q3*q4J3*Eij(1,3)+
     &       J2J4*q2q3*q4J3*Eij(1,3)+q3J2*q3J4*q4J3*Eij(1,3)-
     &       J2J3*q1q3*q4J4*Eij(1,3)-J2J3*q2q3*q4J4*Eij(1,3)+
     &       2*q1J3*q3J2*q4J4*Eij(1,3)+2*q2J3*q3J2*q4J4*Eij(1,3)+
     &       q3J2*q3J3*q4J4*Eij(1,3)-2*J2J3*q1J4*q1q4*Eij(1,4)-
     &       J2J4*q1q4*q2J3*Eij(1,4)-J2J3*q1q4*q2J4*Eij(1,4)-
     &       2*J2J3*q1J4*q2q4*Eij(1,4)-J2J4*q2J3*q2q4*Eij(1,4)-
     &       J2J3*q2J4*q2q4*Eij(1,4)-2*J2J3*q1q4*q3J4*Eij(1,4)-
     &       2*J2J3*q2q4*q3J4*Eij(1,4)-J2J4*q2J3*q3q4*Eij(1,4)+
     &       J2J3*q2J4*q3q4*Eij(1,4)+4*q1J3*q1J4*q4J2*Eij(1,4)-
     &       2*J3J4*q1q2*q4J2*Eij(1,4)-2*J3J4*q1q3*q4J2*Eij(1,4)-
     &       J3J4*q1q4*q4J2*Eij(1,4)-J3J4*q1sq*q4J2*Eij(1,4)+
     &       4*q1J4*q2J3*q4J2*Eij(1,4)+4*q1J3*q2J4*q4J2*Eij(1,4)+
     &       4*q2J3*q2J4*q4J2*Eij(1,4)-2*J3J4*q2q3*q4J2*Eij(1,4)-
     &       J3J4*q2q4*q4J2*Eij(1,4)-J3J4*q2sq*q4J2*Eij(1,4)+
     &       2*q1J4*q3J3*q4J2*Eij(1,4)+2*q2J4*q3J3*q4J2*Eij(1,4)+
     &       4*q1J3*q3J4*q4J2*Eij(1,4)+4*q2J3*q3J4*q4J2*Eij(1,4)+
     &       2*q3J3*q3J4*q4J2*Eij(1,4)-J3J4*q3q4*q4J2*Eij(1,4)-
     &       J3J4*q3sq*q4J2*Eij(1,4)+J2J4*q1q4*q4J3*Eij(1,4)+
     &       q2J4*q4J2*q4J3*Eij(1,4)+q3J4*q4J2*q4J3*Eij(1,4)-
     &       2*J2J3*q1q2*q4J4*Eij(1,4)-J2J3*q1q4*q4J4*Eij(1,4)-
     &       J2J3*q1sq*q4J4*Eij(1,4)-J2J3*q2q3*q4J4*Eij(1,4)-
     &       J2J3*q2q4*q4J4*Eij(1,4)-J2J3*q2sq*q4J4*Eij(1,4)+
     &       q2J3*q3J2*q4J4*Eij(1,4)+2*q1J3*q4J2*q4J4*Eij(1,4)+
     &       2*q2J3*q4J2*q4J4*Eij(1,4)+q3J3*q4J2*q4J4*Eij(1,4)+
     &       q2J2*(-3*J3J4*q1q2*Eij(1,1)-3*J3J4*q1q3*Eij(1,1)-
     &          2*J3J4*q1sq*Eij(1,1)-2*J3J4*q2q3*Eij(1,1)-
     &          J3J4*q2sq*Eij(1,1)+3*q1J4*q3J3*Eij(1,1)+
     &          2*q2J4*q3J3*Eij(1,1)+2*q3J3*q3J4*Eij(1,1)-
     &          J3J4*q3q4*Eij(1,1)-J3J4*q3sq*Eij(1,1)+q3J4*q4J3*Eij(1,1)+
     &          q3J3*q4J4*Eij(1,1)-3*J3J4*q1q2*Eij(1,2)-
     &          2*J3J4*q1q3*Eij(1,2)-J3J4*q1sq*Eij(1,2)-
     &          3*J3J4*q2q3*Eij(1,2)-2*J3J4*q2sq*Eij(1,2)+
     &          2*q1J4*q3J3*Eij(1,2)+3*q2J4*q3J3*Eij(1,2)+
     &          2*q3J3*q3J4*Eij(1,2)-J3J4*q3q4*Eij(1,2)-
     &          J3J4*q3sq*Eij(1,2)+q3J4*q4J3*Eij(1,2)+q3J3*q4J4*Eij(1,2)-
     &          J3J4*q1q3*Eij(1,3)-J3J4*q2q3*Eij(1,3)+
     &          2*q1J4*q3J3*Eij(1,3)+2*q2J4*q3J3*Eij(1,3)+
     &          3*q3J3*q3J4*Eij(1,3)-J3J4*q3sq*Eij(1,3)+
     &          q3J3*q4J4*Eij(1,3)-J3J4*q1q4*Eij(1,4)-J3J4*q2q4*Eij(1,4)-
     &          J3J4*q3q4*Eij(1,4)+2*q1J4*q4J3*Eij(1,4)+
     &          2*q2J4*q4J3*Eij(1,4)+2*q3J4*q4J3*Eij(1,4)+
     &          q3J3*q4J4*Eij(1,4)+q4J3*q4J4*Eij(1,4)+
     &          q1J3*(6*q3J4*Eij(1,1)+3*q4J4*Eij(1,1)+4*q3J4*Eij(1,2)+
     &             2*q4J4*Eij(1,2)+6*q2J4*(Eij(1,1)+Eij(1,2))+
     &             4*q1J4*(2*Eij(1,1)+Eij(1,2))+2*q3J4*Eij(1,3)+
     &             2*q4J4*Eij(1,4))+
     &          q2J3*(4*q3J4*Eij(1,1)+2*q4J4*Eij(1,1)+6*q3J4*Eij(1,2)+
     &             3*q4J4*Eij(1,2)+6*q1J4*(Eij(1,1)+Eij(1,2))+
     &             4*q2J4*(Eij(1,1)+2*Eij(1,2))+2*q3J4*Eij(1,3)+
     &             2*q4J4*Eij(1,4)))+
     &       q1J2*(-8*J3J4*q1q2*Eij(1,1)-8*J3J4*q1q3*Eij(1,1)-
     &          5*J3J4*q1sq*Eij(1,1)-6*J3J4*q2q3*Eij(1,1)-
     &          3*J3J4*q2sq*Eij(1,1)+8*q1J4*q3J3*Eij(1,1)+
     &          6*q2J4*q3J3*Eij(1,1)+6*q3J3*q3J4*Eij(1,1)-
     &          3*J3J4*q3q4*Eij(1,1)-3*J3J4*q3sq*Eij(1,1)+
     &          3*q3J4*q4J3*Eij(1,1)+3*q3J3*q4J4*Eij(1,1)-
     &          2*J3J4*q1q2*Eij(1,2)-2*J3J4*q2q3*Eij(1,2)-
     &          2*J3J4*q2sq*Eij(1,2)+2*q2J4*q3J3*Eij(1,2)-
     &          2*J3J4*q1q3*Eij(1,3)-2*J3J4*q2q3*Eij(1,3)+
     &          4*q1J4*q3J3*Eij(1,3)+4*q2J4*q3J3*Eij(1,3)+
     &          6*q3J3*q3J4*Eij(1,3)-2*J3J4*q3sq*Eij(1,3)+
     &          2*q3J3*q4J4*Eij(1,3)-2*J3J4*q1q4*Eij(1,4)-
     &          2*J3J4*q2q4*Eij(1,4)-2*J3J4*q3q4*Eij(1,4)+
     &          4*q1J4*q4J3*Eij(1,4)+4*q2J4*q4J3*Eij(1,4)+
     &          4*q3J4*q4J3*Eij(1,4)+2*q3J3*q4J4*Eij(1,4)+
     &          2*q4J3*q4J4*Eij(1,4)+
     &          4*q1J3*(5*q1J4*Eij(1,1)+4*q3J4*Eij(1,1)+2*q4J4*Eij(1,1)+
     &             q2J4*(4*Eij(1,1)+Eij(1,2))+q3J4*Eij(1,3)+q4J4*Eij(1,4))
     &           +2*q2J3*(6*q3J4*Eij(1,1)+3*q4J4*Eij(1,1)+
     &             2*q3J4*Eij(1,2)+q4J4*Eij(1,2)+
     &             2*q1J4*(4*Eij(1,1)+Eij(1,2))+
     &             q2J4*(6*Eij(1,1)+4*Eij(1,2))+2*q3J4*Eij(1,3)+
     &             2*q4J4*Eij(1,4))))+
     &    q2J1*(J2J4*q1J3*q1q2*Eij(1,1)-J2J3*q1J4*q1q2*Eij(1,1)+
     &       J2J4*q1J3*q1q3*Eij(1,1)+J2J4*q1J3*q1q4*Eij(1,1)+
     &       J2J4*q1J3*q1sq*Eij(1,1)-J2J3*q1J4*q1sq*Eij(1,1)-
     &       q1J3*q1J4*q4J2*Eij(1,1)+J2J4*q1J3*q1q2*Eij(1,2)-
     &       5*J2J3*q1J4*q1q2*Eij(1,2)-2*J2J3*q1J4*q1sq*Eij(1,2)+
     &       4*q1J3*q1J4*q2J2*Eij(1,2)-2*J3J4*q1q2*q2J2*Eij(1,2)-
     &       2*J3J4*q1q3*q2J2*Eij(1,2)-J3J4*q1sq*q2J2*Eij(1,2)-
     &       2*J2J4*q1q2*q2J3*Eij(1,2)-2*J2J4*q1q3*q2J3*Eij(1,2)+
     &       J2J4*q1q4*q2J3*Eij(1,2)-J2J4*q1sq*q2J3*Eij(1,2)+
     &       4*q1J4*q2J2*q2J3*Eij(1,2)-2*J2J3*q1q2*q2J4*Eij(1,2)+
     &       3*J2J3*q1q3*q2J4*Eij(1,2)-J2J3*q1sq*q2J4*Eij(1,2)+
     &       4*q1J3*q2J2*q2J4*Eij(1,2)+4*q2J2*q2J3*q2J4*Eij(1,2)+
     &       J2J4*q1J3*q2q3*Eij(1,2)-3*J2J3*q1J4*q2q3*Eij(1,2)-
     &       2*J3J4*q2J2*q2q3*Eij(1,2)-2*J2J4*q2J3*q2q3*Eij(1,2)-
     &       2*J2J4*q1J3*q2q4*Eij(1,2)-2*J2J4*q2J3*q2q4*Eij(1,2)+
     &       J2J4*q1J3*q2sq*Eij(1,2)-3*J2J3*q1J4*q2sq*Eij(1,2)-
     &       J3J4*q2J2*q2sq*Eij(1,2)-J2J4*q2J3*q2sq*Eij(1,2)-
     &       J2J3*q2J4*q2sq*Eij(1,2)+2*q1J4*q2J2*q3J3*Eij(1,2)+
     &       2*q2J2*q2J4*q3J3*Eij(1,2)-J2J4*q2q4*q3J3*Eij(1,2)-
     &       4*J2J3*q1q2*q3J4*Eij(1,2)-2*J2J3*q1sq*q3J4*Eij(1,2)+
     &       4*q1J3*q2J2*q3J4*Eij(1,2)+4*q2J2*q2J3*q3J4*Eij(1,2)-
     &       2*J2J3*q2q3*q3J4*Eij(1,2)-J2J3*q2q4*q3J4*Eij(1,2)-
     &       2*J2J3*q2sq*q3J4*Eij(1,2)+2*q2J2*q3J3*q3J4*Eij(1,2)-
     &       J3J4*q2J2*q3q4*Eij(1,2)-J2J4*q2J3*q3q4*Eij(1,2)+
     &       J2J3*q2J4*q3q4*Eij(1,2)-J3J4*q2J2*q3sq*Eij(1,2)-
     &       J2J4*q2J3*q3sq*Eij(1,2)+J2J3*q2J4*q3sq*Eij(1,2)-
     &       2*J3J4*q1q2*q4J2*Eij(1,2)-J3J4*q1sq*q4J2*Eij(1,2)-
     &       q1J4*q2J3*q4J2*Eij(1,2)+2*q1J3*q2J4*q4J2*Eij(1,2)+
     &       2*q2J3*q2J4*q4J2*Eij(1,2)-J3J4*q2q3*q4J2*Eij(1,2)-
     &       J3J4*q2sq*q4J2*Eij(1,2)+q2J4*q3J3*q4J2*Eij(1,2)+
     &       q2J3*q3J4*q4J2*Eij(1,2)+2*J2J4*q1q2*q4J3*Eij(1,2)+
     &       J2J4*q1sq*q4J3*Eij(1,2)+J2J4*q2q3*q4J3*Eij(1,2)+
     &       J2J4*q2sq*q4J3*Eij(1,2)+q2J2*q3J4*q4J3*Eij(1,2)-
     &       2*J2J3*q1q2*q4J4*Eij(1,2)-J2J3*q1sq*q4J4*Eij(1,2)+
     &       2*q1J3*q2J2*q4J4*Eij(1,2)+2*q2J2*q2J3*q4J4*Eij(1,2)-
     &       J2J3*q2q3*q4J4*Eij(1,2)-J2J3*q2sq*q4J4*Eij(1,2)+
     &       q2J2*q3J3*q4J4*Eij(1,2)+J2J4*q1J3*q1q3*Eij(1,3)-
     &       J2J3*q1J4*q1q3*Eij(1,3)+J2J4*q1J3*q2q3*Eij(1,3)-
     &       J2J3*q1J4*q2q3*Eij(1,3)+J2J4*q1q4*q3J3*Eij(1,3)+
     &       J2J3*q1q3*q3J4*Eij(1,3)+J2J4*q1J3*q3sq*Eij(1,3)-
     &       J2J3*q1J4*q3sq*Eij(1,3)-q1J4*q3J3*q4J2*Eij(1,3)+
     &       J2J4*q1J3*q1q4*Eij(1,4)-J2J3*q1J4*q1q4*Eij(1,4)+
     &       J2J4*q1J3*q2q4*Eij(1,4)-J2J3*q1J4*q2q4*Eij(1,4)+
     &       J2J4*q1J3*q3q4*Eij(1,4)-J2J3*q1J4*q3q4*Eij(1,4)+
     &       J2J4*q1q4*q4J3*Eij(1,4)-q1J4*q4J2*q4J3*Eij(1,4)+
     &       J2J3*q1q3*q4J4*Eij(1,4)-
     &       q3J2*(-((J3J4*q2q4-q2J4*q4J3+
     &               q2J3*(2*q1J4+2*q2J4+2*q3J4+q4J4))*Eij(1,2))+
     &          q1J3*(q1J4*Eij(1,1)+q2J4*Eij(1,2)+q3J4*Eij(1,3)+
     &             q4J4*Eij(1,4)))+
     &       q1J2*(-(J3J4*q1q2*Eij(1,1))-J3J4*q1q3*Eij(1,1)-
     &          J3J4*q1sq*Eij(1,1)+q1J4*q3J3*Eij(1,1)-
     &          5*J3J4*q1q2*Eij(1,2)-4*J3J4*q1q3*Eij(1,2)-
     &          2*J3J4*q1sq*Eij(1,2)-5*J3J4*q2q3*Eij(1,2)-
     &          3*J3J4*q2sq*Eij(1,2)+4*q1J4*q3J3*Eij(1,2)+
     &          5*q2J4*q3J3*Eij(1,2)+4*q3J3*q3J4*Eij(1,2)-
     &          2*J3J4*q3q4*Eij(1,2)-2*J3J4*q3sq*Eij(1,2)+
     &          2*q3J4*q4J3*Eij(1,2)+2*q3J3*q4J4*Eij(1,2)-
     &          J3J4*q1q3*Eij(1,3)-J3J4*q2q3*Eij(1,3)+
     &          2*q1J4*q3J3*Eij(1,3)+2*q2J4*q3J3*Eij(1,3)+
     &          3*q3J3*q3J4*Eij(1,3)-J3J4*q3sq*Eij(1,3)+
     &          q3J3*q4J4*Eij(1,3)-J3J4*q1q4*Eij(1,4)-J3J4*q2q4*Eij(1,4)-
     &          J3J4*q3q4*Eij(1,4)+2*q1J4*q4J3*Eij(1,4)+
     &          2*q2J4*q4J3*Eij(1,4)+2*q3J4*q4J3*Eij(1,4)+
     &          q3J3*q4J4*Eij(1,4)+q4J3*q4J4*Eij(1,4)+
     &          q2J3*(12*q2J4*Eij(1,2)+10*q3J4*Eij(1,2)+5*q4J4*Eij(1,2)+
     &             2*q1J4*(Eij(1,1)+5*Eij(1,2))+2*q3J4*Eij(1,3)+
     &             2*q4J4*Eij(1,4))+
     &          q1J3*(2*q3J4*Eij(1,1)+q4J4*Eij(1,1)+8*q3J4*Eij(1,2)+
     &             4*q4J4*Eij(1,2)+4*q1J4*(Eij(1,1)+2*Eij(1,2))+
     &             2*q2J4*(Eij(1,1)+5*Eij(1,2))+2*q3J4*Eij(1,3)+
     &             2*q4J4*Eij(1,4)))))


      HE0=4*m*(C034*J1J2*J3J4+8*E0L*q1J1*q1J2*q1J3*q1J4-
     &    4*E0L*J3J4*q1J1*q1J2*q1q2-4*E0L*J2J3*q1J1*q1J4*q1q2+
     &    2*E0L*J1J2*J3J4*q1q2**2-4*E0L*J3J4*q1J1*q1J2*q1q3+
     &    2*E0L*J1J2*J3J4*q1q2*q1q3+2*E0L*J1J2*J3J4*q1q2*q1q4+
     &    D0234*J1J2*J3J4*q1sq-2*E0L*J3J4*q1J1*q1J2*q1sq-
     &    2*E0L*J2J3*q1J1*q1J4*q1sq-2*E0L*J1J2*q1J3*q1J4*q1sq+
     &    3*E0L*J1J2*J3J4*q1q2*q1sq+2*E0L*J1J2*J3J4*q1q3*q1sq+
     &    E0L*J1J2*J3J4*q1q4*q1sq+E0L*J1J2*J3J4*q1sq**2+
     &    4*E0L*q1J2*q1J3*q1J4*q2J1-2*E0L*J3J4*q1J2*q1q2*q2J1+
     &    2*E0L*J2J4*q1J3*q1q2*q2J1-2*E0L*J2J3*q1J4*q1q2*q2J1-
     &    2*E0L*J3J4*q1J2*q1q3*q2J1+2*E0L*J2J4*q1J3*q1q3*q2J1+
     &    2*E0L*J2J4*q1J3*q1q4*q2J1-E0L*J3J4*q1J2*q1sq*q2J1+
     &    E0L*J2J4*q1J3*q1sq*q2J1-E0L*J2J3*q1J4*q1sq*q2J1+
     &    4*E0L*q1J1*q1J3*q1J4*q2J2-2*E0L*J3J4*q1J1*q1q2*q2J2-
     &    2*E0L*J3J4*q1J1*q1q3*q2J2-E0L*J3J4*q1J1*q1sq*q2J2+
     &    8*E0L*q1J1*q1J2*q1J4*q2J3-2*E0L*J2J4*q1J1*q1q2*q2J3-
     &    2*E0L*J1J2*q1J4*q1q2*q2J3-2*E0L*J2J4*q1J1*q1q3*q2J3-
     &    E0L*J2J4*q1J1*q1sq*q2J3-3*E0L*J1J2*q1J4*q1sq*q2J3+
     &    4*E0L*q1J2*q1J4*q2J1*q2J3+2*E0L*J2J4*q1q4*q2J1*q2J3+
     &    4*E0L*q1J1*q1J4*q2J2*q2J3+8*E0L*q1J1*q1J2*q1J3*q2J4-
     &    2*E0L*J2J3*q1J1*q1q2*q2J4-2*E0L*J1J2*q1J3*q1q2*q2J4+
     &    2*E0L*J2J3*q1J1*q1q3*q2J4-2*E0L*J1J2*q1J3*q1q3*q2J4-
     &    2*E0L*J1J2*q1J3*q1q4*q2J4-E0L*J2J3*q1J1*q1sq*q2J4-
     &    3*E0L*J1J2*q1J3*q1sq*q2J4+4*E0L*q1J2*q1J3*q2J1*q2J4+
     &    2*E0L*J2J3*q1q3*q2J1*q2J4+4*E0L*q1J1*q1J3*q2J2*q2J4+
     &    8*E0L*q1J1*q1J2*q2J3*q2J4-4*E0L*J1J2*q1q2*q2J3*q2J4-
     &    2*E0L*J1J2*q1q3*q2J3*q2J4-2*E0L*J1J2*q1q4*q2J3*q2J4-
     &    4*E0L*J1J2*q1sq*q2J3*q2J4+4*E0L*q1J2*q2J1*q2J3*q2J4+
     &    4*E0L*q1J1*q2J2*q2J3*q2J4-4*E0L*J3J4*q1J1*q1J2*q2q3-
     &    2*E0L*J2J3*q1J1*q1J4*q2q3+2*E0L*J1J2*q1J3*q1J4*q2q3+
     &    2*E0L*J1J2*J3J4*q1q2*q2q3+E0L*J1J2*J3J4*q1q4*q2q3+
     &    2*E0L*J1J2*J3J4*q1sq*q2q3-2*E0L*J3J4*q1J2*q2J1*q2q3+
     &    2*E0L*J2J4*q1J3*q2J1*q2q3-2*E0L*J2J3*q1J4*q2J1*q2q3-
     &    2*E0L*J3J4*q1J1*q2J2*q2q3-2*E0L*J2J4*q1J1*q2J3*q2q3+
     &    2*E0L*J1J2*q1J4*q2J3*q2q3-2*E0L*J2J4*q1J1*q1J3*q2q4+
     &    2*E0L*J1J2*q1J3*q1J4*q2q4-E0L*J1J2*J3J4*q1q3*q2q4-
     &    2*E0L*J2J4*q1J1*q2J3*q2q4+2*E0L*J1J2*q1J4*q2J3*q2q4-
     &    2*E0L*J3J4*q1J1*q1J2*q2sq-2*E0L*J2J3*q1J1*q1J4*q2sq+
     &    2*E0L*J1J2*q1J3*q1J4*q2sq+E0L*J1J2*J3J4*q1q2*q2sq+
     &    E0L*J1J2*J3J4*q1q4*q2sq+E0L*J1J2*J3J4*q1sq*q2sq-
     &    E0L*J3J4*q1J2*q2J1*q2sq+E0L*J2J4*q1J3*q2J1*q2sq-
     &    E0L*J2J3*q1J4*q2J1*q2sq-E0L*J3J4*q1J1*q2J2*q2sq-
     &    E0L*J2J4*q1J1*q2J3*q2sq+E0L*J1J2*q1J4*q2J3*q2sq-
     &    E0L*J2J3*q1J1*q2J4*q2sq+E0L*J1J2*q1J3*q2J4*q2sq+
     &    4*E0L*q1J2*q1J3*q1J4*q3J1-2*E0L*J2J3*q1J4*q1q2*q3J1+
     &    2*E0L*J3J4*q1J2*q1q4*q3J1-2*E0L*J2J3*q1J4*q1sq*q3J1+
     &    2*E0L*q1J3*q1J4*q2J2*q3J1+E0L*J3J4*q1q4*q2J2*q3J1+
     &    2*E0L*q1J2*q1J4*q2J3*q3J1+E0L*J2J4*q1q4*q2J3*q3J1+
     &    4*E0L*q1J2*q1J3*q2J4*q3J1-2*E0L*J2J3*q1q2*q2J4*q3J1-
     &    E0L*J2J3*q1q4*q2J4*q3J1-2*E0L*J2J3*q1sq*q2J4*q3J1+
     &    2*E0L*q1J3*q2J2*q2J4*q3J1+2*E0L*q1J2*q2J3*q2J4*q3J1+
     &    E0L*J3J4*q1J2*q2q4*q3J1-E0L*J2J4*q1J3*q2q4*q3J1+
     &    E0L*J2J3*q1J4*q2q4*q3J1-2*E0L*q1J3*q1J4*q2J1*q3J2-
     &    E0L*J3J4*q1q4*q2J1*q3J2+2*E0L*q1J1*q1J4*q2J3*q3J2-
     &    2*E0L*q1J3*q2J1*q2J4*q3J2+2*E0L*q1J1*q2J3*q2J4*q3J2+
     &    E0L*J3J4*q1J1*q2q4*q3J2+4*E0L*q1J1*q1J2*q1J4*q3J3-
     &    2*E0L*J1J2*q1J4*q1q2*q3J3-2*E0L*J1J2*q1J4*q1sq*q3J3+
     &    2*E0L*q1J2*q1J4*q2J1*q3J3+E0L*J2J4*q1q4*q2J1*q3J3+
     &    2*E0L*q1J1*q1J4*q2J2*q3J3+4*E0L*q1J1*q1J2*q2J4*q3J3-
     &    2*E0L*J1J2*q1q2*q2J4*q3J3-E0L*J1J2*q1q4*q2J4*q3J3-
     &    2*E0L*J1J2*q1sq*q2J4*q3J3+2*E0L*q1J2*q2J1*q2J4*q3J3+
     &    2*E0L*q1J1*q2J2*q2J4*q3J3-E0L*J2J4*q1J1*q2q4*q3J3+
     &    E0L*J1J2*q1J4*q2q4*q3J3+8*E0L*q1J1*q1J2*q1J3*q3J4-
     &    4*E0L*J2J3*q1J1*q1q2*q3J4-2*E0L*J2J3*q1J1*q1sq*q3J4-
     &    2*E0L*J1J2*q1J3*q1sq*q3J4+4*E0L*q1J2*q1J3*q2J1*q3J4+
     &    2*E0L*J2J3*q1q3*q2J1*q3J4+E0L*J2J3*q1q4*q2J1*q3J4+
     &    4*E0L*q1J1*q1J3*q2J2*q3J4+8*E0L*q1J1*q1J2*q2J3*q3J4-
     &    4*E0L*J1J2*q1q2*q2J3*q3J4-2*E0L*J1J2*q1q3*q2J3*q3J4-
     &    E0L*J1J2*q1q4*q2J3*q3J4-4*E0L*J1J2*q1sq*q2J3*q3J4+
     &    4*E0L*q1J2*q2J1*q2J3*q3J4+4*E0L*q1J1*q2J2*q2J3*q3J4-
     &    2*E0L*J2J3*q1J1*q2q3*q3J4+2*E0L*J1J2*q1J3*q2q3*q3J4-
     &    E0L*J2J3*q1J1*q2q4*q3J4+E0L*J1J2*q1J3*q2q4*q3J4-
     &    2*E0L*J2J3*q1J1*q2sq*q3J4+2*E0L*J1J2*q1J3*q2sq*q3J4+
     &    4*E0L*q1J2*q1J3*q3J1*q3J4-2*E0L*J2J3*q1q2*q3J1*q3J4-
     &    2*E0L*J2J3*q1sq*q3J1*q3J4+2*E0L*q1J3*q2J2*q3J1*q3J4+
     &    2*E0L*q1J2*q2J3*q3J1*q3J4-2*E0L*q1J3*q2J1*q3J2*q3J4+
     &    2*E0L*q1J1*q2J3*q3J2*q3J4+4*E0L*q1J1*q1J2*q3J3*q3J4-
     &    2*E0L*J1J2*q1q2*q3J3*q3J4-2*E0L*J1J2*q1sq*q3J3*q3J4+
     &    2*E0L*q1J2*q2J1*q3J3*q3J4+2*E0L*q1J1*q2J2*q3J3*q3J4-
     &    2*E0L*J3J4*q1J1*q1J2*q3q4+E0L*J1J2*J3J4*q1q2*q3q4+
     &    E0L*J1J2*J3J4*q1sq*q3q4-E0L*J3J4*q1J2*q2J1*q3q4+
     &    E0L*J2J4*q1J3*q2J1*q3q4-E0L*J2J3*q1J4*q2J1*q3q4-
     &    E0L*J3J4*q1J1*q2J2*q3q4-E0L*J2J4*q1J1*q2J3*q3q4+
     &    E0L*J1J2*q1J4*q2J3*q3q4+E0L*J2J3*q1J1*q2J4*q3q4-
     &    E0L*J1J2*q1J3*q2J4*q3q4-2*E0L*J3J4*q1J1*q1J2*q3sq+
     &    E0L*J1J2*J3J4*q1q2*q3sq+E0L*J1J2*J3J4*q1sq*q3sq-
     &    E0L*J3J4*q1J2*q2J1*q3sq+E0L*J2J4*q1J3*q2J1*q3sq-
     &    E0L*J2J3*q1J4*q2J1*q3sq-E0L*J3J4*q1J1*q2J2*q3sq-
     &    E0L*J2J4*q1J1*q2J3*q3sq+E0L*J1J2*q1J4*q2J3*q3sq+
     &    E0L*J2J3*q1J1*q2J4*q3sq-E0L*J1J2*q1J3*q2J4*q3sq+
     &    4*E0L*q1J2*q1J3*q1J4*q4J1-2*E0L*J3J4*q1J2*q1q2*q4J1-
     &    2*E0L*J2J3*q1J4*q1q2*q4J1-2*E0L*J3J4*q1J2*q1q3*q4J1-
     &    E0L*J3J4*q1J2*q1sq*q4J1-E0L*J2J4*q1J3*q1sq*q4J1-
     &    E0L*J2J3*q1J4*q1sq*q4J1+2*E0L*q1J3*q1J4*q2J2*q4J1-
     &    E0L*J3J4*q1q3*q2J2*q4J1+4*E0L*q1J2*q1J4*q2J3*q4J1-
     &    2*E0L*J2J4*q1q2*q2J3*q4J1-E0L*J2J4*q1q3*q2J3*q4J1-
     &    2*E0L*J2J4*q1sq*q2J3*q4J1+2*E0L*q1J4*q2J2*q2J3*q4J1+
     &    2*E0L*q1J2*q1J3*q2J4*q4J1+E0L*J2J3*q1q3*q2J4*q4J1+
     &    2*E0L*q1J2*q2J3*q2J4*q4J1-E0L*J3J4*q1J2*q2q3*q4J1+
     &    E0L*J2J4*q1J3*q2q3*q4J1-E0L*J2J3*q1J4*q2q3*q4J1-
     &    E0L*J3J4*q1J2*q2sq*q4J1+E0L*J2J4*q1J3*q2sq*q4J1-
     &    E0L*J2J3*q1J4*q2sq*q4J1+E0L*J3J4*q1q2*q3J2*q4J1+
     &    E0L*J3J4*q1sq*q3J2*q4J1+E0L*q1J4*q2J3*q3J2*q4J1-
     &    E0L*q1J3*q2J4*q3J2*q4J1+2*E0L*q1J2*q1J4*q3J3*q4J1-
     &    E0L*J2J4*q1q2*q3J3*q4J1-E0L*J2J4*q1sq*q3J3*q4J1+
     &    E0L*q1J4*q2J2*q3J3*q4J1+E0L*q1J2*q2J4*q3J3*q4J1+
     &    2*E0L*q1J2*q1J3*q3J4*q4J1-E0L*J2J3*q1q2*q3J4*q4J1-
     &    E0L*J2J3*q1sq*q3J4*q4J1+E0L*q1J3*q2J2*q3J4*q4J1+
     &    E0L*q1J2*q2J3*q3J4*q4J1-2*E0L*J3J4*q1J1*q1q2*q4J2-
     &    E0L*J3J4*q1J1*q1sq*q4J2-2*E0L*q1J3*q1J4*q2J1*q4J2+
     &    E0L*J3J4*q1q3*q2J1*q4J2-2*E0L*q1J4*q2J1*q2J3*q4J2+
     &    2*E0L*q1J1*q1J3*q2J4*q4J2+2*E0L*q1J1*q2J3*q2J4*q4J2-
     &    E0L*J3J4*q1J1*q2q3*q4J2-E0L*J3J4*q1J1*q2sq*q4J2-
     &    E0L*J3J4*q1q2*q3J1*q4J2-E0L*J3J4*q1sq*q3J1*q4J2-
     &    E0L*q1J4*q2J3*q3J1*q4J2+E0L*q1J3*q2J4*q3J1*q4J2-
     &    E0L*q1J4*q2J1*q3J3*q4J2+E0L*q1J1*q2J4*q3J3*q4J2-
     &    E0L*q1J3*q2J1*q3J4*q4J2+E0L*q1J1*q2J3*q3J4*q4J2+
     &    2*E0L*J2J4*q1J1*q1q2*q4J3-2*E0L*J1J2*q1J4*q1q2*q4J3+
     &    E0L*J2J4*q1J1*q1sq*q4J3-E0L*J1J2*q1J4*q1sq*q4J3-
     &    E0L*J2J4*q1q3*q2J1*q4J3+E0L*J1J2*q1q3*q2J4*q4J3+
     &    E0L*J2J4*q1J1*q2q3*q4J3-E0L*J1J2*q1J4*q2q3*q4J3+
     &    E0L*J2J4*q1J1*q2sq*q4J3-E0L*J1J2*q1J4*q2sq*q4J3-
     &    2*E0L*q1J2*q1J4*q3J1*q4J3+E0L*J2J4*q1q2*q3J1*q4J3+
     &    E0L*J2J4*q1sq*q3J1*q4J3-E0L*q1J4*q2J2*q3J1*q4J3-
     &    E0L*q1J2*q2J4*q3J1*q4J3+E0L*q1J4*q2J1*q3J2*q4J3-
     &    E0L*q1J1*q2J4*q3J2*q4J3+2*E0L*q1J1*q1J2*q3J4*q4J3-
     &    E0L*J1J2*q1q2*q3J4*q4J3-E0L*J1J2*q1sq*q3J4*q4J3+
     &    E0L*q1J2*q2J1*q3J4*q4J3+E0L*q1J1*q2J2*q3J4*q4J3+
     &    J1J4*(J2J3*(C034+E0L*q1sq**2+
     &          q1sq*(D0234+E0L*
     &              (3*q1q2+2*q1q3+q1q4+2*q2q3+q2sq+q3q4+q3sq))+
     &          E0L*(2*q1q2**2-q1q3*q2q4+q1q4*(q2q3+q2sq)+
     &             q1q2*(2*q1q3+2*q1q4+2*q2q3+q2sq+q3q4+q3sq)))-
     &       E0L*(q1q4*q2J3*q3J2-q1J3*q2q4*q3J2-q1J3*q1sq*q4J2-
     &          2*q1q2*q2J3*q4J2-q1q3*q2J3*q4J2-2*q1sq*q2J3*q4J2+
     &          q1J3*q2q3*q4J2+q1J3*q2sq*q4J2-q1q2*q3J3*q4J2-
     &          q1sq*q3J3*q4J2+q1q2*q3J2*q4J3+q1sq*q3J2*q4J3+
     &          q2J2*(q1q4*(2*q2J3+q3J3)+
     &             q1J3*(2*q1q2+2*q1q3+2*q1q4+q1sq+2*q2q3+q2sq+
     &                q3q4+q3sq)-q1q3*q4J3)+
     &          q1J2*(2*q1q4*q3J3+q2q4*q3J3+
     &             2*q1J3*(2*q1q2+2*q1q3+2*q1q4+q1sq+2*q2q3+q2q4+
     &                q2sq+q3q4+q3sq)+
     &             q2J3*(2*q1q2+2*q1q3+4*q1q4+q1sq+2*q2q3+2*q2q4+
     &                q2sq+q3q4+q3sq)-2*q1q2*q4J3-2*q1q3*q4J3-
     &             q1sq*q4J3-q2q3*q4J3-q2sq*q4J3)))+
     &    4*E0L*q1J1*q1J2*q1J3*q4J4-2*E0L*J2J3*q1J1*q1q2*q4J4-
     &    E0L*J2J3*q1J1*q1sq*q4J4-E0L*J1J2*q1J3*q1sq*q4J4+
     &    2*E0L*q1J2*q1J3*q2J1*q4J4+E0L*J2J3*q1q3*q2J1*q4J4+
     &    2*E0L*q1J1*q1J3*q2J2*q4J4+4*E0L*q1J1*q1J2*q2J3*q4J4-
     &    2*E0L*J1J2*q1q2*q2J3*q4J4-E0L*J1J2*q1q3*q2J3*q4J4-
     &    2*E0L*J1J2*q1sq*q2J3*q4J4+2*E0L*q1J2*q2J1*q2J3*q4J4+
     &    2*E0L*q1J1*q2J2*q2J3*q4J4-E0L*J2J3*q1J1*q2q3*q4J4+
     &    E0L*J1J2*q1J3*q2q3*q4J4-E0L*J2J3*q1J1*q2sq*q4J4+
     &    E0L*J1J2*q1J3*q2sq*q4J4+2*E0L*q1J2*q1J3*q3J1*q4J4-
     &    E0L*J2J3*q1q2*q3J1*q4J4-E0L*J2J3*q1sq*q3J1*q4J4+
     &    E0L*q1J3*q2J2*q3J1*q4J4+E0L*q1J2*q2J3*q3J1*q4J4-
     &    E0L*q1J3*q2J1*q3J2*q4J4+E0L*q1J1*q2J3*q3J2*q4J4+
     &    2*E0L*q1J1*q1J2*q3J3*q4J4-E0L*J1J2*q1q2*q3J3*q4J4-
     &    E0L*J1J2*q1sq*q3J3*q4J4+E0L*q1J2*q2J1*q3J3*q4J4+
     &    E0L*q1J1*q2J2*q3J3*q4J4-
     &    J1J3*(J2J4*(C034+E0L*q1sq**2+
     &          q1sq*(D0234+E0L*
     &              (3*q1q2+2*q1q3+q1q4+2*q2q3+q2sq+q3q4+q3sq))+
     &          E0L*(2*q1q2**2-q1q3*q2q4+q1q4*(q2q3+q2sq)+
     &             q1q2*(2*q1q3+2*q1q4+2*q2q3+q2sq+q3q4+q3sq)))-
     &       E0L*(-2*q1J2*q1q2*q2J4-2*q1J2*q1q3*q2J4-q1J2*q1sq*q2J4+
     &          2*q1J2*q1J4*q2q3-q1J2*q2J4*q2sq-4*q1J2*q1q2*q3J4-
     &          4*q1J2*q1q3*q3J4-2*q1J2*q1q4*q3J4-2*q1J2*q1sq*q3J4-
     &          2*q1J2*q2q3*q3J4-q1J2*q2q4*q3J4-2*q1J2*q2sq*q3J4+
     &          2*q1J2*q1J4*q3q4+q1J2*q2J4*q3q4+2*q1J2*q1J4*q3sq+
     &          q1J2*q2J4*q3sq+2*q1J4*q1q2*q4J2+q1J4*q1sq*q4J2-
     &          q1q3*q2J4*q4J2+q1J4*q2q3*q4J2+q1J4*q2sq*q4J2+
     &          q1q2*q3J4*q4J2+q1sq*q3J4*q4J2-2*q1J2*q1q2*q4J4-
     &          2*q1J2*q1q3*q4J4-q1J2*q1sq*q4J4-q1J2*q2q3*q4J4-
     &          q1J2*q2sq*q4J4+
     &          q2J2*(-2*q1q3*q2J4-2*q1q3*q3J4-q1q4*q3J4+
     &             q1J4*(2*q1q2+q1sq+2*q2q3+q2sq+q3q4+q3sq)-
     &             q1q3*q4J4)+q3J2*
     &           ((2*q1q2+q1q4+2*q1sq)*q2J4+
     &             q1J4*(2*q1q2+2*q1sq-q2q4)+(q1q2+q1sq)*(2*q3J4+q4J4)
     &             ))))


      HDmunu=-16*m*(J3J4*q3J1*q3J2*Dij(2,2)+J2J3*q3J1*q3J4*Dij(2,2)+
     &    J3J4*q4J1*q4J2*Dij(2,3)+J2J3*q4J1*q4J4*Dij(2,3)+
     &    J3J4*q2J2*q3J1*Dij(2,4)+J2J3*q2J4*q3J1*Dij(2,4)+
     &    J3J4*q2J2*q4J1*Dij(2,5)+J2J3*q2J4*q4J1*Dij(2,5)+
     &    q2J1*(J3J4*q2J2*Dij(2,1)+J3J4*(q3J2*Dij(2,4)+q4J2*Dij(2,5))+
     &       J2J3*(q2J4*Dij(2,1)+q3J4*Dij(2,4)+q4J4*Dij(2,5)))+
     &    J3J4*q3J2*q4J1*Dij(2,6)+J2J3*q3J4*q4J1*Dij(2,6)+
     &    J3J4*q3J1*q4J2*Dij(2,6)+J2J3*q3J1*q4J4*Dij(2,6)-
     &    J1J4*J2J3*Dij(2,7)+J1J2*
     &     (q4J3*q4J4*Dij(2,3)+q2J4*q4J3*Dij(2,5)+
     &       q2J3*(q2J4*Dij(2,1)+q3J4*Dij(2,4)+q4J4*Dij(2,5))+
     &       q3J4*q4J3*Dij(2,6)+
     &       q3J3*(q3J4*Dij(2,2)+q2J4*Dij(2,4)+q4J4*Dij(2,6))-
     &       2*J3J4*Dij(2,7)))


      HDmu=8*m*(J1J4*J2J3*q1q2*Dij(1,1)-J1J3*J2J4*q1q2*Dij(1,1)+
     &    J1J2*J3J4*q1q2*Dij(1,1)+J3J4*q1J1*q2J2*Dij(1,1)-
     &    J1J4*q1J3*q2J2*Dij(1,1)+J1J3*q1J4*q2J2*Dij(1,1)-
     &    J3J4*q2J1*q2J2*Dij(1,1)-J2J4*q2J1*q2J3*Dij(1,1)+
     &    J2J3*q1J1*q2J4*Dij(1,1)-J1J3*q1J2*q2J4*Dij(1,1)+
     &    J1J2*q1J3*q2J4*Dij(1,1)-J2J3*q2J1*q2J4*Dij(1,1)-
     &    4*J1J2*q2J3*q2J4*Dij(1,1)+J1J4*J2J3*q2sq*Dij(1,1)-
     &    J1J3*J2J4*q2sq*Dij(1,1)+J1J2*J3J4*q2sq*Dij(1,1)+
     &    J1J3*q2J4*q3J2*Dij(1,1)-2*J2J3*q2J1*q3J4*Dij(1,1)-
     &    J3J4*q2J1*q4J2*Dij(1,1)+J1J4*q2J3*q4J2*Dij(1,1)+
     &    J2J4*q2J1*q4J3*Dij(1,1)-J2J3*q2J1*q4J4*Dij(1,1)+
     &    J1J4*J2J3*q1q3*Dij(1,2)-J1J3*J2J4*q1q3*Dij(1,2)+
     &    J1J2*J3J4*q1q3*Dij(1,2)-J3J4*q2J2*q3J1*Dij(1,2)-
     &    J2J4*q2J3*q3J1*Dij(1,2)+J3J4*q1J1*q3J2*Dij(1,2)-
     &    J1J4*q1J3*q3J2*Dij(1,2)+J1J3*q1J4*q3J2*Dij(1,2)+
     &    J2J3*q1J1*q3J4*Dij(1,2)-J1J3*q1J2*q3J4*Dij(1,2)+
     &    J1J2*q1J3*q3J4*Dij(1,2)-3*J2J3*q3J1*q3J4*Dij(1,2)+
     &    J1J3*q3J2*q3J4*Dij(1,2)-3*J1J2*q3J3*q3J4*Dij(1,2)+
     &    J1J4*J2J3*q3sq*Dij(1,2)-J1J3*J2J4*q3sq*Dij(1,2)+
     &    J1J2*J3J4*q3sq*Dij(1,2)-J2J4*q3J3*q4J1*Dij(1,2)-
     &    J3J4*q3J1*q4J2*Dij(1,2)+J1J4*q3J3*q4J2*Dij(1,2)+
     &    J2J4*q3J1*q4J3*Dij(1,2)+J1J4*J2J3*q2q3*(Dij(1,1)+Dij(1,2))-
     &    J1J3*J2J4*q2q3*(Dij(1,1)+Dij(1,2))+
     &    J1J2*J3J4*q2q3*(Dij(1,1)+Dij(1,2))-
     &    J2J3*q2J4*q3J1*(Dij(1,1)+Dij(1,2))-
     &    2*J1J2*q2J3*q3J4*(Dij(1,1)+Dij(1,2))-
     &    J1J2*q2J4*q3J3*(Dij(1,1)+2*Dij(1,2))+J1J4*J2J3*q1q4*Dij(1,3)-
     &    J1J3*J2J4*q1q4*Dij(1,3)+J1J2*J3J4*q1q4*Dij(1,3)+
     &    J1J4*J2J3*q2q4*Dij(1,3)-J1J3*J2J4*q2q4*Dij(1,3)+
     &    J1J2*J3J4*q2q4*Dij(1,3)+J1J4*J2J3*q3q4*Dij(1,3)-
     &    J1J3*J2J4*q3q4*Dij(1,3)+J1J2*J3J4*q3q4*Dij(1,3)-
     &    J3J4*q2J2*q4J1*Dij(1,3)-J2J3*q2J4*q4J1*Dij(1,3)-
     &    2*J2J3*q3J4*q4J1*Dij(1,3)+J3J4*q1J1*q4J2*Dij(1,3)-
     &    J1J4*q1J3*q4J2*Dij(1,3)+J1J3*q1J4*q4J2*Dij(1,3)-
     &    J3J4*q4J1*q4J2*Dij(1,3)-2*J1J2*q2J4*q4J3*Dij(1,3)-
     &    2*J1J2*q3J4*q4J3*Dij(1,3)+J1J4*q4J2*q4J3*Dij(1,3)+
     &    J2J3*q1J1*q4J4*Dij(1,3)-J1J3*q1J2*q4J4*Dij(1,3)+
     &    J1J2*q1J3*q4J4*Dij(1,3)+J1J3*q3J2*q4J4*Dij(1,3)-
     &    J2J3*q4J1*q4J4*Dij(1,3)-J1J2*q4J3*q4J4*Dij(1,3)-
     &    J2J4*q2J3*q4J1*(Dij(1,1)+Dij(1,3))-
     &    J2J3*q3J1*q4J4*(Dij(1,2)+Dij(1,3))-
     &    J1J2*q3J3*q4J4*(Dij(1,2)+Dij(1,3))-
     &    J1J2*q2J3*q4J4*(Dij(1,1)+2*Dij(1,3)))


      HD0=-4*D0234*m*(-(J1J4*J2J3*q1q2)+J1J3*J2J4*q1q2-J1J2*J3J4*q1q2-
     &    J1J4*J2J3*q1q4+J1J3*J2J4*q1q4-J1J2*J3J4*q1q4+2*J1J4*J2J3*q1sq-
     &    2*J1J3*J2J4*q1sq+2*J1J2*J3J4*q1sq+
     &    (J3J4*q1J2-J2J4*q1J3+J2J3*q1J4)*q2J1-J3J4*q1J1*q2J2+
     &    J1J4*q1J3*q2J2-J1J3*q1J4*q2J2-J2J4*q1J1*q2J3+J1J4*q1J2*q2J3-
     &    J1J2*q1J4*q2J3-J2J3*q1J1*q2J4+J1J3*q1J2*q2J4-J1J2*q1J3*q2J4+
     &    4*J1J2*q2J3*q2J4-2*J1J4*J2J3*q2q3+2*J1J3*J2J4*q2q3-
     &    2*J1J2*J3J4*q2q3-J1J4*J2J3*q2sq+J1J3*J2J4*q2sq-J1J2*J3J4*q2sq+
     &    2*J2J3*q2J4*q3J1-2*J1J3*q2J4*q3J2+2*J1J2*q2J4*q3J3-
     &    2*J2J3*q1J1*q3J4+2*J1J3*q1J2*q3J4-2*J1J2*q1J3*q3J4+
     &    4*J1J2*q2J3*q3J4+2*J2J3*q3J1*q3J4-2*J1J3*q3J2*q3J4+
     &    2*J1J2*q3J3*q3J4-J1J4*J2J3*q3q4+J1J3*J2J4*q3q4-J1J2*J3J4*q3q4-
     &    J1J4*J2J3*q3sq+J1J3*J2J4*q3sq-J1J2*J3J4*q3sq+
     &    (J3J4*q1J2-J2J4*q1J3+J2J3*q1J4+2*J2J4*q2J3-J3J4*q3J2+
     &       J2J4*q3J3+J2J3*q3J4)*q4J1-J3J4*q1J1*q4J2+J1J4*q1J3*q4J2-
     &    J1J3*q1J4*q4J2-2*J1J4*q2J3*q4J2+J3J4*q3J1*q4J2-J1J4*q3J3*q4J2-
     &    J1J3*q3J4*q4J2+J2J4*q1J1*q4J3-J1J4*q1J2*q4J3+J1J2*q1J4*q4J3-
     &    J2J4*q3J1*q4J3+J1J4*q3J2*q4J3+J1J2*q3J4*q4J3-J2J3*q1J1*q4J4+
     &    J1J3*q1J2*q4J4-J1J2*q1J3*q4J4+2*J1J2*q2J3*q4J4+J2J3*q3J1*q4J4-
     &    J1J3*q3J2*q4J4+J1J2*q3J3*q4J4)


      H4G_LoopDD=(HEmunurhosigma+HEmunurho+HEmunu+HEmu+HE0+HDmunu+HDmu+HD0)/(4d0*m)

      RETURN
      END
