c
c***************************************************************************
c***************************************************************************
c phase space generator for pp --> t q W --> t q l+ l- nu nu~   
c                  and  for pp --> t q Z --> b q Z W --> b q e+ e- mu vmu~ 
c       Manuel Baehr:     mb@particle.uni-karlsruhe.de , Simon Wendel   sw@particle.uni-karlsruhe.de
c       Initial version:  2005 March 07
c       Last modified(MB):    2005 May 02
C       Last modified(SW)    2007 July 05
C      
C  INPUT:   RD        random number array (between 0 and 1)
C           RN        additional random number which will not be driven by
C                     MONACO = azimuthal angle
C           switch    switches between W and Z radiation, switch=0 for W , switch=1 for Z
C
C  OUTPUT:  K1,K2     4-vectors for 2 incoming partons
C           X1,X2     corresponding values of Feynman x
C          % Q(0:4,2)  4-vectors for the vector boson resonances V1 and V2,
C          %           the 4th component contains the mass**2
C           v(0:3,nv)  4-vectors for the decay leptons, assumed massless
C           P(*,N)    4-vectors for the N jets, assumed massless (except top)
C           W         the weight of the phase space point
C
C
C
c***************************************************************************
c***************************************************************************
      
      subroutine lipstopV(rd,rn,switch,k1,k2,p,v,x1,x2,w)
 
      implicit none

#include "VBFNLO/utilities//global.inc"
#include "VBFNLO/utilities//cuts.inc"
c received from global.inc:
c     nrand = phase space dim - 2 - phi
c     ecm   = collider energy


      logical mdebug
      parameter(mdebug=.false.)

      real*8            alfas, xmt, alfa, xmz, xmw, sin2w, xmh
      common /bkopin/   alfas, xmt, alfa, xmz, xmw, sin2w, xmh

c common block for width of particles
      double precision     hwidth, wwidth, zwidth, twidth
      common /COUPL_WIDTH/ hwidth, wwidth, zwidth, twidth
c ----------------------------------------------------
      integer n1, switch
      double precision tpi
      parameter (n1=max_p-nx, tpi=2D0*PI )
      double precision rd(*), rn(*), x1, x2, w    
      double precision k1(0:3),k2(0:3),p(0:3,n1),q(0:3,4),v(0:3,max_v), r(0:3)
c ----------------------------------------------------
c ----------------------------------------------------

      integer i, mu
      double precision pT, pTmax, beta, expo, pL, y2, y2_border, ET !
      parameter ( expo = 3d0 ) ! pT mapping: exponent of pT distribution for pT<pTmin_def

      double precision abs_p(0:4), msq(4), qx(4), 
     &                 qxmax(4), qxmin(4), tanqx(4) !length of momenta in CM1..4 + q^2 values of virt. particles
      double precision xxwidth, xxm
      parameter ( xxwidth = 10d1, xxm = 45d1 )

      double precision coshy2, sinhy2, temp, test
      double precision s, y, y_border, tau
      double precision phi(0:4), costheta(4), sintheta(4)


      double precision mass2
      external mass2
c ---------------------------------------------------- 
      w = 1d0             !init of weightfactor
      s = ecm**2          !Collider Energy squared
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  list of particle q^2 in msq(i)
c  msq(1) = X particle
c  msq(2) = top
c  msq(3) = W from top decay
c  msq(4) = W/Z from X decay in top and W/Z
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c list of leptons in output array v(0:3,4)
c v(*,1) = charged lepton from extra W radiation/ e+ from extra Z radiation
c v(*,2) = neutrino from extra W radiation/ e- from extra Z radiation
c v(*,3) = charged lepton from topdecay
c v(*,4) = neutrino from top decay
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c q^2 of X-Particle [=msq(1)] is generated via tan mapping out of rd(12)
c
c
      qxmin(1) = atan(-xxm/xxwidth + xmt**2/xxm/xxwidth)  
      qxmax(1) = atan( (s-xxm**2)/xxm/xxwidth )

      qx(1)    = qxmin(1) + rd(12) * ( qxmax(1) - qxmin(1) )
      tanqx(1) = tan( qx(1) )

cccccccccccccccc
      msq(1)  = xxm**2 + xxwidth * xxm * tanqx(1) 
cccccccccccccccc
      if (msq(1).le.xmt**2 .or. msq(1).ge.s) then
         print*, "cccccccccc ACHTUNG ccccccccccccccc"
         print*, "c                                c"
         print*, "c       tan-mapping              c"
         print*, "c  verursachte falsche X-masse   c"
         print*, "c                                c"
         print*, "cccccccccccccccccccccccccccccccccc"

         msq(1) = s - 1d-3
      endif

      w = w * xxwidth * xxm * ( 1 + tanqx(1)**2 ) ! Jacobian
      w = w * ( qxmax(1) - qxmin(1) )             ! integration area (inverse of prob density)
      w = w / tpi                                 ! phasespace splitting

c      if ( w .le. 0d0 ) print*, "dps=0 at line 109"
c
c
c end of q^2 generation 
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c q^2 of W- / Z (=msq(4) ) is generated via tan mapping out of rd(14)
c
c
      SELECT CASE(switch)           ! switch between different radiations
      CASE (1)                      ! switch=1 : Z radiation
      qxmin(4) = atan(-xmz/zwidth)
      qxmax(4) = atan( ( (sqrt(msq(1))- xmt)**2 - xmz**2)/xmz/zwidth )

      qx(4) = qxmin(4) + rd(14) * ( qxmax(4) - qxmin(4) )
      tanqx(4) = tan( qx(4) )
cccccccccccccccc
      msq(4)  = xmz**2 + zwidth * xmz * tanqx(4)
cccccccccccccccc
      if ( msq(4).ge.msq(1) ) then
         print*, "cccccccccc ACHTUNG ccccccccccccccc"
         print*, "c                                c"
         print*, "c         tan-mapping            c"
         print*, "c verursachte: Z-masse > X-masse c"
         print*, "c                                c"
         print*, "cccccccccccccccccccccccccccccccccc"

         msq(3) = msq(2) - 1d-3
      endif
      
      w = w * zwidth * xmz * ( 1 + tanqx(4)**2 )  ! Jacobian
      
      CASE (0)                      ! switch=0 : W radiation

          qxmin(4) = atan(-xmw/wwidth)
          qxmax(4) = atan( ( (sqrt(msq(1))- xmt)**2 - xmw**2)/xmw/wwidth )

          qx(4) = qxmin(4) + rd(14) * ( qxmax(4) - qxmin(4) )
          tanqx(4) = tan( qx(4) )
cccccccccccccccc
          msq(4)  = xmz**2 + wwidth * xmw * tanqx(4)
cccccccccccccccc
          if ( msq(4).ge.msq(1) ) then
            print*, "cccccccccc ACHTUNG ccccccccccccccc"
            print*, "c                                c"
            print*, "c         tan-mapping            c"
            print*, "c verursachte: W-masse > X-masse c"
            print*, "c                                c"
            print*, "cccccccccccccccccccccccccccccccccc"

            msq(3) = msq(2) - 1d-3
          endif

          w = w * wwidth * xmw * ( 1 + tanqx(4)**2 )  ! Jacobian
      
      END SELECT
      
      w = w * ( qxmax(4) - qxmin(4) )             ! integration area (inverse of prob density)
      w = w / tpi                                 ! phasespace splitting

c      if ( w .le. 0d0 ) print*, "dps=0 at line 145"
c
c
c end of q^2 ( msq(4) ) generation 
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c q^2 of top-quark(=msq) is generated on shell 
c

c complete available region
      qxmin(2) = atan(-xmt/twidth)
c     qxmax(2) = atan( ( (sqrt(msq(1)) - sqrt(msq(4)) )**2  - xmt**2 ) / xmt / twidth )
      qxmax(2) = atan( (s-xmt**2)/xmt/twidth)  ! qxmax-qxmin=PI

cccccccccccccccc
      msq(2) = xmt**2
cccccccccccccccc

       w = w / tpi * xmt * twidth        ! phase space splitting and Jacobin (x==o)
       w = w * ( qxmax(2) - qxmin(2) )

c      if ( w .le. 0d0 ) print*, "dps=0 at line 167"
c
c
c end of q^2 generation 
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c q^2 of W+ (=msq(3) ) is generated via tan mapping out of rd(13)
c
c
      qxmin(3) = atan(-xmw/wwidth)
      qxmax(3) = atan( (msq(2)-xmw**2)/xmw/wwidth )

      qx(3)    = qxmin(3) + rd(13) * ( qxmax(3) - qxmin(3) )
      tanqx(3) = tan( qx(3) )
cccccccccccccccc
      msq(3)   = xmw**2 + wwidth * xmw * tanqx(3)
cccccccccccccccc
      if ( msq(3).ge.msq(2) ) then
         print*, "cccccccccc ACHTUNG ccccccccccccccc"
         print*, "c                                c"
         print*, "c         tan-mapping            c"
         print*, "c verursachte: W-masse>top-masse c"
         print*, "c                                c"
         print*, "cccccccccccccccccccccccccccccccccc"

         msq(3) = msq(2) - 1d-3
      endif



      w = w * wwidth * xmw * ( 1 + tanqx(3)**2 )  ! Jacobian
      w = w * ( qxmax(3) - qxmin(3) )             ! integration area (inverse of prob density)
      w = w / tpi                                 ! phasespace splitting

c      if ( w .le. 0d0 ) print*, "dps=0 at line 202"
c
c
c end of q^2 (msq2) generation 
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c generate the four integration variables y, pT, y*, phi from plain distributed rd, rn 
c for 2-->2 phasespace of  qq-->Xq with massless quarks and massiv X particle
c
      pTmax = ( s - msq(1) ) / 2d0 / ecm

cccccccccccccc
      phi(0) = rn(4) * tpi
ccccccccccccccc
c generate pT not uniformly:
      beta = ( (expo+1)*pTmin_def*pTmax  ) /
     &       ( (expo+2)*pTmax - (expo+1)*pTmin_def )

      if ( rd(1) .lt. beta/(expo+1)/pTmin_def ) then
         pT = ( rd(1)*(expo+1)/beta*pTmin_def**(expo+2) )**( 1/(expo+1))
      else
         pT = ( ( pTmin_def - beta/(expo+1) ) * pTmax ) /
     &        ( pTmax - beta/(expo+1) - (pTmax-pTmin_def) * rd(1) )
      endif

c debug stuff:
c      print*, "---------------------------------"
c      print*, "pTmax     =", pTmax
c      print*, "pTmin_def =", pTmin_def
c      print*, "beta      =", beta
c      print*, "loss      =", beta/2.0/pTmin_def

c insert inverse Density
      if ( rd(1) .lt. beta/(expo+1)/pTmin_def ) then
         w = w / beta / pT**expo * pTmin_def**(expo+2)
      else
         w = w / beta * pT**2
      endif

c      if ( w .le. 0d0 ) print*, "dps=0 at line 242; pT =", pT

cc flat pT:
c      pT = rd(1)*pTmax
c      w = w * pTmax                !(prop density)^-1 of pT 
ccccccccccccccc

c-----------------------------------------------------
      ET = sqrt( pT**2 + msq(1) )
      temp = ( s + msq(1) ) / ecm / ET
      y2_border = log( ( temp + sqrt(temp**2-4) ) / 2d0 )  ! ln identity of arcosh
c-----------------------------------------------------

ccccccccccccccc
      y2 = y2_border*2d0*( rd(2) - 0.5d0 )   ! create y2 vom rd(2)
ccccccccccccccc

c ---------------------------------------------------
      sinhy2 = sinh(y2)
      coshy2 = sqrt(sinhy2**2+1)
     
      abs_p(0) = sqrt( ET**2 * coshy2**2 - msq(1) )  ! calculate p
      pL = sinhy2 * ET


      tau = ( abs_p(0) + coshy2 * ET )**2 / s
c      tau = 2d0*ET*coshy2/s*(ET*coshy2-abs_p)

      y_border = log(1/tau)
c ---------------------------------------------------

ccccccccccccccc
      y = y_border*( rd(3) - 0.5d0 )
ccccccccccccccc

c
c
c end of CM0 variables generation
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c generate 2body phase space CM1
c for 1-->2 decay X--> t V with massiv X , t ,V 
c
      phi(1) = rd(4) * tpi
      costheta(1) = 2d0 * rd(5) - 1

      sintheta(1) = sqrt(1 - costheta(1)**2)
      abs_p(1) =  1d0/2d0/sqrt( msq(1) ) *
     &     sqrt( ( msq(1) - msq(2) - msq(4) )**2 - 4 * msq(2) * msq(4) )  


      w = w *  abs_p(1)  / sqrt( msq(1) )           
      w = w / tpi / 2d0                             

c      if ( w .le. 0d0 ) print*, "dps=0 at line 296"

c
c
c end of 2body phase space CM1
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c generate 2body phase space CM2
c for 1-->2  decay of t --> b W for massiv top , W and massless b
c
      phi(2) = rd(6) * tpi
      costheta(2) = 2d0 * rd(7) - 1

      sintheta(2) = sqrt(1 - costheta(2)**2)
      abs_p(2) =  1d0/2d0/sqrt( msq(2) ) *
     &       sqrt( ( msq(2) - msq(3) )**2 )  


      w = w *  abs_p(2)  / sqrt( msq(2) )                                             
      w = w / tpi / 2d0

c      if ( w .le. 0d0 ) print*, "dps=0 at line 318"
c
c
c end of 2body phase space CM2
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c generate 2body phase space CM3
c for 1-->2 decay of  massiv W(from t decay)--> massless leptons mu vmu~
c
      phi(3) = rd(8) * tpi
      costheta(3) = 2d0 * rd(9) - 1

      sintheta(3) = sqrt(1 - costheta(3)**2)
      abs_p(3) =  1d0/2d0*sqrt( msq(3) )   

      w = w / tpi / 4d0           

c      if ( w .le. 0d0 ) print*, "dps=0 at line 336"
c
c
c end of 2body phase space CM3
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c generate 2body phase space CM4
c for 1-->2 decay of massiv  W / Z--> massless leptons e- ve~/ e+ e-
c
      phi(4) = rd(10) * tpi
      costheta(4) = 2d0 * rd(11) - 1

      sintheta(4) = sqrt(1 - costheta(4)**2)
      abs_p(4) =  1d0/2d0*sqrt( msq(4) )   

      w = w / tpi / 4d0

c      if ( w .le. 0d0 ) print*, "dps=0 at line 354"
c
c
c end of 2body phase space CM4
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c generate incoming momenta in lab frame + x1 x2 
c

      x1 = sqrt(tau) * exp(y)   
      x2 = sqrt(tau) * exp(-y)

      k1(0) = 0.5d0 * x1 * sqrt(s) 
      k1(1) = 0d0
      k1(2) = 0d0
      k1(3) = 0.5d0 * x1 * sqrt(s) 

      k2(0) = 0.5d0 * x2 * sqrt(s) 
      k2(1) = 0d0
      k2(2) = 0d0
      k2(3) = - 0.5d0 * x2 * sqrt(s) 

c CM0 system momenta:
      
      q(0,1) = sqrt(pT**2 + pL**2 + msq(1) )
      q(1,1) = pT * cos( phi(0) )
      q(2,1) = pT * sin( phi(0) )
      q(3,1) = pL
c      q(4,1) = msq(1)

      
      p(0,1) = sqrt( pT**2 + pL**2 )  ! massless q in final state
      p(1,1) = -q(1,1)
      p(2,1) = -q(2,1)
      p(3,1) = -q(3,1)

      do mu = 0, 3
         r(mu) = k1(mu)+k2(mu)      
      enddo


c boost in lab frame
      call boostn( p(0,1), r, p(0,1) )
      call boostn( q(0,1), r, q(0,1) )


ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c CM1 system momenta: X -> t W-

      q(0,2) = sqrt( abs_p(1)**2 + msq(2) )
      q(1,2) = abs_p(1) * sintheta(1) * cos( phi(1) )
      q(2,2) = abs_p(1) * sintheta(1) * sin( phi(1) )
      q(3,2) = abs_p(1) * costheta(1)       
c      q(4,2) = msq(2)

c boost t-vector in lab-frame (the frame of X [q(0,1)] )
      call boostn(q(0,2), q(0,1), q(0,2)) 


c determine W-/Z momenta [q(mu,4)] from momentum conservation
c of X -> t W- / X->t Z
      do mu = 0, 3
         q(mu,4) = q(mu,1)-q(mu,2)
      enddo

c      q(4,4) = msq(4)
c
c
c end of CM1 momenta cccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c CM2 system momenta: t -> b W+ 
      q(0,3) = sqrt(abs_p(2)**2 + msq(3) )
      q(1,3) = abs_p(2) * sintheta(2) * cos( phi(2) )
      q(2,3) = abs_p(2) * sintheta(2) * sin( phi(2) )
      q(3,3) =  abs_p(2) * costheta(2)
c      q(4,3) = msq(3)
c boost W-vector in lab-frame (the frame of top-quark [q(0,2)] )
      call boostn(q(0,3), q(0,2), q(0,3))

c determine b-quark momenta [p(mu,2)] from momentum conservation
c of top -> W+ b
      do mu = 0, 3
         p(mu,2) = q(mu,2)-q(mu,3)
      enddo

c end of CM2 momenta cccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c CM3 system momenta: W+ -> e+ ve
      v(0,3) = abs_p(3)
      v(1,3) = abs_p(3) * sintheta(3) * cos( phi(3) )
      v(2,3) = abs_p(3) * sintheta(3) * sin( phi(3) )
      v(3,3) = abs_p(3) * costheta(3)

c boost e+-vector in lab-frame (the frame of W+ [q(0,3)] )
      call boostn(v(0,3), q(0,3), v(0,3) )

c determine nu momenta [v(mu,4)] from momentum conservation
c of W+ -> e+ nu
      do mu = 0, 3
         v(mu,4) = q(mu,3)-v(mu,3)
      enddo

c end of CM3 momenta cccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c CM4 system momenta: W- /Z  -> e- ve~ / e+ e-
      v(0,1) = abs_p(4)
      v(1,1) = abs_p(4) * sintheta(4) * cos( phi(4) )
      v(2,1) = abs_p(4) * sintheta(4) * sin( phi(4) )
      v(3,1) =  abs_p(4) * costheta(4)

c boost e- -vector in lab-frame (the frame of W- [q(0,4)] )
      call boostn(v(0,1), q(0,4), v(0,1) )

c determine nu / e+ momenta [v(mu,2)] from momentum conservation
c of W- -> e- nu
      do mu = 0, 3
         v(mu,2) = q(mu,4)-v(mu,1)
      enddo

c end of CM2 momenta cccccccccccccccccccccccccccccccccccccccccc

c
c
c end of momenta generation
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c fluxfactors, phasespacefactors , inverse propability densities and Jacobian
c
c
      w = w * 1d0/tpi/4d0/s
      w = w * pT/abs_p(0)
      w = w * 1/(abs_p(0) + ET*coshy2) 
      w = w * y_border             !(prop density)^-1 of y
      w = w * y2_border * 2d0      !(prop density)^-1 of y2
      w = w * 3.89379323d11         !conversion into pb


c      if ( w .le. 0d0 ) print*, "dps=0 at line 483"

c end -----------------------------------------------------

c ============================= rest passiert im Hauptprogramm    

      if (mdebug) then
         print*, "--------- new event -----------"
         print*, ""
         print*, "random numbers"
c         print*, rd(1), rd(2), rd(3)
c         print*, rd(4), rd(5), rd(6)
c         print*, rd(7)
         print*, "m(1)  = ", sqrt(msq(1))
         print*, "m(2)  = ", sqrt(msq(2))
         print*, "m(3)  = ", sqrt(msq(3))
         print*, "m(4)  = ", sqrt(msq(4))
         print*, "y     = ", y
         print*, "tau   = ", tau
         print*, "x1    = ", x1
         print*, "x2    = ", x2
         print*, "y2    = ", y2
         print*, "wgt   = ", w
         print*, "---------------------------"

         print*, "--- momenta conservation ? ---"
         do mu=0,3
            test = k1(mu)+k2(mu)-p(mu,1)-p(mu,2)-v(mu,1)-v(mu,2)-v(mu,3)-v(mu,4)
            print*, test
         enddo
         print*, "-----------------------"

      endif
      
      return
      end subroutine



c***************************************************************************
c***************************************************************************
c
c phase space generator for pp --> t + q + l + nu
c
c       Manuel Baehr:     mb@particle.uni-karlsruhe.de
c       Initial version:  2005 March 07
c       Last modified:    2005 April 04
C
C  INPUT:   RD        random number array (between 0 and 1)
C           RN        additional random number which will not be driven by
C                     MONACO = azimuthal angle
C
C  OUTPUT:  K1,K2     4-vectors for 2 incoming partons
C           X1,X2     corresponding values of Feynman x
C          % Q(0:4,2)  4-vectors for the vector boson resonances V1 and V2,
C          %           the 4th component contains the mass**2
C           v(0:3,nv)  4-vectors for the decay leptons, assumed massless
C           P(*,N)    4-vectors for the N jets, assumed massless (except top)
C           W         the weight of the phase space point
C
C
C  INPUT INFORMATION on the phase space region is transferred via the
C    common block /BLIPSI/
C

c      REAL*8 RM2,RMG,RM2MIN,RM2MAX, S,M2MIN, YJMIN,YJMAX,PTJMIN,EJMIN
c      INTEGER INFOJ, iwidth
C           S         center of mass energy**2 of the machine
c      COMMON /BLIPSI/ RM2(0:2),RMG(0:2),RM2MIN(0:2),RM2MAX(0:2),
c     1                S,M2MIN,YJMIN(N1),YJMAX(N1),PTJMIN(N1),EJMIN(N1),
c     2                INFOJ(N1), iwidth(0:2)
C
c***************************************************************************
c***************************************************************************
      subroutine lipstopdecay(rd,rn,k1,k2,p,v,x1,x2,w)
                     
      implicit none

#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/cuts.inc"
c received from global.inc:
c     nrand = phase space dim - 2 - phi
c     ecm   = collider energy

      logical mdebug
      parameter(mdebug=.false.)

      integer hmem
      common /PAWC/ hmem
      real*8            alfas, xmt, alfa, xmz, xmw, sin2w, xmh
      common /bkopin/   alfas, xmt, alfa, xmz, xmw, sin2w, xmh

c common block for width of particles
      double precision     hwidth, wwidth, zwidth, twidth
      common /COUPL_WIDTH/ hwidth, wwidth, zwidth, twidth
c ----------------------------------------------------
      integer n1
      double precision tpi
      parameter (n1=max_p-nx, tpi=2D0*PI )
      double precision rd(*), rn, x1, x2, w    
      double precision k1(0:3),k2(0:3),p(0:3,n1),q(0:4,2),v(0:3,max_v)
c ----------------------------------------------------
c ----------------------------------------------------

      integer i, mu
      double precision pT, pTmax, beta, expo, pL, y2, y2_border, ET !
      parameter ( expo = 3d0 ) ! pT mapping: exponent of pT distribution for pT<pTmin_def
      double precision abs_p, abs_p2, abs_p3, msq, msq2 !length of momenta in CM + q^2 values of top/W+
      double precision topx, topxmax, topxmin, tantopx, 
     &                 wx, wxmax, wxmin, tanwx
      double precision coshy2, sinhy2, temp, test
      double precision s, y, y_border, tau, phi
      double precision phi2, costheta2, sintheta2
      double precision phi3, costheta3, sintheta3

      common /mylipso/ pT, costheta2, costheta3, y

c ---------------------------------------------------- 
      w = 1d0             !init of weightfactor
      s = ecm**2          !Collider Energy squared

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c q^2 of top-quark(=msq) is generated via tan mapping out of rd(9)
c
c

c gesamter erlaubter Bereich
      topxmin = atan(-xmt/twidth)
      topxmax = atan( (s-xmt**2)/xmt/twidth )
c um peak konzentriert
c      topxmin =  atan(4d2*twidth/xmt-4d1) !atan(n**2*twidth/xmt +- 2*n)
c      topxmax = - topxmin

c      topx = topxmin + rd(9) * (topxmax-topxmin)
c      tantopx = tan(topx)
cccccccccccccccc
c      msq  = xmt**2 + twidth * xmt * tantopx
      msq = xmt**2
cccccccccccccccc
      if (msq.ge.s) then
         print*, "cccccccccc ACHTUNG ccccccccccccccc"
         print*, "c                                c"
         print*, "c       tan-mapping              c"
         print*, "c  verursachte: top-masse**2>s   c"
         print*, "c                                c"
         print*, "cccccccccccccccccccccccccccccccccc"

         msq = s - 1d-3
      endif


c      w = w * twidth * xmt * (1+tantopx**2) ! Jacobian
c      w = w * (topxmax-topxmin)             ! integration area (inverse of prob density)
c      w = w / tpi

       w = w / tpi * xmt * twidth
       w = w * (topxmax - topxmin)

c
c
c end of q^2 generation 
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c q^2 of W+ (=msq2) is generated via tan mapping out of rd(8)
c
c
      wxmin = atan(-xmw/wwidth)
      wxmax = atan( (msq-xmw**2)/xmw/wwidth )

      wx = wxmin + rd(8) * (wxmax-wxmin)
      tanwx = tan(wx)
cccccccccccccccc
      msq2  = xmw**2 + wwidth * xmw * tanwx
cccccccccccccccc
      if (msq2.ge.msq) then
         print*, "cccccccccc ACHTUNG ccccccccccccccc"
         print*, "c                                c"
         print*, "c         tan-mapping            c"
         print*, "c verursachte: W-masse>top-masse c"
         print*, "c                                c"
         print*, "cccccccccccccccccccccccccccccccccc"

         msq2 = msq - 1d-3
      endif



      w = w * wwidth * xmw * (1+tanwx**2)   ! Jacobian
      w = w * (wxmax-wxmin)                 ! integration area (inverse of prob density)
      w = w / tpi

c
c
c end of q^2 (msq2) generation 
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c generate y, pT, y*, phi from plain distributed rd, rn
c
c
      pTmax = (s-msq)/2d0/ecm

cccccccccccccc
      phi = rn * tpi
ccccccccccccccc
c generate pT not uniformly:
      beta = ( (expo+1)*pTmin_def*pTmax  ) /
     &       ( (expo+2)*pTmax - (expo+1)*pTmin_def )

      if ( rd(1) .lt. beta/(expo+1)/pTmin_def ) then
         pT = ( rd(1)*(expo+1)/beta*pTmin_def**(expo+2) )**( 1/(expo+1))
      else
         pT = ( ( pTmin_def - beta/(expo+1) ) * pTmax ) /
     &        ( pTmax - beta/(expo+1) - (pTmax-pTmin_def) * rd(1) )
      endif

c debug stuff:
c      print*, "---------------------------------"
c      print*, "pTmax     =", pTmax
c      print*, "pTmin_def =", pTmin_def
c      print*, "beta      =", beta
c      print*, "loss      =", beta/2.0/pTmin_def

c insert inverse Density
c      if ( rd(1) .lt. beta/(expo+1)/pTmin_def ) then
c         w = w / beta / pT**expo * pTmin_def**(expo+2)
c      else
c         w = w / beta * pT**2
c      endif

c      print*, w
cc flat pT:
      pT = rd(1)*pTmax
      w = w * pTmax                !(prop density)^-1 of pT 
ccccccccccccccc

c-----------------------------------------------------
      ET = sqrt(pT**2+msq)
      temp = (s+msq)/ecm/ET
      y2_border = log( (temp+sqrt(temp**2-4))/2d0 )
c-----------------------------------------------------

ccccccccccccccc
      y2 = y2_border*2d0*( rd(2) - 0.5d0 )
ccccccccccccccc

c ---------------------------------------------------
      sinhy2 = sinh(y2)
      coshy2 = sqrt(sinhy2**2+1)
     
      abs_p = sqrt(ET**2*coshy2**2-msq)
      pL = sinhy2*ET

      tau = (abs_p + coshy2*ET)**2/s
c      tau = 2d0*ET*coshy2/s*(ET*coshy2-abs_p)

      y_border = log(1/tau)
c ---------------------------------------------------

ccccccccccccccc
      y = y_border*( rd(3) - 0.5d0 )
ccccccccccccccc

c
c
c end of CM1 variables generation
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c generate 2body phase space CM2
c
c
      phi2 = rd(4) * tpi
      costheta2 = 2d0 * rd(5) - 1


      w = w * (msq-msq2)/msq        ! phase space for one massive particle (W+)
      w = w / tpi / 4d0
c
c
c end of 2body phase space CM2
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c generate 2body phase space CM3
c
c
      phi3 = rd(6) * tpi
      costheta3 = 2d0 * rd(7) - 1


      w = w / tpi / 4d0
c
c
c end of 2body phase space CM3
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c generate incoming momenta in lab frame + x1 x2 
c
c
      
      x1 = sqrt(tau) * exp(y)
      
      x2 = sqrt(tau) * exp(-y)

      k1(0) = 0.5d0 * x1 * sqrt(s) 
      k1(1) = 0d0
      k1(2) = 0d0
      k1(3) = 0.5d0 * x1 * sqrt(s) 

      k2(0) = 0.5d0 * x2 * sqrt(s) 
      k2(1) = 0d0
      k2(2) = 0d0
      k2(3) = - 0.5d0 * x2 * sqrt(s) 
      
c CM1 system momenta:
      
      q(0,1) = sqrt(pT**2 + pL**2 + msq)
      q(1,1) = pT*cos(phi)
      q(2,1) = pT*sin(phi)
      q(3,1) = pL
      q(4,1) = msq
      
      p(0,1) = sqrt(pT**2 + pL**2)
      p(1,1) = -q(1,1)
      p(2,1) = -q(2,1)
      p(3,1) = -q(3,1)

c boost in lab frame with -y
      call boostl(-y, q(0,1) )
      call boostl(-y, p(0,1) )


c CM2 system momenta:
cc help variables cccccccccccccccccccccc
c
      abs_p2 = 1d0/2d0/sqrt(msq)*(msq-msq2)
      sintheta2 = sqrt(1 - costheta2**2)
c
cccccccccccccccccccccccccccccccccccccccc
      q(0,2) = sqrt(abs_p2**2 + msq2)
      q(1,2) = abs_p2 * sintheta2 * cos(phi2)
      q(2,2) = abs_p2 * sintheta2 * sin(phi2)
      q(3,2) = abs_p2 * costheta2       
      q(4,2) = msq2
c boost W-vector in lab-frame (the frame of top-quark [q(0,1)] )
      call boostn(q(0,2), q(0,1), q(0,2)) 

c determine b-quark momenta [p(mu,2)] from momentum conservation
c of top -> W+ b
      do mu = 0, 3
         p(mu,2) = q(mu,1)-q(mu,2)
      enddo
c
c
c end of CM2 momenta ccccccccccccccccccc

c CM3 system momenta:
cc help variables cccccccccccccccccccccc
c
      abs_p3 = 1d0/2d0 * sqrt(msq2)      
      sintheta3 = sqrt(1 - costheta3**2)
c
cccccccccccccccccccccccccccccccccccccccc
      v(0,1) = abs_p3
      v(1,1) = abs_p3 * sintheta3 * cos(phi3)
      v(2,1) = abs_p3 * sintheta3 * sin(phi3)
      v(3,1) = abs_p3 * costheta3  
      
c boost e+-vector in lab-frame (the frame of w+ [q(0,2)] )
      call boostn(v(0,1), q(0,2), v(0,1) ) 

c determine nu momenta [v(mu,2)] from momentum conservation
c of W+ -> e+ nu
      do mu = 0, 3
         v(mu,2) = q(mu,2)-v(mu,1)
      enddo
c
c
c end of momenta generation
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c Flussfaktoren, Phasenraumfaktoren, W-dichten + Jacobian
c
c
      w = w * 1d0/tpi/4d0/s
      w = w * pT/abs_p
      w = w * 1/(abs_p + ET*coshy2) 
      w = w * y_border             !(prop density)^-1 of y
      w = w * y2_border * 2d0      !(prop density)^-1 of y2
      w = w * 3.89379323d8         !conversion into fb

c end -----------------------------------------------------

c ============================= rest passiert im Hauptprogramm    

      if (mdebug) then
         print*, "--------- new event -----------"
         print*, ""
         print*, "random numbers"
         print*, rd(1), rd(2), rd(3)
         print*, rd(4), rd(5), rd(6)
         print*, rd(7)
         print*, "y     = ", y
         print*, "tau   = ", tau
         print*, "x1    = ", x1
         print*, "x2    = ", x2
         print*, "y2    = ", y2
         print*, "wgt   = ", w
         print*, "---------------------------"

         print*, "--- momenta conservation ? ---"
         do mu=0,3
            test = k1(mu)+k2(mu)-p(mu,1)-p(mu,2)-v(mu,1)-v(mu,2)
            print*, test
         enddo
         print*, "-----------------------"

      endif

      return
      end subroutine







